; =============================================================================================================================================================
; Anzeigemodul 2a fr Temperaturmesssystem mit ATmega128 von Scott-Falk Hhn (s.huehn@soemtron.de)
; Version 3.001, 23.07.2012
; =============================================================================================================================================================
;
;
; Wichtig! Diese Software ist nur fr den privaten Gebrauch bestimmt. Eine kommerzielle Nutzung ohne Genehmigung des Autors ist verboten.
; Important! This software is intended only for private use. A commercial use without permission of the author is forbidden.
;
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Programmierung der Fusebits
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; M103C=1, WDTON=1, OCDEN=1, JTAGEN=0, SPIEN=0, EESAVE=0
; BOOTSZ1=0, BOOTSZ0=0, BOOTRST=1, BODLEVEL=0, BODEN=0, CKOPT=1
; CKSEL3=1, CKSEL2=1, CKSEL1=1, CKSEL0=1, SUT1=0, SUT0=0
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Belegung der I/O-Ports
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; A0: Datenleitung D0 fr LCD (Ein-/Ausgang)
; A1: Datenleitung D1 fr LCD (Ein-/Ausgang)
; A2: Datenleitung D2 fr LCD (Ein-/Ausgang)
; A3: Datenleitung D3 fr LCD (Ein-/Ausgang)
; A4: Datenleitung D4 fr LCD (Ein-/Ausgang)
; A5: Datenleitung D5 fr LCD (Ein-/Ausgang)
; A6: Datenleitung D6 fr LCD (Ein-/Ausgang)
; A7: Datenleitung D7 fr LCD (Ein-/Ausgang)
;
; B0: Ausgang LCD-Betriebsspannung (L-aktiv)
; B1: Eingang SCK/ISP (Pull-Up)
; B2: Ein/Ausgang SCL2 \ TWI/I2C-Bus fr
; B3: Ein/Ausgang SDA2 / RTC und RAM
; B4: Ausgang Tonfrequenz (H-aktiv)
; B5: Ausgang Bluetooth (H-aktiv)
; B6: Ausgang PWM Lautstrke (H-aktiv)
; B7: Ausgang PWM LCD-Backlight (H-aktiv)
;
; C0: Datenleitung /WR fr LCD (Ausgang)
; C1: Datenleitung /RD fr LCD (Ausgang)
; C2: Datenleitung /CE fr LCD (Ausgang)
; C3: Datenleitung C/D fr LCD (Ausgang)
; C4: Eingang Drehimpulsgeber-A (Pull-Up)
; C5: Eingang Drehimpulsgeber-B (Pull-Up)
; C6: Eingang Taster 1 (Pull-Up)
; C7: Eingang Taster 2 (Pull-Up)
;
; D0: Ein/Ausgang SCL1 \ TWI/I2C-Bus fr
; D1: Ein/Ausgang SDA1 / Externes EEPROM
; D2: Eingang RXD1 Daten vom Sensormodul
; D3: Ausgang TXD1 Datenausgang fr sptere Erweiterungen
; D4: Ausgang LED 1 rot  (Low-aktiv)
; D5: Ausgang LED 2 gelb (Low-aktiv)
; D6: Ausgang Test-LED 1 (Low-aktiv)
; D7: Ausgang Test-LED 2 (Low-aktiv)
;
; E0: Eingang RXD0 und MOSI/ISP
; E1: Ausgang TXD0 und MISO/ISP
; E2: Eingang DCF77-Empfnger
; E3: Ausgang LED 3 blau (Low-aktiv)
; E4: Eingang Jumper 1 - Impulsgeber-Konfiguration 1, offen = 2 Impulse/Schritt, geschlossen = 4 Impulse/Schritt (Pull-Up)
; E5: Eingang Jumper 2 - Impulsgeber-Konfiguration 2, offen = normaler Betrieb, geschlossen = Richtungsumkehr (Pull-Up)
; E6: Eingang (frei, Pull-Up)
; E7: Eingang (frei, Pull-Up)
;
; F0: Eingang Lichtsensor (analog)
; F1: Eingang Batteriespannung (analog)
; F2: Eingang (frei, Pull-Up)
; F3: Eingang (frei, Pull-Up)
; F4: Eingang (Pull-Up) JTAG-TCK \
; F5: Eingang (Pull-Up) JTAG-TMS  \ AVR-JTAG
; F6: Eingang (Pull-Up) JTAG-TDO  / Interface
; F7: Eingang (Pull-Up) JTAG-TMI /
;
; G0: Eingang (frei, Pull-Up)
; G1: Eingang (frei, Pull-Up)
; G2: Eingang (frei, Pull-Up)
; G3: Eingang (frei, Pull-Up)
; G4: Eingang (frei, Pull-Up)
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Belegung der Timer
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Timer0:  Tonerzeugung fr akustischen Alarm
; Timer1a: Haupttimer fr Uhr, Drehimpulsgeber und DCF-Empfang, luft als Interrupt-Routine mit 1600Hz / 625s
; Timer1b: PWM fr Lautstrkeeinstellung, Frequenz 1600Hz
; Timer2:  PWM fr LCD-Beleuchtung, Frequenz ca. 339Hz
; TImer3a: Steuertimer fr Taster, Timeout-Zhler, LEDs und Tonerzeugung, luft als Interrupt-Routine mit 50Hz / 20ms
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Belegung des externen EERPOMs (4 x 24C512 = 256kB) (neu)
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Adresse 00000-2FFFC: 15-Minuten-Sensorwerte, 3855 Datenstze x 51 Bytes = 196605 Bytes (963,75 Stunden = 40 Tage, 3 Stunden und 45 Minuten)
; Adresse 30000-3EB9F: 24-Stunden-Sensorwerte, 416 Datenstze x 145 Bytes = 60320 Bytes (416 Tage)
; Adresse 3EBA0-3F94B: Meldungsdaten, 500 Datenstze x 7 Byte = 3500 Byte (500 Ereignisse)
; Adresse 3F94C-3FFFF: Speicher fr Konfigurations-Backup = 1716 Byte
;
.nolist
.include "m128def.inc"
.list
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Konstanten festlegen
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; System-Daten
;
.equ	clock=	11059200		;System-Taktfrequenz
.equ	cltim1=	1600			;Taktfrequenz Timer1a (1600Hz/625s)
.equ	cltim3=	50			;Taktfrequenz Timer3a (50Hz/20ms)
.equ	baud1=	9600			;Baudrate RS232-1 (9600 Baud)
.equ	cr=	13			;Zeichencode Cariage Return (Enter)
.equ	lf=	10			;Zeichencode fr Line Feed
.equ	year2=	'0'			;Jahr Hunderter (ASCII 0)
.equ	year3=	'2'			;Jahr Tausender (ASCII 2)
.equ	yearx=	2000			;Jahr Basiswert fr manuelle Zeiteingabe
.equ	eeprom=	0			;erste genutzte interne EEPROM-Adresse
.equ	txbu0l=	256			;Lnge des RS232-0 Ring-Sendepuffers
.equ	rxbu0l=	128			;Lnge des RS232-0 Ring-Empfangspuffers
.equ	rxbu1l=	64			;Lnge des RS232-1 Ring-Empfangspuffers
.equ	rxda0l=	32			;Lnge des RS232-0 Empfangsdatenpuffers
.equ	rxda1l=	8			;Lnge des RS232-1 Empfangsdatenpuffers
.equ	sigbul=	8			;Lnge des Signalpuffers, Anzahl der Wertepaare
.equ	timsen=	60			;Timeout-Wert fr Sensorwerte in Sekunden, aktuell eingestellt auf 60 Sekunden
.equ	timmsg=	20			;Timeout-Wert fr alle Meldungen in Sekunden, aktuell eingestellt auf 20 Sekunden
.equ	timmss=	45			;Timeout-Wert fr alle Meldungen in Sekunden, Wert beim Systemstart 45 Sekunden
.equ	timdcf=	24			;Timeout-Wert fr DCF77-Empfang in Stunden, aktuell eingestellt auf 24 Stunden
.equ	timmen=	180			;Timeout-Wert fr automatische Rckkehr zur Normalanzeige in Sekunden, eingestellt auf 180 Sekunden
.equ	timlsp=	3			;Timeout-Wert fr kurzzeitige Lautsprecher-Abschaltung in Sekunden, eingestellt auf 3 Sekunden
.equ	timlcd=	30			;Timeout-Wert fr LCD-Abschaltung in Minuten, eingestellt auf 30 Minuten
.equ	timerr=	100			;Timeout-Wert fr zyklische Fehleranzeige in 20ms, eingestellt auf 2 Sekunden
.equ    timnli= 50                      ;Timeout-Wert fr kurzzeitige Umschaltung auf Nacht-Helligkeit in 20ms, eingestellt auf 1 Sekunde
.equ	batlim=	250			;Batterieschwellwert, entspricht 2,5V
.equ	mintmp=	-550			;kleinste mit Drehimpulsgeber einstellbare Temperatur
.equ	maxtmp=	1250			;grte mit Drehimpulsgeber einstellbare Temperatur
.equ	minfrq=	675			;minimale mit Drehimpulsgeber einstellbare Frequenz
.equ	maxfrq=	5959			;maximale mit Drehimpulsgeber einstellbare Frequenz
.equ    dlight= 32                      ;niedrigster Helligkeitswert beim Tage-Modus
;
; RTC-Daten
;
.equ	rtcadr=	0xa0			;I2C Basisadresse des RTC/RAM (PCF8583)
;
.equ	rtflag=	0x10			;RTC-Adresse: Flagbyte, 0x5a = Zeit gltig
.equ	rtyear=	rtflag + 1		;RTC-Adresse: Jahr Zehner/Einer (BCD)                                                              [0x11]
.equ	rtxs1p=	rtyear + 1		;RTC-Adresse: Ext-EEPROM-Pointer Sensordaten 1: 3 Bytes + 1 Flag                                   [0x12-0x15]
.equ	rtxs2p=	rtxs1p + 4		;RTC-Adresse: Ext-EEPROM-Pointer Sensordaten 2: 3 Bytes + 1 Flag                                   [0x16-0x19]
.equ	rtxmsp=	rtxs2p + 4		;RTC-Adresse: Ext-EEPROM-Pointer Meldungsdaten: 3 Bytes + 1 Flag                                   [0x1a-0x1d]
.equ	rtbval=	rtxmsp + 4		;RTC-Adresse: Backup Minimal-, Maximal und Mittelwerte (47 + 47 + 62 + 62 = 218 Bytes)             [0x1e-0xf7]
.equ	rtbdat=	rtbval + 218		;RTC-Adresse: Datum des letzen Backups (4 Byte): Tag-Zehner, Tag-Einer, Monat-Zehner, Monat-Einer  [0xf8-0xfb]
.equ	rtbflg=	rtbdat + 4		;RTC-Adresse: Flag fr Backup, 0x5a = gltig                                                       [0xfc]
;
; EEPROM-Daten (extern)
;
.equ	xeeadr=	0xa0			;I2C Basisadresse des externen EEPROMs (4 x 64kB)
					;B0: R/W-Bit (0 = Write, 1 = Read)
					;B1: \ Select-Bits, B1 = 0/B2 = 0 EEPROM1, B1 = 1/B2 = 0 EEPROM2
                                        ;B2: / Select-Bits, B1 = 0/B2 = 1 EEPROM3, B1 = 1/B2 = 1 EEPROM4
;
.equ    xes1da= 51                      ;Lnge eines Datensatzes fr 15min-Werte
.equ    xes2da= 145                     ;Lnge eines Datensatzes fr 24h-Werte
.equ    xemsda= 7                       ;Lnge eines Datensatzes fr Meldungen
.equ	xs1sta=	0			;Ext-EEPROM: Beginn der Sensordaten 1 (15min-Werte)                             [Adresse = 0x000000 =      0]
.equ	xs1len=	3855 * xes1da		;Ext-EEPROM: Lnge der Sensordaten 1, entspricht 40,15625 Tage                  [Lnge   = 0x02FFFD = 196605]
.equ	xs2sta=	0x30000 		;Ext-EEPROM: Beginn der Sensordaten 2 (24h-Werte)                               [Adresse = 0x030000 = 196608]
.equ	xs2len=	416 * xes2da		;Ext-EEPROM: Lnge der Sensordaten 2, entspricht 416 Tage                       [Lnge   = 0x00EBA0 =  60320]
.equ	xmssta=	xs2sta + xs2len		;Ext-EEPROM: Beginn der Meldungsdaten                                           [Adresse = 0x03EBA0 = 256928]
.equ	xmslen=	500 * xemsda		;Ext-EEPROM: Lnge der Meldungsdaten, entspricht 500 Ereignismeldungen          [Lnge   = 0x000DAC =   3500]
.equ	cfbsta=	xmssta + xmslen		;Ext-EEPROM: Beginn des Konfig-Backups                                          [Adresse = 0x03F94C = 260428]
;                                                                                                                       [Lnge   = 0x0006B4 =   1716]
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Registernamen festlegen
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
.def	insreg=	r2			;Zwischenspeicher fr SREG whrend Interrupt
.def	txflag=	r3			;Textflags, enthlt Textattribute
					;B0: Textausgabe invers
					;B1: Textausgabe mit Unterstreichung
					;B2: Textausgabe mit kleinem Font (Hhe 5 Pixel)
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Speicheradressen im SRAM
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
.dseg
.org	0x100
;
txbuf0:	.byte	txbu0l		;RS232-0 Ring-Sendepuffer (fr Daten vom Anzeigemodul zum PC)
rxbuf0:	.byte	rxbu0l		;RS232-0 Ring-Empfangspuffer (fr Daten vom PC zum Anzeigemodul)
rxbuf1:	.byte	rxbu1l		;RS232-1 Ring-Empfangspuffer (fr Daten vom Sensormodul zum Anzeigemodul)
rxdat0:	.byte	rxda0l		;RS232-0 Empfangsdaten (fr empfangene Kommandos vom PC)
rxdat1:	.byte	rxda1l		;RS232-1 Empfangsdaten (fr empfangene Sensordaten vom Sensormodul)
sigbuf:	.byte	sigbul * 2	;Signal-Puffer, enthlt die Warteschlange fr die auszugebenden akustischen Signale (Signalnummer und Signaldauer)
;
txp0wr:	.byte	2		;RS232-0 Ring-Sendepuffer Schreibposition
txp0rd:	.byte	2		;RS232-0 Ring-Sendepuffer Leseposition
rxp0wr:	.byte	2		;RS232-0 Ring-Empfangspuffer Schreibposition
rxp0rd:	.byte	2		;RS232-0 Ring-Empfangspuffer Leseposition
rxp1wr:	.byte	2		;RS232-1 Ring-Empfangspuffer Schreibposition
rxp1rd:	.byte	2		;RS232-1 Ring-Empfangspuffer Leseposition
rxdap0:	.byte	2		;RS232-0 Empfangsdatenpuffer Position
rxdap1:	.byte	2		;RS232-1 Empfangsdatenpuffer Position
sigbwr:	.byte	2		;Signal-Puffer Schreibposition
sigbrd:	.byte	2		;Signal-Puffer Leseposition
;
postab: .byte   32              ;Anzeige-Positions-Tabelle fr beide Bildschirm-Seiten, enthlt fr jede Bild-Position den anzuzeigenden Sensor,
                                ;0-26 = T01-T27, 27 = P01, 28-30 = H01-H03, 31 = P01 (grafisch), Wert > 31 = keinen Sensor anzeigen
                                ;Byte  0: Sensornummer an Anzeigeposition  0 von Seite 1 (0-31)
                                ;Byte 15: Sensornummer an Anzeigeposition 15 von Seite 1 (0-31)
                                ;Byte 16: Sensornummer an Anzeigeposition  0 von Seite 2 (0-31)
                                ;Byte 31: Sensornummer an Anzeigeposition 15 von Seite 2 (0-31)
                                ;Die Bit-Positionen 5-7 in jedem Byte enthalten folgende Flags:
                                ;B5: keine Daten ausgeben (leeres Feld)
				;B6: Sensorwert neu ausgeben
                                ;B7: Sensorname neu ausgeben
senflg:	.byte	32		;Flags fr 32 Sensoren (Sensornummer 31 existiert nur virtuell als alternative Luftdruckausgabe)
				;B0: Sensorwert neu ausgeben
				;B1: Sensorname neu ausgeben
				;B2: Temperaturberschreitung                   \
				;B3: Temperaturunterschreitung                   \ werden nur bei Temperatur-Sensoren genutzt
				;B4: Erinnerung nach Temperaturberschreitung    /
				;B5: Erinnerung nach Temperaturunterschreitung  /
				;B6: Sensorausfall
				;B7: Sensor nicht genutzt (ist inaktiv)
bigflg: .byte   2               ;Flags fr groen Temperaturwert der beiden Anzeige-Seiten (Byte 0 fr Seite 1, Byte 1 fr Seite 2):
                                ;B4-B0: Sensornummer (0-26)
                                ;B5: keine Daten ausgeben (leeres Feld)
                                ;B6: Sensorwert neu ausgeben
                                ;B7: Sensorname neu ausgeben
sentio:	.byte	31		;Timeout-Zhler fr jeden Sensor; wird bei Empfang eines Datensatzes auf 60s gesetzt und vom Timer-Interrupt auf 0 gezhlt
                                ;Hauptprogramm signalisiert bei 0 den Sensorausfall und setzt den Wert auf 0xff (Stopp)
senval:	.byte	31 * 2		;Sensorwerte im Zweierkomplement 16 Bit, 0x8000 = Wert ungltig
semin1:	.byte	31 * 2		;ermittelte Sensor-Minimalwerte im Zweierkomplement 16 Bit, Werte fr Anzeige
semax1:	.byte	31 * 2		;ermittelte Sensor-Maximalwerte im Zweierkomplement 16 Bit, Werte fr Anzeige
semin2:	.byte	31 * 2		;ermittelte Sensor-Minimalwerte im Zweierkomplement 16 Bit, Werte fr EEPROM-Log
semax2:	.byte	31 * 2		;ermittelte Sensor-Maximalwerte im Zweierkomplement 16 Bit, Werte fr EEPROM-Log
senavs:	.byte	31 * 3		;Summen fr Mittelwertberechnung im Zweierkomplement 24 Bit, Werte fr EEPROM-Log, jeder empfangene Wert wird addiert
senavc:	.byte	31 * 2		;Zhler fr Mittelwertberechnung im Zweierkomplement 16 Bit, Werte fr EEPROM-Log, bei jedem empfangenen Wert + 1
senlil:	.byte	27 * 2		;untere Temperaturlimits im Zweierkomplement 16 Bit, werden aus dem EEPROM gelesen, 0x8000 = Limit nicht aktiv
senlih:	.byte	27 * 2		;obere Temperaturlimits im Zweierkomplement 16 Bit, werden aus dem EEPROM gelesen, 0x8000 = Limit nicht aktiv
logtb1: .byte   24              ;Log-Tabelle 1, enthlt die Positionen der Sensoren 0-23 (T01-T24) im EEPROM-Log
                                ;B7: Konflikt mit einer anderen Log-Position
logtb2: .byte   3               ;Log-Tabelle 2, enthlt die Positionen der Sensoren 24-26 [28-30] (T25-T27 [H01-H03]) im EEPROM-Log
                                ;B7: Konflikt mit einer anderen Log-Position
prstab: .byte   17              ;Tabelle mit den berechneten Pixelwerten fr die grafische Luftdruckanzeige, jedes Byte enthlt 2 4-Bit-Werte (34 Werte)
almtab:	.byte	4		;Alarm-Anzeige-Positions-Tabelle, enthlt die Bild-Positionen der 4 Alarmeingnge
almbak:	.byte	4		;Kopie der Alarm-Anzeige-Tabelle fr die Positionseingabe
alminv:	.byte	4		;Alarm-Invertierungs-Tabelle; bestimmt, ob die Alarmmeldung invertiert ausgewertet wird:
				;0 = normal
				;1 = invertiert
almflg:	.byte	4		;Flags fr jeden der 4 Alarmeingnge
				;B0: Alarmname neu ausgeben
				;B1: Alarm aktiv
				;B2: Erinnerung nach Alarmauslsung
				;B7: Alarmeingang nicht genutzt
cfgflg:	.byte	8		;Konfiguration-Flags, werden fr Konfigurations-Dialoge verwendet (fr jede Bildschirmzeile):
				;B0: Name neu ausgeben
				;B1: Daten neu ausgeben
senmap:	.byte	8		;empfangene Sensorbelegung (8 Zeichen)
dflags:	.byte	1		;Anzeige-Flags
				;B0: Normalanzeige (Sensoren und Alarme)
				;B1: Anzeige lschen und neu ausgeben
				;B2: Infozeile neu ausgeben (obere Zeile)
				;B3: Menzeile neu ausgeben (untere Zeile)
				;B4: DCF77-Empfang ok (Synchronisierung innerhalb 24h)
				;B5: Lautsprecher ist aktiviert (Symbolanzeige)
				;B6: (nicht genutzt)
				;B7: Anzeige ist abgeschaltet (fr DCF-Empfang), wird durch Routinen dcfon/dcfoff gesteuert
xflags:	.byte	1		;Verschiedene Flags 1
				;B0: Signalausgabe aktiv
				;B1: Erinnerungen sind gesetzt
				;B2: Minimalwerte anstatt aktuelle Sensorwerte
				;B3: Maximalwerte anstatt aktuelle Sensorwerte
				;B4: Statusanzeige fr Backup-Zustand aktualisieren
				;B5: Status der Nachtschaltung (1 = Nachtzeit)
				;B6: Externes EEPROM 1 (24C512) vorhanden
				;B7: Externes EEPROM 2 (24C512) vorhanden
xflag2:	.byte	1		;Verschiedene Flags 2
				;B0: aktuelle Lautstrke > 0 (Lautsprecher ein)
				;B1: Ext-EEPROM-Pointer-Fehler Temp-Daten 1 (15min-Werte)
				;B2: Ext-EEPROM-Pointer-Fehler Temp-Daten 2 (24h-Werte)
				;B3: Ext-EEPROM-Pointer-Fehler Meldungs-Daten
				;B4: Datenfelder der Statusanzeige neu ausgeben
				;B5: Anzeigerichtung fr Meldungen:
				;    0 = vorherige Seite, 1 = nchste Seite
				;B6: Listenende bei Meldungsanzeige (vorherige Seite)
				;B7: Listenende bei Meldungsanzeige (nchste Seite)
xflag3:	.byte	1		;Verschiedene Flags 3
				;B0: Konfiguration im EEPROM speichern
				;B1: Grafikanzeige, Daten aus EERPOM einlesen
				;B2: Grafikanzeige, Grafik auf LCD ausgeben
				;B3: Grafikanzeige, Anzeigerichtung:
				;    0 = nchste Seite, 1 = vorherige Seite
				;B4: Grafikanzeige, letzter Datensatz erreicht
				;B5: Grafikanzeige, erster Datensatz erreicht
xflag4:	.byte	1		;Verschiedene Flags 4
				;B0: Konfiguration - Sensoren - Grenzwerte - Unteren Grenzwert ndern
				;B1: Konfiguration - Sensoren - Grenzwerte - Oberen Grenzwert ndern
				;B2: Konfiguration - Signale - Zuordnung - Nummer ndern
				;B3: Konfiguration - Signale - Zuordnung - Dauer ndern
                                ;B4: Anzeige-Seite (0 = Seite 1, 1 = Seite 2)
                                ;B5: (eventuell Log-Freigabe, zurzeit ohne Funktion)
				;B6: Externes EEPROM 3 (24C512) vorhanden
				;B7: Externes EEPROM 4 (24C512) vorhanden
ledflg:	.byte	1		;LED-Flags
				;B0: LED1 (rot) stndig einschalten
				;B1: LED2 (gelb) stndig einschalten
				;B2: LED1 (rot) Blinken mit 3Hz
				;B3: LED2 (gelb) Blinken mit 3Hz
				;B4: LED1 (rot) Blitzen mit 0,5Hz (Erinnerung)
				;B5: LED2 (gelb) Blitzen mit 0,5Hz (Erinnerung)
confg1:	.byte	1		;Konfigurationsbyte 1
				;B0: Sprache (0 = englisch, 1 = deutsch)
				;B1: Trennzeichen (0 = Punkt, 1 = Komma)
				;B2: Temperaturalarme immer signalisieren
				;B3: Nachtschaltung fr LCD-Beleuchtung aktiv
				;B4: Auto-Reset fr Min/Max-Werte aktiv
				;B5: Nachtschaltung fr Signal-Lautstrke aktiv
				;B6: LCD abschalten fr Uhr-Synchronisierung
				;B7: 0 = Normal, 1 = Abwesend (Licht und Ton aus)
confg2:	.byte	1		;Konfigurationsbyte 2
				;B0: LED-Modus (0 = normal, 1 = debug [DCF,RS-232])
				;B1: LCD-Beleuchtung-Automatik
				;B2: Nachtschaltung fr Bluetooth-Modul aktiv
                                ;B3: Automatisch auf Seite 1 zurckschalten
                                ;B4: DCF77-Signal invertiert
                                ;B5: DCF77-Eingang mit Pull-up Widerstand (fr DCF-Empfnger mit offenem Kollektor-Ausgang)
errflg:	.byte	1		;Fehler-Flags, sind alle Bits auf 0, dann wird in der Infozeile das Datum normal angezeigt, sonst erscheint invers die
				;Fehlermeldung:
				;B0: Uhr nicht eingestellt
				;B1: kein Datenempfang
				;B2: Konfigurationsfehler (internes EEPROM)
				;B3: Speicherfehler (externes EEPROM)
				;B4: RTC-Fehler
				;B5: Pointer-Fehler
                                ;B6: Log-Tabellen-Fehler
				;B7: Fehlererinnerung (Fehler war aufgetreten)
errcpy:	.byte	1		;Fehler-Flags-Kopie, wird fr die zyklische Fehleranzeige bentigt
errmsk:	.byte	1		;Fehler-Maske, ist normalerweise 0 und enthlt (falls mehrere Alarme aktiv sind) das Alarmbit fr den nchsten Anzeigezyklus
errtio:	.byte	1		;Timeout-Zhler fr zyklische Fehleranzeige; wird beim Auftreten mehrerer Fehler vom Hauptprogramm auf 2s gesetzt und vom
                                ;Timer-Interrupt auf 0 gezhlt
msgtio:	.byte	1		;Timeout-Zhler fr alle Meldungen; wird bei Empfang eines Datensatzes auf 20s gesetzt und vom Timer-Interrupt auf 0 gezhlt
                                ;Hauptprogramm signalisiert bei 0 den Temperaturausfall und setzt den Wert auf 0xff (Stopp)
dcftio:	.byte	1		;Timeout-Zhler fr DCF77-Empfang; wird bei erfolgreicher Synchronisierung auf 24h gesetzt und vom Timer-Interrupt auf 0
                                ;gezhlt; Hauptprogramm lscht dann Bit 4 in dflags
nlitio: .byte   1               ;Timeout-Zhler fr die Einstellung der Nacht-Helligkeit; wird beim Speichern eines Wertes auf 1s gesetzt und vom Timer-
                                ;Interrupt auf 0 gezhlt; whrend dieser Zeit wird die LCD-Beleuchtung fest auf die Nachthelligkeit gesetzt
signum:	.byte	8		;Signalnummer-Tabelle, enthlt fr jedes Ereignis die Nummer des Signal-Sounds (0-7)
				;Byte 0: Signalnummer fr Alarmeingang 1
				;Byte 1: Signalnummer fr Alarmeingang 2
				;Byte 2: Signalnummer fr Alarmeingang 3
				;Byte 3: Signalnummer fr Alarmeingang 4
				;Byte 4: Signalnummer fr Temperaturberschreitung
				;Byte 5: Signalnummer fr Temperaturunterschreitung
				;Byte 6: Signalnummer fr Fehlermeldungen
				;Byte 7: Signalnummer fr Sensorausflle
sigdur:	.byte	8		;Signaltabelle, enthlt fr jedes Ereignis die Dauer des akustischen Signals in Sekunden, 0 entspricht kein Signal
				;Byte 0: Signaldauer fr Alarmeingang 1
				;Byte 1: Signaldauer fr Alarmeingang 2
				;Byte 2: Signaldauer fr Alarmeingang 3
				;Byte 3: Signaldauer fr Alarmeingang 4
				;Byte 4: Signaldauer fr Temperaturberschreitung
				;Byte 5: Signaldauer fr Temperaturunterschreitung
				;Byte 6: Signaldauer fr Fehlermeldungen
				;Byte 7: Signaldauer fr Sensorausflle
sigsnd:	.byte	1		;zu erzeugendes akustisches Signal (0-7)
sigtim:	.byte	1		;Dauer der akustischen Signalisierung in Sekunden
sigcnt:	.byte	1		;Signalzhler, zhlt in 20ms-Schritten bis 50
sigvo1:	.byte	1		;Signal-Lautstrke 1 Tag (0-15)
sigvo2:	.byte	1		;Signal-Lautstrke 2 Nacht (0-15)
lsptio:	.byte	1		;Timeout-Zhler fr kurzzeitige Lautsprecher-Abschaltung; wird bei Tastendruck 3s gesetzt und vom Timer-Interrupt auf 0 gezhlt
                                ;Hauptprogramm aktiviert bei 0 den Lautsprecher
t1cnt1:	.byte	1		;Timer1 Interrupt-Zhler 1, zhlt die Interrupts von Timer 1 (bis 32; 625s * 32 = 20ms)
t1cnt2:	.byte	1		;Timer1 Interrupt-Zhler 2, zhlt die Zyklen von Zhler1 (bis 50; 20ms * 50 = 1s)
t3cnt1:	.byte	1		;Timer3 Interrupt-Zhler 1, zhlt die Interrupts von Timer3 (bis 50; 20ms * 50 = 1s)
t3cnt2:	.byte	1		;Timer3 Interrupt-Zhler 2, zhlt die Zyklen von Zhler1 (bis 60; 1s * 60 = 1min)
t3cnt3:	.byte	1		;Timer3 Interrupt-Zhler 3, zhlt die Zyklen von Zhler2 (bis 60; 1min * 60 = 1h)
ensta1:	.byte	1		;Drehimpulsgeber-Status beim letzten Interrupt
				;B2: Zustand Drehimpulsgeber-Ausgang A (nur fr Interrupt-Routine)
				;B3: Zustand Drehimpulsgeber-Ausgang B (nur fr Interrupt-Routine)
ensta2:	.byte	1		;letzter stabiler Drehimpulsgeber-Status
				;B2: Zustand Drehimpulsgeber-Ausgang A (nur fr Interrupt-Routine)
				;B3: Zustand Drehimpulsgeber-Ausgang B (nur fr Interrupt-Routine)
encstp:	.byte	2		;aktueller Drehimpulsgeber-Schritt im Zweierkomplement 16-Bit (nur fr Interrupt-Routine)
encpos:	.byte	1		;aktuelle Drehimpulsgeber-Position im Zweierkomplement 8-Bit
encmem:	.byte	1		;letzte gespeicherte Drehimpulsgeber-Position
encflg:	.byte	1		;Drehimpulsgeber-Flags
				;B0: Impulsgeberposition als Menoption anzeigen
				;B1: Anzeige dezimal 3-stellig (0-199)
				;B2: Anzeige Temperaturwert (-55.0 - 125.0)
				;B3: Anzeige in ASCII (0-maxasc)
				;B4: Anzeige dezimal 4-stellig (0-9999)
				;B5: Anzeige im Zeitformat (0:00-23:59)
                                ;B6: Anzeige der Sensorkennung (3 Zeichen aus Defaulttext)
				;B7: Impulsgeberwert hat sich gendert
encval:	.byte	1		;aktueller Impulsgeberwert L
encvah:	.byte	1		;aktueller Impulsgeberwert H
encmin:	.byte	1		;Impulsgeber-Minimalwert (bei 3-stelliger Anzeige)
encmih:	.byte	1		;Impulsgeber-Minimalwert H (bei 4-stelliger Anzeige)
encmax:	.byte	1		;Impulsgeber-Maximalwert (bei 3-stelliger Anzeige)
encmah:	.byte	1		;Impulsgeber-Maximalwert H (bei 4-stelliger Anzeige)
keyst1:	.byte	1		;Status der beiden Taster beim letzten Interrupt
				;B6: Zustand von Taster1
				;B7: Zustand von Taster2
keyst2:	.byte	1		;Status der beiden Taster nach Entprellung
				;B6: Taster1 \ wird von Interrupt gesetzt und
				;B7: Taster2 / wieder gelscht
keyst3:	.byte	1		;Quittierungs-Status der beiden Taster
				;B6: Taster1 \ wird vom Hauptprogramm gesetzt
				;B7: Taster2 / und von Interrupt wied. gelscht
dcftab:	.byte	6		;DCF77-Tabelle, enthlt empfangene Bits, eingehende Bits werden in Byte 5, Bit 7 eingeschoben und alle weiteren Bits um eine
                                ;Position weitergerckt, Byte/Bit-Positionen nach einem kompletten Minutenzyklus:
				;0/6-7: Zeitzone (6 = 1 Sommer, 7 = 1 Winter)
				;1/2-5: Minute Einer
				;1/6-7: Minute Zehner LSB
				;2/0:   Minute Zehner MSB
				;2/1:   Prfbit 1 (Minuten)
				;2/2-5: Stunde Einer
				;2/6-7: Stunde Zehner
				;3/0:   Prfbit 2 (Stunden)
				;3/1-4: Kalendertag Einer
				;3/5-6: Kalendertag Zehner
				;3/7:   Wochentag LSB
				;4/0-1: Wochentag MSB
				;4/2-5: Monat Einer
				;4/6:   Monat Zehner
				;4/7:   Jahr Einer LSB
				;5/0-2: Jahr Einer MSB
				;5/3-6: Jahr Zehner
				;5/7:   Prfbit 3 (Datum)
dcflag:	.byte	1		;DCF77-Flags
				;B0: aktueller DCF77-Zustand
				;B1: vorheriger DCF77-Zustand
				;B2: Synchronisierungs-Flag, wird bei DCF-Syncronisierung gesetzt und zur vollen Stunde gelscht
				;B3: Beginn einer neuen Minute
dcfcnt:	.byte	1		;DCF77-Zhler zur Ermittlung der Impulslnge
dcfimp:	.byte	1		;DCF77-Zhler zur Ermittlung der Impulsanzahl
dcbuf1:	.byte	12		;DCF-Puffer 1, aktuelle decodierte Uhrzeit
				;Byte  0: Zeitzone
				;Byte  1: Minute Einer
				;Byte  2: Minute Zehner
				;Byte  3: Stunde Einer
				;Byte  4: Stunde Zehner
				;Byte  5: Kalendertag Einer
				;Byte  6: Kalendertag Zehner
				;Byte  7: Wochentag
				;Byte  8: Monat Einer
				;Byte  9: Monat Zehner
				;Byte 10: Jahr Einer
				;Byte 11: Jahr Zehner
dcbuf2:	.byte	12		;DCF-Puffer 2, vorherige decodierte Uhrzeit +1 Minute, Aufbau wie dcbuf1
dcbuf3:	.byte	12		;DCF-Puffer 3, vorherige decodierte Uhrzeit +2 Minuten, Aufbau wie dcbuf1
;
				;aktuelle Uhrzeit
secndl:	.byte	1		;Sekunde Einer (0-9)
secndh:	.byte	1		;Sekunde Zehner (0-5)
minutl:	.byte	1		;Minute Einer (0-9)
minuth:	.byte	1		;Minute Zehner (0-5)
hourl:	.byte	1		;Stunde Einer (0-9)  \ zhlen zusammen
hourh:	.byte	1		;Stunde Zehner (0-2) / bis 24
dayw:	.byte	1		;Wochentag (0-6)
dayl:	.byte	1		;Kalendertag Einer (0-9)
dayh:	.byte	1		;Kalendertag Zehner (0-3)
monthl:	.byte	1		;Monat Einer (0-9)
monthh:	.byte	1		;Monat Zehner (0-1)
yearl:	.byte	1		;Jahr Einer (0-9)
yearh:	.byte	1		;Jahr Zehner (0-9)
tzone:	.byte	1		;Zeitzone (0 = Winterzeit, 1 = Sommerzeit)
;
				;Zeitkopie fr bertragung RAM <-> RTC
ct1cnt:	.byte	1		;Timer1 Interrupt-Zhler 2, zhlt die Zyklen
csecdl:	.byte	1		;Sekunde Einer (0-9)
csecdh:	.byte	1		;Sekunde Zehner (0-5)
cminul:	.byte	1		;Minute Einer (0-9)
cminuh:	.byte	1		;Minute Zehner (0-5)
chourl:	.byte	1		;Stunde Einer (0-9)  \ zhlen zusammen
chourh:	.byte	1		;Stunde Zehner (0-2) / bis 24
cdayw:	.byte	1		;Wochentag (0-6)
cdayl:	.byte	1		;Kalendertag Einer (0-9)
cdayh:	.byte	1		;Kalendertag Zehner (0-3)
cmontl:	.byte	1		;Monat Einer (0-9)
cmonth:	.byte	1		;Monat Zehner (0-1)
cyearl:	.byte	1		;Jahr Einer (0-9)
cyearh:	.byte	1		;Jahr Zehner (0-9)
;
cptiml:	.byte	1		;Editor-Kopie, Uhrzeit (Minutenwert L)
cptimh:	.byte	1		;Editor-Kopie, Uhrzeit (Minutenwert H)
cpday:	.byte	1		;Editor-Kopie, Kalendertag (1-31)
cpmon:	.byte	1		;Editor-Kopie, Monat (1-12)
cpyear:	.byte	1		;Editor-Kopie, Jahr Zehner/Einer (0-99)
cpdayw:	.byte	1		;Editor-Kopie, Wochentag (0-6)
;
mennum:	.byte	1		;aktuelle Mennummer
menopt:	.byte	1		;aktuelle Menoption
mentio:	.byte	1		;Timeout-Zhler fr Men; wird bei Bettigung eines Tasters oder des Drehimpulsgebers auf 180s gesetzt und vom Timer-Interrupt
				;auf 0 gezhlt; das Hauptprogramm kehrt dann zur Normalanzeige zurck, wird Seite 2 angezeigt und ist die Option "Auf Seite 1
                                ;zurckschalten" aktiv, dann wird gleichzeitig wieder auf Seite 1 gewechselt
menext:	.byte	1		;Mentext-Erweiterung; enthlt Stringnummer des zustzlich anzuzeigenden Mentextes oder 0
menpos:	.byte	1		;ausgewhlte Men-Position
lcdtio:	.byte	1		;Timeout-Zhler fr LCD-Abschaltung; wird beim Abschalten des LCD auf 30min gesetzt und vom Hauptprogramm auf 0 gezhlt;
				;das LCD wird dann wieder eingeschaltet
minfl1:	.byte	1		;Minuten-Flag zur Erkennung des Minutenwechsels	fr Neuausgabe der Infozeile
minfl2:	.byte	1		;Minuten-Flag zur Erkennung des Minutenwechsels fr Nachtschaltungen und Min/Max-Autoreset
minfl3:	.byte	1		;Minuten-Flag zur Erkennung des Minutenwechsels	fr Log-Aufzeichnungen
houflg:	.byte	1		;Stunden-Flag, Wert des letzten Stunden-Einers (Erkennung des Stundenwechsels fr RTC-Syncronisierung)
signig:	.byte	4		;Nachtschaltung fr akustische Signalisierung
				;Byte 0/1: Startzeit in Minuten (L/H)
				;Byte 2/3: Endzeit in Minuten (L/H)
nighbl:	.byte	4		;Nachtschaltung LCD-Hintergrundbeleuchtung
				;Byte 0/1: Startzeit in Minuten (L/H)
				;Byte 2/3: Endzeit in Minuten (L/H)
nighbt:	.byte	4		;Nachtschaltung fr Bluetooth-Modul
				;Byte 0/1: Startzeit in Minuten (L/H)
				;Byte 2/3: Endzeit in Minuten (L/H)
aresmm:	.byte	4		;Zeiten fr Auto-Reset der Min/Max-Werte
				;Byte 0/1: Auto-Reset-Zeit fr Minimalwerte
				;Byte 2/3: Auto-Reset-Zeit fr Maximalwerte
nlight:	.byte	1		;Nacht-Helligkeit der Hintergrundbeleuchtung, wird aus EEPROM gelesen (0-255)
bright: .byte   1               ;PWM-Helligkeitswert, wird von der Interrupt-Routine "weich" auf OCR2 bertragen
adclig:	.byte	1		;aktueller 8-Bit-Wert von ADC0 (Lichtsensor), wird von Interrupt-Routine von 10 auf 8 Bit reduziert
adcbat:	.byte	8		;aktueller 10-Bit-Wert von ADC1 (RTC-Batterie) und 3 Werte von vorangegangenen Messungen
adcref:	.byte	8		;aktueller 10-Bit-Wert der ADC-Referenz (1,23V)	und 3 Werte von vorangegangenen Messungen
;
xewbuf:	.byte	4 + (47 * 3)	;Schreibpuffer fr externes EEPROM, enthlt einen Datensatz fr Sensordaten oder Meldungen, wird auch fr das Konfigurations-
                                ;Backup verwendet
				;
				;Byte 0-3: komprimierte Zeit in 4 Bytes
				;  Byte 3     Byte 2     Byte 1     Byte 0
				;[--------] [--------] [--------] [--------]
				; \_____/\_____/\___/   \______/   \______/
				;  Jahr   Monat  Tag     Stunde     Minute
				;
				;-> weitere Bytes bei einem 15min-Sensordatensatz:
				;
				;Byte  4-39: Sensordaten T01-T24 zugeordnet ber Log-Tabelle 1           (36 Bytes)
                                ;Byte 40-45: Sensordaten T25-T27 zugeordnet ber Log-Tabelle 2 und P01    (6 Bytes)
                                ;Byte 46-48: Sensordaten H01-H03 zugeordnet ber Log-Tabelle 2            (3 Bytes)
                                ;Byte 49-50: Batteriespannung                                             (2 Bytes)
				;  Byte 4     Byte 5     Byte 6         Byte 43    Byte 44    Byte 45    Byte 46    Byte 47    Byte 48    Byte 49    Byte 50
				;[--------] [--------] [--------] ... [--------] [--------] [--------] [--------] [--------] [--------] [--------] [--------]
				; \______/   \______/   \__/\__/       \______/   \______/   \__/\__/   \______/   \______/   \______/   \______/   \_____/|
				;   T01L       T02L    T02H  T01H        T27L       P01L    P01H  T27H     H01        H02        H03       BatL      Res. BatH
				;
				;-> weitere Bytes bei einem 24h-Sensordatensatz: 3 Byteblcke mit dem gleichen Aufbau wie oben:
				;
				;Byte  4- 50: Sensordaten (47 Bytes) mit Minimalwerten
				;Byte 51- 97: Sensordaten (47 Bytes) mit Maximalwerten
				;Byte 98-144: Sensordaten (47 Bytes) mit Mittelwerten
				;
				;-> weitere Bytes bei einem Meldungsdatensatz:
				;
				;Byte 4-6: Sekunden und Meldungsdaten (3 Byte)
				;  Byte 4     Byte 5     Byte 6   \
				;[--------] [--------] [--------]  \ Meldungscode: entspricht der String-Nummer (0xee - 0xfd)
				; \______/   \______/   \______/   / Parameter:    Sensornummer (0-30), Alarm-Nummer (0-3) oder leer (0)
				;  Sekunde Meldungscode Parameter /
;
xerbuf:	.byte	4 + (47 * 3)	;Lesepuffer fr externes EEPROM fr PC-Datenausgabe; gleicher Aufbau wie xewbuf; wird auch die Generierung der Daten fr die
                                ;grafische Luftdruckanzeige und die Kontrolle des Konfigurations-Backups verwendet
xedbuf:	.byte	4 + (47 * 3)	;Lesepuffer fr externes EEPROM fr die grafische Ausgabe auf dem LCD; gleicher Aufbau wie xewbuf
xetemp:	.byte	3		;Zwischenspeicher fr die Suchroutine fr die Schreibposition, enthlt Endadresse
xes1po:	.byte	3		;Zeiger (18 Bit) auf aktuelle Schreibposition fr Sensordaten 1 im externen EEPROM (15min-Werte)
xes2po:	.byte	3		;Zeiger (18 Bit) auf aktuelle Schreibposition fr Sensordaten 2 im externen EEPROM (24h-Werte)
xemspo:	.byte	3		;Zeiger (18 Bit) auf aktuelle Schreibposition fr Meldungsdaten im externen EEPROM
baud0:	.byte	1		;Baudrate fr RS232-0, 0 = 9600, 1 = 19200, 2 = 38400, 3 = 57600, 4 = 115200, 5 = 230400, 6 = 460800
pccmd:	.byte	1		;PC-Schnittstelle, auszufhrendes Kommando
pccpnt:	.byte	3		;Zeiger auf aktuelle Position im externen EEPROM whrend Kommando-Bearbeitung
pcccnt:	.byte	2		;Datensatzzhler whrend Kommando-Bearbeitung
pcpara:	.byte	9		;PC-Schnittstelle, Parameterspeicher, Byte 0-7 enthalten den empfangenen Datumswert, Byte 8 speichert ein empfangenes
				;Minuszeichen
ascbuf:	.byte	201		;Zwischenspeicher fr EEPROM-Datenkonvertierung, enthlt die PC-Sendedaten
ascibf:	.byte	16		;ASCII-Eingabepuffer fr Nameneingabe
ascibp:	.byte	1		;ASCII-Eingabepuffer, aktuelle Eingabeposition
ascibm:	.byte	1		;ASCII-Eingabepuffer, maximale Zeichenanzahl
asclst: .byte   1               ;Anzahl der Zeichen in der ASCII-Zeichenliste (wird beim Programmstart ermittelt)
dimode:	.byte	1		;alternativer Anzeigemodus; wird ausgewertet, wenn dflags/Bit0 = 0 ist, mgliche Werte:
				; 1 = Meldungsdaten anzeigen
				; 2 = Status anzeigen
				; 3 = Konfiguration - Sensoren - Einstellungen, Position auswhlen
				; 4 = Konfiguration - Sensoren - Anzeige, Position auswhlen
				; 5 = Konfiguration - Sensoren - Einstellungen, Name eingeben
				; 6 = Konfiguration - Sensoren - Grenzwerte, Position auswhlen
				; 7 = Konfiguration - Sensoren - Grenzwerte, ndern
				; 8 = Konfiguration - Alarme, Position auswhlen
				; 9 = Konfiguration - Alarme, Position verschieben
				;10 = Konfiguration - Alarme, Name eingeben
				;11 = Konfiguration - Signale - Zuordnung, Position auswhlen
				;12 = Konfiguration - Signale - Zuordnung, Nummer/Dauer eingeben
				;13 = Konfiguration - Signale - Audio, Position auswhlen
				;14 = Konfiguration - Signale - Audio, Daten ndern
				;15 = Konfiguration - Zeiten - Schaltzeiten, Position auswhlen
				;16 = Konfiguration - Zeiten - Schaltzeiten, Zeit ndern
				;17 = Konfiguration - Zeiten - Uhr stellen, Position auswhlen
				;18 = Konfiguration - Zeiten - Uhr stellen, Daten ndern
				;19 = Konfiguration - Sonstiges 1, Position auswhlen
				;20 = Konfiguration - Sonstiges 1, Daten ndern
				;21 = Konfiguration - Sensoren - Log-Zuordnung 1, Position auswhlen
				;22 = Konfiguration - Sensoren - Log-Zuordnung 1, ndern
				;23 = Grafik anzeigen
				;24 = Grafik, Fadenkreuz anzeigen und bearbeiten
				;25 = Konfiguration - Grafik-Presets, Position auswhlen
				;26 = Konfiguration - Grafik-Presets, Daten ndern
				;27 = Konfiguration - Sensoren - Anzeige, ndern
                                ;28 = Konfiguration - Sensoren - Groanzeige, Position auswhlen
                                ;29 = Konfiguration - Sensoren - Groanzeige, ndern
				;30 = Konfiguration - Sensoren - Log-Zuordnung 2, Position auswhlen
				;31 = Konfiguration - Sensoren - Log-Zuordnung 2, ndern
				;32 = Konfiguration - Sonstiges 2, Position auswhlen
dipara: .byte   1               ;Parameter fr den Anzeigemodus:
                                ;0 = Einstellungen 1 oder Grenzwerte 1
                                ;1 = Einstellungen 2 oder Grenzwerte 2
msgfil:	.byte	1		;Meldungsfilter, steuert die Menanzeige und die Datenausgabe, folgende Werte sind mglich:
				;0 = alles anzeigen
				;1 = nur Alarme anzeigen
				;2 = nur Temperaturen anzeigen
				;3 = nur Fehler anzeigen
msgpo1:	.byte	3		;(18 Bit) zeigt auf die aktuelle Position im EEPROM der gerade angezeigten Meldung
msgpo2:	.byte	3		;(18 Bit) zeigt auf die Position im EEPROM der Meldung in der ersten Anzeige-Position; fr die Anzeige der vorherigen
				;Meldungsseite
rcvflg:	.byte	1		;wird bei fehlerfreiem RS-232 Empfang auf 0 gesetzt und von der Status-Anzeige ausgewertet
comflg: .byte   1               ;wird bei aktiver PC-Kommunikation auf 0 gesetzt und von der Interrupt-Routine zur Steuerung der LED 3 (blau) verwendet
batval: .byte   2               ;berechnete Batteriespannung, wird beim Logging der 15min-Werte zwischengespeichert und fr die Min-/Max-/Mittelwerte verwendet
grpres:	.byte	1		;ausgewhltes Grafik-Preset (0-23), bei Auswahl werden die folgenden 7 Bytes eingelesen:
grse11:	.byte	1		;Grafikanzeige, Grafik 1 Sensornummer
grse12: .byte   1		;Grafikanzeige, Grafik 1 Differenz-Sensornummer
grse21:	.byte	1		;Grafikanzeige, Grafik 2 Sensornummer
grse22:	.byte	1		;Grafikanzeige, Grafik 2 Differenz-Sensornummer
grmod1:	.byte	1		;Grafikanzeige, Anzeigemodus 1 (0-3):
				;0 = 15min-Werte
				;1 = 24h-Minimalwerte
				;2 = 24h-Maximalwerte
				;3 = 24h-Mittelwerte
grmod2:	.byte	1		;Grafikanzeige, Anzeigemodus 1 (0-3), siehe grmod1, wird nur bei grmod1 > 0 ausgewertet
grflag:	.byte	1		;Grafikanzeige, Flags fr spezielle Funktionen:
				;B0: Sensornamen in Grafik einblenden
				;B1: Skala angleichen (Doppeldiagramm)
				;B2: Differenzmodus fr Diagramm 1
				;B3: Differenzmodus fr Diagramm 2
				;B4: Doppeldiagramm
                                ;B5: Schriftgre (0 = normal, 1 = Mini-Schrift)
grpoin:	.byte	1		;Grafikanzeige, aktuelle Position im Datenpuffer und aktuelle Fadenkreuzposition (0-221)
grlacr:	.byte	1		;letzte Fadenkreuzposition (0-221)
grbuff:	.byte	222 * 8		;Grafikanzeige, Datenpuffer fr 222 Werte:
				;Byte 0-3: komprimiertes Zeitformat wie xewbuf
				;Byte 4-5: Sensorwert 1 im Zweierkomplement, 16 Bit, 0x8000 = kein Wert vorhanden
				;Byte 6-7: Sensorwert 2 im Zweierkomplement, 16 Bit, 0x8000 = kein Wert vorhanden
grcdat:	.byte	4		;Grafikanzeige, Zwischenspeicher fr Datum/Zeit	komprimiertes Format, siehe xewbuf, Byte 0-3
grpsta:	.byte	3	        ;Zeiger (18 Bit) im externen EEPROM, enthlt Adresse des ersten Wertes in der Grafik
grpend:	.byte	3	        ;Zeiger (18 Bit) im externen EEPROM, enthlt Adresse des letzten Wertes in der Grafik
grptem:	.byte	3               ;Zeiger (18 Bit) im externen EEPROM, enthlt Adresse des aktuellen Wertes in der Grafik
grsca1:	.byte	1	        ;Grafikanzeige, Y-Skalierungsfaktor Diagramm 1
grsca2:	.byte	1	        ;Grafikanzeige, Y-Skalierungsfaktor Diagramm 2
grcen1:	.byte	2	        ;Grafikanzeige, Y-Mittelpunktwert fr Diagramm 1
grcen2:	.byte	2	        ;Grafikanzeige, Y-Mittelpunktwert fr Diagramm 2
grysva:	.byte	2	        ;Grafikanzeige, Y-Skale, aktueller Skalenwert
grlax1:	.byte	1	        ;Grafikanzeige, \ letzte Koordinaten von Pixel 1
grlay1:	.byte	1	        ;Grafikanzeige, / dient zum Verbinden der Pixel
grlax2:	.byte	1	        ;Grafikanzeige, \ letzte Koordinaten von Pixel 2
grlay2:	.byte	1	        ;Grafikanzeige, / dient zum Verbinden der Pixel
grrbu0:	.byte	44	        ;Grafikanzeige, Fadenkreuz, LCD-Daten Y
grrbu1:	.byte	28	        ;Grafikanzeige, Fadenkreuz, LCD-Daten X1
grrbu2:	.byte	28	        ;Grafikanzeige, Fadenkreuz, LCD-Daten X2
grry1p:	.byte	1               ;Grafikanzeige, Fadenkreuz, Y-Pixelposition 1
grry2p:	.byte	1	        ;Grafikanzeige, Fadenkreuz, Y-Pixelposition 2
;
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Programmbeginn, Interrupt-Vektoren
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
.cseg
.org	0x0000
	jmp	start			;Programmstart
;
.org	OC1Aaddr			;Timer1 OutputCompareA
	jmp	t1comp			;wird alle 625s ausgelst
;
.org	URXC0addr			;USART0 RX Complete, Zeichen
	jmp	rxcpl0			;an RS232-0 empfangen
;
.org	UDRE0addr			;USART0 Data Register Empty,
	jmp	txdre0			;Zeichen an RS232-0 senden
;
.org	OC3Aaddr			;Timer3 OutputCompareA
	jmp	t3comp			;wird alle 20ms ausgelst
;
.org	URXC1addr			;USART1 RX Complete, Zeichen
	jmp	rxcpl1			;an RS232-1 empfangen
;
.org	SPMRaddr			;letzter INT-Tabelleneintrag
	reti				;ggf. Int-Routine beenden,
;
.org	SPMRaddr+2			;Hauptprogramm beginnt hinter
					;der Vektortabelle
;
; Stackpointer setzen, RAM lschen
;
start:	cli				;Interrupts sperren
	ldi	r16, low(ramend)	;Stackpointer L
	out	spl, r16		;setzen
	ldi	r16, high(ramend)	;Stackpointer H
	out	sph, r16		;setzen
	clr	r16
	ldi	xl, low(0x100)		;Beginn des RAM-Bereiches Low
	ldi	xh, high(0x100)		;Beginn des RAM-Bereiches High
ramclr:	st	x+, r16			;Speicherzelle lschen
	cpi	xl, low(ramend + 1)	;RAM-Ende (Low) erreicht?
	brne	ramclr			;nein -> Schleife
	cpi	xh, high(ramend + 1)	;RAM-Ende (High) erreicht?
	brne	ramclr			;nein -> Schleife
;
; I/O-Ports, ADC und TWI (I2C) initialisieren
;
	ldi	r16, 0xff
	out	porta, r16		;PortA konfigurieren
	clr	r16
	out	ddra, r16		;PortA konfigurieren
	ldi	r16, 0x02
	out	portb, r16		;PortB konfigurieren
	ldi	r16, 0xf1
	out	ddrb, r16		;PortB konfigurieren
	ldi	r16, 0xff
	out	portc, r16		;PortC konfigurieren
	ldi	r16, 0x0f
	out	ddrc, r16		;PortC konfigurieren
	ldi	r16, 0xc8
	out	portd, r16		;PortD konfigurieren
	ldi	r16, 0xf8
	out	ddrd, r16		;PortD konfigurieren
	ldi	r16, 0xf2
	out	porte, r16		;PortE konfigurieren
	ldi	r16, 0x0a
	out	ddre, r16		;PortE konfigurieren
	ldi	r16, 0xfc
	sts	portf, r16		;PortF konfigurieren
	ldi	r16, 0
	sts	ddrf, r16		;PortF konfigurieren
	ldi	r16, 0x1f
	sts	portg, r16		;PortG konfigurieren
	ldi	r16, 0
	sts	ddrg, r16		;PortG konfigurieren
;
	ldi	r16, 0			;Referenz an AREF,
	out	admux, r16		        ;ADC0 auswhlen
	ldi	r16, (1 << aden) | (7 << adps0)	;ADC ein, Vorteiler= 128,
	out	adcsr, r16		        ;ADC-Frequenz= 86400 Hz
;
	ldi	r16, 10			;kleinster mglicher Wert als
	sts	twbr, r16		;TWI-Master (307200 kHz)
	clr	r16
	sts	twsr, r16		;Vorteiler=1 setzen
	ldi	r16, 1 << twen
	sts	twcr, r16		;TWI aktivieren
;
; Pointer der RS232-Schnittstellen und Puffer initialisieren
;
	ldi	xl, low(txbuf0)
	ldi	xh, high(txbuf0)	;Pointer fr RS232-1 S-Puffer
	sts	txp0wr, xl		;Schreibpointer setzen L
	sts	txp0wr + 1, xh		;Schreibpointer setzen H
	sts	txp0rd, xl		;Lesepointer setzen L
	sts	txp0rd + 1, xh		;Lesepointer setzen H
;
	ldi	xl, low(rxbuf0)
	ldi	xh, high(rxbuf0)	;Pointer fr RS232-0 E-Puffer
	sts	rxp0wr, xl		;Schreibpointer setzen L
	sts	rxp0wr + 1, xh		;Schreibpointer setzen H
	sts	rxp0rd, xl		;Lesepointer setzen L
	sts	rxp0rd + 1, xh		;Lesepointer setzen H
;
	ldi	xl, low(rxbuf1)
	ldi	xh, high(rxbuf1)	;Pointer fr RS232-1 E-Puffer
	sts	rxp1wr, xl		;Schreibpointer setzen L
	sts	rxp1wr + 1, xh		;Schreibpointer setzen H
	sts	rxp1rd, xl		;Lesepointer setzen L
	sts	rxp1rd + 1, xh		;Lesepointer setzen H
;
	ldi	xl, low(rxdat0)
	ldi	xh, high(rxdat0)	;Pointer fr RS232-0 D-Puffer
	sts	rxdap0, xl		;Position setzen L
	sts	rxdap0 + 1, xh		;Position setzen H
	ldi	xl, low(rxdat1)
	ldi	xh, high(rxdat1)	;Pointer fr RS232-1 D-Puffer
	sts	rxdap1, xl		;Position setzen L
	sts	rxdap1 + 1, xh		;Position setzen H
;
	ldi	xl, low(sigbuf)
	ldi	xh, high(sigbuf)	;Pointer fr Signalpuffer
	sts	sigbwr, xl		;Schreibpointer setzen L
	sts	sigbwr + 1, xh		;Schreibpointer setzen H
	sts	sigbrd, xl		;Lesepointer setzen L
	sts	sigbrd + 1, xh		;Lesepointer setzen H
;
; LCD einschalten und initialisieren
;
	call	lcdon			;LCD einschalten/initialisieren
;
; Anzeige-Positions-Tabellen fr beide Bildschirm-Seiten aus EEPROM laden
;
	ldi	r19, 32			;Anzeige-Positions-Tabelle, 32 Werte
	ldi	r16, low(epotab)
	ldi	r17, high(epotab)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(postab)
	ldi	xh, high(postab)	;Zeiger auf RAM-Adresse
ldpotb:	call	eeread			;Byte aus EERPOM lesen
        andi    r18, 0x3f               ;Flag-Bits lschen
        st      x+, r18                 ;als Positionswert speichern
	dec	r19			;alle Werte kopiert?
	brne	ldpotb			;nein -> Schleife
;
; Sensor-Flags initialisieren
;
	ldi	r19, 32			;Sensor-Flags, 32 Werte
	ldi	r16, low(esnflg)
	ldi	r17, high(esnflg)	;EEPROM-Adresse der Flags
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
ltmflg:	call	eeread			;Byte aus EERPOM lesen
	andi	r18, 0x80		;Inaktivitts-Flag filtern
	st	x+, r18			;Sensor-Flag speichern
	dec	r19			;alle Werte kopiert?
	brne	ltmflg			;nein -> Schleife
;
; Sensornummern fr Groanzeige aus EEPROM laden
;
	ldi	r16, low(ebiflg)
	ldi	r17, high(ebiflg)	;EEPROM-Adresse Groanzeige-Sensornummern
	call	eeread			;Byte aus EERPOM lesen
        andi    r18, 0x3f               ;Flag-Bits lschen
	sts	bigflg, r18		;und speichern
	call	eeread			;Byte aus EERPOM lesen
        andi    r18, 0x3f               ;Flag-Bits lschen
	sts	bigflg + 1, r18		;und speichern
;
; Sensorwerte initialisieren (0x8000 setzen) und Timeout-Zhler auf Startwert setzen
;
	ldi	r17, 31			;31 Sensorwerte
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Temperaturwerte
	ldi	xl, low(sentio)
	ldi	xh, high(sentio)	;Zeiger auf Timeout-Zhler
	ldi	r16, 0x80		;Wert fr "Temperatur ungltig"
	ldi	r18, timsen		;Startwert fr Timeout-Zhler
initem:	st	x+, r18			;Timeout-Zhler setzen
	std	y + 1, r16		;Wert in H-Byte speichern
	adiw	yl, 2			;Zeiger auf nchsten Wert
	dec	r17			;alle Werte bearbeitet?
	brne	initem			;nein -> Schleife
	call	rsmin1			;Minimal-Werte lschen (Anzeige)
	call	rsmax1			;Maximal-Werte lschen (Anzeige)
	call	rsmin2			;Minimal-Werte lschen (Log)
	call	rsmax2			;Maximal-Werte lschen (Log)
;
; obere Temperaturlimits aus EERPOM laden
;
	ldi	r19, 27 * 2		;27 Werte laden (27 * 2 Bytes)
	ldi	r16, low(eselih)
	ldi	r17, high(eselih)	;EERPOM-Adresse der Daten
	ldi	xl, low(senlih)
	ldi	xh, high(senlih)	;Zeiger auf obere Limits
ldtlih:	call	eeread			;Byte aus EERPOM lesen
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldtlih			;nein -> Schleife
;
; untere Temperaturlimits aus EERPOM laden
;
	ldi	r19, 27 * 2		;27 Werte laden (27 * 2 Bytes)
	ldi	r16, low(eselil)
	ldi	r17, high(eselil)	;EERPOM-Adresse der Daten
	ldi	xl, low(senlil)
	ldi	xh, high(senlil)	;Zeiger auf untere Limits
ldtlil:	call	eeread			;Byte aus EERPOM lesen
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldtlil			;nein -> Schleife
;
; Sensor-Log-Tabelle 1 aus EEPROM laden
;
	ldi	r19, 24			;24 Werte laden
	ldi	r16, low(elotb1)
	ldi	r17, high(elotb1)	;EEPROM-Adresse der Log-Tabelle 1
	ldi	xl, low(logtb1)
	ldi	xh, high(logtb1)	;Zeiger auf RAM-Adresse
ldltb1:	call	eeread			;Byte aus EERPOM lesen
	andi	r18, 0x1f		;Tabellenposition filtern
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldltb1			;nein -> Schleife
;
; Sensor-Log-Tabelle 2 aus EEPROM laden
;
	ldi	r19, 3			;3 Werte laden
	ldi	r16, low(elotb2)
	ldi	r17, high(elotb2)	;EEPROM-Adresse der Log-Tabelle 2
	ldi	xl, low(logtb2)
	ldi	xh, high(logtb2)	;Zeiger auf RAM-Adresse
ldltb2:	call	eeread			;Byte aus EERPOM lesen
	andi	r18, 0x03		;Tabellenposition filtern
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldltb2			;nein -> Schleife
;
; Alarm-Anzeige-Positions-Tabelle aus EEPROM laden, Alarmflags initialisieren
;
	ldi	r19, 4			;Anzeige-Positions-Tabelle, 4 Werte
	ldi	r16, low(ealtab)
	ldi	r17, high(ealtab)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(almtab)
	ldi	xh, high(almtab)	;Zeiger auf RAM-Adresse
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
ldamap:	call	eeread			;Byte aus EERPOM lesen
	bst	r18, 7			;Inaktivitts-Flag holen
	ld	r20, y			;Alarmflags holen
	bld	r20, 7			;Flag fr Inaktivitt setzen
	st	y+, r20			;Flag wieder speichern
	andi	r18, 0x03		;Anzeigeposition filtern
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldamap			;nein -> Schleife
;
; Alarm-Invertierungs-Tabelle aus EEPROM laden
;
	ldi	r19, 4			;Invertierungs-Tabelle, 4 Werte
	ldi	r16, low(ealinv)
	ldi	r17, high(ealinv)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(alminv)
	ldi	xh, high(alminv)	;Zeiger auf RAM-Adresse
ldainv:	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 2			;Wert > 1?
	brcs	ldai10			;nein -> ok
	clr	r18			;sonst Invertierung aus
ldai10:	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldainv			;nein -> Schleife
;
; Konfigurationsbyte 1 aus EEPROM laden
;
	ldi	r16, low(econf1)
	ldi	r17, high(econf1)	;EEPROM-Adresse Konfig-Byte 1
	call	eeread			;Byte aus EERPOM lesen
	sts	confg1, r18		;und speichern
;
; Konfigurationsbyte 2 aus EEPROM laden
;
	ldi	r16, low(econf2)
	ldi	r17, high(econf2)	;EEPROM-Adresse Konfig-Byte 2
	call	eeread			;Byte aus EERPOM lesen
	sts	confg2, r18		;und speichern
;
; Signalnummern-Tabelle aus EEPROM laden
;
	ldi	r19, 8			;Signalnummern-Tabelle, 8 Werte
	ldi	r16, low(esignr)
	ldi	r17, high(esignr)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(signum)
	ldi	xh, high(signum)	;Zeiger auf RAM-Adresse
ldsnum:	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 16			;Signalnummer > 15?
	brcs	ldsn10			;nein -> ok
	clr	r18			;sonst Signalnummer 0 whlen
ldsn10:	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldsnum			;nein -> Schleife
;
; Signaldauer-Tabelle aus EEPROM laden
;
	ldi	r19, 8			;Signaldauer-Tabelle, 8 Werte
	ldi	r16, low(esigdu)
	ldi	r17, high(esigdu)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(sigdur)
	ldi	xh, high(sigdur)	;Zeiger auf RAM-Adresse
ldsdur:	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 31			;Signaldauer > 30?
	brcs	ldsd10			;nein -> ok
	clr	r18			;sonst Signalnummer 0 whlen
ldsd10:	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldsdur			;nein -> Schleife
;
; Timer-Wert zur Erzeugung der Signalfrequenz aus EEPROM laden
;
	ldi	r16, low(esigtm)
	ldi	r17, high(esigtm)	;EEPROM-Adresse Signal-Frequenz
	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 28			;Timerwert >= 28?
	brcc	ldtim0			;ja -> ok
	ldi	r18, 28			;sonst hchste Frequenz whlen
ldtim0:	out	ocr0, r18		;holen und Timer0 einstellen
;
; Lautstrke-Wert 1 (Tag) aus EEPROM laden und einstellen
;
	ldi	r16, low(esigv1)
	ldi	r17, high(esigv1)	;EEPROM-Adresse Lautstrke 1
	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 16			;Timerwert > 15?
	brcs	ldtv10			;nein -> ok
	ldi	r18, 15			;sonst hchste Lautstrke
ldtv10:	sts	sigvo1, r18		;Lautstrke-Wert speichern
;
; Lautstrke-Wert 2 (Nacht) aus EEPROM laden
;
	ldi	r16, low(esigv2)
	ldi	r17, high(esigv2)	;EEPROM-Adresse Lautstrke 2
	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 16			;Timerwert > 15?
	brcs	ldtv30			;nein -> ok
	ldi	r18, 15			;sonst hchste Lautstrke
ldtv30:	sts	sigvo2, r18		;Lautstrke-Wert speichern
;
; Nachtzeiten fr akustische Signalisierung aus EEPROM laden
;
	ldi	r19, 4			;Nachtschaltung, 4 Werte
	ldi	r16, low(esigni)
	ldi	r17, high(esigni)	;EERPOM-Adresse Startzeit
	ldi	xl, low(signig)
	ldi	xh, high(signig)	;Zeiger auf RAM-Adresse
ldsign:	call	eeread			;Byte aus EERPOM lesen
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldsign			;nein -> Schleife
;
; Nachtzeiten fr die LCD-Hintergrundbeleuchtung aus EEPROM laden
;
	ldi	r19, 4			;Nachtschaltung, 4 Werte
	ldi	r16, low(enigbl)
	ldi	r17, high(enigbl)	;EERPOM-Adresse Startzeit
	ldi	xl, low(nighbl)
	ldi	xh, high(nighbl)	;Zeiger auf RAM-Adresse
ldnibl:	call	eeread			;Byte aus EERPOM lesen
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldnibl			;nein -> Schleife
;
; Nachtzeiten fr das Bluetooth-Modul aus EEPROM laden
;
	ldi	r19, 4			;Nachtschaltung, 4 Werte
	ldi	r16, low(enigbt)
	ldi	r17, high(enigbt)	;EERPOM-Adresse Startzeit
	ldi	xl, low(nighbt)
	ldi	xh, high(nighbt)	;Zeiger auf RAM-Adresse
ldnibt:	call	eeread			;Byte aus EERPOM lesen
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldnibt			;nein -> Schleife
;
; Auto-Reset-Zeiten fr Minimal- und Maximalwerte aus EEPROM laden
;
	ldi	r19, 4			;Reset-Zeiten, 2 * 2 Werte
	ldi	r16, low(earesm)
	ldi	r17, high(earesm)	;EERPOM-Adresse Startzeit
	ldi	xl, low(aresmm)
	ldi	xh, high(aresmm)	;Zeiger auf RAM-Adresse
ldares:	call	eeread			;Byte aus EERPOM lesen
	st	x+, r18			;und im RAM speichern
	dec	r19			;alle Werte kopiert?
	brne	ldares			;nein -> Schleife
;
; Nacht-Helligkeit der Hintergrundbeleuchtung lesen
;
	ldi	r16, low(enligh)
	ldi	r17, high(enligh)	;EEPROM-Adresse Nacht-Helligkeit
	call	eeread			;Byte aus EERPOM lesen
	sts	nlight, r18		;Nacht-Helligkeit speichern
;
; Baudrate fr die PC-Verbindung (RS232-0) lesen
;
	ldi	r16, low(ebaud0)
	ldi	r17, high(ebaud0)	;EEPROM-Adr. Baudrate RS232-0
	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 7			;gltiger Wert < 7?
	brcs	ldba10			;ja -> weiter
	ldi	r18, 4			;sonst Wert = 4 setzen (115200)
ldba10:	sts	baud0, r18		;Baudrate RS232-0 speichern
;
; Meldungsfilter lesen
;
	ldi	r16, low(emsgfi)
	ldi	r17, high(emsgfi)	;EEPROM-Adresse Meldungsfilter
	call	eeread			;Byte aus EERPOM lesen
	cpi	r18, 4			;gltiger Wert < 4?
	brcs	ldmf10			;ja -> weiter
	clr	r18			;sonst Wert = 0 setzen (kein Filter)
ldmf10:	sts	msgfil, r18		;Meldungsfilter speichern
;
; Prfen, ob RTC vorhanden ist
;
	ldi	r16, rtcadr		;Slave-Adresse RTC + W-Bit
	mov	r15, r16		;kopieren
	call	i2ctio			;zum RTC senden (15ms)
	brcs	rtch10			;ACK empfangen? nein -> Fehler
	clr	r16			;Adresse=0
	call	i2csen			;senden
	brcs	rtch10			;ACK empfangen? nein -> Fehler
	ldi	r16, rtcadr + 1		;Slave-Adresse RTC + R-Bit
	call	i2crst			;I2C Repeat Start
	brcs	rtch10			;ACK empfangen? nein -> Fehler
	sec				;kein ACK senden
	call	i2crcv			;Byte lesen
	andi	r16, 0x7c		;Bits 7/1/0 lschen, Rest = 0?
	brne	rtch10			;nein -> Fehler
	call	i2csto			;I2C Stop Condition
	rjmp	xechck			;weiter mit EEPROM-Test
;
rtch10:	call	i2csto			;I2C Stop Condition
	call	i2cerr			;RTC-Fehler setzen
;
; Prfen, ob alle 4 externen EEPROM ICs vorhanden sind
;
xechck:	lds	r20, xflags		;verschiedene Flags holen
	lds	r21, xflag4		;verschiedene Flags 4 holen
	ldi	r16, xeeadr		;Slave-Adresse EEPROM1 + W-Bit
	mov	r15, r16		;kopieren
	call	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xech10			;ACK empfangen? nein -> Fehler
	clr	r16			;Adresse H = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech10			;nein -> Fehler
	clr	r16			;Adresse L = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech10			;nein -> Fehler
	call	twista			;TWI Start Condition
	cpi	r17, 0x10		;TWI Start ok?
	brne	xech10			;nein -> Fehler
	ldi	r16, xeeadr + 1		;Slave-Adresse EEPROM1 + R-Bit
	call	twisnd			;senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xech10			;nein -> Fehler
	clt				;kein ACK senden
	call	twircv			;Byte lesen
	cpi	r17, 0x58		;Lesen ok?
	brne	xech10			;nein -> Fehler
	sbr	r20, 0x40		;Flag fr Ext-EEPROM 1 setzen
;
xech10:	call	twisto			;TWI Stop Condition
	ldi	r16,xeeadr + 2		;Slave-Adresse EEPROM2 + W-Bit
	mov	r15, r16		;kopieren
	call	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xech20			;ACK empfangen? nein -> Fehler
	clr	r16			;Adresse H = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech20			;nein -> Fehler
	clr	r16			;Adresse L = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech20			;nein -> Fehler
	call	twista			;TWI Start Condition
	cpi	r17, 0x10		;TWI Start ok?
	brne	xech20			;nein -> Fehler
	ldi	r16, xeeadr + 3		;Slave-Adresse EEPROM2 + R-Bit
	call	twisnd			;senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xech20			;nein -> Fehler
	clt				;kein ACK senden
	call	twircv			;Byte lesen
	cpi	r17, 0x58		;Lesen ok?
	brne	xech20			;nein -> Fehler
	sbr	r20, 0x80		;Flag fr Ext-EEPROM 2 setzen
;
xech20:	call	twisto			;TWI Stop Condition
	ldi	r16,xeeadr + 4		;Slave-Adresse EEPROM3 + W-Bit
	mov	r15, r16		;kopieren
	call	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xech30			;ACK empfangen? nein -> Fehler
	clr	r16			;Adresse H = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech30			;nein -> Fehler
	clr	r16			;Adresse L = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech30			;nein -> Fehler
	call	twista			;TWI Start Condition
	cpi	r17, 0x10		;TWI Start ok?
	brne	xech30			;nein -> Fehler
	ldi	r16, xeeadr + 5		;Slave-Adresse EEPROM3 + R-Bit
	call	twisnd			;senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xech30			;nein -> Fehler
	clt				;kein ACK senden
	call	twircv			;Byte lesen
	cpi	r17, 0x58		;Lesen ok?
	brne	xech30			;nein -> Fehler
	sbr	r21, 0x40		;Flag fr Ext-EEPROM 3 setzen
;
xech30:	call	twisto			;TWI Stop Condition
	ldi	r16,xeeadr + 6		;Slave-Adresse EEPROM4 + W-Bit
	mov	r15, r16		;kopieren
	call	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xech40			;ACK empfangen? nein -> Fehler
	clr	r16			;Adresse H = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech40			;nein -> Fehler
	clr	r16			;Adresse L = 0
	call	twisnd			;senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xech40			;nein -> Fehler
	call	twista			;TWI Start Condition
	cpi	r17, 0x10		;TWI Start ok?
	brne	xech40			;nein -> Fehler
	ldi	r16, xeeadr + 7		;Slave-Adresse EEPROM4 + R-Bit
	call	twisnd			;senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xech40			;nein -> Fehler
	clt				;kein ACK senden
	call	twircv			;Byte lesen
	cpi	r17, 0x58		;Lesen ok?
	brne	xech40			;nein -> Fehler
	sbr	r21, 0x80		;Flag fr Ext-EEPROM 4 setzen
;
xech40:	call	twisto			;TWI Stop Condition
	sts	xflags, r20		;Flags wieder speichern
	sts	xflag4, r21		;Flags 4 wieder speichern
	andi	r20, 0xc0		;Ext-EEPROM-Flags von EEPROM 1 und 2 filtern
        andi    r21, 0xc0               ;Ext-EEPROM-Flags von EEPROM 3 und 4 filtern
        and     r20, r21                ;Flags verknpfen
	cpi	r20, 0xc0		;alle EEPROMs vorhanden?
	breq	xesrch			;ja -> weiter
	rjmp	xes300			;sonst Fehler
;
; Ermitteln der nchsten EEPROM-Schreibposition fr 15min-Werte
;
xesrch:	lds	r16, errflg		        ;Fehler-Flags holen
	bst	r16, 4			        ;RTC-Fehler?
	brts	xes020			        ;ja -> Position suchen
	ldi	xl, low(xes1po)
	ldi	xh, high(xes1po)	        ;Zeigeradresse Sensordaten 1
	ldi	r18, rtxs1p		        ;RTC-Adresse Pointer Sensordaten 1
	call	i2prcv			        ;Pointer aus RTC laden
	brcs	xes010			        ;Fehler? ja -> Status setzen
	cpi	r16, 0x5a		        ;Flag ok?
	brne	xes020			        ;nein -> Position suchen
	rjmp	xes100			        ;sonst nchster Pointer
;
xes010:	call	i2cerr			        ;RTC-Fehlerstatus setzen
xes020:	lds	r16, xflag2		        ;verschiedene Flags 2 holen
	sbr	r16, 0x02		        ;Pointer-Fehler-Flag 1 setzen
	sts	xflag2, r16		        ;Flags wieder speichern
	call	poierr			        ;Pointer-Fehlerstatus setzen
	ldi	r20, low(xs1sta)
	ldi	r21, high(xs1sta)
	ldi	r22, byte3(xs1sta)	                ;Anfangsadresse Sensordaten 1
	ldi	r23, low(xs1sta + xs1len - xes1da)
	ldi	r24, high(xs1sta + xs1len - xes1da)     ;Adresse des letzten mglichen
	ldi	r25, byte3(xs1sta + xs1len - xes1da)    ;Datensatzbeginns laden
	sts	xetemp, r23
	sts	xetemp + 1, r24
	sts	xetemp + 2, r25		        ;Adresse in Zwischenspeicher ablegen
	ldi	r19, xes1da		        ;Datensatz-Offset (51 Bytes)
	call	xswpos			        ;Suche beginnen (Sensordaten 1)
	brcc	xes030			        ;Suche ok? ja -> weiter
	rjmp	xes300			        ;sonst Fehler
;
xes030:	sts	xes1po, r12
	sts	xes1po + 1, r13		        ;Zeiger auf restaurierte
	sts	xes1po + 2, r14		        ;Schreibposition setzen
	lds	r16, errflg	        	;Fehler-Flags holen
	bst	r16, 4			        ;RTC-Fehler?
	brts	xes100			        ;ja -> berspringen
	ldi	r18, rtxs1p	        	;RTC-Adresse Sensordaten 1
	ldi	xl, low(xes1po)
	ldi	xh, high(xes1po)        	;Zeiger auf Sensordaten 1 Pointer
	call	i2psen		        	;neuen Pointer in RTC schreiben
	brcc	xes100		        	;Fehler? nein -> weiter
	call	i2cerr		        	;sonst RTC-Fehlerstatus setzen
;
; Ermitteln der nchsten EEPROM-Schreibposition fr 24h-Werte
;
xes100:	lds	r16, errflg	        	;Fehler-Flags holen
	bst	r16, 4		        	;RTC-Fehler?
	brts	xes120		        	;ja -> Position suchen
	ldi	xl, low(xes2po)
	ldi	xh, high(xes2po)        	;Zeigeradresse Sensordaten 2
	ldi	r18, rtxs2p	        	;RTC-Adresse Pointer Sensordaten 2
	call	i2prcv		        	;Pointer aus RTC laden
	brcs	xes110		        	;Fehler? ja -> Status setzen
	cpi	r16, 0x5a	        	;Flag ok?
	brne	xes120		        	;nein -> Position suchen
	rjmp	xes200		        	;sonst nchster Pointer
;
xes110:	call	i2cerr		        	;RTC-Fehlerstatus setzen
xes120:	lds	r16, xflag2	        	;verschiedene Flags 2 holen
	sbr	r16, 0x04	        	;Pointer-Fehler-Flag 2 setzen
	sts	xflag2, r16	        	;Flags wieder speichern
	call	poierr		        	;Pointer-Fehlerstatus setzen
	ldi	r20, low(xs2sta)
	ldi	r21, high(xs2sta)
	ldi	r22, byte3(xs2sta)	                ;Anfangsadresse Sensordaten 2
	ldi	r23, low(xs2sta + xs2len - xes2da)
	ldi	r24, high(xs2sta + xs2len - xes2da)     ;Adresse des letzten mglichen
	ldi	r25, byte3(xs2sta + xs2len - xes2da)    ;Datensatzbeginns laden
	sts	xetemp, r23
	sts	xetemp + 1, r24
	sts	xetemp + 2, r25	        	;Adresse in Zwischenspeicher ablegen
	ldi	r19, xes2da	        	;Datensatz-Offset (145 Bytes)
	call	xswpos		        	;Suche beginnen (Sensordaten 2)
	brcc	xes130		        	;Suche ok? ja -> weiter
	rjmp	xes300		        	;sonst Fehler
;
xes130:	sts	xes2po, r12
	sts	xes2po + 1, r13	        	;Zeiger auf restaurierte
	sts	xes2po + 2, r14	        	;Schreibposition setzen
	lds	r16, errflg	        	;Fehler-Flags holen
	bst	r16, 4		        	;RTC-Fehler?
	brts	xes200		        	;ja -> berspringen
	ldi	r18, rtxs2p	        	;RTC-Adresse Sensordaten 2
	ldi	xl, low(xes2po)
	ldi	xh, high(xes2po)        	;Zeiger auf Sensordaten 2 Pointer
	call	i2psen		        	;neuen Pointer in RTC schreiben
	brcc	xes200		        	;Fehler? nein -> weiter
	call	i2cerr		        	;sonst RTC-Fehlerstatus setzen
;
; Ermitteln der nchsten EEPROM-Schreibposition fr Meldungsdaten
;
xes200:	lds	r16, errflg	        	;Fehler-Flags holen
	bst	r16, 4		        	;RTC-Fehler?
	brts	xes220		        	;ja -> Position suchen
	ldi	xl, low(xemspo)
	ldi	xh, high(xemspo)        	;Zeigeradresse Meldungsdaten
	ldi	r18, rtxmsp	        	;RTC-Adresse Pointer Meldungsdaten
	call	i2prcv		        	;Pointer aus RTC laden
	brcs	xes210		        	;Fehler? ja -> Status setzen
	cpi	r16, 0x5a	        	;Flag ok?
	brne	xes220		        	;nein -> Position suchen
	rjmp	initim		        	;sonst nchster Abschnitt
;
xes210:	call	i2cerr		        	;RTC-Fehlerstatus setzen
xes220:	lds	r16, xflag2	        	;verschiedene Flags 2 holen
	sbr	r16, 0x08	        	;Pointer-Fehler-Flag 3 setzen
	sts	xflag2, r16	        	;Flags wieder speichern
	call	poierr		        	;Pointer-Fehlerstatus setzen
	ldi	r20, low(xmssta)
	ldi	r21, high(xmssta)
	ldi	r22, byte3(xmssta)	                ;Anfangsadresse Meldungsdaten
	ldi	r23, low(xmssta + xmslen - xemsda)
	ldi	r24, high(xmssta + xmslen - xemsda)     ;Adresse des letzten mglichen
	ldi	r25, byte3(xmssta + xmslen - xemsda)    ;Datensatzbeginns laden
	sts	xetemp, r23
	sts	xetemp + 1, r24
	sts	xetemp + 2, r25	        	;Adresse in Zwischenspeicher ablegen
	ldi	r19, xemsda	        	;Datensatz-Offset (7 Bytes)
	call	xswpos		        	;Suche beginnen (Meldungsdaten)
	brcs	xes300		        	;Suche ok? nein -> Fehler
;
	sts	xemspo, r12
	sts	xemspo + 1, r13	        	;Zeiger auf restaurierte
	sts	xemspo + 2, r14	        	;Schreibposition setzen
	lds	r16, errflg	        	;Fehler-Flags holen
	bst	r16, 4		        	;RTC-Fehler?
	brts	initim		        	;ja -> berspringen
	ldi	r18, rtxmsp	        	;RTC-Adresse Pointer Meldungsdaten
	ldi	xl, low(xemspo)
	ldi	xh, high(xemspo)        	;Zeigeradresse Meldungsdaten
	call	i2psen		        	;neuen Pointer in RTC schreiben
	brcc	initim		        	;Fehler? nein -> weiter
	call	i2cerr		        	;sonst RTC-Fehlerstatus setzen
	rjmp	initim		        	;weiter im nchsten Abschnitt
;
xes300:	lds	r16, errflg	        	;Fehler-Flags holen
	sbr	r16, 0x08	        	;Speicherfehler-Flag setzen
	sts	errflg, r16	        	;Flags wieder speichern
;
; Zeit aus RTC lesen und setzen
;
initim:	lds	r16, errflg                    	;Fehler-Flags holen
	sbr	r16, 0x01                       ;zunchst Uhren-Fehler setzen
	sts	errflg, r16		        ;Flags wieder speichern
	bst	r16, 4			        ;RTC-Fehler?
	brts	initrs			        ;ja -> berspringen
	call	tirrtc		        	;sonst Zeit aus RTC holen
	cli			        	;Interrupts wieder sperren
	brcs	init10		        	;Fehler? ja -> weiter
	cpi	r16, 0x5a	        	;Flag-Byte = 0x5a?
	brne	initrs		        	;nein -> berspringen
	lds	r16, errflg	        	;sonst Fehler-Flags holen
	cbr	r16, 0x01	        	;Uhren-Fehler wieder lschen
	sts	errflg, r16	        	;Flags wieder speichern
	rjmp	initrs		        	;weiter im nchsten Abschnitt
;
init10:	call	i2cerr		        	;RTC-Fehler setzen
;
; serielle Schnittstellen initialisieren
;
initrs:	ldi	r16, low((clock / (16 * baud1)) - 1)			;RS232-1 auf 9600 Baud
	ldi	r17, high((clock / (16 * baud1)) - 1)			;einstellen
	sts	ubrr1l, r16						;Baudratenwert L setzen
	sts	ubrr1h, r17						;Baudratenwert H setzen
	ldi	r16, (3 << ucsz10)					;Datenformat 8N1
	sts	ucsr1c, r16						;Datenformat setzen
	ldi	r16, (1 << rxcie1) | (1 << rxen1)			;RX mit Interrupt aktivieren
	sts	ucsr1b, r16						;Einstellungen setzen
;
	call	setbd0							;Baudrate fr RS232-0 setzen
	ldi	r16, (3 << ucsz00)					;Datenformat 8N1
	sts	ucsr0c, r16						;Datenformat setzen
	ldi	r16, (1 << rxcie0) | (1 << rxen0) | (1 << txen0)	;RX und TX aktivieren, RX Int einschalten
	out	ucsr0b, r16
;
; Timer initialisieren
;
	ldi	r16, (1 << wgm01 | 3 << cs00)                     	;Timer0
	out	tccr0, r16                              		;Mode Normal+CTC, Vorteiler = 32
;
	ldi	r16, low((clock / cltim1) - 1)
	ldi	r17, high((clock / cltim1) - 1)                         ;Teilerwert fr Timer1a laden
	out	ocr1ah, r17
	out	ocr1al, r16		                                ;und setzen (H zuerst)
	clr	r16
	ldi	r17, 1			                                ;Timer1 auf 256 voreinstellen
	out	tcnt1h, r17                             		;damit der Interrupt von Timer1
	out	tcnt1l, r16		                                ;vor Timer3 ausgelst wird
	ldi	r16, (1 << com1b1) | (3 << wgm30)                       ;Timer1 Fast PWM, OC1B aktiv
	out	tccr1a, r16		                                ;setzen
	ldi	r16, (3 << wgm12) | (1 << cs10)	                        ;Timer1 Fast PWM, Vorteiler = 1
	out	tccr1b, r16		                                ;setzen
	ldi	r16, 1 << ocie1a		                        ;Interrupt bei OutputCompareA
	out	timsk, r16		                                ;fr Timer1 aktivieren
;
	ldi	r16, (1 << wgm20) | (1 << com21) | (3 << cs20)	        ;Timer2
	out	tccr2, r16				                ;PhaseCorrect PWM, Vorteiler = 64
;
	ldi	r16, low((clock / (8 * cltim3)) - 1)		        ;Teilerwert fr Timer3a
	ldi	r17, high((clock / (8 * cltim3)) - 1)		        ;laden
	sts	ocr3ah, r17
	sts	ocr3al, r16		                                ;und setzen (H zuerst)
	clr	r16			                                ;Timer3 Mode Normal+CTC, keine
	sts	tccr3a, r16		                                ;OC-Ausgnge aktivieren
	ldi	r16, (1 << wgm32) | (1 << cs31)	                        ;Timer3 Mode CTC, Vorteiler = 8
	sts	tccr3b, r16		                                ;setzen
	ldi	r16, 1 << ocie3a		                        ;Interrupt bei OutputCompareA
	sts	etimsk, r16		                                ;fr Timer3 aktivieren
;
; Anzahl der Zeichen in der ASCII-Zeichenliste ermitteln
;
        ldi     zl, byte3(chrlst * 2)
        out     rampz, zl
        ldi     zl, low(chrlst * 2)
        ldi     zh, high(chrlst * 2)    ;Zeiger auf Zeichenliste
        clr     r17                     ;Zeichenzhler lschen
inichl: elpm    r16, z+                 ;Zeichen aus Liste holen
        cpi     r16, 0xff               ;Listenende erreicht?
        breq    inic10                  ;ja -> Ende
        inc     r17                     ;sonst Zeichenzhler erhhen
        rjmp    inichl                  ;Schleife
inic10: sts     asclst, r17             ;Anzahl der ermittelten Zeichen speichern
;
; Konflikte in den Log-Tabellen ermitteln und markieren
;
        call    chklog                  ;Log-Tabellen auf Konflikte prfen
;
; Drehimpulsgeber initialisieren und Interrupts aktivieren
;
	in	r17, pine		;Impulsgeber Jumper 1 (Geber-Typ) und Jumper 2 (Richtung) holen
	andi	r17, 0x10		;Jumper 1 geschlossen (Impulsgeber mit 4 Schritten)?
	brne	initen			;nein -> berspringen
	ldi	r16, 2			;sonst Schrittwert des Drehimpulsgebers voreinstellen
	sts	encstp, r16		;und speichern
        ser     r16
        sts     comflg, r16             ;Kommunikations-Flag lschen
initen:	sei				;Interrupts aktivieren
;
; Daten fr grafische Luftdruckanzeige generieren
;
        lds     r16, errflg             ;Fehler-Flags holen
        bst     r16, 3                  ;Speicher-Fehler?
        brts    chkbck                  ;ja -> berspringen
        call    genprs                  ;sonst Daten fr grafische Luftdruckanzeige generieren
;
; DCF77-Konfiguration prfen und gegebenenfalls Pull-up aktivieren
;
        lds     r16, confg2             ;Konfigurationsbyte 2 holen
        sbrc    r16, 5                  ;DCF77 mit Pull-up? nein -> berspringen
        sbi     porte, 2                ;sonst Pull-up Widerstand einschalten
;
; Prfen, ob ein aktuelles Backup fr Min-, Max- und Mittelwerte im RTC-RAM vorhanden ist
;
chkbck:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 4			;RTC-Fehler?
	brts	chkcrc			;ja -> berspringen
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	call	i2csta			;I2C Start Condition + Adresse
	brcs	rdberr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtbdat		;Adresse des Backup-Datums
	call	i2csen			;senden
	brcs	rdberr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtcadr + 1		;I2C Basisadresse RTC + Read
	call	i2crst			;I2C Repeat Start Condition
	brcs	rdberr			;ACK empfangen? nein -> Fehler
;
	clc				;ACK zum Slave senden
	call	i2crcv			;Byte empfangen
	mov	r20, r16		;Kalendertag Einer kopieren
	clc				;ACK zum Slave senden
	call	i2crcv			;Byte empfangen
	mov	r21, r16		;Kalendertag Zehner kopieren
	clc				;ACK zum Slave senden
	call	i2crcv			;Byte empfangen
	mov	r22, r16		;Monat Einer kopieren
	clc				;ACK zum Slave senden
	call	i2crcv			;Byte empfangen
	mov	r23, r16		;Monat Zehner kopieren
	sec				;kein ACK zum Slave senden
	call	i2crcv			;Byte empfangen
	call	i2csto			;I2C Stop Condition
;
	cpi	r16, 0x5a		;Backup-Flag ok?
	brne	chkcrc			;nein -> Restore berspringen
	lds	r16, dayl		;Kalendertag Einer holen
	cp	r16, r20		;Kalendertag Einer ok?
	brne	chkcrc			;nein -> Restore berspringen
	lds	r16, dayh		;Kalendertag Zehner holen
	cp	r16, r21		;Kalendertag Einer ok?
	brne	chkcrc			;nein -> Restore berspringen
	lds	r16, monthl		;Monat Einer holen
	cp	r16, r22		;Monat Einer ok?
	brne	chkcrc			;nein -> Restore berspringen
	lds	r16,monthh		;Monat Zehner holen
	cp	r16, r23		;Monat Zehner ok?
	brne	chkcrc			;nein -> Restore berspringen
	call	i2rest			;Werte aus RTC-RAM zurckholen
	brcc	chkcrc			;Fehler? nein -> weiter
;
rdberr:	call	i2cerr			;RTC-Fehler setzen
;
; EEPROM CRC-Wert prfen und gegebenenfalls Systemfehler aktivieren
;
chkcrc:	call	calcrc			;EEPROM CRC-Berechnung
	call	eeread			;gespeicherten CRC Wert holen
	cp	r18, r20		;CRC ok?
	breq	chkerr			;ja -> weiter
	lds	r16, errflg		;sonst Fehler-Flags holen
	sbr	r16, 0x04		;EEPROM-CRC-Fehler setzen
	sts	errflg, r16		;Fehler-Flags wieder speichern
	lds	r18, signum + 6		;Signalnummer fr Fehlermeldung
	lds	r19, sigdur + 6		;Signaldauer fr Fehlermeldung
	call	signal			;Signal ausgeben
;
; Fehler-Flag prfen und Meldungen ins Log schreiben
;
chkerr: ldi     r19, 0xee               ;Meldungscode "System-Neustart"
        ldi     r20, 0                  ;kein Parameter
        call    wrimsg                  ;Meldung schreiben
;
	lds	r24, errflg		;Fehler-Flags holen
	bst	r24, 3			;Speicherfehler?
	brts	initmn			;ja -> keine Meldungen schreiben
	bst	r24, 2			;Konfigurationsfehler?
	brtc	chke10			;nein -> weiter testen
	ldi	r19, 0xf2		;Meldung "Konfigurations-Fehler"
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
chke10:	bst	r24, 4			;RTC-Fehler?
	brtc	chke20			;nein -> weiter testen
	ldi	r19, 0xf4		;Meldung "Echtzeituhr-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
chke20:	bst	r24, 5			;Pointer-Fehler?
	brtc	initmn			;nein -> weiter
	ldi	r19, 0xf5		;Meldung "Pointer-Fehler"
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
;
; Hauptschleife initialisieren
;
initmn:	ldi	r16, timmss		;Meldungs-Timeout zunchst
	sts	msgtio, r16		;auf 45s setzen
	lds	r16, minutl		;Minuten Einer holen
	sts	minfl1, r16		;als Minutenflag1 voreinstellen
	sts	minfl3, r16		;als Minutenflag3 voreinstellen
	ser	r16			;0xff laden
	sts	minfl2, r16		;als Minutenflag2 voreinstellen
	lds	r16, hourl		;Stunden Einer holen
	sts	houflg, r16		;als Stundenflag voreinstellen
	clr	r17			;zunchst Lautstrke = 0 setzen
	call	sicalc			;PWM berechnen, Lautstrke setzen
	ldi	r16, 0xff		;letzte Fadenkreuzposition
	sts	grlacr, r16		;lschen
;
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Beginn Hauptprogrammschleife
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Anzeigesteuerung
;
main:	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m12000			;nein -> berspringen
	sbr	r16, 0x0c		;sonst Flags fr Info/Menzeile setzen
	sts	dflags, r16		;und Flags wieder speichern
;
; Basisanzeige neu ausgeben: LCD lschen, Rahmen zeichnen
;
	call	clrlcd			;LCD lschen
	clr	txflag			;Textflags lschen
	ldi	r18, 0			;X = 0
m11100:	ldi	r19, 0 | 0x80		;Y = 0, oberer Rahmen
	call	pixlcd			;waagerechte Linie zeichnen
	ldi	r19, 14 | 0x80		;Y = 14, Trennlinie Kopfzeile
	call	pixlcd			;waagerechte Linie zeichnen
	ldi	r19, 112 | 0x80		;Y = 112, Trennlinie Fuzeile
	call	pixlcd			;waagerechte Linie zeichnen
	ldi	r19, 127 | 0x80		;Y = 127, unterer Rahmen
	call	pixlcd			;waagerechte Linie zeichnen
	inc	r18			;X-Koordinate erhhen
	cpi	r18, 240		;alle Pixel ausgegeben?
	brne	m11100			;nein -> Schleife
;
	ldi	r19, 0 | 0x80		;Y = 0
m11110:	ldi	r18, 0			;X = 0, linker Rahmen
	call	pixlcd			;senkrechte Linie zeichnen
	ldi	r18, 239		;X = 239, rechter Rahmen
	call	pixlcd			;senkrechte Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	brne	m11110			;nein -> Schleife
;
	ldi	r19, 1 | 0x80		;Y = 1
m11120:	ldi	r18, 174		;X = 174, Trennlinie fr Uhrzeit
	call	pixlcd			;Linie zeichnen
	ldi	r18, 205		;X = 205, Trennlinie fr Antenne
	call	pixlcd			;Linie zeichnen
	ldi	r18, 222		;X = 222, Trennlinie fr Lautsprecher
	call	pixlcd			;Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	cpi	r19, 14 | 0x80		;alle Pixel ausgegeben?
	brne	m11120			;nein -> Schleife
;
	ldi	r21, 2			;Y = 2
	ldi	r20, 224		;X = 224
	ldi	r22, 0x9d		;Lautsprecher-Symbol
	call	asclcd			;ausgeben
;
; Infozeile ausgeben: DCF77- und Lautsprecher-Status
;
m12000:	clr	txflag			;Textflags lschen
	lds	r25, dflags		;Anzeige-Flags holen
	bst	r25, 2			;Neuausgabe der Infozeile?
	brts	m12010			;ja -> bearbeiten
	rjmp	m13000			;sonst berspringen
;
m12010:	cbr	r25, 0x04		;Infozeilen-Flag lschen
	sts	dflags, r25		;und Flags wieder speichern
;
	ldi	r20, 206		;X = 206
	ldi	r21, 2			;Y = 2
	ldi	r22, 0x9c		;Antennen-Symbol
	in	r16, portb		;PortB-Status holen
	bst	r16, 5			;Bluetooth aktiv?
	brtc	m12020			;nein -> weiter
	ldi	r22, 0x96		;sonst Antennensymbol + Bluetooth
m12020:	call	asclcd			;ausgeben
;
	ldi	r20, 214		;X = 214
	lds	r16, pccmd		;laufendes PC-Kommando holen
	tst	r16			;PC-bertragung aktiv?
	breq	m12030			;nein -> weiter
	ldi	r22, 0x9b		;sonst Blitzsymbol laden
	rjmp	m12040
;
m12030: bst	r25, 4			;DCF77-Signalstatus holen
	ldi	r22, 0x9e		;Zeichen fr 'aus' laden
	bld	r22, 0			;DCF77-Signalstatus einsetzen
m12040:	call	asclcd			;DCF77-Signalstatus ausgeben
;
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 7			;Status Abwesend?
	brtc	m12050			;nein -> weiter
	ldi	r22, 0xad		;sonst Schloss-Symbol laden
	rjmp	m12060			;ausgeben
m12050:	bst	r25, 5			;Lautsprecher-Status holen
	ldi	r22, 0x9e		;Zeichen fr 'aus' laden
	bld	r22, 0			;Lautsprecher-Status einsetzen
m12060:	ldi	r20, 231		;X = 231
	call	asclcd			;Lautsprecher-Status ausgeben
;
; Infozeile ausgeben: Uhrzeit
;
	ldi	r20, 176		;X = 176
	ldi	r21, 2			;Y = 2, Position Uhrzeit
	lds	r22, hourh		;Stunden Zehner holen
	ori	r22, 0x30		;in ASCII wandeln
	cpi	r22, 0x30		;Stunden Zehner='0'?
	brne	m12100			;nein -> ausgeben
	ldi	r22, ' '		;sonst Leerzeichen
m12100:	call	asclcd			;ausgeben
	lds	r22, hourl		;Stunden Einer holen
	ori	r22, 0x30		;in ASCII wandeln
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Doppelpunkt als Trennzeichen
	call	asclcd			;ausgeben
	lds	r22, minuth		;Minuten Zehner holen
	ori	r22, 0x30		;in ASCII wandeln
	call	asclcd			;ausgeben
	lds	r22, minutl		;Minuten Zehner holen
	ori	r22, 0x30		;in ASCII wandeln
	call	asclcd			;ausgeben
;
; Infozeile ausgeben: Fehlertext
;
	lds	r16, mennum		;aktuelle Mennummer holen
	tst	r16			;Hauptmen?
	breq	m12110			;ja -> Fehler/Datum ausgeben
	rjmp	m12300			;sonst berspringen
;
m12110:	ldi	r20, 2			;X = 2, Y = 2, Position fr Datum
	ldi	r21, 2			;bzw. Fehlertext
	lds	r16, errflg		;Fehler-Flags holen
	lds	r17, errmsk		;Fehlermaske holen
	tst	r17			;Maske vorhanden?
	breq	m12120			;nein -> berspringen
	and	r16, r17		;sonst Maske ber Alarme legen
m12120:	ldi	r22, 0xf5		;Fehlertext "Pointer-Fehler"
	bst	r16, 5			;Pointer-Flag?
	brts	m12130			;ja -> ausgeben
	ldi	r22, 0xf4		;Fehlertext "Echtzeituhr-Fehler"
	bst	r16, 4			;Echtzeituhrfehler-Flag?
	brts	m12130			;ja -> ausgeben
	ldi	r22, 0xf3		;Fehlertext "Datenspeicherfehler"
	bst	r16, 3			;Datenspeicherfehler-Flag?
	brts	m12130			;ja -> ausgeben
	ldi	r22, 0xf2		;Fehlertext "Konfigurationsfehler"
	bst	r16, 2			;Konfigurationsfehler-Flag?
	brts	m12130			;ja -> ausgeben
	ldi	r22, 0xf1		;Fehlertext "kein Datenempfang"
	bst	r16, 1			;Datenempfangsfehler-Flag?
	brts	m12130			;ja -> ausgeben
	ldi	r22, 0xf0		;Fehlertext "Uhr nicht eingestellt"
	bst	r16, 0			;Uhreneinstellungs-Flag?
        brts    m12130                  ;ja -> ausgeben
        ldi     r22, 0xef               ;Fehlertext "Log-Tabellen-Fehler"
        bst     r16, 6                  ;Log-Tabellen-Fehler-Flag?
	brtc	m12200			;nein -> Datum ausgeben
;
m12130:	ldi	r17, 0x01		;Inversschrift einstellen
	mov	txflag, r17		;Textflag setzen
	call	strlcd			;Fehlertext ausgeben
	rjmp	m12240			;Zeile auffllen
;
; Infozeile ausgeben: Wochentag und Datum
;
m12200:	bst	r16, 7			;Fehlererinnerung gesetzt?
	brtc	m12210			;nein -> weiter
	ldi	r17, 0x02		;sonst Unterstreichung einstellen
	mov	txflag, r17		;und Textflag setzen
m12210:	lds	r22, dayw		;Wochentag holen
	call	strlcd			;Wochtag ausgeben
	ldi	r22, ','		;Komma
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r22, dayh		;Kalendertag Zehner holen
	cpi	r22, 0			;Kalendertag Zehner = 0?
	breq	m12220			;ja -> keine Ausgabe
	ori	r22, 0x30		;sonst in ASCII wandeln
	call	asclcd			;ausgeben
m12220:	lds	r22, dayl		;Kalendertag Einer holen
	ori	r22, 0x30		;in ASCII wandeln
	call	asclcd			;ausgeben
	ldi	r22, '.'		;Punkt
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r23, monthh		;Monat Zehner holen
	tst	r23			;Monat Zehner = 0?
	breq	m12230			;ja -> weiter
	ldi	r23, 10			;sonst auf 10 setzen
m12230:	lds	r22, monthl		;Monat Einer holen
	add	r22, r23		;Zehner addieren
	dec	r22			;korrigieren
	ori	r22, 0x10		;Textnummer generieren
	call	strlcd			;Monatsname ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	ldi	r22, year3		;Jahr Tausender holen
	call	asclcd			;ausgeben
	ldi	r22, year2		;Jahr Hunderter holen
	call	asclcd			;ausgeben
	lds	r22, yearh		;Jahr Zehner holen
	ori	r22, 0x30		;in ASCII wandeln
	call	asclcd			;ausgeben
	lds	r22, yearl		;Jahr Einer holen
	ori	r22, 0x30		;in ASCII wandeln
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;ausgeben
;
m12240:	clr	txflag			;Textflag lschen
	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 174		;Endwert X Datumbereich setzen
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
	rjmp	m13000
;
; Infozeile ausgeben: Men-berschrift
;
m12300:	call	men1rd			;Mentabellenheader lesen
	clr	txflag			;Textflag lschen
	ldi	r20, 2			;X = 2, Y = 2, Position fr
	ldi	r21, 2			;Men-berschrift
	ldi	r22, 0xb3		;Hinweis-Dreieck (Pfeil rechts)
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen (2 Pixel Abstand zum Text)
	call	asclcd			;ausgeben
	mov	r22, r12		;Mentext-Nummer holen
	cpi	r22, 10			;String-Nr = 10 (Grafik-Preset)?
	brne	m12310			;nein -> normal ausgeben
;
	ldi	r22, 0xa0		;sonst Zeichenkette "Preset"
	call	strlcd			;ausgeben
	lds	r18, grpres		;Grafik-Preset-Nummer holen
	inc	r18			;korrigieren
	call	bytasc			;in ASCII wandeln
	mov	r25, r16		;Einer sichern
	mov	r22, r17		;Zehner holen
	call	asclcd			;Zeichen ausgeben
	mov	r22, r25		;gesicherte Einer holen
	call	asclcd			;Zeichen ausgeben
	ldi	r22, ':'		;Trennzeichen laden
	call	asclcd			;Zeichen ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen laden
	call	asclcd			;Zeichen ausgeben
	ldi	r16, low(egrprt)
	ldi	r17, high(egrprt)	;Adresse Preset-Texte im EEPROM
	ldi	r19, 16			;Textlnge 16 Zeichen
	lds	r18, grpres		;Grafik-Preset-Nummer holen
	mul	r19, r18		;Offset fr Textposition berechnen
	add	r16, r0
	adc	r17, r1			;EEPROM-Adresse berechnen
	call	eeplcd			;EEPROM-Text ausgeben
	rjmp	m12240
;
m12310:	call	strlcd			;Mentext ausgeben
	lds	r16, menext		;Men-Erweiterung laden
	tst	r16			;zustzlichen Text ausgeben?
	breq	m12240			;nein -> Zeile auffllen
	ldi	r22, 0xa0		;sonst schmales Leerzeichen
	call	asclcd			;ausgeben
	ldi	r22, '-'		;Trennzeichen
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r22, menext		;Men-Erweiterung laden
	call	strlcd			;zustzlichen Text ausgeben
	rjmp	m12240			;Zeile auffllen
;
; Menzeile ausgeben: Text fr Taster 2 (links)
;
m13000:	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 3			;Menzeile neu ausgeben?
	brts	m13010			;ja -> bearbeiten
	rjmp	m14000			;sonst berspringen
;
m13010:	cbr	r16, 0x08		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
;
	clr	txflag			;Textflag lschen
	ldi	r20, 2			;X = 2, Y = 115, Position fr
	ldi	r21, 115		;Menzeile, linker Rand
	lds	r16, mennum		;aktuelle Mennummer holen
	tst	r16			;Hauptmen (0)?
	breq	m13020			;ja -> bearbeiten
	ldi	r22, 0x1f		;sonst Text "Zurck"
	rjmp	m13040			;ausgeben
;
m13020:	lds	r16, xflags		;verschiedene Flags holen
	bst	r16, 0			;sind Signale aktiv?
	brts	m13030			;ja -> Menzeile setzen
	bst	r16, 1			;sind Erinnerungen gesetzt?
	brtc	m13050			;nein -> Menzeile setzen
	ldi	r22, 0x1d		;sonst Text "Erinnerungen
	rjmp	m13040			;lschen" ausgeben
m13030:	ldi	r22, 0x1e		;Text "Signal ausschalten"
m13040:	call	strlcd			;ausgeben
m13050:	mov	r13, r20		;aktuelle X-Position speichern
;
; Menzeile ausgeben: Text fr Taster 1 (rechts)
;
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brtc	m13060			;nein -> normale Men-Option
	rjmp	m13200			;sonst als Impulsgeberwert ausgeben
;
m13060:	call	men1rd			;sonst Mentabellenheader lesen
	call	men2rd			;Mentabellenoption lesen
	lds	r10, menopt		;aktuelle Menoption holen
	elpm	r12, z+			;String-Nr fr Mentext holen
	mov	r22, r12		;String-Nr kopieren
	cpi	r22, 10			;String-Nr = 10 (Grafik-Preset)?
	brne	m13100			;nein -> normal ausgeben
;
; Menzeile ausgeben: Grafik-Preset Namen ausgeben
;
	ldi	r16, low(egrprt)
	ldi	r17, high(egrprt)	;Adresse Preset-Texte im EEPROM
	ldi	r19, 16			;Textlnge 16 Zeichen
	mul	r19, r10		;Offset fr Textposition
	add	r16, r0
	adc	r17, r1			;EEPROM-Adresse berechnen
	movw	r8, r16			;EEPROM-Adresse sichern
	call	eeplen			;Lnge des Textes ermitteln
	ldi	r18, 19			;Lnge fr Zusatz "01: "
	add	r18, r20		;Gesamtlnge ermitteln
;
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposit. holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Text setzen
;
	mov	r18, r10		;Men-Option holen
	inc	r18			;korrigieren
	call	bytasc			;in ASCII wandeln
	mov	r25, r16		;Einer sichern
	mov	r22, r17		;Zehner holen
	call	asclcd			;Zeichen ausgeben
	mov	r22, r25		;gesicherte Einer holen
	call	asclcd			;Zeichen ausgeben
	ldi	r22, ':'		;Trennzeichen laden
	call	asclcd			;Zeichen ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen laden
	call	asclcd			;Zeichen ausgeben
	movw	r16, r8			;gesicherte EEPROM-Adresse holen
	ldi	r19, 16			;Preset-Textlnge im EEPROM
	call	eeplcd			;EEPROM-Text ausgeben
	rjmp	m13110
;
; Menzeile ausgeben: Men-Option ausgeben
;
m13100:	call	strlng			;Stringlnge ermitteln
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Text setzen
	mov	r22, r12		;gesicherte String-Nummer holen
	call	strlcd			;String ausgeben
;
m13110:	ldi	r20, 224		;X-Position fr Pfeil links
	ldi	r22, 0xb0		;Pfeil links hohl laden
	tst	r10			;erste Option (0)?
	breq	m13120			;ja -> berspringen
	ldi	r22, 0xb1		;sonst Pfeil links voll laden
m13120:	call	asclcd			;ausgeben
	ldi	r20, 231		;X-Position fr Pfeil rechts
	ldi	r22, 0xb2		;Pfeil rechts hohl laden
	mov	r16, r10		;aktuelle Menoption holen
	inc	r16			;um 1 vermindern
	cp	r16, r11		;letzte Option?
	breq	m13130			;ja -> berspringen
	ldi	r22, 0xb3		;sonst Pfeil rechts voll laden
m13130:	call	asclcd			;ausgeben
	rjmp	m14000
;
; Menzeile ausgeben: Impulsgeberwert 3- oder 4-stellig dezimal ausgeben
;
m13200:	bst	r16, 1			;Ausgabe 3-stellig dezimal?
	brts	m13230			;ja -> bearbeiten
	bst	r16, 4			;Ausgabe 4-stellig dezimal?
	brts	m13210			;ja -> bearbeiten
	rjmp	m13600			;sonst weiter testen
;
m13210:	lds	r18, dimode		;Anzeigemodus holen
	cpi	r18, 24			;Modus Grafik/Fadenkreuz?
	brne	m13220			;nein -> Wert normal ausgeben
	rjmp	m13400			;sonst Temperaturwerte ausgeben
;
m13220:	lds	r17, encvah		;Impulsgeberwert H holen
	rjmp	m13240
m13230:	clr	r17			;H-Byte lschen
m13240:	lds	r16,encval		;Impulsgeberwert holen
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	clr	r18			;Anfangswert fr Stringlnge
	ser	r19			;Flag fr Vornullenlschung
;
	mov	r16, r4			;erste Stelle holen
	cpi	r16, 0x30		;erste Stelle = '0'?
	breq	m13250			;ja -> weiter
	clr	r19			;sonst Vornull-Flag lschen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
m13250:	mov	r16, r5			;zweite Stelle holen
	tst	r19			;Vornullen lschen?
	breq	m13260			;nein -> Nulltest berspringen
	cpi	r16, 0x30		;zweite Stelle = '0'?
	breq	m13270			;ja -> berspringen
	clr	r19			;sonst Vornull-Flag lschen
m13260:	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
m13270:	mov	r16, r6			;dritte Stelle holen
	tst	r19			;Vornullen lschen?
	breq	m13280			;nein -> Nulltest berspringen
	cpi	r16, 0x30		;dritte Stelle = '0'?
	breq	m13290			;ja -> berspringen
	clr	r19			;sonst Vornull-Flag lschen
m13280:	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
m13290:	mov	r16, r7			;vierte Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
;
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Textausgabe
;
	ser	r25			;Flag fr Vornullenlschung
	mov	r22, r4			;erste Stelle holen
	cpi	r22, 0x30		;erste Stelle = '0'?
	breq	m13300			;ja -> weiter
	clr	r25			;sonst Vornull-Flag lschen
	call	asclcd			;erste Stelle ausgeben
m13300:	mov	r22, r5			;zweite Stelle holen
	tst	r25			;Vornullen lschen?
	breq	m13310			;nein -> Nulltest berspringen
	cpi	r22, 0x30		;zweite Stelle = '0'?
	breq	m13320			;ja -> berspringen
	clr	r25			;sonst Vornull-Flag lschen
m13310:	call	asclcd			;zweite Stelle ausgeben
m13320:	mov	r22, r6			;dritte Stelle holen
	tst	r25			;Vornullen lschen?
	breq	m13330			;nein -> Nulltest berspringen
	cpi	r22, 0x30		;dritte Stelle = '0'?
	breq	m13340			;ja -> berspringen
m13330:	call	asclcd			;sonst zweite Stelle ausgeben
m13340:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;vierte Stelle ausgeben
;
m13350:	ldi	r20, 224		;X-Position fr Pfeil links
	ldi	r22, 0xb0		;Pfeil links hohl laden
	lds	r16, encval		;Impulsgeberwert holen
	lds	r17, encmin		;Impulsgeber Minimalwert holen
	cp	r16, r17		;Minimalwert erreicht?
	breq	m13360			;ja -> berspringen
	ldi	r22, 0xb1		;sonst Pfeil links voll laden
m13360:	call	asclcd			;ausgeben
	ldi	r20, 231		;X-Position fr Pfeil rechts
	ldi	r22, 0xb2		;Pfeil rechts hohl laden
	lds	r16, encval		;Impulsgeberwert holen
	lds	r17, encmax		;Impulsgeber Maximalwert holen
	cp	r16, r17		;Maximalwert erreicht?
	breq	m13370			;ja -> berspringen
	ldi	r22, 0xb3		;sonst Pfeil rechts voll laden
m13370:	call	asclcd			;ausgeben
	rjmp	m14000
;
; Menzeile ausgeben: Datum, Zeit und Temperaturwerte anstatt des 4-stelligen Impulsgeberwertes ausgeben (fr Grafik/Fadenkreuz)
;
m13400:	ldi	r17, 127		;Position 15min/Einzeldiagamm
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppeldiagramm?
	brtc	m13410			;nein -> Position korrekt
	ldi	r17, 93			;sonst Position Doppeldiagramm
m13410:	lds	r16, grmod1		;Grafikmodus Diagramm 1 holen
	tst	r16			;15min-Werte?
	breq	m13420			;ja -> Position korrekt
	ldi	r16, 31			;sonst Position korrigieren,
	add	r17, r16		;nur Datum anzeigen
m13420:	mov	r20, r17		;ermittelte Position kopieren
	ldi	r21, 126		;bis dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
;
	ldi	xl, low(grbuff)
	ldi	xh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	lds	r16, encval		;neue Fadenkreuzposition holen
	ldi	r17, 8
	mul	r17, r16		;Puffer-Offset berechnen
	add	xl, r0			;Pufferadresse ermitteln
	adc	xh, r1			;bertrag
	ld	r8, x+
	ld	r9, x+
	ld	r10, x+
	ld	r11, x+			;Zeitdaten holen
;
	ldi	r21, 115		;Y-Position fr Textausgabe
	mov	r16, r8
	and	r16, r9
	and	r16, r10
	and	r16, r11		;Zeitdaten verknpfen
	cpi	r16, 0xff		;Zeit gltig?
	brne	m13430			;ja -> Daten ausgeben
;
	ldi	r22, 0xaa		;String "keine Daten"
	call	strlng			;Stringlnge ermitteln
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Text setzen
	ldi	r22, 0xaa		;String "keine Daten"
	call	strlcd			;String ausgeben
	rjmp	m13350			;Richtungspfeile ausgeben
;
m13430:	call	grelcd			;Datum ausgeben
	ldi	r22, '.'		;zustzlichen Punkt
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
;
	lds	r16, grmod1		;Grafikmodus Diagramm 1 holen
	tst	r16			;15min-Werte?
	brne	m13440			;nein -> keine Zeit ausgeben
	call	grelc2			;sonst Zeit ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
;
m13440:	ld	r16, x+
	ld	r17, x+			;Sensorwert 1 holen
	cpi	r17, 0x80		;gltiger Wert?
	brne	m13450			;ja -> ausgeben
        ldi     r16, ' '                ;sonst Leerzeichen
	mov	r4, r16			;als Hunderter setzen
	ldi	r16, '-'		;Strich
	mov	r5, r16			;als Zehner setzen
	mov	r6, r16			;als Einer setzen
	mov	r7, r16			;als Zehntel setzen
	rjmp	m13480			;ausgeben
;
m13450: lds     r18, grse11             ;Grafik 1 Sensor holen
        cpi     r18, 28                 ;Temperatur- oder Drucksensor?
        brcc    m13460                  ;nein -> weiter testen
	call	temasc			;Temperatur oder Druck in ASCII wandeln
        rjmp    m13480
m13460: cpi     r18, 31                 ;Luftfeuchtigkeitssensor?
        brcc    m13470                  ;nein -> Batteriewert
        movw    r12, r20                ;Pixelkoordinaten zwischenspeichern
        clr     r18                     ;Byte 3 fr Divident = 0 setzen
        ldi     r20, 10                 ;Divisor L = 10
        clr     r21                     ;Divisor H = 0
        call    divide                  ;Luftfeuchtigkeitswert / 10
        movw    r20, r12                ;Pixelkoordinaten wieder herstellen
	call	humasc			;Luftfeuchtigkeit in ASCII wandeln
        rjmp    m13480
;
m13470:	call	batasc			;Batteriewert in ASCII wandeln
m13480:	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
        lds     r16, grse11             ;Grafik 1 Sensor holen
        cpi     r16, 31                 ;Batteriewert?
        brne    m13490                  ;nein -> weiter
        subi    r20, 3                  ;sonst X-Koordinate - 3 Pixel
m13490:	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
;
        lds     r16, grse11             ;Grafik 1 Sensor holen
        cpi     r16, 31                 ;Batteriewert?
        brne    m13510                  ;nein -> weiter
	ldi	r22, '.'		;sonst Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m13500			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m13500: call    asclcd                  ;ausgeben
m13510:	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
;
        lds     r16, grse11             ;Grafik 1 Sensor holen
        cpi     r16, 27                 ;Temperatursensor?
        brcc    m13530                  ;nein -> weiter
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m13520			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m13520:	call	asclcd			;ausgeben
m13530:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
;
        lds     r16, grse11             ;Grafik 1 Sensor holen
	ldi	r22, 0xa5		;Grad-Zeichen
        cpi     r16, 27                 ;Temperatursensor?
        brcs    m13540                  ;ja -> ausgeben
	ldi	r22, 'h'		;Hektopascal
        cpi     r16, 28                 ;Luftdrucksensor?
        brcs    m13540                  ;ja -> ausgeben
	ldi	r22, '%'		;Prozent-Zeichen
        cpi     r16, 31                 ;Luftfeuchtigkeitssensor?
        brcs    m13540                  ;ja -> ausgeben
        ldi     r22, 'V'                ;sonst Volt
;
m13540:	call	asclcd			;ausgeben
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppeldiagramm?
        brts    m13550                  ;ja -> bearbeiten
	rjmp	m135g0			;sonst Endezeichen ausgeben
;
m13550:	ldi	r22, 0xa0		;sonst schmales Leerzeichen
	call	asclcd			;ausgeben
;
	ld	r16, x+
	ld	r17, x+			;Temperaturwert 2 holen
	cpi	r17, 0x80		;gltiger Wert?
	brne	m13560			;ja -> ausgeben
	ldi	r16, ' '		;sonst Leerzeichen
	mov	r4, r16			;als Hunderter setzen
	ldi	r16, '-'		;Strich
	mov	r5, r16			;als Zehner setzen
	mov	r6, r16			;als Einer setzen
	mov	r7, r16			;als Zehntel setzen
	rjmp	m13590			;ausgeben
;
m13560: lds     r18, grse21             ;Grafik 2 Sensor holen
        cpi     r18, 28                 ;Temperatur- oder Drucksensor?
        brcc    m13570                  ;nein -> weiter testen
	call	temasc			;Temperatur oder Druck in ASCII wandeln
        rjmp    m13590
m13570: cpi     r18, 31                 ;Luftfeuchtigkeitssensor?
        brcc    m13580                  ;nein -> Batteriewert
        movw    r12, r20                ;Pixelkoordinaten zwischenspeichern
        clr     r18                     ;Byte 3 fr Divident = 0 setzen
        ldi     r20, 10                 ;Divisor L = 10
        clr     r21                     ;Divisor H = 0
        call    divide                  ;Luftfeuchtigkeitswert / 10
        movw    r20, r12                ;Pixelkoordinaten wieder herstellen
	call	humasc			;Luftfeuchtigkeit in ASCII wandeln
        rjmp    m13590
;
m13580:	call	batasc			;Batteriewert in ASCII wandeln
m13590:	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
        lds     r16, grse21             ;Grafik 2 Sensor holen
        cpi     r16, 31                 ;Batteriewert?
        brne    m135a0                  ;nein -> weiter
        subi    r20, 3                  ;sonst X-Koordinate - 3 Pixel
m135a0:	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
;
        lds     r16, grse21             ;Grafik 2 Sensor holen
        cpi     r16, 31                 ;Batteriewert?
        brne    m135c0                  ;nein -> weiter
	ldi	r22, '.'		;sonst Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m135b0			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m135b0: call    asclcd                  ;ausgeben
m135c0:	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
;
        lds     r16, grse21             ;Grafik 2 Sensor holen
        cpi     r16, 27                 ;Temperatursensor?
        brcc    m135e0                  ;nein -> weiter
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m135d0			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m135d0:	call	asclcd			;ausgeben
m135e0:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
;
        lds     r16, grse21             ;Grafik 2 Sensor holen
	ldi	r22, 0xa5		;Grad-Zeichen
        cpi     r16, 27                 ;Temperatursensor?
        brcs    m135f0                  ;ja -> ausgeben
	ldi	r22, 'h'		;Hektopascal
        cpi     r16, 28                 ;Luftdrucksensor?
        brcs    m135f0                  ;ja -> ausgeben
	ldi	r22, '%'		;Prozent-Zeichen
        cpi     r16, 31                 ;Luftfeuchtigkeitssensor?
        brcs    m135f0                  ;ja -> ausgeben
        ldi     r22, 'V'                ;sonst Volt
;
m135f0:	call	asclcd			;ausgeben
m135g0:	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;ausgeben
	rjmp	m13350			;Richtungspfeile ausgeben
;
; Menzeile ausgeben: Impulsgeberwert als Temperaturwert ausgeben
;
m13600:	bst	r16, 2			;Ausgabe als Temperaturwert?
	brts	m13610			;ja -> bearbeiten
	rjmp	m13700			;sonst weiter testen
;
m13610:	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	mov	r12, r13		;X-Anfangsposition sichern
	call	temasc			;Wert in ASCII wandeln
	mov	r16, r4			;erste Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	mov	r18, r16		;Lnge speichern
	mov	r16, r5			;zweite Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
	mov	r16, r6			;dritte Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
	ldi	r16, '.'		;Trennzeichen (Punkt) laden
	lds	r17, confg1		;Konfigurationsbyte 1 holen
	bst	r17, 1			;Trennzeichen Punkt?
	brtc	m13620			;ja -> weiter, sonst
	ldi	r16, ','		;Trennzeichen (Komma) laden
m13620:	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
	mov	r16, r7			;vierte Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
;
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r12		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Textausgabe
;
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;erste Stelle ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;zweite Stelle ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;dritte Stelle ausgeben
	ldi	r22, '.'		;Trennzeichen (Punkt) laden
	lds	r17, confg1		;Konfigurationsbyte 1 holen
	bst	r17, 1			;Trennzeichen Punkt?
	brtc	m13630			;ja -> weiter, sonst
	ldi	r22, ','		;Trennzeichen (Komma) laden
m13630:	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;vierte Stelle ausgeben
;
	ldi	r20, 224		;X-Position fr Pfeil links
	ldi	r22, 0xb0		;Pfeil links hohl laden
	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	ldi	r18, low(mintmp)	;Minimaltemperatur L laden
	ldi	r19, high(mintmp)	;Minimaltemperatur H laden
	sub	r18, r16
	sbc	r19, r17		;Minimalwert erreicht?
	breq	m13640			;ja -> berspringen
	ldi	r22, 0xb1		;sonst Pfeil links voll laden
m13640:	call	asclcd			;ausgeben
	ldi	r20, 231		;X-Position fr Pfeil rechts
	ldi	r22, 0xb2		;Pfeil rechts hohl laden
	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	ldi	r18, low(maxtmp)	;Maximaltemperatur L laden
	ldi	r19, high(maxtmp)	;Maximaltemperatur H laden
	sub	r18, r16
	sbc	r19, r17		;Maximalwert erreicht?
	breq	m13650			;ja -> berspringen
	ldi	r22, 0xb3		;sonst Pfeil rechts voll laden
m13650:	call	asclcd			;ausgeben
	rjmp	m14000			;sonst weiter testen
;
; Menzeile ausgeben: Impulsgeberwert als ASCII-Zeichen ausgeben
;
m13700:	bst	r16, 3			;Ausgabe als ASCII-Zeichen?
	brts	m13710			;ja -> bearbeiten
	rjmp	m13800			;sonst weiter testen
;
m13710:	lds	r18, encval		;Impulsgeberwert holen
        mov     r16, r18                ;Wert kopieren
        call    getchr                  ;entsprechendes Zeichen aus Liste holen
        lds     r17, asclst             ;Anzahl der Zeichen holen
	cp	r18, r17		;ASCII Maximalwert (Delete-Zeichen)?
	brne	m13720			;nein -> weiter
	ldi	r16, 0xaf		;durch Lschzeichen ersetzen
m13720:	inc     r17
        cp	r18, r17		;ASCII Maximalwert + 1 (Ok-Zeichen)?
	brne	m13730			;nein -> weiter
	ldi	r16, 0xae		;durch Ok-Zeichen ersetzen
m13730: mov     r4, r16                 ;Zeichen zwischenspeichern
	call	asclen			;Zeichenlnge ermitteln
;
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r16		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Textausgabe
;
	mov	r22, r4		        ;gespeichertes Zeichen holen
	call	asclcd			;ausgeben
	ldi	r20, 224		;X-Position fr Pfeil links
	ldi	r22, 0xb0		;Pfeil links hohl laden
	lds	r16, encval		;Impulsgeberwert holen
        tst     r16		        ;ASCII Minimalwert erreicht?
	breq	m13760			;ja -> berspringen
	ldi	r22, 0xb1		;sonst Pfeil links voll laden
m13760:	call	asclcd			;ausgeben
	ldi	r20, 231		;X-Position fr Pfeil rechts
	ldi	r22, 0xb2		;Pfeil rechts hohl laden
	lds	r16, encval		;Impulsgeberwert holen
        lds     r17, asclst             ;Anzahl der Zeichen holen
        inc     r17                     ;Del und Ok Zeichen einbeziehen
	cp	r16, r17		;ASCII Maximalwert erreicht?
	breq	m13770			;ja -> berspringen
	ldi	r22, 0xb3		;sonst Pfeil rechts voll laden
m13770:	call	asclcd			;ausgeben
        rjmp    m14000
;
; Menzeile ausgeben: Impulsgeberwert im Zeitformat ausgeben
;
m13800:	bst	r16, 5			;Ausgabe im Zeitformat?
	brts	m13810			;ja -> bearbeiten
	rjmp	m13900			;sonst weiter testen
;
m13810:	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	call	timasc			;Wert in ASCII wandeln
	mov	r16, r4			;erste Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	mov	r18, r16		;Lnge speichern
	mov	r16, r5			;zweite Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
	ldi	r16, ':'		;Doppelpunkt laden
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
	mov	r16, r6			;dritte Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
	mov	r16, r7			;vierte Stelle holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;und zur Gesamtlnge addieren
;
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Textausgabe
;
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;erste Stelle ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;zweite Stelle ausgeben
	ldi	r22, ':'		;Trennzeichen (Doppelpunkt)
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;dritte Stelle ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;vierte Stelle ausgeben
;
	ldi	r20, 224		;X-Position fr Pfeil links
	ldi	r22, 0xb1		;Pfeil links voll laden
	call	asclcd			;ausgeben
	ldi	r20, 231		;X-Position fr Pfeil rechts
	ldi	r22, 0xb3		;Pfeil rechts voll laden
	call	asclcd			;ausgeben
        rjmp    m14000
;
; Menzeile ausgeben: Impulsgeberwert als Sensorname ausgeben

m13900:	bst	r16, 6			;Ausgabe als Sensorname?
	brts	m13910			;ja -> bearbeiten
	rjmp	m14000			;sonst Ende
;
m13910:	lds	r16, encval		;Impulsgeberwert holen
        dec     r16                     ;korrigieren
	cpi	r16, 0xff 		;Anzeige-Position leer?
	breq	m13930			;ja -> weiter
;
        cpi     r16, 31                 ;Sensorname fr grafischen Luftdruck?
        brne    m13920                  ;nein -> berspringen
        ldi     r16, 27                 ;sonst Name fr Luftdruck verwenden
        ldi     r17, 2                  ;Textflags fr
        mov     txflag, r17             ;Unterstreichung setzen
;
m13920:	ldi	r19, 9  		;Sensorname 9 Zeichen
	mul     r16, r19                ;Textposition ermitteln
        ldi     r16, low(esentx)        ;EEPROM-Adresse Sensornamen L
        ldi     r17, high(esentx)       ;EEPROM-Adresse Sensornamen H
        add     r16, r0			;Offset zur Adresse L addieren
        adc     r17, r1			;Offset zur Adresse H addieren
        movw    xl, r16                 ;EEPROM-Adresse sichern
        call    eeplen                  ;Lnge des Sensornamens ermitteln
        mov     r18, r20                ;Lnge kopieren
;
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Textausgabe
        movw    r16, xl                 ;gesicherte EEPROM-Adresse holen
        ldi     r19, 9                  ;Sensorname 9 Zeichen
        call    eeplcd                  ;Sensorname ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
        clr     txflag                  ;Textflags auf normal setzen
        rjmp    m13350                  ;Richtungs-Pfeile ausgeben
;
m13930: ldi     r22, 0x46               ;Text "[leer]"
        call    strlng                  ;Lnge ermitteln
	ldi	r20, 222		;Endposition der Menzeile
	sub	r20, r18		;Text-Anfang berechnen, bis
	ldi	r21, 126		;dahin alle Pixel lschen
	mov	r18, r13		;gesicherte Anfangsposition holen
	ldi	r19, 115		;Y-Anfangsposition setzen
	call	clrwnd			;Text-Zwischenraum lschen
	ldi	r21, 115		;Y-Position fr Textausgabe
        ldi     r22, 0x46               ;Text "[leer]"
        call    strlcd                  ;ausgeben
        rjmp    m13350                  ;Richtungs-Pfeile ausgeben
;
; Hauptanzeige neu ausgeben: zustzliche Linien zeichnen, Seitennummer anzeigen, Flags setzen
;
m14000:	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 0			;Normalanzeige?
	brts	m14010			;ja -> bearbeiten
	rjmp	m15000			;sonst weiter testen
m14010:	bst	r16, 1			;Anzeige neu ausgeben?
        brts    m14020                  ;ja -> weiter
	rjmp	m14100			;sonst berspringen
;
m14020:	cbr	r16, 1 << 1		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
;
	ldi	r19, 15 | 0x80		;Y = 15
m14030:	ldi	r18, 87			;X = 87, erste Trennlinie
	call	pixlcd			;senkrechte Linie zeichnen
	ldi	r18, 174		;X = 174, zweite Trennlinie
	call	pixlcd			;senkrechte Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	cpi	r19, 112 | 0x80		;alle Pixel ausgegeben?
	brne	m14030			;nein -> Schleife
;
	ldi	r18, 175		;X = 175
	ldi	r19, 64 | 0x80		;Y = 64, Trennlinie fr Groanzeige
m14040:	call	pixlcd			;Linie zeichnen
	inc	r18			;X-Koordinate erhhen
	cpi	r18, 239		;alle Pixel ausgegeben?
	brne	m14040			;nein -> Schleife
;
        ldi     r16, 1                  ;Seitennummer invers anzeigen
        mov     txflag, r16             ;Textflags invers setzen
        ldi     r22, '1'                ;Text fr Anzeige-Seite 1
        lds     r16, xflag4             ;verschiedene Flags 4 holen
        bst     r16, 4                  ;Anzeige-Seite 2 eingeschaltet?
        brtc    m14050                  ;nein -> berspringen
        ldi     r22, '2'                ;sonst Text fr Anzeige-Seite 2
m14050: mov     r16, r22                ;Text kopieren
	call	asclen			;Zeichenlnge bestimmen
	ldi	r20, 238		;X-Position rechtsbndig
	sub	r20, r16		;berechnen
        ldi     r21, 65                 ;Y = 65
	call	asclcd			;Zeichen ausgeben
        ldi     r22, 0x7f               ;Endezeichen
	call	asclcd			;ausgeben
        clr     txflag
        call    newsen                  ;alle Sensorwerte neu ausgeben
;
	ldi	xl, low(almflg)
	ldi	xh, high(almflg)	;Zeiger auf Alarmflags
	ldi	r17, 4			;4 Flags bearbeiten
m14060:	ld	r16, x			;Alarmflag holen
	bst	r16, 7			;Alarm inaktiv?
	brts	m14070			;ja -> Flags nicht ndern
	sbr	r16, 0x01		;Flag fr Neuausgabe setzen
m14070:	st	x+, r16			;Alarmflag wieder speichern
	dec	r17			;alle Flags bearbeitet?
	brne	m14060			;nein -> Schleife
;
; Hauptanzeige: Sensornamen aus EEPROM ausgeben - 16 Anzeigepositionen
;
m14100:	ldi	r25, 0			;Anzeigepositionszhler = 0
	ldi	xl, low(postab)
	ldi	xh, high(postab)	;Zeiger auf Positions-Tabelle fr Anzeige-Seite 1
        lds     r16, xflag4		;verschiedene Flags 4 holen
        bst     r16, 4                  ;Anzeige-Seite 2 aktiv?
        brtc    m14110                  ;nein -> berspringen
	ldi	xl, low(postab + 16)    ;sonst
	ldi	xh, high(postab + 16)	;Zeiger auf Positions-Tabelle fr Anzeige-Seite 2
;
m14110: clr     r15                     ;H-Byte lschen
        ld      r16, x                  ;Sensornummer holen
        bst     r16, 5                  ;Anzeigeposition leer?
        brts    m14130                  ;ja -> berspringen
        bst     r16, 7                  ;Sensorname neu ausgeben?
        brtc    m14130                  ;nein -> berspringen
;
	clr	txflag			;Textflags lschen
        cbr     r16, 0x80               ;sonst Ausgabe-Flag lschen
        st      x, r16                  ;Sensornummer wieder speichern
        andi    r16, 0x1f               ;Sensornummer filtern
        mov     r13, r16                ;und zwischenspeichern
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
        add     yl, r16                 ;Tabellenplatz berechnen
        adc     yh, r15                 ;bertrag
        cpi     r16, 31                 ;Sensornummer = 31?
        brne    m14120                  ;nein -> berspringen
        ldi     r16, 27                 ;sonst erfolgen alle weiteren Zugriffe auf die Daten von Sensor 27
        mov     r13, r16                ;korrigierte Sensornummer zwischenspeichern
m14120: mov     r14, r16                ;Sensornummer kopieren
        add     r14, r14                ;und verdoppeln, r15:r14 enthlt Zeiger-Offset fr Zugriff auf Wertetabellen
	ld	r16, y			;Sensor-Flags holen
	bst	r16, 7			;Sensor inaktiv?
	brtc	m14140			;nein -> weiter testen
m14130:	rjmp	m14300			;sonst Ausgabe berspringen

m14140: mov     r18, r13                ;zwischengespeicherte Sensornummer holen
        cpi     r18, 27                 ;Sensornummer > 26 (kein Temperatursensor)?
        brcc    m14200                  ;ja -> Limit-Prfung berspringen
;
	andi	r16, 0x0c		;ber- oder Unterschreitung?
	breq	m14160			;nein -> weiter testen
	ldi	r16, 0x01		;sonst invers darstellen
	rjmp	m14170
m14160:	ld	r16, y			;Flags nochmals holen
	andi	r16, 0x30		;Erinnerungs-Flag(s) gesetzt?
	breq	m14180			;nein -> normal ausgeben
	ldi	r16, 0x02		;sonst unterstrichen darstellen
m14170:	mov	txflag, r16		;Textflags setzen
;
m14180:	lds	r18, xflags		;verschiedene Flags holen
	bst	r18, 2			;Minimal-Werte anzeigen?
	brtc	m14190			;nein -> weiter testen
;
	clr	txflag			;zunchst Textflags lschen
	ldi	zl, low(senlil)
	ldi	zh, high(senlil)	;Zeiger auf untere Temperatur-Limits
	add	zl, r14
	adc	zh, r15			;Tabellenplatz berechnen
	ld	r18, z			;unteres Temperatur-Limit L holen
	ldd	r19, z + 1		;unteres Temperatur-Limit H holen
	cpi	r19, 0x80		;gltiger Wert?
	breq	m14200			;nein -> Ende
	ldi	zl, low(semin1)
	ldi	zh, high(semin1)	;Zeiger auf Minimalwerte (Anzeige)
	add	zl, r14
	adc	zh, r15			;Tabellenplatz berechnen
	ld	r16, z			;Minimalwert L holen
	ldd	r17, z + 1		;Minimalwert H holen
	cpi	r17, 0x80		;gltiger Wert?
	breq	m14200			;nein -> Ende
;
	sub	r16, r18		;Minimalwert - Limit berechnen
	sbc	r17, r19		;Limit unterschritten?
	brpl	m14200			;nein -> weiter
	ldi	r16, 1			;sonst Text invers darstellen
	mov	txflag, r16		;Textflag wieder speichern
	rjmp	m14200
;
m14190:	bst	r18, 3			;Maximal-Werte anzeigen?
	brtc	m14200			;nein -> weiter
;
	clr	txflag			;zunchst Textflags lschen
	ldi	zl, low(senlih)
	ldi	zh, high(senlih)	;Zeiger auf obere Temperatur-Limits
	add	zl, r14
	adc	zh, r15			;Tabellenplatz berechnen
	ld	r18, z			;oberes Temperatur-Limit L holen
	ldd	r19, z + 1		;oberes Temperatur-Limit H holen
	cpi	r19, 0x80		;gltiger Wert?
	breq	m14200			;nein -> Ende
	ldi	zl, low(semax1)
	ldi	zh, high(semax1)	;Zeiger auf Maximalwerte (Anzeige)
	add	zl, r14
	adc	zh, r15			;Tabellenplatz berechnen
	ld	r16, z			;Maximalwert L holen
	ldd	r17, z + 1		;Maximalwert H holen
	cpi	r17, 0x80		;gltiger Wert?
	breq	m14200			;nein -> Ende
;
	sub	r18, r16		;Maximalwert - Limit berechnen
	sbc	r19, r17		;Limit unterschritten?
	brpl	m14200			;nein -> weiter
	ldi	r16, 1			;sonst Text invers darstellen
	mov	txflag, r16		;Textflag wieder speichern
;
m14200:	ldi	r20, 2			;X-Startposition Spalte 1
	ldi	r21, 16			;Y-Startposition
	cpi	r25, 8			;Temperatur in Spalte 1 (Anzeigeposition < 8)?
	brcs	m14210			;ja -> weiter
	ldi	r20, 89			;sonst X-Startposition Spalte 2
m14210:	mov	r16, r25		;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	ldi	r19, 9			;Sensornamenlnge 9 Zeichen
	mul	r19, r13		;Tabellenoffset entsprechend Sensornummer ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;Zeichen ausgeben
;
	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 52			;Endwert X erste Spalte setzen
	cpi	r18, 87			;zweite Spalte?
	brcs	m14220			;nein -> weiter
	ldi	r20, 139		;sonst Endwert X zweite Spalte
m14220:	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	bst	txflag, 0		;Invers-Darstellung?
	bld	r19, 7			;sonst Invers-Bit in den
	bld	r21, 7			;Koordinaten setzen
	call	clrwnd			;Bereich lschen oder setzen
;
m14300:	adiw	xl, 1			;Positions-Tabellen-Zeiger erhhen
	inc	r25			;nchste Anzeigeposition
	cpi	r25, 16			;alle Positionen bearbeitet?
	brcc	m14350			;ja -> weiter
	rjmp	m14110			;sonst Schleife
;
; Hauptanzeige: Sensornamen aus EEPROM ausgeben - Groanzeige
;
m14350: clr     txflag                  ;Textflags lschen
        ldi     xl, low(bigflg)
        ldi     xh, high(bigflg)        ;Zeiger auf Flags fr Groanzeige Seite 1
        lds     r17, xflag4             ;verschiedene Flags 4 holen
        sbrc    r17, 4                  ;Anzeige-Seite 2 aktiv? nein -> berspringen
        adiw    xl, 1                   ;sonst Zeiger auf Flags fr Groanzeige Seite 2
        ld      r16, x                  ;Flags fr Groanzeige holen
        bst     r16, 5                  ;Groanzeige inaktiv?
        brts    m14400                  ;ja -> berspringen
        bst     r16, 7                  ;Neuausgabe erforderlich?
        brtc    m14400                  ;nein -> berspringen
        cbr     r16, 0x80               ;sonst Ausgabe-Flag lschen
        st      x, r16                  ;und Flags wieder speichern
        andi    r16, 0x1f               ;Sensornummer fr Groanzeige filtern
;
        ldi     yl, low(senflg)
        ldi     yh, high(senflg)        ;Zeiger auf Sensorflags
        clr     r17                     ;H-Byte lschen
        add     yl, r16                 ;Tabellenposition ermitteln
        adc     yh, r17                 ;bertrag
        ld      r18, y                  ;Flags des gewhlten Groanzeige-Sensors holen
        bst     r18, 7                  ;Sensor inaktiv?
        brts    m14400                  ;ja -> berspringen
;
	ldi	r19, 9			;Sensornamenlnge 9 Zeichen
	mul	r16, r19		;Tabellenoffset entsprechend Sensornummer ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	ldi	r20, 176		;X-Koordinate Groanzeige
	ldi	r21, 66			;Y-Koordinate Groanzeige
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;Zeichen ausgeben
;
; Hauptanzeige: Sensorwerte ausgeben - 16 Anzeigepositionen
;
m14400:	ldi	r25, 0			;Anzeigepositionszhler = 0
	ldi	xl, low(postab)
	ldi	xh, high(postab)	;Zeiger auf Positions-Tabelle fr Anzeige-Seite 1
        lds     r16, xflag4		;verschiedene Flags 4 holen
        bst     r16, 4                  ;Anzeige-Seite 2 aktiv?
        brtc    m14410                  ;nein -> berspringen
	ldi	xl, low(postab + 16)    ;sonst
	ldi	xh, high(postab + 16)	;Zeiger auf Positions-Tabelle fr Anzeige-Seite 2
;
m14410: clr     r15                     ;H-Byte lschen
        ld      r16, x                  ;Sensornummer holen
        bst     r16, 5                  ;Anzeigeposition leer?
        brts    m14430                  ;ja -> berspringen
        bst     r16, 6                  ;Sensorwert neu ausgeben?
        brtc    m14430                  ;nein -> berspringen
;
        cbr     r16, 0x40               ;sonst Ausgabe-Flag lschen
        st      x, r16                  ;Sensornummer wieder speichern
        andi    r16, 0x1f               ;Sensornummer filtern
        mov     r10, r16                ;und zwischenspeichern
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
        add     yl, r16                 ;Tabellenplatz berechnen
        adc     yh, r15                 ;bertrag
m14420: mov     r14, r16                ;Sensornummer kopieren
        add     r14, r14                ;und verdoppeln, r15:r14 enthlt Zeiger-Offset fr Zugriff auf Wertetabellen
	ld	r16, y			;Sensor-Flags holen
	bst	r16, 7			;Sensor inaktiv?
	brtc	m14440			;nein -> Daten ausgeben
m14430:	rjmp	m144z5			;sonst Ausgabe berspringen
;
m14440:	clr	txflag			;Textflags lschen
	andi	r16, 0x0c		;ber- oder Unterschreitung?
	breq	m14450			;nein -> weiter testen
	ldi	r16, 0x01		;sonst invers darstellen
	rjmp	m14460
m14450:	ld	r16, y			;Flags nochmals holen
	andi	r16, 0x30		;Erinnerungs-Flag(s) gesetzt?
	breq	m14470			;nein -> normal ausgeben
	ldi	r16, 0x02		;sonst unterstrichen ausgeben
m14460:	mov	txflag, r16		;Textflags setzen
;
m14470:	ldi	r20, 52			;X-Startposition Spalte 1
	ldi	r21, 16			;Y-Startposition
	cpi	r25, 8			;Wert in Spalte 1 (Anzeigeposition < 8)?
	brcs	m14480			;ja -> weiter
	ldi	r20, 139		;sonst X-Startposition Spalte 2
m14480:	mov	r16, r25		;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
        movw    r0, r14                 ;Zeiger-Offset fr Wertetabellen kopieren
        mov     r16, r10                ;zwischengespeicherte Sensornummer holen
        cpi     r16, 27                 ;Sensornummer < 27 (Temperatursensor)?
        brcs    m14490                  ;ja -> Temperaturwert ausgeben
        rjmp    m144l0                  ;sonst Luftdruck- und Luftfeuchtigkeitswert ausgeben
;
; Hauptanzeige: Sensorwerte ausgeben - 16 Anzeigepositionen, Temperaturwerte
;
m14490:	movw	r8, yl			;Flag-Zeiger sichern
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	lds	r16, xflags		;verschiedene Flags holen
	bst	r16, 2			;Minimal-Werte anzeigen?
	brtc	m144a0			;nein -> weiter testen
	ldi	yl, low(semin1)
	ldi	yh, high(semin1)	;sonst Zeiger auf Minimalwerte
	clr	txflag			;Textflags lschen
;
	ldi	zl, low(senlil)
	ldi	zh, high(senlil)	;Zeiger auf untere Temperatur-Limits
	add	zl, r0
	adc	zh, r1			;Tabellenplatz berechnen
	ld	r18, z			;unteres Temperatur-Limit L holen
	ldd	r19, z + 1		;unteres Temperatur-Limit H holen
	cpi	r19, 0x80		;gltiger Wert?
	breq	m144b0			;nein -> Ende
	ldi	zl, low(semin1)         ;sonst
	ldi	zh, high(semin1)	;Zeiger auf Minimalwerte (Anzeige)
	add	zl, r0
	adc	zh, r1			;Tabellenplatz berechnen
	ld	r16, z			;Minimalwert L holen
	ldd	r17, z + 1		;Minimalwert H holen
	cpi	r17, 0x80		;gltiger Wert?
	breq	m144b0			;nein -> Ende
;
	sub	r16, r18		;Minimalwert - Limit berechnen
	sbc	r17, r19		;Limit unterschritten?
	brpl	m144b0			;nein -> weiter
	ldi	r16, 1			;sonst Text invers darstellen
	mov	txflag, r16		;Textflag wieder speichern
	rjmp	m144b0
;
m144a0:	bst	r16, 3			;Minimal-Werte anzeigen?
	brtc	m144b0			;nein -> weiter testen
	ldi	yl, low(semax1)
	ldi	yh, high(semax1)	;sonst Zeiger auf Maximalwerte
	clr	txflag			;Textflags lschen
;
	ldi	zl, low(senlih)
	ldi	zh, high(senlih)	;Zeiger auf obere Temperatur-Limits
	add	zl, r0
	adc	zh, r1			;Tabellenplatz berechnen
	ld	r18, z			;oberes Temperatur-Limit L holen
	ldd	r19, z + 1		;oberes Temperatur-Limit H holen
	cpi	r19, 0x80		;gltiger Wert?
	breq	m144b0			;nein -> Ende
	ldi	zl, low(semax1)
	ldi	zh, high(semax1)	;Zeiger auf Maximalwerte (Anzeige)
	add	zl, r0
	adc	zh, r1			;Tabellenplatz berechnen
	ld	r16, z			;Maximalwert L holen
	ldd	r17, z + 1		;Maximalwert H holen
	cpi	r17, 0x80		;gltiger Wert?
	breq	m144b0			;nein -> Ende
;
	sub	r18, r16		;Limit - Maximalwert berechnen
	sbc	r19, r17		;Limit unterschritten?
	brpl	m144b0			;nein -> weiter
	ldi	r16, 1			;sonst Text invers darstellen
	mov	txflag, r16		;Textflag wieder speichern
;
m144b0:	add	yl, r0
	adc	yh, r1			;Tabellenplatz berechnen
	ld	r16, y			;Temperaturwert L holen
	ldd	r17, y + 1		;Temperaturwert H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m144e0			;ja -> ausgeben
;
	lds	r16, xflags		;verschiedene Flags holen
	andi	r16, 0x0c		;Min/Max-Werte anzeigen?
	breq	m144c0			;nein -> weiter
	rjmp	m144z5			;sonst Ausgabe berspringen
;
m144c0:	movw	yl, r8			;Flag-Zeiger zurckholen
	ld	r16, y			;Sensor-Flags holen
	bst	r16, 6			;Sensorausfall?
	brts	m144d0			;ja -> Ausfallzeichen ausgeben
	rjmp	m144z5			;sonst Ausgabe berspringen
m144d0:	ldi	r16, 31
	add	r20, r16		;X-Position auf Ausfallzeichen
	ldi	r22, '!'		;Ausfallzeichen
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;ausgeben
	rjmp	m144z5			;keine weiteren Ausgaben
;
m144e0:	call	temasc			;Wert in ASCII wandeln
	mov	r12, txflag		;Textflags sichern
	mov	r22, r4			;erste Stelle holen
	cpi	r22, ' '		;Leerzeichen?
	brne	m144f0			;nein -> weiter
	ldi	r16, 0x01		;sonst Unterstreichungs-Flag
	and	txflag, r16		;lschen
m144f0:	call	asclcd			;Zeichen ausgeben
	mov	txflag, r12		;Textflags wiederherstellen
	mov	r22, r5			;zweite Stelle holen
	cpi	r22, ' '		;Leerzeichen?
	brne	m144g0			;nein -> weiter
	ldi	r16, 0x01		;sonst Unterstreichungs-Flag
	and	txflag, r16		;lschen
m144g0:	call	asclcd			;Zeichen ausgeben
	mov	txflag, r12		;Textflags wiederherstellen
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m144h0			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m144h0:	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
;
	ldi	r22, 0xa7		;breites Grad-Zeichen (dahinterstehendes Ausfallzeichen lschen)
	movw	yl, r8			;Flag-Zeiger zurckholen
	lds	r16, xflags		;verschiedene Flags holen
	andi	r16, 0x0c		;Min/Max-Werte anzeigen?
	brne	m144k0			;ja -> kein Sonderzeichen
	ld	r16, y			;Sensor-Flags holen
	andi	r16, 0x14		;berschreitung oder berschreitungs-Erinnerung?
	breq	m144i0			;nein -> weiter
;
	ldi	r22, 0xa8		;Grad + Pfeil nach oben
m144i0:	ld	r16, y			;Sensor-Flags holen
	andi	r16, 0x28		;Unterschreitung oder Unterschreitungs-Erinnerung?
	breq	m144j0			;nein -> weiter
	ldi	r22, 0xa9		;Grad + Pfeil nach unten
m144j0:	ld	r16, y			;Sensor-Flags holen
	bst	r16, 6			;Ausfall-Flag gesetzt?
	brtc	m144k0			;nein -> weiter
	ldi	r22, 0xaa		;sonst Ausfall-Symbol holen
m144k0:	call	asclcd			;Zeichen ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;ausgeben
        rjmp    m144z5                  ;Ende
;
; Hauptanzeige: Sensorwerte ausgeben - 16 Anzeigepositionen, Luftdruck- und Luftfeuchtigkeitswerte
;
m144l0: cpi     r16, 31                 ;Sensornummer = 31 (grafischer Luftdruck)?
        brne    m144l5                  ;nein -> weiter
        rjmp    m144v0                  ;sonst direkt zur Ausgabe
;
m144l5: clr     txflag                  ;Textflags lschen
        movw	r8, yl			;Flag-Zeiger sichern
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	lds	r16, xflags		;verschiedene Flags holen
	bst	r16, 2			;Minimal-Werte anzeigen?
	brtc	m144m0			;nein -> weiter testen
	ldi	yl, low(semin1)
	ldi	yh, high(semin1)	;sonst Zeiger auf Minimalwerte
	rjmp	m144n0			;nein -> Ende
;
m144m0:	bst	r16, 3			;Minimal-Werte anzeigen?
	brtc	m144n0			;nein -> weiter testen
	ldi	yl, low(semax1)
	ldi	yh, high(semax1)	;sonst Zeiger auf Maximalwerte
;
m144n0:	add	yl, r0
	adc	yh, r1			;Tabellenplatz berechnen
	ld	r16, y			;Sensorwert L holen
	ldd	r17, y + 1		;Sensorwert H holen
	cpi	r17, 0x80		;Sensorwert gltig?
	brne	m144q0			;ja -> ausgeben
;
	lds	r16, xflags		;sonst verschiedene Flags holen
	andi	r16, 0x0c		;Min/Max-Werte anzeigen?
	breq	m144o0			;nein -> weiter
	rjmp	m144z5			;sonst Ausgabe berspringen
;
m144o0:	movw	yl, r8			;Flag-Zeiger zurckholen
	ld	r16, y			;Sensor-Flags holen
	bst	r16, 6			;Sensorausfall?
	brts	m144p0			;ja -> Ausfallzeichen ausgeben
	rjmp	m144z5			;sonst Ausgabe berspringen
;
m144p0:	ldi	r16, 31
	add	r20, r16		;X-Position auf Ausfallzeichen
	ldi	r22, '!'		;Ausfallzeichen
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;ausgeben
	rjmp	m144z5			;keine weiteren Ausgaben
;
m144q0: mov     r22, r10                ;Sensornummer holen
        cpi     r22, 27                 ;Sensornummer = 27 (Luftdrucksensor)?
        brne    m144r0                  ;nein -> Luftfeuchtigkeit
;
	call	temasc			;sonst Wert in ASCII wandeln
	mov	r22, r4			;Tausender holen
	call	asclcd			;ausgeben
	mov	r22, r5			;Hunderter holen
	call	asclcd			;ausgeben
	mov	r22, r6			;Zehner holen
	call	asclcd			;sonst Zehner ausgeben
	mov	r22, r7			;Einer holen
	call	asclcd			;ausgeben
	ldi	r22, 'h'		;Hektopascal ('h') ausgeben
        rjmp    m144s0
;
m144r0: call    humasc                  ;Wert in ASCII wandeln
	mov	r22, r4			;Tausender holen
	call	asclcd			;ausgeben
	mov	r22, r5			;Hunderter holen
	call	asclcd			;ausgeben
	mov	r22, r6			;Zehner holen
	call	asclcd			;sonst Zehner ausgeben
	mov	r22, r7			;Einer holen
	call	asclcd			;ausgeben
        ldi     r22, '%'                ;Prozent-Zeichen ausgeben
;
m144s0:	call	asclcd			;ausgeben
	movw	yl, r8			;Flag-Zeiger zurckholen
	ld	r16, y			;Sensor-Flags holen
	bst	r16, 6			;Ausfall-Flag gesetzt?
	brtc	m144u0			;nein -> weiter
        ldi     r22, 0x7f               ;sonst Endezeichen (eine Pixelposition weiter)
        call    asclcd                  ;ausgeben
	ldi	r22, '!'		;Ausfall-Symbol
	call	asclcd			;ausgeben
m144t0:	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;ausgeben
        rjmp    m144z5                  ;Ende
;
m144u0: ldi     r22, 0x7f               ;Endezeichen (Leerzeichen mit der Breite 1 Pixel)
	call	asclcd			;ausgeben
	call	asclcd			;ausgeben, gegebenenfalls Lschen des Ausfallsymbols
        rjmp    m144t0
;
m144v0: movw    r12, r20                ;Koordinaten zwischenspeichern
        movw    r18, r20                ;Startpunkt kopieren
        subi    r20, -34                ;X-Endpunkt setzen
        subi    r21, -11                ;Y-Endpunkt setzen
        call    clrwnd                  ;Grafikbereich lschen
        movw    r20, r12                ;gespeicherte Koordinaten holen
;
        ldi     yl, low(prstab)
        ldi     yh, high(prstab)        ;Zeiger auf vorbereitete Luftdruckdaten
        ldi     r24, 34                 ;34 Werte ausgeben
        mov     r18, r20                ;X-Koordinate kopieren
;
m144v5: mov     r19, r21                ;Y-Koordinate kopieren
        bst     r24, 0                  ;erster Wert des Paares?
        brts    m144w0                  ;nein -> weiter
        ld      r20, y+                 ;sonst Luftdruck-Wertepaar holen
        mov     r16, r20                ;und kopieren
        rjmp    m144w5
m144w0: mov     r16, r20                ;Wertepaar kopieren
        swap    r16                     ;zweiten Wert an richtige Position bringen
m144w5: andi    r16, 0x0f               ;Einzelwert filtern
        ldi     r17, 10                 ;Wert 10 laden und Druckwert abziehen
        sub     r17, r16                ;(Wertebereich umkehren 0-10 -> 10-0)
        mov     r22, r17                ;Wert zwischenspeichern
;
        cpi     r24, 34                 ;wird erster Wert bearbeitet?
        breq    m144z0                  ;ja -> Wert direkt ausgeben
        mov     r16, r23                ;sonst gemerkten (vorherigen) Wert holen
        sub     r16, r17                ;Differenz zum vorherigen Wert ermitteln
        brcs    m144x5                  ;Differenz negativ? ja -> bearbeiten
;
        cpi     r16, 2                  ;Differenz > 1?
        brcs    m144z0                  ;nein -> neuen Wert direkt ausgeben
        dec     r16                     ;sonst Differenz korrigieren und
        mov     r14, r16                ;als Pixelzhler verwenden
        dec     r18                     ;X-Koordinate auf vorherigen Wert
        mov     r19, r21                ;Y-Koordinate kopieren
        add     r19, r23                ;vorherigen Wert addieren
        dec     r19                     ;Y-Koordinate nach oben korrigieren
        ori     r19, 0x80               ;Pixel setzen
m144x0: call    pixlcd                  ;Pixel zeichnen (Linie zur nchsten Koordinate)
        dec     r19                     ;nchste Y-Koordinate (nach oben)
        dec     r14                     ;alle Pixel gezeichnet?
        brne    m144x0                  ;nein -> Schleife
        rjmp    m144y5                  ;ursprngliche Daten wieder herstellen
;
m144x5: neg     r16                     ;Ergebnis negieren (Zweierkomplement)
        cpi     r16, 2                  ;Differenz > 1?
        brcs    m144z0                  ;nein -> neuen Wert dirkt ausgeben
        dec     r16                     ;sonst Differenz korrigieren und
        mov     r14, r16                ;als Pixelzhler verwenden
        dec     r18                     ;X-Koordinate auf vorherigen Wert
        mov     r19, r21                ;Y-Koordinate kopieren
        add     r19, r23                ;vorherigen Wert addieren
        inc     r19                     ;Y-Koordinate nach unten korrigieren
        ori     r19, 0x80               ;Pixel setzen
m144y0: call    pixlcd                  ;Pixel zeichnen
        inc     r19                     ;nchste Y-Koordinate (nach unten)
        dec     r14                     ;alle Pixel gezeichnet?
        brne    m144y0                  ;nein -> Schleife
;
m144y5: mov     r17, r22                ;zwischengespeicherten Wert holen
        inc     r18                     ;X-Koordinate wieder herstellen
        mov     r19, r21                ;Y-Koordinate wieder herstellen
;
m144z0: add     r19, r17                ;Luftdruckwert zur Y-Koordinate addieren
        ori     r19, 0x80               ;Pixel setzen
        call    pixlcd                  ;Pixel ausgeben
        inc     r18                     ;X-Koordinate erhhen
        mov     r23, r22                ;zwischengespeicherten Wert merken
        dec     r24                     ;alle Werte bearbeitet?
        brne    m144v5                  ;nein -> Schleife
;
m144z5:	adiw	xl, 1			;Positions-Tabellen-Zeiger erhhen
	inc	r25			;nchste Anzeigeposition
	cpi	r25, 16			;alle Positionen bearbeitet?
	brcc	m14500			;ja -> weiter
	rjmp	m14410			;sonst Schleife
;
; Hauptanzeige: Sensorwerte ausgeben - Groanzeige
;
m14500: ldi     xl, low(bigflg)
        ldi     xh, high(bigflg)        ;Zeiger auf Flags fr Groanzeige Seite 1
        lds     r17, xflag4             ;verschiedene Flags 4 holen
        sbrc    r17, 4                  ;Anzeige-Seite 2 aktiv? nein -> berspringen
        adiw    xl, 1                   ;sonst Zeiger auf Flags fr Groanzeige Seite 2
        ld      r16, x                  ;Flags fr Groanzeige holen
        bst     r16, 5                  ;Groanzeige inaktiv?
        brts    m14530                  ;ja -> berspringen
        bst     r16, 6                  ;Neuausgabe erforderlich?
        brtc    m14530                  ;nein -> berspringen
        cbr     r16, 1 << 6             ;sonst Ausgabe-Flag lschen
        st      x, r16                  ;und Flags wieder speichern
        andi    r16, 0x1f               ;Sensornummer fr Groanzeige filtern
;
        ldi     yl, low(senflg)
        ldi     yh, high(senflg)        ;Zeiger auf Sensorflags
        clr     r17                     ;H-Byte lschen
        add     yl, r16                 ;Tabellenposition ermitteln
        adc     yh, r17                 ;bertrag
        ld      r18, y                  ;Flags des gewhlten Groanzeige-Sensors holen
        bst     r18, 7                  ;Sensor inaktiv?
        brts    m14530                  ;ja -> berspringen
;
        add     r16, r16                ;Sensornummer verdoppeln fr Tabellenoffset
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	lds	r19, xflags		;verschiedene Flags holen
	bst	r19, 2			;Minimal-Werte anzeigen?
	brtc	m14510			;nein -> weiter
	ldi	yl, low(semin1)         ;sonst
	ldi	yh, high(semin1)	;Zeiger auf Minimalwerte
        rjmp    m14520
;
m14510:	bst	r19, 3			;Maximal-Werte anzeigen?
	brtc	m14520			;nein -> weiter
	ldi	yl, low(semax1)         ;sonst
	ldi	yh, high(semax1)	;Zeiger auf Maximalwerte
;
m14520:	add	yl, r16
	adc	yh, r17			;Tabellenplatz berechnen
	ld	r16, y			;Temperaturwert L holen
	ldd	r17, y + 1		;Temperaturwert H holen
        movw    r10, r16                ;Temperaturwert zwischenspeichern
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m14540			;ja -> ausgeben
m14530: rjmp    m14600                  ;sonst Ende
;
; Hauptanzeige: Sensorwerte ausgeben - Groanzeige berbeiten, Prfen ob ganzzahlig angezeigt werden muss
;
m14540:	ldi	r16, low(999)		;3-stellige Groanzeige nur
	ldi	r17, high(999)		;bis 99,9 Grad
	sub	r16, r10
	sbc	r17, r11		;Temperaturwert >99,9?
	brmi	m14570			;ja -> ganzzahlig anzeigen
	ldi	r16, low(-99)		;3-stellige Groanzeige nur
	ldi	r17, high(-99)		;ab -9,9 Grad
	movw	r18, r10		;Temperaturwert kopieren
	sub	r18, r16
	sbc	r19, r17		;Temperaturwert <-9,9?
	brpl	m14550			;nein -> normal ausgeben
;
	ldi	r16, low(-990)		;ganzzahlige Groanzeige nur
	ldi	r17, high(-990)		;ab -99,0 Grad
	movw	r18, r10		;Temperaturwert kopieren
	sub	r18, r16
	sbc	r19, r17		;Temperaturwert <-99,0?
	brpl	m14570			;nein -> ganzzahlig ausgeben
	movw	r10, r16		;sonst immer Wert -99
	rjmp	m14570			;ganzzahlig anzeigen
;
; Hauptanzeige: Sensorwerte ausgeben - normale Groanzeige im Bereich -9,9 bis 99,9 Grad
;
m14550:	movw	r16, r10		;Temperaturwert holen
	call	temasc			;und in ASCII wandeln
;
	ldi	r18, 194		;Zwischenraum 1 lschen
	ldi	r19, 77			;X/Y Anfangs-Koordinaten
	ldi	r20, 197
	ldi	r21, 111		;X/Y End-Koordinaten
	call	clrwnd			;ausgeben
	ldi	r18, 213		;Zwischenraum 2 lschen
	ldi	r19, 77			;X/Y Anfangs-Koordinaten
	ldi	r20, 219
	ldi	r21, 100		;X/Y End-Koordinaten
	call	clrwnd			;ausgeben
;
	mov	r22, r5			;zweite Stelle holen
	ldi	r20, 178		;X Koordinate fr 2. Stelle
	ldi	r21, 77			;Y Koordinate fr 2. Stelle
	call	biglcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	ldi	r20, 197		;X Koordinate fr 3. Stelle
	ldi	r21, 77			;Y Koordinate fr 3. Stelle
	call	biglcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	ldi	r20, 219		;X Koordinate fr 4. Stelle
	ldi	r21, 77			;Y Koordinate fr 4. Stelle
	call	biglcd			;ausgeben
;
        ldi     r20, 213                ;X Koordinate Dezimalpunkt
        ldi     r21, 100 		;Y Koordinate Dezimalpunkt
	ldi	r22, 0x94		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m14560			;ja -> weiter
	ldi	r22, 0x95		;sonst Trennzeichen Komma laden
m14560: clr     txflag                  ;Textflags lschen
        call    asclcd                  ;ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
	rjmp	m14600
;
; Hauptanzeige: Sensorwerte ausgeben - gerundete Groanzeige im Bereich -99 bis -10 und 100 bis 200 Grad
;
m14570:	ldi	r18, 4			;Wert 4 laden fr Rundung
	clr	r19
	movw	r16, r10		;Temperaturwert kopieren
	bst	r17, 7			;Wert negativ?
	brts	m14580			;ja -> weiter
	add	r16, r18		;4 addieren fr Rundung
	adc	r17, r19		;bertrag
	rjmp	m14590
;
m14580:	sub	r16, r18
	sbc	r17, r19		;4 subtrahieren fr Rundung
m14590:	call	temasc			;in ASCII wandeln
;
	ldi	r18, 178		;Zwischenraum 1 lschen
	ldi	r19, 77			;X/Y Anfangs-Koordinaten
	ldi	r20, 180
	ldi	r21, 111		;X/Y End-Koordinaten
	call	clrwnd			;ausgeben
	ldi	r18, 196		;Zwischenraum 2 lschen
	ldi	r19, 77			;X/Y Anfangs-Koordinaten
	ldi	r20, 199
	ldi	r21, 111		;X/Y End-Koordinaten
	call	clrwnd			;ausgeben
	ldi	r18, 215		;Zwischenraum 3 lschen
	ldi	r19, 77			;X/Y Anfangs-Koordinaten
	ldi	r20, 218
	ldi	r21, 111		;X/Y End-Koordinaten
	call	clrwnd			;ausgeben
	ldi	r18, 234		;Zwischenraum 4 lschen
	ldi	r19, 77			;X/Y Anfangs-Koordinaten
	ldi	r20, 235
	ldi	r21, 111		;X/Y End-Koordinaten
	call	clrwnd			;ausgeben
;
	mov	r22, r4			;erste Stelle holen
	ldi	r20, 180		;X Koordinate fr 1. Stelle
	ldi	r21, 77			;Y Koordinate fr 1. Stelle
	call	biglcd			;ausgeben
	mov	r22, r5			;zweite Stelle holen
	ldi	r20, 199		;X Koordinate fr 2. Stelle
	ldi	r21, 77			;Y Koordinate fr 2. Stelle
	call	biglcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	ldi	r20, 218		;X Koordinate fr 3. Stelle
	ldi	r21, 77			;Y Koordinate fr 3. Stelle
	call	biglcd			;ausgeben
;
; Alarmanzeige: Alarmnamen aus EEPROM ausgeben
;
m14600:	ldi	r25, 0			;Alarmnummerzhler = 0
	ldi	xl, low(almtab)
	ldi	xh, high(almtab)	;Zeiger auf Anzeige-Tabelle
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
m14610:	clr	txflag			;Textflags lschen
	ld	r16, y			;Flags holen
	bst	r16, 7			;Alarm inaktiv?
	brtc	m14620			;nein -> weiter
	rjmp	m14710			;sonst Ausgabe berspringen
m14620:	bst	r16, 0			;Neuausgabe erforderlich?
	brts	m14630			;ja -> ausgeben
	rjmp	m14710			;sonst Ausgabe berspringen
m14630:	cbr	r16, 0x01		;sonst Ausgabe-Flag lschen
	st	y, r16			;Flags wieder speichern
	bst	r16, 1			;Alarm aktiv?
	brtc	m14640			;nein -> weiter testen
	ldi	r16, 0x01		;sonst invers darstellen
	rjmp	m14650
m14640:	bst	r16, 2			;Erinnerungs-Flag gesetzt?
	brtc	m14660			;nein -> normal ausgeben
	ldi	r16, 0x02		;sonst unterstrichen darstellen
m14650:	mov	txflag, r16
m14660:	ld	r18, x			;Anzeigeposition holen
	ldi	r20, 176		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	mov	r16, r18		;Anzeigeposition holen
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	ldi	r19, 11			;Alarmnamenlnge im EPROM
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	r16, low(ealmtx)	;EEPROM-Adresse Alarmnamen L
	ldi	r17, high(ealmtx)	;EEPROM-Adresse Alarmnamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
m14670:	call	eeread			;Zeichen aus EEPROM holen
	cpi	r18, 0x20		;Leerzeichen (=Ende)?
	breq	m14700			;ja -> Endezeichen ausgeben
	mov	r22, r18		;Zeichen kopieren
	push	r16
	push	r17			;EEPROM-Adresse sichern
	push	r19			;Zhler sichern
	subi	r18, 0x20		;Korrektur fr Zeichentabelle
	cpi	r18, 0x80		;ASCII-Bereich berschritten?
	brcc	m14680			;ja -> keine Ausgabe
	ldi	r17, 12			;ASCII-Tabellen-Offset
	mul	r18, r17		;Tabellenplatz berechnen
;
	ldi	zl, low(chrset * 2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrset * 2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrset * 2)	;Zeiger auf Zeichentabelle B3
	clr	r17
	add	zl, r0			;Tabellenadresse L berechnen
	adc	zh, r1			;Tabellenadresse H berechnen
	adc	r16, r17		;Tabellenadresse B3 berechnen
	out	rampz, r16
	elpm	r16, z			;Zeichenbreite holen
	inc	r16			;Zeichenbreite korrigieren
	add	r16, r20		;und X-Koordinate addieren
	cpi	r16, 238		;passt Zeichen noch auf Zeile?
	brcc	m14680			;nein -> keine Ausgabe
	call	asclcd			;Zeichen ausgeben
m14680:	pop	r19			;Zhler wieder herstellen
	pop	r17
	pop	r16			;EERPOM-Adresse restaurieren
	dec	r19			;alle Zeichen ausgegeben?
	brne	m14670			;nein -> Schleife
m14700:	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;Zeichen ausgeben
;
	cpi	r20, 238		;Zeile auffllen erforderlich?
	brcc	m14710			;nein -> berspringen
	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 238		;Endwert X setzen
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	bst	txflag, 0		;Invers-Darstellung?
	bld	r19, 7			;sonst Invers-Bit in den
	bld	r21, 7			;Koordinaten setzen
	call	clrwnd			;Bereich lschen oder setzen
;
m14710:	adiw	xl, 1			;Alarm-Anzeige-Positionszeiger erhhen
	adiw	yl, 1			;Flag-Zeiger erhhen
	inc	r25			;nchste Alarmnummer
	cpi	r25, 4			;alle Alarme bearbeitet?
	brcc	m14800			;ja -> weiter
	rjmp	m14610			;sonst Schleife
;
m14800:	jmp	m20000
;
; Anzeigemodus Meldungen: Bildschirmseite neu ausgeben
;
m15000:	lds	r16, dimode		;Anzeigemodus holen
	cpi	r16, 1			;Anzeigemodus Meldungen?
	breq	m15010			;ja -> weiter
	rjmp	m16000			;sonst weiter testen
;
m15010:	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brts	m15020			;ja -> weiter
	jmp	m20000			;sonst Ende
;
m15020:	cbr	r16, 1 << 1		;Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	clr	txflag			;Text-Flag lschen
	ldi	r25, 0			;Zeilenzhler auf 0 setzen
;
m15100:	lds	r19, xflag2		;verschiedene Flags 2 holen
	bst	r19, 5			;nchste Seite?
	brts	m15250			;nein -> vorherige Seite
;
; Anzeigerichtung nchste Seite (ltere Daten)
;
	bst	r19, 6			;Listenende erreicht?
	brtc	m15200			;nein -> weiter
	rjmp	m15600			;sonst Ende
m15200:	lds	r20, msgpo1
	lds	r21, msgpo1 + 1		;Zeiger 1 auf die aktuelle
	lds	r22, msgpo1 + 2		;Anzeigeposition holen
	ldi	r16, low(xmssta)
	ldi	r17, high(xmssta)	;Anfangsadresse Meldungs-Daten
	ldi	r18, byte3(xmssta)	;laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Anfangsadresse erreicht?
	brne	m15210			        ;nein -> weiter
	ldi	r20, low(xmssta + xmslen)
	ldi	r21, high(xmssta + xmslen)	;sonst
	ldi	r22, byte3(xmssta + xmslen)     ;End-Adresse Meldungs-Daten laden
m15210:	subi	r20, xemsda
	sbci	r21, 0			;Zeiger auf nchste Position
	sbci	r22, 0			;setzen (zeitlich davor)
	sts	msgpo1, r20
	sts	msgpo1 + 1, r21		;neuen Zeiger 1 auf die aktuelle
	sts	msgpo1 + 2, r22		;Anzeigeposition speichern
;
	lds	r16, xemspo		;aktuellen EEPROM-Zeiger auf
	lds	r17, xemspo + 1		;Meldungs-Daten holen, zeigt
	lds	r18, xemspo + 2		;auf letzten Listeneintrag
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Listenende erreicht?
	brne	m15220			;nein -> weiter
	lds	r19, xflag2		;sonst verschiedene Flags holen
	sbr	r19, 1 << 6		;Listenende-Flag setzen
	sts	xflag2, r19		;Flags wieder speichern
m15220:	rjmp	m15300
;
; Anzeigerichtung vorherige Seite (neuere Daten)
;
m15250:	bst	r19, 7			;Listenende erreicht?
	brtc	m15260			;nein -> weiter
	rjmp	m15600			;sonst Ende
m15260:	lds	r20, msgpo2
	lds	r21, msgpo2 + 1		;Zeiger 2 auf die aktuelle
	lds	r22, msgpo2 + 2		;Anzeigeposition holen
	lds	r16, xemspo		;aktuellen EEPROM-Zeiger auf
	lds	r17, xemspo + 1		;Meldungs-Daten holen, zeigt
	lds	r18, xemspo + 2		;auf letzten Listeneintrag
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Listenende erreicht?
	brne	m15270			;nein -> weiter
	lds	r19, xflag2		;sonst verschiedene Flags holen
	sbr	r19, 1 << 7		;Listenende-Flag setzen
	sts	xflag2, r19		;Flags wieder speichern
	rjmp	m15600
;
m15270:	ldi	r16, low(xmssta + xmslen)
	ldi	r17, high(xmssta + xmslen)
	ldi	r18, byte3(xmssta + xmslen)     ;End-Adresse Meldungs-Daten laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	m15280			;nein -> weiter
	ldi	r20, low(xmssta)
	ldi	r21, high(xmssta)	;sonst
	ldi	r22, byte3(xmssta)	;Anfangsadresse Meldungs-Daten laden
;
m15280:	movw	r12, r20		;Zeiger 2 auf die aktuelle
	mov	r14, r22		;Anzeigeposition sichern
	ldi	r16, xemsda
	clr	r17
	add	r20, r16
	adc	r21, r17		;Zeiger auf nchsten Eintrag
	adc	r22, r17		;setzen (zeitlich danach)
	sts	msgpo2, r20
	sts	msgpo2 + 1, r21		;neuen Zeiger 2 auf die aktuelle
	sts	msgpo2 + 2, r22		;Anzeigeposition speichern
;
	lds	r16, xemspo		;aktuellen EEPROM-Zeiger auf
	lds	r17, xemspo + 1		;Meldungs-Daten holen, zeigt
	lds	r18, xemspo + 2		;auf letzten Listeneintrag
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Listenende erreicht?
	brne	m15290			;nein -> weiter
	lds	r19, xflag2		;sonst verschiedene Flags holen
	sbr	r19, 1 << 7		;Listenende-Flag setzen
	sts	xflag2, r19		;Flags wieder speichern
m15290:	movw	r20, r12		;Zeiger 2 auf die aktuelle
	mov	r22, r14		;Anzeigeposition zurckholen
;
; Datensatz lesen und Meldungskategorie feststellen
;
m15300:	ldi	r23, xemsda		;7 Bytes lesen (1 Datensatz)
	call	xdread			;Datensatz aus Ext-EEPROM lesen
	brcc	m15310			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	jmp	m20000			;Ausgabe beenden
;
m15310:	lds	r16, xedbuf		;Zeitbyte 0 holen
	lds	r17, xedbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xedbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xedbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	brne	m15330			;ja -> weiter
m15320:	rjmp	m15100			;sonst nchster Datensatz
;
m15330:	lds	r16, xedbuf + 5		;Meldungscode holen
	clr	r22			;Meldungskategorie lschen
	cpi	r16, 0xfe		;Meldungsnummer > 0xfd?
	brcc	m15340			;ja -> keine Kategorie zuordnen
        cpi     r16, 0xee               ;Meldungsnummer < 0xee?
        brcs    m15340                  ;ja -> keine Kategorie zuordnen
	ldi	r22, 3			;Meldungskategorie 3 (Fehler)
	cpi	r16, 0xf7		;Meldungsnummer < 0xf7?
	brcs	m15340			;ja -> Kategorie ok, weiter
	ldi	r22, 1			;Meldungskategorie 1 (Alarme)
	cpi	r16, 0xf9		;Meldungsnummer < 0xf9?
	brcs	m15340			;ja -> Kategorie ok, weiter
	ldi	r22, 3			;Meldungskategorie 3 (Fehler)
	cpi	r16, 0xfb		;Meldungsnummer < 0xfb?
	brcs	m15340			;ja -> Kategorie ok, weiter
	ldi	r22, 2			;sonst Meldungskategorie 2 (Temperaturen)
;
; Kategorie filtern und Daten ausgeben
;
m15340:	lds	r16, msgfil		;sonst Meldungsfilter holen
	tst	r16			;Meldungsfilter aktiviert?
	breq	m15350			;nein -> weiter
	cp	r16, r22		;stimmt Kategorie berein?
	brne	m15320			;nein -> nchster Datensatz
;
m15350:	ldi	r20, 2			;X-Position setzen
	ldi	r21, 16			;Y-Position erste Zeile setzen
	ldi	r16, 12			;Zeilenabstand setzen
;
	mov	r17, r25		;Zeilenzhler kopieren
	lds	r18, xflag2		;verschiedene Flags2 holen
	bst	r18, 5			;nchste Seite?
	brtc	m15360			;ja -> weiter
	com	r17			;sonst Einerkomplement bilden
	andi	r17, 0x07		;Zeile von unten beschreiben
;
m15360:	mul	r16, r17		;Y-Position der Zeile berechnen
	add	r21, r0			;zur ersten Position addieren
	tst	r22			;gltige Kategorie?
	breq	m15370			;nein -> kein Symbol ausgeben
	ori	r22, 0xa0		;Symbol generieren
	call	asclcd			;ausgeben
m15370:	ldi	r20, 11			;X-Position fr Text setzen
	call	xtdtim			;Zeit extrahieren und ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
;
	lds	r16, xedbuf + 5		;Meldungscode holen
	cpi	r16, 0xfe		;gltige Meldungsnummer?
	brcc	m15400			;nein -> Zeilenende
        cpi     r16, 0xee               ;gltige Meldungsnummer?
        brcs    m15400                  ;nein -> Zeilenende
	lds	r22, xedbuf + 5		;Meldungscode holen
	call	strlcd			;Meldungstext ausgeben
	lds	r16, xedbuf + 5		;Meldungscode holen
	cpi	r16, 0xf7		;Parameter ausgeben?
	brcs	m15400			;nein -> Zeilenende
;
	ldi	r22, ','		;Komma
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
        lds     r16, xedbuf + 5         ;Meldungscode holen
	lds	r18, xedbuf + 6		;Parameter holen
	cpi	r16, 0xf9		;Alarmnummer ausgeben?
	brcs	m15380			;ja -> weiter
;
	cpi	r18, 31			;Sensornummer > 30?
	brcc	m15400			;ja -> Zeilenende
	ldi	r19, 9			;sonst 9 Zeichen Sensortext
	ldi	r16, low(esentx)	;Zeiger auf EEPROM-Adresse
	ldi	r17, high(esentx)	;der Sensortexte
	rjmp	m15390			;EEPROM-Text ausgeben
;
m15380:	cpi	r18, 4			;Alarmnummer > 3?
	brcc	m15400			;ja -> Zeilenende
	ldi	r19, 11			;11 Zeichen Alarmtext
	ldi	r16, low(ealmtx)	;Zeiger auf EEPROM-Adresse
	ldi	r17, high(ealmtx)	;der Alarmtexte
;
m15390:	mul	r18, r19		;Adress-Offset ermitteln
	add	r16, r0
	adc	r17, r1			;Tabellenplatz ermitteln
	call	eeplcd			;EEPROM-Text ausgeben
;
m15400:	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;ausgeben
;
; nchste Zeile ausgeben
;
m15500:	inc	r25			;Zeilenzhler erhhen
	cpi	r25, 8			;letzte Zeile erreicht?
	breq	m15620			;ja -> Ende
	rjmp	m15100			;sonst nchste Zeile bearbeiten
;
m15600:	tst	r25			;Listenende, leerer Bildschirm?
	breq	m15610			;ja -> weiter
	jmp	m20000			;sonst Ende
;
m15610:	ldi	r20, 2			;X-Position fr Ende-Text
	ldi	r21, 16			;Y-Position fr Ende-Text
	ldi	r22, 0xb2		;Pfeil rechts (hohl)
	call	asclcd			;ausgeben
	inc	r20			;2 Pixel Zwischenraum
	ldi	r22, 0x42		;Text "Listenende"
	call	strlcd			;ausgeben
m15620:	jmp	m20000
;
; Anzeigemodus Status: Anzeige komplett neu ausgeben
;
m16000:	cpi	r16, 2			;Anzeigemodus Status?
	breq	m16010			;ja -> weiter
	rjmp	m17000			;sonst weiter testen
;
m16010:	lds	r16,dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brts	m16020			;ja -> weiter
	rjmp	m16500			;sonst Datenfelder ausgeben
;
m16020:	cbr	r16, 1 << 1		;Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	clr	txflag			;Text-Flag lschen
;
	ldi	r18, 0			;X = 1
m16030:	ldi	r19, 43 | 0x80		;Y = 43, oberer Trennlinie
	call	pixlcd			;waagerechte Linie zeichnen
	ldi	r19, 95 | 0x80		;Y = 95, untere Trennlinie
	call	pixlcd			;waagerechte Linie zeichnen
	inc	r18			;X-Koordinate erhhen
	cpi	r18, 239		;alle Pixel ausgegeben?
	brne	m16030			;nein -> Schleife
;
	ldi	r20, 2			;X-Position Versionsnummer
	ldi	r21, 18			;Y-Position Versionsnummer
	ldi	r22, 8			;Versionsnummer-Text
	call	strlcd			;ausgeben
	ldi	r20, 2			;X-Position Sensorbelegung
	ldi	r21, 30			;Y-Position Sensorbelegung
	ldi	r22, 0x39		;Text "Sensorbelegung:"
	call	strlcd			;ausgeben
;
	ldi	r20, 2			;X-Position fr LED-Text
	ldi	r21, 99			;Y-Position fr LED-Text
	ldi	r22, 0x38		;Text "LED1/LED2"
	call	strlcd			;ausgeben
;
	ldi	r20, 2			;X-Position EEPROM
	ldi	r21, 46			;Y-Position EEPROM
	ldi	r22, 0x30		;Text "EEPROM"
	call	strlcd			;ausgeben

	ldi	r20, 2			;X-Position Speicher
	ldi	r21, 58			;Y-Position Speicher
	ldi	r22, 0x34		;Text "Speicher"
	call	strlcd			;ausgeben

	ldi	r20, 2			;X-Position Echtzeituhr
	ldi	r21, 70			;Y-Position Echtzeituhr
	ldi	r22, 0x35		;Text "Echtzeituhr"
	call	strlcd			;ausgeben

	ldi	r20, 2			;X-Position Backup
	ldi	r21, 82			;Y-Position Backup
	ldi	r22, 0x55		;Text "Backup"
	call	strlcd			;ausgeben
;
	ldi	r20, 106		;X-Position Pointer
	ldi	r21, 46			;Y-Position Pointer
	ldi	r22, 0x36		;Text "Pointer"
	call	strlcd			;ausgeben

	ldi	r20, 106		;X-Position Batterie
	ldi	r21, 58			;Y-Position Batterie
	ldi	r22, 0x37		;Text "Batterie"
	call	strlcd			;ausgeben

	ldi	r20, 106		;X-Position Lichtsensor
	ldi	r21, 70			;Y-Position Lichtsensor
	ldi	r22, 0x31		;Text "Lichtsensor"
	call	strlcd			;ausgeben
;
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicherfehler?
	brts	m16040			;ja -> berspringen
        call    xvconf                  ;im Ext-EEPROM gespeichertes Backup prfen
        brcc    m16050                  ;Speicherfehler aufgetreten? nein -> weiter
	lds	r16, errflg		;sonst Fehler-Flags holen
	sbr	r16, 1 << 3		;Speicherfehler setzen
	sts	errflg, r16		;Flags wieder speichern
m16040: rjmp    m16500
;
m16050:	ldi	r20, 83		        ;X-Position Backup
	ldi	r21, 82			;Y-Position Backup
	ldi	r22, 0xae		;Ok-Symbol
        tst     r14                     ;Vergleichsfehler beim Lesen des Backups?
        breq    m16060			;nein -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
m16060:	call	asclcd			;ausgeben
;
        ldi     r20, 106                ;X-Position Backup-Zeit
        ldi     r21, 82                 ;Y-Position Backup-Zeit
        ldi     xl, low(xerbuf)
        ldi     xh, high(xerbuf)        ;Zeiger auf Ext-EEPROM-Lesepuffer
        clr     r16                     ;Flag fr Nullbytes lschen
        ser     r17                     ;Flag fr 0xff-Bytes setzen
        ldi     r19, 4                  ;4 Bytes prfen
m16070: ld      r18, x+                 ;Byte aus Lesepuffer holen
        or      r16, r18                ;verknpfen (nur Nullbytes erkennen)
        and     r17, r18                ;verknpfen (nur 0xff-Bytes erkennen)
        dec     r19                     ;alle Bytes geprft?
        brne    m16070                  ;nein -> Schleife
;
        tst     r16                     ;alles Nullbytes im Zeit-Puffer?
        breq    m16080                  ;ja -> Fehlertext ausgeben
        cpi     r17, 0xff               ;alles 0xff-Bytes im Zeit-Puffer?
        breq    m16080                  ;ja -> Fehlertext ausgeben
        lds     r16, xerbuf + 4         ;Prfzeichen (0x5a) holen
        cpi     r16, 0x5a               ;Prfzeichen ok?
        breq    m16090                  ;ja -> Zeit ausgeben
;
m16080:	ldi	r22, 0x3a		;sonst Text "nicht vorhanden"
	call	strlcd			;ausgeben
        rjmp    m16500
;
m16090: ldi     r22, 0x32               ;Text "vom"
        call    strlcd                  ;ausgeben
        ldi     r22, 0xa0               ;schmales Leerzeichen laden
        call    asclcd                  ;ausgeben
        call    xtrtim                  ;sonst Zeit extrahieren
        lds     r22, ascbuf + 6         ;Tag-Zehner holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 7         ;Tag-Einer holen
        call    asclcd                  ;ausgeben
        ldi     r22, '.'                ;Punkt laden
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 4         ;Monat-Zehner holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 5         ;Monat-Einer holen
        call    asclcd                  ;ausgeben
        ldi     r22, '.'                ;Punkt laden
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf             ;Jahr-Tausender holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 1         ;Jahr-Hunderter holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 2         ;Jahr-Zehner holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 3         ;Jahr-Einer holen
        call    asclcd                  ;ausgeben
        ldi     r22, ','                ;Punkt laden
        call    asclcd                  ;ausgeben
        ldi     r22, 0xa0               ;schmales Leerzeichen laden
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 9         ;Stunden-Zehner holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 10        ;Stunden-Einer holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 11        ;Doppelpunkt holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 12        ;Minuten-Zehner holen
        call    asclcd                  ;ausgeben
        lds     r22, ascbuf + 13        ;Minuten-Einer holen
        call    asclcd                  ;ausgeben
        ldi     r22, 0x7f
        call    asclcd
;
; Anzeigemodus Status: Datenfelder neu ausgeben - Sensorbelegung
;
m16500:	lds	r16, xflag2		;verschiedene Flags 2 holen
	bst	r16, 4			;Datenfelder neu ausgeben?
	brts	m16510			;ja -> weiter
	jmp	m20000			;sonst Ende
;
m16510:	cbr	r16, 1 << 4		;Datenfelder-Flag lschen
	sts	xflag2, r16		;Flags wieder speichern
	clr	txflag			;Text-Flag lschen
;
	ldi	r20, 106		;X-Position Sensorbelegung
	ldi	r21, 30			;Y-Position Sensorbelegung
	ldi	xl, low(senmap)         ;Sensorbelegung prfen und Anzahl der Sensoren ermitteln
	ldi	xh, high(senmap)	;Zeiger auf Sensorbelegung
        clr     r12                     ;Sensorzhler = 0 setzen
	ldi	r17, 8			;8 Zeichen prfen
m16520:	ld	r16, x+			;Zeichen holen
	cpi	r16, '0'		;Zeichen < '0'?
	brcs	m16560			;ja -> Meldung ausgeben
        cpi     r16, 'R' + 1            ;Zeichen > 'R'?
	brcc	m16560			;ja -> Meldung ausgeben
        subi    r16, 0x30               ;Zeichen in Ziffernwert umwandeln
        cpi     r16, 10                 ;gltiger Wert 0-9?
        brcs    m16530                  ;ja -> weiter (ok)
        subi    r16, 7                  ;sonst Buchstabe in Wert umwandeln
        cpi     r16, 10                 ;gltiger Wert (> 9)?
        brcs    m16560                  ;nein -> Meldung ausgeben
m16530: add     r12, r16                ;Wert zum Sensorzhler addieren
        dec     r17                     ;alle Zeichen geprft?
        brne    m16520                  ;nein -> Schleife
;
	ldi	xl, low(senmap)
	ldi	xh, high(senmap)	;Zeiger auf Sensorbelegung
	ldi	r25, 8			;8 Zeichen ausgeben
m16540:	ld      r22, x+                 ;Zeichen holen
        call	asclcd			;ausgeben
	dec	r25			;alle Zeichen ausgegeben?
	brne	m16540			;nein -> Schleife
;
        ldi     r22, ' '                ;Leerzeichen
        call    asclcd                  ;ausgeben
        ldi     r22, '('                ;Klammer auf
        call    asclcd                  ;ausgeben
        mov     r18, r12                ;ermittelte Sensoranzahl holen
        call    bytasc                  ;in ASCII wandeln
        mov     r22, r16                ;Einer holen
        cpi     r17, 0x30               ;Zehner = '0'?
        breq    m16550                  ;ja -> nur Einer ausgeben
        mov     r22, r17                ;sonst Zehner holen
        mov     r25, r16                ;Einer zwischenspeichern
        call    asclcd                  ;Zehner ausgeben
        mov     r22, r25                ;Einer gespeicherte Einer holen
m16550: call    asclcd                  ;ausgeben
        ldi     r22, ')'                ;Klammer zu
        call    asclcd                  ;ausgeben
        rjmp    m16570                  ;Zeilenende lschen
;
m16560:	ldi	r22, 0x3a		;Text "nicht vorhanden"
	call	strlcd			;ausgeben
;
m16570:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 200		;Endwert X setzen
	ldi	r21, 41			;Endwert Y setzen
	call	clrwnd			;Bereich lschen
;
; Anzeigemodus Status: Datenfelder neu ausgeben - LED-Status
;
	ldi	r20, 106		;X-Position LED-Modus
	ldi	r21, 99			;Y-Position LED-Modus
	ldi	r22, 0x40		;Text fr LED (normal)
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 0			;LED-Modus normal?
	brtc	m16580			;ja -> weiter
	ldi	r22, 0x41		;sonst Text fr LED (Debug)
m16580:	call	strlcd			;ausgeben
	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 200		;Endwert X setzen
	ldi	r21, 110		;Endwert Y setzen
	call	clrwnd			;Bereich lschen
;
; Anzeigemodus Status: Datenfelder neu ausgeben - EEPROM, Speicher, Echtzeituhr
;
	ldi	r20, 56 		;X-Position EEPROM1
	ldi	r21, 46			;Y-Position EEPROM1
	ldi	r22, 0xae		;Ok-Symbol
	lds	r16, xflags		;verschiedene Flags holen
        sbrs    r16, 6			;EEPROM1 vorhanden? ja -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
	call	asclcd			;ausgeben
;
	ldi	r20, 65		        ;X-Position EEPROM2
	ldi	r21, 46			;Y-Position EEPROM2
	ldi	r22, 0xae		;Ok-Symbol
	lds	r16, xflags		;verschiedene Flags holen
	sbrs	r16, 7			;EEPROM2 vorhanden? ja -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
	call	asclcd			;ausgeben
;
	ldi	r20, 74 		;X-Position EEPROM3
	ldi	r21, 46			;Y-Position EEPROM3
	ldi	r22, 0xae		;Ok-Symbol
	lds	r16, xflag4		;verschiedene Flags 4 holen
	sbrs	r16, 6			;EEPROM3 vorhanden? ja -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
	call	asclcd			;ausgeben
;
	ldi	r20, 83 		;X-Position EEPROM4
	ldi	r21, 46			;Y-Position EEPROM4
	ldi	r22, 0xae		;Ok-Symbol
	lds	r16, xflag4		;verschiedene Flags 4 holen
	sbrs	r16, 7			;EEPROM4 vorhanden? ja -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
	call	asclcd			;ausgeben
;
	ldi	r20, 83			;X-Position Speicher
	ldi	r21, 58			;Y-Position Speicher
	ldi	r22, 0xae		;Ok-Symbol
	lds	r16, errflg		;verschiedene Flags holen
	sbrc	r16, 3			;Speicherfehler? nein -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
	call	asclcd			;ausgeben
;
	ldi	r20, 83			;X-Position Echtzeituhr
	ldi	r21, 70			;Y-Position Echtzeituhr
	ldi	r22, 0xae		;Ok-Symbol
	lds	r16, errflg		;verschiedene Flags holen
	sbrc	r16, 4			;RTC-Fehler? nein -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
	call	asclcd			;ausgeben
;
        lds     r16, xflag2             ;Verschiedene Flags 2 holen
        bst     r16, 1                  ;Pointer-Fehler 15min-Werte?
        brtc    m16600                  ;nein -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
; Anzeigemodus Status: Datenfelder neu ausgeben - Pointer
;
m16600: lds     r16, xes1po
        lds     r17, xes1po + 1
        lds     r18, xes1po + 2         ;Pointer fr 15min-Werte laden
        subi    r16, low(xs1sta)
        sbci    r17, high(xs1sta)       ;Startadresse subtrahieren
        sbci    r18, byte3(xs1sta)      ;war Pointer < Startadresse?
        brpl    m16610                  ;nein -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16610: ldi     r20, low(xes1da)
        ldi     r21, high(xes1da)       ;Datensatzlnge 15min-Werte
        call    divide                  ;aktuelle Datensatznummer berechnen
        mov     r18, r14                ;Divisionsrest L holen und mit Divisionsrest H
        or      r18, r15                ;verknpfen, Divisionsrest = 0?
        breq    m16620                  ;ja -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers
        mov     txflag, r19
;
m16620: ldi     r18, low(3855)
        ldi     r19, high(3855)         ;Anzahl der Datenstze laden
        cp      r16, r18
        cpc     r17, r19                ;Datensatznummer < Anzahl?
        brcs    m16630                  ;ja -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16630: call    bytas4                  ;Datensatznummer in Dezimalwert umrechnen
        ldi     r20, 153                ;X-Position Datensatznummer 15min-Werte
        ldi     r21, 46                 ;Y-Position Datensatznummer 15min-Werte
        mov     r22, r4                 ;Tausenderstelle holen
        call    asclcd                  ;ausgeben
        mov     r22, r5                 ;Hunderterstelle holen
        call    asclcd                  ;ausgeben
        mov     r22, r6                 ;Zehnerstelle holen
        call    asclcd                  ;ausgeben
        mov     r22, r7                 ;Einerstelle holen
        call    asclcd                  ;ausgeben
;
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
        clr     txflag                  ;Textanzeige normal
        ldi     r22, 0xa0               ;schmales Leerzeichen
        call    asclcd                  ;ausgeben
        movw    r12, r20                ;LCD-Koordinaten speichern
;
        lds     r16, xflag2             ;Verschiedene Flags 2 holen
        bst     r16, 2                  ;Pointer-Fehler 24h-Werte?
        brtc    m16640                  ;nein -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16640: lds     r16, xes2po
        lds     r17, xes2po + 1
        lds     r18, xes2po + 2         ;Pointer fr 24h-Werte laden
        subi    r16, low(xs2sta)
        sbci    r17, high(xs2sta)       ;Startadresse subtrahieren
        sbci    r18, byte3(xs2sta)      ;war Pointer < Startadresse?
        brpl    m16650                  ;nein -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16650: ldi     r20, low(xes2da)
        ldi     r21, high(xes2da)       ;Datensatzlnge 24h-Werte
        call    divide                  ;aktuelle Datensatznummer berechnen
        mov     r18, r14                ;Divisionsrest L holen und mit Divisionsrest H
        or      r18, r15                ;verknpfen, Divisionsrest = 0?
        breq    m16660                  ;ja -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers
        mov     txflag, r19
;
m16660: ldi     r18, low(416)
        ldi     r19, high(416)          ;Anzahl der Datenstze laden
        cp      r16, r18
        cpc     r17, r19                ;Datensatznummer < Anzahl?
        brcs    m16670                  ;ja -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16670: call    bytas4                  ;Datensatznummer in Dezimalwert umrechnen
        movw    r20, r12                ;gesicherte LCD_Koordinaten holen
        mov     r22, r5                 ;Hunderterstelle holen
        call    asclcd                  ;ausgeben
        mov     r22, r6                 ;Zehnerstelle holen
        call    asclcd                  ;ausgeben
        mov     r22, r7                 ;Einerstelle holen
        call    asclcd                  ;ausgeben
;
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
        clr     txflag                  ;Textanzeige normal
        ldi     r22, 0xa0               ;schmales Leerzeichen
        call    asclcd                  ;ausgeben
        movw    r12, r20                ;LCD-Koordinaten speichern
;
        lds     r16, xflag2             ;Verschiedene Flags 2 holen
        bst     r16, 3                  ;Pointer-Fehler Meldungsdaten?
        brtc    m16680                  ;nein -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16680: lds     r16, xemspo
        lds     r17, xemspo + 1
        lds     r18, xemspo + 2         ;Pointer fr Meldungsdaten laden
        subi    r16, low(xmssta)
        sbci    r17, high(xmssta)       ;Startadresse subtrahieren
        sbci    r18, byte3(xmssta)      ;war Pointer < Startadresse?
        brpl    m16690                  ;nein -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16690: ldi     r20, low(xemsda)
        ldi     r21, high(xemsda)       ;Datensatzlnge Meldungsdaten
        call    divide                  ;aktuelle Datensatznummer berechnen
        mov     r18, r14                ;Divisionsrest L holen und mit Divisionsrest H
        or      r18, r15                ;verknpfen, Divisionsrest = 0?
        breq    m16700                  ;ja -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers
        mov     txflag, r19
;
m16700: ldi     r18, low(500)
        ldi     r19, high(500)          ;Anzahl der Datenstze laden
        cp      r16, r18
        cpc     r17, r19                ;Datensatznummer < Anzahl?
        brcs    m16710                  ;ja -> weiter (ok)
        ldi     r19, 1                  ;sonst Textanzeige invers setzen
        mov     txflag, r19
;
m16710: call    bytas4                  ;Datensatznummer in Dezimalwert umrechnen
        movw    r20, r12                ;gesicherte LCD_Koordinaten holen
        mov     r22, r5                 ;Hunderterstelle holen
        call    asclcd                  ;ausgeben
        mov     r22, r6                 ;Zehnerstelle holen
        call    asclcd                  ;ausgeben
        mov     r22, r7                 ;Einerstelle holen
        call    asclcd                  ;ausgeben
;
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
        clr     txflag
;
	ldi	r20, 229		;X-Position Pointer
	ldi	r21, 46			;Y-Position Pointer
	ldi	r22, 0xae		;Ok-Symbol
	lds	r16, errflg		;verschiedene Flags holen
	sbrc	r16, 5			;Pointer-Fehler? nein -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
	call	asclcd			;ausgeben
;
; Anzeigemodus Status: Datenfelder neu ausgeben - Batterie
;
	call	batclc                  ;Batteriespannung berechnen
	clr	r25			;Merker fr Batterieschwellwert
	ldi	r18, low(batlim)
	ldi	r19, high(batlim)	;Batterieschwellwert laden
	sub	r18, r16
	sbc	r19, r17		;Schwellwert erreicht?
	brcs	m16720			;nein -> weiter
	inc	r25			;sonst Merker setzen
m16720:	call	batasc			;Hexwert in ASCII wandeln
	ldi	r20, 196 		;X-Position Batteriespannung
	ldi	r21, 58			;Y-Position Batteriespannung
	mov	r22, r5			;Einerstelle kopieren
	call	asclcd			;ausgeben
;
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m16730			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma laden
m16730:	call	asclcd			;ausgeben
	mov	r22, r6			;Zehntelstelle kopieren
	call	asclcd			;ausgeben
	mov	r22, r7			;Hundertstelstelle kopieren
	call	asclcd			;ausgeben
	ldi	r22, 'V'		;'V' laden
	call	asclcd			;ausgeben
;
	ldi	r20, 229	        ;X-Position Batterie Status
        ldi     r21, 58                 ;Y-Position Batterie Status
	ldi	r22, 0xae		;Ok-Symbol
	tst	r25			;Batterieschwelle erreicht?
	breq	m16740			;nein -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
m16740:	call	asclcd			;ausgeben
;
; Anzeigemodus Status: Datenfelder neu ausgeben - Lichtsensor
;
        lds     r16, adclig             ;aktuellen Lichtsensor-Wert holen
        clr     r17                     ;H-Byte lschen
        call    bytas4                  ;in 4-stelligen ASCII-Wert wandeln
	mov	r16, r5			;Hunderterstelle holen
	cpi	r16, '0'		;Hunderterstelle = 0?
	brne	m16750			;nein -> ausgeben
        ldi     r16, ' '                ;sonst durch Leerzeichen
        mov     r5, r16                 ;ersetzen
        mov     r16, r6                 ;Zehnerstelle holen
        cpi     r16, '0'                ;Zehnerstelle = 0?
        brne    m16750                  ;nein -> ausgeben
        ldi     r16, ' '                ;sonst durch Leerzeichen
        mov     r6, r16                 ;ersetzen

m16750:	ldi	r20, 205		;X-Position Lichtsensor-Wert
	ldi	r21, 70			;Y-Position Lichtsensor-Wert
        mov     r22, r5
	call	asclcd			;sonst ausgeben
	mov	r22, r6			;Zehnerstelle holen
	call	asclcd			;ausgeben
	mov	r22, r7			;Einerstelle holen
	call	asclcd			;ausgeben
;
; Anzeigemodus Status: Datenfelder neu ausgeben - Backup-Status
;
        lds     r16, xflags             ;Verschiedene Flags holen
        bst     r16, 4                  ;Backup-Status aktualisieren?
        brtc    m16790                  ;nein -> berspringen
        cbr     r16, 1 << 4             ;sonst Flag lschen
        sts     xflags, r16             ;und wieder speichern
;
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicherfehler?
	brts	m16760			;ja -> kein Symbol ausgeben
        call    xvconf                  ;im Ext-EEPROM gespeichertes Backup prfen
        brcc    m16770                  ;Speicherfehler aufgetreten? nein -> weiter
	lds	r16, errflg		;sonst Fehler-Flags holen
	sbr	r16, 1 << 3		;Speicherfehler setzen
	sts	errflg, r16		;Flags wieder speichern
;
m16760:	ldi	r20, 83		        ;X-Position Backup
	ldi	r21, 82			;Y-Position Backup
        ldi     r22, 0xa0               ;schmales Leerzeichen
        call    asclcd                  ;ausgeben
        ldi     r22, 0xa0               ;schmales Leerzeichen
        rjmp    m16780                  ;ausgeben
;
m16770:	ldi	r20, 83		        ;X-Position Backup
	ldi	r21, 82			;Y-Position Backup
	ldi	r22, 0xae		;Ok-Symbol
        tst     r14                     ;Vergleichsfehler beim Lesen des Backups?
        breq    m16780			;nein -> weiter
	ldi	r22, 0xa4		;sonst Nicht-Ok-Symbol
m16780:	call	asclcd			;ausgeben
m16790:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sensoren - Einstellungen 1/2: Anzeige komplett neu ausgeben
;
m17000:	cpi	r16, 3			;Anzeigemodus Einstellungen - Sensoren - Position?
	breq	m17010			;ja -> weiter
	cpi	r16, 5			;Anzeigemodus Einstellungen - Sensoren - Position - Name?
	breq	m17010			;ja -> weiter
	rjmp	m17500			;sonst weiter testen
;
m17010:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m17100			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
;
	ldi	r19, 15 | 0x80		;Y = 15
m17030:	ldi	r18, 119		;X = 119, Trennlinie
	call	pixlcd			;senkrechte Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	cpi	r19, 112 | 0x80		;alle Pixel ausgegeben?
	brne	m17030			;nein -> Schleife
	call	newsen			;alle Namen und Werte neu ausgeben
;
; Anzeigemodus Konfiguration - Sensoren - Einstellungen 1/2: Sensornamen ausgeben
;
m17100: ldi	r25, 0			;Sensornummerzhler = 0
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
        lds     r16, dipara             ;Anzeigemodus Parameter holen
        tst     r16                     ;Einstellungen 1?
        breq    m17110                  ;ja -> weiter
        ldi     r25, 16                 ;sonst Sensornummer = 16 (Einstellungen 2)
        ldi     yl, low(senflg + 16)
	ldi	yh, high(senflg + 16)	;Zeiger auf Sensor-Flags korrigieren
;
m17110:	ld	r16, y			;Sensor-Flags holen
	bst	r16, 1			;Name neu ausgeben?
	brts	m17120			;ja -> weiter
	rjmp	m17190			;sonst berspringen
;
m17120:	cbr	r16, 0x02		;nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	mov	r18, r25		;Sensornummer holen
        andi    r18, 0x0f               ;Anzeigeposition ermitteln
	mov	r11, r18		;Anzeigeposition sichern (r11 enthlt gesicherte Anzeigeposition)
	ldi	r20, 2			;X-Startposition Spalte 1
	ldi	r21, 16			;Y-Startposition
	cpi	r18, 8			;Position in Spalte 1?
	brcs	m17130			;ja -> weiter
	ldi	r20, 121		;sonst X-Startposition Spalte 2
m17130:	mov	r16, r18		;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern (r13:r12 enthlt gesicherte X/Y-Startposition)
;
	lds	r19, menpos		;ausgewhlte Men-Position holen
	dec	r19			;korrigieren
	cp	r18, r19		;Position identisch?
	brne	m17140			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen

m17140: ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensortexte
        ldi     r16, 9                  ;Textabstand 9 Bytes
        mul     r16, r25                ;Textposition ermitteln
        add     zl, r0                  ;Sensor-Textposition ermitteln
        adc     zh, r1                  ;bertrag
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;Zeichen ausgeben
	clr	txflag
;
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r16, 22			;Position fr Checkbox
	add	r20, r16		;berechnen
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	ld	r16, y			;Sensor-Flags holen
	bst	r16, 7			;Sensor inaktiv?
	brts	m17150			;ja -> weiter
	ldi	r22, 0xb5		;sonst Zeichen "Checkbox1 ein"
m17150:	call	asclcd			;Zeichen ausgeben
;
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r16, 32			;Position fr Sensorname
	add	r20, r16		;berechnen
	lds	r16, dimode		;Anzeigemodus holen
	cpi	r16, 5			;Anzeigemodus Einstellungen - Name?
	brne	m17160			;nein -> normal ausgeben
	lds	r16, menpos		;sonst ausgewhlte Position holen
	dec	r16			;korrigieren
	cp	r16, r11		;Position identisch?
	brne	m17160			;nein -> normal ausgeben
	call	txblcd			;sonst Texteingabepuffer ausgeben
	rjmp	m17170
;
m17160:	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
;
m17170:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 86			;X-Endwert Spalte 1
	mov	r22, r11		;Anzeigeposition holen
	cpi	r22, 8			;Position in Spalte 1?
	brcs	m17180			;ja -> weiter
	ldi	r20, 206		;sonst X-Endwert Spalte 2
m17180:	ldi	r22, 11			;zum Y-Endwert 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m17190:	adiw	yl, 1			;Sensor-Flag Tabellenzeiger erhhen
	inc	r25			;nchste Sensornummer
        mov     r16, r25                ;Sensornummer kopieren
        andi    r16, 0x0f               ;Anzeigeposition filtern
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Einstellungen 1?
        brne    m171b0                  ;nein -> weiter
        tst     r16                     ;sonst Endwert 16 erreicht?
	breq	m17200			;ja -> weiter
m171a0:	rjmp	m17110			;sonst Schleife
m171b0: cpi     r16, 15                 ;Einstellungen 2 - Endwert 15 erreicht?
        brne    m171a0                  ;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Einstellungen 1/2: Sensorwerte ausgeben
;
m17200:	ldi	r25, 0			;Sensornummerzhler = 0
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
        lds     r16, dipara             ;Anzeigemodus Parameter holen
        tst     r16                     ;Einstellungen 1?
        breq    m17210                  ;ja -> weiter
        ldi     r25, 16                 ;sonst Sensornummer = 16 (Einstellungen 2)
        ldi     yl, low(senflg + 16)
	ldi	yh, high(senflg + 16)	;Zeiger auf Sensor-Flags korrigieren
;
m17210:	ld	r16, y			;Flags holen
	bst	r16, 0			;Wert neu ausgeben?
	brts	m17220                  ;ja -> weiter
        rjmp    m172b0			;sonst keine Ausgabe
;
m17220:	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	mov	r18, r25		;Sensornummer holen
        andi    r18, 0x0f               ;Anzeigeposition ermitteln
	mov	r11, r18		;Anzeigeposition sichern (r11 enthlt gesicherte Anzeigeposition)
	ldi	r20, 87			;X-Startposition Spalte 1
	ldi	r21, 16			;Y-Startposition
	cpi	r18, 8			;Temperatur in Spalte 1?
	brcs	m17230			;ja -> weiter
	ldi	r20, 207		;sonst X-Startposition Spalte 2
m17230:	mov	r16, r18		;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	ldi	r19, 2			;Temperaturwert 2 Byte
	mul	r25, r19		;Tabellenoffset ermitteln
	movw	r8, yl			;Flag-Zeiger sichern
;
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	add	yl, r0			;Tabellenoffset L addieren
	adc	yh, r1			;Tabellenoffset H addieren
	ld	r16, y			;Temperaturwert L holen
	ldd	r17, y + 1		;Temperaturwert H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	breq	m17280			;nein -> Feld lschen
;
	call	temasc			;Wert in ASCII wandeln
        cpi     r25, 27                 ;Sensortyp prfen
        brcs    m17260                  ;Temperatursensor? ja -> weiter
        breq    m17250                  ;Luftdrucksensor? ja -> weiter
;
        subi    r20, -5                 ;sonst X-Position fr Luftfeuchtigkeit korrigieren
        mov     r22, r5                 ;Hunderter holen
        call    asclcd                  ;ausgeben
        mov     r22, r6                 ;Zehner holen
        cpi     r22, '0'                ;Zehner = '0'?
        brne    m17240                  ;nein -> ausgeben
        mov     r16, r5                 ;sonst Hunderter holen
        cpi     r16, '1'                ;Hunderter = '1' (Wert = 100)?
        breq    m17240                  ;ja -> ausgeben
        ldi     r22, ' '                ;sonst Zehner durch Leerzeichen ersetzen
m17240: call    asclcd                  ;ausgeben
        mov     r22, r7                 ;Einer holen
        call    asclcd                  ;ausgeben
        ldi     r22, '%'                ;Prozentzeichen laden
        call    asclcd                  ;ausgeben
        rjmp    m172a0
;
m17250: subi    r20, -5                 ;X-Position fr Luftdruck korrigieren
        mov     r22, r4                 ;Tausender holen
        call    asclcd                  ;ausgeben
        mov     r22, r5                 ;Hunderter holen
        call    asclcd                  ;ausgeben
        mov     r22, r6                 ;Zehner holen
        call    asclcd                  ;ausgeben
        mov     r22, r7                 ;einer holen
        call    asclcd                  ;ausgeben
        rjmp    m172a0
;
m17260:	mov	r22, r4			;erste Stelle holen
	call	asclcd			;Zeichen ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;Zeichen ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m17270			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m17270:	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, 0xa5		;schmales Grad-Zeichen
	call	asclcd			;Zeichen ausgeben
	rjmp	m172a0
;
m17280:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 119		;X-Endwert Spalte 1
	mov	r22, r11		;Anzeigeposition holen
	cpi	r22, 8			;Position in Spalte 1?
	brcs	m17290			;ja -> weiter
	ldi	r20, 238		;sonst X-Startposition Spalte 2
m17290:	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m172a0:	movw	yl, r8			;Flag-Zeiger zurckholen
m172b0:	adiw	yl, 1			;Sensor-Flag Tabellenzeiger erhhen
	inc	r25			;nchste Sensornummer
        mov     r16, r25                ;Sensornummer kopieren
        andi    r16, 0x0f               ;Anzeigeposition filtern, alle Sensoren bearbeitet?
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Einstellungen 1?
        brne    m172d0                  ;nein -> weiter
        tst     r16                     ;sonst Endwert 16 erreicht?
	breq	m17300			;ja -> weiter
m172c0:	rjmp	m17210			;sonst Schleife
m172d0: cpi     r16, 15                 ;Einstellungen 2 - Endwert 15 erreicht?
        brne    m172c0                  ;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Einstellungen 1/2: Auswahl bearbeiten
;
m17300:	lds	r16, dimode		;Anzeigemodus holen?
	cpi	r16, 3			;Anzeigemodus Einstellungen - Position auswhlen?
	brne	m17500			;nein -> weiter
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m17320			;nein -> weiter
;
        lds     r18, dipara             ;Anzeigemodus Parameter holen
        tst     r18                     ;Einstellungen 1?
        breq    m17310                  ;ja -> weiter
        ldi     r18, 16                 ;sonst Offset fr Sensoren 17-30 (Einstellungen 2)
m17310:	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	movw	zl, yl			;Zeiger kopieren
	clr	r19			;H-Byte = 0
	add	yl, r18			;Flag-Position ermitteln (Sensorgruppe)
	adc	yh, r19			;bertrag
        add     yl, r16                 ;aktuelle Men-Position addieren (entspricht alter Sensornummer)
        adc     yh, r19                 ;bertrag
        sbiw    yl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, y			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	add	zl, r18			;Flag-Position ermitteln (Sensorgruppe)
	adc	zh, r19			;bertrag
        add     zl, r17                 ;aktuellen Impulsgeberwert addieren (entspricht neuer Sensornummer)
	adc	zh, r19			;bertrag
        sbiw    zl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, z			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	z, r16			;Flags wieder speichern
	sts	menpos, r17		;neuen Impulsgeberwert als neue Position setzen
m17320:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sensoren - Grenzwerte 1/2: Anzeige komplett neu ausgeben
;
m17500:	cpi	r16, 6			;Anzeigemodus Konfiguration - Sensoren - Grenzwerte?
	breq	m17510			;ja -> weiter
	rjmp	m17900			;sonst weiter testen
;
m17510:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m17600			;nein -> weiter
	cbr	r16, 1 << 1		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
;
	ldi	r19, 15 | 0x80		;Y = 15
m17520:	ldi	r18, 119		;X = 119, Trennlinie
	call	pixlcd			;senkrechte Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	cpi	r19, 112 | 0x80		;alle Pixel ausgegeben?
	brne	m17520			;nein -> Schleife
	call	newsen			;alle Namen und Werte neu ausgeben
;
; Anzeigemodus Konfiguration - Sensoren - Grenzwerte 1/2: Sensornamen ausgeben
;
m17600:	ldi	r25, 0			;Sensornummerzhler = 0
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
        lds     r16, dipara             ;Anzeigemodus Parameter holen
        tst     r16                     ;Grenzwerte 1?
        breq    m17610                  ;ja -> weiter
        ldi     r25, 16                 ;sonst Sensornummer = 16 (Grenzwerte 2)
        ldi     yl, low(senflg + 16)
	ldi	yh, high(senflg + 16)	;Zeiger auf Sensor-Flags korrigieren
;
m17610:	ld	r16, y			;Sensor-Flags holen
	bst	r16, 1			;Name neu ausgeben?
	brtc	m17640			;nein -> berspringen
	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	mov	r18, r25		;Sensornummer holen
        andi    r18, 0x0f               ;Anzeigeposition ermitteln
	ldi	r20, 2			;X-Startposition Spalte 1
	ldi	r21, 16			;Y-Startposition
	cpi	r18, 8			;Position in Spalte 1?
	brcs	m17620			;ja -> weiter
	ldi	r20, 121		;sonst X-Startposition Spalte 2
m17620:	mov	r16, r18		;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Y-Startposition addieren
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r18, r19		;Position identisch?
	brne	m17630			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
m17630:	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;Zeichen ausgeben
	clr	txflag			;Textflags lschen
;
m17640:	adiw	yl, 1			;Sensor-Flag Tabellenzeiger erhhen
	inc	r25			;nchste Sensornummer
        mov     r16, r25                ;Sensornummer kopieren
        andi    r16, 0x0f               ;Anzeigeposition filtern
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Grenzwerte 1?
        brne    m17650                  ;nein -> weiter
        tst     r16                     ;sonst Endwert 16 erreicht?
	brne	m17610			;nein -> Schleife
	rjmp	m17700
m17650: cpi     r16, 11                 ;Grenzwerte 2 - Endwert 11 erreicht?
        brne    m17610                  ;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Grenzwerte 1/2: Werte ausgeben
;
m17700:	ldi	r25, 0			;Sensornummerzhler = 0
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
        lds     r16, dipara             ;Anzeigemodus Parameter holen
        tst     r16                     ;Grenzwerte 1?
        breq    m17710                  ;ja -> weiter
        ldi     r25, 16                 ;sonst Sensornummer = 16 (Grenzwerte 2)
        ldi     yl, low(senflg + 16)
	ldi	yh, high(senflg + 16)	;Zeiger auf Sensor-Flags korrigieren
;
m17710:	mov	r18, r25		;Sensornummer holen
        andi    r18, 0x0f               ;Anzeigeposition ermitteln
	ld	r16, y			;Flags holen
	bst	r16, 0			;Wertnderung?
	brts	m17720			;ja -> ausgeben
	rjmp	m17810			;sonst keine Ausgabe
;
m17720:	cbr	r16, 0x01		;nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	ldi	r20, 52			;X-Startposition Spalte 1
	ldi	r21, 16			;Y-Startposition
	cpi	r18, 8			;Position in Spalte 1?
	brcs	m17730			;ja -> weiter
	ldi	r20, 171		;sonst X-Startposition Spalte 2
m17730:	mov	r16, r18		;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Y-Startposition addieren
	movw	r10, r20		;X/Y-Startpositionen sichern
;
	ldi	r19, 2			;Grenzwert 2 Byte
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	xl, low(senlil)
	ldi	xh, high(senlil)	;Zeiger auf untere Grenzwerte
	add	xl, r0			;Tabellenoffset L addieren
	adc	xh, r1			;Tabellenoffset H addieren
	ld	r16, x			;unteren Grenzwert L holen
        adiw    xl, 1                   ;Tabellenzeiger erhhen
	ld	r17, x		        ;unteren Grenzwert H holen
	cpi	r17, 0x80		;Grenzwert gltig?
	breq	m17750			;nein -> Feld lschen
	call	temasc			;Wert in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m17740			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m17740:	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, 0xab		;schmales Grad-Zeichen
	call	asclcd			;ausgeben
	rjmp	m17770
;
m17750:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 86			;X-Endwert Spalte 1
	mov	r22, r25		;Sensornummer holen
        andi    r22, 0x0f               ;Anzeigeposition ermitteln
	cpi	r22, 8			;Position in Spalte 1?
	brcs	m17760			;ja -> weiter
	ldi	r20, 205		;sonst X-Startposition Spalte 2
m17760:	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m17770:	movw	r20, r10		;X/Y-Koordinaten restaurieren
	subi	r20, -34		;X-Position fr obere Limits
	ldi	r19, 2			;Grenzwert 2 Byte
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	xl, low(senlih)
	ldi	xh, high(senlih)	;Zeiger auf obere Grenzwerte
	add	xl, r0			;Tabellenoffset L addieren
	adc	xh, r1			;Tabellenoffset H addieren
	ld	r16, x			;oberen Grenzwert L holen
        adiw    xl, 1                   ;Tabellenzeiger erhhen
	ld	r17, x		        ;oberen Grenzwert H holen
	cpi	r17, 0x80		;Grenzwert gltig?
	breq	m17790			;nein -> Feld lschen
	call	temasc			;Wert in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16,confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m17780			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m17780:	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, 0xab		;schmales Grad-Zeichen
	call	asclcd			;ausgeben
	rjmp	m17810
;
m17790:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 118		;X-Endwert Spalte 1
	mov	r22, r25		;Sensornummer holen
        andi    r22, 0x0f               ;Anzeigeposition ermitteln
	cpi	r22, 8			;Position in Spalte 1?
	brcs	m17800			;ja -> weiter
	ldi	r20, 238		;sonst X-Startposition Spalte 2
m17800:	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m17810:	adiw	yl, 1			;nchstes Sensor-Flag
	inc	r25			;nchste Sensornummer
        mov     r16, r25                ;Sensornummer kopieren
        andi    r16, 0x0f               ;Anzeigeposition filtern
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Grenzwerte 1?
        brne    m17820                  ;nein -> weiter
        tst     r16                     ;sonst Endwert 16 erreicht?
	breq	m17850			;ja -> weiter
	rjmp	m17710                  ;sonst Schleife
m17820: cpi     r16, 11                 ;Grenzwerte 2 - Endwert 11 erreicht?
        breq    m17850                  ;ja -> weiter
	rjmp	m17710			;sonst Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Grenzwerte 1/2: Auswahl bearbeiten
;
m17850:	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m17870			;nein -> weiter
;
        lds     r18, dipara             ;Anzeigemodus Parameter holen
        tst     r18                     ;Grenzwerte 1?
        breq    m17860                  ;ja -> weiter
        ldi     r18, 16                 ;sonst Offset fr Sensoren 17-27 (Grenzwerte 2)
m17860:	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	movw	zl, yl			;Zeiger kopieren
	clr	r19			;H-Byte = 0
	add	yl, r18			;Flag-Position ermitteln (Sensorgruppe)
	adc	yh, r19			;bertrag
        add     yl, r16                 ;aktuelle Men-Position addieren (entspricht alter Sensornummer)
        adc     yh, r19                 ;bertrag
        sbiw    yl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, y			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	add	zl, r18			;Flag-Position ermitteln (Sensorgruppe)
	adc	zh, r19			;bertrag
        add     zl, r17                 ;aktuellen Impulsgeberwert addieren (entspricht neuer Sensornummer)
	adc	zh, r19			;bertrag
        sbiw    zl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, z			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	z, r16			;Flags wieder speichern
	sts	menpos, r17		;neuen Impulsgeberwert als neue Position setzen
m17870:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sensoren - Grenzwerte 1/2: Grenzwert ndern
;
m17900:	cpi	r16, 7			;Anzeigemodus Konfiguration - Sensoren - Grenzwerte, ndern?
	brne	m17a00			;nein -> weiter testen
;
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 7			;Impulsgeber-nderung?
	brtc	m17940			;nein -> keine Ausgabe
	cbr	r16, 1 << 7		;sonst nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
;
	lds	r18, menpos		;Menposition holen
	dec	r18			;korrigieren
	ldi	r20, 52			;X-Startposition Spalte 1
	ldi	r21, 16			;Y-Startposition
	cpi	r18, 8			;Position in Spalte 1?
	brcs	m17910			;ja -> weiter
	ldi	r20, 171		;sonst X-Startposition Spalte 2
m17910:	mov	r16, r18		;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Y-Startposition addieren
;
	lds	r16, xflag4		;verschiedene Flags 4 holen
	bst	r16, 0			;Unteren Grenzwert ndern?
	brts	m17920			;ja -> Wert ausgeben
	subi	r20, -34		;X-Position fr oberen Grenzwert
	bst	r16, 1			;Oberen Grenzwert ndern?
	brtc	m17940			;nein -> keine Ausgabe
;
m17920:	lds	r16, encval		;Impulsgeber-Wert L holen
	lds	r17, encvah		;Impulsgeber-Wert H holen
	call	temasc			;Wert in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, '.'		;Trennzeichen Punkt
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m17930			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma
m17930:	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, 0xab		;schmales Grad-Zeichen
	call	asclcd			;ausgeben
m17940: jmp     m20000
;
; Anzeigemodus Konfiguration - Sensoren - Anzeige: Anzeige komplett neu ausgeben
;
m17a00: cpi     r16, 4                  ;Anzeigemodus Konfiguration - Sensoren - Anzeige
        breq    m17a10                  ;ja -> weiter
        rjmp    m17c00                  ;sonst weiter testen
;
m17a10: clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m17b00			;nein -> weiter
	cbr	r16, 1 << 1		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
;
	ldi	r19, 15 | 0x80		;Y = 15
m17a20:	ldi	r18, 60		        ;X = 60, Trennlinie fr Seite 1
	call	pixlcd			;senkrechte Linie zeichnen
	ldi	r18, 119		;X = 119, breite Trennlinie zwischen den Seiten
	call	pixlcd			;senkrechte Linie zeichnen
        ldi     r18, 120		;X = 120, breite Trennlinie zwischen den Seiten
	call	pixlcd			;senkrechte Linie zeichnen
        ldi     r18, 180		;X = 180, Trennlinie fr Seite 2
	call	pixlcd			;senkrechte Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	cpi	r19, 112 | 0x80		;alle Pixel ausgegeben?
	brne	m17a20			;nein -> Schleife
        call    newpos                  ;alle Anzeigepositionen neu ausgeben
;
; Anzeigemodus Konfiguration - Sensoren - Anzeige: Sensornamen ausgeben
;
m17b00:	ldi	r25, 0			;Anzeigepositions-Zhler = 0
	ldi	yl, low(postab)
	ldi	yh, high(postab)	;Zeiger auf Positions-Tabelle
;
m17b10:	ld	r18, y			;Sensornummer holen
	bst	r18, 7			;Sensorname neu ausgeben?
	brtc	m17b90			;nein -> berspringen
	cbr	r18, 0x80		;sonst nderungs-Flag lschen
	st	y, r18			;Sensornummer wieder speichern
;
	mov	r16, r25		;Anzeigeposition holen
        ror     r16
        ror     r16
        ror     r16                     ;Bits 3 und 4 entsprechen der Spaltennummer
        andi    r16, 0x03               ;relevante Bits filtern
        ldi     r17, 60                 ;mit Spaltenabstand
        mul     r16, r17                ;multiplizieren
	ldi	r20, 2			;X-Startposition Spalte 1
        add     r20, r0                 ;Spaltenposition addieren
        mov     r10, r20                ;X-Position zwischenspeichern
;
        mov     r16, r25                ;Anzeigeposition holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	ldi	r21, 16			;Y-Startposition
	add	r21, r0			;Zeilenposition addieren
;
        clr     txflag                  ;Textflags lschen
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m17b30			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
;
m17b30: bst     r18, 5                  ;Anzeigeposition leer?
        brts    m17b50                  ;ja -> Text "[leer]" ausgeben
        andi    r18, 0x1f               ;sonst Sensornummer filtern
        cpi     r18, 31                 ;Sensornummer = 31 (grafischer Luftdruck)?
        brne    m17b40                  ;nein -> weiter
        ldi     r18, 27                 ;sonst durch Sensornummer 27 ersetzen
        mov     r16, txflag             ;Textflags holen
        sbr     r16, 0x02               ;Unterstreichung setzen
        mov     txflag, r16             ;Textflags wieder speichern
;
m17b40:	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;ausgeben
        rjmp    m17b60
;
m17b50: ldi     r22, 0x46               ;Text "[leer]"
        call    strlcd                  ;ausgeben
m17b60:	clr	txflag			;Textflags lschen
	movw	r18, r20		;X/Y Koordinaten kopieren
        mov     r20, r10                ;zwischengespeicherte X-Position holen
        subi    r20, -50                ;X-Endposition ermitteln (50 Pixel addieren)
	subi	r21, -11		;zum Y-Endwert 11 Pixel addieren
	call	clrwnd			;Bereich lschen oder setzen
;
m17b90: adiw    yl, 1                   ;nchster Tabelleneintrag
	inc	r25			;nchste Anzeigeposition
        cpi     r25, 32                 ;Endwert erreicht?
        brcs    m17b10                  ;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Anzeige: Auswahl bearbeiten
;
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m17ba0			;nein -> weiter
;
	ldi	yl, low(postab)
	ldi	yh, high(postab)	;Zeiger auf Anzeigepositions-Tabelle
        movw    zl, yl                  ;Zeiger kopieren
	clr	r19			;H-Byte = 0
        dec     r16                     ;korrigieren
        add     yl, r16                 ;als Tabellenoffset addieren
	adc	yh, r19			;bertrag
        ld      r16, y                  ;Sensornummer und Flag holen (von alter Sensornummer)
        sbr     r16, 0x80               ;Sensorname neu ausgeben
        st      y, r16                  ;Sensornummer und Flag wieder speichern
;
        mov     r18, r17                ;Impulsgeberwert holen
        dec     r18                     ;korrigieren
	add	zl, r18			;als Tabellenoffset addieren
	adc	zh, r19			;bertrag
	ld	r16, z			;Sensornummer und Flag holen (von neuer Sensornummer)
	sbr	r16, 0x80		;Sensorname neu ausgeben
	st	z, r16			;Sensornummer und Flags wieder speichern
	sts	menpos, r17		;Impulsgeberwert als neue Position setzen
m17ba0:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sensoren - Anzeige: ndern bearbeiten
;
m17c00: cpi     r16, 27                 ;Anzeigemodus Konfiguration - Sensoren - Anzeige, ndern?
        breq    m17c10                  ;ja -> weiter
        rjmp    m17d00                  ;sonst weiter testen
;
m17c10: lds     r16, encflg             ;Impulsgeber-Flags holen
        bst     r16, 7                  ;Impulsgeber-nderung?
        brtc    m17ba0                  ;nein -> Ende
;
        cbr     r16, 0x80               ;sonst nderungs-Flag lschen
        sts     encflg, r16             ;Flags wieder speichern
        ldi     r16, 1
        mov     txflag, r16             ;Textflag invers setzen
	lds	r16, menpos		;Anzeigeposition holen
        dec     r16                     ;korrigieren
        ror     r16
        ror     r16
        ror     r16                     ;Bits 3 und 4 entsprechen der Spaltennummer
        andi    r16, 0x03               ;relevante Bits filtern
        ldi     r17, 60                 ;mit Spaltenabstand
        mul     r16, r17                ;multiplizieren
	ldi	r20, 2			;X-Startposition Spalte 1
        add     r20, r0                 ;Spaltenposition addieren
        mov     r25, r20                ;X-Position zwischenspeichern
;
        lds     r16, menpos             ;Anzeigeposition holen
        dec     r16                     ;korrigieren
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	ldi	r21, 16			;Y-Startposition
	add	r21, r0			;Zeilenposition addieren
;
        lds     r18, encval             ;Impulsgeberwert holen
        dec     r18                     ;korrigieren
        cpi     r18, 0xff               ;leere Anzeigeposition ausgewhlt?
        breq    m17c30                  ;ja -> Text "[leer]" ausgeben
        cpi     r18, 31                 ;alternativer Luftdruck ausgewhlt?
        brne    m17c20                  ;nein -> weiter
        ldi     r18, 27                 ;sonst durch Luftdruck ersetzen
        ldi     r16, 3
        mov     txflag, r16             ;zustzlich Textflag Unterstreichung setzen
;
m17c20:	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;ausgeben
        rjmp    m17c40
;
m17c30: ldi     r22, 0x46               ;Text "[leer]"
        call    strlcd                  ;ausgeben
m17c40: clr     txflag                  ;Textflag normal setzen
        movw    r18, r20                ;aktuelle X/Y-Position kopieren
        mov     r20, r25                ;zwischengespeicherte X-Position holen
        subi    r20, -50                ;X-Endposition ermitteln (50 Pixel addieren)
	subi	r21, -11		;zum Y-Endwert 11 Pixel addieren
	call	clrwnd			;Bereich lschen
        jmp     m20000
;
; Anzeigemodus Konfiguration - Sensoren - Groanzeige: Anzeige komplett neu ausgeben
;
m17d00: cpi     r16, 28                 ;Anzeigemodus Konfiguration - Sensoren - Groanzeige?
        breq    m17d10                  ;ja -> weiter
        rjmp    m17f00                  ;sonst weiter testen
;
m17d10: clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m17e00			;nein -> weiter
	cbr	r16, 1 << 1		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
        call    newpos                  ;alle Anzeigepositionen neu ausgeben
;
; Anzeigemodus Konfiguration - Sensoren - Groanzeige: Texte und Sensornamen ausgeben
;
m17e00:	ldi	r25, 0			;Anzeigepositions-Zhler = 0
	ldi	yl, low(bigflg)
	ldi	yh, high(bigflg)	;Zeiger auf Groanzeige-Flags
;
m17e10:	ld	r18, y			;Sensornummer holen
	bst	r18, 7			;Sensorname neu ausgeben?
	brtc	m17e70			;nein -> berspringen
	cbr	r18, 0x80		;sonst nderungs-Flag lschen
	st	y, r18			;Sensornummer wieder speichern
;
	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
        mov     r16, r25                ;Anzeigeposition holen
        tst     r16                     ;erste Position?
        breq    m17e20                  ;ja -> weiter
        subi    r21, -12                ;sonst Y-Position auf zweite Zeile
;
m17e20: clr     txflag                  ;Textflags lschen
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m17e30			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
;
m17e30: ldi     r22, 0x47               ;Text "Groanzeige"
        call    strlcd                  ;ausgeben
        ldi     r22, ' '                ;Leerzeichen
        call    asclcd                  ;ausgeben
        ldi     r22, 0x48               ;Texte "Seite"
        call    strlcd                  ;ausgeben
        ldi     r22, ' '                ;Leerzeichen
        call    asclcd                  ;ausgeben
        mov     r22, r25                ;Anzeigeposition holen
        subi    r22, -49                ;in ASCII wandeln (Seitennummer)
        call    asclcd                  ;ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
;
        clr     txflag                  ;Textflags lschen
        ldi     r20, 135                ;X-Position fr Sensornamen
        ld      r18, y                  ;Sensornummer holen
        bst     r18, 5                  ;Anzeigeposition leer?
        brts    m17e50                  ;ja -> Text "[leer]" ausgeben
        andi    r18, 0x1f               ;sonst Sensornummer filtern
;
	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;ausgeben
        rjmp    m17e60
;
m17e50: ldi     r22, 0x46               ;Text "[leer]"
        call    strlcd                  ;ausgeben
m17e60:	clr	txflag			;Textflags lschen
	movw	r18, r20		;X/Y Koordinaten kopieren
        ldi     r20, 100                ;X-Position
        subi    r20, -50                ;X-Endposition ermitteln (50 Pixel addieren)
	subi	r21, -11		;zum Y-Endwert 11 Pixel addieren
	call	clrwnd			;Bereich lschen oder setzen
;
m17e70: adiw    yl, 1                   ;nchster Tabelleneintrag
	inc	r25			;nchste Anzeigeposition
        cpi     r25, 2                  ;Endwert erreicht?
        brcc    m17e80                  ;ja -> weiter
        rjmp    m17e10                  ;sonst Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Groanzeige: Auswahl bearbeiten
;
m17e80:	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m17e90			;nein -> weiter
;
	ldi	yl, low(bigflg)
	ldi	yh, high(bigflg)	;Zeiger auf Groanzeigepositions-Flags
        ld      r16, y                  ;Sensornummer und Flag holen (Seite 1)
        sbr     r16, 0x80               ;Sensorname neu ausgeben
        st      y+, r16                 ;Sensornummer und Flag wieder speichern
	ld	r16, y			;Sensornummer und Flag holen (Seite 2)
	sbr	r16, 0x80		;Sensorname neu ausgeben
	st	y, r16			;Sensornummer und Flags wieder speichern
	sts	menpos, r17		;Impulsgeberwert als neue Position setzen
m17e90:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sensoren - Groanzeige: ndern bearbeiten
;
m17f00: cpi     r16, 29                 ;Anzeigemodus Konfiguration - Sensoren - Groanzeige, ndern?
        breq    m17f10                  ;ja -> weiter
        rjmp    m18000                  ;sonst weiter testen
;
m17f10: lds     r16, encflg             ;Impulsgeber-Flags holen
        bst     r16, 7                  ;Impulsgeber-nderung?
        brtc    m17e90                  ;nein -> Ende
;
        cbr     r16, 0x80               ;sonst nderungs-Flag lschen
        sts     encflg, r16             ;Flags wieder speichern
	ldi	r20, 135		;X-Startposition
	ldi	r21, 16			;Y-Startposition
        lds     r16, menpos             ;ausgewhlte Position holen
        cpi     r16, 1                  ;erste Position?
        breq    m17f20                  ;ja -> weiter
        subi    r21, -12                ;sonst Y-Position auf zweite Zeile
;
m17f20: clr     txflag                  ;Textflags lschen
        lds     r18, encval             ;Impulsgeberwert holen
        dec     r18                     ;korrigieren
        cpi     r18, 0xff               ;leere Anzeigeposition ausgewhlt?
        breq    m17f30                  ;ja -> Text "[leer]" ausgeben
	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Ende-Zeichen
	call	asclcd			;ausgeben
        rjmp    m17f40
;
m17f30: ldi     r22, 0x46               ;Text "[leer]"
        call    strlcd                  ;ausgeben
m17f40: movw    r18, r20                ;aktuelle X/Y-Position kopieren
        ldi     r20, 185                ;X-Endposition
	subi	r21, -11		;zum Y-Endwert 11 Pixel addieren
	call	clrwnd			;Bereich lschen
        jmp     m20000
;
; Anzeigemodus Konfiguration - Alarme: Anzeige komplett neu ausgeben
;
m18000:	cpi	r16, 8			;Anzeigemodus Alarme, Auswhlen?
	breq	m18010			;ja -> weiter
	cpi	r16, 9			;Anzeigemodus Alarme, Verschieben?
	breq	m18010			;ja -> weiter
	cpi	r16, 10			;Anzeigemodus Alarme, Namen?
	breq	m18010			;ja -> weiter
	rjmp	m18500			;sonst weiter testen
;
m18010:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m18100			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newalm			;alle Alarmdaten neu ausgeben
;
; Anzeigemodus Konfiguration - Alarme: Alarmdaten ausgeben
;
m18100:	ldi	r25, 0			;Alarmnummerzhler = 0
	ldi	xl, low(almbak)
	ldi	xh, high(almbak)	;Zeiger auf Anzeige-Positions-Tabelle
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
m18110:	ld	r16, y			;Alarmflags holen
	bst	r16, 0			;Daten neu ausgeben?
	brts	m18120			;ja -> weiter
	rjmp	m18180			;sonst berspringen
;
m18120:	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	ld	r18, x			;Anzeigeposition holen
	mov	r11, r18		;Anzeigeposition sichern
	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r18, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r18, r19		;Position identisch?
	brne	m18130			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
m18130:	ldi	r22, 'A'		;Zeichen 'A' (Alarm)
	call	asclcd			;ausgeben
	ldi	r22, 'L'		;Zeichen 'L' (Alarm)
	call	asclcd			;ausgeben
	mov	r16, r25		;Alarmnummer kopieren
	inc	r16			;fr Anzeige korrigieren
	clr	r17			;H-Byte lschen
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;Zeichen ausgeben
	clr	txflag
;
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r16, 22			;Position fr Checkbox
	add	r20, r16		;berechnen
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	ld	r16, y			;Alarmflags holen
	bst	r16, 7			;Alarm inaktiv?
	brts	m18140			;ja -> weiter
	ldi	r22, 0xb5		;sonst Zeichen "Checkbox1 ein"
m18140:	call	asclcd			;Zeichen ausgeben
;
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r16, 33			;Position fr Alarmname
	add	r20, r16		;berechnen
	lds	r16, dimode		;Anzeigemodus holen
	cpi	r16, 10			;Anzeigemodus Alarme, Namen?
	brne	m18150			;nein -> normal ausgeben
	lds	r16, menpos		;sonst ausgewhlte Position holen
	dec	r16			;korrigieren
	cp	r16, r11		;Position identisch?
	brne	m18150			;nein -> normal ausgeben
	call	txblcd			;sonst Texteingabepuffer ausgeben
	rjmp	m18160
;
m18150:	ld	r16, y			;Alarmflags holen
	bst	r16, 1			;Alarm aktiv?
	brtc	m18155			;nein -> weiter
	ldi	r16, 1
	mov	txflag, r16		;sonst Text invers darstellen
m18155:	ldi	r19, 11			;Alarmnamenlnge im EEPROM
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	r16, low(ealmtx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(ealmtx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;Zeichen ausgeben
	clr	txflag
;
m18160:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 99			;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r16, 99			;Position fr Alarminvertierung
	add	r20, r16		;berechnen
	ldi	zl, low(alminv)
	ldi	zh, high(alminv)	;Zeiger auf Alarm-Invertierung
	clr	r16
	add	zl, r25			;Tabellenplatz berechnen
	adc	zh, r16			;bertrag
	ldi	r22, 0x62		;Text "normal"
	ld	r16, z			;Alarm-Invertierung holen
	tst	r16			;Alarm invertiert?
	breq	m18170			;nein -> weiter
	ldi	r22, 0x63		;sonst Text "invertiert"
m18170:	call	strlcd			;ausgeben
;
	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 200		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m18180:	adiw	xl, 1			;Anzeige-Positions-Tabellenzeiger erhhen
	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Alarmnummer
	cpi	r25, 4			;alle Alarme bearbeitet?
	brcc	m18300			;ja -> weiter
	rjmp	m18110			;sonst Schleife
;
; Anzeigemodus Konfiguration - Alarme: Auswahl bearbeiten
;
m18300:	lds	r16, dimode		;Anzeigemodus holen?
	cpi	r16, 8			;Anzeigemodus Alarme, Auswhlen?
	brne	m18350			;nein -> weiter
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m18310			;nein -> weiter
;
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
	movw	zl, yl			;Zeiger kopieren
	call	salbak			;Alarmnummer suchen
	clr	r19			;H-Byte = 0
	add	yl, r18			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Alarm-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeberwert holen
	call	salbak			;Alarmnummer suchen
	add	zl, r18			;Flag-Position ermitteln
	adc	zh, r19			;bertrag
	ld	r16, z			;Alarm-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	z, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m18310:	jmp	m20000
;
; Anzeigemodus Konfiguration - Alarme: Verschiebung bearbeiten
;
m18350:	cpi	r16, 9			;Anzeigemodus Alarme, Verschieben?
	brne	m18400			;nein -> weiter
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m18400			;nein -> weiter
;
m18360:	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
	movw	zl, yl			;Zeiger kopieren
	call	salbak			;Alarmnummer suchen
	mov	r20, r18		;Alarmnummer 1 sichern
	clr	r19			;H-Byte = 0
	add	yl, r18			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Alarm-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeberwert holen
	call	salbak			;Alarmnummer suchen
	mov	r21, r18		;Alarmnummer 2 sichern
	add	zl, r18			;Flag-Position ermitteln
	adc	zh, r19			;bertrag
	ld	r16, z			;Alarm-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	z, r16			;Flags wieder speichern
;
	ldi	yl, low(almbak)
	ldi	yh, high(almbak)	;Zeiger auf Anzeige-Positions-Backup
	movw	zl, yl			;Zeiger kopieren
	mov	r18, r20		;Alarmnummer 1 holen
	add	yl, r18			;Tabellenplatz ermitteln
	adc	yh, r19			;bertrag
	mov	r18, r21		;Alarmnummer 2 holen
	add	zl, r18			;Tabellenplatz ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Anzeigeposition 1 holen
	ld	r17, z			;Anzeigeposition 2 holen
	st	y, r17			;Positionen tauschen und
	st	z, r16			;speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m18400:	jmp	m20000
;
; Anzeigemodus Konfiguration - Signale - Zuordnung: Anzeige komplett neu ausgeben
;
m18500:	cpi	r16, 11			;Anzeigemodus Konfiguration - Signale - Zuordnung, Auswhlen?
	breq	m18510			;ja -> weiter
	cpi	r16, 12                 ;Anzeigemodus Konfiguration - Signale - Zuordnung, Nummer/Dauer
	breq	m18510                  ;ja -> weiter
	rjmp	m19000			;sonst weiter testen
;
m18510:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m18600			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newcfg			;alle Konfigzeilen neu ausgeben
;
; Anzeigemodus Konfiguration - Signale - Zuordnung: Namen ausgeben
;
m18600:	ldi	r25, 0			;Signal-Zhler = 0
	ldi	yl, low(cfgflg)
	ldi	yh, high(cfgflg)	;Zeiger auf Konfigurationsflags
m18610:	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r25, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	ld	r16, y			;Konfigurationflags holen
	bst	r16, 0			;Name neu ausgeben?
	brtc	m18650			;nein -> berspringen
	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m18620			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
m18620:	ldi	r22, 0x65		;Text "Alarm 1"
	cpi	r25, 0			;Zhler auf Alarm 1?
	breq	m18630			;ja -> weiter
	ldi	r22, 0x66		;Text "Alarm 2"
	cpi	r25, 1			;Zhler auf Alarm 2?
	breq	m18630			;ja -> weiter
	ldi	r22, 0x67		;Text "Alarm 3"
	cpi	r25, 2			;Zhler auf Alarm 3?
	breq	m18630			;ja -> weiter
	ldi	r22, 0x68		;Text "Alarm 4"
	cpi	r25, 3			;Zhler auf Alarm 4?
	breq	m18630			;ja -> weiter
	ldi	r22, 0xfb		;Text "Temp zu hoch"
	cpi	r25, 4			;Zhler auf bertemperatur?
	breq	m18630			;ja -> weiter
	ldi	r22, 0xfc		;Text "Temp zu niedrig"
	cpi	r25, 5			;Zhler auf Untertemperatur?
	breq	m18630			;ja -> weiter
	ldi	r22, 0x69		;Text "Systemfehler"
	cpi	r25, 6			;Zhler auf Systemfehler?
	breq	m18630			;ja -> weiter
	ldi	r22, 0xf9		;sonst Text "Sensorausfall"
m18630:	call	strlcd			;Text ausgeben
	clr	txflag
;
; Anzeigemodus Konfiguration - Signale - Zuordnung: Nummer und Dauer ausgeben
;
m18650:	lds	r16, menpos		;Menposition holen
	dec	r16			;korrigieren
	cp	r16, r25		;Anzeigeposition = Menposition?
	brne	m18660			;nein -> nderungsflag prfen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	andi	r16, 0x0c		;Nummer oder Dauer ndern?
	breq	m18660			;nein -> nderungsflag prfen
	lds	r16, encflg		;sonst Impulsgeber-Flags holen
	bst	r16, 7			;Impulsgeber-nderung?
	brts	m18680			;ja -> Wert ausgeben
;
m18660:	ld	r16, y			;Konfigurationflags holen
	bst	r16, 1			;Daten neu ausgeben?
	brts	m18670			;ja -> weiter
	rjmp	m18780			;sonst berspringen
;
m18670:	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
m18680:	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 84			;Position fr Signalnummer
	ldi	r22, 'N'		;Zeichen "N" (Nummer)
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Doppelpunkt
	call	asclcd			;ausgeben
;
	lds	r16, xflag4		;verschiedene Flags 4 holen
	bst	r16, 2			;Signalnummer ndern?
	brtc	m18690			;nein -> Wert normal ausgeben
	lds	r16, encflg		;sonst Impulsgeber-Flags holen
	bst	r16, 7			;Impulsgeber-nderung?
	brtc	m18690			;nein -> Wert normal ausgeben
	cbr	r16, 1 << 7		;sonst nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	dec	r16			;korrigieren
	rjmp	m18700			;Impulsgeber-Wert ausgeben
;
m18690:	ldi	xl, low(signum)
	ldi	xh, high(signum)	;Zeiger auf Signalnummer-Tabelle
	clr	r17
	add	xl, r25			;Tabellenplatz berechnen
	adc	xh, r17			;bertrag
	ld	r16, x			;Signalnummer holen
m18700:	mov	r11, r16		;Signalnummer sichern
	inc	r16			;fr Anzeige korrigieren
	clr	r17			;H-Byte lschen
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
;
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 110		;Position fr Signaldauer
	ldi	r22, 'D'		;Zeichen "D" (Dauer)
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Doppelpunkt
	call	asclcd			;ausgeben
;
	lds	r16, xflag4		;verschiedene Flags 4 holen
	bst	r16, 3			;Signaldauer ndern?
	brtc	m18710			;nein -> Wert normal ausgeben
	lds	r16, encflg		;sonst Impulsgeber-Flags holen
	bst	r16, 7			;Impulsgeber-nderung?
	brtc	m18710			;nein -> Wert normal ausgeben
	cbr	r16, 1 << 7		;sonst nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	rjmp	m18720			;Impulsgeber-Wert ausgeben
;
m18710:	ldi	xl, low(sigdur)
	ldi	xh, high(sigdur)	;Zeiger auf Signaldauer-Tabelle
	clr	r17
	add	xl, r25			;Tabellenplatz berechnen
	adc	xh, r17			;bertrag
	ld	r16, x			;Signaldauer holen
m18720:	clr	r17			;H-Byte lschen
	call	bytas4			;Wert in 3 Byte ASCII wandeln
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
;
; Anzeigemodus Konfiguration - Signale - Zuordnung: Signaldiagramm ausgeben
;
	ldi	r18, 136		;X-Startposition fr Grafik
	mov	r19, r13		;Y-Startposition holen
	ldi	r16, 3			;Grafik beginnt 3 Pixel tiefer
	add	r19, r16		;Y-Startposition berechnen
	ldi	r20, 237		;X-Endposition fr Grafik
	mov	r21, r13		;Y-Startposition holen
	ldi	r16, 9			;Grafik endet 9 Pixel tiefer
	add	r21, r16		;Y-Endposition berechnen
	call	clrwnd			;Bildausschnitt lschen
;
	ldi	r18, 136		;Position fr Signaldiagramm
	mov	r16, r11		;gesicherte Signalnummer holen
	add	r16, r16
	add	r16, r16
	add	r16, r16		;Tabellenoffset berechnen (x8)
	clr	r17
	ldi	zl, low(sndtab * 2)
	ldi	zh, high(sndtab * 2)	;Zeiger auf Signaltabelle L/H
	add	zl, r16			;Tabellenplatz ermitteln
	adc	zh, r17			;bertrag
	ldi	r16, byte3(sndtab * 2)	;Zeiger auf Signaltabelle B3
	adc	r16, r17
	out	rampz, r16
;
	clr	r24			;Zeitmarkenzhler = 0
	clr	r23			;Tabellenzhler = 0
	ldi	r22, 8			;Pegelwert = Low
	mov	r19, r13		;gesicherte Y-Position holen
	ldi	r16, 8			;erstes Pixel hat Low-Pegel
	add	r19, r16		;addieren
	ori	r19, 0x80		;Pixel setzen
	call	pixlcd			;ausgeben
	inc	r18			;nchste X-Position
;
m18730:	elpm	r21, z			;Eintrag aus Signaltabelle holen
	cp	r24, r21		;Zeitmarke = Eintrag?
	brne	m18770			;nein -> letzten Zustand zeichnen
	adiw	zl, 1			;sonst nchster Tabelleneintrag
	cpi	r22, 8			;Pegelwert=Low?
	breq	m18740			;ja -> auf High setzen
	ldi	r22, 8			;sonst auf Low setzen
	rjmp	m18750
m18740:	ldi	r22, 3			;Pegelwert auf High setzen
m18750:	ldi	r20, 3			;Flanke zeichnen; Y-Offset
m18760:	mov	r19, r13		;gesicherte Y-Position holen
	add	r19, r20		;Offset addieren
	ori	r19, 0x80		;Pixel setzen
	call	pixlcd			;ausgeben
	inc	r20			;nchster Y-Wert
	cpi	r20, 9			;Endwert erreicht?
	brne	m18760			;nein -> Schleife
;
m18770:	mov	r19, r13		;gesicherte Y-Position holen
	add	r19, r22		;Pegel addieren
	ori	r19, 0x80		;Pixel setzen
	call	pixlcd			;ausgeben
	inc	r18			;nchste X-Position
	call	pixlcd			;ausgeben
	inc	r18			;nchste X-Position
	inc	r24			;nchste Zeitmarke
	cpi	r24, 50			;letzte Zeitmarke erreicht?
	brcs	m18730			;nein -> Schleife
;
m18780:	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Alarmnummer
	cpi	r25, 8			;alle Signale bearbeitet?
	brcc	m18800			;ja -> weiter
	rjmp	m18610			;sonst Schleife
;
; Anzeigemodus Konfiguration - Signale - Zuordnung: Auswahl bearbeiten
;
m18800:	lds	r16, dimode		;Anzeigemodus holen?
	cpi	r16, 11			;Anzeigemodus Konfiguration - Signale- Zuordnung, Auswhlen?
	brne	m18810			;nein -> weiter
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m18810			;nein -> weiter
;
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	movw	yl, xl			;Zeiger kopieren
	dec	r16			;Positionswert korrigieren
	clr	r19
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r19			;bertrag
	ld	r16, x			;Alarm-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	x, r16			;Flags wieder speichern
	dec	r17			;Impulsgeberwert korrigieren
	add	yl, r17			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Alarm-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m18810:	jmp	m20000
;
; Anzeigemodus Konfiguration - Signale - Audio: Anzeige komplett neu ausgeben
;
m19000:	cpi	r16, 13			;Anzeigemodus Konfiguration - Signale - Audio, Auswhlen?
	breq	m19010			;ja -> weiter
	rjmp	m19400			;sonst weiter testen
;
m19010:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m19100			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newcfg			;alle Konfigzeilen neu ausgeben
;
; Anzeigemodus Konfiguration - Signale - Audio: Namen ausgeben
;
m19100:	ldi	r25, 0			;Zeilen-Zhler = 0
	ldi	yl, low(cfgflg)
	ldi	yh, high(cfgflg)	;Zeiger auf Konfigurationsflags
m19110:	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r25, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	ld	r16, y			;Konfigurationflags holen
	bst	r16, 0			;Name neu ausgeben?
	brtc	m19150			;nein -> berspringen
	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m19120			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
m19120:	ldi	r22, 0x6f		;Text "Signalfrequenz"
	add	r22, r25		;Zhler addieren -> Textnummer
	call	strlcd			;Text ausgeben
	clr	txflag
;
; Anzeigemodus Konfiguration - Signale - Audio: Daten ausgeben
;
m19150:	ld	r16, y			;Konfigurationflags holen
	bst	r16, 1			;Daten neu ausgeben?
	brts	m19160			;ja -> weiter
	rjmp	m19210			;sonst berspringen
;
m19160:	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 110		;Position fr Daten
	cpi	r25, 0			;Zhler auf Signalfrequenz?
	brne	m19180			;nein -> weiter testen
	movw	r12, r20		;X/Y-Koordinaten sichern
	call	timfrq			;sonst Timer0-Wert in Frequenz
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	call	bytas4			;umrechnen und in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	cpi	r22, '0'		;erste Stelle = 0?
	breq	m19170			;ja -> berspringen
	call	asclcd			;sonst ausgeben
m19170:	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	ldi	r22, 'H'		;Zeichen 'H'
	call	asclcd			;ausgeben
	ldi	r22, 'z'		;Zeichen 'z' (Hz)
	call	asclcd			;ausgeben
	ldi	r22, ' '		;Leerzeichen
	call	asclcd			;ausgeben
	rjmp	m19210
;
m19180:	lds	r16, sigvo1		;Signal Lautstrke Tag holen
	cpi	r25, 1			;Zhler auf Lautstrke Tag?
	breq	m19190			;ja -> weiter
	lds	r16, sigvo2		;sonst Lautstrke Nacht holen
m19190:	clr	r17			;H-Byte lschen
	call	bytas4			;in ASCII wandeln
	mov	r22, r6			;dritte Stelle holen
	cpi	r22, '0'		;dritte Stelle = 0?
	breq	m19200			;ja -> berspringen
	call	asclcd			;sonst ausgeben
m19200:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, ' '		;Leerzeichen
	call	asclcd			;ausgeben
;
m19210:	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Zeile
	cpi	r25, 3			;alle Zeilen bearbeitet?
	brcc	m19300			;ja -> weiter
	rjmp	m19110			;sonst Schleife
;
; Anzeigemodus Konfiguration - Signale - Audio: Auswahl bearbeiten
;
m19300:	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m19310			;nein -> weiter
;
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	movw	yl, xl			;Zeiger kopieren
	dec	r16			;Positionswert korrigieren
	clr	r19
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r19			;bertrag
	ld	r16, x			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	x, r16			;Flags wieder speichern
	dec	r17			;Impulsgeberwert korrigieren
	add	yl, r17			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m19310:	jmp	m20000
;
; Anzeigemodus Konfiguration - Signale - Audio: ndern bearbeiten
;
m19400:	cpi	r16, 14			;Anzeigemodus Konfiguration - Signale - Audio, ndern?
	breq	m19410			;ja -> weiter
	rjmp	m19500			;sonst weiter testen
;
m19410:	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 7			;Wert gendert?
	brts	m19420			;ja -> ausgeben
	rjmp	m19470			;sonst Ende
;
m19420:	cbr	r16, 1 << 7		;Impulsgeber nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	lds	r16, menpos		;ausgewhlte Position holen
	dec	r16			;Positionswert korrigieren
	ldi	r20, 110		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
	cpi	r16, 0			;Position auf Signalfrequenz?
	brne	m19440			;nein -> Lautstrke bearbeiten
	lds	r16, encval		;Impulsgeber-Wert L holen
	lds	r17, encvah		;Impulsgeber-Wert H holen
	call	bytas4			;in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	cpi	r22, '0'		;erste Stelle = 0?
	breq	m19430			;ja -> berspringen
	call	asclcd			;sonst ausgeben
m19430:	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	ldi	r22, 'H'		;Zeichen 'H'
	call	asclcd			;ausgeben
	ldi	r22, 'z'		;Zeichen 'z' (Hz)
	call	asclcd			;ausgeben
	ldi	r22, ' '		;Leerzeichen
	call	asclcd			;ausgeben
	rjmp	m19460			;Zeilenende lschen
;
m19440:	lds	r16, encval		;Impulsgeber-Wert holen
	clr	r17			;H-Byte lschen
	call	bytas4			;in ASCII wandeln
	mov	r22, r6			;dritte Stelle holen
	cpi	r22, '0'		;dritte Stelle = 0?
	breq	m19450			;ja -> berspringen
	call	asclcd			;sonst ausgeben
m19450:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, ' '		;Leerzeichen
	call	asclcd			;ausgeben
;
m19460:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 237		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
m19470:	jmp	m20000
;
; Anzeigemodus Konfiguration - Zeiten - Schaltzeiten: Anzeige komplett neu ausgeben
;
m19500:	cpi	r16, 15			;Anzeigemodus Konfiguration - Zeiten - Schaltzeiten, Auswhlen?
	breq	m19510			;ja -> weiter
	rjmp	m19900			;sonst weiter testen
;
m19510:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m19600			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newcfg			;alle Konfigzeilen neu ausgeben
;
; Anzeigemodus Konfiguration - Zeiten - Schaltzeiten: Namen ausgeben
;
m19600:	ldi	r25, 0			;Zeilen-Zhler = 0
	ldi	yl, low(cfgflg)
	ldi	yh, high(cfgflg)	;Zeiger auf Konfigurationsflags
m19610:	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r25, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	ld	r16, y			;Konfigurationflags holen
	bst	r16, 0			;Name neu ausgeben?
	brtc	m19700			;nein -> berspringen
	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m19620			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
m19620:	cpi	r25, 4			;Zhler > 3?
	brcc	m19630			;ja -> weiter
	ldi	r22, 0x74		;Text "Signalmodus Nacht von"
	rjmp	m19650			;Text ausgeben
m19630: cpi	r25, 6			;Zhler > 5?
	brcc	m19640			;ja -> weiter
	ldi	r22, 0xb1 - 4		;Text "Bluetooth aus von"
	rjmp	m19650			;Text ausgeben
m19640:	ldi	r22, 0x78 - 6		;Text "Auto-Reset Min-Werte"
m19650:	add	r22, r25		;Zhler addieren -> Textnummer
	call	strlcd			;Text ausgeben
	clr	txflag
;
; Anzeigemodus Konfiguration - Zeiten - Schaltzeiten: Daten ausgeben
;
m19700:	ld	r16, y			;Konfigurationflags holen
	bst	r16, 1			;Daten neu ausgeben?
	brts	m19710			;ja -> weiter
	rjmp	m19790			;sonst berspringen
;
m19710:	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 198		;Position fr Checkbox
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	lds	r17, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 5			;Flag Nachtschaltung Signale
	cpi	r25, 2			;Zeile 0/1 Nachtschaltung Signale?
	brcs	m19720			;ja -> bearbeiten
	bst	r16, 3			;Flag Nachtschaltung Licht
	cpi	r25, 4			;Zeile 2/3 Nachtschaltung Licht?
	brcs	m19720			;ja -> bearbeiten
	bst	r17, 2			;Flag Nachtschaltung Bluetooth
	cpi	r25, 6			;Zeile 4/5 Nachtschaltung Bluetooth?
	brcs	m19720			;ja -> bearbeiten
	bst	r16, 4			;Flag Auto-Reset Min/Max-Werte?
m19720:	brtc	m19730			;Flag gelscht? ja -> weiter
	ldi	r22, 0xb5		;sonst Zeichen "Checkbox1 ein"
m19730:	call	asclcd			;Zeichen ausgeben
;
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 210		;Position fr Daten
	lds	r16, signig
	lds	r17, signig + 1		;Zeit Signalmodus Nacht 1 laden
	cpi	r25, 0			;Zhler Signalmodus Nacht 1?
	breq	m19740			;ja -> bearbeiten
	lds	r16, signig + 2
	lds	r17, signig + 3		;Zeit Signalmodus Nacht 2 laden
	cpi	r25, 1			;Zhler Signalmodus Nacht 2?
	breq	m19740			;ja -> bearbeiten
	lds	r16, nighbl
	lds	r17, nighbl + 1		;Zeit LCD Beleuchtung 1 laden
	cpi	r25, 2			;Zhler LCD Beleuchtung 1?
	breq	m19740			;ja -> bearbeiten
	lds	r16, nighbl + 2
	lds	r17, nighbl + 3		;Zeit LCD Beleuchtung 2 laden
	cpi	r25, 3			;Zhler LCD Beleuchtung 2?
	breq	m19740			;ja -> bearbeiten
	lds	r16, nighbt
	lds	r17, nighbt + 1		;Zeit Bluetooth 1 laden
	cpi	r25, 4			;Zhler Bluetooth Zeit 1?
	breq	m19740			;ja -> bearbeiten
	lds	r16, nighbt + 2
	lds	r17, nighbt + 3		;Zeit Bluetooth 2 laden
	cpi	r25, 5			;Zhler Bluetooth Zeit 2?
	breq	m19740			;ja -> bearbeiten
	lds	r16, aresmm
	lds	r17, aresmm + 1		;Zeit Reset Minimalwerte laden
	cpi	r25, 6			;Zhler Reset Minimalwerte?
	breq	m19740			;ja -> bearbeiten
	lds	r16, aresmm + 2		;sonst
	lds	r17, aresmm + 3		;Zeit Reset Maximalwerte laden
;
m19740:	call	timasc			;Zeit in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Trennzeichen laden
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;sonst ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
;
m19790:	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Zeile
	cpi	r25, 8			;alle Zeilen bearbeitet?
	brcc	m19800			;ja -> weiter
	rjmp	m19610			;sonst Schleife
;
; Anzeigemodus Konfiguration - Zeiten - Schaltzeiten: Auswahl bearbeiten
;
m19800:	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m19810			;nein -> weiter
;
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	movw	yl, xl			;Zeiger kopieren
	dec	r16			;Positionswert korrigieren
	clr	r19
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r19			;bertrag
	ld	r16, x			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	x, r16			;Flags wieder speichern
	dec	r17			;Impulsgeberwert korrigieren
	add	yl, r17			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m19810:	jmp	m20000
;
; Anzeigemodus Konfig - Zeiten - Schaltzeiten: ndern bearbeiten
;
m19900:	cpi	r16, 16			;Anzeigemodus Konfiguration - Zeiten - Schaltzeiten, ndern?
	breq	m19910			;ja -> weiter
	rjmp	m1a000			;sonst weiter testen
;
m19910:	lds	r16,encflg		;Impulsgeber-Flags holen
	bst	r16, 7			;Wert gendert?
	brtc	m19920			;nein -> Ende
;
	cbr	r16, 1 << 7		;Impulsgeber nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	lds	r16, menpos		;ausgewhlte Position holen
	dec	r16			;Positionswert korrigieren
	ldi	r20, 210		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
	lds	r16, encval		;Impulsgeber-Wert L holen
	lds	r17, encvah		;Impulsgeber-Wert H holen
	call	timasc			;Zeit in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Trennzeichen laden
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;sonst ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
m19920:	jmp	m20000
;
; Anzeigemodus Konfiguration - Zeiten - Uhr stellen: Anzeige komplett neu ausgeben
;
m1a000:	cpi	r16, 17			;Anzeigemodus Konfiguration - Zeiten - Uhr stellen, Auswhlen?
	breq	m1a010			;ja -> weiter
	rjmp	m1a400			;sonst weiter testen
;
m1a010:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m1a100			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newcfg			;alle Konfigzeilen neu ausgeben
;
; Anzeigemodus Konfiguration - Zeiten - Uhr stellen: Namen ausgeben
;
m1a100:	ldi	r25, 0			;Zeilen-Zhler = 0
	ldi	yl, low(cfgflg)
	ldi	yh, high(cfgflg)	;Zeiger auf Konfigurationsflags
m1a110:	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r25, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	ld	r16, y			;Konfigurationflags holen
	bst	r16, 0			;Name neu ausgeben?
	brtc	m1a150			;nein -> berspringen
	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m1a120			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
m1a120:	ldi	r22, 0x7a		;Text "Uhrzeit"
	add	r22, r25		;Zhler addieren -> Textnummer
	call	strlcd			;Text ausgeben
	clr	txflag
;
; Anzeigemodus Konfiguration - Zeiten - Uhr stellen: Daten ausgeben
;
m1a150:	ld	r16, y			;Konfigurationflags holen
	bst	r16, 1			;Daten neu ausgeben?
	brts	m1a160			;ja -> weiter
	rjmp	m1a250			;sonst berspringen
;
m1a160:	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 110		;Position fr Daten
;
	cpi	r25, 0			;Zhler auf Zeit?
	brne	m1a180			;nein -> weiter testen
	lds	r16, cptiml		;sonst
	lds	r17, cptimh		;Zeit laden
	call	timasc			;Zeit in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	cpi	r22, ' '		;Leerzeichen?
	breq	m1a170			;ja -> nicht ausgeben
	call	asclcd			;ausgeben
m1a170:	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Trennzeichen laden
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;sonst ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	rjmp	m1a240
;
m1a180:	cpi	r25, 1			;Zhler auf Tag?
	brne	m1a200			;nein -> weiter testen
	lds	r16, cpday		;sonst Tag laden
	clr	r17
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r6			;dritte Stelle holen
	cpi	r22, '0'		;dritte Stelle = 0?
	breq	m1a190			;ja -> berspringen
	call	asclcd			;ausgeben
m1a190:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	rjmp	m1a240
;
m1a200:	cpi	r25, 2			;Zhler auf Monat?
	brne	m1a210			;nein -> weiter testen
	lds	r22, cpmon		;sonst Monat laden
	ldi	r16, 0x10 - 1		;Stringnummer Monate - 1
	add	r22, r16		;Stringnummer addieren
	call	strlcd			;ausgeben
	rjmp	m1a240
;
m1a210:	cpi	r25, 3			;Zhler auf Jahr?
	brne	m1a220			;nein -> weiter testen
	ldi	r22, year3		;Jahr Tausender holen
	call	asclcd			;ausgeben
	ldi	r22, year2		;Jahr Hunderter holen
	call	asclcd			;ausgeben
	lds	r16, cpyear		;Jahr laden
	clr	r17
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	rjmp	m1a240
;
m1a220:	cpi	r25, 4			;Zhler auf Wochentag?
	brne	m1a250			;nein -> Ende
	lds	r22, cpdayw		;sonst Wochentag laden
	call	strlcd			;ausgeben
;
m1a240:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 210		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m1a250:	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Zeile
	cpi	r25, 6			;alle Zeilen bearbeitet?
	brcc	m1a300			;ja -> weiter
	rjmp	m1a110			;sonst Schleife
;
; Anzeigemodus Konfiguration - Zeiten - Uhr stellen: Auswahl bearbeiten
;
m1a300:	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m1a310			;nein -> weiter
;
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	movw	yl, xl			;Zeiger kopieren
	dec	r16			;Positionswert korrigieren
	clr	r19
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r19			;bertrag
	ld	r16, x			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	x, r16			;Flags wieder speichern
	dec	r17			;Impulsgeberwert korrigieren
	add	yl, r17			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m1a310:	jmp	m20000
;
; Anzeigemodus Konfiguration - Zeiten - Uhr stellen: ndern bearbeiten
;
m1a400:	cpi	r16, 18			;Anzeigemodus Konfiguration - Zeiten - Uhr stellen, ndern?
	breq	m1a410			;ja -> weiter
	rjmp	m1a500			;sonst weiter testen
;
m1a410:	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 7			;Wert gendert?
	brts	m1a420			;ja -> ausgeben
	rjmp	m1a4a0			;sonst Ende
;
m1a420:	cbr	r16, 1 << 7		;Impulsgeber nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	lds	r16, menpos		;ausgewhlte Position holen
	dec	r16			;Positionswert korrigieren
	ldi	r20, 110		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
	cpi	r16, 0			;Position auf Zeit?
	brne	m1a440			;nein -> weiter testen
	lds	r16, encval		;Impulsgeber-Wert L holen
	lds	r17, encvah		;Impulsgeber-Wert H holen
	call	timasc			;Zeit in ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	cpi	r22, ' '		;Leerzeichen?
	breq	m1a430			;ja -> nicht ausgeben
	call	asclcd			;ausgeben
m1a430:	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Trennzeichen laden
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;sonst ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	rjmp	m1a490			;Zeilenende lschen
;
m1a440:	cpi	r16, 1			;Position auf Tag?
	brne	m1a460			;nein -> weiter testen
	lds	r16, encval		;Impulsgeber-Wert holen
	clr	r17
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r6			;dritte Stelle holen
	cpi	r22, '0'		;dritte Stelle = 0?
	breq	m1a450			;ja -> berspringen
	call	asclcd			;ausgeben
m1a450:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	rjmp	m1a490			;Zeilenende lschen
;
m1a460:	cpi	r16, 2			;Position auf Monat?
	brne	m1a470			;nein -> weiter testen
	lds	r22, encval		;Impulsgeber-Wert holen
	ldi	r16, 0x10 - 1		;Stringnummer Monate - 1
	add	r22, r16		;Stringnummer addieren
	call	strlcd			;ausgeben
	rjmp	m1a490			;Zeilenende lschen
;
m1a470:	cpi	r16, 3			;Position auf Jahr?
	brne	m1a480			;nein -> Wochentag bearbeiten
	lds	r16, encval		;Impulsgeber-Wert L holen
	lds	r17, encvah		;Impulsgeber-Wert H holen
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r4			;erste Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r5			;zweite Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r6			;dritte Stelle holen
	call	asclcd			;ausgeben
	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	rjmp	m1a490			;Zeilenende lschen
;
m1a480:	lds	r22, encval		;Impulsgeber-Wert holen (Wochentag)
	dec	r22			;korrigieren
	call	strlcd			;ausgeben
;
m1a490:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 237		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
m1a4a0:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sonstiges 1: Anzeige komplett neu ausgeben
;
m1a500:	cpi	r16, 19			;Anzeigemodus Konfiguration - Sonstiges 1?
	breq	m1a510			;ja -> weiter
	rjmp	m1a900			;sonst weiter testen
;
m1a510:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m1a600			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newcfg			;alle Konfigurtionszeilen neu ausgeben
;
; Anzeigemodus Konfiguration - Sonstiges 1: Namen ausgeben
;
m1a600:	ldi	r25, 0			;Zeilen-Zhler = 0
	ldi	yl, low(cfgflg)
	ldi	yh, high(cfgflg)	;Zeiger auf Konfigurations-Flags
m1a610:	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r25, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	ld	r16, y			;Konfigurations-Flags holen
	bst	r16, 0			;Name neu ausgeben?
	brtc	m1a650			;nein -> berspringen
	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m1a620			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
;
m1a620:	ldi     r22, 0x80               ;Text "Sprache"
        cpi     r25, 0                  ;Zhler auf Sprache?
        breq    m1a630                  ;ja -> ausgeben
        ldi     r22, 0x81               ;Text "Trennzeichen"
        cpi     r25, 1                  ;Zhler auf Trennzeichen?
        breq    m1a630                  ;ja -> ausgeben
        ldi     r22, 0x82               ;Text "Temp-Alarm immer signalisieren"
        cpi     r25, 2                  ;Zhler auf Temp-Alarm?
        breq    m1a630                  ;ja -> ausgeben
        ldi     r22, 0x83               ;Text "LCD-Abschaltung"
        cpi     r25, 3                  ;Zhler auf LCD-Abschaltung?
        breq    m1a630                  ;ja -> ausgeben
        ldi	r22, 0x9b		;Text "LCD-Automatik"
	cpi	r25, 4			;Zhler auf LCD-Automatik?
	breq	m1a630			;ja -> ausgeben
	ldi	r22, 0x84		;Text "LCD Nacht-Helligkeit"
	cpi	r25, 5			;Zhler auf LCD Nacht-Helligkeit?
	breq	m1a630			;ja -> ausgeben
	ldi	r22, 0x33		;Text "Auf Seite 1 zurckschalten"
	cpi	r25, 6			;Zhler auf Seite 1 zurckschalten?
	breq	m1a630			;ja -> ausgeben
	ldi	r22, 0x85		;sonst Text "Baudrate PC-Schnittstelle"
;
m1a630:	call	strlcd			;Text ausgeben
	clr	txflag
;
; Anzeigemodus Konfiguration - Sonstiges 1: Daten ausgeben
;
m1a650:	ld	r16, y			;Konfigurations-Flags holen
	bst	r16, 1			;Daten neu ausgeben?
	brts	m1a660			;ja -> weiter
	rjmp	m1a830			;sonst berspringen
;
m1a660:	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 180		;Position fr Daten
;
	cpi	r25, 0			;Zhler auf Sprache?
	brne	m1a670			;nein -> weiter testen
	ldi	r22, 9			;Sprache laden
	call	strlcd			;ausgeben
	rjmp	m1a820
;
m1a670:	cpi	r25, 1			;Zhler auf Trennzeichen?
	brne	m1a690			;nein -> weiter testen
	ldi	r22, '.'		;Trennzeichen Punkt laden
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 1			;Trennzeichen Punkt?
	brtc	m1a680			;ja -> weiter
	ldi	r22, ','		;sonst Trennzeichen Komma laden
m1a680:	call	asclcd			;ausgeben
	rjmp	m1a820
;
m1a690:	cpi	r25, 2			;Zhler auf Temperatur-Alarm-Modus?
	brne	m1a710			;nein -> weiter testen
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 2			;Temperatur-Alarme immer melden?
	brtc	m1a700			;nein -> weiter
	ldi	r22, 0xb5		;Zeichen "Checkbox1 ein"
m1a700:	call	asclcd			;ausgeben
	rjmp	m1a820
;
m1a710:	cpi	r25, 3			;Zhler auf LCD-Abschaltung?
	brne	m1a730			;nein -> weiter testen
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 6			;LCD Abschaltung 3:00?
	brtc	m1a720			;nein -> weiter
	ldi	r22, 0xb5		;Zeichen "Checkbox1 ein"
m1a720:	call	asclcd			;ausgeben
	rjmp	m1a820
;
m1a730:	cpi	r25, 4			;Zhler auf LCD-Beleuchtungs-Automatik?
	brne	m1a750			;nein -> weiter testen
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 1			;LCD Beleuchtungs-Automatik?
	brtc	m1a740			;nein -> weiter
	ldi	r22, 0xb5		;Zeichen "Checkbox1 ein"
m1a740:	call	asclcd			;ausgeben
	rjmp	m1a820
;
m1a750:	cpi	r25, 5			;Zhler auf LCD Nacht-Helligkeit?
	brne	m1a790			;nein -> weiter testen
	lds	r16, nlight		;LCD Nacht-Helligkeit
	clr	r17
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r5			;zweite Stelle holen
	cpi	r22, '0'		;zweite Stelle = 0?
	breq	m1a760			;ja -> berspringen
	call	asclcd			;sonst ausgeben
	mov	r22, r6			;dritte Stelle holen
	rjmp	m1a770			;sofort ausgeben
m1a760:	mov	r22, r6			;dritte Stelle holen
	cpi	r22, '0'		;dritte Stelle = 0?
	breq	m1a780			;ja -> berspringen
m1a770:	call	asclcd			;ausgeben
m1a780:	mov	r22, r7			;vierte Stelle holen
	call	asclcd			;ausgeben
	rjmp	m1a820
;
m1a790: cpi	r25, 6			;Zhler auf Seite-1-Rckschaltung?
	brne	m1a810			;nein -> weiter
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 3			;Rckschaltung auf Seite 1?
	brtc	m1a800			;nein -> weiter
	ldi	r22, 0xb5		;Zeichen "Checkbox1 ein"
m1a800:	call	asclcd			;ausgeben
	rjmp	m1a820
;
m1a810:	ldi	r22, 0x86		;Text "9600"
	lds	r16, baud0		;Baudrate PC-Schnittstelle
	add	r22, r16		;Baudratentext generieren
	call	strlcd			;ausgeben
;
m1a820:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 237		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m1a830:	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Zeile
	cpi	r25, 8			;alle Zeilen bearbeitet?
	brcc	m1a850			;ja -> weiter
	rjmp	m1a610			;sonst Schleife
;
; Anzeigemodus Konfiguration - Sonstiges 1: Auswahl bearbeiten
;
m1a850:	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m1a860			;nein -> weiter
;
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurations-Flags
	movw	yl, xl			;Zeiger kopieren
	dec	r16			;Positionswert korrigieren
	clr	r19
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r19			;bertrag
	ld	r16, x			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	x, r16			;Flags wieder speichern
	dec	r17			;Impulsgeberwert korrigieren
	add	yl, r17			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m1a860:	jmp	m20000
;
; Anzeigemodus Konfig - Sonstiges 1: ndern bearbeiten
;
m1a900:	cpi	r16, 20			;Anzeigemodus Konfiguration - Sonstiges 1, ndern?
	breq	m1a910			;ja -> weiter
	rjmp	m1aa00			;sonst weiter testen
;
m1a910:	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 7			;Wert gendert?
	brtc	m1a950			;nein -> Ende
;
	cbr	r16, 1 << 7		;Impulsgeber nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	lds	r16, menpos		;ausgewhlte Position holen
	dec	r16			;Positionswert korrigieren
	ldi	r20, 180		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
	cpi	r16, 5			;Position auf Nacht-Helligkeit?
	brne	m1a950			;nein -> Ende
	lds	r16, encval		;sonst Impulsgeber-Wert holen
	clr	r17			;H-Byte = 0
	call	bytas4			;Wert in 4 Byte ASCII wandeln
	mov	r22, r5			;Hunderterstelle holen
	cpi	r22, '0'		;Hunderterstelle = 0?
	breq	m1a920			;ja -> berspringen
	call	asclcd			;sonst ausgeben
	mov	r22, r6			;Zehnerstelle holen
	rjmp	m1a930			;sofort ausgeben
m1a920:	mov	r22, r6			;Zehnerstelle holen
	cpi	r22, '0'		;Zehnerstelle = 0?
	breq	m1a940			;ja -> berspringen
m1a930:	call	asclcd			;ausgeben
m1a940:	mov	r22, r7			;Einerstelle holen
	call	asclcd			;ausgeben
;
	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 237		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
m1a950:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sonstiges 2: Anzeige komplett neu ausgeben
;
m1aa00:	cpi	r16, 32			;Anzeigemodus Konfiguration - Sonstiges 2?
	breq	m1aa10			;ja -> weiter
	rjmp	m1b000			;sonst weiter testen
;
m1aa10:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m1ab00			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newcfg			;alle Konfigurtionszeilen neu ausgeben
;
; Anzeigemodus Konfiguration - Sonstiges 1: Namen ausgeben
;
m1ab00:	ldi	r25, 0			;Zeilen-Zhler = 0
	ldi	yl, low(cfgflg)
	ldi	yh, high(cfgflg)	;Zeiger auf Konfigurations-Flags
m1ab10:	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r25, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	ld	r16, y			;Konfigurations-Flags holen
	bst	r16, 0			;Name neu ausgeben?
	brtc	m1ab40			;nein -> berspringen
	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m1ab20			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
;
m1ab20:	ldi     r22, 0x9c               ;Text "DCF77-Signal invertiert"
        cpi     r25, 0                  ;Zhler auf DCF77-Signal invertiert?
        breq    m1ab30                  ;ja -> ausgeben
        ldi     r22, 0x9d               ;sonst Text "DCF77-Eingang mit Pull-up"
m1ab30:	call	strlcd			;Text ausgeben
	clr	txflag
;
; Anzeigemodus Konfiguration - Sonstiges 2: Daten ausgeben
;
m1ab40:	ld	r16, y			;Konfigurations-Flags holen
	bst	r16, 1			;Daten neu ausgeben?
	brtc	m1ab90			;nein -> berspringen
;
	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 180		;Position fr Daten
;
	cpi	r25, 0			;Zhler auf DCF77-Signal invertiert?
	brne	m1ab60			;nein -> weiter
	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 4			;DCF77-Signal invertiert?
	brtc	m1ab50			;nein -> weiter
	ldi	r22, 0xb5		;Zeichen "Checkbox1 ein"
m1ab50:	call	asclcd			;ausgeben
	rjmp	m1ab80
;
m1ab60:	ldi	r22, 0xb4		;Zeichen "Checkbox1 aus"
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 5			;DCF77-Eingang mit Pull-up?
	brtc	m1ab70			;nein -> weiter
	ldi	r22, 0xb5		;Zeichen "Checkbox1 ein"
m1ab70:	call	asclcd			;ausgeben
;
m1ab80:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 237		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m1ab90:	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Zeile
	cpi	r25, 2			;alle Zeilen bearbeitet?
	brcs	m1ab10			;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sonstiges 2: Auswahl bearbeiten
;
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m1ac00			;nein -> weiter
;
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurations-Flags
	movw	yl, xl			;Zeiger kopieren
	dec	r16			;Positionswert korrigieren
	clr	r19
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r19			;bertrag
	ld	r16, x			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	x, r16			;Flags wieder speichern
	dec	r17			;Impulsgeberwert korrigieren
	add	yl, r17			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m1ac00:	jmp	m20000
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1: Anzeige komplett neu ausgeben
;
m1b000:	cpi	r16, 21			;Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1?
	breq	m1b010			;ja -> weiter
	rjmp	m1b300			;sonst weiter testen
;
m1b010:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m1b100			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
;
	ldi	r19, 15 | 0x80		;Y = 15
m1b020:	ldi	r18, 80			;X = 80, Trennlinie 1
	call	pixlcd			;senkrechte Linie zeichnen
	ldi	r18, 160		;X = 160, Trennlinie 2
	call	pixlcd			;senkrechte Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	cpi	r19, 112 | 0x80		;alle Pixel ausgegeben?
	brne	m1b020			;nein -> Schleife
	call	newsen			;alle Namen und Werte neu ausgeben
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1: Sensornamen ausgeben
;
m1b100: ldi	r25, 0			;Sensornummerzhler = 0
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
;
m1b110:	ld	r16, y			;Sensor-Flags holen
	bst	r16, 1			;Name neu ausgeben?
	brtc	m1b190			;nein -> berspringen
;
	cbr	r16, 0x02		;nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	mov	r18, r25		;Sensornummer holen
        ror     r18
        ror     r18
        ror     r18                     ;durch 8 teilen
        andi    r18, 0x03               ;Anzeigespalte ermitteln
        ldi     r19, 80                 ;Spaltenbreite = 80
        mul     r18, r19                ;Spaltenposition ermitteln
	ldi	r20, 2			;X-Startposition Spalte 1
        add     r20, r0                 ;Spaltenposition addieren
	ldi	r21, 16			;Y-Startposition
	mov	r16, r25		;Sensornummer holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
	lds	r19, menpos		;ausgewhlte Men-Position holen
	dec	r19			;korrigieren
	cp	r19, r25		;Position identisch?
	brne	m1b120			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen

m1b120: ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
        clr     txflag                  ;Textflags lschen
;
m1b190:	adiw	yl, 1			;Sensor-Flag Tabellenzeiger erhhen
	inc	r25			;nchste Sensornummer
        cpi     r25, 24                 ;alle Sensoren bearbeitet?
        brcs    m1b110                  ;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1: Log-Positionen ausgeben
;
        ldi	r25, 0			;Sensornummerzhler = 0
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
        ldi     xl, low(logtb1)
        ldi     xh, high(logtb1)        ;Zeiger auf Log-Tabelle 1
;
m1b200:	ld	r16, y			;Sensor-Flags holen
	bst	r16, 0			;Wert neu ausgeben?
	brtc	m1b230			;nein -> berspringen
;
	cbr	r16, 0x01		;nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	mov	r18, r25		;Sensornummer holen
        ror     r18
        ror     r18
        ror     r18                     ;durch 8 teilen
        andi    r18, 0x03               ;Anzeigespalte ermitteln
        ldi     r19, 80                 ;Spaltenbreite = 80
        mul     r18, r19                ;Spaltenposition ermitteln
	ldi	r20, 52			;X-Startposition Spalte 1
        add     r20, r0                 ;Spaltenposition addieren
	ldi	r21, 16			;Y-Startposition
	mov	r16, r25		;Sensornummer holen
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
        ld      r18, x                  ;Log-Tabelleneintrag holen
        bst     r18, 7                  ;Konflikt-Flag gesetzt?
        brtc    m1b210                  ;nein -> weiter
        ldi     r22, 0xb3               ;sonst Pfeil rechts
        rjmp    m1b220                  ;ausgeben
;
m1b210: ldi     r22, ' '                ;alternativ Leerzeichen
m1b220: call    asclcd                  ;ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
;
        ldi     r22, 'L'                ;Zeichen 'L'
        call    asclcd                  ;ausgeben
        ld      r18, x                  ;Log-Tabelleneintrag holen
        andi    r18, 0x1f               ;Log-Position filtern
        inc     r18                     ;korrigieren
        call    bytasc                  ;in ASCII wandeln
        mov     r13, r16                ;Einer zwischenspeichern
        mov     r22, r17                ;Zehner holen
        call    asclcd                  ;ausgeben
        mov     r22, r13                ;Einer holen
        call    asclcd                  ;ausgeben
;
m1b230:	adiw	yl, 1			;Sensor-Flag Tabellenzeiger erhhen
        adiw    xl, 1                   ;Log-Tabellenzeiger erhhen
	inc	r25			;nchste Sensornummer
        cpi     r25, 24                 ;alle Sensoren bearbeitet?
        brcs    m1b200                  ;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1: Auswhlen bearbeiten
;
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
        breq    m1b290                  ;nein -> Ende
;
	ldi	yl, low(senflg)         ;sonst
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	movw	zl, yl			;Zeiger kopieren
	clr	r19			;H-Byte = 0
        add     yl, r16                 ;aktuelle Men-Position addieren (entspricht alter Sensornummer)
        adc     yh, r19                 ;bertrag
        sbiw    yl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, y			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
        add     zl, r17                 ;aktuellen Impulsgeberwert addieren (entspricht neuer Sensornummer)
	adc	zh, r19			;bertrag
        sbiw    zl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, z			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	z, r16			;Flags wieder speichern
	sts	menpos, r17		;neuen Impulsgeberwert als neue Position setzen
m1b290: jmp     m20000
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1: Log-Zuordnung ndern
;
m1b300:	cpi	r16, 22			;Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1, ndern?
	breq	m1b310			;ja -> weiter
        rjmp    m1b500                  ;sonst weiter testen
;
m1b310:	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 7			;Wert gendert?
	brtc	m1b390			;nein -> Ende
;
	cbr	r16, 1 << 7		;sonst Impulsgeber nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;Positionswert korrigieren
        ror     r18
        ror     r18
        ror     r18                     ;durch 8 teilen
        andi    r18, 0x03               ;Anzeigespalte ermitteln
        ldi     r19, 80                 ;Spaltenbreite = 80
        mul     r18, r19                ;Spaltenposition ermitteln
	ldi	r20, 65			;X-Startposition Spalte 1
        add     r20, r0                 ;Spaltenposition addieren
	ldi	r21, 16			;Y-Startposition
	lds	r16, menpos		;ausgewhlte Position holen
        dec     r16                     ;korrigieren
	andi	r16, 0x07		;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
        lds     r18, encval             ;Impulsgeberwert holen
        call    bytasc                  ;in ASCII wandeln
        mov     r13, r16                ;Einer zwischenspeichern
        mov     r22, r17                ;Zehner holen
        call    asclcd                  ;ausgeben
        mov     r22, r13                ;Einer holen
        call    asclcd                  ;ausgeben
m1b390: jmp     m20000
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2: Anzeige komplett neu ausgeben
;
m1b500:	cpi	r16, 30			;Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2?
	breq	m1b510			;ja -> weiter
	rjmp	m1b800			;sonst weiter testen
;
m1b510:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m1b600			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newsen			;alle Namen und Werte neu ausgeben
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2: Sensornamen ausgeben
;
m1b600: ldi	r25, 28			;Sensornummerzhler = 28
	ldi	yl, low(senflg + 28)
	ldi	yh, high(senflg + 28)	;Zeiger auf Sensor-Flags
;
m1b610:	ld	r16, y			;Sensor-Flags holen
	bst	r16, 1			;Name neu ausgeben?
        brts    m1b620                  ;ja -> bearbeiten
	rjmp	m1b690			;sonst berspringen
;
m1b620:	cbr	r16, 0x02		;nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	mov	r16, r25		;Sensornummer holen
        subi    r16, 28                 ;korrigieren
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
	lds	r19, menpos		;ausgewhlte Men-Position holen
	dec	r19			;korrigieren
	cp	r19, r16		;Position identisch?
	brne	m1b630			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen

m1b630: ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r25, r19		;Tabellenoffset ermitteln
        ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensornamen
	add	zl, r0			;Offset zur Adresse L addieren
	adc	zh, r1			;Offset zur Adresse H addieren
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;Zeichen ausgeben
;
        ldi     r22, 0xa0               ;schmales Leerzeichen
	call	asclcd			;ausgeben
        ldi     r22, '('                ;Klammer auf
	call	asclcd			;ausgeben
        ldi	r19, 9			;Sensornamenlnge im EEPROM
        mov     r18, r25                ;Sensornummer kopieren
        subi    r18, 4		        ;Sensornummer des zugehrigen Temperatursensors ermitteln
	mul	r18, r19		;Tabellenoffset ermitteln
        ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensornamen
	add	zl, r0			;Offset zur Adresse L addieren
	adc	zh, r1			;Offset zur Adresse H addieren
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
        ldi     r22, ')'                ;Klammer zu
	call	asclcd			;ausgeben
	ldi	r22, 0x7f		;Endezeichen
	call	asclcd			;ausgeben
        clr     txflag                  ;Textflags lschen
;
        ldi     r22, ' '                ;Leerzeichen
	call	asclcd			;ausgeben
        ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r25, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
;
        ldi     r22, 0xa0               ;schmales Leerzeichen
	call	asclcd			;ausgeben
        ldi     r22, '('                ;Klammer auf
	call	asclcd			;ausgeben
        ldi	r19, 9			;Sensornamenlnge im EEPROM
        mov     r18, r25                ;Sensornummer kopieren
        subi    r18, 4		        ;Sensornummer des zugehrigen Temperatursensors ermitteln
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
        ldi     r22, ')'                ;Klammer zu
	call	asclcd			;ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
;
m1b690:	adiw	yl, 1			;Sensor-Flag Tabellenzeiger erhhen
	inc	r25			;nchste Sensornummer
        cpi     r25, 31                 ;alle Sensoren bearbeitet?
        brcc    m1b695                  ;ja -> Ende
        rjmp    m1b610                  ;sonst Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2: Log-Positionen ausgeben
;
m1b695: ldi	r25, 28			;Sensornummerzhler = 28
	ldi	yl, low(senflg + 28)
	ldi	yh, high(senflg + 28)	;Zeiger auf Sensor-Flags
        ldi     xl, low(logtb2)
        ldi     xh, high(logtb2)        ;Zeiger auf Log-Tabelle 2
;
m1b700:	ld	r16, y			;Sensor-Flags holen
	bst	r16, 0			;Wert neu ausgeben?
	brtc	m1b730			;nein -> berspringen
;
	cbr	r16, 0x01		;nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	ldi	r20, 176		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	mov	r16, r25		;Sensornummer holen
	subi	r16, 28		        ;Zeilenposition filtern
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
        ld      r18, x                  ;Log-Tabelleneintrag holen
        bst     r18, 7                  ;Konflikt-Flag gesetzt?
        brtc    m1b710                  ;nein -> weiter
        ldi     r22, 0xb3               ;sonst Pfeil rechts
        rjmp    m1b720                  ;ausgeben
;
m1b710: ldi     r22, ' '                ;alternativ Leerzeichen
m1b720: call    asclcd                  ;ausgeben
        ldi     r22, 0x7f               ;Endezeichen
        call    asclcd                  ;ausgeben
;
        ldi     r22, 'L'                ;Zeichen 'L'
        call    asclcd                  ;ausgeben
        ld      r22, x                  ;Log-Tabelleneintrag holen
        andi    r22, 0x03               ;Log-Position filtern
        inc     r22                     ;korrigieren
        ori     r22, 0x30               ;in ASCII wandeln
        call    asclcd                  ;ausgeben
;
m1b730:	adiw	yl, 1			;Sensor-Flag Tabellenzeiger erhhen
        adiw    xl, 1                   ;Log-Tabellenzeiger erhhen
	inc	r25			;nchste Sensornummer
        cpi     r25, 31                 ;alle Sensoren bearbeitet?
        brcs    m1b700                  ;nein -> Schleife
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2: Auswhlen bearbeiten
;
	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
        breq    m1b790                  ;nein -> Ende
;
	ldi	yl, low(senflg + 28)    ;sonst
	ldi	yh, high(senflg + 28)	;Zeiger auf Sensor-Flags
	movw	zl, yl			;Zeiger kopieren
	clr	r19			;H-Byte = 0
        add     yl, r16                 ;aktuelle Men-Position addieren (entspricht alter Sensornummer)
        adc     yh, r19                 ;bertrag
        sbiw    yl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, y			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
        add     zl, r17                 ;aktuellen Impulsgeberwert addieren (entspricht neuer Sensornummer)
	adc	zh, r19			;bertrag
        sbiw    zl, 1                   ;korrigieren fr logische Sensornummer
	ld	r16, z			;Sensor-Flags holen
	sbr	r16, 0x02		;Name neu ausgeben
	st	z, r16			;Flags wieder speichern
	sts	menpos, r17		;neuen Impulsgeberwert als neue Position setzen
m1b790: jmp     m20000
;
; Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2: Log-Zuordnung ndern
;
m1b800:	cpi	r16, 31			;Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2, ndern?
	breq	m1b810			;ja -> weiter
        rjmp    m1c000                  ;sonst weiter testen
;
m1b810:	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 7			;Wert gendert?
	brtc	m1b890			;nein -> Ende
;
	cbr	r16, 1 << 7		;sonst Impulsgeber nderungs-Flag lschen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r20, 189		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	lds	r16, menpos		;ausgewhlte Position holen
        dec     r16                     ;korrigieren
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
        lds     r22, encval             ;Impulsgeberwert holen
        ori     r22, 0x30               ;in ASCII wandeln
        call    asclcd                  ;ausgeben
m1b890: jmp     m20000
;
; Anzeigemodus Grafik: Anzeige komplett neu ausgeben
;
m1c000:	cpi	r16, 23			;Anzeigemodus Grafik?
	breq	m1c010			;ja -> weiter
	rjmp	m1f000			;sonst weiter testen
;
m1c010:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m1c100			;nein -> weiter testen
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
;
	ldi	r18, 15			;X=15 \ Anfangspunkt Y-Achse
	ldi	r19, 16 | 0x80		;Y=16 / fr beide Diagramme
m1c020:	call	pixlcd			;senkrechte Linie zeichnen
	inc	r19			;Y-Koordinate erhhen
	cpi	r19, 104 | 0x80		;alle Pixel ausgegeben?
	brne	m1c020			;nein -> Schleife
;
	ldi	r18, 16			;X=16  \ Anfangspunkt X-Achse
	ldi	r19, 103 | 0x80		;Y=103 / fr Diagramm 1
m1c030:	call	pixlcd			;waagerechte Linie zeichnen
	inc	r18			;X-Koordinate erhhen
	cpi	r18, 238		;alle Pixel ausgegeben?
	brne	m1c030			;nein -> Schleife
;
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppeldiagramm?
	brtc	m1c050			;nein -> nur ein Diagramm
;
	ldi	r18, 16			;X=16  \ Anfangspunkt X-Achse
	ldi	r19, 59 | 0x80		;Y=103 / fr Diagramm 2
m1c040:	call	pixlcd			;waagerechte Linie zeichnen
	inc	r18			;X-Koordinate erhhen
	cpi	r18, 238		;alle Pixel ausgegeben?
	brne	m1c040			;nein -> Schleife
;
m1c050:	call	grclrb			;Grafikpuffer/Pointer initialisieren
;
m1c100:	lds	r16, xflag3		;verschiedene Flags 3 holen
	bst	r16, 1			;Daten aus EEPROM einlesen?
	brts	m1c110			;ja -> Daten einlesen
	rjmp	m1d000			;sonst weiter testen
;
; Anzeigemodus Grafik: Daten aus dem Externen EERPOM einlesen und im Datenpuffer speichern
;
m1c110:	lds	r17, errflg		;Fehler-Flags holen
	bst	r17, 3			;Speicherfehler?
	brtc	m1c210			;nein -> weiter
;
m1c200:	lds	r16, xflag3		;sonst verschiedene Flags holen
	cbr	r16, 1 << 1		;Daten einlesen beenden und im
	sbr	r16, 1 << 2		;Grafikausgabe im nchsten Zyklus
	sts	xflag3, r16		;Flags wieder speichern
;
	lds	r16, grptem
	lds	r17, grptem + 1
	lds	r18, grptem + 2		;Adresse aktueller Datensatz
	sts	grpsta, r16
	sts	grpsta + 1, r17
	sts	grpsta + 2, r18		;als Anfangsadresse speichern
	rjmp	m20000			;nchster Abschnitt
;
m1c210:	bst	r16, 3			;vorherige Seite anzeigen?
	brts	m1c220			;ja - bearbeiten
	rjmp	m1c500			;sonst nchste Seite
;
; Anzeigemodus Grafik: Einlesen der Daten rckwrts
;
m1c220:	lds	r16, xflag3		;verschiedene Flags 3 holen
	bst	r16, 5			;erster Datensatz erreicht?
	brts	m1c200			;ja -> Daten einlesen beenden
;
	lds	r20, grptem
	lds	r21, grptem+1		;Zeiger auf die aktuelle Posi-
	lds	r22, grptem+2		;tion im Externen EEPROM holen
	ldi	r23, xes1da		;Datensatzlnge 15min-Werte
	lds	r16, grmod1		;Anzeigemodus 1 holen
	tst	r16			;15-Minuten-Werte anzeigen?
	breq	m1c230			;ja -> weiter
	ldi	r23, xes2da		;sonst Datensatzlnge 24h-Werte
m1c230:	call	xdread			;Datensatz aus EEPROM lesen
	brcc	m1c240			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m1c200			;Daten einlesen beenden
;
m1c240:	lds	r16, xedbuf		;Zeitbyte 0 holen
	lds	r17, xedbuf + 1		;Zeitbyte 1 holen
	lds	r18, xedbuf + 2		;Zeitbyte 2 holen
	lds	r19, xedbuf + 3		;Zeitbyte 3 holen
	mov	r20, r16		;Zeitbyte 0 kopieren
	and	r20, r17		;mit Zeitbyte 1 verknpfen
	and	r20, r18		;mit Zeitbyte 2 verknpfen
	and	r20, r19		;mit Zeitbyte 3 verknpfen
	cpi	r20, 0xff		;gltiger Datenssatz?
	breq	m1c200			;nein -> Ende
;
	lds	r20, grpoin		;Grafikpuffer-Zeiger holen
	cpi	r20, 221		;Zeiger auf letzten Platz?
	brne	m1c250			;nein -> weiter
	sts	grcdat, r16
	sts	grcdat + 1, r17
	sts	grcdat + 2, r18		;sonst Zeitdaten in Zwischen-
	sts	grcdat + 3, r19		;speicher legen
	rjmp	m1c310
;
m1c250:	lds	r20, grcdat
	lds	r21, grcdat + 1
	lds	r22, grcdat + 2		;Zeitdaten aus Zwischenspeicher
	lds	r23, grcdat + 3		;holen
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22
	sbc	r19, r23		;stimmt Zeit berein?
	breq	m1c310			;ja -> Daten in Grafikpuffer
	brcc	m1c300			;Zeit grer? ja -> weiter
;
	ldi	r16, 0			;sonst
	ldi	r17, 0x80		;Sensorwert ungltig setzen
	call	grbts1			;Zeit/Sensorwert 1 in Puffer
	ldi	r16, 0
	ldi	r17, 0x80		;Sensorwert ungltig setzen
	call	grbts2			;Sensorwert 2 in Puffer
	call	grprev			;vorherigen Zeitpunkt ermitteln
	lds	r16, grpoin		;Datenpuffer-Zeiger holen
	cpi	r16, 0			;Pufferanfang erreicht?
	brne	m1c290			;nein -> weiter
	rjmp	m1c200			;sonst Daten einlesen beenden
;
m1c290:	dec	r16			;Zeiger auf vorherigen Platz
	sts	grpoin, r16		;Datenpuffer-Zeiger speichern
	rjmp	m1c240			;Datensatz erneut prfen
;
m1c300:	call	grdeca			;Adresse vorheriger Datensatz
	rjmp	m1c220			;nchsten Datensatz einlesen
;
m1c310:	lds	r16, grse11		;Grafik 1 Sensornummer holen
	lds	r17, grmod1		;Anzeigemodus 1 holen
	call	grgsen			;Sensorwert aus Puffer holen
	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 2			;Differenz-Modus aktiv?
	brtc	m1c340			;nein -> Wert speichern
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c340			;ja -> Wert speichern
;
	movw	r20, r16		;Sensorwert kopieren
	lds	r16, grse12		;Grafik 1 Differenz-Sensornummer holen
	lds	r17, grmod1		;Anzeigemodus 1 holen
	call	grgsen			;Sensorwert aus Puffer holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c340			;ja -> Wert speichern
	sub	r20, r16		;sonst Werte subtrahieren
	sbc	r21, r17		;bertrag
	movw	r16, r20		;Ergebnis kopieren
m1c340:	call	grbts1			;Zeit/Sensorwert 1 in Puffer
;
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppeldiagramm?
	brtc	m1c380			;nein -> nur einen Wert lesen
	lds	r16, grse21		;Grafik 2 Sensornummer holen
	lds	r17, grmod2		;Anzeigemodus 2 holen
	call	grgsen			;Sensorwert aus Puffer holen
	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 3			;Differenz-Modus aktiv?
	brtc	m1c370			;nein -> Wert speichern
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c370			;ja -> Wert speichern
;
	movw	r20, r16		;Sensorwert kopieren
	lds	r16, grse22		;Grafik 2 Differenz-Sensornummer holen
	lds	r17, grmod2		;Anzeigemodus 2 holen
	call	grgsen			;Sensorwert aus Puffer holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c370			;ja -> Wert speichern
	sub	r20, r16		;sonst Werte subtrahieren
	sbc	r21, r17		;bertrag
	movw	r16, r20		;Ergebnis kopieren
m1c370:	call	grbts2			;Sensorwert 2 in Puffer
;
m1c380:	call	grprev			;vorherigen Zeitpunkt ermitteln
	lds	r16, grpoin		;Datenpuffer-Zeiger holen
	cpi	r16, 0			;Pufferanfang erreicht?
	brne	m1c390			;nein -> weiter
	rjmp	m1c200			;sonst Daten einlesen beenden
;
m1c390:	dec	r16			;Zeiger auf vorherigen Platz
	sts	grpoin, r16		;Datenpuffer-Zeiger speichern
	call	grdeca			;Adresse vorheriger Datensatz
	rjmp	m1c220			;sonst weiter Daten einlesen
;
; Anzeigemodus Grafik: Einlesen der Daten vorwrts
;
m1c500:	lds	r16, xflag3		;verschiedene Flags 3 holen
	bst	r16, 4			;letzter Datensatz erreicht?
	brtc	m1c520			;nein -> Daten einlesen
;
m1c510:	lds	r16, xflag3		;sonst verschiedene Flags holen
	cbr	r16, 1 << 1		;Daten einlesen beenden und im
	sbr	r16, 1 << 2		;Grafikausgabe im nchsten Zyklus
	sts	xflag3, r16		;Flags wieder speichern
;
	lds	r16, grptem
	lds	r17, grptem + 1
	lds	r18, grptem + 2		;Datensatz-Adresse holen
	sts	grpend, r16
	sts	grpend + 1, r17
	sts	grpend + 2, r18		;als Endadresse speichern
	rjmp	m20000			;nchster Abschnitt
;
m1c520:	lds	r20, grptem
	lds	r21, grptem + 1		;Zeiger auf die aktuelle Position
	lds	r22, grptem + 2		;im Externen EEPROM holen
	ldi	r23, xes1da             ;Datensatzlnge 15min-Werte
	lds	r16, grmod1		;Anzeigemodus 1 holen
	tst	r16			;15-Minuten-Werte anzeigen?
	breq	m1c530			;ja -> weiter
	ldi	r23, xes2da		;sonst Datensatzlnge 24h-Werte
m1c530:	call	xdread			;Datensatz aus EEPROM lesen
	brcc	m1c540			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m1c510			;Daten einlesen beenden
;
m1c540:	lds	r16, xedbuf		;Zeitbyte 0 holen
	lds	r17, xedbuf + 1		;Zeitbyte 1 holen
	lds	r18, xedbuf + 2		;Zeitbyte 2 holen
	lds	r19, xedbuf + 3		;Zeitbyte 3 holen
	mov	r20, r16		;Zeitbyte 0 kopieren
	and	r20, r17		;mit Zeitbyte 1 verknpfen
	and	r20, r18		;mit Zeitbyte 2 verknpfen
	and	r20, r19		;mit Zeitbyte 3 verknpfen
	cpi	r20, 0xff		;gltiger Datenssatz?
	breq	m1c510			;nein -> Ende
;
	lds	r20, grpoin		;Grafikpuffer-Zeiger holen
	cpi	r20, 0			;Zeiger auf ersten Platz?
	brne	m1c550			;nein -> weiter
	sts	grcdat, r16
	sts	grcdat + 1, r17
	sts	grcdat + 2, r18		;sonst Zeitdaten in Zwischen-
	sts	grcdat + 3, r19		;speichern legen
	rjmp	m1c600
;
m1c550:	lds	r20, grcdat
	lds	r21, grcdat + 1
	lds	r22, grcdat + 2		;Zeitdaten aus Zwischenspeicher
	lds	r23, grcdat + 3		;holen
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22
	sbc	r19, r23		;stimmt Zeit berein?
	breq	m1c600			;ja -> Daten in Grafikpuffer
	brcc	m1c570			;Zeit grer? ja -> weiter
	call	grinca			;sonst Adresse nchster Datensatz
	rjmp	m1c500			;nchsten Datensatz einlesen
;
m1c570:	ldi	r16, 0
	ldi	r17, 0x80		;Sensorwert ungltig setzen
	call	grbts1			;Zeit/Sensorwert 1 in Puffer
	ldi	r16, 0
	ldi	r17, 0x80		;Sensorwert ungltig setzen
	call	grbts2			;Sensorwert 2 in Puffer
	call	grnext			;nchsten Zeitpunkt ermitteln
	lds	r16, grpoin		;Datenpuffer-Zeiger holen
	cpi	r16, 221		;Pufferende erreicht?
	brne	m1c580			;nein -> weiter
	rjmp	m1c510			;sonst Daten einlesen beenden
;
m1c580:	inc	r16			;Datenpufferzeiger erhhen
	sts	grpoin, r16		;und wieder speichern
	rjmp	m1c540			;Datensatz erneut prfen
;
m1c600:	lds	r16, grse11		;Grafik 1 Sensornummer holen
	lds	r17, grmod1		;Anzeigemodus 1 holen
	call	grgsen			;Sensorwert aus Puffer holen
	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 2			;Differenz-Modus aktiv?
	brtc	m1c630			;nein -> Wert speichern
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c630			;ja -> Wert speichern
;
	movw	r20, r16		;Sensorwert kopieren
	lds	r16, grse12		;Grafik 1 Differenz-Sensornummer holen
	lds	r17, grmod1		;Anzeigemodus 1 holen
	call	grgsen			;Sensorwert aus Puffer holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c630			;ja -> Wert speichern
	sub	r20, r16		;sonst Werte subtrahieren
	sbc	r21, r17		;bertrag
	movw	r16, r20		;Ergebnis kopieren
m1c630:	call	grbts1			;Zeit/Sensorwert 1 in Puffer
;
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppeldiagramm?
	brtc	m1c670			;nein -> nur einen Wert lesen
	lds	r16, grse21		;Grafik 2 Sensornummer holen
	lds	r17, grmod2		;Anzeigemodus 2 holen
	call	grgsen			;Sensorwert aus Puffer holen
	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 3			;Differenz-Modus aktiv?
	brtc	m1c660			;nein -> Wert speichern
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c660			;ja -> Wert speichern
;
	movw	r20, r16		;Temperaturwert kopieren
	lds	r16, grse22		;Grafik 2 Differenz-Sensornummer holen
	lds	r17, grmod2		;Anzeigemodus 2 holen
	call	grgsen			;Sensorwert aus Puffer holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1c660			;ja -> Wert speichern
	sub	r20, r16		;sonst Werte subtrahieren
	sbc	r21, r17		;bertrag
	movw	r16, r20		;Ergebnis kopieren
m1c660:	call	grbts2			;Sensorwert 2 in Puffer
;
m1c670:	call	grnext			;nchsten Zeitpunkt ermitteln
	lds	r16, grpoin		;Datenpuffer-Zeiger holen
	cpi	r16, 221		;Pufferende erreicht?
	brne	m1c680			;nein -> weiter
	rjmp	m1c510			;sonst Daten einlesen beenden
;
m1c680:	inc	r16			;Datenpufferzeiger erhhen
	sts	grpoin, r16		;und wieder speichern
	call	grinca			;Adresse nchster Datensatz
	rjmp	m1c500			;nchsten Datensatz lesen
;
; Anzeigemodus Grafik: Daten auf LCD ausgeben
;
m1d000:	bst	r16, 2			;Daten auf LCD ausgeben?
	brts	m1d010			;ja -> Daten ausgeben
	rjmp	m20000			;sonst zum nchsten Abschnitt
;
; Anzeigemodus Grafik: Minimal- und Maximalwerte im Puffer ermitteln
;
m1d010:	ldi	zl, low(grbuff)
	ldi	zh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	ldi	r20, 0			;Anfangswert Pufferzhler
	ldi	r16, low(-1000)
	ldi	r17, high(-1000)	;Anfangswert Maximum (-100,0)
	movw	r8, r16			;Maximalwert fr Sensornummer 1
	movw	r12, r16		;Maximalwert fr Sensornummer 2
	ldi	r16, low(2000)
	ldi	r17, high(2000)		;Anfangswert Minimum (200,0)
	movw	r10, r16		;Minimalwert fr Sensornummer 1
	movw	r14, r16		;Minimalwert fr Sensornummer 2
;
m1d020:	adiw	zl, 4			;Datum/Zeit berspringen
	ld	r16, z+			;Sensorwert 1 L holen
	ld	r17, z+			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1d040			;ja -> nicht auswerten
	movw	r18, r8			;Maximalwert kopieren
	sub	r18, r16
	sbc	r19, r17		;Neuer Wert > Maximalwert?
	brpl	m1d030			;nein -> weiter
	movw	r8, r16			;sonst neuen Maximalwert setzen
m1d030:	movw	r18, r16		;aktuellen Wert kopieren
	sub	r18, r10
	sbc	r19, r11		;Neuer Wert < Minimalwert?
	brpl	m1d040			;nein -> weiter
	movw	r10, r16		;sonst neuen Minimalwert setzen
;
m1d040:	ld	r16, z+			;Sensorwert 2 L holen
	ld	r17, z+			;Sensorwert 2 H holen
	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 4			;Doppeldiagramm?
	brtc	m1d060			;nein -> Sensorwert 2 ignorieren
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1d060			;ja -> nicht auswerten
	movw	r18, r12		;Maximalwert kopieren
	sub	r18, r16
	sbc	r19, r17		;Neuer Wert > Maximalwert?
	brpl	m1d050			;nein -> weiter
	movw	r12, r16		;sonst neuen Maximalwert setzen
m1d050:	movw	r18, r16		;aktuellen Wert kopieren
	sub	r18, r14
	sbc	r19, r15		;Neuer Wert < Minimalwert?
	brpl	m1d060			;nein -> weiter
	movw	r14, r16		;sonst neuen Minimalwert setzen
;
m1d060:	inc	r20			;Pufferposition erhhen
	cpi	r20, 222		;letzter Y-Wert?
	brne	m1d020			;nein -> Schleife
;
; Anzeigemodus Grafik: Skalierungen und Y-Mittelpunktwerte berechnen
;
	movw	r16, r8			;Maxwert Sensornummer 1 holen
	cpi	r17, high(-1000)	;wurde Anfangswert H verndert?
	brne	m1d200			;ja -> weiter
	cpi	r16, low(-1000)		;wurde Anfangswert L verndert?
	brne	m1d200			;ja -> weiter
	rjmp	m1d260			;sonst Berechnung berspringen
;
m1d200:	sub	r8, r10			;Differenz
	sbc	r9, r11			;Maximalwert - Minimalwert
	movw	r16, r8			;Differenz kopieren
	asr	r17
	ror	r16			;Differenz halbieren
	brcc	m1d210			;Runden? nein -> weiter
	inc	r16			;sonst aufrunden
	brne	m1d210			;bertrag? nein -> weiter
	inc	r17			;sonst bertrag addieren
m1d210:	add	r16, r10
	adc	r17, r11		;zum Minimalwert addieren
	sts	grcen1, r16
	sts	grcen1 + 1, r17		;Y-Mittelpunktwert 1 speichern
;
	ldi	r16, 86			;Y-Bereich fr Einzeldiagramm
	ldi	r17, 0			;86 Pixel
	lds	r20,grflag		;Grafik-Flags holen
	bst	r20, 4			;Doppeldiagramm?
	brtc	m1d220			;nein -> Y-Bereich ok
	ldi	r16, 41			;sonst Y-Bereich nur 41 Pixel
m1d220:	ldi	r20, 1			;Skalierungsfaktor = 1 setzen
m1d230:	movw	r18, r16		;Bereichswert kopieren
	sub	r18, r8
	sbc	r19, r9			;Bereich berschritten?
	brcc	m1d250			;nein -> weiter
	asr	r9			;sonst
	ror	r8			;Differenz halbieren
	brcc	m1d240			;Runden? nein -> weiter
	inc	r8			;sonst aufrunden
	brne	m1d240			;bertrag? nein -> weiter
	inc	r9			;sonst bertrag addieren
m1d240:	add	r20, r20		;Skalierungsfaktor verdoppeln
	rjmp	m1d230			;Schleife
m1d250:	sts	grsca1, r20		;Skalierungsfaktor 1 speichern
;
m1d260:	movw	r16, r12		;Maxwert Sensornummer 2 holen
	cpi	r17, high(-1000)	;wurde Anfangswert H verndert?
	brne	m1d270			;ja -> weiter
	cpi	r16, low(-1000)		;wurde Anfangswert L verndert?
	brne	m1d270			;ja -> weiter
	rjmp	m1d400			;sonst Berechnung berspringen
;
m1d270:	sub	r12, r14		;Differenz
	sbc	r13, r15		;Maximalwert - Minimalwert
	movw	r16, r12		;Differenz kopieren
	asr	r17
	ror	r16			;Differenz halbieren
	brcc	m1d280			;Runden? nein -> weiter
	inc	r16			;sonst aufrunden
	brne	m1d280			;bertrag? nein -> weiter
	inc	r17			;sonst bertrag addieren
m1d280:	add	r16, r14
	adc	r17, r15		;zum Minimalwert addieren
	sts	grcen2, r16
	sts	grcen2 + 1, r17		;Y-Mittelpunktwert 2 speichern
;
	ldi	r16, 41			;Y-Bereich fr Diagramm 2
	ldi	r17, 0			;41 Pixel
	ldi	r20, 1			;Skalierungsfaktor = 1 setzen
m1d290:	movw	r18, r16		;Bereichswert kopieren
	sub	r18, r12
	sbc	r19, r13		;Bereich berschritten?
	brcc	m1d310			;nein -> weiter
	asr	r13			;sonst
	ror	r12			;Differenz halbieren
	brcc	m1d300			;Runden? nein -> weiter
	inc	r12			;sonst aufrunden
	brne	m1d300			;bertrag? nein -> weiter
	inc	r13			;sonst bertrag addieren
m1d300:	add	r20, r20		;Skalierungsfaktor verdoppeln
	rjmp	m1d290			;Schleife
;
m1d310:	sts	grsca2, r20		;Skalierungsfaktor 2 speichern
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 1			;Skala angleichen?
	brtc	m1d400			;nein -> berspringen
;
	lds	r19, grsca1		;sonst Skalierungsfaktor 1 holen
	cp	r19, r20		;Skalierungsfaktoren vergleichen
	breq	m1d400			;Faktoren gleich? ja -> Ende
	brcc	m1d320			;Faktor 1 grer? ja -> weiter
	mov	r19, r20		;sonst Faktor 2 verwenden
	rjmp	m1d330
m1d320:	mov	r20, r19		;Faktor 1 verwenden
m1d330:	sts	grsca1, r19		;Skalierungsfaktor 1 speichern
	sts	grsca2, r20		;Skalierungsfaktor 2 speichern
;
; Anzeigemodus Grafik: Grafiken zeichnen, Y-Pixelposition berechnen, Y = Y-Mittelposition - ((Tempwert - Y-Mittelpunktwert) / Skalierung)
;
m1d400:	clr	r16
	sts	grlax1, r16		;letzte
	sts	grlay1, r16		;Pixelkoordinaten 1 lschen
	sts	grlax2, r16		;letzte
	sts	grlay2, r16		;Pixelkoordinaten 2 lschen
	ldi	zl, low(grbuff)
	ldi	zh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	ldi	r20, 0			;Anfangswert Pufferzhler
	ldi	r16, 59			;Y-Position 1 Mittelwert = 59
	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 4			;Doppeldiagramm?
	brtc	m1d410			;nein -> Y-Position ok
	ldi	r16, 37			;sonst Y-Position Mittelwert = 37
m1d410:	ldi	r17, 81			;Y-Position 2 Mittelwert = 81
	movw	r8, r16			;Y-Positionen kopieren
	lds	r10, grsca1		;Skalierungsfaktor 1 holen
	lds	r11, grsca2		;Skalierungsfaktor 2 holen
	lds	r12, grcen1
	lds	r13, grcen1 + 1		;Y-Mittelpunktwert 1 holen
	lds	r14, grcen2
	lds	r15, grcen2 + 1		;Y-Mittelpunktwert 2 holen
;
; Anzeigemodus Grafik: Pixel von Sensorwert 1 berechnen
;
m1d420:	adiw	zl, 4			;Datum/Zeit berspringen
	ld	r16, z+			;Sensorwert 1 L holen
	ld	r17, z+			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Wert ungltig?
	brne	m1d430			;nein -> bearbeiten
	rjmp	m1d550			;sonst keine Ausgabe
;
m1d430:	sub	r16, r12
	sbc	r17, r13		;Sensorwert - Mittelpunktwert
	mov	r18, r10		;Skalierungsfaktor 1 holen
m1d440:	lsr	r18			;Halbierung notwendig?
	breq	m1d450			;nein -> weiter
	asr	r17
	ror	r16			;sonst Anzeigewert halbieren
	brcc	m1d440			;Runden? nein -> Schleife
	inc	r16			;sonst aufrunden
	brne	m1d440			;bertrag? nein -> Schleife
	inc	r17			;sonst bertrag addieren
	rjmp	m1d440			;Schleife
;
m1d450:	mov	r19, r8			;Y-Position Mittelwert kopieren
	sub	r19, r16		;Y-Pixel-Position ermitteln
	sbr	r19, 0x80		;Bit7 setzen (Pixel setzen)
	ldi	r18, 16			;X Position Diagrammanfang
	add	r18, r20		;Pufferposition addieren
	call	pixlcd			;Pixel ausgeben
;
; Anzeigemodus Grafik: Pixel von Grafik 1 verbinden
;
	lds	r16, grlax1		;sonst letzte X-Koordinate holen
	lds	r17, grlay1		;letzte Y-Koordinate holen
	sts	grlax1, r18		;aktuelle X-Koordinate speichern
	sts	grlay1, r19		;aktuelle Y-Koordinate speichern
	bst	r17, 7			;Koordinate gltig?
	brtc	m1d550			;nein -> berspringen
	sub	r18, r16		;X-Differenz berechnen
	sub	r19, r17		;Y-Differenz berechnen
	mov	r21, r19		;Y-Differenz kopieren
	mov	r22, r19		;Y-Differenz nochmals kopieren
	brcc	m1d460			;Differenz positiv? ja -> weiter
	neg	r22			;in positiven Wert wandeln
m1d460:	mov	r23, r22		;positive Y-Differenz kopieren
	or	r23, r18		;mit X-Differenz verknpfen
	cpi	r23, 2			;Differenz < 2?
	brcs	m1d550			;ja -> keine Pixel zu verbinden
	mov	r23, r18		;X-Differenz kopieren
	movw	r18, r16		;Ursprungskoordinaten kopieren
	ldi	r24, 1			;Zykluszhler Startwert
	cp	r23, r22		;Y-Differenz > X-Differenz?
	brcs	m1d500			;ja -> bearbeiten
;
	mov	r25, r23		;X-Differenz kopieren
	lsr	r25			;halbieren -> Startwert Y-Zhler
m1d470:	inc	r18			;X-Koordinate erhhen
	sub	r25, r22		;Y-Zhler - Y-Differenz
	brcc	m1d490			;Y-Zhler negativ? nein -> weiter
	add	r25, r23		;sonst X-Differenz addieren
	bst	r21, 7			;Y-Differenz negativ?
	brts	m1d480			;ja -> Y-Koordinate vermindern
	inc	r19			;sonst Y-Koordinate erhhen
	rjmp	m1d490
m1d480:	dec	r19			;Y-Koordinate vermindern
m1d490:	call	pixlcd			;Pixel ausgeben
	inc	r24			;nchster Zyklus
	cp	r24, r23		;alle Pixel ausgegeben?
	brne	m1d470			;nein -> Schleife
	rjmp	m1d550
;
m1d500:	mov	r25, r22		;Y-Differenz kopieren
	lsr	r25			;halbieren -> Startwert X-Zhler
m1d510:	bst	r21, 7			;Y-Differenz negativ?
	brts	m1d520			;ja -> Y-Koordinate vermindern
	inc	r19			;sonst Y-Koordinate erhhen
	rjmp	m1d530
m1d520:	dec	r19			;Y-Koordinate vermindern
m1d530:	sub	r25, r23		;X-Zhler - X-Differenz
	brcc	m1d540			;Y-Zhler negativ? nein -> weiter
	add	r25, r22		;sonst Y-Differenz addieren
	inc	r18			;X-Koordinate erhhen
m1d540:	call	pixlcd			;Pixel ausgeben
	inc	r24			;nchster Zyklus
	cp	r24, r22		;alle Pixel ausgegeben?
	brne	m1d510			;nein -> Schleife
;
; Anzeigemodus Grafik: Pixel von Sensorwert 2 berechnen
;
m1d550:	ld	r16, z+			;Sensorwert 2 L holen
	ld	r17, z+			;Sensorwert 2 H holen
	cpi	r17, 0x80		;Wert ungltig?
	brne	m1d560			;nein -> bearbeiten
	rjmp	m1d680			;sonst keine Ausgabe
;
m1d560:	sub	r16, r14
	sbc	r17, r15		;Sensorwert - Mittelpunktwert
	mov	r18, r11		;Skalierungsfaktor 2 holen
m1d570:	lsr	r18			;Halbierung notwendig?
	breq	m1d580			;nein -> weiter
	asr	r17
	ror	r16			;sonst Anzeigewert halbieren
	brcc	m1d570			;Runden? nein -> Schleife
	inc	r16			;sonst aufrunden
	brne	m1d570			;bertrag? nein -> Schleife
	inc	r17			;sonst bertrag addieren
	rjmp	m1d570			;Schleife
;
m1d580:	mov	r19, r9			;Y-Position Mittelwert kopieren
	sub	r19, r16		;Y-Pixel-Position ermitteln
	sbr	r19, 0x80		;Bit7 setzen (Pixel setzen)
	ldi	r18, 16			;X Position Diagrammanfang
	add	r18, r20		;Pufferposition addieren
	call	pixlcd			;Pixel ausgeben
;
; Anzeigemodus Grafik: Pixel von Grafik 2 verbinden
;
	lds	r16, grlax2		;sonst letzte X-Koordinate holen
	lds	r17, grlay2		;letzte Y-Koordinate holen
	sts	grlax2, r18		;aktuelle X-Koordinate speichern
	sts	grlay2, r19		;aktuelle Y-Koordinate speichern
	bst	r17, 7			;Koordinate gltig?
	brtc	m1d680			;nein -> berspringen
	sub	r18, r16		;X-Differenz berechnen
	sub	r19, r17		;Y-Differenz berechnen
	mov	r21, r19		;Y-Differenz kopieren
	mov	r22, r19		;Y-Differenz nochmals kopieren
	brcc	m1d590			;Differenz positiv? ja -> weiter
	neg	r22			;in positiven Wert wandeln
m1d590:	mov	r23, r22		;positive Y-Differenz kopieren
	or	r23, r18		;mit X-Differenz verknpfen
	cpi	r23, 2			;Differenz < 2?
	brcs	m1d680			;ja -> keine Pixel zu verbinden
	mov	r23, r18		;X-Differenz kopieren
	movw	r18, r16		;Ursprungskoordinaten kopieren
	ldi	r24, 1			;Zykluszhler Startwert
	cp	r23, r22		;Y-Differenz > X-Differenz?
	brcs	m1d630			;ja -> bearbeiten
;
	mov	r25, r23		;X-Differenz kopieren
	lsr	r25			;halbieren -> Startwert Y-Zhler
m1d600:	inc	r18			;X-Koordinate erhhen
	sub	r25, r22		;Y-Zhler - Y-Differenz
	brcc	m1d620			;Y-Zhler negativ? nein -> weiter
	add	r25, r23		;sonst X-Differenz addieren
	bst	r21, 7			;Y-Differenz negativ?
	brts	m1d610			;ja -> Y-Koordinate vermindern
	inc	r19			;sonst Y-Koordinate erhhen
	rjmp	m1d620
m1d610:	dec	r19			;Y-Koordinate vermindern
m1d620:	call	pixlcd			;Pixel ausgeben
	inc	r24			;nchster Zyklus
	cp	r24, r23		;alle Pixel ausgegeben?
	brne	m1d600			;nein -> Schleife
	rjmp	m1d680
;
m1d630:	mov	r25, r22		;Y-Differenz kopieren
	lsr	r25			;halbieren -> Startwert X-Zhler
m1d640:	bst	r21, 7			;Y-Differenz negativ?
	brts	m1d650			;ja -> Y-Koordinate vermindern
	inc	r19			;sonst Y-Koordinate erhhen
	rjmp	m1d660
m1d650:	dec	r19			;Y-Koordinate vermindern
m1d660:	sub	r25, r23		;X-Zhler - X-Differenz
	brcc	m1d670			;Y-Zhler negativ? nein -> weiter
	add	r25, r22		;sonst Y-Differenz addieren
	inc	r18			;X-Koordinate erhhen
m1d670:	call	pixlcd			;Pixel ausgeben
	inc	r24			;nchster Zyklus
	cp	r24, r22		;alle Pixel ausgegeben?
	brne	m1d640			;nein -> Schleife
;
m1d680:	inc	r20			;Pufferposition erhhen
	cpi	r20, 222		;letzter Y-Wert?
	breq	m1e000			;ja -> Ende
	rjmp	m1d420			;sonst Schleife
;
; Anzeigemodus Grafik: Beschriftung der Y-Achse von Diagramm 1
;
m1e000:	ldi	r16, 4
	mov	txflag, r16		;Mini-Schrift einstellen
	lds	r16, grsca1		;Skalierungsfaktor 1 holen
	tst	r16			;gltiger Wert?
	brne	m1e010			;ja -> bearbeiten
	rjmp	m1e200			;sonst weiter zu Diagramm 2
;
m1e010:	ldi	r25, 102		;untere Y-Position = 102
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppeldiagramm?
	brtc	m1e020			;nein -> untere Position ok
	ldi	r25, 58			;sonst untere Y-Position = 58
;
m1e020:	mov	r16, r25		;untere Position kopieren
	sub	r16, r8			;Y-Position Mittelwert subtrahieren
	mul	r16, r10		;mit Skalierung 1 multiplizieren
	movw	r16, r12		;Y-Mittelpunktwert 1 holen
	sub	r16, r0			;Ergebnis vom Mittelpunktwert
	sbc	r17, r1			;subtrahieren = kleinster Wert
;
m1e030:	lds	r21, grsca1		;Skalierungsfaktor 1 holen
	call	grchek			;Wert passend zur Skalierung?
	breq	m1e040			;ja -> weiter
	inc	r16			;sonst Wert + 1
	brne	m1e030			;bertrag? nein -> weiter
	inc	r17			;sonst H-Byte erhhen
	rjmp	m1e030			;Schleife
;
m1e040:	sts	grysva, r16
	sts	grysva + 1, r17		;Y-Skalenwert speichern
	lds	r18, grcen1
	lds	r19, grcen1 + 1		;Mittelpunktwert holen
	sub	r16, r18
	sbc	r17, r19		;Sensorwert - Mittelpunktwert
	lds	r18, grsca1		;Skalierungsfaktor 1 holen
m1e050:	lsr	r18			;Halbierung notwendig?
	breq	m1e060			;nein -> weiter
	asr	r17
	ror	r16			;sonst Anzeigewert halbieren
	brcc	m1e050			;Runden? nein -> Schleife
	inc	r16			;sonst aufrunden
	brne	m1e050			;bertrag? nein -> Schleife
	inc	r17			;sonst bertrag addieren
	rjmp	m1e050			;Schleife
;
m1e060:	mov	r19, r8			;Y-Position Mittelwert kopieren
	sub	r19, r16		;Y-Pixel-Position ermitteln
	cpi	r19, 16			;Y-Bereich berschritten?
	brcs	m1e200			;ja -> Ende
	mov	r21, r19		;neue Y-Skalenposition kopieren
	sbr	r19, 0x80		;Bit7 setzen (Pixel setzen)
	ldi	r18, 14			;X Position Gradmarkierung
	call	pixlcd			;Pixel ausgeben
	subi	r21, 2			;Y-Position von Skalenwert
	cpi	r21, 16			;Y-Bereich berschritten?
	brcs	m1e200			;ja -> Ende
;
	mov	r20, r25		;untere Y-Position kopieren
	cpi	r20, 102		;groes Diagramm?
	breq	m1e070			;ja -> weiter
	dec	r20			;Korrektur fr oberes Diagramm
m1e070:	subi	r20, 2			;fr Skalentext korrigieren
	cp	r21, r20		;unterer Y-Bereich berschritten?
	brcc	m1e080			;ja -> keinen Skalentext ausgeben
;
	lds	r16, grysva		;sonst
	lds	r17, grysva + 1		;letzten Y-Skalenwert holen
	ldi	r18, low(-1000)
	ldi	r19, high(-1000)	;Vergleichswert -100 Grad laden
	sub	r18, r16
	sbc	r19, r17		;Wert noch darstellbar (> -100 Grad)?
	brpl	m1e080			;nein -> Ausgabe berspringen
;
	call	temasc			;in ASCII wandeln
	ldi	r20, 1			;X-Position
	mov	r22, r4			;Hunderter holen
	call	asclcd			;ausgeben
	mov	r22, r5			;Zehner holen
	call	asclcd			;ausgeben
	mov	r22, r6			;Einer holen
	call	asclcd			;ausgeben
;
m1e080:	lds	r16, grysva
	lds	r17, grysva + 1		;letzten Y-Skalenwert holen
	lds	r21, grsca1		;Skalierungsfaktor 1 holen
	call	grysca			;nchsten Skalenwert ermitteln
	rjmp	m1e040
;
; Anzeigemodus Grafik: Beschriftung der Y-Achse von Diagramm 2
;
m1e200:	lds	r16, grsca2		;Skalierungsfaktor 2 holen
	tst	r16			;gltiger Wert?
	brne	m1e210			;ja -> bearbeiten
	rjmp	m1e400			;sonst Ende
;
m1e210:	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppelgrafik?
	brts	m1e220			;ja -> bearbeiten
	rjmp	m1e400			;sonst Ende
;
m1e220:	ldi	r16, 102		;untere Y-Position = 102
	sub	r16, r9			;Y-Position Mittelwert subtrahieren
	mul	r16, r11		;mit Skalierung 2 multiplizieren
	lds	r16, grcen2
	lds	r17, grcen2 + 1		;Y-Mittelpunktwert 2 holen
	sub	r16, r0			;Ergebnis vom Mittelpunktwert
	sbc	r17, r1			;subtrahieren = kleinster Wert
;
m1e230:	lds	r21, grsca2		;Skalierungsfaktor 2 holen
	call	grchek			;Wert passend zur Skalierung?
	breq	m1e240			;ja -> weiter
	inc	r16			;sonst Wert + 1
	brne	m1e230			;bertrag? nein -> weiter
	inc	r17			;sonst H-Byte erhhen
	rjmp	m1e230			;Schleife
;
m1e240:	sts	grysva, r16
	sts	grysva + 1, r17		;Y-Skalenwert speichern
	lds	r18, grcen2
	lds	r19, grcen2 + 1		;Mittelpunktwert holen
	sub	r16, r18
	sbc	r17, r19		;Sensorwert - Mittelpunktwert
	lds	r18, grsca2		;Skalierungsfaktor 2 holen
m1e250:	lsr	r18			;Halbierung notwendig?
	breq	m1e260			;nein -> weiter
	asr	r17
	ror	r16			;sonst Anzeigewert halbieren
	brcc	m1e250			;Runden? nein -> Schleife
	inc	r16			;sonst aufrunden
	brne	m1e250			;bertrag? nein -> Schleife
	inc	r17			;sonst bertrag addieren
	rjmp	m1e250			;Schleife
;
m1e260:	mov	r19, r9			;Y-Position Mittelwert kopieren
	sub	r19, r16		;Y-Pixel-Position ermitteln
	cpi	r19, 60			;Y-Bereich berschritten?
	brcs	m1e400			;ja -> Ende
	mov	r21, r19		;neue Y-Skalenposition kopieren
	sbr	r19, 0x80		;Bit7 setzen (Pixel setzen)
	ldi	r18, 14			;X-Position Gradmarkierung
	call	pixlcd			;Pixel ausgeben
	subi	r21, 2			;Y-Position von Skalenwert
	cpi	r21, 60			;Y-Bereich berschritten?
	brcs	m1e400			;ja -> Ende
;
	cpi	r21, 100		;unterer Y-Bereich berschritten?
	brcc	m1e280			;ja -> keinen Skalentext ausgeben
	lds	r16, grysva		;sonst
	lds	r17, grysva + 1		;letzten Y-Skalenwert holen
	ldi	r18, low(-1000)
	ldi	r19, high(-1000)	;Vergleichswert -100 Grad laden
	sub	r18, r16
	sbc	r19, r17		;Wert noch darstellbar (> -100 Grad)?
	brpl	m1e280			;nein -> Ausgabe berspringen

	call	temasc			;in ASCII wandeln
	ldi	r20, 1			;X-Position
	mov	r22, r4			;Hunderter holen
	call	asclcd			;ausgeben
	mov	r22, r5			;Zehner holen
	call	asclcd			;ausgeben
	mov	r22, r6			;Einer holen
	call	asclcd			;ausgeben
;
m1e280:	lds	r16, grysva
	lds	r17, grysva + 1		;letzten Y-Skalenwert holen
	lds	r21, grsca2		;Skalierungsfaktor 2 holen
	call	grysca			;nchsten Skalenwert ermitteln
	rjmp	m1e240
;
; Anzeigemodus Grafik: Beschriftung der X-Achse, 15min-Werte
;
m1e400:	lds	r16, grsca1		;Skalierungsfaktor 1 holen
	lds	r17, grsca2		;Skalierungsfaktor 2 holen
	or	r16, r17		;mindestens 1 Wert gltig?
	brne	m1e405			;ja -> weiter
	rjmp	m1e800			;sonst Ende
;
m1e405:	ldi	yl, low(grbuff)
	ldi	yh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	ldi	r25,0			;Anfangswert Pufferzhler
	lds	r16, grmod1		;Anzeigemodus Sensornummer 1
	tst	r16			;15-Minuten-Werte anzeigen?
	breq	m1e410			;ja -> bearbeiten
	rjmp	m1e600			;sonst 24h-Werte bearbeiten
;
m1e410:	ld	r8, y+
	ld	r9, y+
	ld	r10, y+
	ld	r11, y+			;Zeitdaten holen
	mov	r16, r8			;Minutenwert kopieren
	or	r16, r9			;mit Stundenwert verknpfen
	brne	m1e420			;Tageswechsel? nein -> weiter
;
	ldi	r18, 16			;X-Anfangsposition
	add	r18, r25		;Pufferposition addieren
	ldi	r19, 104 | 0x80		;Y-Position Skalenmarkierung
	call	pixlcd			;Skalenmarkierung setzen
	mov	r20, r18		;X-Position holen
	subi	r20, 9			;Position fr Datumtext
	cpi	r20, 221		;X-Endposition erreicht?
	brcc	m1e450			;ja -> Datumausgabe berspringen
	ldi	r21, 106		;sonst Y-Position fr Datumtext
	call	grelcd			;Datum ausgeben
	rjmp	m1e450
;
m1e420:	mov	r16, r8			;Minutenwert kopieren
	tst	r16			;Minutenwert = 0?
	brne	m1e450			;nein -> keine Zeit ausgeben
	mov	r16, r9			;sonst Stundenwert kopieren
	andi	r16, 0x03		;Stundenwert / 4 teilbar?
	brne	m1e450			;nein -> keine Zeit ausgeben
;
	ldi	r18, 16			;X-Anfangsposition
	add	r18, r25		;Pufferposition addieren
	ldi	r19, 104 | 0x80		;Y-Position Skalenmarkierung
	call	pixlcd			;Skalenmarkierung setzen
	mov	r20, r18		;X-Position holen
	subi	r20, 2			;X-Position fr Zeittext-Einer
	ldi	r21, 106		;Y-Position fr Zeittext
;
	mov	r18, r9			;Stundenwert holen
	call	bytasc			;in ASCII wandeln
	cpi	r17, '0'		;Zehner = 0?
	brne	m1e430			;nein -> weiter
	cpi	r20, 235		;X-Endposition erreicht?
	brcc	m1e450			;ja -> Zeitausgabe berspringen
	mov	r22, r16		;sonst Einer holen
	rjmp	m1e440			;ausgeben
m1e430:	subi	r20, 2			;X-Position fr Zeittext-Zehner
	cpi	r20, 231		;X-Endposition erreicht?
	brcc	m1e450			;ja -> Zeitausgabe berspringen
	mov	r13, r16		;Einer sichern
	mov	r22, r17		;Zehner holen
	call	asclcd			;ausgeben
	mov	r22, r13		;gesicherte Einer holen
m1e440:	call	asclcd			;ausgeben
;
m1e450:	adiw	yl, 4			;Datenbytes berspringen
	inc	r25			;Pufferzhler erhhen
	cpi	r25, 222		;letzte Position erreicht?
	breq	m1e800			;ja -> Ende
	rjmp	m1e410			;sonst Schleife
;
; Anzeigemodus Grafik: Beschriftung der X-Achse, 24h-Werte
;
m1e600:	adiw	yl, 2			;Stunden/Minuten berspringen
	ld	r10, y+
	ld	r11, y+			;Zeitdaten Byte 2 + 3 holen
	mov	r16, r10		;Zeitbyte 2 kopieren
	andi	r16, 0x1f		;Tageswert filtern
	cpi	r16, 1			;Tag 1 (Monatsanfang)?
	brne	m1e610			;nein -> weiter
;
	ldi	r18, 16			;sonst X-Anfangsposition
	add	r18, r25		;Pufferposition addieren
	ldi	r19, 104 | 0x80		;Y-Position Skalenmarkierung
	call	pixlcd			;Skalenmarkierung setzen
	mov	r20, r18		;X-Position holen
	subi	r20, 9			;Position fr Datumtext
	cpi	r20, 221		;X-Endposition erreicht?
	brcc	m1e620			;ja -> Datumausgabe berspringen
	ldi	r21, 106		;sonst Y-Position fr Datumtext
	call	grelcd			;Datum ausgeben
	rjmp	m1e620
;
m1e610:	cpi	r16, 15			;Tag 15 (Monatsmitte)?
	brne	m1e620			;nein -> Ende
	ldi	r18, 16			;X-Anfangsposition
	add	r18, r25		;Pufferposition addieren
	ldi	r19, 104 | 0x80		;Y-Position Skalenmarkierung
	call	pixlcd			;Skalenmarkierung setzen
;
m1e620:	adiw	yl, 4			;Datenbytes berspringen
	inc	r25			;Pufferzhler erhhen
	cpi	r25, 222		;letzte Position erreicht?
	brne	m1e600			;nein -> Schleife
;
; Anzeigemodus Grafik: Beschriftung der Diagramme mit Sensornamen; es wird ermittelt, ob die ersten 60 Grafikpuffer-Werte schwerpunktmig ber oder unter dem
; Mittelpunktwert liegen; die Ausgabe der Texte erfolgt davon abhngig oben oder unten
;
m1e800:	clr	txflag			;Normalschrift einstellen
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 0			;Sensornamen anzeigen?
	brts	m1e810			;ja -> bearbeiten
	rjmp	m1ea80			;sonst Ende
;
m1e810:	ldi	zl, low(grbuff)
	ldi	zh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	lds	r8, grcen1
	lds	r9, grcen1 + 1		;Y-Mittelpunktwert 1 holen
	lds	r10, grcen2
	lds	r11, grcen2 + 1		;Y-Mittelpunktwert 2 holen
	clr	r12
	clr	r13			;Mittelwert 1 lschen
	clr	r14
	clr	r15			;Mittelwert 2 lschen
	ldi	r20, 0			;Anfangswert Pufferzhler
;
m1e820:	adiw	zl, 4			;Datum/Zeit berspringen
	ld	r16, z+			;Sensorwert 1 L holen
	ld	r17, z+			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1e830			;ja -> nicht auswerten
	sub	r16, r8
	sbc	r17, r9			;Mittelpunktwert 1 subtrahieren
	add	r12, r16
	adc	r13, r17		;zum Mittelwert 1 addieren
;
m1e830:	ld	r16, z+			;Sensorwert 2 L holen
	ld	r17, z+			;Sensorwert 2 H holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m1e840			;ja -> nicht auswerten
	sub	r16, r10
	sbc	r17, r11		;Mittelpunktwert 2 subtrahieren
	add	r14, r16
	adc	r15, r17		;zum Mittelwert 2 addieren
;
m1e840:	inc	r20			;Pufferposition erhhen
	cpi	r20, 60			;letzter relevanter Y-Wert?
	brne	m1e820			;nein -> Schleife
;
	lds	r16, grflag		;Grafik-Flags holen
        bst     r16, 5                  ;Schriftgre "klein"?
        brtc    m1e850                  ;nein -> weiter
        ldi     r17, 1 << 2             ;sonst auf Minischrift
        mov     txflag, r17             ;setzen
;
m1e850:	mov	r25, r15		;Mittelwert 2 H-Byte sichern
	ldi	r20, 17			;X-Position fr Sensorname 1
	bst	r13, 7			;Mittelwert 1 negativ?
	brtc	m1e860			;nein -> Sensorname nach unten
	ldi	r21, 16			;sonst obere Y-Position = 16
	rjmp	m1e880			;Sensorname ausgeben
;
m1e860:	ldi	r21, 91			;untere Y-Position = 91
        bst     r16, 5                  ;Schriftgre "klein"?
        brtc    m1e870                  ;nein -> weiter
        ldi     r21, 97                 ;sonst untere Y-Position = 97
;
m1e870:	bst	r16, 4			;Doppeldiagramm?
	brtc	m1e880			;nein -> Y-Position ok
	ldi	r21, 47			;sonst untere Y-Position = 47
        bst     r16, 5                  ;Schriftgre "klein"?
        brtc    m1e880                  ;nein -> Y-Position ok
        ldi     r21, 53                 ;sonst untere Y-Position = 53
;
m1e880: andi    r16, 1 << 4 | 1 << 1    ;Flag fr Doppeldiagramm und Skala angleichen filtern
        cpi     r16, 1 << 4 | 1 << 1    ;sind beide Flags gesetzt?
        brne    m1e890                  ;nein -> berspringen
        ldi     r22, 0x93               ;sonst Symbol fr "Skala angleichen"
        call    asclcd                  ;ausgeben
        ldi     r22, 0xa0               ;schmales Leerzeichen
        call    asclcd                  ;ausgeben
;
m1e890:	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 2			;Differenz-Modus aktiv?
	brtc	m1e900			;nein -> weiter
	ldi	r22, 0x97		;sonst Delta-Zeichen laden
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
;
m1e900:	lds	r18, grse11		;Grafik 1 Sensornummer holen
        cpi     r18, 31                 ;Batteriewert anzeigen?
        brcs    m1e910                  ;nein -> EEPROM-Text ausgeben
        ldi     r22, 0x37               ;Text "Batterie"
        call    strlcd                  ;ausgeben
        dec     r20                     ;Endezeichen wieder entfernen
        rjmp    m1e920
;
m1e910:	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
;
m1e920:	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 2			;Differenz-Modus aktiv?
	brtc	m1e930			;nein -> weiter
	ldi	r22, '/'		;sonst Schrgstrich laden
	call	asclcd			;ausgeben
	lds	r18, grse12		;Grafik 1 Differenz-Sensornummer holen
	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
;
m1e930:	lds	r16, grmod1		;Anzeigemodus Sensornummer 1
	tst	r16			;15-Minuten-Werte anzeigen?
	breq	m1e940			;ja -> weiter
	ldi	r22, 0xa0		;sonst schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r16, grmod1		;Anzeigemodus Sensornummer 1
	ldi	r22, 0x98 - 1		;Zeichen fr Minimalwert laden
	add	r22, r16		;Anzeigemodus addieren; Min,
	call	asclcd			;Max oder Avg-Zeichen ausgeben
;
m1e940: ldi     r22, 0xa0               ;schmales Leerzeichen
        call    asclcd                  ;ausgeben
        lds     r16, grse11             ;Grafik 1 Sensornummer holen
        ldi     r22, 0x49               ;Text "(C)" vorbereiten
        cpi     r16, 27                 ;Temperatursensor?
        brcc    m1e950                  ;nein -> weiter
        lds     r17, grflag             ;sonst Grafik-Flags holen
        sbrc    r17, 2                  ;Grafik 1 Differenz-Modus?
        ldi     r22, 0x4d               ;ja -> Text "(K)" verwenden
        rjmp    m1e960
;
m1e950: ldi     r22, 0x4a               ;Text "(kPa)" vorbereiten
        cpi     r16, 28                 ;Luftdrucksensor?
        brcs    m1e960                  ;ja -> ausgeben
        ldi     r22, 0x4b               ;Text "(%)" vorbereiten
        cpi     r16, 31                 ;Luftfeuchtigkeitssensor
        brcs    m1e960                  ;ja -> ausgeben
        ldi     r22, 0x4c               ;sonst Text "(100mV)"
m1e960: call    strlcd                  ;ausgeben
;
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppelgrafik?
        brts    m1e970                  ;ja -> bearbeiten
	rjmp	m1ea80			;sonst berspringen
;
m1e970:	ldi	r20, 17			;X-Position fr Sensorname 2
	ldi	r21, 91			;untere Y-Position = 91
        bst     r16, 5                  ;Schriftgre "klein"?
        brtc    m1e980                  ;nein -> Y-Position ok
        ldi     r21, 97                 ;sonst untere Y-Position = 97
;
m1e980:	bst	r25, 7			;Mittelwert 2 negativ?
	brtc	m1e990			;nein -> Sensorname nach unten
	ldi	r21, 61			;sonst obere Y-Position = 61
;
m1e990: andi    r16, 1 << 4 | 1 << 1    ;Flag fr Doppeldiagramm und Skala angleichen filtern
        cpi     r16, 1 << 4 | 1 << 1    ;sind beide Flags gesetzt?
        brne    m1ea00                  ;nein -> berspringen
        ldi     r22, 0x93               ;sonst Symbol fr "Skala angleichen"
        call    asclcd                  ;ausgeben
        ldi     r22, 0xa0               ;schmales Leerzeichen
        call    asclcd                  ;ausgeben
;
m1ea00:	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 3			;Differenz-Modus aktiv?
	brtc	m1ea10			;nein -> weiter
	ldi	r22, 0x97		;sonst Delta-Zeichen laden
	call	asclcd			;ausgeben
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
;
m1ea10:	lds	r18, grse21		;Grafik 2 Sensornummer holen
        cpi     r18, 31                 ;Batteriewert anzeigen?
        brcs    m1ea20                  ;nein -> EEPROM-Text ausgeben
        ldi     r22, 0x37               ;sonst Text "Batterie"
        call    strlcd                  ;ausgeben
        dec     r20                     ;Endezeichen wieder entfernen
        rjmp    m1ea30
;
m1ea20:	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
;
m1ea30:	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 3			;Differenz-Modus aktiv?
	brtc	m1ea40			;nein -> weiter
	ldi	r22, '/'		;sonst Schrgstrich laden
	call	asclcd			;ausgeben
	lds	r18, grse22		;Grafik 2 Differenz-Sensornummer holen
	ldi	r19, 9			;Sensornamenlnge im EEPROM
	mul	r18, r19		;Tabellenoffset ermitteln
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	call	eeplcd			;EEPROM-Text ausgeben
;
m1ea40:	lds	r16, grmod2		;Anzeigemodus Sensornummer 2
	tst	r16			;15-Minuten-Werte anzeigen?
	breq	m1ea50			;ja -> weiter
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r16, grmod2		;Anzeigemodus Sensornummer 2
	ldi	r22, 0x98 - 1		;Zeichen fr Minimalwert laden
	add	r22, r16		;Anzeigemodus addieren; Min,
	call	asclcd			;Max oder Avg-Zeichen ausgeben
;
m1ea50: ldi     r22, 0xa0               ;schmales Leerzeichen
        call    asclcd                  ;ausgeben
        lds     r16, grse21             ;Grafik 2 Sensornummer holen
        ldi     r22, 0x49               ;Text "(C)" vorbereiten
        cpi     r16, 27                 ;Temperatursensor?
        brcc    m1ea60                  ;nein -> weiter
        lds     r17, grflag             ;sonst Grafik-Flags holen
        sbrc    r17, 3                  ;Grafik 2 Differenz-Modus?
        ldi     r22, 0x4d               ;ja -> Text "(K)" verwenden
        rjmp    m1ea70
;
m1ea60: ldi     r22, 0x4a               ;Text "(kPa)" vorbereiten
        cpi     r16, 28                 ;Luftdrucksensor?
        brcs    m1ea70                  ;ja -> ausgeben
        ldi     r22, 0x4b               ;Text "(%)" vorbereiten
        cpi     r16, 31                 ;Luftfeuchtigkeitssensor
        brcs    m1ea70                  ;ja -> ausgeben
        ldi     r22, 0x4c               ;sonst Text "(100mV)"
m1ea70: call    strlcd                  ;ausgeben
;
m1ea80:	lds	r16, xflag3		;verschiedene Flags 3 holen
	cbr	r16, 1 << 2		;Grafikausgabe beendet
	sts	xflag3, r16		;Flags wieder speichern
        clr     txflag                  ;Textflags wieder lschen
	rjmp	m20000
;
; Anzeigemodus Grafik: Fadenkreuz neu ausgeben
;
m1f000:	cpi	r16, 24			;Anzeigemodus Grafik - Fadenkreuz?
	breq	m1f010			;ja -> weiter
	rjmp	m1g000			;sonst weiter testen
;
m1f010:	lds	r18, grpoin		;aktuelle Fadenkreuzposition holen
	lds	r19, encval		;aktuellen Impulsgeberwert holen
	cp	r18, r19		;Positionsnderung?
	breq	m1f100			;nein -> keine Ausgabe
	sts	grlacr, r19		;Fadenkreuzposition sichern
	sts	grpoin, r19		;neue Fadenkreuzposition setzen
	call	grscrr			;Grafik wieder herstellen
	lds	r18, grpoin		;aktuelle Fadenkreuzposition holen
	call	grscxp			;X-Koordinaten fr Fadenkreuz
	lds	r18, grpoin		;aktuelle Fadenkreuzposition holen
	call	grscrb			;Grafik sichern
	lds	r18, grpoin		;aktuelle Fadenkreuzposition holen
	call	grcros			;Fadenkreuz ausgeben
m1f100:	rjmp	m20000
;
; Anzeigemodus Konfiguration - Grafik-Presets: Anzeige komplett neu ausgeben
;
m1g000:	cpi	r16, 25			;Anzeigemodus Konfiguration - Grafik-Presets?
	breq	m1g010			;ja -> weiter
	rjmp	m1g500			;sonst weiter testen
;
m1g010:	clr	txflag			;Text-Flag lschen
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 1			;Anzeige neu ausgeben?
	brtc	m1g100			;nein -> weiter
	cbr	r16, 0x02		;sonst Neuausgabe-Flag lschen
	sts	dflags, r16		;und Flags wieder speichern
	call	newcfg			;alle Konfigzeilen neu ausgeben
;
; Anzeigemodus Konfiguration - Grafik-Presets: Namen ausgeben
;
m1g100:	ldi	r25, 0			;Zeilen-Zhler = 0
	ldi	yl, low(cfgflg)
	ldi	yh, high(cfgflg)	;Zeiger auf Konfigurationsflags
m1g110:	ldi	r20, 2			;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r25, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
	movw	r12, r20		;X/Y-Startpositionen sichern
;
	ld	r16, y			;Konfigurationflags holen
	bst	r16, 0			;Name neu ausgeben?
	brtc	m1g140			;nein -> berspringen
	cbr	r16, 0x01		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
;
	lds	r19, menpos		;ausgewhlte Position holen
	dec	r19			;korrigieren
	cp	r25, r19		;Position identisch?
	brne	m1g120			;nein -> weiter
	ldi	r16, 0x01		;sonst
	mov	txflag, r16		;Textflag invers setzen
m1g120:	ldi	r22, 0xa0		;Text "Preset"
	cpi	r25, 0			;Zhler auf Position 1?
	breq	m1g130			;ja -> Text ausgeben
	ldi	r22, 0x5c		;Text "Name"
	cpi	r25, 1			;Zhler auf Position 2?
	breq	m1g130			;ja -> Text ausgeben
	ldi	r22, 0xa2		;sonst Basiswert laden
	add	r22, r25		;Position addieren -> Text-Nummer
m1g130:	call	strlcd			;Text ausgeben
	clr	txflag
;
; Anzeigemodus Konfiguration - Grafik-Presets: Daten ausgeben
;
m1g140:	ld	r16, y			;Konfigurationflags holen
	bst	r16, 1			;Daten neu ausgeben?
	brts	m1g150			;ja -> weiter
	rjmp	m1g290			;sonst berspringen
;
m1g150:	cbr	r16, 0x02		;sonst nderungs-Flag lschen
	st	y, r16			;Flags wieder speichern
	movw	r20, r12		;X/Y-Koordinaten restaurieren
	ldi	r20, 128		;X-Position fr Daten
;
	cpi	r25, 0			;Zhler auf Preset-Nummer?
	brne	m1g160			;nein -> weiter testen
	lds	r18, grpres		;aktuelle Preset-Nummer holen
	inc	r18			;korrigieren
	call	bytlcd			;auf LCD ausgeben
	rjmp	m1g280			;Bereich hinter Text lschen
;
m1g160:	cpi	r25, 1			;Zhler auf Preset-Name?
	brne	m1g170			;nein -> weiter testen
	ldi	r16, low(egrprt)
	ldi	r17, high(egrprt)	;EEPROM-Adresse Preset-Namen
	lds	r18, grpres		;aktuelle Preset-Nummer holen
	ldi	r19, 16			;Offset/Lnge 16 Bytes
	mul	r18, r19		;Adressoffset berechnen
	add	r16, r0
	adc	r17, r1			;EEPROM-Adresse ermitteln
	call	eeplcd			;Preset-Name ausgeben
	rjmp	m1g280			;Bereich hinter Text lschen
;
m1g170:	cpi	r25, 2			;Zhler auf Grafik 1 Sensor?
	brne	m1g190			;nein -> weiter testen
;
        ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensortexte
        ldi     r16, 9                  ;Textabstand 9 Bytes
	lds	r17, grse11		;Grafik 1 Sensornummer holen
        mul     r16, r17                ;Textposition ermitteln
        add     zl, r0                  ;Sensor-Textposition ermitteln
        adc     zh, r1                  ;bertrag
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
;
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r18, grse11		;Grafik 1 Sensornummer holen
m1g175: cpi     r18, 31                 ;Sensornummer fr Batteriewert?
        brne    m1g180                  ;nein -> weiter
        ldi     r22, 0x37               ;sonst Text "Batterie"
        call    strlcd                  ;ausgeben
        rjmp    m1g280                  ;Bereich hinter Text lschen
;
m1g180:	ldi	r19, 9			;Offset/Lnge 9 Bytes
	mul	r18, r19		;Adressoffset berechnen
	ldi	r16, low(esentx)
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensor-Namen
	add	r16, r0
	adc	r17, r1			;EEPROM-Adresse ermitteln
	call	eeplcd			;Preset-Name ausgeben
	rjmp	m1g280			;Bereich hinter Text lschen
;
m1g190:	cpi	r25, 3			;Zhler auf Grafik 1 Differenz-Sensor?
	brne	m1g200			;nein -> weiter testen
	lds	r16, grflag		;sonst Grafik-Flags holen
	bst	r16, 2			;Differenz-Modus aktiv?
	brtc	m1g220			;nein -> Text "aus" ausgeben
;
        ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensortexte
        ldi     r16, 9                  ;Textabstand 9 Bytes
	lds	r17, grse12		;Grafik 1 Differenz-Sensornummer holen
        mul     r16, r17                ;Textposition ermitteln
        add     zl, r0                  ;Sensor-Textposition ermitteln
        adc     zh, r1                  ;bertrag
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
;
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r18, grse12		;Grafik 1 Differenz-Sensornummer holen
	rjmp	m1g175			;Name ausgeben
;
m1g200:	cpi	r25, 4			;Zhler auf Grafik 1 Modus?
	brne	m1g210			;nein -> weiter testen
	lds	r16, grmod1		;sonst Grafik 1 Modus holen
	rjmp	m1g260			;Text ermitteln und ausgeben
;
m1g210:	cpi	r25, 5			;Zhler auf Grafik 2 Sensor?
	brne	m1g240			;nein -> weiter testen
	lds	r16, grflag		;sonst Grafik-Flags holen
	bst	r16, 4			;Doppelgrafik?
	brts	m1g230			;ja -> bearbeiten
m1g220:	ldi	r22, 0xb0		;sonst Text "aus"
	rjmp	m1g270			;ausgeben
;
m1g230: ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensortexte
        ldi     r16, 9                  ;Textabstand 9 Bytes
	lds	r17, grse21		;Grafik 2 Sensornummer holen
        mul     r16, r17                ;Textposition ermitteln
        add     zl, r0                  ;Sensor-Textposition ermitteln
        adc     zh, r1                  ;bertrag
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
;
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r18, grse21		;Grafik 2 Sensornummer holen
	rjmp	m1g175			;Name ausgeben
;
m1g240:	cpi	r25, 6			;Zhler auf Grafik 2 Differenz-Sensor?
	brne	m1g250			;nein -> weiter testen
	lds	r16, grflag		;sonst Grafik-Flags holen
	bst	r16, 4			;Doppelgrafik?
	brtc	m1g220			;nein -> "aus" ausgeben
	bst	r16, 3			;Differenz-Modus aktiv?
	brtc	m1g220			;nein -> "aus" ausgeben
;
        ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensortexte
        ldi     r16, 9                  ;Textabstand 9 Bytes
	lds	r17, grse22		;Grafik 2 Differenz-Sensornummer holen
        mul     r16, r17                ;Textposition ermitteln
        add     zl, r0                  ;Sensor-Textposition ermitteln
        adc     zh, r1                  ;bertrag
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
;
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r18, grse22		;Grafik 2 Differenz-Sensornummer holen
	rjmp	m1g175			;Name ausgeben
;
m1g250:	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppelgrafik?
	brts	m1g255			;ja -> bearbeiten
	ldi	r22, 0xb0		;sonst Text "aus"
	rjmp	m1g270			;ausgeben
;
m1g255:	lds	r16, grmod2		;sonst Grafik-Modus 2 holen
m1g260:	ldi	r22, 0xab		;Text "15 min Werte"
	add	r22, r16		;Grafik-Modus addieren
m1g270:	call	strlcd			;Text ausgeben
;
m1g280:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 237		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
;
m1g290:	adiw	yl, 1			;Flag Tabellenzeiger erhhen
	inc	r25			;nchste Zeile
	cpi	r25, 8			;alle Zeilen bearbeitet?
	brcc	m1g400			;ja -> weiter
	rjmp	m1g110			;sonst Schleife
;
; Anzeigemodus Konfiguration - Grafik-Presets: Auswahl bearbeiten
;
m1g400:	lds	r16, menpos		;ausgewhlte Position holen
	lds	r17, encval		;Impulsgeberwert holen
	cp	r16, r17		;nderung der Position?
	breq	m1g410			;nein -> weiter
;
	ldi	xl, low(cfgflg)         ;sonst
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	movw	yl, xl			;Zeiger kopieren
	dec	r16			;Positionswert korrigieren
	clr	r19
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r19			;bertrag
	ld	r16, x			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	x, r16			;Flags wieder speichern
	dec	r17			;Impulsgeberwert korrigieren
	add	yl, r17			;Flag-Position ermitteln
	adc	yh, r19			;bertrag
	ld	r16, y			;Zeilen-Flags holen
	sbr	r16, 0x01		;Name neu ausgeben
	st	y, r16			;Flags wieder speichern
	lds	r16, encval		;Impulsgeber-Wert holen
	sts	menpos, r16		;ausgewhlte Position setzen
m1g410:	rjmp	m20000
;
; Anzeigemodus Konfiguration - Grafik-Presets: ndern bearbeiten
;
m1g500:	cpi	r16, 26			;Anzeigemodus Konfiguration - Grafik-Presets, ndern?
	breq	m1g510			;ja -> weiter
	rjmp	m1h000			;sonst weiter testen
;
m1g510:	lds	r19, encflg		;Impulsgeber-Flags holen
	bst	r19, 7			;Wert gendert?
	brts	m1g530			;ja -> bearbeiten
	ldi	xl, low(cfgflg)         ;sonst
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	lds	r16, menpos		;ausgewhlte Position holen
	dec	r16			;Positionswert korrigieren
	clr	r17
	add	xl, r16			;Flag-Position ermitteln
	adc	xh, r17			;bertrag
	ld	r18, x			;Zeilen-Flags holen
	bst	r18, 1			;Daten neu ausgeben?
	brts	m1g520			;ja -> weiter
	rjmp	m1g600			;sonst Ende
;
m1g520:	cbr	r18, 0x02		;Neuausgabe-Flag lschen
	st	x, r18			;Flags wieder speichern
m1g530:	cbr	r19, 1 << 7		;Impulsgeber nderungs-Flag lschen
	sts	encflg, r19		;Impulsgeber-Flags wieder speichern
	lds	r16, menpos		;ausgewhlte Position holen
	dec	r16			;Positionswert korrigieren
	ldi	r20, 128		;X-Startposition
	ldi	r21, 16			;Y-Startposition
	ldi	r17, 12			;Zeilenabstand festlegen
	mul	r16, r17		;neue Zeilenposition ermitteln
	add	r21, r0			;zur Startposition addieren
;
	cpi	r16, 0			;Position auf Preset-Nummer?
	brne	m1g540			;nein -> weiter testen
	lds	r18, encval		;aktuellen Impulsgeberwert holen
	call	bytlcd			;Wert ausgeben
	rjmp	m1g590			;Zeilenende lschen
;
m1g540:	cpi	r16, 1			;Position auf Namen-Eingabe?
	brne	m1g550			;nein -> weiter testen
	call	txblcd			;sonst Eingabepuffer ausgeben
	rjmp	m1g590			;Zeilenende lschen
;
m1g550:	cpi	r16, 4			;Position auf Grafik 1 Modus?
	breq	m1g580			;ja -> Modus ausgeben
	cpi	r16, 7			;Position auf Grafik 1 Modus?
	breq	m1g580			;ja -> Modus ausgeben
;
	lds	r18, encval		;aktuellen Impulsgeberwert holen
	tst	r18			;Impulsgeberwert = 0?
	brne	m1g560			;nein -> Sensordaten ausgeben
	ldi	r22, 0xb0		;sonst Text "aus"
	call	strlcd			;ausgeben
	rjmp	m1g590			;Zeilenende lschen
;
m1g560:ldi     zl, byte3(dsentx * 2)
        out     rampz, zl
        ldi     zl, low(dsentx * 2)
        ldi     zh, high(dsentx * 2)    ;Zeiger auf Default-Sensortexte
        ldi     r16, 9                  ;Textabstand 9 Bytes
	lds	r17, encval		;aktuellen Impulsgeberwert holen
        dec     r17			;korrigieren
        mul     r16, r17                ;Textposition ermitteln
        add     zl, r0                  ;Sensor-Textposition ermitteln
        adc     zh, r1                  ;bertrag
        elpm    r22, z+                 ;Zeichen 1 holen
        elpm    r5, z+                  ;Zeichen 2 holen
        elpm    r6, z+                  ;Zeichen 3 holen
	call	asclcd			;Zeichen 1 ausgeben
        mov     r22, r5                 ;Zeichen 2 holen
	call	asclcd			;ausgeben
        mov     r22, r6                 ;Zeichen 3 holen
	call	asclcd			;ausgeben
;
	ldi	r22, 0xa0		;schmales Leerzeichen
	call	asclcd			;ausgeben
	lds	r18, encval		;aktuellen Impulsgeberwert holen
	dec	r18			;korrigieren
        cpi     r18, 31                 ;Sensornummer fr Batteriewert?
        brne    m1g570                  ;nein -> weiter
        ldi     r22, 0x37               ;sonst Text "Batterie"
        call    strlcd                  ;ausgeben
        rjmp    m1g590			;Zeilenende lschen
;
m1g570:	ldi	r19, 9			;Offset/Lnge 9 Bytes
	mul	r18, r19		;Adressoffset berechnen
	ldi	r16, low(esentx)
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensor-Namen
	add	r16, r0
	adc	r17, r1			;EEPROM-Adresse ermitteln
	call	eeplcd			;Preset-Name ausgeben
	rjmp	m1g590			;Zeilenende lschen
;
m1g580:	lds	r16, encval		;sonst Impulsgeberwert holen
	ldi	r22, 0xab		;Text "15min Werte"
	add	r22, r16		;Grafik-Modus addieren
	call	strlcd			;Text ausgeben
;
m1g590:	movw	r18, r20		;X/Y Koordinaten kopieren
	ldi	r20, 237		;X-Endwert
	ldi	r22, 11			;zum Endwert Y 11 Pixel
	add	r21, r22		;addieren
	call	clrwnd			;Bereich lschen
m1g600:	rjmp	m20000
;
m1h000:	rjmp	m20000			;eventuell weitere Anzeigemodi
;
; Empfangspuffer RS232-1 lesen (Datenempfang vom Sensormodul)
;
m20000:	call	get1ch			;Zeichen aus Empfangspuffer
	tst	r17			;holen, neues Zeichen?
	brne	m20010			;nein -> Ende
	cpi	r16,lf			;LF (10) empfangen?
	breq	m20010			;ja -> ignorieren
	cpi	r16,cr			;CR (13) empfangen?
	breq	m20100			;ja -> Puffer auswerten
;
	lds	xl, rxdap1
	lds	xh, rxdap1 + 1		        ;Zeiger auf RX-Datenpuffer 1
	ldi	yl, low(rxdat1 + rxda1l)
	ldi	yh, high(rxdat1 + rxda1l)	;Lnge des RX-Datenpuffers 1
	sub	yl, xl
	sbc	yh, xh			;Pufferende erreicht (voll)?
	breq	m20010			;ja -> Zeichen verwerfen
	st	x+, r16			;sonst Zeichen im Puffer ablegen
	sts	rxdap1, xl
	sts	rxdap1 + 1, xh		;Zeiger wieder speichern
m20010:	rjmp	m30000			;weiter zum nchsten Abschnitt
;
m20100:	lds	xl, rxdap1
	lds	xh, rxdap1 + 1		;Zeiger auf RX-Datenpuffer 1
	ldi	yl, low(rxdat1)
	ldi	yh, high(rxdat1)	;Adresse des RX-Datenpuffers 1
	sub	xl, yl
	sbc	xh, yh			;Pufferposition berechnen
	tst	xh			;gltige Position?
	brne	m20200			;nein -> Ende
	cpi	xl, 7			;Sensor-Daten empfangen?
	breq	m21000			;ja -> bearbeiten
	cpi	xl, 3			;Alarm-Daten empfangen?
	brne	m20110			;nein -> weiter testen
	rjmp	m22000			;sonst Alarm bearbeiten
m20110:	cpi	xl, 8			;Sensorbelegung empfangen?	
	brne	m20200			;nein -> Ende
	rjmp	m23000			;sonst Sensorbelegung bearbeiten
;
m20200:	ldi	yl,low(rxdat1)		;sonst Zeiger fr Datenpuffer 1
	ldi	yh,high(rxdat1)		;wieder auf Anfangswert setzen
	sts	rxdap1, yl
	sts	rxdap1 + 1, yh		;Zeiger speichern
	rjmp	m30000			;weiter zum nchsten Abschnitt
;
; empfangene Zeile (Sensorwert) auf Plausibilitt prfen
;
m21000:	ld	r16, y+			;erstes Zeichen holen
	cpi	r16, '1'		;Zeichen < "1"?
	brcs	m20200			;ja -> Fehler
	cpi	r16,'z' + 1		;Zeichen > "z"?
	brcc	m20200			;ja -> Fehler
	cpi	r16, '9'		;Zeichen < "9"?
	brcs	m21010			;ja -> weiter (Sensor 1-8)
	cpi	r16, 'a'		;Zeichen < "a"?
	brcs	m20200			;ja -> Fehler
        cpi     r16, 't'                ;Zeichen < "t"?
        brcs    m21010                  ;ja -> weiter (Sensor a-s)
        cpi     r16, 'w'                ;Zeichen < "w"?
        brcs    m20200                  ;ja -> Fehler
        rjmp    m21700                  ;sonst weiter zu Luftdruck und Luftfeuchtigkeit
;
; empfangenen Temperaturwert auf Plausibilitt prfen
;
m21010:	ld	r16, y+			;zweites Zeichen holen
	cpi	r16, ':'		;Doppelpunkt?
	brne	m20200			;nein -> Fehler
	ld	r16, y+			;drittes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	breq	m21020			;ja -> weiter
	cpi	r16, '-'		;Minus-Zeichen?
	breq	m21020			;ja -> weiter
	cpi	r16, '1'		;Zeichen = "1"?
	brne	m20200			;nein -> Fehler
m21020:	ld	r16, y+			;viertes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	breq	m21030			;ja -> weiter
	cpi	r16, '-'		;Minus-Zeichen?
	breq	m21030			;ja -> weiter
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m20200			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m20200			;ja -> Fehler
m21030:	ld	r16, y+			;fnftes Zeichen holen
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m20200			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m20200			;ja -> Fehler
	ld	r16, y+			;sechstes Zeichen holen
	cpi	r16, '.'		;Punkt?
	breq	m21040			;ja -> weiter
	cpi	r16, ','		;Komma?
	brne	m20200			;nein -> Fehler
m21040:	ld	r16, y+			;siebtes Zeichen holen
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m20200			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m20200			;ja -> Fehler
;
; Temperaturdaten sind ok, in RAM-Tabelle speichern
;
	clr	r16			;Flag fr fehlerfreien Empfang
	sts	rcvflg, r16		;setzen (fr Status-Anzeige)
	lds	r16, rxdat1		;erstes Zeichen aus Datenpuffer holen
	cpi	r16, '9'		;Sensorgruppe 1-8?
	brcc	m21100			;nein -> weiter
	subi	r16, '1'		;sonst in Sensornummer 0-7 umwandeln
	rjmp	m21110
;
m21100:	subi	r16, 'a' - 8		;in Sensornummer 8-26 umwandeln
m21110:	mov	r11, r16		;Sensornummer fr Meldung sichern
	mov	r20, r16		;Sensornummer zwischenspeichern
	add	r20, r20		;Sensornummer verdoppeln
	clr	r21
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Temperaturwerte
	add	yl, r20			;Temperaturwert-Adresse berechnen
	adc	yh, r21			;bertrag addieren
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	add	xl, r16			;Flag-Adresse berechnen
	adc	xh, r21			;bertrag addieren
	call	temhex			;Temperaturwert in Hex wandeln
	ld	r18, y			;alten L-Wert aus Tabelle holen
	cp	r16, r18		;hat sich der Wert gendert?
	brne	m21120			;ja -> Flag setzen
	ldd	r18, y + 1		;alten H-Wert aus Tabelle holen
	cp	r17, r18		;hat sich der Wert gendert?
	brne	m21120			;ja -> Flag setzen
	rjmp	m21300			;sonst Limit und Min/Max berspringen
;
m21120:	st	y, r16			;neuen Wert L speichern
	std	y + 1, r17		;neuen Wert H speichern
	ld	r22, x			;Sensor-Flags holen
	sbr	r22, 0x01		;nderungs-Flag setzen, Wert neu ausgeben
;
; Temperatur auf Limit ber- oder Unterschreitung prfen und Flags setzen
;
	ldi	zl, low(senlih)
	ldi	zh, high(senlih)	;Zeiger auf obere Temperatur-Limits (Schwellwerte)
	add	zl, r20			;obere Limit-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ldd	r19, z + 1		;oberes Limit H holen
	cpi	r19, 0x80		;Alarmberwachung aktiv?
	breq	m21170			;nein -> Prfung berspringen
	bst	r22, 7			;Sensor inaktiv?
	brts	m21170			;ja -> Prfung berspringen
;
	ld	r18, z			;sonst oberes Limit L holen
	sub	r18, r16
	sbc	r19, r17		;oberes Limit berschritten?
	brpl	m21160			;nein -> Flags prfen
	bst	r22, 2			;berschreitungs-Flag gesetzt?
	brts	m21170			;ja -> weiter
	cbr	r22, 0x28		;sonst Unterschreitung + Unterschreitungs-Erinnerung lschen
	sbr	r22, 0x06		;berschreitung setzen, Sensorname neu ausgeben
;
	mov	r18, r11		;gesicherte Sensornummer holen
	ori	r18, 0x60		;Fehlercode "berschreitung" setzen
	mov	r11, r18		;Meldungscode wieder speichern
;
	lds	r18, confg1		;Konfigurationsbyte 1 holen
	bst	r18, 2			;jede berschreitung melden?
	brts	m21150			;ja -> Signal ausgeben
	bst	r22, 4			;berschreitungserinnerung?
	brts	m21170			;ja -> kein weiteres Signal
m21150:	lds	r18, signum + 4		;Signalnummer fr berschreitung
	lds	r19, sigdur + 4		;Signaldauer fr berschreitung
	call	signal			;Signal ausgeben
	rjmp	m21170
;
m21160:	bst	r22, 2			;berschreitungs-Flag gesetzt?
	brtc	m21170			;nein -> weiter
	cbr	r22, 0x24		;sonst berschreitung + Unterschreitungs-Erinnerung lschen
	sbr	r22, 0x12		;berschreitungs-Erinnerung setzen, Sensorname neu ausgeben
;
	mov	r18, r11		;gesicherte Sensornummer holen
	ori	r18, 0xa0		;Fehlercode "Temperatur normal"
	mov	r11, r18		;Meldungscode wieder speichern
;
m21170:	ldi	zl, low(senlil)
	ldi	zh, high(senlil)	;Zeiger auf untere Temperatur-Limits (Schwellwerte)
	add	zl, r20			;untere Limit-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ldd	r19, z + 1		;unteres Limit H holen
	cpi	r19, 0x80		;Alarmberwachung aktiv?
	breq	m21200			;nein -> Prfung berspringen
	bst	r22, 7			;Sensor inaktiv?
	brts	m21200			;ja -> Prfung berspringen
;
	ld	r18, z			;sonst unteres Limit L holen
	sub	r16, r18
	sbc	r17, r19		;unteres Limit unterschritten?
	brpl	m21190			;nein -> Flags prfen
	bst	r22, 3			;Unterschreitungs-Flag gesetzt?
	brts	m21200			;ja -> weiter
	cbr	r22, 0x14		;sonst berschreitung + berschreitungs-Erinnerung lschen
	sbr	r22, 0x0a		;Unterschreitung setzen, Sensorname neu ausgeben
;
	mov	r18, r11		;gesicherte Sensornummer holen
	ori	r18, 0x80		;Fehlercode "Unterschreitung"
	mov	r11, r18		;Meldungscode wieder speichern
;
	lds	r18, confg1		;Konfigurationsbyte 1 holen
	bst	r18, 2			;jede Unterschreitung melden?
	brts	m21180			;ja -> Signal ausgeben
	bst	r22, 5			;Unterschreitungserinnerung?
	brts	m21200			;ja -> kein weiteres Signal
m21180:	lds	r18, signum + 5		;Signalnummer fr Unterschreitung
	lds	r19, sigdur + 5		;Signaldauer fr Unterschreitung
	call	signal			;Signal ausgeben
	rjmp	m21200
;
m21190:	bst	r22, 3			;Unterschreitungs-Flag gesetzt?
	brtc	m21200			;nein -> weiter
	cbr	r22, 0x18		;sonst Unterschreitung + berschreitungs-Erinnerung lschen
	sbr	r22, 0x22		;Unterschreitungs-Erinnerung setzen, Sensorname neu ausgeben
;
	mov	r18, r11		;gesicherte Sensornummer holen
	ori	r18, 0xa0		;Fehlercode "Temperatur normal"
	mov	r11, r18		;Meldungscode wieder speichern
;
m21200:	st	x, r22			;Flags wieder speichern
;
; Maximal- und Minimal-Werte ermitteln und speichern (Anzeige)
;
	ld	r16, y
	ldd	r17, y + 1		;Sensorwert holen
	ldi	zl, low(semax1)
	ldi	zh, high(semax1)	;Zeiger auf Maximalwerte (Anzeige)
	add	zl, r20			;Maximalwert-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ldd	r19, z + 1		;Maximalwert H holen
	cpi	r19, 0x80		;Maximalwert gltig
	breq	m21210			;nein -> aktuellen Wert setzen
	ld	r18, z			;Maximalwert L holen
	sub	r18, r16		;bisheriger Maximalwert
	sbc	r19, r17		;berschritten?
	brpl	m21220			;nein -> weiter
m21210:	st	z, r16			;sonst neuen Maximalwert
	std	z + 1, r17		;speichern
;
m21220:	ldi	zl, low(semin1)
	ldi	zh, high(semin1)	;Zeiger auf Minimalwerte (Anzeige)
	add	zl, r20			;Minimalwert-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ldd	r19, z + 1		;Minimalwert H holen
	cpi	r19, 0x80		;Minimalwert gltig?
	breq	m21230			;nein -> aktuellen Wert setzen
	ld	r18, z			;Minimalwert L holen
	sub	r16, r18		;bisheriger Minimalwert
	sbc	r17, r19		;unterschritten?
	brpl	m21240			;nein -> weiter
	ld	r16, y			;sonst nochmals aktuellen
	ldd	r17, y + 1		;Sensorwert holen
m21230:	st	z, r16			;und als neuen Minimalwert
	std	z + 1, r17		;speichern
;
; Maximal- und Minimal-Werte ermitteln und speichern (EEPROM-Log)
;
m21240:	ld	r16, y
	ldd	r17, y + 1		;Sensorwert holen
	ldi	zl, low(semax2)
	ldi	zh, high(semax2)	;Zeiger auf Maximalwerte (Log)
	add	zl, r20			;Maximalwert-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ldd	r19, z + 1		;Maximalwert H holen
	cpi	r19, 0x80		;Maximalwert gltig
	breq	m21250			;nein -> aktuellen Wert setzen
	ld	r18, z			;Maximalwert L holen
	sub	r18, r16		;bisheriger Maximalwert
	sbc	r19, r17		;berschritten?
	brpl	m21260			;nein -> weiter
m21250:	st	z, r16			;sonst neuen Maximalwert
	std	z + 1, r17		;speichern
;
m21260:	ldi	zl, low(semin2)
	ldi	zh, high(semin2)	;Zeiger auf Minimalwerte (Log)
	add	zl, r20			;Minimalwert-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ldd	r19, z + 1		;Minimalwert H holen
	cpi	r19, 0x80		;Minimalwert gltig?
	breq	m21270			;nein -> aktuellen Wert setzen
	ld	r18, z			;Minimalwert L holen
	sub	r16, r18		;bisheriger Minimalwert
	sbc	r17, r19		;unterschritten?
	brpl	m21300			;nein -> weiter
	ld	r16, y			;sonst nochmals aktuellen
	ldd	r17, y + 1		;Sensorwert holen
m21270:	st	z, r16			;und als neuen Minimalwert
	std	z + 1, r17		;speichern
;
; Messwert zur Mittelwertsumme addieren und Mittelwertzhler erhhen
;
m21300:	ldi	zl, low(senavc)
	ldi	zh, high(senavc)	;Zeiger auf Mittelwertzhler
	add	zl, r20			;Zhler-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ld	r16, z			;Zhlerwert L holen
	ldd	r17, z + 1		;Zhlerwert H holen
	inc	r16			;L-Wert erhhen
	brne	m21310			;bertrag? nein -> weiter
	inc	r17			;sonst H-Wert erhhen
m21310:	st	z, r16			;Zhlerwert L wieder speichern
	std	z + 1, r17		;Zhlerwert H wieder speichern
;
	ldi	zl, low(senavs)
	ldi	zh, high(senavs)	;Zeiger auf Mittelwertsummen
	mov	r23, r20		;Adress-Offset kopieren
	lsr	r23			;halbieren und addieren wegen
	add	r20, r23		;Tabellen-Offset von 3 Byte
	add	zl, r20			;Zhler-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ld	r16, z			;Summenwert L holen
	ldd	r17, z + 1		;Summenwert H holen
	ldd	r18, z + 2		;Summenwert B3 holen
	ld	r13, y			;Sensorwert L holen
	ldd	r14, y + 1		;Sensorwert H holen
	clr	r15			;Sensorwert B3 = 0
	bst	r14, 7			;Sensorwert negativ?
	brtc	m21320			;nein -> weiter
	dec	r15			;sonst B3 = 0xff
m21320:	add	r16, r13
	adc	r17, r14
	adc	r18, r15		;Sensorwert zur Summe addieren
	st	z, r16			;neuen Summenwert L speichern
	std	z + 1, r17		;neuen Summenwert H speichern
	std	z + 2, r18		;neuen Summenwert B3 speichern
;
; Timeout-Zhler und einige Flags neu setzen, Fehlermeldungen ausgeben, auf Anzeige-Seite mit Fehlersensor umschalten
;
	ldi	zl, low(sentio)
	ldi	zh, high(sentio)	;Zeiger auf Timeout-Zhler
	add	zl, r23			;Zhler-Adresse berechnen
	adc	zh, r21			;bertrag addieren
	ldi	r16, timsen		;Timeout-Wert laden
	st	z, r16			;und als Timeout setzen
	ldi	r16, timmsg		;Meldungs-Timeout-Wert laden
	sts	msgtio, r16		;und als Timeout neu setzen
	ld	r22, x			;Sensor-Flags holen
	bst	r22, 6			;war Ausfall-Flag gesetzt?
	brtc	m21500			;nein -> berspringen
	cbr	r22, 0x40		;sonst Ausfall-Flag lschen
	sbr	r22, 0x01		;und nderungs-Flag setzen
	st	x, r22			;Flags wieder speichern
;
        mov     r20, r11                ;Fehlercode holen
        andi    r20, 0x1f               ;Sensornummer filtern
        ldi     r19, 0xfa               ;Meldung "Sensor ok" generieren
	call	wrimsg			;Fehlermeldung schreiben
;
m21500:	mov	r19, r11		;Fehlercode holen
	andi	r19, 0xe0		;Fehlermeldung vorhanden?
	breq	m21600			;nein -> weiter
        ror     r19                     ;an richtige Position schieben
        swap    r19                     ;in unteres Nibble legen
        ori     r19, 0xf8               ;Meldungscode generieren
	mov	r20, r11		;Fehlercode holen
        andi    r20, 0x1f               ;Sensornummer filtern
	call	wrimsg			;Fehlermeldung schreiben
;
        mov     r16, r11                ;Fehlercode holen
        andi    r16, 0xe0               ;Fehler filtern
        cpi     r16, 0x60               ;Fehler "Temperatur zu hoch"?
        breq    m21510                  ;ja -> bearbeiten
        cpi     r16, 0x80               ;Fehler "Temperatur zu niedrig"?
        brne    m21600                  ;nein -> berspringen
;
m21510: mov     r16, r11                ;Fehlercode holen
        andi    r16, 0x1f               ;Sensornummer filtern
        call    pagesw                  ;gegebenenfalls Anzeige-Seite umschalten
;
m21600:	ldi	r16, timmsg		;Meldungs-Timeout-Wert laden
	sts	msgtio, r16		;und als Timeout neu setzen
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 1			;Flag "Datenempfangsfehler"?
	brtc	m21610			;nein -> weiter
	cbr	r16, 0x02		;Datenempfangsfehler lschen
	sbr	r16, 0x80		;Fehlererinnerung setzen
	sts	errflg, r16		;Flags wieder speichern
;
	ldi	r19, 0xf6		;Meldung "Datenempfang ok"
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Flags wieder speichern
m21610:	rjmp	m20200			;Zeiger zurcksetzen, Ende
;
; empfangenen Luftdruckwert auf Plausibilitt prfen
;
m21700: cpi     r16, 'w'                ;Luftdrucksensor?
        brne    m21720                  ;nein -> weiter mit Luftfeuchtigkeitssensoren
;
	ld	r16, y+			;zweites Zeichen holen
	cpi	r16, ':'		;Doppelpunkt?
	brne	m21610			;nein -> Fehler
	ld	r16, y+			;drittes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	brne	m21610			;nein -> Fehler
	ld	r16, y+			;viertes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	breq	m21710			;ja -> weiter
	cpi	r16, '1'		;Zeichen = "1"?
	brne	m21610			;nein -> Fehler
m21710:	ld	r16, y+			;fnftes Zeichen holen
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m21610			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m21610			;ja -> Fehler
	ld	r16, y+			;sechstes Zeichen holen
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m21610			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m21610			;ja -> Fehler
	ld	r16, y+			;siebtes Zeichen holen
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m21610			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m21610			;ja -> Fehler
        rjmp    m21750                  ;sonst Daten speichern
;
; empfangenen Luftfeuchtigkeitswert auf Plausibilitt prfen
;
m21720: ld	r16, y+			;zweites Zeichen holen
	cpi	r16, ':'		;Doppelpunkt?
	brne	m21610			;nein -> Fehler
	ld	r16, y+			;drittes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	brne	m21610			;nein -> Fehler
	ld	r16, y+			;viertes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	brne	m21610			;nein -> Fehler
	ld	r16, y+			;fnftes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	breq	m21730			;ja -> weiter
	cpi	r16, '1'		;Zeichen = "1"?
	brne	m21610			;nein -> Fehler
m21730:	ld	r16, y+			;sechstes Zeichen holen
	cpi	r16, ' '		;Leerzeichen?
	breq	m21740			;ja -> weiter
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m21610			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m21610			;ja -> Fehler
m21740:	ld	r16, y+			;siebtes Zeichen holen
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m21610			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen > "9"?
	brcc	m21610			;ja -> Fehler
;
; Luftdruck- und Luftfeuchtigkeitsdaten sind ok, in RAM-Tabelle speichern
;
m21750:	clr	r16			;Flag fr fehlerfreien Empfang
	sts	rcvflg, r16		;setzen (fr Status-Anzeige)
	lds	r16, rxdat1		;erstes Zeichen aus Datenpuffer holen
        subi    r16, 'w' - 27           ;in logische Sensornummer umwandeln
	mov	r11, r16		;Sensornummer fr Meldung sichern
	mov	r20, r16		;Sensornummer zwischenspeichern
	add	r20, r20		;Sensornummer verdoppeln
	clr	r21
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	add	yl, r20			;Sensorwert-Adresse berechnen
	adc	yh, r21			;bertrag addieren
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	add	xl, r16			;Flag-Adresse berechnen
	adc	xh, r21			;bertrag addieren
	call	senhex			;Sensorwert in Hex wandeln
	ld	r18, y			;alten L-Wert aus Tabelle holen
	cp	r16, r18		;hat sich der Wert gendert?
	brne	m21760			;ja -> Flag setzen
	ldd	r18, y + 1		;alten H-Wert aus Tabelle holen
	cp	r17, r18		;hat sich der Wert gendert?
	brne	m21760			;ja -> Flag setzen
	rjmp	m21300			;sonst Min/Max berspringen, weiterer Ablauf wie Temperaturwerte
;
m21760:	st	y, r16			;neuen Wert L speichern
	std	y + 1, r17		;neuen Wert H speichern
	ld	r22, x			;sonst Sensor-Flags holen
	sbr	r22, 0x01		;nderungs-Flag setzen
        rjmp    m21200                  ;Min/Max-Werte ermitteln, weiterer Ablauf wie Temperaturwerte
;
; empfangene Zeile (Alarmmeldung) auf Plausibilitt prfen
;
m22000:	ld	r16, y+			;erstes Zeichen holen
	cpi	r16, 'A'		;Zeichen < "A"?
	brcs	m22010			;ja -> Fehler
	cpi	r16, 'D' + 1		;Zeichen > "D"?
        brcs    m22020                  ;nein -> weiter
m22010:	rjmp	m20200			;sonst Fehler
;
m22020:	ld	r16, y+			;zweites Zeichen holen
	cpi	r16, ':'		;Doppelpunkt?
	brne	m22010			;nein -> Fehler
	ld	r16, y+			;drittes Zeichen holen
	cpi	r16, '0'		;Zeichen = "0"?
	breq	m22100			;ja -> bearbeiten
	cpi	r16, '1'		;Zeichen = "1"?
	brne	m22010			;nein -> Fehler
;
; Alarm-Daten sind ok, in SRAM-Tabelle speichern
;
m22100:	clr	r16			;Flag fr fehlerfreien Empfang
	sts	rcvflg, r16		;setzen (fr Status-Anzeige)
	lds	r16, rxdat1		;erstes Zeichen aus Datenpuffer
	andi	r16, 0x07		;in Ziffernwert wandeln
	dec	r16			;korrigieren (= Alarmummer)
	clr	r17
	mov	r11, r16		;Alarmnummer fr Meldung sichern
	ldi	xl, low(almflg)
	ldi	xh, high(almflg)	;Zeiger auf Alarmflags
	add	xl, r16			;Tabellenplatz berechnen
	adc	xh, r17			;bertrag
	ldi	yl, low(alminv)
	ldi	yh, high(alminv)	;Zeiger auf Alarm-Invertierung
	add	yl, r16			;Tabellenplatz berechnen
	adc	yh, r17			;bertrag
	lds	r18, rxdat1 + 2		;Alarmwert holen
	andi	r18, 0x01		;in Binrwert wandeln
	ld	r19, y			;Invertierungswert holen
	eor	r18, r19		;Alarmwert invertieren
	ld	r19, x			;Alarmflags holen
	lsr	r19			;Aktiv-Flag an Alarmposition
	andi	r19, 0x01		;Aktiv-Flag filtern
	cp	r18, r19		;hat sich der Wert gendert?
	breq	m22120			;nein -> weiter
	ld	r19, x			;sonst Alarmflags holen
	bst	r19, 7			;wird Alarm genutzt?
	brts	m22120			;nein -> nicht signalisieren
;
	bst	r18, 0			;neuen Alarmwert holen und
	bld	r19, 1			;als Aktiv-Flag setzen
	sbr	r19, 0x01		;nderungs-Flag setzen
	st	x, r19			;Flags wieder speichern
	brts	m22110			;Alarm ein? ja -> Signal
	sbr	r19, 0x04		;sonst Erinnerung setzen
	st	x, r19			;Flags wieder speichern
;
	mov	r18, r11		;gesicherte Alarmnummer holen
	ori	r18, 0x80		;Fehlercode "Alarm aus" setzen
	mov	r11, r18		;Meldungscode wieder speichern
	rjmp	m22120
;
m22110:	ldi	xl, low(signum)
	ldi	xh, high(signum)	;Zeiger auf Signalnummern
	add	xl, r16			;Tabellenplatz berechnen
	adc	xh, r17			;bertrag
	ldi	yl, low(sigdur)
	ldi	yh, high(sigdur)	;Zeiger auf Signaldauern
	add	yl, r16			;Tabellenplatz berechnen
	adc	yh, r17			;bertrag
	ld	r18, x			;zugeordnete Signalnummer holen
	ld	r19, y			;zugeordnete Signaldauer holen
	call	signal			;Signal ausgeben
;
	mov	r18, r11		;gesicherte Alarmnummer holen
	ori	r18, 0x70		;Fehlercode "Alarm ein" setzen
	mov	r11, r18		;Meldungscode wieder speichern
;
m22120:	mov	r19, r11		;Fehlercode holen
	andi	r19, 0xf0		;Fehlermeldung vorhanden?
	breq	m22130			;nein -> weiter
        swap    r19                     ;sonst in unteres Nibble legen
        ori     r19, 0xf0               ;Meldungscode generieren
	mov	r20, r11		;Fehlercode holen
        andi    r20, 0x03               ;Sensornummer filtern
	call	wrimsg			;Fehlermeldung schreiben
m22130:	rjmp	m21600			;Empfangsfehler-Flag testen
;
; empfangene Zeile (Sensorbelegung) auf Plausibilitt prfen
;
m23000:	ldi	r17, 8			;8 Zeichen prfen
m23010:	ld	r16, y+			;Zeichen aus Puffer holen
	cpi	r16, '0'		;Zeichen < "0"?
	brcs	m23200			;ja -> Fehler
	cpi	r16, 'R' + 1		;Zeichen > "R"?
	brcc	m23200			;ja -> Fehler
	cpi	r16, '9' + 1		;Zeichen =< "9"?
	brcs	m23020			;ja -> weiter
	cpi	r16, 'A'		;Zeichen < "A"?
	brcs	m23200			;ja -> Fehler
m23020:	dec	r17			;alle Zeichen bearbeitet?
	brne	m23010			;nein -> Schleife
;
; Sensorbelegung ist ok, Daten speichern
;
	clr	r16			;Flag fr fehlerfreien Empfang
	sts	rcvflg, r16		;setzen (fr Status-Anzeige)
	ldi	r17, 8			;8 Zeichen kopieren
	ldi	yl, low(rxdat1)
	ldi	yh, high(rxdat1)	;Zeiger auf RX Datenpuffer 1
	ldi	xl, low(senmap)
	ldi	xh, high(senmap)	;Zeiger auf Sensorbelegung
m23030:	ld	r16, y+			;Byte aus RX-Datenpuffer holen
	st	x+, r16			;in Sensorbelegung speichern
	dec	r17			;alle Daten kopiert?
	brne	m23030			;nein -> Schleife
	rjmp	m21600			;Empfangsfehler-Flag testen
m23200:	rjmp	m20200			;Zeiger zurcksetzen, Ende
;
; Empfangspuffer RS232-0 lesen (Datenempfang von PC-Schnittstelle)
;
m30000:	call	get0ch			;Zeichen aus Empfangspuffer holen
	tst	r17			;neues Zeichen?
	brne	m30010			;nein -> Ende
	cpi	r16, lf			;LF (10) empfangen?
	breq	m30010			;ja -> ignorieren
	cpi	r16, cr			;CR (13) empfangen?
	breq	m30100			;ja -> Puffer auswerten
;
	lds	xl, rxdap0
	lds	xh, rxdap0 + 1			;Zeiger auf RX-Datenpuffer 0
	ldi	yl, low(rxdat0 + rxda0l)
	ldi	yh, high(rxdat0 + rxda0l)	;Lnge des RX-Datenpuffers 0
	sub	yl, xl
	sbc	yh, xh			;Pufferende erreicht (voll)?
	breq	m30010			;ja -> Zeichen verwerfen
	st	x+, r16			;sonst Zeichen im Puffer ablegen
	sts	rxdap0, xl
	sts	rxdap0 + 1, xh		;Zeiger wieder speichern
m30010:	rjmp	m40000			;weiter zum nchsten Abschnitt
;
m30100:	lds	r16, rxdap0		;Zeiger auf aktuelle Position
	lds	r17, rxdap0 + 1		;im RX-Datenpuffer 0
	ldi	yl, low(rxdat0)
	ldi	yh, high(rxdat0)	;Startadresse des RX-Datenpuffers 0
	sub	r16, yl
	sbc	r17, yh			;Anzahl der Zeichen im Puffer berechnen
	tst	r17			;Zeichenanzahl < 256?
	breq	m30120			;ja -> weiter
m30110:	rjmp	m30300			;sonst Ende
m30120:	tst	r16			;Zeichenanzahl = 0?
	breq	m30110			;ja -> Ende
;
; Empfangsdatenpuffer lesen, Kommando und Parameter ermitteln
;
	mov	r21, r16		;Zeichenanzahl kopieren
	clr	r19			;Zhler fr Kommandolnge = 0
m30130:	ld	r16, y+			;Zeichen aus Puffer holen
	cpi	r16, ' '		;Kommando-Ende?
	breq	m30140			;ja -> weiter
	inc	r19			;sonst Zhler erhhen
	cp	r19, r21		;letztes Zeichen im Puffer?
	brne	m30130			;nein -> Schleife
;
m30140:	ldi	xl, low(pccmdl * 2)
	ldi	xh, high(pccmdl * 2)
	ldi	r22, byte3(pccmdl * 2)	;Zeiger auf Kommandoliste
	clr	r20			;Kommandonummer = 0 setzen
m30150:	inc	r20			;Kommandonummer erhhen
	movw	zl, xl			;Listenzeiger L/H kopieren
	out	rampz, r22
	elpm	r16, z			;erstes Zeichen holen
	cpi	r16, 0xff		;Listen-Ende?
	breq	m30300			;ja -> Kommando nicht gefunden
	clr	r18			;Kommandolnge = 0 setzen
m30160:	elpm	r16, z+			;Zeichen aus Liste holen
	cpi	r16, ' '		;Kommando-Ende?
	breq	m30170			;ja -> weiter
	inc	r18			;Komandolnge + 1
	cpi	r18, 8			;alle 8 Zeichen gelesen?
	brcs	m30160			;nein -> Schleife
m30170:	cp	r18, r19		;Zeichenlnge identisch?
	breq	m30190			;ja -> weiter
m30180:	adiw	xl, 8			;sonst Zeiger auf nchstes Kommando
	clr	r16
	adc	r22, r16		;bertrag in Byte3
	rjmp	m30150			;Schleife
;
m30190:	movw	zl, xl			;Listenzeiger L/H kopieren
	out	rampz, r22
	ldi	yl, low(rxdat0)
	ldi	yh, high(rxdat0)	;Adresse des RX-Datenpuffers 0
m30200:	elpm	r16, z+			;Zeichen aus Liste holen
	ld	r17, y+			;Zeichen aus Datenpuffer holen
	ori	r17, 0x20		;in Kleinbuchstaben wandeln
	cp	r16, r17		;bereinstimmung?
	brne	m30180			;nein -> nchstes Kommando
	dec	r18			;alle Zeichen verglichen?
	brne	m30200			;nein -> Schleife
	cpi	r20, 1			;Ende/Abbruch-Kommando (1)?
	breq	m30210			;ja -> speichern
	lds	r16, pccmd		;sonst aktuelles Kommando holen
	tst	r16			;wird gerade Kommando bearbeitet?
	brne	m30300			;ja -> neues Kommando ignorieren
m30210:	sts	pccmd, r20		;sonst Kommandonummer speichern
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x04		;Info-Zeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
        clr     r16
        sts     comflg, r16             ;Kommunikations-Flag setzen
;
	call	clpara			;Parameterspeicher lschen
	cp	r19, r21		;wurde ein Parameter empfangen?
	breq	m30300			;nein -> Ende
	ldi	xl, low(pcpara)		;sonst
	ldi	xh, high(pcpara)	;Zeiger auf Parameterspeicher
	ldi	r17, 8			;Parameterlnge 8 Zeichen
m30230:	ld	r16, y+			;Zeichen aus Puffer holen
	cpi	r16, ' '		;Leerzeichen?
	breq	m30250			;ja -> berspringen
	cpi	r16, '-'		;Minuszeichen (Bindestrich)?
	brne	m30240			;nein -> weiter
	sts	pcpara + 8, r16		;sonst Minuszeichen speichern
	rjmp	m30300			;Ende
;
m30240:	tst	r17			;Parameterspeicher voll?
	breq	m30250			;ja -> berspringen, sonst
	st	x+, r16			;Zeichen in Parameterspeicher
	dec	r17			;Zhler - 1
m30250:	inc	r19			;Zeichenzhler + 1
	cp	r19, r21		;Pufferende erreicht?
	brne	m30230			;nein -> nchstes Zeichen
;
m30300:	ldi	yl, low(rxdat0)		;sonst Zeiger fr Datenpuffer 0
	ldi	yh, high(rxdat0)	;wieder auf Anfangswert setzen
	sts	rxdap0, yl
	sts	rxdap0 + 1, yh		;Zeiger speichern
	rjmp	m40000			;weiter zum nchsten Abschnitt
;
; Steuerung RS232-0 TX
;
m40000:	lds	r16, pccmd		;aktuelles Kommando holen
	cpi	r16, 0			;wird ein Kommando ausgefhrt?
	brne	m40100			;ja -> weiter
m40010:	rjmp	m50000			;sonst zum nchsten Abschnitt
;
; Kommando 1 bearbeiten (Ende/Abbruch)
;
m40100:	cpi	r16, 1			;Kommando 1 (Ende/Abbruch)?
	brne	m40500			;nein -> nchstes Kommando
m40200:	clr	r16			;sonst
	sts	pccmd, r16		;aktuelles Kommando beenden
	sts	pccpnt, r16
	sts	pccpnt + 1, r16
	sts	pccpnt + 2, r16		;EEPROM-Zeigeradresse lschen
	sts	pcccnt, r16
	sts	pcccnt + 1, r16		;Datensatz-Zhler lschen
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x04		;Info-Zeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
	rjmp	m50000
;
; Kommando 2 bearbeiten (Info)
;
m40500:	cpi	r16, 2			;Kommando 2 (Info)?
	brne	m41000			;nein -> nchstes Kommando
;
	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m40510			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
m40510:	ldi	r17, 8			;String 0x08 (Versionsnummer)
	ldi	xl, low(ascbuf)
	ldi	xh, high(ascbuf)	;Zeiger auf ASCII-Puffer
	call	xtrfst			;String in ASCII-Puffer legen
	call	xtbyte			;Anzahl der Bytes ermitteln
	call	senasc			;ASCII-Puffer senden
	rjmp	m40200			;Kommando beenden
;
; Kommando 3 bearbeiten (15min-Werte senden)
;
m41000:	cpi	r16, 3			;Kommando 3 (15min-Werte)?
	breq	m41010			;ja -> ausfhren
	rjmp	m42000			;sonst nchstes Kommando
;
m41010:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicher-Fehler?
	brtc	m41020			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
m41020:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r24, pcccnt
	lds	r25, pcccnt + 1		;Datensatz-Zhler holen
	mov	r16, r24
	or	r16, r25		;Datensatz-Zhler = 0?
        breq    m41030                  ;ja -> Suchmodus ermitteln
	rjmp	m41140			;sonst nchsten Datensatz bearbeiten
;
m41030:	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
        breq    m41040                  ;ja -> intelligente Suche beginnen
        rjmp    m41120                  ;sonst Standard-Suche beginnen
;
m41040:	lds	r20, xes1po             ;intelligente Suche beginnen
	lds	r21, xes1po + 1
	lds	r22, xes1po + 2		;aktuellen Zeiger der 15min-Werte holen
        ldi     r16, 19                 ;maximal 19 Lesetests im Abstand von
        mov     r12, r16                ;192 Datenstzen durchfhren
;
m41050: subi    r20, low(192 * xes1da)
        sbci    r21, high(192 * xes1da)         ;Zeigerposition 192 Datenstze
        sbci    r22, byte3(192 * xes1da)        ;in die Vergangenheit setzen
        ldi     r16, low(xs1sta)
        ldi     r17, high(xs1sta)
        ldi     r18, byte3(xs1sta)      ;Startadresse 15min-Werte laden
        cp      r20, r16
        cpc     r21, r17
        cpc     r22, r18                ;Startadresse unterschritten?
        brpl    m41060                  ;nein -> weiter
        subi    r20, low(-xs1len)
        sbci    r21, high(-xs1len)      ;sonst
        sbci    r22, byte3(-xs1len)     ;Adresse korrigieren
;
m41060:	ldi	r23, xes1da		;51 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m41070			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
;
m41070: subi    r20, low(xes1da)
        sbci    r21, high(xes1da)       ;Zeigeradresse wieder auf den
        sbci    r22, byte3(xes1da)      ;Stand vor dem Lesen setzen
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m41130			;nein -> Standard-Suche fortsetzen
;
	call	xtrtim			;sonst Zeit extrahieren
        ldi     r18, 8                  ;8 Zeichen vergleichen
	ldi	zl, low(ascbuf + 8)
	ldi	zh, high(ascbuf + 8)	;Zeiger auf ASCII-Puffer (Tag-Einer)
	ldi	yl, low(pcpara + 8)
	ldi	yh, high(pcpara + 8)	;Zeiger auf Parameterspeicher (Ende)
	clc				;bertrag lschen
m41080:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m41100			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m41080			;nein -> Schleife
        rjmp    m41110                  ;sonst Treffer (nchster Lesetest)
;
m41090:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m41100:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m41090			;nein -> Schleife
	brcs	m41130			;Param > Puffer? -> Standard-Suche fortsetzen
;
m41110:	dec     r12                     ;alle Lesetests ausgefhrt?
        brne    m41050                  ;nein -> Schleife, sonst Standard-Suche beginnen
;
m41120:	lds	r20, xes1po
	lds	r21, xes1po + 1		;aktuellen Zeiger der 15min-Werte holen,
	lds	r22, xes1po + 2		;zeigt auf ltesten Datensatz
m41130:	sts	pccpnt, r20
	sts	pccpnt + 1, r21
	sts	pccpnt + 2, r22		;EEPROM-Zeigeradresse setzen
;
m41140:	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m41150			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
m41150:	ldi	r23, xes1da		;51 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m41160			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
m41160:	sts	pccpnt, r20
	sts	pccpnt + 1, r21
	sts	pccpnt + 2, r22		;neue EEPROM-Zeigeradresse speichern
	adiw	r24, 1			;Datensatz-Zhler erhhen
	sts	pcccnt, r24
	sts	pcccnt + 1, r25		;Datensatz-Zhler speichern
;
; Kommando 3 bearbeiten: Daten aufbereiten und ausgeben
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m41300			;nein -> Ausgabe berspringen
;
	call	xtrtim			;sonst Zeit extrahieren
	ldi	yl, low(pcpara)
	ldi	yh, high(pcpara)	;Zeiger auf Parameterspeicher
	ld	r16, y			;erstes Zeichen vom Parameter
	tst	r16			;Parameter gltig?
	breq	m41250			;nein -> Datensatz ausgeben
;
	ldi	r18, 8			;8 Zeichen vergleichen
	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
	breq	m41210			;ja -> bearbeiten
;
	ldi	zl, low(ascbuf)		;sonst
	ldi	zh, high(ascbuf)	;Zeiger auf ASCII-Puffer
m41200:	ld	r16, y+			;Zeichen aus Parameterspeicher holen
	ld	r17, z+			;Zeichen aus ASCII-Puffer holen
	tst	r16			;Parameter-Ende?
	breq	m41250			;ja -> Datensatz ausgeben
	cp	r16, r17		;Zeichen identisch?
	brne	m41300			;nein -> Ausgabe berspringen
	dec	r18			;alle Zeichen verglichen?
	brne	m41200			;nein -> Schleife
	rjmp	m41250
;
m41210:	ldi	zl, low(ascbuf + 8)
	ldi	zh, high(ascbuf + 8)	;Zeiger auf ASCII-Puffer (Tag-Einer)
	ldi	yl, low(pcpara + 8)
	ldi	yh, high(pcpara + 8)	;Zeiger auf Parameterspeicher (Ende)
	clc				;bertrag lschen
m41220:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m41240			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m41220			;nein -> Schleife
	rjmp	m41250			;sonst Datensatz ausgeben
;
m41230:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m41240:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m41230			;nein -> Schleife
	brcs	m41300			;Param > Puffer? -> keine Ausgabe
;
m41250:	call	xtrse0			;Sekunde 00 ergnzen
	ldi	yl, low(xerbuf + 4)
	ldi	yh, high(xerbuf + 4)	;Zeiger auf Datensatz (Datenbereich)
	call	xtrdat			;Datensatz extrahieren
	call	senasc			;ASCII-Puffer senden
;
m41300:	ldi	r16, low(xs1sta + xs1len)
	ldi	r17, high(xs1sta + xs1len)
	ldi	r18, byte3(xs1sta + xs1len)     ;End-Adresse 15-min-Werte laden
	sub	r20, r16
	sbc	r21, r17		;aktuelle Adresse noch im
	sbc	r22, r18		;gltigen Speicherbereich?
	brcs	m41310			;ja -> weiter
	ldi	r16, low(xs1sta)
	ldi	r17, high(xs1sta)	;sonst
	ldi	r18, byte3(xs1sta)	;Adresse wieder auf Anfang setzen
	sts	pccpnt, r16
	sts	pccpnt + 1, r17		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r18		;speichern
;
m41310:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r16, xes1po
	lds	r17, xes1po + 1		;aktuellen EEPROM-Zeiger auf 15min-Daten holen
	lds	r18, xes1po + 2		;= End-Adresse fr Datenausgabe
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	m41400			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
m41400:	rjmp	m50000			;Ende
;
; Kommando 4 bearbeiten (24h-Werte senden, Minimal-Werte)
;
m42000:	cpi	r16, 4			;Kommando 4 (Minmal-Werte)?
	breq	m42010			;ja -> ausfhren
	rjmp	m43000			;sonst nchstes Kommando
;
m42010:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicher-Fehler?
	brtc	m42020			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
m42020:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r24, pcccnt
	lds	r25, pcccnt + 1		;Datensatz-Zhler holen
	mov	r16, r24
	or	r16, r25		;Datensatz-Zhler = 0?
        breq    m42030                  ;ja -> Suchmodus ermitteln
	rjmp	m42140			;sonst nchsten Datensatz bearbeiten
;
m42030:	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
        breq    m42040                  ;ja -> intelligente Suche beginnen
        rjmp    m42120                  ;sonst Standard-Suche beginnen
;
m42040:	lds	r20, xes2po             ;intelligente Suche beginnen
	lds	r21, xes2po + 1
	lds	r22, xes2po + 2		;aktuellen Zeiger der 24h-Werte holen
        ldi     r16, 9                  ;maximal 9 Lesetests im Abstand von
        mov     r12, r16                ;40 Datenstzen durchfhren
;
m42050: subi    r20, low(40 * xes2da)
        sbci    r21, high(40 * xes2da)  ;Zeigerposition 40 Datenstze
        sbci    r22, byte3(40 * xes2da) ;in die Vergangenheit setzen
        ldi     r16, low(xs2sta)
        ldi     r17, high(xs2sta)
        ldi     r18, byte3(xs2sta)      ;Startadresse 24h-Werte laden
        cp      r20, r16
        cpc     r21, r17
        cpc     r22, r18                ;Startadresse unterschritten?
        brpl    m42060                  ;nein -> weiter
        subi    r20, low(-xs2len)
        sbci    r21, high(-xs2len)      ;sonst
        sbci    r22, byte3(-xs2len)     ;Adresse korrigieren
;
m42060:	ldi	r23, xes2da		;145 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m42070			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
;
m42070: subi    r20, low(xes2da)
        sbci    r21, high(xes2da)       ;Zeigeradresse wieder auf den
        sbci    r22, byte3(xes2da)      ;Stand vor dem Lesen setzen

	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m42130			;nein -> Standard-Suche fortsetzen
;
	call	xtrtim			;sonst Zeit extrahieren
        ldi     r18, 8                  ;8 Zeichen vergleichen
	ldi	zl, low(ascbuf + 8)
	ldi	zh, high(ascbuf + 8)	;Zeiger auf ASCII-Puffer (Tag-Einer)
	ldi	yl, low(pcpara + 8)
	ldi	yh, high(pcpara + 8)	;Zeiger auf Parameterspeicher (Ende)
	clc				;bertrag lschen
m42080:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m42100			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m42080			;nein -> Schleife
        rjmp    m42110                  ;sonst Treffer (nchster Lesetest)
;
m42090:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m42100:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m42090			;nein -> Schleife
	brcs	m42130			;Param > Puffer? -> Standard-Suche fortsetzen
;
m42110:	dec     r12                     ;alle Lesetests ausgefhrt?
        brne    m42050                  ;nein -> Schleife, sonst Standard-Suche beginnen
;
m42120:	lds	r20, xes2po		;sonst
	lds	r21, xes2po + 1		;aktuellen Zeiger der 24h-Werte holen,
	lds	r22, xes2po + 2		;zeigt auf ltesten Datensatz
m42130:	sts	pccpnt, r20
	sts	pccpnt + 1, r21
	sts	pccpnt + 2, r22		;EEPROM-Zeigeradresse setzen
;
m42140:	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m42150			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
m42150:	ldi	r23, xes2da		;145 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m42160			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
m42160:	sts	pccpnt, r20
	sts	pccpnt + 1, r21
	sts	pccpnt + 2, r22		;neue EEPROM-Zeigeradresse speichern
	adiw	r24, 1			;Datensatz-Zhler erhhen
	sts	pcccnt, r24
	sts	pcccnt + 1, r25		;Datensatz-Zhler speichern
;
; Kommando 4 bearbeiten: Daten aufbereiten und ausgeben
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m42300			;nein -> Ausgabe berspringen
;
	call	xtrtim			;sonst Zeit extrahieren
	ldi	yl, low(pcpara)
	ldi	yh, high(pcpara)	;Zeiger auf Parameterspeicher
	ld	r16, y			;erstes Zeichen vom Parameter
	tst	r16			;Parameter gltig?
	breq	m42250			;nein -> Datensatz ausgeben
;
	ldi	r18, 8			;8 Zeichen vergleichen
	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
	breq	m42210			;ja -> bearbeiten
;
	ldi	zl, low(ascbuf)
	ldi	zh, high(ascbuf)	;Zeiger auf ASCII-Puffer
m42200:	ld	r16, y+			;Zeichen aus Parameterspeicher holen
	ld	r17, z+			;Zeichen aus ASCII-Puffer holen
	tst	r16			;Parameter-Ende?
	breq	m42250			;ja -> Datensatz ausgeben
	cp	r16, r17		;Zeichen identisch?
	brne	m42300			;nein -> Ausgabe berspringen
	dec	r18			;alle Zeichen verglichen?
	brne	m42200			;nein -> Schleife
	rjmp	m42250			;sonst Datensatz ausgeben
;
m42210:	ldi	zl, low(ascbuf + 8)	;Zeiger auf ASCII-Puffer
	ldi	zh, high(ascbuf + 8)	;(Tag-Einer)
	ldi	yl, low(pcpara + 8)	;Zeiger auf Parameterspeicher
	ldi	yh, high(pcpara + 8)	;(Ende)
	clc				;bertrag lschen
m42220:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m42240			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m42220			;nein -> Schleife
	rjmp	m42250			;sonst Datensatz ausgeben
;
m42230:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m42240:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m42230			;nein -> Schleife
	brcs	m42300			;Param > Puffer? -> keine Ausgabe
;
m42250:	call	xtrse0			;Sekunde 00 ergnzen
	ldi	yl, low(xerbuf + 4)
	ldi	yh, high(xerbuf + 4)	;Zeiger auf Datensatz (Datenbereich Minimal-Werte)
	call	xtrdat			;Datensatz extrahieren
	call	senasc			;ASCII-Puffer senden
;
m42300:	ldi	r16, low(xs2sta + xs2len)
	ldi	r17, high(xs2sta + xs2len)
	ldi	r18, byte3(xs2sta + xs2len)     ;End-Adresse 24h-Werte laden
	sub	r20, r16
	sbc	r21, r17		;aktuelle Adresse noch im
	sbc	r22, r18		;gltigen Speicherbereich?
	brcs	m42310			;ja -> weiter
	ldi	r16, low(xs2sta)
	ldi	r17, high(xs2sta)	;sonst
	ldi	r18, byte3(xs2sta)	;Adresse wieder auf Anfang setzen
	sts	pccpnt, r16
	sts	pccpnt + 1, r17		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r18		;speichern
;
m42310:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r16, xes2po
	lds	r17, xes2po + 1		;aktuellen EEPROM-Zeiger auf 24h-Werte
	lds	r18, xes2po + 2		;= End-Adresse fr Datenausgabe
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	m42400			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
m42400:	rjmp	m50000			;Ende
;
; Kommando 5 bearbeiten (24h-Werte senden, Maximal-Werte)
;
m43000:	cpi	r16, 5			;Kommando 5 (Maximal-Werte)?
	breq	m43010			;ja -> ausfhren
	rjmp	m44000			;sonst nchstes Kommando
;
m43010:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicher-Fehler?
	brtc	m43020			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
m43020:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r24, pcccnt
	lds	r25, pcccnt + 1		;Datensatz-Zhler holen
	mov	r16, r24
	or	r16, r25		;Datensatz-Zhler = 0?
        breq    m43030                  ;ja -> Suchmodus ermitteln
	rjmp	m43140			;sonst nchsten Datensatz bearbeiten
;
m43030:	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
        breq    m43040                  ;ja -> intelligente Suche beginnen
        rjmp    m43120                  ;sonst Standard-Suche beginnen
;
m43040:	lds	r20, xes2po             ;intelligente Suche beginnen
	lds	r21, xes2po + 1
	lds	r22, xes2po + 2		;aktuellen Zeiger der 24h-Werte holen
        ldi     r16, 9                  ;maximal 9 Lesetests im Abstand von
        mov     r12, r16                ;40 Datenstzen durchfhren
;
m43050: subi    r20, low(40 * xes2da)
        sbci    r21, high(40 * xes2da)  ;Zeigerposition 40 Datenstze
        sbci    r22, byte3(40 * xes2da) ;in die Vergangenheit setzen
        ldi     r16, low(xs2sta)
        ldi     r17, high(xs2sta)
        ldi     r18, byte3(xs2sta)      ;Startadresse 24h-Werte laden
        cp      r20, r16
        cpc     r21, r17
        cpc     r22, r18                ;Startadresse unterschritten?
        brpl    m43060                  ;nein -> weiter
        subi    r20, low(-xs2len)
        sbci    r21, high(-xs2len)      ;sonst
        sbci    r22, byte3(-xs2len)     ;Adresse korrigieren
;
m43060:	ldi	r23, xes2da		;145 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m43070			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
;
m43070: subi    r20, low(xes2da)
        sbci    r21, high(xes2da)       ;Zeigeradresse wieder auf den
        sbci    r22, byte3(xes2da)      ;Stand vor dem Lesen setzen
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m43130			;nein -> Standard-Suche fortsetzen
;
	call	xtrtim			;sonst Zeit extrahieren
        ldi     r18, 8                  ;8 Zeichen vergleichen
	ldi	zl, low(ascbuf + 8)
	ldi	zh, high(ascbuf + 8)	;Zeiger auf ASCII-Puffer (Tag-Einer)
	ldi	yl, low(pcpara + 8)
	ldi	yh, high(pcpara + 8)	;Zeiger auf Parameterspeicher (Ende)
	clc				;bertrag lschen
m43080:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m43100			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m43080			;nein -> Schleife
        rjmp    m43110                  ;sonst Treffer (nchster Lesetest)
;
m43090:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m43100:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m43090			;nein -> Schleife
	brcs	m43130			;Param > Puffer? -> Standard-Suche fortsetzen
;
m43110:	dec     r12                     ;alle Lesetests ausgefhrt?
        brne    m43050                  ;nein -> Schleife, sonst Standard-Suche beginnen
;
m43120:	lds	r20, xes2po		;sonst
	lds	r21, xes2po + 1		;aktuellen Zeiger der 24h-Werte holen,
	lds	r22, xes2po + 2		;zeigt auf ltesten Datensatz
m43130:	sts	pccpnt, r20
	sts	pccpnt + 1, r21
	sts	pccpnt + 2, r22		;EEPROM-Zeigeradresse setzen
;
m43140:	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m43150			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
m43150:	ldi	r23, xes2da		;145 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m43160			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
m43160:	sts	pccpnt, r20
	sts	pccpnt + 1, r21		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r22		;speichern
	adiw	r24, 1			;Datensatz-Zhler erhhen
	sts	pcccnt, r24
	sts	pcccnt + 1, r25		;Datensatz-Zhler speichern
;
; Kommando 5 bearbeiten: Daten aufbereiten und ausgeben
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m43300			;nein -> Ausgabe berspringen
;
	call	xtrtim			;sonst Zeit extrahieren
	ldi	yl, low(pcpara)
	ldi	yh, high(pcpara)	;Zeiger auf Parameterspeicher
	ld	r16, y			;erstes Zeichen vom Parameter
	tst	r16			;Parameter gltig?
	breq	m43250			;nein -> Datensatz ausgeben
;
	ldi	r18, 8			;8 Zeichen vergleichen
	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
	breq	m43210			;ja -> bearbeiten
;
	ldi	zl, low(ascbuf)		;sonst
	ldi	zh, high(ascbuf)	;Zeiger auf ASCII-Puffer
m43200:	ld	r16, y+			;Zeichen aus Parameterspeicher holen
	ld	r17, z+			;Zeichen aus ASCII-Puffer holen
	tst	r16			;Parameter-Ende?
	breq	m43250			;ja -> Datensatz ausgeben
	cp	r16, r17		;Zeichen identisch?
	brne	m43300			;nein -> Ausgabe berspringen
	dec	r18			;alle Zeichen verglichen?
	brne	m43200			;nein -> Schleife
	rjmp	m43250
;
m43210:	ldi	zl, low(ascbuf + 8)	;Zeiger auf ASCII-Puffer
	ldi	zh, high(ascbuf + 8)	;(Tag-Einer)
	ldi	yl, low(pcpara + 8)	;Zeiger auf Parameterspeicher
	ldi	yh, high(pcpara + 8)	;(Ende)
	clc				;bertrag lschen
m43220:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m43240			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m43220			;nein -> Schleife
	rjmp	m43250			;sonst Datensatz ausgeben
;
m43230:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m43240:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m43230			;nein -> Schleife
	brcs	m43300			;Param > Puffer? -> keine Ausgabe
;
m43250:	call	xtrse0				;Sekunde 00 ergnzen
	ldi	yl, low(xerbuf + 4 + 47)
	ldi	yh, high(xerbuf + 4 + 47)	;Zeiger auf Datensatz (Datenbereich Maximalwerte)
	call	xtrdat				;Datensatz extrahieren
	call	senasc				;ASCII-Puffer senden
;
m43300:	ldi	r16, low(xs2sta + xs2len)
	ldi	r17, high(xs2sta + xs2len)
	ldi	r18, byte3(xs2sta + xs2len)	;End-Adresse 24h-Werte laden
	sub	r20, r16
	sbc	r21, r17		;aktuelle Adresse noch im
	sbc	r22, r18		;gltigen Speicherbereich?
	brcs	m43310			;ja -> weiter
	ldi	r16, low(xs2sta)
	ldi	r17, high(xs2sta)	;sonst Adresse wieder auf
	ldi	r18, byte3(xs2sta)	;Anfang setzen
	sts	pccpnt, r16
	sts	pccpnt + 1, r17		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r18		;speichern
;
m43310:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r16, xes2po
	lds	r17, xes2po + 1		;aktuellen EEPROM-Zeiger auf 24h-Werte
	lds	r18, xes2po + 2		;= End-Adresse fr Datenausgabe
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	m43400			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
m43400:	rjmp	m50000			;Ende
;
; Kommando 6 bearbeiten (24h-Werte senden, Mittelwerte)
;
m44000:	cpi	r16, 6			;Kommando 6 (Mittelwerte)?
	breq	m44010			;ja -> ausfhren
	rjmp	m45000			;sonst nchstes Kommando
;
m44010:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicher-Fehler?
	brtc	m44020			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
m44020:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r24, pcccnt
	lds	r25, pcccnt + 1		;Datensatz-Zhler holen
	mov	r16, r24
	or	r16, r25		;Datensatz-Zhler = 0?
        breq    m44030                  ;ja -> Suchmodus ermitteln
	rjmp	m44140			;sonst nchsten Datensatz bearbeiten
;
m44030:	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
        breq    m44040                  ;ja -> intelligente Suche beginnen
        rjmp    m44120                  ;sonst Standard-Suche beginnen
;
m44040:	lds	r20, xes2po             ;intelligente Suche beginnen
	lds	r21, xes2po + 1
	lds	r22, xes2po + 2		;aktuellen Zeiger der 24h-Werte holen
        ldi     r16, 9                  ;maximal 9 Lesetests im Abstand von
        mov     r12, r16                ;40 Datenstzen durchfhren
;
m44050: subi    r20, low(40 * xes2da)
        sbci    r21, high(40 * xes2da)  ;Zeigerposition 40 Datenstze
        sbci    r22, byte3(40 * xes2da) ;in die Vergangenheit setzen
        ldi     r16, low(xs2sta)
        ldi     r17, high(xs2sta)
        ldi     r18, byte3(xs2sta)      ;Startadresse 24h-Werte laden
        cp      r20, r16
        cpc     r21, r17
        cpc     r22, r18                ;Startadresse unterschritten?
        brpl    m44060                  ;nein -> weiter
        subi    r20, low(-xs2len)
        sbci    r21, high(-xs2len)      ;sonst
        sbci    r22, byte3(-xs2len)     ;Adresse korrigieren
;
m44060:	ldi	r23, xes2da		;145 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m44070			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m44200			;Kommando beenden
;
m44070: subi    r20, low(xes2da)
        sbci    r21, high(xes2da)       ;Zeigeradresse wieder auf den
        sbci    r22, byte3(xes2da)      ;Stand vor dem Lesen setzen
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m44130			;nein -> Standard-Suche fortsetzen
;
	call	xtrtim			;sonst Zeit extrahieren
        ldi     r18, 8                  ;8 Zeichen vergleichen
	ldi	zl, low(ascbuf + 8)
	ldi	zh, high(ascbuf + 8)	;Zeiger auf ASCII-Puffer (Tag-Einer)
	ldi	yl, low(pcpara + 8)
	ldi	yh, high(pcpara + 8)	;Zeiger auf Parameterspeicher (Ende)
	clc				;bertrag lschen
m44080:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m44100			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m44080			;nein -> Schleife
        rjmp    m44110                  ;sonst Treffer (nchster Lesetest)
;
m44090:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m44100:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m44090			;nein -> Schleife
	brcs	m44130			;Param > Puffer? -> Standard-Suche fortsetzen
;
m44110:	dec     r12                     ;alle Lesetests ausgefhrt?
        brne    m44050                  ;nein -> Schleife, sonst Standard-Suche beginnen
;
m44120:	lds	r20, xes2po		;sonst
	lds	r21, xes2po + 1		;aktuellen Zeiger der 24h-Werte holen,
	lds	r22, xes2po + 2		;zeigt auf ltesten Datensatz
m44130:	sts	pccpnt, r20
	sts	pccpnt + 1, r21
	sts	pccpnt + 2, r22		;EEPROM-Zeigeradresse setzen
;
m44140:	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m44150			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
m44150:	ldi	r23, xes2da		;145 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m44160			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
m44160:	sts	pccpnt, r20
	sts	pccpnt + 1, r21		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r22		;speichern
	adiw	r24, 1			;Datensatz-Zhler erhhen
	sts	pcccnt, r24
	sts	pcccnt + 1, r25		;Datensatz-Zhler speichern
;
; Kommando 6 bearbeiten: Daten aufbereiten und ausgeben
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m44300			;nein -> Ausgabe berspringen
;
	call	xtrtim			;sonst Zeit extrahieren
	ldi	yl, low(pcpara)
	ldi	yh, high(pcpara)	;Zeiger auf Parameterspeicher
	ld	r16, y			;erstes Zeichen vom Parameter
	tst	r16			;Parameter gltig?
	breq	m44250			;nein -> Datensatz ausgeben
;
	ldi	r18, 8			;8 Zeichen vergleichen
	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
	breq	m44210			;ja -> bearbeiten
;
	ldi	zl, low(ascbuf)		;sonst
	ldi	zh, high(ascbuf)	;Zeiger auf ASCII-Puffer
m44200:	ld	r16, y+			;Zeichen aus Parameterspeicher holen
	ld	r17, z+			;Zeichen aus ASCII-Puffer holen
	tst	r16			;Parameter-Ende?
	breq	m44250			;ja -> Datensatz ausgeben
	cp	r16, r17		;Zeichen identisch?
	brne	m44300			;nein -> Ausgabe berspringen
	dec	r18			;alle Zeichen verglichen?
	brne	m44200			;nein -> Schleife
	rjmp	m44250
;
m44210:	ldi	zl, low(ascbuf + 8)	;Zeiger auf ASCII-Puffer
	ldi	zh, high(ascbuf + 8)	;(Tag-Einer)
	ldi	yl, low(pcpara + 8)	;Zeiger auf Parameterspeicher
	ldi	yh, high(pcpara + 8)	;(Ende)
	clc				;bertrag lschen
m44220:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m44240			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m44220			;nein -> Schleife
	rjmp	m44250			;sonst Datensatz ausgeben
;
m44230:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m44240:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m44230			;nein -> Schleife
	brcs	m44300			;Param > Puffer? -> keine Ausgabe
;
m44250:	call	xtrse0				;Sekunde 00 ergnzen
	ldi	yl, low(xerbuf + 4 + 47 * 2)
	ldi	yh, high(xerbuf + 4 + 47 * 2)	;Zeiger auf Datensatz (Daten-Bereich Mittelwerte)
	call	xtrdat				;Datensatz extrahieren
	call	senasc				;ASCII-Puffer senden
;
m44300:	ldi	r16, low(xs2sta + xs2len)
	ldi	r17, high(xs2sta + xs2len)
	ldi	r18, byte3(xs2sta + xs2len)	;End-Adresse 24h-Werte laden
	sub	r20, r16
	sbc	r21, r17		;aktuelle Adresse noch im
	sbc	r22, r18		;gltigen Speicherbereich?
	brcs	m44310			;ja -> weiter
	ldi	r16, low(xs2sta)
	ldi	r17, high(xs2sta)	;sonst Adresse wieder auf
	ldi	r18, byte3(xs2sta)	;Anfang setzen
	sts	pccpnt, r16
	sts	pccpnt + 1, r17		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r18		;speichern
;
m44310:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r16, xes2po
	lds	r17, xes2po + 1		;aktuellen EEPROM-Zeiger auf 24h-Werte
	lds	r18, xes2po + 2		;= End-Adresse fr Datenausgabe
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	m44400			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
m44400:	rjmp	m50000			;Ende
;
; Kommando 7 bearbeiten (Meldungsdaten senden)
;
m45000:	cpi	r16, 7			;Kommando 7 (Meldungs-Daten)?
	breq	m45010			;ja -> ausfhren
	rjmp	m46000			;sonst nchstes Kommando
;
m45010:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicher-Fehler?
	brtc	m45020			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
m45020:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r24, pcccnt
	lds	r25, pcccnt + 1		;Datensatz-Zhler holen
	mov	r16, r24
	or	r16, r25		;Datensatz-Zhler = 0?
        breq    m45030                  ;ja -> Suchmodus ermitteln
	rjmp	m45140			;sonst nchsten Datensatz bearbeiten
;
m45030:	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
        breq    m45040                  ;ja -> intelligente Suche beginnen
        rjmp    m45120                  ;sonst Standard-Suche beginnen
;
m45040:	lds	r20, xemspo             ;sonst intelligente Suche beginnen
	lds	r21, xemspo + 1
	lds	r22, xemspo + 2		;aktuellen Zeiger der Meldungsdaten holen
        ldi     r16, 3                  ;maximal 3 Lesetests im Abstand von
        mov     r12, r16                ;125 Datenstzen durchfhren
;
m45050: subi    r20, low(125 * xemsda)
        sbci    r21, high(125 * xemsda)         ;Zeigerposition 125 Datenstze
        sbci    r22, byte3(125 * xemsda)        ;in die Vergangenheit setzen
        ldi     r16, low(xmssta)
        ldi     r17, high(xmssta)
        ldi     r18, byte3(xmssta)      ;Startadresse der Meldungsdaten laden
        cp      r20, r16
        cpc     r21, r17
        cpc     r22, r18                ;Startadresse unterschritten?
        brpl    m45060                  ;nein -> weiter
        subi    r20, low(-xmslen)
        sbci    r21, high(-xmslen)      ;sonst
        sbci    r22, byte3(-xmslen)     ;Adresse korrigieren
;
m45060:	ldi	r23, xemsda		;7 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m45070			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
;
m45070: subi    r20, low(xemsda)
        sbci    r21, high(xemsda)       ;Zeigeradresse wieder auf den
        sbci    r22, byte3(xemsda)      ;Stand vor dem Lesen setzen
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	breq	m45130			;nein -> Standard-Suche fortsetzen
;
	call	xtrtim			;sonst Zeit extrahieren
        ldi     r18, 8                  ;8 Zeichen vergleichen
	ldi	zl, low(ascbuf + 8)
	ldi	zh, high(ascbuf + 8)	;Zeiger auf ASCII-Puffer (Tag-Einer)
	ldi	yl, low(pcpara + 8)
	ldi	yh, high(pcpara + 8)	;Zeiger auf Parameterspeicher (Ende)
	clc				;bertrag lschen
m45080:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m45100			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m45080			;nein -> Schleife
        rjmp    m45110                  ;sonst Treffer (nchster Lesetest)
;
m45090:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m45100:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m45090			;nein -> Schleife
	brcs	m45130			;Param > Puffer? -> Standard-Suche fortsetzen
;
m45110:	dec     r12                     ;alle Lesetests ausgefhrt?
        brne    m45050                  ;nein -> Schleife, sonst Standard-Suche beginnen
;
m45120:	lds	r20, xemspo		;sonst aktuellen Zeiger der
	lds	r21, xemspo + 1		;Meldungs-Daten holen,
	lds	r22, xemspo + 2		;zeigt auf ltesten Datensatz
m45130:	sts	pccpnt, r20
	sts	pccpnt + 1, r21
	sts	pccpnt + 2, r22		;EEPROM-Zeigeradresse setzen
;
m45140:	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m45150			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
m45150:	ldi	r23, xemsda		;7 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	m45160			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m40200			;Kommando beenden
m45160:	sts	pccpnt, r20
	sts	pccpnt + 1, r21		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r22		;speichern
	adiw	r24, 1			;Datensatz-Zhler erhhen
	sts	pcccnt, r24
	sts	pcccnt + 1, r25		;Datensatz-Zhler speichern
;
; Kommando 7 bearbeiten: Daten aufbereiten und ausgeben
;
	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
	brne	m45190			;ja -> ausgeben
m45180:	rjmp	m45300			;sonst Ausgabe berspringen
;
m45190:	call	xtrtim			;sonst Zeit extrahieren
	ldi	yl, low(pcpara)
	ldi	yh, high(pcpara)	;Zeiger auf Parameterspeicher
	ld	r16, y			;erstes Zeichen vom Parameter
	tst	r16			;Parameter gltig?
	breq	m45250			;nein -> Datensatz ausgeben
;
	ldi	r18, 8			;8 Zeichen vergleichen
	lds	r16, pcpara + 8		;Parameter-Zusatz holen
	cpi	r16, '-'		;Minuszeichen?
	breq	m45210			;ja -> bearbeiten
;
	ldi	zl, low(ascbuf)		;sonst
	ldi	zh, high(ascbuf)	;Zeiger auf ASCII-Puffer
m45200:	ld	r16, y+			;Zeichen aus Parameterspeicher holen
	ld	r17, z+			;Zeichen aus ASCII-Puffer holen
	tst	r16			;Parameter-Ende?
	breq	m45250			;ja -> Datensatz ausgeben
	cp	r16, r17		;Zeichen identisch?
	brne	m45180			;nein -> Ausgabe berspringen
	dec	r18			;alle Zeichen verglichen?
	brne	m45200			;nein -> Schleife
	rjmp	m45250
;
m45210:	ldi	zl, low(ascbuf + 8)	;Zeiger auf ASCII-Puffer
	ldi	zh, high(ascbuf + 8)	;(Tag-Einer)
	ldi	yl, low(pcpara + 8)	;Zeiger auf Parameterspeicher
	ldi	yh, high(pcpara + 8)	;(Ende)
	clc				;bertrag lschen
m45220:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
	tst	r16			;gltiges Zeichen?
	brne	m45240			;ja -> weiter
	dec	r18			;sonst alle Zeichen bearbeitet?
	brne	m45220			;nein -> Schleife
	rjmp	m45250			;sonst Datensatz ausgeben
;
m45230:	ld	r17, -z			;Zeichen aus ASCII-Puffer holen
	ld	r16, -y			;Zeichen aus Parameterspeicher holen
m45240:	sbc	r17, r16		;Pufferposition - Parameterposition
	dec	r18			;alle Zeichen verglichen?
	brne	m45230			;nein -> Schleife
	brcs	m45300			;Param > Puffer? -> keine Ausgabe
;
m45250:	call	xtrsec			;Sekunden ergnzen
	lds	r17, xerbuf + 5		;Meldungscode holen
        cpi     r17, 0xee               ;Meldungscode < 0xee?
        brcs	m45280			;nein -> keinen String ausgeben
	cpi	r17, 0xfe		;Meldungscode > 0xfd?
	brcc	m45280			;nein -> keinen String ausgeben
	call	xtrfst			;Meldungstext in Puffer legen
	ldi	r16, ';'		;Listentrennzeichen laden
	st	x+, r16			;Zeichen in Puffer legen
	cpi	r17, 0xf7		;Parameter ausgeben?
	brcs	m45280			;nein -> Daten senden
;
	lds	r18, xerbuf + 6		;Parameter holen
	cpi	r17, 0xf9		;Alarmnummer ausgeben?
	brcs	m45260			;ja -> weiter
;
        cpi     r18, 31                 ;Sensornummer > 30?
        brcc    m45280			;ja -> berspringen
	ldi	r19, 9			;9 Zeichen Sensortext
	ldi	r16, low(esentx)	;Zeiger auf EEPROM-Adresse
	ldi	r17, high(esentx)	;der Sensortexte
	rjmp	m45270			;EEPROM-Text ausgeben
;
m45260:	cpi	r18, 4			;Alarmnummer > 3?
	brcc	m45280			;ja -> berspringen
	ldi	r19, 11			;11 Zeichen Alarmtext
	ldi	r16, low(ealmtx)	;Zeiger auf EEPROM-Adresse
	ldi	r17, high(ealmtx)	;der Alarmtexte
;
m45270:	mul	r18, r19		;Adress-Offset ermitteln
	add	r16, r0
	adc	r17, r1			;Tabellenplatz ermitteln
        mov     r23, r18                ;Sensornummer zwischenspeichern
	call	xtrest			;Sensor-/Alarmtext in Puffer
        cpi     r23, 28                 ;Luftfeuchtigkeitssensor?
        brcs    m45280                  ;nein -> Daten senden
        ldi     r16, 'H'                ;sonst zustzlich 'H'
        st      x+, r16                 ;anhngen
;
m45280:	call	xtbyte			;Anzahl der Bytes ermitteln
	call	senasc			;ASCII-Puffer senden
;
m45300:	ldi	r16, low(xmssta + xmslen)
	ldi	r17, high(xmssta + xmslen)
	ldi	r18, byte3(xmssta + xmslen)     ;End-Adresse Meldungs-Daten
	sub	r20, r16
	sbc	r21, r17		;aktuelle Adresse noch im
	sbc	r22, r18		;gltigen Speicherbereich?
	brcs	m45310			;ja -> weiter
	ldi	r16, low(xmssta)
	ldi	r17, high(xmssta)	;sonst Adresse wieder auf
	ldi	r18, byte3(xmssta)	;Anfang setzen
	sts	pccpnt, r16
	sts	pccpnt + 1, r17		;neue EEPROM-Zeigeradresse
	sts	pccpnt + 2, r18		;speichern
;
m45310:	lds	r20, pccpnt
	lds	r21, pccpnt + 1
	lds	r22, pccpnt + 2		;EEPROM-Zeigeradresse holen
	lds	r16, xemspo
	lds	r17, xemspo + 1		;aktuellen EEPROM-Zeiger auf Meldungs-Daten holen
	lds	r18, xemspo + 2		;= End-Adresse fr Datenausgabe
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	m45400			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
m45400:	rjmp	m50000			;Ende
;
; Kommando 8 bearbeiten (Konfiguration senden)
;
m46000:	cpi	r16, 8			;Kommando 8 (Konfiguration)?
	breq	m46010			;ja -> ausfhren
	rjmp	m47000			;sonst nchstes Kommando
;
m46010:	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m46100			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
; Daten aufbereiten und ausgeben, die Datensatznummer bestimmt die auszugebenden Elemente:
; 0-23: Log-Tabelle 1 (L01-L24), 24-26: Log-Tabelle 2 (L25-L27), 27-57: Sensornamen (S01-S31), 58-61: Alarme (AL1-AL4)
;
m46100:	ldi	xl, low(ascbuf)		;zunchst Log-Tabelle 1 ausgeben
	ldi	xh, high(ascbuf)	;Zeiger auf ASCII-Puffer
	lds	r24, pcccnt		;Datensatz-Zhler L holen
	cpi	r24, 24			;Log-Tabelle 1 senden (Wert < 24)?
	brcc	m46150			;nein -> weiter zur Log-Tabelle 2
;
	ldi	yl, low(logtb1)         ;sonst
	ldi	yh, high(logtb1)	;Zeiger auf Log-Tabelle 1
        mov     r18, r24                ;Datensatz-Zhler entspricht logischer Sensornummer
        clr     r19                     ;H-Byte = 0 setzen
        add     yl, r18                 ;Log-Tabellen-Platz ermitteln
        adc     yh, r19                 ;bertrag
;
m46110:	inc	r18			;Sensornummer korrigieren
	ldi	r16, 'L'		;Zeichen 'L'
	st	x+, r16			;in ASCII-Puffer speichern
	call	bytasc			;in zweistellige Dezimalzahl wandeln
	st	x+, r17			;Zehner in ASCII-Puffer
	st	x+, r16			;Einer in ACSII-Puffer
	ldi	r16, ':'		;Doppelpunkt
	st	x+, r16			;in ASCII-Puffer speichern
;
	ld	r18, y+			;Tabelleneintrag holen
        inc     r18                     ;korrigieren
	andi	r18, 0x1f		;Konflikt-Flag lschen
	call	bytasc			;in zweistellige Dezimalzahl wandeln
	st	x+, r17			;Zehner in ASCII-Puffer
	st	x+, r16			;Einer in ACSII-Puffer
	rjmp	m46410			;Daten ausgeben
;
m46150: cpi     r24, 27                 ;Log-Tabelle 2 senden (Wert < 27)?
        brcc    m46200                  ;nein -> weiter zu den Sensornamen
	ldi	yl, low(logtb2)         ;sonst
	ldi	yh, high(logtb2)	;Zeiger auf Log-Tabelle 2
        mov     r18, r24                ;Datensatz-Zhler entspricht logischer Sensornummer + 24
        subi    r18, 24                 ;in Sensornummer wandeln
        clr     r19                     ;H-Byte = 0
        add     yl, r18                 ;Log-Tabellen-Platz ermitteln
        adc     yh, r19                 ;bertrag
        mov     r18, r24                ;Datensatz-Zhler entspricht logischer Sensornummer
        rjmp    m46110                  ;weiter wie bei Log-Tabelle 1
;
m46200:	cpi	r24, 58			;Sensornamen senden (< 58)?
	brcc	m46300			;nein -> weiter zu den Alarmdaten
	ldi	r16, 'S'		;sonst Zeichen 'S'
	st	x+, r16			;in ASCII-Puffer speichern
	mov	r18, r24		;aktuelle Datensatznummer holen
	subi	r18, 26			;in Sensornummer wandeln
	call	bytasc			;in zweistellige Dezimalzahl wandeln
	st	x+, r17			;Zehner in ASCII-Puffer
	st	x+, r16			;Einer in ACSII-Puffer
	ldi	r16, ':'		;Doppelpunkt
	st	x+, r16			;in ASCII-Puffer speichern
;
	ldi	yl,low(senflg)
	ldi	yh,high(senflg)		;Zeiger auf Sensor-Flags
	mov	r18, r24		;aktuelle Datensatznummer holen
        subi    r18, 27                 ;in logische Sensornummer wandeln
	clr	r19                     ;H-Byte = 0
	add	yl, r18
	adc	yh, r19			;Offset addieren
	ld	r16, y			;Sensor-Flags holen
	bst	r16, 7			;Sensor inaktiv?
	brts	m46410			;ja -> keinen Text ausgeben
;
	ldi	r19, 9			;sonst 9 Zeichen Sensortext
	ldi	r16, low(esentx)	;Zeiger auf EEPROM-Adresse
	ldi	r17, high(esentx)	;der Sensortexte
	rjmp	m46400			;Text ausgeben
;
m46300:	ldi	r16, 'A'		;Zeichen 'A'
	st	x+, r16			;in ASCII-Puffer speichern
	ldi	r16, 'L'		;Zeichen 'L'
	st	x+, r16			;in ASCII-Puffer speichern
	mov	r16, r24		;Datensatznummer kopieren
	subi	r16, 57			;Alarmnummer korrigieren
	ori	r16, 0x30		;in ASCII wandeln
	st	x+, r16			;in ACSII-Puffer speichern
	ldi	r16, ':'		;Doppelpunkt
	st	x+, r16			;in ASCII-Puffer speichern
;
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarm-Flags
        mov     r18, r24                ;aktuelle Datensatznummer holen
        subi    r18, 58                 ;in logische Alarmnummer wandeln
	clr	r19                     ;H-Byte = 0
	add	yl, r18
	adc	yh, r19			;Offset addieren
	ld	r16, y			;Alarm-Flags holen
	bst	r16, 7			;Alarm inaktiv?
	brts	m46410			;ja -> keinen Text ausgeben
;
	ldi	r19, 11			;11 Zeichen Alarmtext
	ldi	r16, low(ealmtx)	;Zeiger auf EEPROM-Adresse
	ldi	r17, high(ealmtx)	;der Alarmtexte
;
m46400:	mul	r18, r19		;Adress-Offset ermitteln
	add	r16, r0
	adc	r17, r1			;Tabellenplatz ermitteln
	call	xtrest			;Sensor-/Alarmtext in Puffer
        cpi     r24, 58                 ;Datensatznummer > Luftfeuchtigkeitssensoren?
        brcc    m46410                  ;ja -> weiter
        cpi     r24, 55                 ;Datensatznumemr < Luftfeuchtigkeitssensoren?
        brcs    m46410                  ;ja -> weiter
        ldi     r16, 'H'                ;sonst zustzlich Zeichen 'H'
        st      x+, r16			;in ASCII-Puffer speichern
;
m46410:	call	xtbyte			;Anzahl der Bytes ermitteln
	call	senasc			;ASCII-Puffer senden
;
	lds	r24, pcccnt		;Datensatz-Zhler L holen
	inc	r24			;Datensatz-Zhler L erhhen
	sts	pcccnt, r24		;Datensatz-Zhler L speichern
	cpi	r24, 62			;Log-, Sensor- und Alarmdaten gesendet?
	brcs	m46500			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
m46500:	rjmp	m50000			;Ende
;
; Kommando 9 bearbeiten (LCD-Bildschirminhalt senden)
;
m47000:	cpi	r16, 9			;Kommando 9 (Bildinhalt)?
	breq	m47010			;ja -> ausfhren
	rjmp	m48000			;sonst nchstes Kommando
;
m47010:	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Sendepuffer Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Sendepuffer Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Sendepuffer leer?
	breq	m47100			;ja -> weiter
	rjmp	m50000			;sonst Ende
;
; Eine Pixelzeile aus LCD auslesen, Daten aufbereiten und ausgeben
;
m47100:	ldi	xl, low(ascbuf)
	ldi	xh, high(ascbuf)	;Zeiger auf ASCII-Puffer
	lds	r19, pcccnt		;Datensatz-Zhler L holen (Pixelzeilenzhler)
	ldi	r18, 30			;30 Bytes aus LCD lesen (240 Pixel)
        mul     r18, r19                ;mit Pixelzeilenzhler multiplizieren (Adresse ermitteln)
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r0		        ;Adresse L-Byte kopieren
	call	outlcd			;Byte an LCD ausgeben
	mov	r17, r1			;Adresse H-Byte kopieren
	call	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	call	outlcd			;Byte an LCD ausgeben
;
m47110:	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc1		;Kommando "Data Read Increment"
	call	outlcd			;Byte an LCD ausgeben
	call	in_lcd			;Datenbyte aus LCD auslesen
        mov     r16, r17                ;kopieren
        clr     r17                     ;H-Byte lschen
        call    bytas4                  ;Wert in 4 Byte ASCII wandeln
;
	mov	r16, r5			;Hunderterstelle holen
	cpi	r16, '0'		;Hunderterstelle = 0?
	breq	m47120			;ja -> berspringen
	st	x+, r16			;sonst im Puffer speichern
	mov	r16, r6			;Zehnerstelle holen
	rjmp	m47130			;direkt speichern
m47120:	mov	r16, r6			;Zehnerstelle holen
	cpi	r16, '0'		;Zehnerstelle = 0?
	breq	m47140			;ja -> berspringen
m47130:	st	x+, r16			;sonst im Puffer speichern
m47140:	mov	r16, r7			;Einerstelle holen
	st	x+, r16			;im Puffer speichern
;
        cpi     r18, 1                  ;letzte Pixelzeile?
        breq    m47150                  ;ja -> berspringen
        ldi     r16,';'                 ;sonst Trennzeichen ';'
	st	x+, r16			;im Puffer speichern
;
m47150:	dec	r18			;alle Datenbytes gelesen?
	brne	m47110			;nein -> Schleife
	call	xtbyte			;Anzahl der Bytes ermitteln
	call	senasc			;ASCII-Puffer senden
;
	lds	r19, pcccnt		;Datensatz-Zhler L holen
	inc	r19			;Datensatz-Zhler L erhhen
	sts	pcccnt, r19		;Datensatz-Zhler L speichern
	cpi	r19, 128		;alle Pixelzeilen gesendet?
	brcs	m50000			;nein -> weiter
	rjmp	m40200			;sonst Kommando beenden
;
; Weitere Kommandos bearbeiten
;
m48000: nop
;
; DCF77-Auswertung in mehreren Stufen
;
m50000:	lds	r16, dcflag		;sonst DCF77-Flags holen
	bst	r16, 3			;Beginn einer neuen Minute?
	brts	m50010			;ja -> auswerten
	rjmp	m51000			;sonst zum nchsten Abschnitt
m50010:	cbr	r16, 0x08		;Minuten-Flag lschen
	sts	dcflag, r16		;DCF77-Flags wieder speichern
;
; DCF77-Auswertung: Parittsprfung fr Minuten, Stunden und Datum
;
	clr	r19			;Fehlerzhler lschen
	clr	r21			;fr Parittsberechnung
	lds	r22, dcfimp		;DCF77-Impulszhler holen
	sts	dcfimp, r21		;DCF77-Impulszhler lschen
	cpi	r22, 59			;sind 59 Impulse eingegangen?
	breq	m50100			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
	rjmp	m50500			;Auswertung vorzeitig beenden
;
m50100:	clr	r22			;Zhler fr Parittsberechnung
	lds	r23, dcftab + 1		;Byte 1 aus Tabelle holen
	ldi	r20, 6			;6 Minuten-Bits auswerten
m50110:	rol	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry = 1)
	dec	r20			;alle Bits ausgewertet?
	brne	m50110			;nein -> Schleife
	lds	r23, dcftab + 2		;Byte 2 aus Tabelle holen
	ror	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry = 1)
	ror	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry = 1)
	bst	r22, 0			;war das Ergebnis geradzahlig?
	brtc	m50120			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
;
m50120:	clr	r22			;Zhler fr Parittsberechnung
	ldi	r20, 6			;6 Stunden-Bits auswerten
m50130:	ror	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry=1)
	dec	r20			;alle Bits ausgewertet?
	brne	m50130			;nein -> Schleife
	lds	r23, dcftab + 3		;Byte 3 aus Tabelle holen
	ror	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry = 1)
	bst	r22, 0			;war das Ergebnis geradzahlig?
	brtc	m50140			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
;
m50140:	clr	r22			;Zhler fr Parittsberechnung
	ldi	r20, 7			;7 Datum-Bits auswerten
m50150:	ror	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry = 1)
	dec	r20			;alle Bits ausgewertet?
	brne	m50150			;nein -> Schleife
	lds	r23, dcftab + 4		;Byte 4 aus Tabelle holen
	ldi	r20, 8			;weitere 8 Datum-Bits auswerten
m50160:	ror	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry = 1)
	dec	r20			;alle Bits ausgewertet?
	brne	m50160			;nein -> Schleife
	lds	r23,dcftab + 5		;Byte 5 aus Tabelle holen
	ldi	r20, 8			;weitere 8 Datum-Bits auswerten
m50170:	ror	r23			;relevantes Bit in Carry
	adc	r22, r21		;Carry addieren (wenn Carry = 1)
	dec	r20			;alle Bits ausgewertet?
	brne	m50170			;nein -> Schleife
	bst	r22, 0			;war das Ergebnis geradzahlig?
	brtc	m50180			;ja -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
m50180:	tst	r19			;Parittsfehler aufgetreten?
	breq	m50200			;nein -> weiter, sonst
	rjmp	m50500			;Auswertung vorzeitig beenden
;
; DCF77-Auswertung: Ablegen der Zeitinformation in Puffer 1
;
m50200:	ldi	zl, low(dcbuf1)
	ldi	zh, high(dcbuf1)	;Zeiger auf DCF77-Puffer 1
	lds	r23, dcftab		;Byte 0 aus Tabelle holen
	rol	r23
	rol	r23			;Bit 6 auf Bitposition 0
	rol	r23			;schieben
	andi	r23, 0x01		;relevantes Bit filtern und
	st	z+, r23			;als Zeitzone speichern
	lds	r23, dcftab + 1		;Byte 1 aus Tabelle holen
	ror	r23			;Bits 2-5 auf Bitposition 0-3
	ror	r23			;schieben
	andi	r23, 0x0f		;relevante Bits filtern und
	st	z+, r23			;als Minuten Einer speichern
	lds	r23, dcftab + 1		;Byte 1 aus Tabelle holen
	lds	r22, dcftab + 2		;Byte 2 aus Tabelle holen
	rol	r23
	rol	r22			;Bits 6-7 aus r23 und Bit 0
	rol	r23			;aus r22 auf Bitposition 0-2
	rol	r22			;von r22 schieben
	andi	r22, 0x07		;relevante Bits filtern und
	st	z+, r22			;als Minuten Zehner speichern
	lds	r23, dcftab + 2		;Byte 2 aus Tabelle holen
	ror	r23			;Bits 2-5 auf Bitposition 0-3
	ror	r23			;schieben
	andi	r23, 0x0f		;relevante Bits filtern und
	st	z+, r23			;als Stunden Einer speichern
	lds	r23, dcftab + 2		;Byte 2 aus Tabelle holen
	rol	r23
	rol	r23			;Bits 6-7 auf Bitposition 0-1
	rol	r23			;schieben
	andi	r23, 0x03		;relevante Bits filtern und
	st	z+, r23			;als Stunden Zehner speichern
	lds	r23, dcftab + 3		;Byte 3 aus Tabelle holen
	ror	r23			;Bits 1-4 auf Bits 0-3 schieben
	andi	r23, 0x0f		;relevante Bits filtern und als
	st	z+, r23			;Kalendertag Einer speichern
	lds	r23, dcftab + 3		;Byte 3 aus Tabelle holen
	ror	r23			;Bits 5-6 auf Bits 4-5 schieben
	swap	r23			;Nibbles tauschen
	andi	r23, 0x03		;relevante Bits filtern und als
	st	z+, r23			;Kalendertag Zehner speichern
	lds	r23, dcftab + 3		;Byte 3 aus Tabelle holen
	rol	r23			;Bit 7 in Carry schieben
	lds	r23, dcftab + 4		;Byte 4 aus Tabelle holen
	rol	r23			;Carry in Bit 0 schieben
	andi	r23, 0x07		;relevante Bits filtern und
	st	z+, r23			;als Wochentag speichern
	lds	r23, dcftab + 4		;Byte 4 aus Tabelle holen
	ror	r23			;Bits 2-5 auf Bitposition 0-3
	ror	r23			;schieben
	andi	r23, 0x0f		;relevante Bits filtern und
	st	z+, r23			;als Monat Einer speichern
	lds	r23, dcftab + 4		;Byte 4 aus Tabelle holen
	rol	r23
	rol	r23			;Bit 6 auf Bitposition 0
	rol	r23			;schieben
	andi	r23, 0x01		;relevantes Bit filtern und
	st	z+, r23			;als Monat Zehner speichern
	lds	r23, dcftab + 4		;Byte 4 aus Tabelle holen
	rol	r23			;Bit 7 in Carry schieben
	lds	r23, dcftab + 5		;Byte 5 aus Tabelle holen
	rol	r23			;Carry in Bit 0 schieben
	andi	r23, 0x0f		;relevante Bits filtern und
	st	z+, r23			;als Jahr Einer speichern
	lds	r23, dcftab + 5		;Byte 5 aus Tabelle holen
	rol	r23			;Bits 3-6 in Bits 4-7 schieben
	swap	r23			;Nibbles tauschen
	andi	r23, 0x0f		;relevante Bits filtern und
	st	z+, r23			;als Jahr Zehner speichern
;
; DCF77-Auswertung: Vergleich der aktuellen Zeit mit der in den beiden vorherigen Zyklen ermittelten Zeit, gleichzeitig Puffer 2 nach Puffer 3 und Puffer 1
; nach Puffer 2 kopieren
;
	ldi	yl, low(dcbuf1)
	ldi	yh, high(dcbuf1)	;Zeiger auf DCF77-Puffer 1
	ldi	zl, low(dcbuf2)
	ldi	zh, high(dcbuf2)	;Zeiger auf DCF77-Puffer 2
	ldi	xl, low(dcbuf3)
	ldi	xh, high(dcbuf3)	;Zeiger auf DCF77-Puffer 3
	clr	r18			;Nullbyte-Detektor lschen
	ldi	r23, 12			;12 Bytes vergleichen
m50300:	ld	r22, y+			;Puffer 1 Inhalt holen
	or	r18, r22		;Nullbyte-Detektor bearbeiten
	ld	r21, z			;Puffer 2 Inhalt holen
	ld	r20, x			;Puffer 3 Inhalt holen
	st	x+, r21			;Puffer 2 nach 3 kopieren
	st	z+, r22			;Puffer 1 nach 2 kopieren
	cp	r21, r22		;Inhalt Puffer 1 und Puffer 2 identisch?
	brne	m50310			;nein -> Fehler
	cp	r20, r21		;Inhalt Puffer 2 und Puffer 3 identisch?
	breq	m50320			;ja -> weiter
m50310:	inc	r19			;sonst Fehlerzhler erhhen
m50320:	dec	r23			;alle Bytes bearbeitet?
	brne	m50300			;nein -> Schleife
	tst	r18			;alles Nullbytes im Puffer?
	brne	m50400			;nein -> weiter
	inc	r19			;sonst Fehlerzhler erhhen
;
; DCF77-Auswertung: Puffer 2 und 3 um eine Minute erhhen
;
m50400:	ldi	zl, low(dcbuf2)
	ldi	zh, high(dcbuf2)	;Zeiger auf DCF77-Puffer 2
	call	incdcb			;um eine Minute erhhen
	ldi	zl, low(dcbuf3)
	ldi	zh, high(dcbuf3)	;Zeiger auf DCF77-Puffer 3
	call	incdcb			;um eine Minute erhhen
;
; DCF77-Auswertung: DCF77-Tabelle lschen
;
m50500:	ldi	zl, low(dcftab)		;Zeiger auf DCF77-Tabelle
	ldi	zh, high(dcftab)
	ldi	r23, 6			;6 Bytes bearbeiten
	clr	r22			;Nullwert
m50510:	st	z+, r22			;Tabellenplatz lschen
	dec	r23			;alle Pltze bearbeitet?
	brne	m50510			;nein -> Schleife
	tst	r19			;DCF77 ok?
	breq	m50600			;ja -> Uhr stellen
	rjmp	m51000			;sonst zum nchsten Abschnitt
;
; DCF77-Auswertung: aktuelle Zeit in den Uhrenbereich kopieren und Uhr synchronisieren
;
m50600:	cli				;Int vorbergehend sperren
	lds	r23, dcbuf1		;Zeitzone holen
	sts	tzone, r23		;und speichern
	lds	r23, dcbuf1 + 5		;Kalendertag Einer holen
	sts	dayl, r23		;und speichern
	lds	r23, dcbuf1 + 6		;Kalendertag Zehner holen
	sts	dayh, r23		;und speichern
	lds	r23, dcbuf1 + 7		;Wochentag holen
	dec	r23			;korrigieren (1-7 -> 0-6)
	sts	dayw, r23		;und speichern
	lds	r23, dcbuf1 + 8		;Monat Einer holen
	sts	monthl, r23		;und speichern
	lds	r23, dcbuf1 + 9		;Monat Zehner holen
	sts	monthh, r23		;und speichern
	lds	r23, dcbuf1 + 10	;Jahr Einer holen
	sts	yearl, r23		;und speichern
	lds	r23, dcbuf1 + 11	;Jahr Zehner holen
	sts	yearh, r23		;und speichern
	lds	r23, dcbuf1 + 1		;Minuten Einer holen
	sts	minutl, r23		;und speichern
	lds	r23, dcbuf1 + 2		;Minuten Zehner holen
	sts	minuth, r23		;und speichern
	lds	r23, dcbuf1 + 3		;Stunden Einer holen
	sts	hourl, r23		;und speichern
	lds	r23, dcbuf1 + 4		;Stunden Zehner holen
	sts	hourh, r23		;und speichern
	clr	r23
	sts	secndl, r23		;Sekunden Einer lschen
	sts	secndh, r23		;Sekunden Zehner lschen
	sts	t1cnt2, r23		;Timer1 Interruptzhler lschen
	lds	r16, dcflag		;DCF77-Flags holen
	sbr	r16, 0x04		;Synchronisierungs-Flag setzen
	sts	dcflag, r16		;und wieder speichern
	ldi	r16, timdcf		;DCF77-Timeout-Wert laden
	sts	dcftio, r16		;und als Timeout neu setzen
	sei				;Interrupts wieder aktivieren
;
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 0			;Uhreinstellungsfehler?
	brtc	m50610			;nein -> berspringen
	cbr	r16, 0x01		;sonst Uhren-Fehler lschen
	sbr	r16, 0x80		;Fehlererinnerung setzen
	sts	errflg, r16		;Flags wieder speichern
	lds	r17, minutl		;Minuten Einer holen
	sts	minfl1, r17		;als Minutenflag1 voreinstellen
	sts	minfl3, r17		;als Minutenflag3 voreinstellen
	ser	r17			;0xff laden
	sts	minfl2, r17		;als Minutenflag2 voreinstellen
	lds	r17, hourl		;Stunden Einer holen
	sts	houflg, r17		;als Stundenflag voreinstellen
	bst	r16, 4			;RTC-Fehler?
	brts	m50610			;ja -> berspringen
	call	tiwrtc			;sonst Zeit in RTC bertragen
	brcc	m50610			;Fehler? nein -> weiter
	call	i2cerr			;RTC-Fehler setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
;
m50610:	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x14		;Infozeile neu ausgeben, DCF ok
	sts	dflags, r16		;Anzeige-Flags wieder speichern
	bst	r16, 7			;LCD wegen Synchronisierung abgeschaltet?
	brtc	m51000			;nein -> weiter
	call	lcdon			;sonst LCD wieder einschalten
;
; Anzeige-Flags bearbeiten: Verschieben der Sensor-Flags zur Daten-Neuausgabe in die Positions-Tabelle (nur bei normaler Anzeige aktiv)
;
m51000: lds     r16, dflags             ;Anzeige-Flags holen
        bst     r16, 0                  ;Normalanzeige aktiv?
        brtc    m51100                  ;nein -> berspringen
;
        ldi     r20, 0                  ;Sensornummer-Zhler = 0
        ldi     xl, low(senflg)
        ldi     xh, high(senflg)        ;Zeiger auf Sensor-Flags
;
m51010: ld      r19, x                  ;Sensor-Flags holen
        bst     r19, 7                  ;Sensor inaktiv?
        brts    m51050                  ;ja -> berspringen
        mov     r18, r19                ;sonst Sensor-Flags kopieren
        cbr     r19, 0x03               ;Neuausgabe-Flags lschen
        st      x, r19                  ;Flags wieder speichern
        rol     r18                     ;Bits 1:0 an Position 7:6 bringen:
        rol     r18                     ;2x nach links schieben
        swap    r18                     ;Nibbles tauschen
        andi    r18, 0xc0               ;Flags filtern
;
        ldi     r17, 32                 ;32 Anzeige-Positionen durchsuchen
        ldi     yl, low(postab)
        ldi     yh, high(postab)        ;Zeiger auf Anzeige-Positionen
;
m51020: ld      r16, y                  ;Sensornummer holen
        bst     r16, 5                  ;Anzeige-Position leer?
        brts    m51030                  ;ja -> weiter
        andi    r16, 0x3f               ;sonst Neuausgabe-Flags lschen
        cp      r20, r16                ;Sensornummer gefunden?
        brne    m51030                  ;nein -> weiter
        or      r16, r18                ;sonst Sensor-Flags fr Neuausgabe bertragen
        st      y, r16                  ;Sensornummer wieder speichern
;
m51030: adiw    yl, 1                   ;Zeiger auf nchste Anzeige-Position
        dec     r17                     ;alle Anzeige-Positionen bearbeitet?
        brne    m51020                  ;nein -> Schleife
;
        lds     r16, bigflg             ;Groanzeige-Flag fr Seite 1 holen
        bst     r16, 5                  ;Groanzeige-Position leer?
        brts    m51040                  ;ja -> weiter
        andi    r16, 0x3f               ;sonst Neuausgabe-Flags lschen
        cp      r20, r16                ;Sensornummer gefunden?
        brne    m51040                  ;nein -> weiter
        or      r16, r18                ;sonst Sensor-Flags fr Neuausgabe bertragen
        sts     bigflg, r16             ;und Flags wieder speichern
;
m51040: lds     r16, bigflg + 1         ;Groanzeige-Flag fr Seite 2 holen
        bst     r16, 5                  ;Groanzeige-Position leer?
        brts    m51050                  ;ja -> weiter
        andi    r16, 0x3f               ;sonst Neuausgabe-Flags lschen
        cp      r20, r16                ;Sensornummer gefunden?
        brne    m51050                  ;nein -> weiter
        or      r16, r18                ;sonst Sensor-Flags fr Neuausgabe bertragen
        sts     bigflg + 1, r16         ;und Flags wieder speichern
;
m51050: adiw    xl, 1                   ;Zeiger auf nchsten Sensor
        inc     r20                     ;nchster Sensor
        cpi     r20, 32                 ;alle Sensoren bearbeitet?
        brcs    m51010                  ;nein -> Schleife
;
; Prfen der Konklikt-Flags der Log-Tabellen und Erzeugen einer Alarmmeldung beim Auftreten eines Konklikts
;
m51100: lds     r16, dflags             ;Anzeige-Flags holen
        bst     r16, 0                  ;Normalanzeige?
        brtc    m60000                  ;nein -> Konflikt-Prfung berspringen
;
        lds     r20, errflg             ;sonst Fehler-Flags holen
        call    chkcfl                  ;Konflikte in den Log-Tabellen gefunden?
        breq    m51110                  ;nein -> weiter
        bst     r20, 6                  ;Log-Tabellen-Alarm bereits aktiv?
        brts    m60000                  ;ja -> Ende
        sbr     r20, 1 << 6             ;sonst Log-Tabellen-Alarm aktivieren
        sts     errflg, r20             ;Fehler-Flags wieder speichern
        ldi     r19, 0xef               ;Fehlercode "Log-Tabellen-Fehler"
        ldi     r20, 0                  ;kein Parameter
        call    wrimsg                  ;Fehlermeldung schreiben
        rjmp    m60000
;
m51110: bst     r20, 6                  ;Log-Tabellen-Alarm noch aktiv?
        brtc    m60000                  ;nein -> Ende
        cbr     r20, 1 << 6             ;sonst Log-Tabellen-Alarm lschen
        sbr     r20, 1 << 7             ;Alarm-Erinnerung setzen
        sts     errflg, r20             ;Fehler-Flags wieder speichern
;
; Timeouts fr Temperaturen bearbeiten
;
m60000:	ldi	xl, low(sentio)
	ldi	xh, high(sentio)	;Zeiger auf Timeout-Zhler
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	clr	r18			;Sensornummern-Zhler
m60010:	ld	r17, y			;Sensor-Flags holen
	bst	r17, 7			;Temperatursensor aktiv?
	brts	m60030			;nein -> berspringen
	ld	r16, x			;sonst Timeout-Zhler holen
	tst	r16			;Zhler = 0?
	brne	m60030			;nein -> berspringen
	sbr	r17, 0x41		;sonst nderungs-Flag und Temperatur-Ausfall setzen
	st	y, r17			;Flags wieder speichern
	dec	r16			;Timeout-Zhler auf 0xff setzen (Stopp)
	st	x, r16			;und wieder speichern
;
	lds	r19, errflg		;Fehler-Flags holen
	bst	r19, 1			;Datenempfangsfehler?
	brts	m60020			;ja -> keine Sensorausflle
	push	r18			;sonst akustisch melden
	lds	r18, signum + 7		;Signalnummer fr Sensorausfall
	lds	r19, sigdur + 7		;Signaldauer fr Sensorausfall
	call	signal			;Signal ausgeben
	pop	r18			;Register wieder herstellen
;
m60020:	push	xl
	push	xh
	push	yl
	push	yh
	push	r18			;Register sichern
	ldi	r19, 0xf9		;Fehler "Sensorausfall" setzen
	mov	r20, r18		;Sensornummer als Parameter kopieren
	call	wrimsg			;Fehlermeldung schreiben
	pop	r16			;Register mit Sensornummer wieder herstellen
        push    r16                     ;und wieder sichern
        call    pagesw                  ;gegebenenfalls Anzeigeseite wechseln
        pop     r18
	pop	yh
	pop	yl
	pop	xh
	pop	xl                      ;Register wieder herstellen
;
m60030:	adiw	xl, 1			;Zeiger auf nchsten Tabellenplatz setzen
	adiw	yl, 1			;Zeiger auf nchsten Tabellenplatz setzen
	inc	r18			;nchsten Sensor bearbeiten
	cpi	r18, 31			;alle 31 Sensoren bearbeitet?
	brne	m60010			;nein -> Schleife
;
; Timeout fr Meldungen bearbeiten
;
	lds	r16, msgtio		;Meldungs-Timeout-Zhler holen
	tst	r16			;Zhler = 0?
	brne	m60040			;nein -> berspringen
	dec	r16			;Timeout-Zhler auf 0xff setzen (Stopp)
	sts	msgtio, r16		;und wieder speichern
	lds	r16, errflg		;Fehler-Flags holen
	sbr	r16, 0x02		;Datenempfangsfehler setzen
	sts	errflg, r16		;Fehler-Flags wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
	lds	r18, signum + 6		;Signalnummer fr Fehlermeldungen
	lds	r19, sigdur + 6		;Signaldauer fr Fehlermeldungen
	call	signal			;Signal ausgeben
	ldi	r19, 0xf1		;Fehlercode "kein Datenempfang"
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
;
; Timeout fr DCF77-Empfang bearbeiten
;
m60040:	lds	r16, dcftio		;DCF77-Timeout-Zhler holen
	tst	r16			;Zhler = 0?
	brne	m60050			;nein -> berspringen
	dec	r16			;Timeout-Zhler auf 0xff setzen (Stopp)
	sts	dcftio, r16		;und wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x10		;DCF77-Empfangs-Flag lschen
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
;
; Timeout fr Men-Rckkehr und Wechsel zu Seite 1 bearbeiten
;
m60050:	lds	r16, mentio		;Men-Timeout-Zhler holen
	tst	r16			;Zhler = 0?
	brne	m60080			;nein -> berspringen
	dec	r16			;Timeout-Zhler auf 0xff setzen (Stopp)
	sts	mentio, r16		;und wieder speichern
	clr	r18			;neue Menebene und Menoption
	lds	r17, dflags		;Anzeigeflags holen
	lds	r16, menopt		;derzeitige Menoption holen
	cp	r16, r18		;nderung der Menoption?
	breq	m60060			;nein -> berspringen
	sts	menopt, r18		;sonst Menoption 0 setzen
	sbr	r17, 0x08		;Menzeile neu ausgeben
m60060:	lds	r16, mennum		;derzeitige Mennummer holen
	cp	r16, r18		;nderung der Menebene?
	breq	m60070			;nein -> berspringen
	sts	mennum, r18		;sonst Mennummer 0 setzen
	lds	r16, xflags		;verschiedene Flags holen
	cbr	r16, 0x0c		;Min/Max-Anzeige abschalten
	sts	xflags, r16		;Flags wieder speichern
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags lschen
	sbr	r17, 0x0f		;Normalanzeige neu ausgeben
;
m60070: lds     r16, xflag4             ;Verschiedene Flags 4 holen
        bst     r16, 4                  ;Seite 2 aktiv?
        brtc    m60075                  ;nein -> berspringen
        lds     r18, confg2             ;sonst Konfigurationsbyte 2 holen
        bst     r18, 3                  ;Option "Auf Seite 1 zurckschalten" aktiv?
        brtc    m60075                  ;nein -> berspringen
        cbr     r16, 1 << 4             ;sonst auf Seite 1 schalten
        sts     xflag4, r16             ;Flags wieder speichern
	sbr	r17, 0x0f		;Normalanzeige neu ausgeben
m60075:	sts	dflags, r17		;Anzeigeflags wieder speichern
;
; Timeout fr kurzzeitige Lautsprecher-Abschaltung bearbeiten
;
m60080:	lds	r16, lsptio		;Lautsprecher-Timeout-Zhler holen
	tst	r16			;Zhler = 0?
	brne	m60090			;nein -> berspringen
	dec	r16			;Timeout-Zhler auf 0xff setzen (Stopp)
	sts	lsptio, r16		;und wieder speichern
	lds	r17, xflag2		;verschiedene Flags 2 holen
	lds	r16, dflags		;Anzeigeflags holen
	bst	r17, 0			;Lautsprecher-Status holen
	bld	r16, 5			;und neu setzen
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeigeflags wieder speichern
;
; Timeout fr LCD-Abschaltung bearbeiten
;
m60090:	lds	r16, lcdtio		;LCD-Timeout-Zhler holen
	tst	r16			;Zhler = 0?
	brne	m61000			;nein -> berspringen
	dec	r16			;Timeout-Zhler auf 0xff setzen (Stopp)
	sts	lcdtio, r16		;und wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 7			;LCD wegen Synchronisierung abgeschaltet?
	brtc	m61000			;nein -> weiter
	call	lcdon			;LCD wieder einschalten
;
; Timeout fr zyklische Fehleranzeige bearbeiten
;
m61000:	lds	r16, errflg		;Fehler-Flags holen
	lds	r17, errcpy		;Fehler-Flags Kopie holen
	sts	errcpy, r16		;neue Kopie erzeugen
	cp	r16, r17		;Fehler-nderungen?
	brne	m61010			;ja -> bearbeiten
	lds	r17, errtio		;Fehleranzeige-Timeout holen
	cpi	r17, 0xff		;Zhler = 0xff (gestoppt)?
	breq	m61010			;ja -> Fehler prfen
	tst	r17			;Zhler = 0?
	brne	m61100			;nein -> berspringen
	dec	r17			;Timeout-Zhler auf 0xff setzen (Stopp)
	sts	errtio, r17		;und wieder speichern
;
m61010:	lds	r17, errmsk		;Fehler-Maske holen
	mov	r19, r17		;aktuelle Maske kopieren
	andi	r16, 0x7f		;Fehler-Erinnerung ausblenden
	breq	m61030			;Fehler? nein -> weiter
;
	ldi	r20, 9			;9 Maskenverschiebungen
	clc				;Carry lschen
	tst	r17			;Maske vorhanden?
	brne	m61020			;ja -> weiter
	sec				;sonst Carry setzen
m61020:	rol	r17			;Maske auf nchsten Fehler
	mov	r18, r16		;Fehler-Flags kopieren
	and	r18, r17		;Maske ber Fehler-Flags legen
	brne	m61040			;Fehler gefunden? ja -> weiter
	dec	r20			;alle Verschiebungen bearbeitet?
	brne	m61020			;ja -> Schleife
;
m61030:	clr	r17			;sonst Maske lschen
m61040:	sts	errmsk, r17		;und speichern
	cp	r17, r19		;wurde Maske gendert?
	breq	m61100			;nein -> weiter
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
	tst	r17			;Maske vorhanden?
	breq	m61100			;nein -> Ende
	ldi	r16, timerr		;sonst Timeout-Wert holen
	sts	errtio, r16		;und Timeout-Zhler neu laden
;
; Timeout fr kurzzeitige Umschaltung auf Nacht-Helligkeit bearbeiten
;
m61100: lds     r16, nlitio             ;Timeout-Zhler fr Nacht-Helligkeit holen
        tst     r16                     ;Zhler = 0?
        brne    m62000                  ;nein -> berspringen
        dec     r16                     ;Timeout-Zhler auf 0xff setzen (Stopp)
        sts     nlitio, r16             ;und wieder speichern
;
; Optische Signalisierung von Alarmen ber LED1 und LED2:
; 1. Systemfehler:	LED1 (rot) leuchtet stndig
; 2. Sensorausfall:	LED2 (gelb) leuchtet stndig
; 3. Alarmmeldung:	LED1 (rot) blinkt
; 4. Temperaturalarm:	LED2 (gelb) blinkt
;
m62000:	lds	r17, ledflg		;LED-Flags holen
	cbr	r17, 0x0f		;zunchst alle LEDs ausschalten
	lds	r16, errflg		;Fehlerflags holen
	andi	r16, 0x7f		;Fehlererinnerung ausblenden
	breq	m62100			;Fehler? nein -> weiter
	sbr	r17, 0x01		;sonst LED1 (rot) einschalten
;
m62100:	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	r20, 32			;32 Temperaturen bearbeiten
	clr	r19			;Alarmspeicher lschen
m62110:	ld	r18, x+			;Sensor-Flags holen
	andi	r18, 0x40		;Sensorausfall-Flag filtern
	or	r19, r18		;Flag speichern wenn gesetzt
	dec	r20			;alle Flags bearbeitet?
	brne	m62110			;nein -> Schleife
	tst	r19			;Sensorausflle vorhanden?
	breq	m62200			;nein -> weiter
	sbr	r17, 0x02		;sonst LED2 (gelb) einschalten
;
m62200:	ldi	xl, low(almflg)
	ldi	xh, high(almflg)	;Zeiger auf Alarmflags
	ldi	r20, 4			;4 Alarme bearbeiten
	clr	r19			;Alarmspeicher lschen
m62210:	ld	r18, x+			;Alarmflag holen
	andi	r18, 0x02		;Flag fr aktiven Alarm filtern
	or	r19, r18		;Flag verknpfen
	dec	r20			;alle Alarme bearbeitet?
	brne	m62210			;nein -> Schleife
	tst	r19			;aktive Alarme vorhanden?
	breq	m62300			;nein -> weiter
	sbr	r17, 0x04		;sonst LED1 (rot) blinken
;
m62300:	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	r20, 32			;32 Sensoren bearbeiten
	clr	r19			;Alarmspeicher lschen
m62310:	ld	r18, x+			;Sensor-Flag holen
	andi	r18, 0x0c		;Temperatur-Alarmflags filtern
	or	r19, r18		;Flag speichern wenn gesetzt
	dec	r20			;alle Flags bearbeitet?
	brne	m62310			;nein -> Schleife
	tst	r19			;aktive Temperatur-Alarme vorhanden?
	breq	m62400			;nein -> weiter
	sbr	r17, 0x08		;sonst LED2 (gelb) blinken
;
m62400:	sts	ledflg, r17		;LED-Flags wieder speichern
;
; Akustische Signalisierung steuern, Warteschlange auslesen
;
m64000:	lds	xl, sigbwr
	lds	xh, sigbwr + 1		;Schreibzeiger holen
	lds	yl, sigbrd
	lds	yh, sigbrd + 1		;Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;sind Signaldaten im Puffer?
	breq	m65000			;nein -> Ende
	lds	r16, sigtim		;Status des Signaltimers holen
	tst	r16			;werden Signal ausgegeben?
	brne	m65000			;ja -> berspringen
	ld	r16, y+			;Signalnummer holen, Pointer + 1
	ld	r17, y+			;Signaldauer holen, Pointer + 1
	lds	r18, dflags		;Anzeige-Flags holen
	bst	r18, 5			;Lautsprecher eingeschaltet?
	brtc	m64010			;nein -> keine Signalausgabe
	cli				;Interrupts sperren
	sts	sigsnd, r16		;Signalnummer setzen
	sts	sigtim, r17		;Signaldauer setzen
	sei				        ;Interrupts freigeben
m64010:	ldi	xl, low(sigbuf + (sigbul * 2))
	ldi	xh, high(sigbuf + (sigbul * 2)) ;Adresse Pufferende laden
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;Endadresse erreicht?
	brne	m64020			;nein -> weiter
	ldi	yl, low(sigbuf)
	ldi	yh, high(sigbuf)	;sonst Adresse auf Pufferanfang
m64020:	sts	sigbrd, yl
	sts	sigbrd + 1, yh		;Schreibzeiger speichern
;
; Signalausgabe prfen und Signal-Flag setzen
;
m65000:	clr	r18			;Signalmerker lschen
	lds	r16, sigtim		;Signaldauer holen?
	tst	r16			;wird ein Signal ausgegeben?
	brne	m65010			;ja -> merken
	lds	xl, sigbwr		;Signalpuffer Schreibzeiger
	lds	xh, sigbwr + 1		;holen
	lds	yl, sigbrd		;Signalpuffer Lesezeiger
	lds	yh, sigbrd + 1		;holen
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;sind Signaldaten im Puffer?
	breq	m65020			;nein -> weiter
m65010:	ldi	r18, 0x01		;Signalmerker setzen
m65020:	lds	r16, xflags		;verschiedene Flags holen
	andi	r16, 0x01		;Signalflag filtern
	cp	r16, r18		;Statusnderung?
	breq	m65030			;nein -> berspringen
	lds	r17, mennum		;aktuelle Mennummer holen
	tst	r17			;Hauptmen (0)?
	brne	m65030			;nein -> berspringen
	lds	r17, dflags		;sonst Anzeige-Flags holen
	sbr	r17, 0x08		;Menzeile neu ausgeben
	sts	dflags, r17		;Anzeige-Flags wieder speichern
m65030:	lds	r16, xflags		;verschiedene Flags holen
	bst	r18, 0			;ermittelten Signalstatus
	bld	r16, 0			;holen und neu setzen
	sts	xflags, r16		;Flags wieder speichern
;
; alle Erinnerungen prfen und allgemeines Erinnerungs-Flag setzen, auerdem Erinnerungsflags fr LED-Signalisierung setzen
;
m66000: clr	r18			;Erinnerungsmerker lschen
	lds	r19, ledflg		;LED-Flags holen
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 7			;Erinnerung gesetzt?
	brtc	m66010			;nein -> weiter
	ldi	r18, 0x02		;sonst Erinnerung merken
	sbr	r19, 0x10		;LED1 Erinnerung setzen
;
m66010: cbr     r19, 0x20               ;zunchst LED2 Erinnerung lschen
	ldi	r17, 32			;16 Sensor-Flags bearbeiten
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
m66020:	ld	r16, x+			;Flags holen
	bst	r16, 7			;Sensor inaktiv?
	brts	m66030			;ja -> berspringen
	andi	r16, 0x30		;Erinnerungen gesetzt?
	breq	m66030			;nein -> weiter
	ldi	r18, 0x02		;sonst Erinnerung merken
	sbr	r19, 0x20		;LED2 Erinnerung setzen
m66030:	dec	r17			;alle Sensoren bearbeitet?
	brne	m66020			;nein -> Schleife
;
	ldi	r17, 4			;4 Alarmflags bearbeiten
	ldi	xl, low(almflg)
	ldi	xh, high(almflg)	;Zeiger auf Alarmflags
m66040:	ld	r16, x+			;Flags holen
	bst	r16, 7			;Alarmeingang inaktiv?
	brts	m66050			;ja -> berspringen
	bst	r16, 2			;Erinnerung gesetzt?
	brtc	m66050			;nein -> weiter
	ldi	r18, 0x02		;sonst Erinnerung merken
	sbr	r19, 0x10		;LED1 Erinnerung setzen
m66050:	dec	r17			;alle Alarme bearbeitet?
	brne	m66040			;nein -> Schleife
	sts	ledflg, r19		;LED-Flags wieder speichern
;
	lds	r16, xflags		;verschiedene Flags holen
	andi	r16, 0x02		;allgemeine Erinnerung filtern
	cp	r16, r18		;Statusnderung?
	breq	m66060			;nein -> berspringen
	lds	r17, mennum		;aktuelle Mennummer holen
	tst	r17			;Hauptmen (0)?
	brne	m66060			;nein -> berspringen
	lds	r17, dflags		;sonst Anzeige-Flags holen
	sbr	r17, 0x08		;Menzeile neu ausgeben
	sts	dflags, r17		;Anzeige-Flags wieder speichern
m66060:	lds	r16, xflags		;verschiedene Flags holen
	bst	r18, 1			;ermittelten Erinnerungsstatus
	bld	r16, 1			;holen und neu setzen
	sts	xflags, r16		;Flags wieder speichern
;
; LCD-Hintergrundbeleuchtung steuern
;
	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 7			;LCD wegen DCF-Synchronisierung ausgeschaltet?
	brtc	m67000			;nein -> weiter
        clr     r16                     ;sonst PWM-Wert auf 0 setzen
        rjmp    m67070                  ;und ausgeben
;
m67000: lds     r16, confg2             ;Konfigurationsbyte 2 holen
	lds	r17, xflags		;verschiedene Flags holen
        lds     r18, mentio             ;Men-Timeout-Zhler holen
        bst     r16, 1                  ;Beleuchtungs-Automatik aktiv?
        brtc    m67050                  ;nein -> einfache Lichtsteuerung
;
        lds     r16, nlitio             ;Timeout-Zhler fr Nacht-Helligkeit holen
        cpi     r16, 0xff               ;Timeout-Zhler aktiv?
        breq    m67010                  ;nein -> weiter
        lds     r16, nlight             ;sonst Nacht-Helligkeit holen
        rjmp    m67070                  ;und als PWM-Wert direkt einstellen
;
m67010:	lds	r19, adclig		;ADC0-Wert (Lichtsensor) holen
        lds     r16, nlight             ;Nacht-Helligkeit holen
        cp      r19, r16                ;Lichtsensor-Wert < Nacht-Helligkeit?
        brcc    m67020                  ;nein -> weiter
        mov     r19, r16                ;sonst Nacht-Helligkeit verwenden
m67020:	bst	r17, 5			;Nacht-Modus aktiv?
        brtc    m67030                  ;nein -> weiter
        lds     r19, nlight             ;sonst Nacht-Helligkeit setzen
        lds     r17, confg1             ;Konfigurationsbyte 1 holen
        bst     r17, 7                  ;Status Abwesend?
        brtc    m67030                  ;nein -> weiter
        clr     r19                     ;sonst Helligkeit auf 0 setzen
;
m67030: cpi     r18, 0xff               ;ist Men-Timeout-Zhler aktiv?
        breq    m67040                  ;nein -> weiter
        ser     r16                     ;sonst PWM-Wert auf volle Helligkeit setzen
        out     ocr2, r16               ;und direkt einstellen
        mov     r19, r16                ;volle Helligkeit auf PWM-Wert bertragen
m67040: sts     bright, r19 		;neuen PWM-Wert fr weiche Helligkeitssteuerung setzen
        rjmp    m67080
;
m67050:	bst	r17, 5			;Nacht-Modus aktiv?
        brtc    m67060                  ;nein -> weiter
        clr     r16                     ;sonst Nacht-Helligkeit setzen
        cpi     r18, 0xff               ;ist Men-Timeout-Zhler aktiv?
        breq    m67070                  ;nein -> weiter
m67060: ser     r16                     ;sonst PWM-Wert auf volle Helligkeit
m67070: sts     bright, r16             ;neuen PWM-Wert setzen
        out     ocr2, r16               ;und direkt einstellen
;
; Lausprecherstatus setzen
;
m67080:	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 7			;Status Abwesend?
	brts	m67100			;ja -> berspringen
	lds	r16, lsptio		;Lautsprecher-Timeout-Zhler holen
	cpi	r16, 0xff		;Zhler aktiv?
	brne	m67100			;ja -> berspringen
	lds	r16, xflag2		;sonst verschiedene Flags 2 holen
	lds	r17, dflags		;Anzeigeflags holen
	mov	r18, r17		;und kopieren
	bst	r16, 0			;Lautsprecherstatus holen
	bld	r17, 5			;und neu setzen
	cp	r17, r18		;hat sich der Status gendert?
	breq	m67090			;nein -> weiter
	sbr	r17, 0x04		;sonst Infozeile neu ausgeben
m67090:	sts	dflags, r17		;Anzeigeflags wieder speichern
;
; Konfiguration nach nderung im EEPROM speichern
;
m67100:	lds	r16, xflag3		;verschiedene Flags 3 holen
	bst	r16, 0			;Konfiguration im EEPROM speichern?
	brtc	m68000			;nein -> berspringen
	cbr	r16, 0x01		;sonst Konfigurationsspeicherungs-Flag lschen
	sts	xflag3, r16		;Flags wieder speichern
	call	strwai			;Warte-Text ausgeben
	call	eewcfg			;Konfiguration im EEPROM speichern
;
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 1 << 3		;Menzeile neu ausgeben
	sts	dflags, r16		;Flags wieder speichern
        lds     r16, xflags             ;Verschiedene Flags holen
        sbr     r16, 1 << 4             ;Backup-Status-Anzeige erneuern
        sts     xflags, r16             ;Flags wieder speichern
;
; Zeitabhngige Vorgnge steuern - Minutenwechsel prfen, Infozeile neu ausgeben
;
m68000:	cli				;Interrupts sperren
	lds	r16, minutl		;aktuellen Minuten-Einer holen
	lds	r17, minuth		;aktuellen Minuten-Zehner holen
	lds	r18, hourl		;aktuellen Stunden-Einer holen
	lds	r19, hourh		;aktuellen Stunden-Zehner holen
	sei				;Interrupts wieder freigeben
	lds	r20, minfl1		;Minuten-Flag 1 holen
	cp	r16, r20		;Minuten-Wechsel?
	breq	m68010			;nein -> berspringen
	sts	minfl1, r16		;neues Minuten-Flag 1 speichern
	lds	r20, dflags		;Anzeigeflags holen
	sbr	r20, 0x04		;Infozeile neu ausgeben
	sts	dflags, r20		;Flags wieder speichern
;
; Stunden- und Tageswechsel prfen; bei DCF77-Synchronisierung in der vergangenen Stunde wird die RTC neu gesetzt, sonst werden die Zeitdaten von der RTC
; geholt zum Tageswechsel erfolgt keine Synchronisierung mit der RTC
;
m68010:	lds	r20, errflg		;Fehler-Flags holen
	bst	r20, 4			;RTC-Fehler?
	brts	m68100			;ja -> berspringen
	lds	r20, houflg		;sonst Stunden-Flag holen
	cp	r18, r20		;Stunden-Wechsel?
	breq	m68100			;nein -> berspringen
	sts	houflg, r18		;neues Stunden-Flag speichern
;
	cpi	r16, 9			;Minuten-Einer = 9?
	brne	m68020			;nein -> 0:00 Uhr testen
	cpi	r17, 5			;Minuten-Zehner = 5?
	brne	m68020			;nein -> 0:00 Uhr testen
	cpi	r18, 3			;Stunden-Einer = 3?
	brne	m68020			;nein -> 0:00 Uhr testen
	cpi	r19, 2			;Stunden-Zehner = 2?
	breq	m68100			;ja -> kein Uhrenabgleich
m68020:	or	r16, r17
	or	r16, r18		;Stunden- und Minuten-Werte
	or	r16, r19		;verknpfen, Zeit 0:00 Uhr?
	breq	m68100			;ja -> kein Uhrenabgleich
;
	lds	r16, dcflag		;DCF77-Flags holen
	bst	r16, 2			;Synchronisierungs-Flag gesetzt?
	brtc	m68030			;nein -> Zeit aus RTC holen
	cbr	r16, 1 << 2		;Synchronisierungs-Flag lschen
	sts	dcflag, r16		;Flags wieder speichern
	call	tiwrtc			;Uhrzeit in RTC bertragen
	rjmp	m68040
m68030:	call	tirrtc			;sonst Uhrzeit aus RTC holen
m68040:	brcc	m68100			;I2C-Fehler? nein -> weiter
	call	i2cerr			;sonst Fehler melden
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
;
; Weitere Aktionen nur bei synchronisierter Uhr ausfhren
;
m68100:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 0			;ist Uhr eingestellt?
	brtc	m68110			;ja -> bearbeiten
	rjmp	m70000			;sonst berspringen
;
m68110:	lds	r16, minutl		;aktuellen Minuten-Einer holen
	lds	r17, minfl2		;Minuten-Flag 2 holen
	cp	r16, r17		;Minuten-Wechsel?
	brne	m68120			;ja -> weiter
	rjmp	m69000			;sonst berspringen
m68120:	sts	minfl2, r16		;sonst Minuten-Flag 2 speichern
;
; Aktuelle Uhrzeit zu einem 16-Bit-Minutenwert zusammenfassen
;
	ldi	r16, 10			;Multiplikator 10 laden
	lds	r15, hourh		;Stunden Zehner holen
	mul	r15, r16		;verzehnfachen
	lds	r15, hourl		;Stunden Einer holen
	add	r15, r0			;Zehner addieren
	lds	r14, minuth		;Minuten Zehner holen
	mul	r14, r16		;verzehnfachen
	lds	r14, minutl		;Minuten Einer holen
	add	r14, r0			;Zehner addieren
	ldi	r16, 60			;Multiplikator 60 laden
	mul	r15, r16		;Stunden in Minuten umrechnen
	clr	r16
	add	r0, r14			;Minuten und bertrag addieren
	adc	r1, r16			;Minutenwert in r0/r1 (L/H)
;
; Nachtschaltung der LCD-Hintergrundbeleuchtung steuern
;
	lds	r20, xflags		;verschiedene Flags holen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 7			;Status Abwesend?
	brts	m68210			;ja -> Nachtmodus aktivieren
	bst	r16, 3			;LCD-Nachtschaltung aktiv?
	brtc	m68220			;nein -> Nachtmodus deaktivieren
;
	lds	r16, nighbl		;Nacht-Startzeit L holen
	lds	r17, nighbl + 1		;Nacht-Startzeit H holen
	lds	r18, nighbl + 2		;Nacht-Endezeit L holen
	lds	r19, nighbl + 3		;Nacht-Endezeit H holen
	movw	r14, r18		;Endezeit kopieren
	sub	r14, r16		;Endezeit - Startzeit
	sbc	r15, r17		;Startzeit > Endezeit?
	brcs	m68200			;ja -> bearbeiten
;
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r16
	sbc	r15, r17		;Startzeit erreicht?
	brcs	m68220			;nein -> Nachtmodus aus
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r18
	sbc	r15, r19		;Endezeit erreicht?
	brcs	m68210			;nein -> Nachtmodus ein
	rjmp	m68220			;sonst Nachtmodus aus
;
m68200:	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r18
	sbc	r15, r19		;Endezeit erreicht?
	brcs	m68210			;nein -> Nachtmodus ein
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r16
	sbc	r15, r17		;Startzeit erreicht?
	brcs	m68220			;nein -> Nachtmodus aus
;
m68210:	sbr	r20, 0x20		;Nachtmodus aktivieren
	rjmp	m68230
m68220:	cbr	r20, 0x20		;Nachtmodus deaktivieren
m68230:	sts	xflags, r20		;Flags wieder speichern
;
; Nachtschaltung des Bluetooth-Moduls steuern
;
m68250:	lds	r20, xflags		;verschiedene Flags holen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 7			;Status Abwesend?
	brts	m68270			;ja -> Bluetooth ausschalten
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 2			;Bluetooth-Nachtschaltung aktiv?
	brtc	m68280			;nein -> Bluetooth einschalten
;
	lds	r16, nighbt		;Nacht-Startzeit L holen
	lds	r17, nighbt + 1		;Nacht-Startzeit H holen
	lds	r18, nighbt + 2		;Nacht-Endezeit L holen
	lds	r19, nighbt + 3		;Nacht-Endezeit H holen
	movw	r14, r18		;Endezeit kopieren
	sub	r14, r16		;Endezeit - Startzeit
	sbc	r15, r17		;Startzeit > Endezeit?
	brcs	m68260			;ja -> bearbeiten
;
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r16
	sbc	r15, r17		;Startzeit erreicht?
	brcs	m68280			;nein -> Nachtmodus aus
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r18
	sbc	r15, r19		;Endezeit erreicht?
	brcs	m68270			;nein -> Nachtmodus ein
	rjmp	m68280			;sonst Nachtmodus aus
;
m68260:	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r18
	sbc	r15, r19		;Endezeit erreicht?
	brcs	m68270			;nein -> Nachtmodus ein
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r16
	sbc	r15, r17		;Startzeit erreicht?
	brcs	m68280			;nein -> Nachtmodus aus
;
m68270:	cbi	portb, 5		;Bluetooth ausschalten
	rjmp	m68300
m68280:	sbi	portb, 5		;Bluetooth einschalten
;
; Automatisches Rcksetzen der Minimal- und Maximalwerte steuern
;
m68300:	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 4			;Auto-Reset Min/Maxwerte aktiv?
	brtc	m68400			;nein -> berspringen
;
	lds	r16, aresmm		;Auto-Reset Min-Werte L holen
	lds	r17, aresmm + 1		;Auto-Reset Min-Werte H holen
	sub	r16, r0
	sbc	r17, r1			;Auto-Reset-Zeit erreicht?
	brne	m68310			;nein -> weiter
	call	rsmin1			;sonst Minimal-Werte lschen
;
m68310:	lds	r16, aresmm + 2		;Auto-Reset Max-Werte L holen
	lds	r17, aresmm + 3		;Auto-Reset Max-Werte H holen
	sub	r16, r0
	sbc	r17, r1			;Auto-Reset-Zeit erreicht?
	brne	m68400			;nein -> weiter
	call	rsmax1			;sonst Maximal-Werte lschen
;
; Nachtschaltung der Signal-Lautstrke steuern
;
m68400:	lds	r16, dimode		;alternativen Anzeigemodus holen
	cpi	r16, 13			;Modus Konfiguration - Signale - Audio - Auswhlen?
	breq	m68500			;ja -> berspringen
	cpi	r16, 14			;Modus Konfiguration - Signale - Audio - ndern?
	breq	m68500			;ja -> berspringen
;
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 5			;Signal-Nachtschaltung aktiv?
	brtc	m68430			;nein -> Taglautstrke setzen
;
	lds	r16, signig		;Nacht-Startzeit L holen
	lds	r17, signig + 1		;Nacht-Startzeit H holen
	lds	r18, signig + 2		;Nacht-Endezeit L holen
	lds	r19, signig + 3		;Nacht-Endezeit H holen
	movw	r14, r18		;Endezeit kopieren
	sub	r14, r16		;Endezeit - Startzeit
	sbc	r15, r17		;Startzeit > Endezeit?
	brcs	m68410			;ja -> bearbeiten
;
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r16
	sbc	r15, r17		;Startzeit erreicht?
	brcs	m68430			;nein -> Nachtmodus aus
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r18
	sbc	r15, r19		;Endezeit erreicht?
	brcs	m68420			;nein -> Nachtmodus ein
	rjmp	m68430			;sonst Nachtmodus aus
;
m68410:	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r18
	sbc	r15, r19		;Endezeit erreicht?
	brcs	m68420			;nein -> Nachtmodus ein
	movw	r14, r0			;aktuelle Zeit kopieren
	sub	r14, r16
	sbc	r15, r17		;Startzeit erreicht?
	brcs	m68430			;nein -> Nachtmodus aus
;
m68420:	lds	r17, sigvo2		;Nachtmodus ein, Nachtlautstrke holen
	rjmp	m68440
m68430:	lds	r17,sigvo1		;Nachtmodus aus, Taglautstrke
m68440:	call	sicalc			;umrechnen und PWM einstellen
;
; LCD zum Synchronisieren der Uhr abschalten
;
m68500:	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 6			;LCD-Abschaltung fr DCF-Synchronisierung?
	brtc	m69000			;nein -> berspringen
	lds	r16, mentio		;Men-Timeout-Zhler holen
	cpi	r16, 0xff		;Zhler aktiv?
	brne	m69000			;ja -> berspringen
	ldi	r16, low(3 * 60)
	ldi	r17, high(3 * 60)	;Minutenwert fr 3:00 Uhr laden
	sub	r16, r0
	sbc	r17, r1			;Zeitpunkt erreicht?
	brne	m69000			;nein -> berspringen
	call	lcdoff			;sonst LCD ausschalten
	ldi	r16, timlcd		;Timeout-Zhler fr LCD-
	sts	lcdtio, r16		;Abschaltung neu setzen
;
; Przisen Minutenwechsel fr Log-Funktion prfen
;
m69000:	lds	r16, minutl		;aktuellen Minuten-Einer holen
	lds	r17, minfl3		;Minuten-Flag 3 holen
	inc	r17			;Minuten-Flag erhhen
	cpi	r17, 10			;Flag > 9?
	brcs	m69010			;nein -> weiter
	clr	r17			;sonst Flag = 0 setzen
m69010:	cp	r16, r17		;erwartete Minute erreicht?
	breq	m69020			;ja -> weiter
	rjmp	m70000			;sonst berspringen
;
; Zeitdaten fr weitere Verabeitung komprimieren und im EEPROM Schreib-Puffer ablegen
;
m69020:	sts	minfl3, r16		;Minuten-Flag 3 neu setzen
	call	cpstim			;Zeitdaten komprimieren
;
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicherfehler?
	brtc	m69100			;nein -> weiter
	rjmp	m70000			;sonst berspringen
m69100: call    chkcfl                  ;Konflikte in den Log-Tabellen gefunden?
        breq    m69110                  ;nein -> weiter
        rjmp    m70000                  ;sonst berspringen
;
; Zur Minute 0, 15, 30 und 45 einen Temperaturdatensatz ins externe EEPROM schreiben
;
m69110:	lds	r16, xewbuf		;aktuelle Zeit (Minuten) holen
	cpi	r16, 0			;Minuten = 0?
	breq	m69200			;ja -> bearbeiten
	cpi	r16, 15			;Minuten = 15?
	breq	m69200			;ja -> bearbeiten
	cpi	r16, 30			;Minuten = 30?
	breq	m69200			;ja -> bearbeiten
	cpi	r16, 45			;Minuten = 45?
	breq	m69200			;ja -> bearbeiten
	rjmp	m69300			;sonst berspringen
;
; 15min-Werte - Sensordaten aufbereiten: aktuelle Werte von T01-T24 (ber Log-Tabelle 1) als 12-Bit-Daten in 36 Bytes speichern
;
m69200:	ldi	yl, low(xewbuf + 4)
	ldi	yh, high(xewbuf + 4)	;Zeiger auf Ext-EEPROM Schreib-Puffer setzen
	ldi	r18, 12			;12 Wertepaare mit je 3 Byte
	ldi	r19, 0			;Zhler fr Log-Position
;
m69210:	call	slog1a			;Sensornummer suchen (ungerade Nummer des Wertepaares)
	ld	r16, z			;Sensor-Flag holen
	bst	r16, 6			;Sensorausfall?
	brts	m69220			;ja -> Wert nicht verarbeiten
	bst	r16, 7			;Sensor inaktiv?
	brtc	m69230			;nein -> Wert verarbeiten
m69220:	clr	r16                     ;sonst
	st	y+, r16			;1 L in Ext-EEPROM-Puffer legen
	ldi	r17, 0x08		;Wert ungltig setzen
	rjmp	m69240			;H-Byte bearbeiten
;
m69230:	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m69240			;ja -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m69240:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
	call	slog1a			;Sensornummer suchen (gerade Nummer des Wertepaares)
	ld	r16, z			;Sensor-Flag holen
	bst	r16, 6			;Sensorausfall?
	brts	m69250			;ja -> Wert nicht verarbeiten
	bst	r16, 7			;Sensor inaktiv?
	brtc	m69260			;nein -> Wert verarbeiten
m69250:	clr	r16                     ;sonst
	st	y+, r16			;2 L in Ext-EEPROM-Puffer legen
	ldi	r16, 0x80		;Wert ungltig setzen
	rjmp	m69270			;H-Byte bearbeiten
;
m69260:	ld	r16, x+			;Sensorwert 2 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert 2 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m69270			;nein -> weiter
	swap	r16			;sonst in oberes Nibble legen
m69270:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert 1 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
	dec	r18			;alle Werte verarbeitet?
	brne	m69210			;nein -> Schleife
;
; 15min-Werte - Sensordaten aufbereiten: aktuelle Werte von T25-T27 (ber Log-Tabelle 2) und P01 als 12-Bit-Daten in 6 Bytes speichern
;
        ldi     r19, 0                  ;Zhler fr Log-Position
	call	slog2a			;Sensornummer suchen (ungerade Nummer des Wertepaares)
	ld	r16, z			;Sensor-Flag holen
	bst	r16, 6			;Sensorausfall?
	brts	m69280			;ja -> Wert nicht verarbeiten
	bst	r16, 7			;Sensor inaktiv?
	brtc	m69290			;nein -> Wert verarbeiten
m69280:	clr	r16                     ;sonst
	st	y+, r16			;1 L in Ext-EEPROM-Puffer legen
	ldi	r17, 0x08		;Wert ungltig setzen
	rjmp	m692a0			;H-Byte bearbeiten
;
m69290:	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m692a0			;ja -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m692a0:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
	call	slog2a			;Sensornummer suchen (gerade Nummer des Wertepaares)
	ld	r16, z			;Sensor-Flag holen
	bst	r16, 6			;Sensorausfall?
	brts	m692b0			;ja -> Wert nicht verarbeiten
	bst	r16, 7			;Sensor inaktiv?
	brtc	m692c0			;nein -> Wert verarbeiten
m692b0:	clr	r16                     ;sonst
	st	y+, r16			;2 L in Ext-EEPROM-Puffer legen
	ldi	r16, 0x80		;Wert ungltig setzen
	rjmp	m692d0			;H-Byte bearbeiten
;
m692c0:	ld	r16, x+			;Sensorwert 2 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert 2 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m692d0			;nein -> weiter
	swap	r16			;sonst in oberes Nibble legen
m692d0:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert 1 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
;
	call	slog2a			;Sensornummer suchen (ungerade Nummer des Wertepaares)
	ld	r16, z			;Sensor-Flag holen
	bst	r16, 6			;Sensorausfall?
	brts	m692e0			;ja -> Wert nicht verarbeiten
	bst	r16, 7			;Sensor inaktiv?
	brtc	m692f0			;nein -> Wert verarbeiten
m692e0:	clr	r16                     ;sonst
	st	y+, r16			;1 L in Ext-EEPROM-Puffer legen
	ldi	r17, 0x08		;Wert ungltig setzen
	rjmp	m692g0			;H-Byte bearbeiten
;
m692f0:	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m692g0			;ja -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m692g0:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
	lds	r16, senflg + 27	;Sensor-Flag holen (Sensor P01)
        ldi     xl, low(senval + 54)
        ldi     xh, high(senval + 54)   ;Zeiger auf Sensorwert von P01
	bst	r16, 6			;Sensorausfall?
	brts	m692h0			;ja -> Wert nicht verarbeiten
	bst	r16, 7			;Sensor inaktiv?
	brtc	m692i0			;nein -> Wert verarbeiten
m692h0:	clr	r16                     ;sonst
	st	y+, r16			;P01 L in Ext-EEPROM-Puffer legen
	ldi	r16, 0x80		;Wert ungltig setzen
	rjmp	m692j0			;H-Byte bearbeiten
;
m692i0:	ld	r16, x+			;Sensorwert P01 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert P01 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m692j0			;nein -> weiter
	swap	r16			;sonst in oberes Nibble legen
m692j0:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert P01 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
;
; 15min-Werte - Sensordaten aufbereiten: aktuelle Werte von H01-H03 (ber Log-Tabelle 2) als 8-Bit-Daten in 3 Bytes und Batteriespannung in 2 Bytes speichern
;
        ldi     r18, 3                  ;3 Werte bearbeiten
        ldi     r19, 0                  ;Zhler fr Log-Position
;
m692k0:	call	slog2a			;Sensornummer suchen
        adiw    zl, 4                   ;Zeigerposition fr Luftfeuchtigkeitssensoren korrigieren
        adiw    xl, 8                   ;Zeigerposition fr Luftfeuchtigkeitssensoren korrigieren
	ld	r16, z			;Sensor-Flag holen
	bst	r16, 6			;Sensorausfall?
	brts	m692l0			;ja -> Wert nicht verarbeiten
	bst	r16, 7			;Sensor inaktiv?
	brtc	m692m0			;nein -> Wert verarbeiten
m692l0:	ldi	r16, 0x80		;sonst Wert ungltig setzen
	rjmp	m692n0			;speichern
m692m0:	ld	r16, x+			;Sensorwert L holen
	ld	r17, x+			;Sensorwert H holen
	cpi	r17, 0x80		;Wert ungltig?
	breq	m692l0			;ja -> ungltig setzen
m692n0:	st	y+, r16			;8-Bit-Wert in Ext-EEPROM-Puffer legen
	dec	r18			;alle Werte verarbeitet?
	brne	m692k0			;nein -> Schleife
;
        call    batclc                  ;Batteriespannung berechnen
	sts	batval, r16		;L-Wert fr 24h-Werte zwischenspeichern
	sts	batval + 1, r17		;H-Wert fr 24h-Werte zwischenspeichern
        st      y+, r16                 ;L-Wert speichern
        st      y+, r17                 ;H-Wert speichern
;
; 15min-Werte - aufbereitete Daten ins Ext-EEPROM schreiben und Pointer bearbeiten
;
	lds	r20, xes1po
	lds	r21, xes1po + 1		;Zeiger auf aktuelle Position
	lds	r22, xes1po + 2		;im externen EEPROM holen
	ldi	r23, xes1da		;einen Datensatz (51 Bytes) zum Ext-EEPROM senden
	call	xwrite			;Puffer zum Ext-EEPROM senden
	brcc	m692o0			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m69300			;Adressberechnung berspringen
;
m692o0:	ldi	r16, low(xs1sta + xs1len - xes1da)
	ldi	r17, high(xs1sta + xs1len - xes1da)
	ldi	r18, byte3(xs1sta + xs1len - xes1da)    ;Adresse des letzten mglichen Datensatzbeginns laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;aktuelle Adresse noch im gltigen Speicherbereich?
	brcc	m692p0			;ja -> weiter
	ldi	r20, low(xs1sta)
	ldi	r21, high(xs1sta)	;sonst
	ldi	r22, byte3(xs1sta)	;Adresse wieder auf Anfang setzen
m692p0:	sts	xes1po, r20
	sts	xes1po + 1, r21
	sts	xes1po + 2, r22		;Zeiger fr Position im externen EEPROM speichern
;
	lds	r16,errflg		;Fehler-Flags holen
	bst	r16, 4			;RTC-Fehler?
	brts	m69300			;ja -> berspringen
	ldi	r18, rtxs1p		;RTC-Adresse Sensordaten 1
	ldi	xl, low(xes1po)
	ldi	xh, high(xes1po)	;Zeiger auf Sensordaten 1 Pointer
	call	i2psen			;Pointer in RTC schreiben
	brcc	m69300			;Fehler? nein -> weiter
	call	i2cerr			;sonst RTC Fehlerstatus setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
;
; Zeit 23:59 Uhr prfen und Tageswerte ins Externe EEPROM schreiben
;
m69300:	lds	r16, xewbuf		;Minutenwert holen
	cpi	r16, 59			;Minute = 59?
	brne	m69310			;nein -> berspringen
	lds	r16, xewbuf + 1		;Stundenwert holen
	cpi	r16, 23			;Stunde = 23?
	breq	m69320			;ja -> Tageswerte schreiben
m69310:	rjmp	m69800			;sonst berspringen
;
; 24h-Werte - Sensordaten aufbereiten: Minimalwerte von T01-T24 (ber Log-Tabelle 1) als 12-Bit-Daten in 36 Bytes speichern (ab Position 4)
;
m69320:	ldi	yl, low(xewbuf + 4)
	ldi	yh, high(xewbuf + 4)	;Zeiger auf Ext-EEPROM Schreib-Puffer setzen
	ldi	r18, 12			;12 Wertepaare mit je 3 Byte
	ldi	r19, 0			;Zhler fr Log-Position
;
m69330:	call	slog1b			;Sensornummer suchen
	ldi	xl, low(semin2)         ;(ungerade Nummer des Wertepaares)
	ldi	xh, high(semin2)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Sensorwert gltig?
	brne	m69340			;nein -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m69340:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
	call	slog1b			;Sensornummer suchen
	ldi	xl, low(semin2)         ;(gerade Nummer des Wertepaares)
	ldi	xh, high(semin2)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 2 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert 2 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m69350			;ja -> weiter
	swap	r16			;sonst in oberes Nibble legen
m69350:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert 1 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
	dec	r18			;alle Werte verarbeitet?
	brne	m69330			;nein -> Schleife
;
; 24h-Werte - Sensordaten aufbereiten: Minimalwerte von T25-T27 (ber Log-Tabelle 2) und P01 als 12-Bit-Daten in 6 Bytes speichern
;
        ldi     r19, 0                  ;Zhler fr Log-Position
	call	slog2b			;Sensornummer suchen (ungerade Nummer des Wertepaares)
	ldi	xl, low(semin2 + 48)    ;(ungerade Nummer des Wertepaares)
	ldi	xh, high(semin2 + 48)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m69360			;ja -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m69360:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
	call	slog2b			;Sensornummer suchen (gerade Nummer des Wertepaares)
	ldi	xl, low(semin2 + 48)    ;(gerade Nummer des Wertepaares)
	ldi	xh, high(semin2 + 48)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 2 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert 2 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m69370			;nein -> weiter
	swap	r16			;sonst in oberes Nibble legen
m69370:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert 1 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
;
	call	slog2b			;Sensornummer suchen (ungerade Nummer des Wertepaares)
	ldi	xl, low(semin2 + 48)    ;(gerade Nummer des Wertepaares)
	ldi	xh, high(semin2 + 48)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m69380			;ja -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m69380:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
        ldi     xl, low(semin2 + 54)
        ldi     xh, high(semin2 + 54)   ;Zeiger auf Sensorwert von P01
	ld	r16, x+			;Sensorwert P01 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert P01 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m69390			;nein -> weiter
	swap	r16			;sonst in oberes Nibble legen
m69390:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert P01 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
;
; 24h-Werte - Sensordaten aufbereiten: Minimalwerte von H01-H03 (ber Log-Tabelle 2) als 8-Bit-Daten in 3 Bytes und Batteriespannung in 2 Bytes speichern
;
        ldi     r18, 3                  ;3 Werte bearbeiten
        ldi     r19, 0                  ;Zhler fr Log-Position
;
m693a0:	call	slog2b			;Sensornummer suchen
	ldi	xl, low(semin2 + 56)
	ldi	xh, high(semin2 + 56)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert L holen
        ld      r17, x+                 ;Sensorwert H holen
        cpi     r17, 0x80               ;Wert ungltig?
        brne    m693b0                  ;nein -> weiter
	ldi	r16, 0x80		;sonst Wert ungltig setzen
m693b0:	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	dec	r18			;alle Werte verarbeitet?
	brne	m693a0			;nein -> Schleife
;
	lds	r16, batval
	lds	r17, batval + 1		;gespeicherte Batteriespannung holen
	mov	r18, r16
	or	r18, r17		;Wert noch leer (= 0)?
	brne	m693c0			;nein -> weiter
	call    batclc                  ;sonst Batteriespannung berechnen
	sts     batval, r16             ;Batteriespannung L zwischenspeichern
	sts     batval + 1, r17         ;Batteriespannung H zwischenspeichern
m693c0:	st      y+, r16                 ;L-Wert speichern
        st      y+, r17                 ;H-Wert speichern
;
; 24h-Werte - Sensordaten aufbereiten: Maximalwerte von T01-T24 (ber Log-Tabelle 1) als 12-Bit-Daten in 36 Bytes speichern (ab Position 51)
;
	ldi	yl, low(xewbuf + 4 + 47)
	ldi	yh, high(xewbuf + 4 + 47)	;Zeiger auf Ext-EEPROM Schreib-Puffer setzen
	ldi	r18, 12			        ;12 Wertepaare mit je 3 Byte
	ldi	r19, 0			        ;Zhler fr Log-Position
;
m69400:	call	slog1b			;Sensornummer suchen
	ldi	xl,low(semax2)          ;(ungerade Nummer des Wertepaares)
	ldi	xh,high(semax2)		;Zeiger auf Sensormaximalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Sensorwert gltig?
	brne	m69410			;nein -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m69410:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
	call	slog1b			;Sensornummer suchen
	ldi	xl, low(semax2)         ;(gerade Nummer des Wertepaares)
	ldi	xh, high(semax2)	;Zeiger auf Sensormaximalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 2 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert 2 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m69420			;ja -> weiter
	swap	r16			;sonst in oberes Nibble legen
m69420:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert 1 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
	dec	r18			;alle Werte verarbeitet?
	brne	m69400			;nein -> Schleife
;
; 24h-Werte - Sensordaten aufbereiten: Maximalwerte von T25-T27 (ber Log-Tabelle 2) und P01 als 12-Bit-Daten in 6 Bytes speichern
;
        ldi     r19, 0                  ;Zhler fr Log-Position
	call	slog2b			;Sensornummer suchen (ungerade Nummer des Wertepaares)
	ldi	xl, low(semax2 + 48)    ;(ungerade Nummer des Wertepaares)
	ldi	xh, high(semax2 + 48)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m69430			;ja -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m69430:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
	call	slog2b			;Sensornummer suchen (gerade Nummer des Wertepaares)
	ldi	xl, low(semax2 + 48)    ;(gerade Nummer des Wertepaares)
	ldi	xh, high(semax2 + 48)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 2 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert 2 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m69440			;nein -> weiter
	swap	r16			;sonst in oberes Nibble legen
m69440:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert 1 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
;
	call	slog2b			;Sensornummer suchen (ungerade Nummer des Wertepaares)
	ldi	xl, low(semax2 + 48)    ;(gerade Nummer des Wertepaares)
	ldi	xh, high(semax2 + 48)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert 1 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r17, x			;Sensorwert 1 H holen
	cpi	r17, 0x80		;Temperaturwert gltig?
	brne	m69450			;ja -> weiter
	swap	r17			;sonst Kennung = 0x08 setzen
m69450:	andi	r17, 0x0f		;nur unteres Nibble verwenden
;
        ldi     xl, low(semax2 + 54)
        ldi     xh, high(semax2 + 54)   ;Zeiger auf Sensorwert von P01
	ld	r16, x+			;Sensorwert P01 L holen
	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	ld	r16, x			;Sensorwert P01 H holen
	cpi	r16, 0x80		;Sensorwert gltig?
	breq	m69460			;nein -> weiter
	swap	r16			;sonst in oberes Nibble legen
m69460:	andi	r16, 0xf0		;nur oberes Nibble verwenden
	or	r16, r17		;mit Wert P01 H verknpfen
	st	y+, r16			;verknpfte H-Werte speichern
;
; 24h-Werte - Sensordaten aufbereiten: Maximalwerte von H01-H03 (ber Log-Tabelle 2) als 8-Bit-Daten in 3 Bytes und Batteriespannung in 2 Bytes speichern
;
        ldi     r18, 3                  ;3 Werte bearbeiten
        ldi     r19, 0                  ;Zhler fr Log-Position
;
m69470:	call	slog2b			;Sensornummer suchen
	ldi	xl, low(semax2 + 56)
	ldi	xh, high(semax2 + 56)	;Zeiger auf Sensorminimalwerte
	add	xl, r20			;Tabellenoffset addieren
	adc	xh, r21			;bertrag
	ld	r16, x+			;Sensorwert L holen
        ld      r17, x+                 ;Sensorwert H holen
        cpi     r17, 0x80               ;Wert ungltig?
        brne    m69480                  ;nein -> weiter
	ldi	r16, 0x80		;sonst Wert ungltig setzen
m69480:	st	y+, r16			;und in Ext-EEPROM-Puffer legen
	dec	r18			;alle Werte verarbeitet?
	brne	m69470			;nein -> Schleife
;
        lds     r16, batval             ;Batteriespannung L holen
        lds     r17, batval + 1         ;Batteriespannung H holen
        st      y+, r16                 ;L-Wert speichern
        st      y+, r17                 ;H-Wert speichern
;
; 24h-Werte - Sensordaten aufbereiten: Mittelwerte von T01-T24 (ber Log-Tabelle 1) als 12-Bit-Daten in 36 Bytes speichern (ab Position 98)
;
	ldi	zl, low(xewbuf + 4 + 47 * 2)
	ldi	zh, high(xewbuf + 4 + 47 * 2)	;Zeiger auf Ext-EEPROM Schreib-Puffer setzen
	ldi	r24, 12			        ;12 Wertepaare mit je 3 Bytes
	clr	r8			        ;Zhler fr Log-Position
;
m69500:	ldi	r23, 2			;je 2 Mittelwerte berechnen
m69510:	ldi	xl, low(logtb1)
	ldi	xh, high(logtb1)	;Zeiger auf Log-Tabelle 1
	clr	r10			;Suchzhler fr Log-Position
m69520:	ld	r11, x+			;Tabelleneintrag holen
	cp	r8, r11			;Log-Position gefunden?
	breq	m69530			;ja -> Ende
	inc	r10			;Suchzhler erhhen
	mov	r16, r10		;Suchzhler kopieren
	cpi	r16, 24			;letzte Position erreicht?
	brcs	m69520			;nein -> Schleife
m69530:	inc	r8			;nchste Log-Position
	clr	r11			;H-Byte fr Tabellenoffset
	mov	r9, r10			;Offset kopieren
	add	r10, r9			;Offset verdoppeln
;
	ldi	yl, low(senavc)
	ldi	yh, high(senavc)	;Zeiger auf Mittelwertzhler
	add	yl, r10			;Tabellenoffset addieren
	adc	yh, r11			;bertrag
	ldi	xl, low(senavs)
	ldi	xh, high(senavs)	;Zeiger auf Mittelwertsummen
	add	r10, r9			;Offset verdreifachen
	add	xl, r10			;Tabellenoffset addieren
	adc	xh, r11			;bertrag
;
	ld	r16, x+
	ld	r17, x+
	ld	r18, x			;Mittelwertsumme holen
	bst	r18, 7			;Wert negativ?
	brtc	m69540			;nein -> weiter
	clr	r19
	clr	r20
	clr	r21			;sonst Zweierkomplement bilden
	sub	r19, r16
	sbc	r20, r17
	sbc	r21, r18		;Wert = 0 - Wert
	mov	r16, r19
	mov	r17, r20
	mov	r18, r21		;Ergebnis kopieren
m69540:	lsl	r16
	rol	r17			;Wert verdoppeln zur Erhhung
	rol	r18			;der Genauigkeit
	ld	r20, y+
	ld	r21, y			;Mittelwertzhler holen
	mov	r22, r20		;L kopieren
	or	r22, r21		;mit H verknpfen
	brne	m69550			;Zhler = 0? nein -> dividieren
	clr	r16
	ldi	r17, 0x80		;sonst Wert = 0x8000 setzen
	rjmp	m69570
;
m69550:	call	divide			;Mittelwert berechnen
	lsr	r17
	ror	r16			;Wert halbieren
	brcc	m69560			;Nachkommastelle = 0? nein -> weiter
	inc	r16			;sonst Ergebnis aufrunden
	brne	m69560			;bertrag? nein -> weiter
	inc	r17			;sonst H-Byte erhhen
m69560:	brtc	m69570			;war Mittelwertsumme negativ?
	clr	r18			;nein -> weiter
	clr	r19			;sonst Zweierkomplement bilden
	sub	r18, r16
	sbc	r19, r17		;Wert = 0 - Wert
	movw	r16, r18		;Ergebnis kopieren
;
m69570:	cpi	r23, 2			;erster Wert?
	brne	m69580			;nein -> weiter
	movw	r12, r16		;sonst Wert sichern
m69580:	dec	r23			;alle Werte des Paares berechnet?
	breq	m69590			;ja -> weiter
	rjmp	m69510			;sonst Schleife
;
m69590:	movw	r18, r12		;ersten Wert holen
	st	z+, r18			;Wert 1 L in Ext-EEPROM-Puffer
	cpi	r19, 0x80		;Wert 1 gltig?
	brne	m69600			;ja -> weiter
	swap	r19			;sonst Kennung = 0x08 setzen
m69600:	andi	r19, 0x0f		;nur unteres Nibble verwenden
	st	z+, r16			;Wert 2 L in Ext-EEPROM-Puffer
	cpi	r17, 0x80		;Wert 2 gltig?
	breq	m69610			;nein -> weiter
	swap	r17			;sonst in oberes Nibble legen
m69610:	andi	r17, 0xf0		;nur oberes Nibble verwenden
	or	r17, r19		;mit Wert 1 H verknpfen
	st	z+, r17			;verknpfte H-Werte speichern
	dec	r24			;alle Werte verarbeitet?
	breq	m69620			;ja -> weiter
	rjmp	m69500			;sonst Schleife
;
; 24h-Werte - Sensordaten aufbereiten: Mittelwerte von T25-T27 (ber Log-Tabelle 2) und P01 als 12-Bit-Daten in 6 Bytes speichern
;
m69620:	ldi	r24, 2		        ;2 Wertepaare mit je 3 Bytes
	clr	r8		        ;Zhler fr Log-Position
;
m69630:	ldi	r23, 2			;je 2 Mittelwerte berechnen
m69640:	ldi	xl, low(logtb2)
	ldi	xh, high(logtb2)	;Zeiger auf Log-Tabelle 2
	clr	r10			;Suchzhler fr Log-Position
m69650:	ld	r11, x+			;Tabelleneintrag holen
	cp	r8, r11			;Log-Position gefunden?
	breq	m69660			;ja -> Ende
	inc	r10			;Suchzhler erhhen
	mov	r16, r10		;Suchzhler kopieren
	cpi	r16, 3			;letzte Position erreicht?
	brcs	m69650			;nein -> Schleife
m69660:	inc	r8			;nchste Log-Position
;
        cpi     r24, 1                  ;letztes Wertepaar?
        brne    m69670                  ;nein -> weiter
        cpi     r23, 1                  ;letzter Wert (Luftdruck)
        brne    m69670                  ;nein -> weiter
        ldi     yl, low(senavc + 54)    ;sonst
        ldi     yh, high(senavc + 54)   ;Zeiger auf Mittelwertzhler Luftdruck
        ldi     xl, low(senavs + 81)
        ldi     xh, high(senavs + 81)   ;Zeiger auf Mittelwertsumme Luftdruck
        rjmp    m69680
;
m69670:	clr	r11			;H-Byte fr Tabellenoffset
	mov	r9, r10			;Offset kopieren
	add	r10, r9			;Offset verdoppeln
	ldi	yl, low(senavc + 48)
	ldi	yh, high(senavc + 48)	;Zeiger auf Mittelwertzhler
	add	yl, r10			;Tabellenoffset addieren
	adc	yh, r11			;bertrag
	ldi	xl, low(senavs + 72)
	ldi	xh, high(senavs + 72)	;Zeiger auf Mittelwertsummen
	add	r10, r9			;Offset verdreifachen
	add	xl, r10			;Tabellenoffset addieren
	adc	xh, r11			;bertrag
;
m69680:	ld	r16, x+
	ld	r17, x+
	ld	r18, x			;Mittelwertsumme holen
	bst	r18, 7			;Wert negativ?
	brtc	m69690			;nein -> weiter
	clr	r19
	clr	r20
	clr	r21			;sonst Zweierkomplement bilden
	sub	r19, r16
	sbc	r20, r17
	sbc	r21, r18		;Wert = 0 - Wert
	mov	r16, r19
	mov	r17, r20
	mov	r18, r21		;Ergebnis kopieren
m69690:	lsl	r16
	rol	r17			;Wert verdoppeln zur Erhhung
	rol	r18			;der Genauigkeit
	ld	r20, y+
	ld	r21, y			;Mittelwertzhler holen
	mov	r22, r20		;L kopieren
	or	r22, r21		;mit H verknpfen
	brne	m696a0			;Zhler = 0? nein -> dividieren
	clr	r16
	ldi	r17, 0x80		;sonst Wert = 0x8000 setzen
	rjmp	m696c0
;
m696a0:	call	divide			;Mittelwert berechnen
	lsr	r17
	ror	r16			;Wert halbieren
	brcc	m696b0			;Nachkommastelle = 0? nein -> weiter
	inc	r16			;sonst Ergebnis aufrunden
	brne	m696b0			;bertrag? nein -> weiter
	inc	r17			;sonst H-Byte erhhen
m696b0:	brtc	m696c0			;war Mittelwertsumme negativ?
	clr	r18			;nein -> weiter
	clr	r19			;sonst Zweierkomplement bilden
	sub	r18, r16
	sbc	r19, r17		;Wert = 0 - Wert
	movw	r16, r18		;Ergebnis kopieren
;
m696c0:	cpi	r23, 2			;erster Wert?
	brne	m696d0			;nein -> weiter
	movw	r12, r16		;sonst Wert sichern
m696d0:	dec	r23			;alle Werte des Paares berechnet?
	breq	m696e0			;ja -> weiter
	rjmp	m69640			;sonst Schleife
;
m696e0:	movw	r18, r12		;ersten Wert holen
	st	z+, r18			;Wert 1 L in Ext-EEPROM-Puffer
	cpi	r19, 0x80		;Wert 1 gltig?
	brne	m696f0			;ja -> weiter
	swap	r19			;sonst Kennung = 0x08 setzen
m696f0:	andi	r19, 0x0f		;nur unteres Nibble verwenden
	st	z+, r16			;Wert 2 L in Ext-EEPROM-Puffer
	cpi	r17, 0x80		;Wert 2 gltig?
	breq	m696g0			;nein -> weiter
	swap	r17			;sonst in oberes Nibble legen
m696g0:	andi	r17, 0xf0		;nur oberes Nibble verwenden
	or	r17, r19		;mit Wert 1 H verknpfen
	st	z+, r17			;verknpfte H-Werte speichern
	dec	r24			;alle Werte verarbeitet?
	breq	m696h0			;ja -> weiter
	rjmp	m69630			;sonst Schleife
;
; 24h-Werte - Sensordaten aufbereiten: Mittelwerte von H01-H03 (ber Log-Tabelle 2) als 8-Bit-Daten in 3 Bytes und Batteriespannung in 2 Bytes speichern
;
m696h0:	ldi	r24, 3		        ;3 Werte bearbeiten
	clr	r8		        ;Zhler fr Log-Position
;
m696i0:	ldi	xl, low(logtb2)
	ldi	xh, high(logtb2)	;Zeiger auf Log-Tabelle 2
	clr	r10			;Suchzhler fr Log-Position
m696j0:	ld	r11, x+			;Tabelleneintrag holen
	cp	r8, r11			;Log-Position gefunden?
	breq	m696k0			;ja -> Ende
	inc	r10			;Suchzhler erhhen
	mov	r16, r10		;Suchzhler kopieren
	cpi	r16, 3			;letzte Position erreicht?
	brcs	m696j0			;nein -> Schleife
m696k0:	inc	r8			;nchste Log-Position
;
	clr	r11			;H-Byte fr Tabellenoffset
	mov	r9, r10			;Offset kopieren
	add	r10, r9			;Offset verdoppeln
	ldi	yl, low(senavc + 56)
	ldi	yh, high(senavc + 56)	;Zeiger auf Mittelwertzhler
	add	yl, r10			;Tabellenoffset addieren
	adc	yh, r11			;bertrag
	ldi	xl, low(senavs + 84)
	ldi	xh, high(senavs + 84)	;Zeiger auf Mittelwertsummen
	add	r10, r9			;Offset verdreifachen
	add	xl, r10			;Tabellenoffset addieren
	adc	xh, r11			;bertrag
;
	ld	r16, x+
	ld	r17, x+
	ld	r18, x			;Mittelwertsumme holen
	lsl	r16
	rol	r17			;Wert verdoppeln zur Erhhung
	rol	r18			;der Genauigkeit
	ld	r20, y+
	ld	r21, y			;Mittelwertzhler holen
	mov	r22, r20		;L kopieren
	or	r22, r21		;mit H verknpfen
	brne	m696l0			;Zhler = 0? nein -> dividieren
	clr	r16
	ldi	r17, 0x80		;sonst Wert = 0x8000 setzen
	rjmp	m696m0
;
m696l0:	call	divide			;Mittelwert berechnen
	lsr	r17
	ror	r16			;Wert halbieren
	brcc	m696m0			;Nachkommastelle = 0? nein -> weiter
	inc	r16			;sonst Ergebnis aufrunden
	brne	m696m0			;bertrag? nein -> weiter
	inc	r17			;sonst H-Byte erhhen
m696m0:	cpi	r17, 0x80		;Wert gltig?
	brne	m696n0			;ja -> weiter
	ldi	r16, 0x80		;sonst Kennung = 0x80 setzen
m696n0:	st	z+, r16			;Wert (8 Bit) in Ext-EEPROM-Puffer
	dec	r24			;alle Werte verarbeitet?
	brne	m696i0			;nein -> Schleife
;
        lds     r16, batval             ;Batteriespannung L holen
        lds     r17, batval + 1         ;Batteriespannung H holen
        st      z+, r16                 ;L-Wert speichern
        st      z+, r17                 ;H-Wert speichern
;
; 24h-Werte - aufbereitete Daten ins Ext-EEPROM schreiben und Pointer bearbeiten
;
	lds	r20, xes2po
	lds	r21, xes2po + 1		;Zeiger auf aktuelle Position
	lds	r22, xes2po + 2		;im externen EEPROM holen
	ldi	r23, xes2da		;145 Bytes zum Ext-EEPROM senden
	call	xwrite			;Puffer zum Ext-EEPROM senden
	brcc	m69770			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	m69790			;Adressberechnung berspringen
;
m69770:	ldi	r16, low(xs2sta + xs2len - xes2da)
	ldi	r17, high(xs2sta + xs2len - xes2da)
	ldi	r18, byte3(xs2sta + xs2len - xes2da)    ;Adresse des letzten mglichen Datensatzbeginns laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;aktuelle Adresse noch im gltigen Speicherbereich?
	brcc	m69780			;ja -> weiter
	ldi	r20, low(xs2sta)
	ldi	r21, high(xs2sta)	;sonst
	ldi	r22, byte3(xs2sta)	;Adresse wieder auf Anfang setzen
m69780:	sts	xes2po, r20
	sts	xes2po + 1, r21		;Zeiger fr Position im
	sts	xes2po + 2, r22		;externen EEPROM speichern
;
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 4			;RTC-Fehler?
	brts	m69790			;ja -> berspringen
	ldi	r18, rtxs2p		;RTC-Adresse Sensordaten 2
	ldi	xl, low(xes2po)
	ldi	xh, high(xes2po)	;Zeiger auf Sensordaten 2 Pointer
	call	i2psen			;Pointer in RTC schreiben
	brcc	m69790			;Fehler? nein -> weiter
	call	i2cerr			;sonst RTC Fehlerstatus setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
;
m69790:	call	rsmin2			;Minimalwerte (Log) lschen
	call	rsmax2			;Maximalwerte (Log) lschen
	call	resavg			;Mittelwerte (Log) lschen
;
; Zur Minute 14, 29, 44 und 59 (auer 23:59) ein Backup der Min-, Max- und Mittelwerte in das RTC-RAM schreiben
;
m69800:	lds	r16, xewbuf		;aktuelle Zeit (Minuten) holen
	cpi	r16, 14			;Minuten = 14?
	breq	m69810			;ja -> bearbeiten
	cpi	r16, 29			;Minuten = 29?
	breq	m69810			;ja -> bearbeiten
	cpi	r16, 44			;Minuten = 44?
	breq	m69810			;ja -> bearbeiten
	cpi	r16, 59			;Minuten = 59?
	brne	m69900			;nein -> berspringen
	lds	r16, xewbuf + 1		;aktuelle Zeit (Stunden) holen
	cpi	r16, 23			;Stunde = 23 (Zeit 23:59)?
	breq	m69900			;ja -> berspringren
;
m69810:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 4			;RTC-Fehler?
	brts	m69900			;ja -> berspringen
	call	i2back			;Werte im RTC-RAM sichern
	brcc	m69900			;Fehler? nein -> weiter
	call	i2cerr			;sonst RTC-Fehlerstatus setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
;
; Zur Minute 1, 16, 31 und 46 neue Daten fr die grafische Luftdruckanzeige generieren
;
m69900:	lds	r16, xewbuf		;aktuelle Zeit (Minuten) holen
	cpi	r16, 1			;Minuten = 1?
	breq	m69910			;ja -> bearbeiten
	cpi	r16, 16			;Minuten = 16?
	breq	m69910			;ja -> bearbeiten
	cpi	r16, 31			;Minuten = 31?
	breq	m69910			;ja -> bearbeiten
	cpi	r16, 46			;Minuten = 46?
	brne	m70000			;ja -> bearbeiten
;
m69910: lds     r16, errflg             ;Fehler-Flags holen
        bst     r16, 3                  ;Speicher-Fehler?
        brts    m70000                  ;ja -> berspringen
        call    genprs                  ;sonst Daten fr grafische Luftdruckanzeige generieren
        lds     r16, senflg + 31        ;Flags vom virtuellen Sensor 31 holen
        sbr     r16, 0x01               ;Daten neu ausgeben
        sts     senflg + 31, r16        ;Sensor-Flags wieder speichern
;
;
; Mensteuerung: Auswertung der Taster
;
m70000:	lds	r16, keyst2		;Tastenstatus holen
	lds	r17, keyst3		;Quittierungsstatus holen
	bst	r16, 6			;wurde Taster 1 gedrckt?
	brtc	m70010			;nein -> weiter testen
	bst	r17, 6			;wurde Taster 1 quittiert?
	brts	m70010			;ja -> Taster 1 ignorieren
	rjmp	m72000			;nein -> Taster 1 bearbeiten
m70010:	bst	r16, 7			;wurde Taster 2 gedrckt?
	brtc	m70020			;nein -> weiter testen
	bst	r17, 7			;wurde Taster 2 quittiert?
	brts	m70020			;ja -> Taster 2 ignorieren
	rjmp	m73000			;sonst Taster 2 bearbeiten
m70020:	lds	r16, encpos		;aktuelle Impulsgeber-Position
	lds	r17, encmem		;gespeicherte Impulsgeber-Position
	cp	r16, r17		;hat sich der Wert gendert?
	brne	m71000			;ja -> bearbeiten
	jmp	m99400
;
; Mensteuerung: Bearbeiten einer Impulsgeber-Vernderung (Mentext)
;
m71000:	lds	r18, dflags		;sonst Anzeige-Flags holen
	bst	r18, 7			;LCD wegen Syncronisierung abgeschaltet?
	brtc	m71010			;nein -> weiter
	sts	encpos, r17		;aktuelle Impulsgeber-Position lschen
	call	lcdon			;LCD wieder einschalten
	jmp	m99400			;Ende
;
m71010:	sts	encmem, r16		;neue Impulsgeber-Position speichern
	sub	r16, r17		;Anzahl der Schritte ermitteln
	lds	r18, encflg		;Impulsgeber-Flags holen
	bst	r18, 0			;Impulsgeberwert im Men anzeigen?
	brts	m71100			;ja -> bearbeiten
	lds	r10, menopt		;aktuelle Menoption holen
	add	r10, r16		;Schrittanzahl addieren
	tst	r10			;ermittelte Menoption < 0?
	brpl	m71020			;nein -> weiter testen
	clr	r10			;sonst Menuoption 0 setzen
	rjmp	m71030			;neue Option speichern
;
m71020:	call	men1rd			;Mentabellenheader lesen
	cp	r10, r11		;Option > Maximalwert?
	brcs	m71030			;nein -> neue Option speichern
	mov	r10, r11		;sonst auf letzte Option setzen
	dec	r10			;und korrigieren
m71030:	lds	r16, menopt		;bisherige Menoption holen
	cp	r10, r16		;nderung?
	breq	m71050			;nein -> weiter
	sts	menopt, r10		;sonst neue Option speichern
m71040:	lds	r18, dflags		;Anzeigeflags holen
	sbr	r18, 0x08		;Menzeile neu ausgeben
	sts	dflags, r18		;Anzeigeflags wieder speichern
m71050:	jmp	m99300
;
; Mensteuerung: Bearbeiten einer Impulsgeber-Vernderung (3-stellig dezimal)
;
m71100:	sbr	r18, 1 << 7		;nderungs-Flag setzen
	sts	encflg, r18		;Impulsgeber-Flags wieder speichern
	bst	r18, 1			;Anzeige 3-stellig dezimal?
	brtc	m71200			;nein -> weiter testen
;
	lds	r17, encval		;Impulsgeberwert holen
	add	r16, r17		;Schrittanzahl addieren
	cpi	r16, 225		;Wert > 225 (negativer berlauf)?
	brcs	m71110			;nein -> weiter
	clr	r16			;sonst Wert = 0 setzen
m71110:	cpi	r16, 200		;Wert > 200 (positiver berlauf)?
	brcs	m71120			;nein -> weiter
	ldi	r16, 199		;sonst Wert = 199 setzen
m71120:	lds	r17, encmax		;Impulsgeber Maximalwert holen
	cp	r17, r16		;Maximalwert berschritten?
	brcc	m71130			;nein -> weiter
	mov	r16, r17		;sonst auf Maximalwert setzen
m71130:	lds	r17, encmin		;Impulsgeber Minimalwert holen
	cp	r16, r17		;Minimalwert unterschritten?
	brcc	m71140			;nein -> weiter
	mov	r16, r17		;sonst auf Minimalwert setzen
m71140:	sts	encval, r16		;neuen Impulsgeberwert speichern
	rjmp	m71040			;Menzeile neu ausgeben
;
; Mensteuerung: Bearbeiten einer Impulsgeber-Vernderung (Temperaturwert)
;
m71200:	bst	r18, 2			;Anzeige Temperaturwert?
	brtc	m71300			;nein -> weiter testen
	mov	r18, r16		;Impulsgeber-Schrittanzahl kopieren
	tst	r18			;Wert negativ?
	brpl	m71210			;nein -> weiter
	neg	r18			;sonst in positiven Wert wandeln
m71210:	cpi	r18, 2			;Schrittwert < 2?
	brcs	m71220			;ja -> weiter
	dec	r18			;sonst um 1 vermindern und
m71220:	muls	r16, r18		;neue Schrittanzahl berechnen
;
	lds	r18, encval		;Impulsgeberwert L holen
	lds	r19, encvah		;Impulsgeberwert H holen
	add	r18, r0			;Schrittanzahl addieren
	adc	r19, r1			;bertrag addieren
	ldi	r16, low(maxtmp)
	ldi	r17, high(maxtmp)	;Temperaturmaximalwert laden
	sub	r16, r18
	sbc	r17, r19		;Maximalwert berschritten?
	brpl	m71230			;nein -> weiter
	ldi	r18, low(maxtmp)	;sonst
	ldi	r19, high(maxtmp)	;Temperaturmaximalwert laden
m71230:	movw	r20, r18		;Impulsgeberwert sichern
	ldi	r16, low(mintmp)
	ldi	r17, high(mintmp)	;Temperaturminimalwert laden
	sub	r18, r16
	sbc	r19, r17		;Minimalwert unterschritten?
	movw	r18, r20		;Impulsgeberwert restaurieren
	brpl	m71240			;nein -> weiter
	ldi	r18, low(mintmp)	;sonst
	ldi	r19, high(mintmp)	;Temperaturminimalwert laden
m71240:	sts	encval, r18		;neuen Impulsgeberwert L speichern
	sts	encvah, r19		;neuen Impulsgeberwert H speichern
	rjmp	m71040			;Menzeile neu ausgeben
;
; Mensteuerung: Bearbeiten einer Impulsgeber-Vernderung (ASCII-Zeichen)
;
m71300:	bst	r18, 3			;Anzeige ASCII-Zeichen?
	brtc	m71400			;nein -> weiter testen
	mov	r18, r16		;Schrittanzahl sichern
	lds	r17, encval		;Impulsgeberwert holen
	add	r16, r17		;Schrittanzahl addieren
        tst     r18                     ;Schrittanzahl negativ?
        brmi    m71310                  ;ja - > bearbeiten
;
	lds	r17, asclst		;Anzahl der Zeichen holen
        subi    r17, -2                 ;Steuerzeichen Del und Ok einbeziehen
	cp	r16, r17		;letztes Zeichen berschritten?
	brcs	m71320			;nein -> weiter
	lds	r16, asclst		;sonst letztes Zeichen setzen
        inc     r16                     ;korrigieren fr Del und Ok
        rjmp    m71320                  ;neuen Wert speichern
;
m71310:	cpi	r16, 225		;Wert > 225 (negativer berlauf)?
	brcs	m71320			;nein -> weiter
	clr	r16		        ;sonst erstes Zeichen setzen
m71320:	sts	encval, r16		;neuen Impulsgeberwert speichern
	rjmp	m71040			;Menzeile neu ausgeben
;
; Mensteuerung: Bearbeiten einer Impulsgeber-Vernderung (4-stellig dezimal)
;
m71400:	bst	r18, 4			;Anzeige 4-stellig dezimal?
	brtc	m71500			;nein -> weiter testen
	mov	r18, r16		;Impulsgeber-Schrittanzahl kopieren
	tst	r18			;Wert negativ?
	brpl	m71410			;nein -> weiter
	neg	r18			;sonst in positiven Wert wandeln
m71410:	ldi	r17, 1			;Increment/Decrement auf 1 setzen
	cpi	r18, 3			;mehr als 2 Schritte?
	brcs	m71420			;nein -> berspringen, sonst
	ldi	r17, 8			;Increment/Decrement auf 8 setzen
	cpi	r18, 5			;mehr als 4 Schritte?
	brcs	m71420			;nein -> berspringen, sonst
	ldi	r17, 20			;Increment/Decrement auf 20 setzen
m71420:	muls	r16, r17		;mit Schrittanzahl multiplizieren
;
	lds	r18, encval		;Impulsgeberwert L holen
	lds	r19, encvah		;Impulsgeberwert H holen
	add	r18, r0			;Schrittanzahl addieren
	adc	r19, r1			;bertrag addieren
	lds	r16, encmax
	lds	r17, encmah		;Maximalwert laden
	sub	r16, r18
	sbc	r17, r19		;Maximalwert berschritten?
	brpl	m71430			;nein -> weiter
	lds	r18, encmax		;sonst
	lds	r19, encmah		;Maximalwert laden
m71430:	movw	r20, r18		;Impulsgeberwert sichern
	lds	r16, encmin
	lds	r17, encmih		;Minimalwert laden
	sub	r18, r16
	sbc	r19, r17		;Minimalwert unterschritten?
	movw	r18, r20		;Impulsgeberwert restaurieren
	brpl	m71440			;nein -> weiter
	lds	r18, encmin		;sonst
	lds	r19, encmih		;Minimalwert laden
m71440:	sts	encval, r18		;neuen Impulsgeberwert L speichern
	sts	encvah, r19		;neuen Impulsgeberwert H speichern
	rjmp	m71040			;Menzeile neu ausgeben
;
; Mensteuerung: Bearbeiten einer Impulsgeber-Vernderung (Zeitwert)
;
m71500:	bst	r18, 5			;Anzeige Temperaturwert?
	brtc	m71600			;nein -> weiter testen
	mov	r18, r16		;Impulsgeber-Schrittanzahl kopieren
	tst	r18			;Wert negativ?
	brpl	m71510			;nein -> weiter
	neg	r18			;sonst in positiven Wert wandeln
m71510:	ldi	r17, 1			;Increment/Decrement auf 1 setzen
	cpi	r18, 3			;mehr als 2 Schritte?
	brcs	m71520			;nein -> berspringen, sonst
	ldi	r17, 8			;Increment/Decrement auf 8 setzen
m71520:	muls	r16, r17		;mit Schrittanzahl multiplizieren
;
	lds	r18, encval		;Impulsgeberwert L holen
	lds	r19, encvah		;Impulsgeberwert H holen
	add	r18, r0			;Schrittanzahl addieren
	adc	r19, r1			;bertrag addieren
	ldi	r16, low(23 * 60 + 59)
	ldi	r17, high(23 * 60 + 59)	;Maximalwert (23:59) laden
	sub	r16, r18
	sbc	r17, r19		;Maximalwert berschritten?
	brpl	m71530			;nein -> weiter
	clr	r18			;sonst
	clr	r19			;wieder bei 0 beginnen
m71530:	movw	r20, r18		;Impulsgeberwert sichern
	clr	r16
	clr	r17			;Minimalwert (0) laden
	sub	r18, r16
	sbc	r19, r17		;Minimalwert unterschritten?
	movw	r18, r20		;Impulsgeberwert restaurieren
	brpl	m71540			;nein -> weiter
	ldi	r18, low(23 * 60 + 59)	;sonst wieder beim
	ldi	r19, high(23 * 60 + 59)	;Maximalwert (23:59) beginnen
m71540:	sts	encval, r18		;neuen Impulsgeberwert L speichern
	sts	encvah, r19		;neuen Impulsgeberwert H speichern
        rjmp    m71040                  ;Menzeile neu ausgeben
;
; Mensteuerung: Bearbeiten einer Impulsgeber-Vernderung (Sensorname)
;
m71600:	bst	r18, 6			;Anzeige als Sensorname?
	brtc	m71700			;nein -> weiter testen
	lds	r17, encval		;Impulsgeberwert holen
	add	r16, r17		;Schrittanzahl addieren
	cpi	r16, 225		;Wert > 225 (negativer berlauf)?
	brcs	m71610			;nein -> weiter
	clr	r16			;sonst Wert = 0 setzen
m71610:	cpi	r16, 200		;Wert > 200 (positiver berlauf)?
	brcs	m71620			;nein -> weiter
	ldi	r16, 199		;sonst Wert = 199 setzen
m71620:	lds	r17, encmax		;Impulsgeber Maximalwert holen
	cp	r17, r16		;Maximalwert berschritten?
	brcc	m71630			;nein -> weiter
	mov	r16, r17		;sonst auf Maximalwert setzen
m71630:	lds	r17, encmin		;Impulsgeber Minimalwert holen
	cp	r16, r17		;Minimalwert unterschritten?
	brcc	m71640			;nein -> weiter
	mov	r16, r17		;sonst auf Minimalwert setzen
m71640:	sts	encval, r16		;neuen Impulsgeberwert speichern
m71700:	rjmp	m71040			;Menzeile neu ausgeben
;
; Mensteuerung: Bearbeiten der Bettigung von Taster 1 (rechts)
;
m72000:	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 7			;LCD wegen Synchronisierung abgeschaltet?
	brtc	m72010			;nein -> weiter
	call	lcdon			;sonst LCD wieder einschalten
	jmp	m99400			;Ende
;
m72010:	cli				;Interrupts sperren
	lds	r16, keyst3		;Tasterquittungs-Flags holen
	sbr	r16, 0x40		;Taster 1 quittieren
	sts	keyst3, r16		;Flags wieder speichern
	sei				;Interrupts wieder freigeben
	clr	r16			;zunchst
	sts	menext, r16		;Menerweiterung lschen
	call	men1rd			;Mentabellenheader lesen
	call	men2rd			;Mentabellenoption lesen
	elpm	r16, z+			;Infotextstring lesen (dummy)
	elpm	r16, z+			;Untermen-Nummer lesen
	cpi	r16, 0xff		;gltiges Untermen?
	breq	m72020			;nein -> Men nicht ndern
	sts	mennum, r16		;neue Mennummer setzen
	clr	r16
	sts	menopt, r16		;neue Menoption 0 setzen
m72020:	elpm	r16, z+			;Programmadresse L holen
	elpm	r17, z			;Programmadresse H holen
	tst	r17			;gltige Programmadresse?
	brne	m72030			;ja -> Programm aufrufen
	lds	r16, dflags		;sonst Anzeigeflags holen
	sbr	r16, 0x0c		;Info/Menzeile neu ausgeben
	sts	dflags, r16		;Anzeigeflags wieder speichern
	jmp	m99300
m72030:	movw	zl, r16			;Adresse kopieren
	ijmp				;Menprogramm starten
;
; Mensteuerung: Bearbeiten der Bettigung von Taster 2 (links)
;
m73000:	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 7			;LCD wegen Synchronisierung abgeschaltet?
	brtc	m73010			;nein -> weiter
	call	lcdon			;sonst LCD wieder einschalten
	jmp	m99400			;Ende
;
m73010:	cli				;Interrupts sperren
	lds	r16, keyst3		;Tasterquittungs-Flags holen
	sbr	r16, 0x80		;Taster 2 quittieren
	sts	keyst3, r16		;Flags wieder speichern
	sei				;Interrupts wieder freigeben
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert anzeigen?
	brts	m73120			;ja -> bearbeiten
;
m73020:	lds	r16, mennum		;aktuelle Mennummer holen
	tst	r16			;Hauptmen (0)?
	breq	m73100			;ja -> bearbeiten
	call	men1rd			;Mentabellenheader lesen
	elpm	r16, z+			;bergeordnete Mennummer holen
	elpm	r17, z			;bergeordnete Menoption holen
	bst	r17, 7			;Neuausgabe-Flag sichern
	andi	r17, 0x7f		;bergeordnete Menoption filtern
	tst	r17			;Menoption = 0?
	breq	m73030			;ja -> weiter
	dec	r17			;sonst korrigieren
m73030:	sts	mennum, r16		;neue Mennummer setzen
	sts	menopt, r17		;neue Menoption setzen
	lds	r18, dflags		;Anzeigeflags holen
	sbr	r18, 0x0c		;Info/Menzeile neu ausgeben
	brtc	m73040			;Neuausgabe? nein -> weiter
	sbr	r18, 0x03		;sonst Temperatur-Anzeige neu ausgebeben
m73040:	sts	dflags, r18		;Anzeigeflags wieder speichern
	lds	r16, xflags		;verschiedene Flags holen
	cbr	r16, 0x0c		;Min/Max-Anzeige-Flag lschen
	sts	xflags, r16		;Flags wieder speichern
m73050:	clr	r16
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99300
;
m73100:	lds	r16, xflags		;verschiedene Flags holen
	bst	r16, 0			;sind Signale aktiv?
	brtc	m73105			;nein -> weiter
	rjmp	m73200			;sonst Tonausgabe stoppen
;
m73105:	bst	r16, 1			;sind Erinnerungen gesetzt?
	brts	m73110			;ja -> Erinnerungen lschen
	jmp	m99300			;sonst Ende
m73110:	rjmp	m73300			;Erinnerungen lschen
;
m73120:	clr	r16			;Impulsgeber-Flags zurcksetzen
	sts	encflg, r16		;und speichern
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	lds	r18, dflags		;Anzeigeflags holen
	sbr	r18, 0x0c		;Info/Menzeile neu ausgeben
	sts	dflags, r18		;Anzeigeflags wieder speichern
	lds	r16, dimode		;Anzeigemodus holen
	cpi	r16, 4			;Modus Konfiguration - Sensoren - Anzeige?
	brne	m73130			;nein -> weiter testen
	call	newsen			;sonst Daten neu ausgeben
	rjmp	m73050
;
m73130:	cpi	r16, 5			;Modus Konfiguration - Sensoren - Einstellungen, Name?
	brne	m73140			;nein -> weiter testen
	ldi	r16, 3			;sonst
	sts	dimode, r16		;Anzeigemodus 3 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	newsen			;Daten neu ausgeben
	rjmp	m73050
;
m73140:	cpi	r16, 7			;Modus Konfiguration - Sensoren - Grenzwerte, ndern?
	brne	m73150			;nein -> weiter testen
	ldi	r16, 6			;sonst
	sts	dimode, r16		;Anzeigemodus 6 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	cbr	r16, 3			;Flags fr Grenzwertnderung lschen
	sts	xflag4, r16		;Flags wieder speichern
	call	newsen			;Daten neu ausgeben
	rjmp	m73050
;
m73150:	cpi	r16, 9			;Modus Konfiguration - Alarme - Verschieben?
	brne	m73160			;nein -> weiter testen
	call	calmbk			;sonst Anzeige-Positions-Tabelle kopieren
	call	newalm			;und Daten neu ausgeben
	rjmp	m73050
;
m73160:	cpi	r16, 10			;Modus Konfiguration - Alarme - Name?
	brne	m73170			;nein -> weiter testen
	ldi	r16, 8			;sonst
	sts	dimode, r16		;Anzeigemodus 8 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	newalm			;Daten neu ausgeben
	rjmp	m73050
;
m73170:	cpi	r16, 12			;Modus Konfiguration - Signale - Zuordnung, ndern?
	brne	m73180			;nein -> weiter testen
	ldi	r16, 11			;sonst
	sts	dimode, r16		;Anzeigemodus 11 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	cbr	r16, 1 << 2 | 1 << 3	;Flags "Nummer/Dauer ndern" lschen
	sts	xflag4, r16		;Flags wieder speichern
	call	newcf2			;Daten neu ausgeben
	rjmp	m73050
;
m73180:	cpi	r16, 14			;Modus Konfiguration - Signale - Audio, ndern?
	brne	m73190			;nein -> weiter testen
	ldi	r16, 13			;sonst
	sts	dimode, r16		;Anzeigemodus 13 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	newcf2			;Daten neu ausgeben
	rjmp	m73050
;
m73190:	cpi	r16, 16			;Modus Konfiguration - Zeiten - Schaltzeiten, ndern?
	brne	m731a0			;nein -> weiter testen
	ldi	r16, 15			;sonst
	sts	dimode, r16		;Anzeigemodus 15 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	newcf2			;Daten neu ausgeben
	rjmp	m73050
;
m731a0:	cpi	r16, 18			;Modus Konfiguration - Zeiten - Uhr stellen, ndern?
	brne	m731b0			;nein -> weiter testen
	ldi	r16, 17			;sonst
	sts	dimode, r16		;Anzeigemodus 17 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	newcf2			;Daten neu ausgeben
	rjmp	m73050
;
m731b0:	cpi	r16, 20			;Modus Konfiguration - Sonstiges 1, ndern?
	brne	m731c0			;nein -> weiter testen
	ldi	r16, 19			;sonst
	sts	dimode, r16		;Anzeigemodus 19 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	newcf2			;Daten neu ausgeben
	rjmp	m73050
;
m731c0:	cpi	r16, 24			;Modus Grafik - Preset, Fadenkreuz"?
	brne	m731d0			;nein -> weiter testen
	ldi	r16, 23			;sonst
	sts	dimode, r16		;Anzeigemodus 23 setzen
	lds	r18, grpoin		;aktuelle Fadenkreuzposition holen
	call	grscrr			;Grafik wieder herstellen
	rjmp	m73020			;eine Menebene zurck
;
m731d0:	cpi	r16, 26			;Modus Konfiguration - Grafik-Presets, ndern?
	brne	m731e0			;nein -> weiter testen
	ldi	r16, 25			;sonst
	sts	dimode, r16		;Anzeigemodus 25 setzen
	call	newcf2			;Datenfelder neu ausgeben
	rjmp	m73050
;
m731e0: cpi     r16, 27                 ;Modus Konfiguration - Sensoren - Anzeige, ndern?
        brne    m731f0                  ;nein -> weiter testen
        ldi     r16, 4                  ;sonst
        sts     dimode, r16             ;Anzeigemodus Konfiguration - Sensoren - Anzeige setzen
	call	newpos			;Anzeigepositionen neu ausgeben
	rjmp	m73050
;
m731f0: cpi     r16, 29                 ;Modus Konfiguration - Sensoren - Groanzeige, ndern?
        brne    m731g0                  ;nein -> weiter testen
        ldi     r16, 28                 ;sonst
        sts     dimode, r16             ;Anzeigemodus Konfiguration - Sensoren - Groanzeige setzen
	call	newpos			;Anzeigepositionen neu ausgeben
	rjmp	m73050
;
m731g0: cpi     r16, 22                 ;Modus Konfiguration - Sensoren - Log-Zuordnung 1, ndern?
        brne    m731h0                  ;nein -> weiter testen
        ldi     r16, 21                 ;sonst
        sts     dimode, r16             ;Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 1 setzen
        call    newval                  ;Sensorwerte neu ausgeben
	rjmp	m73050
;
m731h0: cpi     r16, 31                 ;Modus Konfiguration - Sensoren - Log-Zuordnung 2, ndern?
        brne    m731i0                  ;nein -> weiter testen
        ldi     r16, 30                 ;sonst
        sts     dimode, r16             ;Anzeigemodus Konfiguration - Sensoren - Log-Zuordnung 2 setzen
        call    newval                  ;Sensorwerte neu ausgeben
m731i0:	rjmp	m73050
;
; Mensteuerung Taster 2: Signal ausschalten
;
m73200:	lds	r16, dflags		;Anzeigeflags holen
	cbr	r16, 0x20		;Lautsprecher abschalten
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeigeflags wieder speichern
	ldi	r16, timlsp		;Timeout-Wert Lautsprecher-Abschaltung
	sts	lsptio, r16		;neu setzen
	jmp	m99300
;
; Mensteuerung Taster 2: Erinnerungen lschen
;
m73300:	lds	r16, errflg		;Fehler-Flags holen
	cbr	r16, 0x80		;Erinnerung lschen
	sts	errflg, r16		;Fehler-Flags wieder speichern
;
	ldi	r17, 32			;32 Sensor-Flags bearbeiten
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
m73310:	ld	r16, x			;Flags holen
	bst	r16, 7			;Sensor inaktiv?
	brts	m73320			;ja -> berspringen
	andi	r16, 0x30		;Erinnerungen gesetzt?
	breq	m73320			;nein -> weiter
	ld	r16, x			;sonst Flags nochmals holen
	cbr	r16, 0x30		;Erinnerungen lschen
	sbr	r16, 0x03		;Name und Wert neu ausgeben
	st	x, r16			;Flags wieder speichern
m73320:	adiw	xl, 1			;Zeiger auf nchsten Sensor
	dec	r17			;alle Sensoren bearbeitet?
	brne	m73310			;nein -> Schleife
;
	ldi	r17, 4			;4 Alarmflags bearbeiten
	ldi	xl, low(almflg)
	ldi	xh, high(almflg)	;Zeiger auf Alarmflags
m73330:	ld	r16, x			;Flags holen
	bst	r16, 7			;Alarmeingang inaktiv?
	brts	m73340			;ja -> berspringen
	bst	r16, 2			;Erinnerung gesetzt?
	brtc	m73340			;nein -> weiter
	cbr	r16, 0x04		;Erinnerung lschen
	sbr	r16, 0x01		;Alarmname neu ausgeben
m73340:	st	x+, r16			;Flags wieder speichern
	dec	r17			;alle Alarme bearbeitet?
	brne	m73330			;nein -> Schleife
;
	lds	r16, ledflg		;LED-Flags holen
	cbr	r16, 0x30		;Erinnerungen lschen
	sts	ledflg, r16		;LED-Flags wieder speichern
	lds	r16, xflags		;diverse Flags holen
	cbr	r16, 0x02		;allgemeines Erinnerungsflag lschen
	sts	xflags, r16		;Flags wieder speichern
	lds	r16, dflags		;Anzeigeflags holen
	sbr	r16, 0x0c		;Info/Menzeile neu ausgeben
	sts	dflags, r16		;Anzeigeflags wieder speichern
	jmp	m99300
;
; Menprogramm: Anzeige der Minimalwerte
;
m80000:	lds	r16, xflags		;verschiedene Flags holen
	sbr	r16, 0x04		;Flag fr Minimalanzeige setzen
	sts	xflags, r16		;Flags wieder speichern
	jmp	m99000			;Temperaturanzeige neu ausgeben
;
m80100:	call	rsmin1			;Minimalwerte zurcksetzen
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200			;Temperaturanzeige neu ausgeben
;
; Menprogramm: Anzeige der Maximalwerte
;
m80200:	lds	r16, xflags		;verschiedene Flags holen
	sbr	r16, 0x08		;Flag fr Maximalanzeige setzen
	sts	xflags, r16		;Flags wieder speichern
	jmp	m99000			;Temperaturanzeige neu ausgeben
;
m80300:	call	rsmax1			;Maximalwerte zurcksetzen
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200			;Temperaturanzeige neu ausgeben
;
; Menprogramm: Anzeige der Meldungen, aktuelle zuerst zeigen
;
m81000:	lds	r16, xemspo
	lds	r17, xemspo + 1		;aktuelle Meldungsposition
	lds	r18, xemspo + 2		;holen
	sts	msgpo1, r16
	sts	msgpo1 + 1, r17		;und als Zeiger fr die Anzeige
	sts	msgpo1 + 2, r18		;speichern
	sts	msgpo2, r16
	sts	msgpo2 + 1, r17		;zustzlich als Zeiger fr die
	sts	msgpo2 + 2, r18		;vorherige Seite speichern
	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 1			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Meldungsanzeige setzen
	lds	r16, xflag2		;Verschiedene Flags 2 holen
	cbr	r16, 1 << 5 | 1 << 6	;nchste Seite, kein Listenende
	sbr	r16, 1 << 7		;Listenende fr vorherige Seite
m81010:	sts	xflag2, r16		;Flags wieder speichern
	lds	r16, msgfil		;Meldungsfilter holen
	ldi	r17, 0x2a		;Stringnummer fr Filtertext
	add	r16, r17		;Stringnummer berechnen
	sts	menext, r16		;Menerweiterung setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Anzeige der Meldungen, vorherige Seite anzeigen
;
m81100:	lds	r16, xflag2		;Verschiedene Flags 2 holen
	bst	r16, 5			;Richtungsnderung?
	brts	m81120			;ja -> weiter
	bst	r16, 6			;Listenende erreicht?
	brtc	m81120			;nein -> weiter
m81110:	lds	r16, msgfil		;sonst Meldungsfilter holen
	ldi	r17, 0x2a		;Stringnummer fr Filtertext
	add	r16, r17		;Stringnummer berechnen
	sts	menext, r16		;Menerweiterung setzen
	jmp	m99300			;Ende ohne Neuausgabe
m81120:	lds	r20, msgpo1
	lds	r21, msgpo1 + 1		;aktuelle Meldungsposition
	lds	r22, msgpo1 + 2		;fr vorherige Seite holen
	sts	msgpo2, r20
	sts	msgpo2 + 1, r21		;und als Position fr die
	sts	msgpo2 + 2, r22		;nchste Seite speichern
	cbr	r16, 1 << 5 | 1 << 6	;vorherige Seite, kein Listenende
	rjmp	m81010			;Flags speichern, Anzeige neu ausgeben
;
; Menprogramm: Anzeige der Meldungen, nchste Seite anzeigen
;
m81200:	lds	r16, xflag2		;Verschiedene Flags 2 holen
	bst	r16, 5			;Richtungsnderung?
	brtc	m81210			;ja -> weiter
	bst	r16, 7			;Listenende erreicht?
	brts	m81110			;ja -> Ende ohne Neuausgabe
m81210:	lds	r20, msgpo2
	lds	r21, msgpo2 + 1		;aktuelle Meldungsposition
	lds	r22, msgpo2 + 2		;fr nchste Seite holen
	sts	msgpo1, r20
	sts	msgpo1 + 1, r21		;und als Position fr die
	sts	msgpo1 + 2, r22		;vorherige Seite speichern
	sbr	r16, 1 << 5		;nchste Seite
	cbr	r16, 1 << 7		;Listenende-Flags lschen
	rjmp	m81010			;Flags speichern, Anzeige neu ausgeben
;
; Menprogramm: Anzeige der Meldungen, Filter umschalten
;
m81300:	lds	r16, msgfil		;Meldungsfilter holen
	inc	r16			;nchstes Filter setzen
	cpi	r16, 4			;Endwert berschritten?
	brcs	m81310			;nein -> weiter
	clr	r16			;sonst erstes Filter setzen
m81310:	sts	msgfil, r16		;und wieder speichern
	call	okbeep			;Tonsignal ausgeben
	call	eewcon			;Konfiguration im EEPROM sichern
	rjmp	m81000			;Liste neu anzeigen
;
; Menprogramm: Status-Anzeige, Anzeige neu ausgeben
;
m82000:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 2			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Statusanzeige setzen
	ldi	r16, 0x2e		;Stringnummer "Normal"
	lds	r17, confg1		;Konfigurationsbyte 1 holen
	bst	r17, 7			;Status Abwesend gesetzt?
	brtc	m82020			;nein -> weiter
	ldi	r16, 0x2f		;sonst Stringnummer "Abwesend"
m82020:	sts	menext, r16		;Menerweiterung setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Status-Anzeige, Normal/Abwesend umschalten
;
m82400:	lds	r18, dflags		;Anzeige-Flags holen
	lds	r17, confg1		;Konfigurationsbyte 1 holen
	bst	r17, 7			;Status Abwesend gesetzt?
	brtc	m82410			;nein -> weiter
	sbr	r18, 0x20		;sonst Lautsprecher einschalten
	cbr	r17, 0x80		;Status Normal setzen
	ldi	r16, 0x2e		;Stringnummer "Normal"
	rjmp	m82420
m82410:	ser	r16
	sts	lsptio, r16		;Lautsprecher-Timeout stopppen
	cbr	r18, 0x20		;Lautsprecher ausschalten
	sbr	r17, 0x80		;Status Abwesend setzen
	ldi	r16, 0x2f		;Stringnummer "Abwesend"
m82420:	sts	dflags, r18		;Anzeige-Flags wieder speichern
	sts	confg1, r17		;Konfigurationsbyte 1 speichern
	sts	menext, r16		;Menerweiterung setzen
	call	okbeep			;Tonsignal ausgeben
	call	eewcon			;Konfiguration im EEPROM sichern
	jmp	m99200			;Ende, Menzeile neu ausgeben
;
; Menprogramm: Status-Anzeige, LED umschalten
;
m82500:	lds	r17, confg2		;Konfigurationsbyte 2 holen
	bst	r17, 0			;LED in Debug-Modus?
	brtc	m82510			;nein -> weiter
	cbr	r17, 1			;sonst in Normal-Modus setzen
	rjmp	m82520
m82510:	ser	r16
	sts	rcvflg, r16		;RS-232 Empfangsflag lschen
	sbr	r17, 1			;LED in Debug-Modus setzen
m82520:	sts	confg2, r17		;Konfigurationsbyte speichern
	call	eewcon			;Konfiguration im EEPROM sichern
	rjmp	m82810			;Tonsignal ausgeben, Menerweiterung setzen
;
; Menprogramm: Status-Anzeige, Speicherfehler lschen
;
m82600:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicherfehler?
	brtc	m82820			;nein -> Ende
	lds	r17, xflags		;verschiedene Flags holen
	lds	r18, xflag4		;verschiedene Flags 4 holen
	andi	r17, 0xc0		;EEPROM-Bits filtern
	cpi	r17, 0xc0		;sind EEPROMs 1 und 2 vorhanden?
	brne	m82820			;nein -> Ende
	andi	r18, 0xc0		;EEPROM-Bits filtern
	cpi	r18, 0xc0		;sind EEPROMs 3 und 4 vorhanden?
	brne	m82820			;nein -> Ende
;
	cbr	r16, 1 << 3		;sonst Speicherfehler lschen
	sts	errflg, r16		;Fehler-Flags wieder speichern
	rjmp	m82810			;Tonsignal ausgeben, Men-Erweiterung setzen
;
; Menprogramm: Status-Anzeige, RTC-Fehler lschen
;
m82700:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 4			;RTC-Fehler?
	brtc	m82820			;nein -> Ende
	cbr	r16, 1 << 4		;sonst RTC-Fehler lschen
	sts	errflg, r16		;Fehler-Flags wieder speichern
	rjmp	m82810			;Tonsignal ausgeben, Men-Erweiterung setzen
;
; Menprogramm: Status-Anzeige, Pointer-Fehler lschen
;
m82800:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 5			;Pointer-Fehler?
	brtc	m82820			;nein -> Ende
	cbr	r16, 1 << 5		;sonst Pointer-Fehler lschen
	sts	errflg, r16		;Fehler-Flags wieder speichern
	lds	r16, xflag2		;verschiedene Flags 2 holen
	cbr	r16, 0x0e		;Pointer-Flags lschen
	sts	xflag2, r16		;Flags wieder speichern
;
m82810:	call	okbeep			;Tonsignal ausgeben
m82820:	ldi	r16, 0x2e		;Stringnummer "Normal"
	lds	r17, confg1		;Konfigurationsbyte 1 holen
	bst	r17, 7			;Status Abwesend gesetzt?
	brtc	m82830			;nein -> weiter
	ldi	r16, 0x2f		;sonst Stringnummer "Abwesend"
m82830:	sts	menext, r16		;Menerweiterung setzen
	jmp	m99300			;Ende
;
; Menprogramm: Konfiguration - Sensoren - Einstellungen 1
;
m83000:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 3			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Einstellungen setzen
        clr     r16                     ;Parameter auf 0 setzen (Einstellungen 1)
        sts     dipara, r16             ;Parameter speichern
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 17			;Wert > 16?
	brcs	m83010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m83010:	tst	r16			;Position = 0?
	brne	m83020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m83020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 16
	sts	encmax, r16		;Impulsgeber-Maximum = 16 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sensoren - Einstellungen 2
;
m83050:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 3			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Einstellungen setzen
        ldi     r16, 1                  ;Parameter auf 1 setzen (Einstellungen 2)
        sts     dipara, r16             ;Parameter speichern
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 16			;Wert > 15?
	brcs	m83060			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m83060:	tst	r16			;Position = 0?
	brne	m83070			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m83070:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 15
	sts	encmax, r16		;Impulsgeber-Maximum = 15 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sensoren - Einstellungen 1/2, Auswhlen
;
m83100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m83150			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 3			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Einstellungen setzen
	jmp	m99200
;
m83150:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Einstellungen 1/2, Ein/Ausschalten
;
m83200:	lds	r18, menpos		;ausgewhlte Position holen
        dec     r18                     ;korrigieren und als Sensornummer verwenden
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Einstellungen 1 aktiv?
        breq    m83210                  ;ja -> weiter
        subi    r18, -16                ;sonst Sensornummer fr Einstellungen 2
m83210:	clr	r19                     ;H-Byte = 0 setzen
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Sensor-Flags holen
	bst	r16, 7			;Sensor inaktiv?
	brts	m83220			;ja -> bearbeiten
;
	ldi	r16, 0x83		;sonst inaktiv setzen, Neuausgabe
	st	x, r16			;Sensor-Flags speichern
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
        mov     r17, r18                ;Sensornummer zwischenspeichern
	add	r18, r18		;Sensornummer (Offset) verdoppeln
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	clr	r20
	ldi	r21, 0x80		;ungltigen Wert laden
	st	y, r20
	std	y + 1, r21		;und als Sensorwert speichern
;
        cpi     r17, 27                 ;Sensornummer = 27 (Luftdruck)
        brne    m83230                  ;nein -> weiter
        adiw    xl, 4                   ;sonst Sensornummer auf grafischen Luftdruck setzen
	st	x, r16			;ebenfalls inaktiv setzen, Neuausgabe
	rjmp	m83230
;
m83220:	ldi	r16, 0x03		;aktiv setzen, Neuausgabe
	st	x, r16			;Sensor-Flags speichern
	ldi	yl, low(sentio)
	ldi	yh, high(sentio)	;Zeiger auf Timeout-Zhler
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ldi	r17, timsen		;Timeoutwert holen
	st	y, r17			;Timeout-Zhler neu setzen
;
        cpi     r18, 27                 ;Sensornummer = 27 (Luftdruck)
        brne    m83230                  ;nein -> weiter
        adiw    xl, 4                   ;sonst Sensornummer auf grafischen Luftdruck setzen
	st	x, r16			;ebenfalls inaktiv setzen, Neuausgabe
;
m83230:	call	eewcon			;Konfiguration im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99300
;
; Menprogramm: Konfiguration - Sensoren - Einstellungen 1/2, Name ndern
;
m83300:	ldi	r16, 0x5c		;Stringnummer "Name"
	sts	menext, r16		;Menerweiterung setzen
	lds	r18, menpos		;ausgewhlte Position holen
        dec     r18                     ;korrigieren und als Sensornummer verwenden
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Einstellungen 1 aktiv?
        breq    m83310                  ;ja -> weiter
        subi    r18, -16                ;sonst Sensornummer fr Einstellungen 2
m83310:	clr	r19                     ;H-Byte = 0 setzen
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ld	r17, y			;Sensor-Flags holen
	sbr	r17, 0x02		;Name neu ausgeben
	st	y, r17			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m83350			;ja -> bearbeiten
;
	ldi	r19, 9			;Sensornamenlnge 9 Zeichen
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	call	cpeeab			;EEPROM-Text in Eingabepuffer
	ldi	r16, 0x09		;Impulsgeberwert als ASCII anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
        lds     r16, asclst             ;Anzahl der Zeichen in der Liste holen
        inc     r16                     ;auf Ok-Zeichen setzen
	sts	encval, r16		;und speichern
	ldi	r16, 5			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Einstellungen, Name setzen
	jmp	m99200
;
m83350: lds	r20, ascibp		;aktuelle Pufferposition holen
	lds	r16, encval		;aktuellen Impulsgeberwert holen
        lds     r17, asclst             ;Anzahl der Zeichen holen
        cp      r16, r17                ;Lschzeichen?
        breq    m83360                  ;ja -> bearbeiten
        inc     r17                     ;Anzahl der Zeichen korrigieren
	cp	r16, r17	        ;Ok-Zeichen?
	breq	m83390			;ja -> String speichern
;
	cpi	r20, 9			;letzte Position?
	breq	m83380			;ja -> Zeichen ignorieren
        call    getchr                  ;sonst Zeichen aus Liste holen
	mov	r13, r16		;Zeichen sichern
	call	txblen			;Pufferlnge in Pixel ermitteln
	mov	r16, r13		;Zeichen neu holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;zur Gesamtlnge addieren
	cpi	r18, 49			;maximale Lnge berschritten?
	brcc	m83380			;ja -> Zeichen ignorieren
;
	mov	r16, r13		;sonst Zeichen neu holen
	ldi	xl, low(ascibf)
	ldi	xh, high(ascibf)	;Zeiger auf Texteingabepuffer
	clr	r21
	add	xl, r20			;Pufferadresse ermitteln
	adc	xh, r21			;bertrag
	st	x, r16			;Zeichen speichern
	inc	r20			;aktuelle Position erhhen
	rjmp	m83370			;speichern
;
m83360:	tst	r20			;aktuelle Position = 0?
	breq	m83380			;ja -> berspringen
	dec	r20			;sonst eine Position zurck
m83370:	sts	ascibp, r20		;neue Position speichern	
m83380:	jmp	m99200
;
m83390:	ldi	r19, 9			;Sensornamenlnge 9 Zeichen
	ldi	r16, low(esentx)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(esentx)	;EEPROM-Adresse Sensornamen H
	call	cpabee			;Texteingabepuffer in EEPROM
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 3			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Einstellungen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	eewcrc			;EERPOM-CRC berechnen/speichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Grenzwerte 1
;
m84000: lds     r16, xflag4             ;Verschiedene Flags 4 holen
        cbr     r16, 0x03               ;Flags fr Grenzwert-nderung lschen
        sts     xflag4, r16             ;Flags wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 6			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, Auswhlen setzen
        clr     r16                     ;Parameter auf 0 setzen (Grenzwerte 1)
        sts     dipara, r16             ;Parameter speichern
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 17			;Wert > 16?
	brcs	m84010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84010:	tst	r16			;Position = 0?
	brne	m84020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 16
	sts	encmax, r16		;Impulsgeber-Maximum = 16 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sensoren - Grenzwerte 2
;
m84050: lds     r16, xflag4             ;Verschiedene Flags 4 holen
        cbr     r16, 0x03               ;Flags fr Grenzwert-nderung lschen
        sts     xflag4, r16             ;Flags wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 6			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, Auswhlen setzen
        ldi     r16, 1                  ;Parameter auf 1 setzen (Grenzwerte 2)
        sts     dipara, r16             ;Parameter speichern
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 12			;Wert > 11?
	brcs	m84060			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84060:	tst	r16			;Position = 0?
	brne	m84070			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84070:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 11
	sts	encmax, r16		;Impulsgeber-Maximum = 11 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration  - Sensoren - Grenzwerte 1/2, Auswhlen
;
m84100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84150			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 6			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, Auswhlen setzen
	jmp	m99200
;
m84150:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Grenzwerte 1/2, Unteren Grenzwert ndern
;
m84200:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
	lds	r18, menpos		;ausgewhlte Position holen
        dec     r18                     ;korrigieren und als Sensornummer verwenden
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Grenzwerte 1 aktiv?
        breq    m84210                  ;ja -> weiter
        subi    r18, -16                ;sonst Sensornummer fr Grenzwerte 2
m84210:	clr	r19                     ;H-Byte = 0 setzen
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ld	r17, y			;Sensor-Flags holen
	sbr	r17, 0x01		;Werte neu ausgeben
	st	y, r17			;Flags wieder speichern
	add	r18, r18		;Sensornummer (Offset) verdoppeln
	ldi	yl, low(senlil)
	ldi	yh, high(senlil)	;Zeiger auf untere Temperatur-Grenzwerte
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84250			;ja -> bearbeiten
;
	ld	r16, y
	ldd	r17, y + 1		;unteren Temperatur-Grenzwert holen
	cpi	r17, 0x80		;Wert gltig?
	brne	m84220			;ja -> weiter
	clr	r16
	clr	r17			;sonst Anfangswert = 0.0 Grad
m84220:	sts	encval, r16		;Impulsgeberwert L setzen
	sts	encvah, r17		;Impulsgeberwert H setzen
	ldi	r16, 0x05		;Impulsgeberwert als Temperatur anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 7			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, ndern
	lds	r16, xflag4		;verschiedene Flags 4 holen
	sbr	r16, 1			;Flag fr "Unteren Grenzwert ndern" setzen
	sts	xflag4, r16		;Flags wieder speichern
	jmp	m99200
;
m84250:	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	st	y, r16
	std	y + 1, r17		;als Temperatur-Grenzwert speichern
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	clr	r16
        ldi     r17, 0x80               ;ungltigen Wert laden
        st      y, r16
        std     y + 1, r17              ;Sensorwert lschen
;
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 6			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, Auswhlen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	cbr	r16, 1			;Flag fr "Unteren Grenzwert ndern" lschen
	sts	xflag4, r16		;Flags wieder speichern
	call	eewcon			;Konfiguration im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Grenzwerte 1/2, Unteren Grenzwert lschen
;
m84300:	lds	r18, menpos		;ausgewhlte Position holen
        dec     r18                     ;korrigieren und als Sensornummer verwenden
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Grenzwerte 1 aktiv?
        breq    m84310                  ;ja -> weiter
        subi    r18, -16                ;sonst Sensornummer fr Grenzwerte 2
m84310:	clr	r19                     ;H-Byte = 0 setzen
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ld	r17, y			;Sensor-Flags holen
	sbr	r17, 0x01		;Wert neu ausgeben
	st	y, r17			;Flags wieder speichern
	add	r18, r18		;Sensornummer (Offset) verdoppeln
	ldi	yl, low(senlil)
	ldi	yh, high(senlil)	;Zeiger auf untere Temperatur-Grenzwerte
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ldi	r16, 0
	ldi	r17, 0x80		;Temperaturwert lschen
	st	y, r16
	std	y + 1, r17		;als Temperatur-Grenzwert speichern
	ldi	r16, 6			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, Auswhlen setzen
	call	eewcon			;Konfiguration im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99300
;
; Menprogramm: Konfiguration - Sensoren - Grenzwerte 1/2, Oberen Grenzwert ndern
;
m84400:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
	lds	r18, menpos		;ausgewhlte Position holen
        dec     r18                     ;korrigieren und als Sensornummer verwenden
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Grenzwerte 1 aktiv?
        breq    m84410                  ;ja -> weiter
        subi    r18, -16                ;sonst Sensornummer fr Grenzwerte 2
m84410:	clr	r19                     ;H-Byte = 0 setzen
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ld	r17, y			;Sensor-Flags holen
	sbr	r17, 0x01		;Werte neu ausgeben
	st	y, r17			;Flags wieder speichern
	add	r18, r18		;Sensornummer (Offset) verdoppeln
	ldi	yl, low(senlih)
	ldi	yh, high(senlih)	;Zeiger auf obere Temperatur-Grenzwerte
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84450			;ja -> bearbeiten
;
	ld	r16, y
	ldd	r17, y + 1		;oberen Temperatur-Grenzwert holen
	cpi	r17, 0x80		;Wert gltig?
	brne	m84420			;ja -> weiter
	clr	r16
	clr	r17			;sonst Anfangswert= 0.0 Grad
m84420:	sts	encval, r16		;Impulsgeberwert L setzen
	sts	encvah, r17		;Impulsgeberwert H setzen
	ldi	r16, 0x05		;Impulsgeberwert als Temp anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 7			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, ndern
	lds	r16, xflag4		;verschiedene Flags 4 holen
	sbr	r16, 1 << 1		;Flag fr "Oberen Grenzwert ndern" setzen
	sts	xflag4, r16		;Flags wieder speichern
	jmp	m99200
;
m84450:	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	st	y, r16
	std	y + 1, r17		;als Temperatur-Grenzwert speichern
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	clr	r16
        ldi     r17, 0x80               ;ungltigen Wert laden
        st      y, r16
        std     y + 1, r17              ;Sensorwert lschen
;
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 6			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, Auswhlen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	cbr	r16, 1 << 1		;Flag fr "Oberen Grenzwert ndern" lschen
	sts	xflag4, r16		;Flags wieder speichern
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Grenzwerte 1/2, Oberen Grenzwert lschen
;
m84500:	lds	r18, menpos		;ausgewhlte Position holen
        dec     r18                     ;korrigieren und als Sensornummer verwenden
        lds     r17, dipara             ;Anzeigemodus Parameter holen
        tst     r17                     ;Grenzwerte 1 aktiv?
        breq    m84510                  ;ja -> weiter
        subi    r18, -16                ;sonst Sensornummer fr Grenzwerte 2
m84510:	clr	r19                     ;H-Byte = 0 setzen
	ldi	yl, low(senflg)
	ldi	yh, high(senflg)	;Zeiger auf Sensor-Flags
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ld	r17, y			;Sensor-Flags holen
	sbr	r17, 0x01		;Werte neu ausgeben
	st	y, r17			;Flags wieder speichern
	add	r18, r18		;Sensornummer (Offset) verdoppeln
	ldi	yl, low(senlih)
	ldi	yh, high(senlih)	;Zeiger auf obere Temperatur-Grenzwerte
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ldi	r16, 0
	ldi	r17, 0x80		;Temperaturwert lschen
	st	y, r16
	std	y + 1, r17		;als Temperatur-Grenzwert speichern
	ldi	r16, 6			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Grenzwerte, Auswhlen setzen
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99300
;
; Menprogramm: Konfiguration - Sensoren - Anzeige
;
m84600:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 4			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Anzeige setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 33			;Wert > 32?
	brcs	m84610			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84610:	tst	r16			;Position = 0?
	brne	m84620			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84620:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 32
	sts	encmax, r16		;Impulsgeber-Maximum = 32 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sensoren - Anzeige, Auswhlen
;
m84700:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84750			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 4			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Anzeige setzen
	jmp	m99200
;
m84750:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Anzeige, ndern
;
m84800:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
        ldi     yl, low(postab)
        ldi     yh, high(postab)        ;Zeiger auf Anzeigepositions-Tabelle
        clr     r17                     ;H-Byte = 0
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        dec     r16                     ;korrigieren
        add     yl, r16                 ;Tabellenplatz berechnen
        adc     yh, r17                 ;bertrag
;
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84850			;ja -> bearbeiten
	sbr	r16, 0x41		;sonst Impulsgeberwert als Sensorname anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 27			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Anzeige, ndern setzen
;
        ld      r16, y                  ;Sensornummer holen
        bst     r16, 5                  ;Anzeigeposition leer?
        brts    m84810                  ;ja -> weiter
        andi    r16, 0x1f               ;sonst Sensornummer filtern
        inc     r16                     ;korrigieren
        rjmp    m84820
;
m84810: clr     r16                     ;Impulsgeberwert fr [leer]
m84820: sts     encval, r16             ;neuen Impulsgeberwert setzen
        ldi     r16, 0
        sts     encmin, r16             ;Impulsgeber-Minimum = 0 setzen
        ldi     r16, 32
        sts     encmax, r16             ;Impulsgeber-Maximum = 32 setzen
	jmp	m99200
;
m84850:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 4			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Anzeige setzen
;
        lds     r16, encval             ;Impulsgeberwert holen
        dec     r16                     ;korrigieren
        cpi     r16, 0xff               ;Wert fr [leer]?
        brne    m84860                  ;nein -> weiter
        ldi     r16, 0x20               ;sonst Flag fr leere Anzeigeposition
m84860: st      y, r16                  ;speichern
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        sts     encval, r16             ;auf Impulsgeberwert bertragen
	call	eewcon			;Konfiguration im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Groanzeige
;
m84900:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 28			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Groanzeige setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 3			;Wert > 2?
	brcs	m84910			;nein -> weiter
	ldi	r16, 2			;sonst Position = 2 setzen
m84910:	tst	r16			;Position = 0?
	brne	m84920			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84920:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 2
	sts	encmax, r16		;Impulsgeber-Maximum = 2 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sensoren - Groanzeige, Auswhlen
;
m84a00:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84a50			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 28			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Groanzeige setzen
	jmp	m99200
;
m84a50:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Groanzeige, ndern
;
m84b00:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
        ldi     yl, low(bigflg)
        ldi     yh, high(bigflg)        ;Zeiger auf Groanzeige-Flags
        clr     r17                     ;H-Byte = 0
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        dec     r16                     ;korrigieren
        add     yl, r16                 ;Tabellenplatz berechnen
        adc     yh, r17                 ;bertrag
;
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84b50			;ja -> bearbeiten
	sbr	r16, 0x41		;sonst Impulsgeberwert als Sensorname anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 29			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Groanzeige, ndern setzen
;
        ld      r16, y                  ;Sensornummer holen
        bst     r16, 5                  ;Groanzeige leer?
        brts    m84b10                  ;ja -> weiter
        andi    r16, 0x1f               ;sonst Sensornummer filtern
        inc     r16                     ;korrigieren
        cpi     r16, 28                 ;Temperatursensor?
        brcs    m84b20                  ;ja -> weiter
m84b10: clr     r16                     ;sonst auf [leer] setzen
m84b20: sts     encval, r16             ;neuen Impulsgeberwert setzen
        ldi     r16, 0
        sts     encmin, r16             ;Impulsgeber-Minimum = 0 setzen
        ldi     r16, 27
        sts     encmax, r16             ;Impulsgeber-Maximum = 27 setzen
	jmp	m99200
;
m84b50:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 28			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Groanzeige setzen
;
        lds     r16, encval             ;Impulsgeberwert holen
        dec     r16                     ;korrigieren
        cpi     r16, 0xff               ;Groanzeige leer?
        brne    m84b60                  ;nein -> weiter
        ldi     r16, 0x20               ;sonst auf [leer] setzen
m84b60: st      y, r16                  ;als neue Sensornummer speichern
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        sts     encval, r16             ;auf Impulsgeberwert bertragen
	call	eewcon			;Konfiguration im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Log-Zuordnung 1
;
m84c00:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 21			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 1 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 25			;Wert > 24?
	brcs	m84c10			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84c10:	tst	r16			;Position = 0?
	brne	m84c20			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84c20:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 24
	sts	encmax, r16		;Impulsgeber-Maximum = 24 setzen
        call    chklog                  ;Log-Tabellen auf Konflikte prfen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sensoren - Log-Zuordnung 1, Auswhlen
;
m84d00:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84d50			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 21			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 1 setzen
	jmp	m99200
;
m84d50:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Log-Zuordnung 1, ndern
;
m84e00:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
        ldi     yl, low(logtb1)
        ldi     yh, high(logtb1)        ;Zeiger auf Log-Positions-Tabelle 1
        clr     r17                     ;H-Byte = 0
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        dec     r16                     ;korrigieren
        add     yl, r16                 ;Tabellenplatz berechnen
        adc     yh, r17                 ;bertrag
;
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84e50			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 22			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 1, ndern setzen
;
        ld      r16, y                  ;Log-Position holen
        andi    r16, 0x1f               ;Log-Position filtern
        inc     r16                     ;korrigieren
        sts     encval, r16             ;neuen Impulsgeberwert setzen
        ldi     r16, 1
        sts     encmin, r16             ;Impulsgeber-Minimum = 1 setzen
        ldi     r16, 24
        sts     encmax, r16             ;Impulsgeber-Maximum = 24 setzen
	jmp	m99200
;
m84e50:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 21			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 1 setzen
;
        lds     r16, encval             ;Impulsgeberwert holen
        dec     r16                     ;korrigieren
        st      y, r16                  ;speichern
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        sts     encval, r16             ;auf Impulsgeberwert bertragen
	call	eewcon			;Konfiguration im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
        call    chklog                  ;Log-Tabellen auf Konflikte prfen
        call    newval                  ;Sensorwerte neu ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Log-Zuordnung 2
;
m84f00:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 30			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 2 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 4			;Wert > 3?
	brcs	m84f10			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84f10:	tst	r16			;Position = 0?
	brne	m84f20			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m84f20:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 3
	sts	encmax, r16		;Impulsgeber-Maximum = 3 setzen
        call    chklog                  ;Log-Tabellen auf Konflikte prfen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sensoren - Log-Zuordnung 2, Auswhlen
;
m84g00:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84g50			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 30			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 2 setzen
	jmp	m99200
;
m84g50:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Sensoren - Log-Zuordnung 2, ndern
;
m84h00:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
        ldi     yl, low(logtb2)
        ldi     yh, high(logtb2)        ;Zeiger auf Log-Positions-Tabelle 2
        clr     r17                     ;H-Byte = 0
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        dec     r16                     ;korrigieren
        add     yl, r16                 ;Tabellenplatz berechnen
        adc     yh, r17                 ;bertrag
;
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m84h50			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 31			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 2, ndern setzen
;
        ld      r16, y                  ;Log-Position holen
        andi    r16, 0x1f               ;Log-Position filtern
        inc     r16                     ;korrigieren
        sts     encval, r16             ;neuen Impulsgeberwert setzen
        ldi     r16, 1
        sts     encmin, r16             ;Impulsgeber-Minimum = 1 setzen
        ldi     r16, 3
        sts     encmax, r16             ;Impulsgeber-Maximum = 3 setzen
	jmp	m99200
;
m84h50:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 30			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sensoren - Log-Zuordnung 2 setzen
;
        lds     r16, encval             ;Impulsgeberwert holen
        dec     r16                     ;korrigieren
        st      y, r16                  ;speichern
        lds     r16, menpos             ;ausgewhlte Men-Position holen
        sts     encval, r16             ;auf Impulsgeberwert bertragen
	call	eewcon			;Konfiguration im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
        call    chklog                  ;Log-Tabellen auf Konflikte prfen
        call    newval                  ;Sensorwerte neu ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Alarme
;
m85000:	call	calmbk			;Anzeige-Positions-Tabelle kopieren
	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 8			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Alarme setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 5			;Wert > 4?
	brcs	m85010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m85010:	tst	r16			;Position = 0?
	brne	m85020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m85020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 4
	sts	encmax, r16		;Impulsgeber-Maximum = 4 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Alarme, Auswhlen
;
m85100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m85150			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 8			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Alarme setzen
	jmp	m99200
;
m85150:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Alarme, Ein/Ausschalten
;
m85200:	ldi	r16, 8			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Alarme setzen
	lds	r16, menpos		;ausgewhlte Position holen
	call	salmap			;Alarmnummer suchen
	clr	r19
	ldi	xl, low(almflg)
	ldi	xh, high(almflg)	;Zeiger auf Alarmflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Alarmflags holen
	bst	r16, 7			;Sensor inaktiv?
	brts	m85210			;ja -> bearbeiten
	ldi	r16, 0x81		;sonst inaktiv, Neuausgabe
	rjmp	m85220
m85210:	ldi	r16, 0x01		;aktiv setzen, Neuausgabe
m85220:	st	x, r16			;Alarmflags speichern
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99300
;
; Menprogramm: Konfiguration - Alarme, Name
;
m85300:	ldi	r16, 0x5c		;Stringnummer "Name"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, menpos		;ausgewhlte Position holen
	call	salmap			;Alarmnummer suchen
	clr	r19
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ld	r17, y			;Alarmflags holen
	sbr	r17, 0x01		;Name neu ausgeben
	st	y, r17			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m85350			;ja -> bearbeiten
;
	ldi	r19, 11			;Alarmnamenlnge im EEPROM
	ldi	r16, low(ealmtx)	;EEPROM-Adresse Alarmnamen L
	ldi	r17, high(ealmtx)	;EEPROM-Adresse Alarmnamen H
	call	cpeeab			;EEPROM-Text in Eingabepuffer
	ldi	r16, 0x09		;Impulsgeberwert als ASCII anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
        lds     r16, asclst             ;Anzahl der Zeichen in der Liste holen
        inc     r16                     ;auf Ok-Zeichen setzen
	sts	encval, r16		;und speichern
	ldi	r16, 10			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Alarme, Name setzen
	jmp	m99200
;
m85350:	lds	r20, ascibp		;aktuelle Pufferposition holen
	lds	r16, encval		;aktuellen Impulsgeberwert holen
        lds     r17, asclst             ;Anzahl der Zeichen holen
        cp      r16, r17                ;Lschzeichen?
        breq    m85360                  ;ja -> bearbeiten
        inc     r17                     ;Anzahl der Zeichen korrigieren
	cp	r16, r17	        ;Ok-Zeichen?
	breq	m85390			;ja -> String speichern
;
	cpi	r20, 11			;letzte Position?
	breq	m85380			;ja -> Zeichen ignorieren
        call    getchr                  ;sonst Zeichen aus Liste holen
	mov	r13, r16		;Zeichen sichern
	call	txblen			;Pufferlnge in Pixel ermitteln
	mov	r16, r13		;Zeichen neu holen
	call	asclen			;Zeichenlnge ermitteln
	add	r18, r16		;zur Gesamtlnge addieren
	cpi	r18, 62			;maximale Lnge berschritten?
	brcc	m85380			;ja -> Zeichen ignorieren
;
	mov	r16, r13		;Zeichen neu holen
	ldi	xl, low(ascibf)
	ldi	xh, high(ascibf)	;Zeiger auf Texteingabepuffer
	clr	r21
	add	xl, r20			;Pufferadresse ermitteln
	adc	xh, r21			;bertrag
	st	x, r16			;Zeichen speichern
	inc	r20			;aktuelle Position erhhen
	rjmp	m85370			;speichern
;
m85360:	tst	r20			;aktuelle Position = 0?
	breq	m85370			;ja -> berspringen
	dec	r20			;sonst eine Position zurck
m85370:	sts	ascibp, r20		;neue Position speichern	
m85380:	jmp	m99200
;
m85390:	ldi	r19, 11			;Alarmnamenlnge im EEPROM
	ldi	r16, low(ealmtx)	;EEPROM-Adresse Alarmnamen L
	ldi	r17, high(ealmtx)	;EEPROM-Adresse Alarmnamen H
	call	cpabee			;Texteingabepuffer in EEPROM
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 8			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Alarme, Auswhlen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	eewcrc			;EERPOM-CRC berechnen/speichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Alarme, Invertieren
;
m85400:	ldi	r16, 8			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Alarme, Auswhlen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	call	salmap			;Alarmnummer suchen
	clr	r19
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
	add	yl, r18			;Position berechnen
	adc	yh, r19			;bertrag
	ld	r17, y			;Alarmflags holen
	sbr	r17, 0x01		;Name neu ausgeben
	st	y, r17			;Flags wieder speichern
;
	ldi	xl, low(alminv)
	ldi	xh, high(alminv)	;Zeiger auf Alarminvertierung
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Invertierung holen
	ldi	r17, 0x01
	eor	r16, r17		;Alarm Invertieren
	st	x, r16			;Invertierung speichern
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99300
;
; Menprogramm: Konfiguration - Alarme, Verschieben
;
m85500:	ldi	r16, 0x5d		;Stringnummer "Verschieben"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m85550			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 9			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Alarme, Verschieben setzen
	jmp	m99200
;
m85550:	call	cbkalm			;Anzeige-Positions-Tabelle aus Kopie
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Signale - Zuordnung
;
m86000:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 11			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Signalzuordnung setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 9			;Wert > 9?
	brcs	m86010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m86010:	tst	r16			;Position = 0?
	brne	m86020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m86020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	jmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Signale - Zuordnung, Auswhlen
;
m86100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m86150			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	ldi	r16, 11			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Zuordnung, Auswhlen setzen
	jmp	m99200
;
m86150:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	jmp	m99200
;
; Menprogramm: Konfiguration - Signale - Zuordnung, Nummer
;
m86200:	ldi	r16, 0x6c		;Stringnummer "Nummer"
	sts	menext, r16		;Menerweiterung setzen
	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	clr	r19
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	ldi	xl, low(signum)
	ldi	xh, high(signum)	;Zeiger auf Signalnummern
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m86250			;ja -> bearbeiten
;
	ld	r16, x			;Signalnummer holen
	inc	r16			;fr Anzeige korrigieren
	sts	encval, r16		;Impulsgeberwert L setzen
	ldi	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 0 setzen
	ldi	r16, 15
	sts	encmax, r16		;Impulsgeber-Maximum = 15 setzen
	ldi	r16, 12			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Zuordnung, Nummer setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	sbr	r16, 1 << 2		;Flag fr "Nummer ndern" setzen
	sts	xflag4, r16		;Flags wieder speichern
	jmp	m99200
;
m86250:	lds	r16, encval		;Impulsgeberwert L holen
	dec	r16			;korrigieren und
	st	x, r16			;als Signalnummer speichern
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	ldi	r16, 11			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Zuordnung, Auswhlen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	cbr	r16, 1 << 2		;Flag fr "Nummer ndern" lschen
	sts	xflag4, r16		;Flags wieder speichern
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	jmp	m99200
;
; Menprogramm: Konfiguration - Signale - Zuordnung, Dauer
;
m86300:	ldi	r16, 0x6d		;Stringnummer "Dauer"
	sts	menext, r16		;Menerweiterung setzen
	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	clr	r19
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	ldi	xl, low(sigdur)
	ldi	xh, high(sigdur)	;Zeiger auf Signaldauer
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m86350			;ja -> bearbeiten
;
	ld	r16, x			;Signalnummer holen
	sts	encval, r16		;Impulsgeberwert L setzen
	ldi	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 0
	sts	encmin, r16		;Impulsgeber-Minimum = 0 setzen
	ldi	r16, 30
	sts	encmax, r16		;Impulsgeber-Maximum = 30 setzen
	ldi	r16, 12			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Zuordnung, Dauer setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	sbr	r16, 1 << 3		;Flag fr "Dauer ndern" setzen
	sts	xflag4, r16		;Flags wieder speichern
	jmp	m99200
;
m86350:	lds	r16, encval		;Impulsgeberwert L holen
	st	x, r16			;als Signaldauer speichern
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	ldi	r16, 11			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Zuordnung, Auswhlen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	lds	r16, xflag4		;verschiedene Flags 4 holen
	cbr	r16, 1 << 3		;Flag fr "Dauer ndern" lschen
	sts	xflag4, r16		;Flags wieder speichern
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	rjmp	m99200
;
; Menprogramm: Konfiguration - Signale - Zuordnung, Abspielen
;
m86400:	lds	r16, menpos		;ausgewhlte Position holen
	dec	r16			;korrigieren
	clr	r17
	ldi	xl, low(signum)
	ldi	xh, high(signum)	;Zeiger auf Signalnamen
	add	xl, r16			;Position berechnen
	adc	xh, r17			;bertrag
	ldi	yl, low(sigdur)
	ldi	yh, high(sigdur)	;Zeiger auf Signaldauer
	add	yl, r16			;Position berechnen
	adc	yh, r17			;bertrag
	ld	r18, x			;Signalnummer holen
	ld	r19, y			;Signaldauer holen
	call	signal			;Signal ausgeben
	rjmp	m99300
;
; Menprogramm: Konfiguration - Signale - Audio
;
m87000:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 13			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Audio setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 4			;Wert > 3?
	brcs	m87010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m87010:	tst	r16			;Position = 0?
	brne	m87020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m87020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 3
	sts	encmax, r16		;Impulsgeber-Maximum = 3 setzen
	rjmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Signale - Audio, Auswhlen
;
m87100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m87150			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 3
	sts	encmax, r16		;Impulsgeber-Maximum = 3 setzen
	ldi	r16, 13			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Audio setzen
	rjmp	m99200
;
m87150:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	rjmp	m99200
;
; Menprogramm: Konfiguration - Signale - Audio, ndern
;
m87200:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	clr	r19
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m87250			;ja -> bearbeiten
;	
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16,1			;Position auf Signalfrequenz?
	brne	m87210			;nein -> weiter testen
	call	timfrq			;sonst Timer0-Wert in Frequenz
	sts	encval, r16		;Impulsgeberwert L setzen
	sts	encvah, r17		;Impulsgeberwert H setzen
	ldi	r16, low(minfrq)
	ldi	r17, high(minfrq)	;minimale Frequenz laden
	sts	encmin, r16
	sts	encmih, r17		;als Minimalwert setzen
	ldi	r16, low(maxfrq)
	ldi	r17, high(maxfrq)	;maximale Frequenz laden
	sts	encmax, r16
	sts	encmah, r17		;als Maximalwert setzen
	ldi	r16, 0x11		;Impulsgeberwert dezimal4 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 14			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Audio, ndern setzen
	rjmp	m99200
;
m87210:	lds	r17, sigvo1		;Signal-Lautstrke Tag holen
	cpi	r16, 2			;Position auf Lautstrke Tag?
	breq	m87220			;ja -> weiter
	lds	r17, sigvo2		;sonst Lautstrke Nacht holen
m87220:	sts	encval, r17		;Impulsgeberwert setzen
	clr	r16
	sts	encmin, r16		;Impulsgeber-Minimum = 0 setzen
	ldi	r16, 15
	sts	encmax, r16		;Impulsgeber-Maximum = 15 setzen
	ldi	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 14			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Audio, ndern setzen
	rjmp	m99200
;
m87250:	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 1			;Position auf Signalfrequenz?
	brne	m87260			;nein -> weiter testen
	lds	r20, encval		;Impulsgeberwert L holen
	lds	r21, encvah		;Impulsgeberwert H holen
	call	frqtim			;Frequenz in Timer0-Wert
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 13			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Audio setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	rjmp	m99200
;
m87260:	lds	r17, encval		;Impulsgeberwert L holen
	cpi	r16, 2			;Position auf Lautstrke Tag?
	brne	m87270			;nein -> weiter
	sts	sigvo1, r17		;als Lautstrke Tag speichern
	rjmp	m87280
m87270:	sts	sigvo2, r17		;als Lautstrke Nacht speichern
m87280:	call	sicalc			;Lautstrke einstellen
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 3
	sts	encmax, r16		;Impulsgeber-Maximum = 3 setzen
	ldi	r16, 13			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Signale - Audio setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	rjmp	m99200
;
; Menprogramm: Konfiguration - Signale - Audio, Abspielen
;
m87300:	ldi	r18, 1			;Signalnummer 1 setzen
	ldi	r19, 3			;Signaldauer 3s setzen
	call	signal			;Signal ausgeben
	rjmp	m99300
;
; Menprogramm: Konfiguration - Zeiten - Schaltzeiten
;
m87500:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 15			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Schaltzeiten setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 9			;Wert > 8?
	brcs	m87510			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m87510:	tst	r16			;Position=0?
	brne	m87520			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m87520:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	rjmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Zeiten - Schaltzeiten, Auswhlen
;
m87600:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m87650			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	ldi	r16, 15			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Schaltzeiten, ndern setzen
	rjmp	m99200
;
m87650:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	rjmp	m99200
;
; Menprogramm: Konfiguration - Zeiten - Schaltzeiten, Ein/Ausschalten
;
m87700:	ldi	r16, 15			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Schaltzeiten setzen
	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	mov	r20, r18		;korrigierte Position sichern
	cbr	r18, 0x01		;Bit 0 lschen, erstes Zeile
	clr	r19			;eines Paares neu ausgeben
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
;
	sbr	r18, 0x01		;Bit 0 setzen, zweites Zeile
	ldi	xl, low(cfgflg)		;des Paares neu ausgeben
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
;
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	lds	r18, confg2		;Konfigurationsbyte 2 holen
	clr	r17			;Maskenbyte 1 lschen
	clr	r19			;Maskenbyte 2 lschen
	cpi	r20, 2			;Zeile 0/1 (Signalmodus Nacht)?
	brcc	m87710			;nein -> weiter testen
	ldi	r17, 1 << 5		;Maske fr Signalmodus Nacht
	rjmp	m87740
m87710:	cpi	r20, 4			;Zeile 2/3 (LCD-Beleuchtung)?
	brcc	m87720			;nein -> weiter testen
	ldi	r17, 1 << 3		;Maske fr LCD-Beleuchtung
	rjmp	m87740
m87720:	cpi	r20, 6			;Zeile 4/5 (Bluetooth)?
	brcc	m87730			;nein -> weiter testen
	ldi	r19, 1 << 2		;Maske fr Bluetooth
	rjmp	m87740
m87730:	ldi	r17, 1 << 4		;Maske fr Min/Max-Reset
m87740:	eor	r16, r17		;Flag invertieren
	eor	r18, r19		;Flag invertieren
	sts	confg1, r16		;Konfigurationsbyte 1 speichern 
	sts	confg2, r18		;Konfigurationsbyte 2 speichern 
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	rjmp	m99300
;
; Menprogramm: Konfiguration - Zeiten - Schaltzeiten, ndern
;
m87800:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	clr	r19
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m87850			;ja -> bearbeiten
;	
	lds	r16, signig
	lds	r17, signig + 1		;Zeit Signalmodus Nacht 1 laden
	cpi	r18, 0			;Position Signalmodus Nacht 1?
	breq	m87810			;ja -> bearbeiten
	lds	r16, signig + 2
	lds	r17, signig + 3		;Zeit Signalmodus Nacht 2 laden
	cpi	r18, 1			;Position Signalmodus Nacht 2?
	breq	m87810			;ja -> bearbeiten
	lds	r16, nighbl
	lds	r17, nighbl + 1		;Zeit LCD Beleuchtung 1 laden
	cpi	r18, 2			;Position LCD Beleuchtung 1?
	breq	m87810			;ja -> bearbeiten
	lds	r16, nighbl + 2
	lds	r17, nighbl + 3		;Zeit LCD Beleuchtung 2 laden
	cpi	r18, 3			;Position LCD Beleuchtung 2?
	breq	m87810			;ja -> bearbeiten
	lds	r16, nighbt
	lds	r17, nighbt + 1		;Zeit Bluetooth 1 laden
	cpi	r18, 4			;Position Bluetooth Zeit 1?
	breq	m87810			;ja -> bearbeiten
	lds	r16, nighbt + 2
	lds	r17, nighbt + 3		;Zeit Bluetooth 2 laden
	cpi	r18, 5			;Position Bluetooth Zeit 2?
	breq	m87810			;ja -> bearbeiten
	lds	r16, aresmm
	lds	r17, aresmm + 1		;Zeit Reset Minimalwerte laden
	cpi	r18, 6			;Position Reset Minimalwerte?
	breq	m87810			;ja -> bearbeiten
	lds	r16, aresmm + 2		;sonst
	lds	r17, aresmm + 3		;Zeit Reset Maximalwerte laden
m87810:	sts	encval, r16		;Impulsgeberwert L setzen
	sts	encvah, r17		;Impulsgeberwert H setzen
	ldi	r16, 0x21		;Impulsgeberwert als Zeit anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 16			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Schaltzeiten, ndern setzen
	rjmp	m99200
;
m87850:	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	lds	r18, menpos		;ausgewhlte Position holen
	cpi	r18, 1			;Position Signalmodus Nacht 1?
	brne	m87860			;nein -> weiter testen
	sts	signig, r16
	sts	signig + 1, r17		;Signalmodus Nacht 1 speichern
	rjmp	m87950
m87860:	cpi	r18, 2			;Position Signalmodus Nacht 2?
	brne	m87870			;nein -> weiter testen
	sts	signig + 2, r16
	sts	signig + 3, r17		;Signalmodus Nacht 2 speichern
	rjmp	m87950
m87870:	cpi	r18, 3			;Position LCD Beleuchtung 1?
	brne	m87880			;nein -> weiter testen
	sts	nighbl, r16
	sts	nighbl + 1, r17		;LCD Beleuchtung 1 speichern
	rjmp	m87950
m87880:	cpi	r18, 4			;Position LCD Beleuchtung 2?
	brne	m87890			;nein -> weiter testen
	sts	nighbl + 2, r16
	sts	nighbl + 3, r17		;LCD Beleuchtung 2 speichern
	rjmp	m87950
m87890:	cpi	r18, 5			;Position Bluetooth Zeit 1?
	brne	m87900			;nein -> weiter testen
	sts	nighbt, r16
	sts	nighbt + 1, r17		;Bluetooth Zeit 1 speichern
	rjmp	m87950
m87900:	cpi	r18, 6			;Position Bluetooth Zeit 2?
	brne	m87910			;nein -> weiter testen
	sts	nighbt + 2, r16
	sts	nighbt + 3, r17		;Bluetooth Zeit 2 speichern
	rjmp	m87950
m87910:	cpi	r18, 7			;Position Reset Minimalwerte?
	brne	m87920			;nein -> weiter
	sts	aresmm, r16
	sts	aresmm + 1, r17		;Reset Minimalwerte speichern
	rjmp	m87950
m87920:	sts	aresmm + 2, r16		;sonst
	sts	aresmm + 3, r17		;Reset Maximalwerte speichern
;
m87950:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 15			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Schaltzeiten setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	call	eewcon			;Konfig im EEPROM sichern
	call	okbeep			;Tonsignal ausgeben
	rjmp	m99200
;
; Menprogramm: Konfiguration - Zeiten - Uhr stellen
;
m88000:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 17			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Uhr stellen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 6			;Wert > 5?
	brcs	m88010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m88010:	tst	r16			;Position = 0?
	brne	m88020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m88020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 6
	sts	encmax, r16		;Impulsgeber-Maximum = 6 setzen
	rcall	cpytim			;aktuelle Zeitdaten kopieren
	rjmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Zeiten - Uhr stellen, Auswhlen
;
m88100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m88150			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 6
	sts	encmax, r16		;Impulsgeber-Maximum = 6 setzen
	ldi	r16, 17			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Uhr stellen setzen
	rjmp	m99200
;
m88150:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	lds	r16, encval		;Impulsgeberwert holen
	cpi	r16, 6
	brne	m88170
;
	rcall	cftime			;Zeit aus Zwischenspeicher kopieren
	lds	r16, errflg		;Fehler-Flags holen
	cbr	r16, 0x01		;Uhren-Fehler lschen
	sts	errflg, r16		;Flags wieder speichern
	lds	r17, minutl		;Minuten Einer holen
	sts	minfl1, r17		;als Minutenflag1 voreinstellen
	sts	minfl3, r17		;als Minutenflag3 voreinstellen
	ser	r17			;0xff laden
	sts	minfl2, r17		;als Minutenflag2 voreinstellen
	lds	r17, hourl		;Stunden Einer holen
	sts	houflg, r17		;als Stundenflag voreinstellen
	bst	r16, 4			;RTC-Fehler?
	brts	m88160			;ja -> berspringen
	call	tiwrtc			;sonst Zeit in RTC bertragen
	brcc	m88160			;Fehler? nein -> weiter
	call	i2cerr			;RTC-Fehler setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	call	wrimsg			;Fehlermeldung schreiben
m88160:	rcall	okbeep
m88170:	rjmp	m99200
;
; Menprogramm: Konfiguration - Zeiten - Uhr stellen, ndern
;
m88200:	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	cpi	r18, 5			;Position auf "Zeit bernehmen"?
	brne	m88210			;nein -> weiter
	rjmp	m99200			;sonst keine Aktion
;
m88210:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
;
	clr	r19			;H-Byte fr Offset
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brtc	m88220			;nein -> ndern vorbereiten
	rjmp	m88300			;sonst neuen Wert speichern
;	
m88220:	cpi	r18, 0			;Position auf Uhrzeit?
	brne	m88230			;nein -> weiter testen
	lds	r16, cptiml
	lds	r17, cptimh		;Uhrzeit laden
	sts	encval, r16		;Impulsgeberwert L setzen
	sts	encvah, r17		;Impulsgeberwert H setzen
	ldi	r16, 0x21		;Impulsgeberwert als Zeit anzeigen
	rjmp	m88280
;
m88230:	cpi	r18, 1			;Position auf Kalendertag?
	brne	m88240			;nein -> weiter testen
	lds	r16, cpday		;sonst Tag laden
	sts	encval, r16		;als Impulsgeberwert setzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 31
	sts	encmax, r16		;Impulsgeber-Maximum = 31 setzen
	ldi	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	rjmp	m88280
;
m88240:	cpi	r18, 2			;Position auf Monat?
	brne	m88250			;nein -> weiter testen
	lds	r16, cpmon		;sonst Monat laden
	sts	encval, r16		;als Impulsgeberwert setzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 12
	sts	encmax, r16		;Impulsgeber-Maximum = 12 setzen
	ldi	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	rjmp	m88280
;
m88250:	cpi	r18, 3			;Position auf Jahr?
	brne	m88260			;nein -> weiter testen
	lds	r16, cpyear		;sonst Jahr laden
	clr	r17
	ldi	r18, low(yearx)
	ldi	r19, high(yearx)	;Jahr Basiswert laden
	add	r16, r18
	adc	r17, r19		;zum Jahreswert addieren
	sts	encval, r16
	sts	encvah, r17		;als Impulsgeberwert setzen
	ldi	r16, low(2012)
	ldi	r17, high(2012)
	sts	encmin, r16
	sts	encmih, r17		;Impulsgeber-Minimum = 2012 setzen
	ldi	r16, low(2099)
	ldi	r17, high(2099)
	sts	encmax, r16
	sts	encmah, r17		;Impulsgeber-Maximum = 2099 setzen
	ldi	r16, 0x11		;Impulsgeberwert dezimal4 anzeigen
	rjmp	m88280
;
m88260:	cpi	r18, 4			;Position auf Wochentag?
	brne	m88290			;nein -> Ende
	lds	r16, cpdayw		;sonst Wochentag laden
	inc	r16			;korrigieren
	sts	encval, r16		;als Impulsgeberwert setzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 7
	sts	encmax, r16		;Impulsgeber-Maximum = 7 setzen
	ldi	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
;
m88280:	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 18			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Uhr stellen, ndern setzen
m88290:	rjmp	m99200
;
m88300:	cpi	r18, 0			;Position auf Uhrzeit?
	brne	m88310			;nein -> weiter testen
	lds	r16, encval		;Impulsgeberwert L holen
	lds	r17, encvah		;Impulsgeberwert H holen
	sts	cptiml, r16
	sts	cptimh, r17		;Uhrzeit speichern
	rjmp	m88380
;
m88310:	cpi	r18, 1			;Position auf Kalendertag?
	brne	m88320			;nein -> weiter testen
	lds	r16, encval		;Impulsgeberwert holen
	sts	cpday, r16		;Kalendertag speichern
	rjmp	m88380
;
m88320:	cpi	r18, 2			;Position auf Monat?
	brne	m88330			;nein -> weiter testen
	lds	r16, encval		;Impulsgeberwert holen
	sts	cpmon, r16		;Monat speichern
	rjmp	m88380
;
m88330:	cpi	r18, 3			;Position auf Jahr?
	brne	m88340			;nein -> weiter testen
	lds	r16, encval
	lds	r17, encvah		;Impulsgeberwert holen
	ldi	r18, low(yearx)
	ldi	r19, high(yearx)	;Jahr Basiswert laden
	sub	r16, r18
	sbc	r17, r19		;vom Impulsgeberwert abziehen
	sts	cpyear, r16		;Jahr speichern
	rjmp	m88380
;
m88340:	cpi	r18, 4			;Position auf Wochentag?
	brne	m88390			;nein -> Ende
	lds	r16, encval		;Impulsgeberwert holen
	dec	r16			;korrigieren
	sts	cpdayw, r16		;Wochentag speichern
;
m88380:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 17			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Zeiten - Uhr stellen, Auswhlen setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
m88390:	rjmp	m99200
;
; Menprogramm: Konfiguration - Sonstiges 1
;
m88500:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 19			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sonstiges 1 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 9			;Wert > 8?
	brcs	m88510			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m88510:	tst	r16			;Position = 0?
	brne	m88520			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m88520:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	rjmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sonstiges 1, Auswhlen
;
m88600:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m88650			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	ldi	r16, 19			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sonstiges 1 setzen
	rjmp	m99200
;
m88650:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	rjmp	m99200
;
; Menprogramm: Konfiguration - Sonstiges 1, ndern
;
m88700:	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	clr	r19
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurations-Flags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurations-Flags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brtc	m88710			;nein -> ndern vorbereiten
	rjmp	m88810			;sonst neuen Wert speichern
;	
m88710:	cpi	r18, 0			;Position auf Sprache?
	brne	m88720			;nein -> weiter testen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	ldi	r17, 0x01		;Maske fr Sprachauswahl laden
	eor	r16, r17		;Sprache umschalten
	sts	confg1, r16		;Konfigurationsbyte 1 speichern
	call	eewcon			;Konfig im EEPROM sichern
	rcall	okbeep			;Tonsignal ausgeben
	rjmp	m99100
;
m88720:	cpi	r18, 1			;Position auf Trennzeichen?
	brne	m88730			;nein -> weiter testen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	ldi	r17, 0x02		;Maske fr Trennzeichen laden
	eor	r16, r17		;Trennzeichen umschalten
	sts	confg1, r16		;Konfigurationsbyte 1 speichern
	rjmp	m88800			;Konfiguration sichern
;
m88730:	cpi	r18, 2			;Position auf Temperatur-Signalisierung?
	brne	m88740			;nein -> weiter testen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	ldi	r17, 0x04		;Maske fr Temperatur-Signalisierung laden
	eor	r16, r17		;Temperatur-Signalisierung umschalten
	sts	confg1, r16		;Konfigurationsbyte 1 speichern
	rjmp	m88800			;Konfiguration sichern
;
m88740:	cpi	r18, 3			;Position auf LCD-Abschaltung?
	brne	m88750			;nein -> weiter testen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	ldi	r17, 0x40		;Maske fr LCD-Abschaltung laden
	eor	r16, r17		;LCD-Abschaltung umschalten
	sts	confg1, r16		;Konfigurationsbyte 1 speichern
	rjmp	m88800			;Konfiguration sichern
;
m88750:	cpi	r18, 4			;Position auf LCD-Backlight?
	brne	m88760			;nein -> weiter testen
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	ldi	r17, 0x02		;Maske fr LCD-Backlight laden
	eor	r16, r17		;weiches Backlight umschalten
	sts	confg2, r16		;Konfigurationsbyte 2 speichern
	rjmp	m88800			;Konfiguration sichern
;
m88760:	cpi	r18, 5			;Position auf Nacht-Helligkeit?
	brne	m88770			;nein -> weiter testen
	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
	clr	r17
	lds	r16, nlight		;Nacht-Helligkeit holen
	sts	encval, r16		;Impulsgeberwert L setzen
	sts	encvah, r17		;Impulsgeberwert H setzen
	ldi	r16, 0
	sts	encmin, r16
	sts	encmih, r17		;Minimalwert 0 setzen
	ldi	r16, 255
	sts	encmax, r16
	sts	encmah, r17		;Maximalwert 255 setzen
	ldi	r16, 0x11		;Impulsgeberwert dezimal4 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 20			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sonstiges 1, ndern setzen
	rjmp	m99200
;
m88770: cpi     r18, 6                  ;Position auf Seite-1-Rckschaltung?
	brne	m88780			;nein -> weiter
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	ldi	r17, 0x08		;Maske fr Seite-1-Rckschaltung laden
	eor	r16, r17		;Seite-1-Rckschaltung umschalten
	sts	confg2, r16		;Konfigurationsbyte 2 speichern
	rjmp	m88800			;Konfiguration sichern
;
m88780:	lds	r16, baud0		;Baudrate PC-Schnittstelle
	inc	r16			;Wert erhhen
	cpi	r16, 7			;Endwert berschritten?
	brcs	m88790			;nein -> weiter
	clr	r16			;sonst auf Anfangswert setzen
m88790:	sts	baud0, r16		;Baudrate wieder speichern
	call	setbd0			;neue Baudrate setzen
m88800:	call	eewcon			;Konfig im EEPROM sichern
	rcall	okbeep			;Tonsignal ausgeben
	rjmp	m99200
;
m88810:	cpi	r18, 5			;Position auf Nacht-Helligkeit?
	brne	m88820			;nein -> Ende
	lds	r16, encval		;Impulsgeberwert L holen
	sts	nlight, r16		;neue Nacht-Helligkeit speichern
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 19			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sonstiges 1 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
;
        ldi     r16, timnli             ;Timeout-Wert fr Nacht-Helligkeit laden
        sts     nlitio, r16             ;und Zhler fr Nacht-Helligkeit setzen
	lds	r16, dflags		;Anzeigeflags holen
	sbr	r16, 0x0c		;Info/Menzeile neu ausgeben
	sts	dflags, r16		;Anzeigeflags wieder speichern
	call	eewcon			;Konfig im EEPROM sichern
	rcall	okbeep			;Tonsignal ausgeben
m88820:	rjmp	m99200
;
; Menprogramm: Konfiguration - Backup, Sichern
;
m89000:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicherfehler?
	brts	m89010			;ja -> berspringen
	bst	r16, 2			;Konfigurationsfehler?
	brts	m89010			;ja -> berspringen
	call	xwconf			;sonst Konfiguration sichern
	brcs	m89020			;Fehler? ja -> melden
	call	okbeep			;sonst Tonsignal ausgeben
m89010:	rjmp	m99200
;
m89020:	lds	r16, errflg		;Fehler-Flags holen
	sbr	r16, 0x08		;Speicherfehler setzen
	sts	errflg, r16		;Flags wieder speichern
	rjmp	m99200
;
; Menprogramm: Konfiguration - Backup, Laden/Neustart
;
m89100:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicherfehler?
	brts	m89010			;ja -> berspringen
	call	strwai			;Warte-Text ausgeben
	call	xrconf			;sonst Konfiguration laden
	brcs	m89020			;Fehler? ja -> melden
m89110:	cli				;sonst Interrupts sperren
	call	wait50			;50ms warten
	jmp	start			;System neu starten
;
; Menprogramm: Konfiguration - Grundeinstellung - Basiskonfiguration laden
;
m89500:	call	strwai			;Warte-Text ausgeben
	call	eedcfg			;Default-Konfiguration laden
	rjmp	m89110			;System neu starten
;
; Menprogramm: Konfiguration - Grundeinstellung - 15min Werte lschen
;
m89600:	lds	r16, errflg		;Fehler-Flags holen
	andi	r16, 0x18		;Ext-EEPROM- oder RTC-Fehler?
	brne	m89630			;ja - berspringen
	call	strwai			;Warte-Text ausgeben
	ldi	r20, low(xs1sta)
	ldi	r21, high(xs1sta)	        ;Anfangsadresse von EEPROM-
	ldi	r22, byte3(xs1sta)	        ;Speicherbereich 1
	ldi	r23, low(xs1sta + xs1len)
	ldi	r24, high(xs1sta + xs1len)	;Endadresse von EEPROM-
	ldi	r25, byte3(xs1sta + xs1len)     ;Speicherbereich 1
	call	eraepr			        ;EEPROM-Speicherbereich lschen
	brcc	m89610			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
m89610:	ldi	r16, low(xs1sta)
	ldi	r17, high(xs1sta)	;Anfangsadresse von EEPROM-
	ldi	r18, byte3(xs1sta)	;Speicherbereich 1
	sts	xes1po, r16
	sts	xes1po + 1, r17		;als neuen Pointerwert 1
	sts	xes1po + 2, r18		;speichern
	ldi	xl, low(xes1po)
	ldi	xh, high(xes1po)	;Zeiger auf Pointer 1 setzen
	ldi	r18, rtxs1p		;RTC-Adresse von Pointer 1
	call	i2psen			;Pointer in RTC schreiben
	brcc	m89620			;Fehler? nein -> weiter
	call	i2cerr			;sonst RTC-Fehler setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	rcall	wrimsg			;Fehlermeldung schreiben
m89620:	rcall	okbeep			;Tonsignal ausgeben
m89630:	rjmp	m99200			;Ende, Men-Rckkehr
;
; Menprogramm: Konfiguration - Grundeinstellung - 24h Werte lschen
;
m89700:	lds	r16, errflg		;Fehler-Flags holen
	andi	r16, 0x18		;Ext-EEPROM- oder RTC-Fehler?
	brne	m89730			;ja - berspringen
	call	strwai			;Warte-Text ausgeben
	ldi	r20, low(xs2sta)
	ldi	r21, high(xs2sta)	        ;Anfangsadresse von EEPROM-
	ldi	r22, byte3(xs2sta)	        ;Speicherbereich 2
	ldi	r23, low(xs2sta + xs2len)
	ldi	r24, high(xs2sta + xs2len)	;Endadresse von EEPROM-
	ldi	r25, byte3(xs2sta + xs2len)     ;Speicherbereich 2
	call	eraepr			        ;EEPROM-Speicherbereich lschen
	brcc	m89710			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
m89710:	ldi	r16, low(xs2sta)
	ldi	r17, high(xs2sta)	;Anfangsadresse von EEPROM-
	ldi	r18, byte3(xs2sta)	;Speicherbereich 2
	sts	xes2po, r16
	sts	xes2po + 1, r17		;als neuen Pointerwert 2
	sts	xes2po + 2, r18		;speichern
	ldi	xl, low(xes2po)
	ldi	xh, high(xes2po)	;Zeiger auf Pointer 2 setzen
	ldi	r18, rtxs2p		;RTC-Adresse von Pointer 2
	call	i2psen			;Pointer in RTC schreiben
	brcc	m89720			;Fehler? nein -> weiter
	call	i2cerr			;sonst RTC-Fehler setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	rcall	wrimsg			;Fehlermeldung schreiben
m89720:	rcall	okbeep			;Tonsignal ausgeben
m89730:	rjmp	m99200			;Ende, Men-Rckkehr
;
; Menprogramm: Konfiguration - Grundeinstellung - Meldungen lschen
;
m89800:	lds	r16, errflg		;Fehler-Flags holen
	andi	r16, 0x18		;Ext-EEPROM- oder RTC-Fehler?
	brne	m89830			;ja - berspringen
	call	strwai			;Warte-Text ausgeben
	ldi	r20, low(xmssta)
	ldi	r21, high(xmssta)	        ;Anfangsadresse von EEPROM-
	ldi	r22, byte3(xmssta)	        ;Speicherbereich 3
	ldi	r23, low(xmssta + xmslen)
	ldi	r24, high(xmssta + xmslen)	;Endadresse von EEPROM-
	ldi	r25, byte3(xmssta + xmslen)     ;Speicherbereich 3
	call	eraepr			        ;EEPROM-Speicherbereich lschen
	brcc	m89810			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
m89810:	ldi	r16, low(xmssta)
	ldi	r17, high(xmssta)	;Anfangsadresse von EEPROM-
	ldi	r18, byte3(xmssta)	;Speicherbereich 3
	sts	xemspo, r16
	sts	xemspo + 1, r17		;als neuen Pointerwert 3
	sts	xemspo + 2, r18		;speichern
	ldi	xl, low(xemspo)
	ldi	xh, high(xemspo)	;Zeiger auf Pointer 3 setzen
	ldi	r18, rtxmsp		;RTC-Adresse von Pointer 3
	call	i2psen			;Pointer in RTC schreiben
	brcc	m89820			;Fehler? nein -> weiter
	call	i2cerr			;sonst RTC-Fehler setzen
	ldi	r19, 0xf4		;Meldung "RTC-Fehler" setzen
        ldi     r20, 0                  ;kein Parameter
	rcall	wrimsg			;Fehlermeldung schreiben
m89820:	rcall	okbeep			;Tonsignal ausgeben
m89830:	rjmp	m99200			;Ende, Men-Rckkehr
;
; Menprogramm: Grafik - Preset-01
;
m8a000:	ldi	r16, 0			;Preset-01 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-02
;
m8a010:	ldi	r16, 1			;Preset-02 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-03
;
m8a020:	ldi	r16, 2			;Preset-03 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-04
;
m8a030:	ldi	r16, 3			;Preset-04 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-05
;
m8a040:	ldi	r16, 4			;Preset-05 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-06
;
m8a050:	ldi	r16, 5			;Preset-06 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-07
;
m8a060:	ldi	r16, 6			;Preset-07 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-08
;
m8a070:	ldi	r16, 7			;Preset-08 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-09
;
m8a080:	ldi	r16, 8			;Preset-09 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-10
;
m8a090:	ldi	r16, 9			;Preset-10 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-11
;
m8a100:	ldi	r16, 10			;Preset-11 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-12
;
m8a110:	ldi	r16, 11			;Preset-12 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-13
;
m8a120:	ldi	r16, 12			;Preset-13 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-14
;
m8a130:	ldi	r16, 13			;Preset-14 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-15
;
m8a140:	ldi	r16, 14			;Preset-15 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-16
;
m8a150:	ldi	r16, 15			;Preset-16 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-17
;
m8a160:	ldi	r16, 16			;Preset-17 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-18
;
m8a170:	ldi	r16, 17			;Preset-18 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-19
;
m8a180:	ldi	r16, 18			;Preset-19 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-20
;
m8a190:	ldi	r16, 19			;Preset-20 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-21
;
m8a200:	ldi	r16, 20			;Preset-21 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-22
;
m8a210:	ldi	r16, 21			;Preset-22 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-23
;
m8a220:	ldi	r16, 22			;Preset-23 auswhlen
	rjmp	m8a250
;
; Menprogramm: Grafik - Preset-24
;
m8a230:	ldi	r16, 23			;Preset-24 auswhlen
;
m8a250:	sts	grpres, r16		;Grafik-Preset speichern
	ldi	r17, 7			;Grafik-Presets jeweils 7 Bytes
	mul	r16, r17		;Tabellenoffset ermitteln
	ldi	r16, low(egrpre)	;EEPROM-Adresse Presets L
	ldi	r17, high(egrpre)	;EEPROM-Adresse Presets H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
;
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse11, r18		;als Sensornummer 1 speichern
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse12, r18		;als Differenz-Sensornummer 1 speichern
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse21, r18		;als Sensornummer 2 speichern
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse22, r18		;als Differenz-Sensornummer 2 speichern
	call	eeread			;Zeichen aus EEPROM holen
	andi	r18, 0x03		;relevante Bits filtern
	sts	grmod1, r18		;als Anzeigemodus 1 speichern
	call	eeread			;Zeichen aus EEPROM holen
	andi	r18, 0x03		;relevante Bits filtern
	sts	grmod2, r18		;als Anzeigemodus 2 speichern
	call	eeread			;Zeichen aus EEPROM holen
	sts	grflag, r18		;als Grafik-Flags speichern
;
	lds	r20, dflags		;Anzeige-Flags holen
	cbr	r20, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r20		;Flags wieder speichern
	ldi	r20, 23			;alternativen Anzeigemodus
	sts	dimode, r20		;auf Grafikanzeige setzen
	lds	r20, xflag3		;verschiedene Flags 3 holen
	sbr	r20, 1 << 3 | 1 << 1	        ;Grafikdaten aus EEPROM lesen
	cbr	r20, 1 << 5 | 1 << 4 | 1 << 2	;Grafik noch nicht anzeigen
	sts	xflag3, r20		        ;Flags wieder speichern
	lds	r16, grmod1		;Anzeigemodus 1 holen
	tst	r16			;Modus 15min-Anzeige?
	brne	m8a260			;nein -> weiter
;
	lds	r20, xes1po		;sonst
	lds	r21, xes1po + 1		;aktuelle Position fr 15min
	lds	r22, xes1po + 2		;Temperaturdaten holen
	rjmp	m8a270
;
m8a260:	lds	r20, xes2po
	lds	r21, xes2po + 1		;aktuelle Position fr 24h
	lds	r22, xes2po + 2		;Temperaturdaten holen
;
m8a270:	call	grdecx			;Zeiger auf letzten Datensatz
	sts	grpend, r20
	sts	grpend + 1, r21		;und Zeiger fr letzten Wert
	sts	grpend + 2, r22		;der Grafikanzeige speichern
	rjmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Grafik - Preset, Rckwrts blttern
;
m8a300:	lds	r19, xflag3		;verschiedene Flags 3 holen
	bst	r19, 5			;erster Datensatz erreicht?
	brtc	m8a310			;nein -> weiter
	rjmp	m99200			;sonst keine Neuausgabe
;
m8a310:	lds	r19, xflag3		;verschiedene Flags 3 holen
	sbr	r19, 1 << 3 | 1 << 1	;Grafikdaten aus EEPROM lesen
	cbr	r19, 1 << 4 | 1 << 2	;Grafik noch nicht anzeigen
	sts	xflag3, r19		;Flags wieder speichern
;
	lds	r16, grpsta
	lds	r17, grpsta + 1		;Position vom ersten Grafik-
	lds	r18, grpsta + 2		;pufferwert holen
	sts	grptem, r16		;auerdem
	sts	grptem + 1, r17		;als Zeiger fr aktuellen Wert
	sts	grptem + 2, r18		;der Grafikanzeige speichern
	call	grdeca			;Adresse vorheriger Datensatz
	sts	grpend, r20
	sts	grpend + 1, r21		;und Zeiger fr letzten Wert
	sts	grpend + 2, r22		;der Grafikanzeige speichern
	rjmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Grafik - Preset, Vorwrts blttern
;
m8a400:	lds	r19, xflag3		;verschiedene Flags 3 holen
	bst	r19, 4			;letzter Datensatz erreicht?
	brtc	m8a410			;nein -> weiter
	rjmp	m99200			;sonst keine Neuausgabe
;
m8a410:	lds	r16, grpend
	lds	r17, grpend + 1		;Position vom letzten Grafik-
	lds	r18, grpend + 2		;pufferwert holen
	sts	grptem, r16
	sts	grptem + 1, r17		;als Zeiger fr aktuellen Wert
	sts	grptem + 2, r18		;der Grafikanzeige speichern
	call	grinca			;Adresse nchster Datensatz
	lds	r16, grmod1		;Anzeigemodus 1 holen
	tst	r16			;Modus 15-min-Anzeige?
	brne	m8a420			;nein -> weiter
;
	lds	r16, xes1po		;sonst
	lds	r17, xes1po + 1		;aktuelle Position fr 15-min
	lds	r18, xes1po + 2		;Temperaturdaten holen
	rjmp	m8a430
;
m8a420:	lds	r16, xes2po
	lds	r17, xes2po + 1		;aktuelle Position fr 24-h
	lds	r18, xes2po + 2		;Temperaturdaten holen
;
m8a430:	sub	r16, r20
	sbc	r17, r21		;zeigt die Grafik die letzten
	sbc	r18, r22		;Werte?
	brne	m8a440			;nein -> weiter
	rjmp	m99200			;sonst keine Neuausgabe
;
m8a440:	lds	r19, xflag3		;verschiedene Flags 3 holen
	sbr	r19, 1 << 1		        ;Grafikdaten aus EEPROM lesen
	cbr	r19, 1 << 5 | 1 << 3 | 1 << 2	;Grafik noch nicht anzeigen
	sts	xflag3, r19		        ;Flags wieder speichern
;
	sts	grpsta, r20
	sts	grpsta + 1, r21		;und als Zeiger fr ersten Wert
	sts	grpsta + 2, r22		;der Grafikanzeige speichern
	sts	grptem, r20		;auerdem
	sts	grptem + 1, r21		;als Zeiger fr aktuellen Wert
	sts	grptem + 2, r22		;der Grafikanzeige speichern
	rjmp	m99100
;
; Menprogramm: Grafik - Preset, Fadenkreuz aktivieren
;
m8a500:	clr	r17			;H-Byte = 0
	lds	r16, grlacr		;letzte Fadenkreuzposition
	cpi	r16, 222		;gltige Position?
	brcs	m8a510			;ja -> bernehmen, sonst
	ldi	r16, 110		;Fadenkreuzposition auf Mitte
        sts     grlacr, r16             ;und speichern
m8a510:	sts	encval, r16		;Impulsgeberwert L setzen
	sts	encvah, r17		;Impulsgeberwert H setzen
	clr	r16
	sts	encmin, r16
	sts	encmih, r17		;Minimalwert 0 setzen
	ldi	r16, 221
	sts	encmax, r16
	sts	encmah, r17		;Maximalwert 221 setzen
	ldi	r16, 0x11		;Impulsgeberwert dezimal4 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
	ldi	r16, 24			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Grafikanzeige, Fadenkreuz
;
        lds     r18, grlacr		;letzte Fadenkreuzposition holen
	sts	grpoin, r18		;als Fadenkreuzposition setzen
	call	grscxp			;X-Koordinaten fr Fadenkreuz
	lds	r18, grpoin		;aktuelle Fadenkreuzposition holen
	call	grscrb			;LCD-Daten Fadenkreuz sichern
	lds	r18, grpoin		;aktuelle Fadenkreuzpositio holen
	call	grcros			;Fadenkreuz zeichnen
	rjmp	m99200
;
; Menprogramm: Grafik - Preset, Sensornamen anzeigen
;
m8a600:	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 0			;Option "Sensornamen" aktiv?
	brts	m8a630			;ja -> bearbeiten
	sbr	r16, 1 << 0		;sonst Option einschalten
	sts	grflag, r16		;Grafik-Flags wieder speichern

m8a610:	lds	r16, xflag3		;verschiedene Flags 3 holen
	sbr	r16, 1 << 2		;Grafik neu ausgeben
	sts	xflag3, r16		;Flags wieder speichern
;
m8a620:	ldi	r16, low(egrpre)
	ldi	r17, high(egrpre)	;EEPROM-Adresse Preset-Daten
	lds	r18, grpres		;aktuelle Grafik-Preset-Nummer
	ldi	r19, 7			;Offset 7 Bytes
	mul	r18, r19		;Adressoffset Preset-Daten
	add	r16, r0
	adc	r17, r1			;EEPROM-Adresse ermitteln
	ldi	r18, 6			;Offset fr Grafik-Flags
	clr	r19
	add	r16, r18
	adc	r17, r19		;EEPROM-Adresse Grafik-Flags
	lds	r18, grflag		;Grafik-Flags holen
	call	eewrit			;im EEPROM speichern
	call	eewcrc			;EERPOM-CRC berechnen/speichern
	rcall	okbeep			;Tonsignal ausgeben
	rjmp	m99200
;
m8a630:	cbr	r16, 1 << 0		;Option ausschalten
	sts	grflag, r16		;Grafik-Flags wieder speichern
	ldi	r18, 16
	ldi	r19, 16			;Start-Koordinaten
	ldi	r20, 238
	ldi	r21, 103		;End-Koordinaten
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 4			;Doppeldiagramm?
	brtc	m8a640			;nein -> ein Diagramm lschen
	ldi	r21, 59			;sonst Endkoordinate Diagramm 1
	call	clrwnd			;Grafikbereich lschen
	ldi	r18, 16
	ldi	r19, 60			;Start-Koordinaten
	ldi	r20, 238
	ldi	r21, 103		;End-Koordinaten
m8a640:	call	clrwnd			;Grafikbereich lschen
	rjmp	m8a610			;und neu ausgeben
;
; Menprogramm: Grafik - Preset, Skala angleichen
;
m8a700:	lds	r16, grflag		;Grafik-Flag holen
	bst	r16, 4			;Doppelgrafik?
	brtc	m8a720			;nein -> Option ignorieren
;
	lds	r16, grflag		;Grafik-Flags holen
	bst	r16, 1			;Option "Skala" aktiv?
	brts	m8a730			;ja -> bearbeiten
	sbr	r16, 1 << 1		;sonst Option einschalten
m8a710:	sts	grflag, r16		;Grafik-Flags wieder speichern
;
	ldi	r18, 1
	ldi	r19, 16			;Start-Koordinaten Diagramm 1
	ldi	r20, 15
	ldi	r21, 59			;End-Koordinaten Diagramm 1
	call	clrwnd			;Grafikbereich lschen
	ldi	r18, 16
	ldi	r19, 16			;Start-Koordinaten Diagramm 1
	ldi	r20, 238
	ldi	r21, 59			;End-Koordinaten Diagramm 1
	call	clrwnd			;Grafikbereich lschen
	ldi	r18, 1
	ldi	r19, 60			;Start-Koordinaten Diagramm 1
	ldi	r20, 15
	ldi	r21, 104		;End-Koordinaten Diagramm 1
	call	clrwnd			;Grafikbereich lschen
	ldi	r18, 16
	ldi	r19, 60			;Start-Koordinaten Diagramm 2
	ldi	r20, 238
	ldi	r21, 103		;End-Koordinaten Diagramm 2
	call	clrwnd			;Grafikbereich lschen
	rjmp	m8a610			;Grafik neu ausgeben, nderung im EEPROM speichern
;
m8a720:	rjmp	m99200
;
m8a730:	cbr	r16, 1 << 1		;Option ausschalten
	rjmp	m8a710
;
; Menprogramm: Grafik - Preset, Schriftgre
;
m8a800:	lds	r16, grflag		;Grafik-Flags holen
        bst     r16, 0                  ;Option "Sensornamen" aktiv?
        brtc    m8a720                  ;nein -> Option ignorieren
;
	bst	r16, 5			;Option "Schriftgre" aktiv?
	brts	m8a830			;ja -> bearbeiten
	sbr	r16, 1 << 5		;sonst Option einschalten
m8a810:	sts	grflag, r16		;Grafik-Flags wieder speichern
;
	ldi	r18, 16
	ldi	r19, 16			;Start-Koordinaten
	ldi	r20, 238
	ldi	r21, 103		;End-Koordinaten
	bst	r16, 4			;Doppeldiagramm?
	brtc	m8a820			;nein -> ein Diagramm lschen
	ldi	r21, 59			;sonst Endkoordinate Diagramm 1
	call	clrwnd			;Grafikbereich lschen
	ldi	r18, 16
	ldi	r19, 60			;Start-Koordinaten Diagramm 2
	ldi	r20, 238
	ldi	r21, 103		;End-Koordinaten Diagramm 2
m8a820:	call	clrwnd			;Grafikbereich lschen
	rjmp	m8a610			;Grafik neu ausgeben, nderung im EEPROM speichern
;
m8a830:	cbr	r16, 1 << 5		;Option ausschalten
	rjmp	m8a810
;
; Menprogramm: Konfiguration - Grafik-Presets
;
m8b000:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 25			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Grafik-Presets setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 9			;Wert > 8?
	brcs	m8b010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m8b010:	tst	r16			;Position = 0?
	brne	m8b020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m8b020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
;
m8b030:	lds	r16, grpres		;aktuelles Grafik-Preset holen
	ldi	r17, 7			;Grafik-Presets jeweils 7 Bytes
	mul	r16, r17		;Tabellenoffset ermitteln
	ldi	r16, low(egrpre)	;EEPROM-Adresse Presets L
	ldi	r17, high(egrpre)	;EEPROM-Adresse Presets H
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
;
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse11, r18		;als Sensornummer 1 speichern
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse12, r18		;als Differenz-Sensornummer 1 speichern
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse21, r18		;als Sensornummer 2 speichern
	call	eeread			;Zeichen aus EEPROM holen
        andi    r18, 0x1f               ;Sensornummer filtern
	sts	grse22, r18		;als Differenz-Sensornummer 2 speichern
	call	eeread			;Zeichen aus EEPROM holen
	andi	r18, 0x03		;relevante Bits filtern
	sts	grmod1, r18		;als Grafik-Anzeigemodus 1 speichern
	call	eeread			;Zeichen aus EEPROM holen
	andi	r18, 0x03		;relevante Bits filtern
	sts	grmod2, r18		;als Grafik-Anzeigemodus 2 speichern
	call	eeread			;Zeichen aus EEPROM holen
	sts	grflag, r18		;als Grafik-Flags speichern
	rjmp	m99100			;Ende, Anzeige neu ausgeben
; 
; Menprogramm: Konfiguration - Grafik-Presets, Auswhlen
;
m8b100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m8b150			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 8
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	ldi	r16, 25			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Grafik-Presets
	rjmp	m99200
;
m8b150:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	rjmp	m99200
; 
; Menprogramm: Konfiguration - Grafik-Presets, ndern
;
m8b200:	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	clr	r19
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurationsflags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brtc	m8b210			;nein -> ndern vorbereiten
	rjmp	m8b400			;sonst neuen Wert speichern
;
; Menprogramm: Konfiguration - Grafik-Presets, ndern (nderung vorbereiten)
;
m8b210:	lds	r17, grflag		;Grafik-Flags holen
	cpi	r18, 0			;Position auf Preset-Nummer?
	brne	m8b220			;nein -> weiter testen
	lds	r16, grpres		;aktuelle Preset-Nummer holen
	inc	r16			;korrigieren
	ldi	r18, 1			;Minimalwert 1 setzen
	ldi	r19, 24			;Maximalwert 24 setzen
	rjmp	m8b330			;Impulsgeberparameter setzen
;
m8b220:	cpi	r18, 1			;Position auf Preset-Name?
	brne	m8b230			;nein -> weiter testen
	lds	r18, grpres		;Grafik-Preset-Nummer holen
	ldi	r19, 16			;Sensornamenlnge im EEPROM
	ldi	r16, low(egrprt)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(egrprt)	;EEPROM-Adresse Sensornamen H
	call	cpeeab			;EEPROM-Text in Eingabepuffer
	ldi	r16, 0x09		;Impulsgeberwert als ASCII anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
        lds     r16, asclst             ;Anzahl der Zeichen in der Liste holen
        inc     r16                     ;auf Ok-Zeichen setzen
	sts	encval, r16		;und speichern
	rjmp	m8b340			;auf ndern umschalten
;
m8b230:	cpi	r18, 2			;Position auf Grafik 1 Sensor?
	brne	m8b240			;nein -> weiter testen
	lds	r16, grse11		;Grafik 1 Sensornummer holen
	inc	r16			;korrigieren
	ldi	r18, 1			;Minimalwert 1 setzen
	ldi	r19, 32			;Maximalwert 32 setzen
	rjmp	m8b330			;Impulsgeberparameter setzen
;
m8b240:	cpi	r18, 3			;Position auf Grafik 1 Differenz-Sensor?
	brne	m8b260			;nein -> weiter testen
        lds     r20, grse11             ;Grafik 1 Sensor holen
	lds	r16, grse12		;Grafik 1 Differenz-Sensornummer holen
	inc	r16			;korrigieren
	ldi	r18, 0			;Minimalwert 0 setzen
	ldi	r19, 27			;Maximalwert 27 setzen
        cpi     r20, 27                 ;Grafik 1 Sensor = Temperatursensor?
        brcs    m8b250                  ;ja -> weiter
        rjmp    m8b360                  ;sonst keine Eingabe mglich
;
m8b250:	bst	r17, 2			;Differenzmodus aktiv?
	brts	m8b330			;ja -> Impulsgeberparameter setzen
	clr	r16			;sonst Sensornummer = 0 setzen (aus)
	rjmp	m8b330			;Impulsgeberparameter setzen
;
m8b260:	cpi	r18, 4			;Position auf Grafik 1 Modus?
	brne	m8b270			;nein -> weiter testen
	lds	r16, grmod1		;Grafik 1 Modus holen
	ldi	r18, 0			;Minimalwert 0 setzen
	ldi	r19, 3			;Maximalwert 3 setzen
	rjmp	m8b330			;Impulsgeberparameter setzen
;
m8b270:	cpi	r18, 5			;Position auf Grafik 2 Sensor?
	brne	m8b290			;nein -> weiter testen
	clr	r16			;Wert fr "aus" voreinstellen
	bst	r17, 4			;Doppelgrafik?
	brtc	m8b280			;nein -> "aus" voreinstellen
	lds	r16, grse21		;Grafik 2 Sensornummer holen
	inc	r16			;korrigieren
m8b280:	ldi	r18, 0			;Minimalwert 0 setzen
	ldi	r19, 32			;Maximalwert 32 setzen
	rjmp	m8b330			;Impulsgeberparameter setzen
;
m8b290:	cpi	r18, 6			;Position auf Grafik 2 Differenz-Sensor?
	brne	m8b300			;nein -> Grafik 2 Modus
	bst	r17, 4			;Doppelgrafik?
	brtc	m8b360			;nein -> keine Eingabe mglich
;
        lds     r20, grse21             ;Grafik 2 Sensor holen
	lds	r16, grse22		;Grafik 2 Differenz-Sensornummer holen
	inc	r16			;korrigieren
	ldi	r18, 0			;Minimalwert 0 setzen
	ldi	r19, 27			;Maximalwert 27 setzen
        cpi     r20, 27                 ;Grafik 2 Sensor = Temperatursensor?
        brcc    m8b360                  ;nein -> keine Eingabe mglich
;
	bst	r17, 3			;Differenzmodus aktiv?
	brts	m8b330			;ja -> Impulsgeberparameter setzen
	clr	r16			;sonst Sensornummer = 0 setzen (aus)
	rjmp	m8b330			;Impulsgeberparameter setzen
;
m8b300:	bst	r17, 4			;Doppelgrafik?
	brtc	m8b360			;nein -> keine Eingabe mglich
	lds	r16, grmod1		;Grafik 1 Modus holen
	lds	r17, grmod2		;Grafik 2 Modus holen
	cpi	r16, 0			;Grafik 1 Modus 0?
	brne	m8b310			;nein -> weiter
	mov	r17, r16		;sonst Grafik Modus kopieren
	mov	r18, r16		;Minimalwert 0 setzen
	mov	r19, r16		;Maximalwert 0 setzen
	rjmp	m8b330			;Impulsgeberparameter setzen
;
m8b310:	cpi	r17, 0			;Grafik 2 Modus 0?
	breq	m8b320			;ja -> Wert wird von Grafik 1 Modus bernommen
	mov	r16, r17		;sonst Grafik 2 Modus verwenden
m8b320:	ldi	r18, 1			;Minimalwert 1 setzen
	ldi	r19, 3			;Maximalwert 3 setzen
m8b330:	sts	encval, r16		;Impulsgeberwert setzen
	sts	encmin, r18		;Minimalwert setzen
	sts	encmax, r19		;Maximalwert setzen
	ldi	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags setzen
m8b340:	ldi	r16, 26			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Grafik-Presets, ndern setzen
m8b350:	ldi	r16, 0x60		;Stringnummer "ndern"
	sts	menext, r16		;Menerweiterung setzen
m8b360:	rjmp	m99200
;
; Menprogramm: Konfiguration - Grafik-Presets, ndern (nderung durchfhren)
;
m8b400:	lds	r16, encval		;Impulsgeberwert holen
	cpi	r18, 0			;Position auf Preset-Nummer?
	brne	m8b410			;nein -> weiter testen
	dec	r16			;korrigieren
	sts	grpres, r16		;Preset-Nummer speichern
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 25			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Grafik setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	rjmp	m8b030			;Preset-Daten ausgeben
;
m8b410:	cpi	r18, 1			;Position auf Preset-Name?
	brne	m8b450			;nein -> weiter testen
	lds	r20, ascibp		;aktuelle Pufferposition holen
        lds     r17, asclst             ;Anzahl der Zeichen holen
        cp      r16, r17                ;Lschzeichen?
        breq    m8b420                  ;ja -> bearbeiten
        inc     r17                     ;Anzahl der Zeichen korrigieren
	cp	r16, r17	        ;Ok-Zeichen?
	breq	m8b440			;ja -> String speichern
;
	cpi	r20, 16			;letzte Position?
	breq	m8b350			;ja -> Zeichen ignorieren
        call    getchr                  ;sonst Zeichen aus Liste holen
	ldi	xl, low(ascibf)
	ldi	xh, high(ascibf)	;Zeiger auf Texteingabepuffer
	clr	r21
	add	xl, r20			;Pufferadresse ermitteln
	adc	xh, r21			;bertrag
	st	x, r16			;Zeichen speichern
	inc	r20			;aktuelle Position erhhen
	rjmp	m8b430			;speichern
;
m8b420:	tst	r20			;aktuelle Position = 0?
	breq	m8b350			;ja -> berspringen
	dec	r20			;sonst eine Position zurck
m8b430:	sts	ascibp, r20		;neue Position speichern	
	rjmp	m8b350			;Menerweiterung neu setzen
;
m8b440:	lds	r18, grpres		;Grafik-Preset-Nummer holen
	ldi	r19, 16			;Sensornamenlnge im EEPROM
	ldi	r16, low(egrprt)	;EEPROM-Adresse Sensornamen L
	ldi	r17, high(egrprt)	;EEPROM-Adresse Sensornamen H
	call	cpabee			;Texteingabepuffer in EEPROM
	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 25			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Grafik-Presets setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	rjmp	m8b610			;CRC berechnen und speichern
;
m8b450:	cpi	r18, 2			;Position auf Grafik 1 Sensor?
	brne	m8b470			;nein -> weiter testen
	dec	r16			;Impulsgeberwert korrigieren
        cpi     r16, 27                 ;Grafik 1 Sensor = Temperatursensor?
        brcs    m8b460                  ;ja -> weiter
	lds	r17, grflag		;sonst Grafik-Flags holen
	cbr	r17, 1 << 2		;Differenzmodus ausschalten
        clr     r18                     ;Differenz-Sensornummer lschen
        sts     grse12, r18             ;Differenz-Sensornummer speichern
        sts     grflag, r17             ;Grafik-Flags speichern
m8b460:	sts	grse11, r16		;Grafik 1 Sensor speichern
	rjmp	m8b600			;Daten im EEPROM speichern
;
m8b470:	cpi	r18, 3			;Position auf Grafik 1 Differenz-Sensor?
	brne	m8b500			;nein -> weiter testen
	lds	r17, grflag		;Grafik-Flags holen
	tst	r16			;Impulsgeberwert = 0 (aus)?
	brne	m8b480			;nein -> weiter
	cbr	r17, 1 << 2		;Differenzmodus ausschalten
	rjmp	m8b490
;
m8b480:	sbr	r17, 1 << 2		;Differenzmodus einschalten
	dec	r16			;Impulsgeberwert korrigieren
m8b490:	sts	grflag, r17		;Grafik-Flags wieder speichern
	sts	grse12, r16		;Grafik 1 Differenz-Sensor speichern
	rjmp	m8b600			;Daten im EEPROM speichern
;
m8b500:	cpi	r18, 4			;Position auf Grafik 1 Modus?
	brne	m8b510			;nein -> weiter testen
	sts	grmod1, r16		;Grafik 1 Modus speichern
	rjmp	m8b540			;Modus prfen, Daten speichern
;
m8b510:	cpi	r18, 5			;Position auf Grafik 2 Sensor?
	brne	m8b560			;nein -> weiter testen
	lds	r17, grflag		;Grafik-Flags holen
	tst	r16			;Impulsgeberwert = 0 (aus)?
	brne	m8b520			;nein -> weiter
	cbr	r17, 1 << 4 | 1 << 3	;sonst Doppelgrafik/Differenzmodus aus
	rjmp	m8b530                  ;Daten speichern
;
m8b520:	sbr	r17, 1 << 4		;Doppelgrafik einschalten
	dec	r16			;Impulsgeberwert korrigieren
        cpi     r16, 27                 ;Grafik 2 Sensor = Temperatursensor?
        brcs    m8b530                  ;ja -> weiter
	cbr	r17, 1 << 3		;sonst Differenzmodus ausschalten
        clr     r18                     ;Differenz-Sensornummer lschen
        sts     grse22, r18             ;Differenz-Sensornummer speichern
m8b530:	sts	grflag, r17		;Grafik-Flags wieder speichern
	sts	grse21, r16		;Grafik 2 Sensor speichern
;
m8b540:	lds	r16, grmod1		;Grafik 1 Modus holen
	lds	r17, grmod2		;Grafik 2 Modus holen
	tst	r16			;Grafik 1 Modus = 0?
	breq	m8b550			;ja -> in Grafik 2 kopieren
	tst	r17			;sonst Grafik 2 Modus = 0?
	brne	m8b600			;nein -> Daten im EEPROM speichern
m8b550:	sts	grmod2, r16		;sonst Grafik Modus kopieren
	rjmp	m8b600			;Daten im EEPROM speichern
;
m8b560:	cpi	r18, 6			;Position auf Grafik 2 Differenz-Sensor?
	brne	m8b590			;nein -> Grafik 2 Modus
	lds	r17, grflag		;Grafik-Flags holen
	tst	r16			;Impulsgeberwert = 0 (aus)?
	brne	m8b570			;nein -> weiter
	cbr	r17, 1 << 3		;Differenzmodus ausschalten
	rjmp	m8b580
;
m8b570:	sbr	r17, 1 << 3		;Differenzmodus einschalten
	dec	r16			;Impulsgeberwert korrigieren
m8b580:	sts	grflag, r17		;Grafik-Flags wieder speichern
	sts	grse22, r16		;Grafik 2 Differenz-Sensor speichern
	rjmp	m8b600			;Daten im EEPROM speichern
;
m8b590:	sts	grmod2, r16		;Grafik 2 Modus speichern
	rjmp	m8b540			;Modus prfen, Daten speichern
;
m8b600:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 25			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Grafik-Presets setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
;
	ldi	r16, low(egrpre)
	ldi	r17, high(egrpre)	;EEPROM-Adresse Grafik-Presets
	lds	r18, grpres		;aktuelles Grafik-Preset holen
	ldi	r19, 7			;Adress-Offset 7 Bytes
	mul	r18, r19		;Adress-Offset ermitteln
	add	r16, r0
	adc	r17, r1			;EEPROM-Adresse ermitteln
	lds	r18, grse11		;Grafik 1 Sensor holen
	call	eewrit			;im EERPOM speichern
	lds	r18, grse12		;Grafik 1 Differenz-Sensor holen
	call	eewrit			;im EERPOM speichern
	lds	r18, grse21		;Grafik 2 Sensor holen
	call	eewrit			;im EERPOM speichern
	lds	r18, grse22		;Grafik 2 Differenz-Sensor holen
	call	eewrit			;im EERPOM speichern
	lds	r18, grmod1		;Grafik 1 Modus holen
	call	eewrit			;im EERPOM speichern
	lds	r18, grmod2		;Grafik 2 Modus holen
	call	eewrit			;im EERPOM speichern
	lds	r18, grflag		;Grafik-Flags holen
	call	eewrit			;im EERPOM speichern
;
m8b610:	call	eewcrc			;EERPOM-CRC berechnen/speichern
	call	okbeep			;Tonsignal ausgeben
	rcall	newcf2			;Datenfelder neu ausgeben
	rjmp	m99200
;
; Menprogramm: Konfiguration - Grundeinstellung, Grafik-Presets zurcksetzen
;
m8c000:	rcall	strwai			;Warte-Text ausgeben
	call	eedgrp			;Default-Grafik-Presets laden
	call	eewcrc			;EERPOM-CRC berechnen/speichern
	call	okbeep			;Tonsignal ausgeben
	rjmp	m99200
;
; Menprogramm: Seite wechseln
;
m8d000:	lds	r16, xflag4		;verschiedene Flags 4 holen
        ldi     r17, 0x10               ;Maske fr Bit 4 (Anzeige-Seite)
        eor     r16, r17                ;Bit fr Anzeigeseite umschalten
        sts     xflag4, r16             ;Flags wieder speichern
	jmp	m99000			;Temperaturanzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sonstiges 2
;
m8e000:	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x01		;Temperaturanzeige ausschalten
	sts	dflags, r16		;Flags wieder speichern
	ldi	r16, 32			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sonstiges 2 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	cpi	r16, 3			;Wert > 2?
	brcs	m8e010			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m8e010:	tst	r16			;Position = 0?
	brne	m8e020			;nein -> weiter
	ldi	r16, 1			;sonst Position = 1 setzen
m8e020:	sts	menpos, r16		;ausgewhlte Position speichern
	sts	encval, r16		;und als Impulsgeberwert benutzen
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 2
	sts	encmax, r16		;Impulsgeber-Maximum = 2 setzen
	rjmp	m99100			;Ende, Anzeige neu ausgeben
;
; Menprogramm: Konfiguration - Sonstiges 2, Auswhlen
;
m8e100:	ldi	r16, 0x5a		;Stringnummer "Auswhlen"
	sts	menext, r16		;Menerweiterung setzen
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brts	m8e050			;ja -> bearbeiten
	sbr	r16, 0x03		;Impulsgeberwert dezimal3 anzeigen
	sts	encflg, r16		;Impulsgeber-Flags wieder speichern
	ldi	r16, 1
	sts	encmin, r16		;Impulsgeber-Minimum = 1 setzen
	ldi	r16, 2
	sts	encmax, r16		;Impulsgeber-Maximum = 8 setzen
	ldi	r16, 32			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sonstiges 2 setzen
	rjmp	m99200
;
m8e050:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	rjmp	m99200
;
; Menprogramm: Konfiguration - Sonstiges 2, ndern
;
m8e200:	lds	r18, menpos		;ausgewhlte Position holen
	dec	r18			;korrigieren
	clr	r19
	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurations-Flags
	add	xl, r18			;Position berechnen
	adc	xh, r19			;bertrag
	ld	r16, x			;Konfigurations-Flags holen
	sbr	r16, 0x02		;Daten neu ausgeben
	st	x, r16			;Flags wieder speichern
	lds	r16, encflg		;Impulsgeber-Flags holen
	bst	r16, 0			;Impulsgeberwert als Men-Option?
	brtc	m8e210			;nein -> ndern vorbereiten
	rjmp	m8e300			;sonst neuen Wert speichern
;	
m8e210:	cpi	r18, 0			;Position auf DCF77-Signal invertiert?
	brne	m8e220			;nein -> weiter
	lds	r16, confg2		;Konfigurationsbyte 2 holen
	ldi	r17, 1 << 4		;Maske fr DCF77-Signal-Invertierung laden
	eor	r16, r17		;DCF77-Signal-Invertierung umschalten
        rjmp    m8e240                  ;Speichern und Signal ausgeben
;
m8e220:	lds	r16, confg2		;Konfigurationsbyte 2 holen
	ldi	r17, 1 << 5		;Maske fr DCF77-Eingangs-Pull-up laden
	eor	r16, r17		;DCF77-Eingangs-Pull-up umschalten
        bst     r16, 5                  ;Pull-up jetzt einschalten?
        brts    m8e230                  ;ja -> weiter
        cbi     porte, 2                ;sonst Pull-up ausschalten
        rjmp    m8e240
m8e230: sbi     porte, 2                ;Pull-up einschalten
;
m8e240:	sts	confg2, r16		;Konfigurationsbyte 2 speichern
	call	eewcon			;Konfig im EEPROM sichern
	rcall	okbeep			;Tonsignal ausgeben
	rjmp	m99100
;
m8e300:	clr	r16
	sts	encflg, r16		;Impulsgeber-Flags zurcksetzen
	sts	menext, r16		;Menerweiterung lschen
	ldi	r16, 32			;alternativen Anzeigemodus
	sts	dimode, r16		;auf Konfiguration - Sonstiges 2 setzen
	lds	r16, menpos		;ausgewhlte Position holen
	sts	encval, r16		;als Impulsgeberwert setzen
	rjmp	m99200
;
; Ende des Menprogramms, Anzeige wieder herstellen
;
m99000:	clr	r16			;alternativen Anzeigemodus
	sts	dimode, r16		;lschen
	lds	r16, dflags		;Anzeigeflags holen
	sbr	r16, 0x0f		;Temperaturanzeige neu ausgeben
	rjmp	m99210			;Flags wieder speichern
m99100:	lds	r16, dflags		;Anzeigeflags holen
	sbr	r16, 0x0e		;Anzeige komplett neu ausgeben
	rjmp	m99210			;Flags wieder speichern
m99200:	lds	r16, dflags		;Anzeigeflags holen
	sbr	r16, 0x0c		;Info/Menzeile neu ausgeben
m99210:	sts	dflags, r16		;Anzeigeflags wieder speichern
m99300:	ldi	r16, timmen		;Timeout-Wert fr Men-Rckkehr
	sts	mentio, r16		;neu setzen
m99400:	jmp	main
;
; Ende des Menprogramms, Rckkehr ins bergeordnete Men
;
m99500:	rcall	men1rd			;Mentabellenheader lesen
	elpm	r16, z+			;bergeordnete Mennummer holen
	elpm	r17, z			;bergeordnete Menoption holen
	andi	r17, 0x7f		;bergeordn. Menoption filtern
	tst	r17			;Menoption = 0?
	breq	m99510			;ja -> weiter
	dec	r17			;sonst korrigieren
m99510:	sts	mennum, r16		;neue Mennummer setzen
	sts	menopt, r17		;neue Menoption setzen
	rjmp	m99000			;Anzeige neu ausgeben
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Unterprogramme
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Unterprogramm 1 zum Auslesen der Daten aus den Mentabellen
; Register: r16, r17, r18, r19
;	    r11 -> Anzahl der Menoptionen
;	    r12 -> Menstring
;	    z   -> Zeiger auf Menkopfdaten Byte 2 (bergeordnetes Men)
;
men1rd:	ldi	zl, low(menidx * 2)
	ldi	zh, high(menidx * 2)
	ldi	r18, byte3(menidx * 2)	;Zeiger auf Menindex
	lds	r16, mennum		;aktuelle Mennummer holen
	add	r16, r16		;verdoppeln
	clr	r17			;H-Byte = 0
	add	zl, r16
	adc	zh, r17
	adc	r18, r17		;Men-Indexposition ermitteln
	out	rampz, r18
	elpm	r18, z+			;Adresse von Mentabelle
	elpm	r19, z			;holen
	movw	zl, r18			;Adresse kopieren
	clr	r18
	add	zl, zl
	adc	zh, zh
	adc	r18, r18		;Men-Adresse verdoppeln
	out	rampz, r18
	elpm	r11, z+			;Anzahl der Menoptionen holen
	elpm	r12, z+			;Menstring holen
	ret
;
; Unterprogramm 2 zum Auslesen der Daten aus den Mentabellen (muss immer direkt nach Unterprogramm men1rd aufgerufen werden)
; Register: wie men1rd, zustzlich:
;	    z   -> Zeiger auf aktuelle Optionsdaten
;
men2rd:	elpm	r16, z+
	elpm	r16, z+			;Zeiger auf Optionsdaten setzen
	lds	r16, menopt		;aktuelle Menoption holen
	add	r16, r16		;zweimal verdoppeln
	add	r16, r16		;Offset = 4 Bytes
	in	r18, rampz
	add	zl, r16
	adc	zh, r17			;Optionsdaten-Adresse ermitteln
	adc	r18, r17
	out	rampz, r18
	ret
;
; Unterprogramm zum Suchen einer Sensornummer in der Log-Tabelle 1 und Ermitteln der Adresse des Sensorwertes und der Flags, fr 15min-Werte
; Register: r20, r21
;	    r19 <- gesuchte Log-Position (wird am Ende um 1 erhht)
;	    x   -> Zeiger auf ermittelten Sensorwert (T01-T24)
;	    z   -> Zeiger auf ermitteltes Sensor-Flag (T01-T24)
;
slog1a:	ldi	xl, low(logtb1)
	ldi	xh, high(logtb1)	;Zeiger auf Log-Tabelle 1
	ldi	r20, 0			;Suchzhler fr Log-Position
slog10:	ld	r21, x+			;Tabelleneintrag holen
	cp	r19, r21		;Log-Position gefunden?
	breq	slog20			;ja -> Ende
	inc	r20			;Suchzhler erhhen
	cpi	r20, 24			;letzte Position erreicht?
	brcs	slog10			;nein -> Schleife
;
slog20:	clr	r21			;H-Byte fr Tabellenoffset
	ldi	zl, low(senflg)
	ldi	zh, high(senflg)	;Zeiger auf Sensor-Flags
	add	zl, r20			;Tabellenplatz berechnen
	adc	zh, r21			;bertrag
	ldi	xl, low(senval)
	ldi	xh, high(senval)	;Zeiger auf Sensorwert
	add	r20, r20		;Offset verdoppeln
	add	xl, r20			;Tabellenplatz berechnen
	adc	xh, r21			;bertrag
	inc	r19			;nchste Log-Position
	ret
;
; Unterprogramm zum Suchen einer Sensornummer in der Log-Tabelle 2 und Ermitteln der Adresse des Sensorwertes und der Flags, fr 15min-Werte
; Register: r20, r21
;	    r19 <- gesuchte Log-Position (wird am Ende um 1 erhht)
;	    x   -> Zeiger auf ermittelten Sensorwert (T25-T27)
;	    z   -> Zeiger auf ermitteltes Sensor-Flag (T25-T27)
;
slog2a:	ldi	xl, low(logtb2)
	ldi	xh, high(logtb2)	;Zeiger auf Log-Tabelle 2
	ldi	r20, 0			;Suchzhler fr Log-Position
slog30:	ld	r21, x+			;Tabelleneintrag holen
	cp	r19, r21		;Log-Position gefunden?
	breq	slog40			;ja -> Ende
	inc	r20			;Suchzhler erhhen
	cpi	r20, 3			;letzte Position erreicht?
	brcs	slog30			;nein -> Schleife
;
slog40:	clr	r21			;H-Byte fr Tabellenoffset
	ldi	zl, low(senflg + 24)
	ldi	zh, high(senflg + 24)	;Zeiger auf Sensor-Flags (Gruppe 2)
	add	zl, r20			;Tabellenplatz berechnen
	adc	zh, r21			;bertrag
	ldi	xl, low(senval + 48)
	ldi	xh, high(senval + 48)	;Zeiger auf Sensorwert (Gruppe 2)
	add	r20, r20		;Offset verdoppeln
	add	xl, r20			;Tabellenplatz berechnen
	adc	xh, r21			;bertrag
	inc	r19			;nchste Log-Position
	ret
;
; Unterprogramm zum Suchen einer Sensornummer in der Log-Tabelle 1 und Ermitteln des Adressoffset, fr Minimal- und Maximalwerte
; Register: x
;	    r19      <- gesuchte Log-Position (wird am Ende um 1 erhht)
;	    r20, r21 -> Adressoffset in der Log-Tabelle 1
;
slog1b:	ldi	xl, low(logtb1)
	ldi	xh, high(logtb1)	;Zeiger auf Log-Tabelle 1
	ldi	r20, 0			;Suchzhler fr Log-Position
slog50:	ld	r21, x+			;Tabelleneintrag holen
	cp	r19, r21		;Log-Position gefunden?
	breq	slog60			;ja -> Ende
	inc	r20			;Suchzhler erhhen
	cpi	r20, 24			;letzte Position erreicht?
	brcs	slog50			;nein -> Schleife
slog60:	clr	r21			;H-Byte fr Tabellenoffset
	add	r20, r20		;Offset verdoppeln
	inc	r19			;nchste Log-Position
	ret
;
; Unterprogramm zum Suchen einer Sensornummer in der Log-Tabelle 2 und Ermitteln des Adressoffset, fr Minimal- und Maximalwerte
; Register: x
;	    r19      <- gesuchte Log-Position (wird am Ende um 1 erhht)
;	    r20, r21 -> Adressoffset in der Log-Tabelle 2
;
slog2b:	ldi	xl, low(logtb2)
	ldi	xh, high(logtb2)	;Zeiger auf Log-Tabelle 2
	ldi	r20, 0			;Suchzhler fr Log-Position
slog70:	ld	r21, x+			;Tabelleneintrag holen
	cp	r19, r21		;Log-Position gefunden?
	breq	slog80			;ja -> Ende
	inc	r20			;Suchzhler erhhen
	cpi	r20, 3			;letzte Position erreicht?
	brcs	slog70			;nein -> Schleife
slog80:	clr	r21			;H-Byte fr Tabellenoffset
	add	r20, r20		;Offset verdoppeln
	inc	r19			;nchste Log-Position
	ret
;
; Unterprogramm zum Kopieren der Alarm-Anzeige-Positions-Tabelle in den Backup-Speicher
; Register: r16, r17, x, y
;
calmbk:	ldi	xl, low(almtab)		;Anzeige-Positions-Tabelle kopieren
	ldi	xh, high(almtab)	;Zeiger auf Posisions-Tabelle
	ldi	yl, low(almbak)
	ldi	yh, high(almbak)	;Zeiger auf Positions-Tabellen-Backup
	ldi	r17, 4			;4 Werte bearbeiten
calm10:	ld	r16, x+			;Wert aus Tabelle holen
	st	y+, r16			;und in Kopie speichern
	dec	r17			;alle Werte bearbeitet?
	brne	calm10			;nein -> Schleife
	ret
;
; Unterprogramm zum Kopieren des Alarm-Backup-Speichers in die Alarm-Anzeige-Positions-Tabelle
; Register: r16, r17, x, y
;
cbkalm:	ldi	xl, low(almtab)		;Anzeige-Positions-Tabelle aus Kopie
	ldi	xh, high(almtab)	;Zeiger auf Positions-Tabelle
	ldi	yl, low(almbak)
	ldi	yh, high(almbak)	;Zeiger auf Positions-Tabellen-Backup
	ldi	r17, 4			;4 Werte bearbeiten
cbka10:	ld	r16, y+			;Wert aus Kopie holen
	st	x+, r16			;und in Tabelle speichern
	dec	r17			;alle Werte bearbeitet?
	brne	cbka10			;nein -> Schleife
	ret
;
; Unterprogramm zum Suchen der Alarmnummer in der Alarm-Anzeige-Positions-Tabelle
; Register: r17, x
;	    r16 <- Anzeigeposition (1-4)
;	    r18 -> Alarmnummer (0-3)
;
salmap:	ldi	xl, low(almtab)
	ldi	xh, high(almtab)	;Zeiger auf Positions-Tabelle
	rjmp	sal010
;
; Unterprogramm zum Suchen der Alarmnummer in der Anzeige-Positions-Kopie
; Register: wie stemap
;
salbak:	ldi	xl, low(almbak)
	ldi	xh, high(almbak)	;Zeiger auf Positions-Tabellen-Kopie
sal010:	dec	r16			;Anzeigeposition korrigieren
	clr	r18			;Alarmnummer Anfangswert
sal020:	ld	r17, x+			;Anzeigeposition holen
	cp	r16, r17		;Anzeigeposition gefunden?
	brne	sal030			;nein -> weiter
	ret				;sonst Ende
sal030:	inc	r18			;Alarmnummer erhhen
	cpi	r18, 4			;Endwert erreicht?
	brcs	sal020			;nein -> Schleife
	clr	r18			;sonst Alarmnummer 0 setzen
	ret
;
; Unterprogramm zur Neuausgabe alle Anzeigepositionen (Flags setzen)
; Register: r16, r17, x
;
newpos: ldi     xl, low(postab)
        ldi     xh, high(postab)        ;Zeiger auf Positions-Tabelle
	ldi	r17, 32			;32 Anzeigepositionen neu ausgeben
newp10: ld      r16, x                  ;Sensornummer holen
        sbr     r16, 0xc0               ;nderungs-Flags setzen
        st      x+, r16                 ;Sensornummer wieder speichern
        dec     r17                     ;alle Anzeigepositionen bearbeitet?
        brne    newp10                  ;nein -> Schleife
;
        ldi     xl, low(bigflg)
        ldi     xh, high(bigflg)        ;Zeiger auf Groanzeige-Flags
	ldi	r17, 2			;2 Anzeigepositionen neu ausgeben
newp20: ld      r16, x                  ;Sensornummer holen
        sbr     r16, 0xc0               ;nderungs-Flags setzen
        st      x+, r16                 ;Sensornummer wieder speichern
        dec     r17                     ;alle Anzeigepositionen bearbeitet?
        brne    newp20                  ;nein -> Schleife
        ret
;
; Unterprogramm zur Neuausgabe aller Sensornamen und Sensorwerte (Flags setzen)
; Register: r16, r17, x
;
newsen:	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	r17, 32			;32 Flags bearbeiten
newt10:	ld	r16, x			;Sensor-Flag holen
	sbr	r16, 0x03		;Flag fr Neuausgabe setzen
	st	x+, r16			;Sensor-Flag wieder speichern
	dec	r17			;alle Flags bearbeitet?
	brne	newt10			;nein -> Schleife
	ret
;
; Unterprogramm zur Neuausgabe aller Sensorwerte (Flags setzen)
; Register: r16, r17, x
;
newval:	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	r17, 32			;32 Flags bearbeiten
newv10:	ld	r16, x			;Sensor-Flag holen
	sbr	r16, 0x01		;Flag fr Neuausgabe setzen
	st	x+, r16			;Sensor-Flag wieder speichern
	dec	r17			;alle Flags bearbeitet?
	brne	newv10			;nein -> Schleife
	ret
;
; Unterprogramm zur Neuausgabe aller Alarmwerte (Flags setzen)
; Register: r16, r17, x
;
newalm:	ldi	xl, low(almflg)
	ldi	xh, high(almflg)	;Zeiger auf Alarmflags
	ldi	r17, 4			;4 Flags bearbeiten
newa10:	ld	r16, x			;Alarmflag holen
	sbr	r16, 0x01		;Flag fr Neuausgabe setzen
	st	x+, r16			;Alarmflag wieder speichern
	dec	r17			;alle Flags bearbeitet?
	brne	newa10			;nein -> Schleife
	ret
;
; Unterprogramm zur kompletten Neuausgabe aller Konfigurations-Zeilen
; (Flags setzen)
; Register: r16, r17, r18, x
;
newcfg:	ldi	r18, 0x03		;Flag-Maske fr Namen und Daten
	rjmp	newc10
;
; Unterprogramm zur Neuausgabe der Datenfelder aller Konfigurationszeilen (Flags setzen)
; Register: r16, r17, r18, x
;
newcf2:	ldi	r18, 0x02		;Flag-Maske fr Datenfelder
newc10:	ldi	xl, low(cfgflg)
	ldi	xh, high(cfgflg)	;Zeiger auf Konfigurationsflags
	ldi	r17, 8			;8 Flags bearbeiten
newc20:	ld	r16, x			;Konfigurationsflag holen
	or	r16, r18		;Flags fr Neuausgabe setzen
	st	x+, r16			;Konfig-Flag wieder speichern
	dec	r17			;alle Flags bearbeitet?
	brne	newc20			;nein -> Schleife
	ret
;
; Unterprogramm zur Prfung der Log-Tabellen auf Konflikte (Konflikt-Flags setzen)
; Register: r16, r17, r18, r19, x
;
chklog: ldi     r17, 24                 ;24 Tabelleneintrge bearbeiten
        ldi     xl, low(logtb1)
        ldi     xh, high(logtb1)        ;Zeiger auf Log-Tabelle 1
chkl10: ld      r16, x                  ;Log-Position holen
        andi    r16, 0x1f               ;Konflikt-Flag lschen
        st      x+, r16                 ;Log-Position wieder speichern
        dec     r17                     ;alle Tabellen-Eintrge bearbeitet?
        brne    chkl10                  ;nein -> Schleife
;
        ldi     r19, 0                  ;alle Log-Positionen prfen, Startwert = 0
chkl20: ldi     r18, 0                  ;Eintrags-Zhler lschen
        ldi     r17, 24                 ;24 Tabelleneintrge prfen
        ldi     xl, low(logtb1)
        ldi     xh, high(logtb1)        ;Zeiger auf Log-Tabelle 1
;
chkl30: ld      r16, x+                 ;Log-Position holen
        andi    r16, 0x1f               ;Log-Position filtern
        cp      r16, r19                ;Log-Position gefunden?
        brne    chkl40                  ;nein -> berspringen
        inc     r18                     ;sonst Eintrags-Zhler erhhen
chkl40: dec     r17                     ;alle Tabelleneintrge geprft?
        brne    chkl30                  ;nein -> Schleife
;
        cpi     r18, 2                  ;mehr als 1 Eintrag gefunden?
        brcs    chkl70                  ;nein -> berspringen
;
        ldi     r17, 24                 ;sonst 24 Tabelleneintrge bearbeiten
        ldi     xl, low(logtb1)
        ldi     xh, high(logtb1)        ;Zeiger auf Log-Tabelle 1

chkl50: ld      r16, x                  ;Log-Position holen
        andi    r16, 0x1f               ;Log-Position filtern
        cp      r16, r19                ;Log-Position gefunden?
        brne    chkl60                  ;nein -> berspringen
        sbr     r16, 0x80               ;sonst Konflikt markieren
        st      x, r16                  ;Log-Position wieder speichern
chkl60: adiw    xl, 1                   ;nchster Tabellenplatz
        dec     r17                     ;alle Tabelleneintrge geprft?
        brne    chkl50                  ;nein -> Schleife
;
chkl70: inc     r19                     ;nchste Log-Position
        cpi     r19, 24                 ;alle Positionen bearbeitet?
        brcs    chkl20                  ;nein -> Schleife
;
; Log-Tabelle 2
;
        ldi     r17, 3                  ;3 Tabelleneintrge bearbeiten
        ldi     xl, low(logtb2)
        ldi     xh, high(logtb2)        ;Zeiger auf Log-Tabelle 2
chkl80: ld      r16, x                  ;Log-Position holen
        andi    r16, 0x03               ;Konflikt-Flag lschen
        st      x+, r16                 ;Log-Position wieder speichern
        dec     r17                     ;alle Tabellen-Eintrge bearbeitet?
        brne    chkl80                  ;nein -> Schleife
;
        ldi     r19, 0                  ;alle Log-Positionen prfen, Startwert = 0
chkl90: ldi     r18, 0                  ;Eintrags-Zhler lschen
        ldi     r17, 3                  ;3 Tabelleneintrge prfen
        ldi     xl, low(logtb2)
        ldi     xh, high(logtb2)        ;Zeiger auf Log-Tabelle 2
;
chkla0: ld      r16, x+                 ;Log-Position holen
        andi    r16, 0x03               ;Log-Position filtern
        cp      r16, r19                ;Log-Position gefunden?
        brne    chklb0                  ;nein -> berspringen
        inc     r18                     ;sonst Eintrags-Zhler erhhen
chklb0: dec     r17                     ;alle Tabelleneintrge geprft?
        brne    chkla0                  ;nein -> Schleife
;
        cpi     r18, 2                  ;mehr als 1 Eintrag gefunden?
        brcs    chkle0                  ;nein -> berspringen
;
        ldi     r17, 3                  ;sonst 3 Tabelleneintrge bearbeiten
        ldi     xl, low(logtb2)
        ldi     xh, high(logtb2)        ;Zeiger auf Log-Tabelle 2

chklc0: ld      r16, x                  ;Log-Position holen
        andi    r16, 0x03               ;Log-Position filtern
        cp      r16, r19                ;Log-Position gefunden?
        brne    chkld0                  ;nein -> berspringen
        sbr     r16, 0x80               ;sonst Konflikt markieren
        st      x, r16                  ;Log-Position wieder speichern
chkld0: adiw    xl, 1                   ;nchster Tabellenplatz
        dec     r17                     ;alle Tabelleneintrge geprft?
        brne    chklc0                  ;nein -> Schleife
;
chkle0: inc     r19                     ;nchste Log-Position
        cpi     r19, 3                  ;alle Positionen bearbeitet?
        brcs    chkl90                  ;nein -> Schleife
        ret
;
; Unterprogramm zur Prfung der Konflikt-Flags der Log-Tabellen
; Register: r16, r17, r18, x
;           r16 -> 0 = ok, 0x80 = Konflikt
;
chkcfl: clr     r16                     ;Flag-Speicher lschen
        ldi     r18, 24                 ;24 Tabelleneintrge bearbeiten
        ldi     xl, low(logtb1)
        ldi     xh, high(logtb1)        ;Zeiger auf Log-Tabelle 1
chkc10: ld      r17, x+                 ;Log-Position holen
        or      r16, r17                ;mit Flag-Speicher verknpfen
        dec     r18                     ;alle Tabellen-Eintrge bearbeitet?
        brne    chkc10                  ;nein -> Schleife
;
        ldi     r18, 3                  ;3 Tabelleneintrge bearbeiten
        ldi     xl, low(logtb2)
        ldi     xh, high(logtb2)        ;Zeiger auf Log-Tabelle 2
chkc20: ld      r17, x+                 ;Log-Position holen
        or      r16, r17                ;mit Flag-Speicher verknpfen
        dec     r18                     ;alle Tabellen-Eintrge bearbeitet?
        brne    chkc20                  ;nein -> Schleife
;
        andi    r16, 0x80               ;Flag filtern
        ret
;
; Unterprogramm zum Ermitteln der Seite, auf welcher ein Sensor angezeigt wird und gegebenenfalls Umschaltung auf diese Seite (Seitenwechsel bei Alarm)
; Register: r17, r18, x
;           r16 <- Sensornummer (0-31)
;
pagesw: lds     r17, dflags             ;Anzeige-Flags holen
        bst     r17, 0                  ;Normale Anzeige aktiv?
        brtc    page50                  ;nein -> berspringen
        lds     r17, xflags             ;Verschiedene Flags holen
        andi    r17, 0x0c               ;Anzeige der Min- oder Max-Werte?
        brne    page50                  ;ja -> berspringen
        lds     r17, errflg             ;Fehler-Flags holen
        bst     r17, 1                  ;Fehler "kein Datenempfang"?
        brts    page50                  ;ja -> berspringen
;
        ldi     xl, low(postab)
        ldi     xh, high(postab)        ;Zeiger auf Anzeige-Positions-Tabelle
        ldi     r18, 0                  ;Anzeige-Positions-Zhler = 0
;
page10: ld      r17, x+                 ;Sensornummer aus Tabelle holen
        andi    r17, 0x1f               ;Sensornummer filtern
        cp      r17, r16                ;Sensor gefunden?
        breq    page20                  ;ja -> weiter
        inc     r18                     ;Anzeige-Positions-Zhler erhhen
        cpi     r18, 32                 ;alle Positionen geprft?
        brcs    page10                  ;nein -> Schleife
        ret                             ;Sensor nicht gefunden        
;
page20: lds     r16, xflag4             ;Verschiedene Flags 4 holen
        lds     r17, dflags             ;Anzeige-Flags holen
        cpi     r18, 16                 ;Sensor auf Seite 2?
        brcc    page30                  ;ja -> bearbeiten
        bst     r16, 4                  ;Seite 1 bereits aktiv?
        brtc    page50                  ;ja -> berspringen
        cbr     r16, 1 << 4             ;sonst auf Seite 1 umschalten
        rjmp    page40                  ;Flags wieder speichern
;
page30: bst     r16, 4                  ;Seite 2 bereits aktiv?
        brts    page50                  ;ja -> berspringen
        sbr     r16, 1 << 4             ;sonst auf Seite 2 umschalten
page40: sts     xflag4, r16             ;Flags wieder speichern
	sbr	r17, 0x0f		;Normalanzeige neu ausgeben
	sts	dflags, r17		;Anzeigeflags wieder speichern
page50: ret
;
; Unterprogramm zur CRC8-Berechnung fr das interne EEPROM, basierend auf dem Algorithmus der 1-Wire Devices nach Application Note 27 von Dallas/Maxim
; Register: r14, r15, r16, r17, r18, r19
;	    r20 -> ermittelter CRC-Wert
;
calcrc:	ldi	r16, low(eeprom)
	ldi	r17, high(eeprom)	;Zeiger auf EEPROM-Beginn
	ldi	r19, 0x18		;Konstante fr Berechnung
	mov	r14, r19		;bereitlegen
	ser	r20			;CRC-Startwert = 0xff setzen
calc10:	call	eeread			;Byte aus EEPROM holen
	mov	r15, r18		;Byte zwischenspeichern
	ldi	r19, 8			;8 Verschiebungen
calc20:	eor	r18, r20		;CRC berechnen
	ror	r18			;Ergebnis in Carry schieben
	mov	r18, r20		;letzten CRC-Wert holen
	brcc	calc30			;war Ergebnis = 0? ja -> weiter
	eor	r18, r14		;sonst CRC-Wert updaten
calc30:	ror	r18			;neuen CRC-Wert positionieren
	mov	r20, r18		;und speichern
	mov	r18, r15		;restliche Bits holen
	bst	r18, 0			;LSB sichern, nchste Bitposition
	ror	r18			;gesichertes Bit als MSB ein-
	bld	r18, 7			;setzen (Ersatz fr RR-Befehl)
	mov	r15, r18		;und Wert zwischenspeichern
	dec	r19			;alle Verschiebungen erledigt?
	brne	calc20			;nein -> Schleife
	cpi	r16, low(eprcrc)	;End-Adresse L erreicht?
	brne	calc10			;nein -> Schleife
	cpi	r17, high(eprcrc)	;End-Adresse H erreicht?
	brne	calc10			;nein -> Schleife
	ret				;sonst Ende
;
; Unterprogramm zum Kopieren der aktuellen Zeit in einen temporren Zwischenspeicher zum Einstellen der Zeitdaten
; Register: r0, r1, r16, r17, r18, r19, r20
;
cpytim:	cli				;Interrupts sperren
	lds	r16, minutl
	lds	r17, minuth
	lds	r18, hourl
	lds	r19, hourh		;aktuelle Zeit holen
	sei				;Interrupts wieder freigeben
	ldi	r20, 10
	mul	r20, r19		;Stunden Zehner x10
	add	r18, r0			;Einer addieren
	ldi	r20, 60
	mul	r20, r18		;Gesamtstunden x60
	movw	r18, r0			;Ergebnis kopieren
	ldi	r20, 10
	mul	r20, r17		;Minuten Einer x10
	add	r18, r0
	adc	r19, r1			;zum Gesamtwert addieren
	clr	r17
	add	r18, r16
	adc	r19, r17		;Minuten Einer addieren
	sts	cptiml, r18
	sts	cptimh, r19		;Minutenwert speichern
;
	cli				;Interrupts sperren
	lds	r16, dayl
	lds	r17, dayh		;aktuellen Kalendertag holen
	sei				;Interrupts wieder freigeben
	ldi	r20, 10
	mul	r20, r17		;Kalendertag Zehner x10
	add	r0, r16			;Einer addieren
	sts	cpday, r0		;Kalendertag speichern
;
	cli				;Interrupts sperren
	lds	r16, monthl
	lds	r17, monthh		;aktuellen Monat holen
	sei				;Interrupts wieder freigeben
	ldi	r20, 10
	mul	r20, r17		;Monat Zehner x10
	add	r0, r16			;Einer addieren
	sts	cpmon, r0		;Monat speichern
;
	cli				;Interrupts sperren
	lds	r16, yearl
	lds	r17, yearh		;aktuelles Jahr holen
	sei				;Interrupts wieder freigeben
	ldi	r20, 10
	mul	r20, r17		;Monat Zehner x10
	add	r0, r16			;Einer addieren
	sts	cpyear, r0		;Monat speichern
;
	lds	r16, dayw		;aktuellen Wochentag holen
	sts	cpdayw, r16		;Wochentag speichern
	ret
;
; Unterprogramm zum bernehmen der Zeit aus dem temporren Zwischenspeicher
; Register: r16, r17, r18, r19, r20
;
cftime:	lds	r16, cptiml		;Zeit aus Zwischenspeicher
	lds	r17, cptimh		;holen
	ldi	r18, 60
	clr	r19			;Subtrahend 60 laden
	clr	r20			;Stundenzhler lschen
cft010:	sub	r16, r18		;60 subtrahieren
	sbc	r17, r19		;zuviel subtrahiert?
	brmi	cft020			;ja -> weiter
	inc	r20			;sonst Stundenzhler erhhen
	rjmp	cft010			;Schleife
cft020:	add	r16, r18		;Wert wieder herstellen
	adc	r17, r19		;bertrag
;
cft030:	cpi	r16, 10			;Minuten > 9?
	brcs	cft040			;ja -> weiter
	subi	r16, 10			;sonst 10 subtrahieren
	inc	r17			;Minutenzehnerzhler erhhen
	rjmp	cft030			;Schleife
;
cft040:	cli				;Interrupts sperren
	clr	r18
	sts	secndl, r18		;Sekunden Einer lschen
	sts	secndh, r18		;Sekunden Zehner lschen
	sts	t1cnt2, r18		;Timer1 Interruptzhler lschen
	sts	minutl, r16		;Minuten Einer speichern
	sts	minuth, r17		;Minuten Zehner speichern
	sei				;Interrupts wieder freigeben
;
	clr	r17			;Stundenzehnerzhler lschen
cft050:	cpi	r20, 10			;Stunden < 10?
	brcs	cft060			;ja -> weiter
	subi	r20, 10			;sonst 10 subtrahieren
	inc	r17			;Stundenzehnerzhler erhhen
	rjmp	cft050			;Schleife
cft060:	sts	hourl, r20		;Stunden Einer speichern
	sts	hourh, r17		;Stunden Zehner speichern
;
	lds	r16, cpday		;Kalendertag holen
	clr	r17			;Tagzehnerzhler lschen
cft070:	cpi	r16, 10			;Kalendertag < 10?
	brcs	cft080			;ja -> weiter
	subi	r16, 10			;sonst 10 subtrahieren
	inc	r17			;Tagzehnerzhler erhhen
	rjmp	cft070			;Schleife
cft080:	sts	dayl, r16		;Kalendertag Einer speichern
	sts	dayh, r17		;Kalendertag Zehner speichern
;
	lds	r16,cpmon		;Monat holen
	clr	r17			;Monatzehnerzhler lschen
	cpi	r16, 10			;Monat < 10?
	brcs	cft090			;ja -> weiter
	subi	r16, 10			;sonst 10 subtrahieren
	inc	r17			;Monatzehnerzhler erhhen
cft090:	sts	monthl, r16		;Monat Einer speichern
	sts	monthh, r17		;Monat Zehner speichern
;
	lds	r16, cpyear		;Jahr holen
	clr	r17			;Jahrzehnerzhler lschen
cft100:	cpi	r16, 10			;Jahr < 10?
	brcs	cft110			;ja -> weiter
	subi	r16, 10			;sonst 10 subtrahieren
	inc	r17			;Tagzehnerzhler erhhen
	rjmp	cft100			;Schleife
cft110:	sts	yearl, r16		;Jahr Einer speichern
	sts	yearh, r17		;Jahr Zehner speichern
;
	lds	r16, cpdayw		;Wochentag holen
	sts	dayw, r16		;Wochentag speichern
	ret
;
; Unterprogramm zum Erhhen des DCF77-Pufferinhaltes um eine Minute, es werden nur Minuten und Stunden bercksichtigt
; Register: r22, r23
;           z <- Zeiger auf DCF77-Puffer
;
incdcb:	ldd	r23, z + 1		;Minuten Einer holen
	inc	r23			;erhhen
	std	z + 1, r23		;und wieder speichern
	cpi	r23, 10			;bertrag?
	brcs	incd20			;nein -> weiter
	clr	r23			;sonst zurcksetzen
	std	z + 1, r23		;und speichern
	ldd	r23, z + 2		;Minuten Zehner holen
	inc	r23			;erhhen
	std	z + 2, r23		;und wieder speichern
	cpi	r23, 6			;bertrag?
	brcs	incd20			;nein -> weiter
	clr	r23			;sonst zurcksetzen
	std	z + 2, r23		;und speichern
	ldd	r22, z + 4		;Stunden Zehner holen
	ldd	r23, z + 3		;Stunden Einer holen
	inc	r23			;erhhen
	std	z + 3, r23		;und wieder speichern
	cpi	r23, 10			;bertrag?
	brcs	incd10			;nein -> Test auf "24"
	clr	r23			;sonst zurcksetzen
	std	z + 3, r23		;und speichern
	inc	r22			;Stunden Zehner erhhen
	std	z + 4, r22		;und speichern
incd10:	cpi	r22, 2			;Stunden Zehner = 2?
	brne	incd20			;nein -> weiter
	cpi	r23, 4			;Stunden Einer = 4 (= 24) ?
	brne	incd20			;nein -> weiter
	clr	r23			;sonst Stunden zurcksetzen
	std	z + 3, r23		;Stunden Einer speichern
	std	z + 4, r23		;Stunden Zehner speichern
incd20:	ret
;
; Unterprogramm zum Erzeugen eines Tonsignals als Tastenquittung
; Register: r18, r19
;
okbeep:	ldi	r18, 15			;Signalnummer fr Tasten-Beep
	ldi	r19, 1			;Signaldauer fr Tasten-Beep
;
; Unterprogramm: neues Signal-Ereignis in die Warteschlange einreihen
; Register: r18 <- Signalnummer
;	    r19 <- Signaldauer
;
signal:	push	xl
	push	xh
	push	yl
	push	yh			;Zeigerregister sichern
	lds	xl, sigbwr
	lds	xh, sigbwr + 1		;Schreibzeiger holen
	st	x+, r18			;Signalnummer speichern
	st	x+, r19			        ;Signaldauer speichern
	ldi	yl, low(sigbuf + (sigbul * 2))
	ldi	yh, high(sigbuf + (sigbul * 2)) ;Adresse Pufferende laden
	sub	yl, xl
	sbc	yh, xh			;Differenz berechnen
	or	yh, yl			;Endadresse erreicht?
	brne	sig010			;nein -> weiter
	ldi	xl, low(sigbuf)
	ldi	xh, high(sigbuf)	;sonst Adresse auf Anfangswert
sig010:	sts	sigbwr, xl
	sts	sigbwr + 1, xh		;Schreibzeiger speichern
	pop	yh
	pop	yl
	pop	xh
	pop	xl			;Zeigerregister wieder herst.
	ret
;
; Unterprogramm zum Umrechnen des Lautstrkewertes in den PWM-Wert fr Timer1b und Einstellen des PWM-Wertes
; Register: r16
;	    r17 <- Lautstrkewert (0-15)
;
sicalc:	lds	r16, xflag2		;verschiedene Flags 2 holen
	cbr	r16, 0x01		;Lautsprecher-Flag lschen
	tst	r17			;Lautstrke-Wert=0?
	breq	sica10			;ja -> weiter, sonst
	sbr	r16, 0x01		;Lautsprecher-Flag setzen
	dec	r17			;Wertebereich auf 0-14 setzen
sica10:	sts	xflag2, r16		;Flags wieder speichern
	add	r17, r17		;Wert verdoppeln, Bereich 0-28
	ldi	r16, 247		;Multiplikator 247 setzen
	push	r0
	push	r1			;Register sichern
	mul	r16, r17		;Ergebnis im Bereich 0-6916
	out	ocr1bh, r1		;PWM-Wert H setzen
	out	ocr1bl, r0		;PWM-Wert L setzen
	pop	r1
	pop	r0			;Register wiederherstellen
	ret
;
; Unterprogramm zum Umrechnen des Timerwertes (Timer0) in die Frequenz, Formel: f = (Clock/32) / (1+ocr0), Ergebnis hat den doppelten Wert, LSB wird zum Runden
; benutzt
; Register: r14, r15, r18, r19, r20, r21
;	    r16, r17 -> Frequenz (2 Byte)
;
timfrq:	ldi	r16, low(clock / 32)
	ldi	r17, high(clock / 32)	;Divident laden
	ldi	r18, byte3(clock / 32)	;Taktfrequenz / 32
	in	r20, ocr0		;Timer0-Wert holen
	clr	r21			;H-Byte lschen
	inc	r20			;um 1 erhhen
	brne	timf10			;bertrag? nein -> weiter
	inc	r21			;sonst bertrag auf H-Byte
timf10:	rcall	divide			;Division 24 Bit / 16 Bit
	asr	r17			;Ergebnis H halbieren
	ror	r16			;Ergebnis L halbieren
	brcc	timf20			;Runden? nein -> Ende
	inc	r16			;sonst aufrunden
	brne	timf20			;bertrag? nein -> weiter
	inc	r17			;sonst bertrag auf H-Byte
timf20:	ret
;
; Unterprogramm zum Umrechnen der Frequenz in den Timerwert (Timer0), Formel: ocr0 = ((Clock/32) / f) - 1, Ergebnis hat den doppelten Wert, LSB wird zum Runden
; benutzt
; Register: r14, r15, r16, r17, r18, r19
;	    r20, r21 <- Frequenz (2 Byte)
;
frqtim:	ldi	r16, low(clock / 32)
	ldi	r17, high(clock / 32)
	ldi	r18, byte3(clock / 32)
	rcall	divide			;Division 24 Bit / 16 Bit
	asr	r17			;Ergebnis H halbieren
	ror	r16			;Ergebnis L halbieren
	brcc	frqt10			;Runden? nein -> weiter
	inc	r16			;sonst aufrunden
	brne	frqt10			;bertrag? nein -> weiter
	inc	r17			;sonst bertrag auf H-Byte
frqt10:	subi	r16, 1			;um 1 vermindern
	sbci	r17, 0			;bertrag
	tst	r17			;H-Byte = 0?
	breq	frqt20			;ja -> weiter
	ldi	r16, 0xff		;sonst auf Maxwert 255 setzen
frqt20:	cpi	r16, 28			;Timerwert >= 28?
	brcc	frqt30			;ja -> weiter
	ldi	r16, 28			;sonst auf Minwert 28 setzen
frqt30:	out	ocr0, r16		;Timer0 setzen
	ret
;
; Berechnung der Batteriespannung nach folgender Formel: U = Batteriewert * 1,23 / Referenzwert, zur Reduzierung von Schwankungen werden die letzten 4
; Messwerte (Batterie und Referenz) addiert
; Register: r14, r15, r18, r19, r20, r21, r22, r23
;           r16, r17 -> Batteriespannung in 1/100 Volt (z.B. 308 = 3,08V)
;
batclc:	lds	r16, adcbat
	lds	r17, adcbat + 1		;Batterie-Wert 0 holen
	lds	r18, adcbat + 2
	lds	r19, adcbat + 3		;Batterie-Wert 1 holen
	add	r16, r18
	adc	r17, r19		;addieren
	lds	r18, adcbat + 4
	lds	r19, adcbat + 5		;Batterie-Wert 2 holen
	add	r16, r18
	adc	r17, r19		;addieren
	lds	r18, adcbat + 6
	lds	r19, adcbat + 7		;Batterie-Wert 3 holen
	add	r16, r18		;addieren
	adc	r17, r19		;Ergebnis = 4 * Batterie-Wert
;
	ldi	r18, low(123)
	ldi	r19, high(123)		;Referenzspannung (1,23V) laden
	rcall	multip			;mit 4 * Batterie-Wert multiplizieren
	movw	r16, r18
	mov	r18, r20		;Ergebnis kopieren
;
	lds	r20, adcref
	lds	r21, adcref + 1		;Referenz-Wert 0 holen
	lds	r22, adcref + 2
	lds	r23, adcref + 3		;Referenz-Wert 1 holen
	add	r20, r22
	adc	r21, r23		;addieren
	lds	r22, adcref + 4
	lds	r23, adcref + 5		;Referenz-Wert 2 holen
	add	r20, r22
	adc	r21, r23		;addieren
	lds	r22, adcref + 6
	lds	r23, adcref + 7		;Referenz-Wert 3 holen
	add	r20, r22		;addieren
	adc	r21, r23		;Ergebnis =  4 * Referenz-Wert
	rcall	divide			;Ergebnis = Spannung in 1/100V
	ret
;
; Unterprogramm zur Multiplikation 16 Bit * 16 Bit ohne Vorzeichen (nach Application Note AVR200)
; Register: r22
;	    r16, r17  <- Faktor1 (2 Byte)
;	    r18, r19  <- Faktor2 (2 Byte)
;	    r18 - r21 -> Ergebnis (4 Byte)
;
multip:	clr	r20
	clr	r21			;obere Ergebnis-Bytes lschen
	ldi	r22, 16			;16 Schleifen
	lsr	r19
	ror	r18			;Faktor2 LSB in Carry schieben
mult10:	brcc	mult20			;LSB gesetzt? nein -> weiter
	add	r20, r16		;Faktor1L + Ergebnis-Byte 2
	adc	r21, r17		;Faktor1H + Ergebnis-Byte 3
mult20:	ror	r21			;Ergebnis-Byte 3 verschieben
	ror	r20			;Ergebnis-Byte 2 verschieben
	ror	r19			;Ergebnis-Byte 1 + Faktor2H
	ror	r18			;Ergebnis-Byte 0 + Faktor2L
	dec	r22			;alle Schleifen ausgefhrt?
	brne	mult10			;nein -> Schleife
	ret
;
; Unterprogramm zur Division 24 Bit / 16 Bit ohne Vorzeichen (nach Application Note AVR200, etwas modifiziert)
; Register: r19
;	    r16 - r18 <- Dividend (3 Byte)
;	    r20, r21  <- Divisor (2 Byte)
;	    r16, r17  -> Ergebnis (2 Byte)
;	    r14, r15  -> Rest (2 Byte)
;
divide:	clr	r14			;Rest L lschen
	sub	r15, r15		;Rest H und Carry lschen
	ldi	r19, 25			;Schleifenzhler setzen
divu10:	rol	r16
	rol	r17			;Dividend nach links verschieben
	rol	r18			;hchstwertiges Bit in Carry
	dec	r19			;alle Schleifen durchlaufen?
	brne	divu20			;nein -> weiter
	ret				;sonst Ende
divu20:	rol	r14			;Dividend in Rest schieben
	rol	r15
	sub	r14, r20		;Rest = Rest - Divisor
	sbc	r15, r21
	brcc	divu30			;wenn Ergebnis negativ ->
	add	r14, r20		;Rest wieder herstellen
	adc	r15, r21
	clc				;Carry lschen und ins Ergebnis
	rjmp	divu10			;einschieben, oder
divu30:	sec				;Carry setzen und ins Ergebnis
	rjmp	divu10			;einschieben
;
; Unterprogramm zum Abschalten des LCD (fr fehlerfreien DCF77-Empfang)
; Register: r16
;
lcdoff:	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x80		;Flag fr Abschaltung setzen
	sts	dflags, r16		;und Flags wieder speichern
	clr	r16
	out	porta, r16		;PortA Pull-Ups abschalten
	in	r16, portc		;PortC Status holen
	cbr	r16, 0x0f		;alle LCD-Steuerleitungen auf
	out	portc, r16		;Low setzen
	sbi	portb, 0		;LCD-Spannung abschalten
	ret
;
; Unterprogramm zum Einschalten und Initialisieren des LCD
; Register: r16
;
lcdon:	cbi	portb, 0		;LCD-Spannung einschalten
	in	r16, portc		;PortC Status holen
	sbr	r16, 0x0f		;alle LCD-Steuerleitungen auf
	out	portc, r16		;High setzen
	ldi	r16, 0xff
	out	porta, r16		;PortA Pull-Ups einschalten
	lds	r16, dflags		;Anzeige-Flags holen
	cbr	r16, 0x80		;Abschaltungs-Flag lschen
	sbr	r16, 0x0f		;Flags fr Neuausgabe der
	sts	dflags, r16		;Anzeige setzen, speichern
;
; LCD im Grafikmode initialisieren und lschen
;
	call	wait50			;50ms auf LCD warten
	clr	r17			;LCD Grafik-Adresse = 0
	clt				;Data senden
	rcall	outlcd			;ausgeben Low Address
	rcall	outlcd			;ausgeben High Address
	ldi	r17, 0x42		;Graphic Home Address Set
	set				;Command senden
	rcall	outlcd			;ausgeben
	ldi	r17, 30			;LCD Area = 0
	clt				;Data senden
	rcall	outlcd			;ausgeben
	clr	r17			;Datenbyte2 = 0
	rcall	outlcd			;ausgeben
	ldi	r17, 0x43		;Graphic Area Set
	set				;Command senden
	rcall	outlcd			;ausgeben
	rcall	clrlcd			;alle Pixel lschen
	ldi	r17, 0x98		;LCD in Grafikmode
	set				;Command senden
	rcall	outlcd			;ausgeben
	ldi	r17, 0x80		;Text/Grafik in OR-Mode
	set				;Command senden
	rcall	outlcd			;ausgeben
	ret
;
; Unterprogramm zum Rcksetzen der Minimalwerte (fr Anzeige) durch Kopieren der aktuellen Sensorwerte
; Register: r16, r17, r18, r19, r20, x, y, z
;
rsmin1:	lds	r19, xflags		;verschiedene Flags holen
	ldi	r20, 31			;31 Minimalwerte bearbeiten
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	ldi	zl, low(semin1)
	ldi	zh, high(semin1)	;Zeiger auf Minimalwerte (Anzeige)
rsmi10:	ld	r18, x			;Sensor-Flags holen
	bst	r18, 7			;Sensor inaktiv?
	brtc	rsmi20			;nein -> weiter
	clr	r16
	ldi	r17, 0x80		;sonst Wert lschen (0x8000)
	rjmp	rsmi40			;Wert speichern
rsmi20:	bst	r19, 2			;Minimalanzeige gerade aktiv?
	brtc	rsmi30			;nein -> berspringen
	sbr	r18, 0x01		;sonst Sensorwert neu ausgeben
	st	x, r18			;Flags wieder speichern
rsmi30:	ld	r16, y			;Sensorwert L holen
	ldd	r17, y + 1		;Sensorwert H holen
rsmi40:	st	z, r16			;als Minimalwert L speichern
	std	z + 1, r17		;als Minimalwert H speichern
	adiw	zl, 2			;nchster Minimalwert
	adiw	yl, 2			;nchster Sensorwert
	adiw	xl, 1			;nchstes Sensor-Flag
	dec	r20			;alle Werte bearbeitet?
	brne	rsmi10			;nein -> Schleife
	ret
;
; Unterprogramm zum Rcksetzen der Maximalwerte (fr Anzeige) durch Kopieren der aktuellen Sensorwerte
; Register: r16, r17, r18, r19, r20, x, y, z
;
rsmax1:	lds	r19, xflags		;verschiedene Flags holen
	ldi	r20, 31			;31 Maximalwerte bearbeiten
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	ldi	zl, low(semax1)
	ldi	zh, high(semax1)	;Zeiger auf Maximalwerte (Anzeige)
rsma10:	ld	r18, x			;Sensor-Flags holen
	bst	r18, 7			;Sensor inaktiv?
	brtc	rsma20			;nein -> weiter
	clr	r16
	ldi	r17, 0x80		;sonst Wert lschen (0x8000)
	rjmp	rsma40			;Wert speichern
rsma20:	bst	r19, 3			;Maximalanzeige gerade aktiv?
	brtc	rsma30			;nein -> berspringen
	sbr	r18, 0x01		;sonst Sensorwert neu ausgeben
	st	x, r18			;Flags wieder speichern
rsma30:	ld	r16, y			;Sensorwert L holen
	ldd	r17, y + 1		;Sensorwert H holen
rsma40:	st	z, r16			;als Maximalwert L speichern
	std	z + 1, r17		;als Maximalwert H speichern
	adiw	zl, 2			;nchster Maximalwert
	adiw	yl, 2			;nchster Sensorwert
	adiw	xl, 1			;nchstes Sensor-Flag
	dec	r20			;alle Werte bearbeitet?
	brne	rsma10			;nein -> Schleife
	ret
;
; Unterprogramm zum Rcksetzen der Minimalwerte (fr EEPROM-Log) durch Kopieren der aktuellen Sensorwerte
; Register: r16, r17, r18, r20, x, y, z
;
rsmin2:	ldi	r20, 31			;31 Minimalwerte bearbeiten
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	ldi	zl, low(semin2)
	ldi	zh, high(semin2)	;Zeiger auf Minimalwerte (Log)
rsmi50:	ld	r18, x			;Sensor-Flags holen
	bst	r18, 7			;Sensor inaktiv?
	brtc	rsmi60			;nein -> weiter
	clr	r16
	ldi	r17, 0x80		;sonst Wert lschen (0x8000)
	rjmp	rsmi70			;Wert speichern
rsmi60:	ld	r16, y			;Sensorwert L holen
	ldd	r17, y + 1		;Sensorwert H holen
rsmi70:	st	z, r16			;als Minimalwert L speichern
	std	z + 1, r17		;als Minimalwert H speichern
	adiw	zl, 2			;nchster Minimalwert
	adiw	yl, 2			;nchster Sensorwert
	adiw	xl, 1			;nchstes Sensor-Flag
	dec	r20			;alle Werte bearbeitet?
	brne	rsmi50			;nein -> Schleife
	ret
;
; Unterprogramm zum Rcksetzen der Maximalwerte (fr EEPROM-Log) durch Kopieren der aktuellen Sensorwerte
; Register: r16, r17, r18, r20, x, y, z
;
rsmax2:	ldi	r20, 31			;31 Maximalwerte bearbeiten
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf Sensor-Flags
	ldi	yl, low(senval)
	ldi	yh, high(senval)	;Zeiger auf Sensorwerte
	ldi	zl, low(semax2)
	ldi	zh, high(semax2)	;Zeiger auf Maximalwerte (Log)
rsma50:	ld	r18, x			;Sensor-Flags holen
	bst	r18, 7			;Sensor inaktiv?
	brtc	rsma60			;nein -> weiter
	clr	r16
	ldi	r17, 0x80		;sonst Wert lschen (0x8000)
	rjmp	rsma70			;Wert speichern
rsma60:	ld	r16, y			;Sensorwert L holen
	ldd	r17, y + 1		;Sensorwert H holen
rsma70:	st	z, r16			;als Maximalwert L speichern
	std	z + 1, r17		;als Maximalwert H speichern
	adiw	zl, 2			;nchster Maximalwert
	adiw	yl, 2			;nchster Sensorwert
	adiw	xl, 1			;nchstes Sensor-Flag
	dec	r20			;alle Werte bearbeitet?
	brne	rsma50			;nein -> Schleife
	ret
;
; Unterprogramm zum Rcksetzen der Mittelwerte (Summe und Zhler auf 0 setzen)
; Register: r16, r17, y
;
resavg:	ldi	r17, 31 * 3		;31 Summenwerte mit je 24bit
	ldi	yl, low(senavs)		;bearbeiten
	ldi	yh, high(senavs)	;Zeiger auf Summenwerte
	clr	r16
resa10:	st	y+, r16			;Byte lschen
	dec	r17			;alle Werte bearbeitet?
	brne	resa10			;nein -> Schleife
;
	ldi	r17, 31 * 2		;31 Zhlerwerte mit je 16 Bit
	ldi	yl, low(senavc)		;bearbeiten
	ldi	yh, high(senavc)	;Zeiger auf Zhlerwerte
resa20:	st	y+, r16			;Byte lschen
	dec	r17			;alle Werte bearbeitet?
	brne	resa20			;nein -> Schleife
	ret
;
; Unterprogramm zum Lschen des Parameterspeichers (PC-Schnittstelle)
; Register: r16, r17
;
clpara:	push	xl
	push	xh			;Register sichern
	ldi	xl, low(pcpara)
	ldi	xh, high(pcpara)	;Zeiger auf Parameterspeicher
	ldi	r17, 9			;9 Zeichen lschen
	clr	r16
clpa10:	st	x+, r16			;Parameterzeichen lschen
	dec	r17			;alle Zeichen bearbeitet?
	brne	clpa10			;nein -> Schleife
	pop	xh
	pop	xl			;Register wieder herstellen
	ret
;
; Unterprogramm zum Komprimieren einer Meldung, Ablegen im EEPROM Schreibpuffer und Schreiben in das externe EEPROM
; Register: r0, r1, r15, r16, r17, r18, r21, r22, r23, x, y
;	    r19 <- Meldungscode
;	    r20 <- Parameter, Sensor- oder Alarmnummer
;
wrimsg:	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 3			;Speicherfehler?
	brts	wrim30			;ja -> Ende
	bst	r16, 0			;Uhr schon eingestellt?
	brts	wrim30			;nein -> Ende
	rcall	cpstim			;sonst Zeitdaten komprimieren
	lds	r17, secndh		;Sekunden Zehner holen
	mul	r17, r16		;verzehnfachen (r16 = 10)
	lds	r16, secndl		;Sekunden Einer holen
	add	r16, r0			;Zehner addieren
	sts	xewbuf + 4, r16		;Sekunden speichern
	sts	xewbuf + 5, r19		;Meldungscode speichern
        sts     xewbuf + 6, r20         ;Parameter speichern
;
	lds	r20, xemspo
	lds	r21, xemspo + 1		;Zeiger auf aktuelle Position
	lds	r22, xemspo + 2		;im externen EEPROM holen
	ldi	r23, xemsda		;7 Bytes zum Ext-EEPROM senden
	call	xwrite			;Puffer zum Ext-EEPROM senden
	brcc	wrim10			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	rjmp	wrim30			;Adressberechnung berspringen
;
wrim10:	ldi	r16, low(xmssta + xmslen - xemsda)
	ldi	r17, high(xmssta + xmslen - xemsda)     ;Adresse des letzten mglichen
	ldi	r18, byte3(xmssta + xmslen - xemsda)    ;Datensatzbeginns laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;aktuelle Adresse noch im gltigen Speicherbereich?
	brcc	wrim20			;ja -> weiter
	ldi	r20, low(xmssta)
	ldi	r21, high(xmssta)	;sonst
	ldi	r22, byte3(xmssta)	;Adresse wieder auf Anfang setzen
wrim20:	sts	xemspo, r20
	sts	xemspo + 1, r21		;Zeiger fr Position im
	sts	xemspo + 2, r22		;externen EEPROM speichern
;
	lds	r16, errflg		;Fehler-Flags holen
	bst	r16, 4			;RTC-Fehler?
	brts	wrim30			;ja -> Ende
	ldi	r18, rtxmsp		;RTC-Adresse Meldungs-Daten
	ldi	xl, low(xemspo)
	ldi	xh, high(xemspo)	;Zeiger auf Meldungsdaten-Pointer
	call	i2psen			;Pointer in RTC schreiben
	brcc	wrim30			;Fehler? nein -> weiter
	call	i2cerr			;sonst RTC Fehlerstatus setzen
wrim30:	ret
;
; Unterprogramm zum Komprimieren der Zeitdaten und Ablegen im EEPROM Schreibpuffer
; Register: r0, r1, r16, r17, r18
;
cpstim:	ldi	r16, 10			;Zehner-Multiplikator laden
	lds	r17, dayh		;Tag Zehner holen
	mul	r17, r16		;verzehnfachen
	lds	r17, dayl		;Tag Einer holen
	add	r17, r0			;Zehner addieren
	mov	r18, r17		;Ergebnis zwischenspeichern
	lds	r17, monthh		;Monat Zehner holen
	mul	r17, r16		;verzehnfachen
	lds	r17, monthl		;Monat Einer holen
	add	r17, r0			;Zehner addieren
	swap	r17			;Nibbles tauschen
	rol	r17			;Wert verschieben, MSB in Carry
	andi	r17, 0xe0		;relevante Bits filtern und
	or	r18, r17		;mit Tag verknpfen
	sts	xewbuf + 2, r18		;kompakte Daten speichern
	rol	r18			;MSB vom Monat aus Carry holen
	andi	r18, 0x01		;und filtern
	lds	r17, yearh		;Jahr Zehner holen
	mul	r17, r16		;verzehnfachen
	lds	r17, yearl		;Jahr Einer holen
	add	r17, r0			;Zehner addieren
	rol	r17			;Wert verschieben
	andi	r17, 0xfe		;relevante filtern und
	or	r18, r17		;mit MSB vom Monat verknpfen
	sts	xewbuf + 3, r18		;kompakte Daten speichern
;
	lds	r17, hourh		;Stunden Zehner holen
	mul	r17, r16		;verzehnfachen
	lds	r17, hourl		;Stunden Einer holen
	add	r17, r0			;Zehner addieren
	sts	xewbuf + 1, r17		;kompakten Stundenwert speichern
	lds	r17, minuth		;Minuten Zehner holen
	mul	r17, r16		;verzehnfachen
	lds	r17, minutl		;Minuten Einer holen
	add	r17, r0			;Zehner addieren
	sts	xewbuf, r17		;kompakten Minutenwert speichern
	ret
;
; Unterprogramm zum Extrahieren der Zeitdaten aus einem EEPROM-Datensatz in den ASCII-Puffer fr die Datenausgabe zum PC
; Register: r16, r17, r18, x
;
xtrtim:	ldi	xl, low(ascbuf)
	ldi	xh, high(ascbuf)	;Zeiger auf ASCII-Puffer
	ldi	r16, year3		;Jahr Tausender holen
	st	x+, r16			;in Puffer speichern
	ldi	r16, year2		;Jahr Hunderter holen
	st	x+, r16			;in Puffer speichern
	lds	r18, xerbuf + 3		;komprimierte Zeit Byte 3 holen
	bst	r18, 0			;LSB sichern, gehrt zum Monat
	lsr	r18			;Jahr an Position schieben
	rcall	bytasc			;in ASCII-Wert wandeln
	st	x+, r17			;Jahr Zehner speichern
	st	x+, r16			;Jahr Einer speichern
	lds	r18, xerbuf + 2		;komprimierte Zeit Byte 2 holen
	ror	r18			;Monat nach rechts schieben
	bld	r18, 7			;LSB von Byte 3 holen, Monat an
	swap	r18			;richtige Position schieben
	andi	r18, 0x0f		;Nicht relevante Bits filtern
	rcall	bytasc			;in ASCII-Wert wandeln
	st	x+, r17			;Monat Zehner speichern
	st	x+, r16			;Monat Einer speichern
	lds	r18, xerbuf + 2		;komprimierte Zeit Byte 2 holen
	andi	r18, 0x1f		;Nicht relevante Bits filtern
	rcall	bytasc			;in ASCII-Wert wandeln
	st	x+, r17			;Tag Zehner speichern
	st	x+, r16			;Tag Einer speichern
	ldi	r16, ';'		;Feldtrennzeichen holen
	st	x+, r16			;Feldtrennzeichen speichern
	lds	r18, xerbuf + 1		;komprimierte Zeit Byte 1 holen
	rcall	bytasc			;in ASCII-Wert wandeln
	st	x+, r17			;Stunde Zehner speichern
	st	x+, r16			;Stunde Einer speichern
	ldi	r16, ':'		;Trennzeichen holen
	st	x+, r16			;Trennzeichen speichern
	lds	r18, xerbuf		;komprimierte Zeit Byte 0 holen
	rcall	bytasc			;in ASCII-Wert wandeln
	st	x+, r17			;Minute Zehner speichern
	st	x+, r16			;Minute Einer speichern
	ldi	r16, ':'		;Trennzeichen holen
	st	x+, r16			;Trennzeichen speichern
	ret
;
; Unterprgramm zum Ergnzen der 00-Sekunde bei Sensordaten
; Register: r16
;	    x <- Zeiger auf ASCII-Puffer
;
xtrse0:	ldi	r16, '0'		;Sekunden = 00
	st	x+, r16			;Sekunden Zehner speichern
	st	x+, r16			;Sekunden Einer speichern
xtrse1:	ldi	r16, ';'		;Feldtrennzeichen holen
	st	x+, r16			;Feldtrennzeichen speichern
	ret
;
; Unterprogramm zum Ergnzen der Sekunden bei Meldungsdaten
; Register: r16, r17, r18
;	    x <- Zeiger auf ASCII-Puffer
;
xtrsec:	lds	r18, xerbuf + 4		;komprimierte Sekunden holen
	rcall	bytasc			;in ASCII-Wert wandeln
	st	x+, r17			;Sekunde Zehner speichern
	st	x+, r16			;Sekunde Einer speichern
	rjmp	xtrse1			;Feldtrennzeichen ausgeben
;
; Unterprogramm zum Extrahieren der Sensordaten aus einem EEPROM-Datensatz in den ASCII-Puffer fr die Datenausgabe zum PC
; Register: r13, r14, r15, r16, r17, r18, r19, r23
;	    x	<- Zeiger auf ASCII-Puffer
;	    y	<- Zeiger auf Datensatz im RAM-Puffer
;	    r19	-> Anzahl der Bytes im Puffer
;
xtrdat:	ldi	r23, 14			;14 Wertepaare bearbeiten
xtrd10:	ld	r16, y+			;L-Byte von Sensorwert 1 holen
	ld	r19, y+			;L-Byte von Sensorwert 2 holen
	ld	r17, y			;H-Bytes von Sensorwert 1/2 holen
	andi	r17, 0x0f		;Bits von Sensorwert 1H filtern
	cpi	r17, 0x08		;gltiger Wert?
	breq	xtrd30			;nein -> Ausgabe berspringen
	brcs	xtrd20			;Wert negativ? nein -> weiter
	ori	r17, 0xf0		;sonst Vorzeichen setzen
xtrd20:	rcall	temasc			;Temperaturwert in ASCII wandeln
	st	x+, r4			;Hunderter ausgeben
	st	x+, r5			;Zehner ausgeben
	st	x+, r6			;Einer ausgeben
	ldi	r16, ','		;Komma holen
	st	x+, r16			;Komma ausgeben
	st	x+, r7			;Zehntel ausgeben
xtrd30:	ldi	r16, ';'		;Feldtrennzeichen laden
	st	x+, r16			;Feldtrennzeichen ausgeben
;
	mov	r16, r19		;L-Byte von Sensorwert 2 holen
	ld	r17, y+			;H-Bytes von Sensorwert 1/2 holen
	swap	r17			;H-Byte an richtige Position legen
	andi	r17, 0x0f		;Bits von Sensorwert 2H filtern
	cpi	r17, 0x08		;gltiger Wert?
	breq	xtrd60			;nein -> Ausgabe berspringen
	brcs	xtrd40			;Wert negativ? nein -> weiter
	ori	r17, 0xf0		;sonst Vorzeichen setzen
xtrd40:	rcall	temasc			;Temperaturwert in ASCII wandeln
	st	x+, r4			;Hunderter ausgeben
	st	x+, r5			;Zehner ausgeben
	st	x+, r6			;Einer ausgeben
        cpi     r23, 1                  ;letztes Wertepaar (Luftdruck)?
        breq    xtrd50                  ;ja -> kein Komma ausgeben
	ldi	r16, ','		;Komma holen
	st	x+, r16			;Komma ausgeben
xtrd50:	st	x+, r7			;Zehntel ausgeben
xtrd60:	ldi	r16, ';'		;Feldtrennzeichen laden
	st	x+, r16			;Feldtrennzeichen ausgeben
	dec	r23			;alle Wertepaare bearbeitet?
	brne	xtrd10			;nein -> Schleife
;
	ldi	r23, 3			;3 Werte bearbeiten
xtrd70:	ld	r16, y+			;Luftfeuchtigkeitswert holen
	cpi	r16, 0x80		;gltiger Wert?
	breq	xtrd80			;nein -> Ausgabe berspringen
	clr	r17			;H-Byte lschen
	rcall	humasc			;Luftfeuchtigkeitswert in ASCII wandeln
	st	x+, r5			;Hunderter ausgeben
	st	x+, r6			;Zehner ausgeben
	st	x+, r7			;Einer ausgeben
xtrd80:	ldi	r16, ';'		;Feldtrennzeichen laden
	st	x+, r16			;Feldtrennzeichen ausgeben
	dec	r23			;alle Wertepaare bearbeitet?
	brne	xtrd70			;nein -> Schleife
;
	ld	r16, y+			;L-Byte vom Batteriespannungswert holen
	ld	r17, y+			;H-Byte vom Batteriespannungswert holen
	rcall	batasc			;Batteriespannungswert in ASCII wandeln
	st	x+, r5			;Einer ausgeben
	ldi	r16, ','		;Komma holen
	st	x+, r16			;Komma ausgeben
	st	x+, r6			;Zehntel ausgeben
	st	x+, r7			;Hundertstel ausgeben
;
; Unterprogramm zum Ermitteln der Byteanzahl im ASCII-Puffer
; Register: r16, r17
;	    x   <- Zeiger auf aktuelle Position im ASCII-Puffer
;	    r19 -> Anzahl der Bytes im Puffer
;
xtbyte:	ldi	r16, low(ascbuf)
	ldi	r17, high(ascbuf)	;Anfangsadresse ASCII-Puffer
	sub	xl, r16			;Differenz zur aktuellen
	sbc	xh, r17			;Adresse berechnen = Anzahl
	mov	r19, xl			;Byteanzahl kopieren
	ret
;
; Unterprogramm zum Extrahieren einer Zeichenkette aus dem Flash-ROM in
; den ASCII-Puffer fr die Datenausgabe zum PC
; Register: r16, z
;	    r17 <- Stringnummer (0x00-0xfd)
;	    x   <- Zeiger auf ASCII-Puffer
;
xtrfst:	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 0			;Sprache englisch?
	brts	xtrf10			;nein -> weiter
	ldi	zl, byte3(stengl * 2)	;sonst
	out	rampz, zl
	ldi	zl, low(stengl * 2)
	ldi	zh, high(stengl * 2)	;Zeiger auf englische Texte
	rjmp	xtrf20
;
xtrf10:	ldi	zl, byte3(stgerm * 2)
	out	rampz, zl
	ldi	zl, low(stgerm * 2)
	ldi	zh, high(stgerm * 2)	;Zeiger auf deutsche Texte
;
xtrf20:	elpm	r16, z+			;Zeichen holen
	cpi	r16, 0xfe		;String-Kennung gefunden?
	brne	xtrf20			;nein -> weiter suchen
	elpm	r16, z+			;Zeichen holen
	cp	r16, r17		;Zeichenkette gefunden?
	breq	xtrf40			;ja -> ausgeben
	cpi	r16, 0xfe		;Endezeichen gefunden?
	brne	xtrf20			;nein -> weitersuchen
xtrf30:	ret				;Ende
;
xtrf40:	elpm	r16, z+			;Zeichen holen
	cpi	r16, 0xff		;Endezeichen?
	breq	xtrf30			;ja -> Ende
	cpi	r16, 0x80		;Zeichencode > 0x7f?
	brcs	xtrf50			;nein -> Zeichen in Puffer
	cpi	r16, 0x90		;Zeichencode > 0x8f?
	brcc	xtrf40			;ja -> berspringen
	rcall	xtruml			;sonst Umlaute konvertieren
xtrf50:	st	x+, r16			;Zeichen in Puffer legen
	rjmp	xtrf40			;Schleife
;
; Unterprogramm zum Extrahieren einer Zeichenkette aus dem EEPROM in den ASCII-Puffer fr die Datenausgabe zum PC
; Register: r18
;	    r16, r17 <- EEPROM-Adresse der Texte (esentx, ealmtx)
;	    r19	     <- Lnge der Zeichenkette
;	    x	     <- Zeiger auf ASCII-Puffer
;
xtrest:	call	eeread			;Zeichen aus EEPROM holen
	cpi	r18, 0x20		;Leerzeichen (= Ende)?
	brne	xtre10			;nein -> weiter
	ret				;sonst Ende
;
xtre10:	cpi	r18, 0x80		;Zeichencode > 0x7f?
	brcs	xtre20			;nein -> Zeichen in Puffer
	cpi	r18, 0x90		;Zeichencode > 0x8f?
	brcc	xtre30			;ja -> berspringen
	push	r16			;sonst EEPROM-Adresse sichern
	mov	r16, r18		;Zeichencode kopieren
	rcall	xtruml			;Umlaute konvertieren
	mov	r18, r16		;Zeichen kopieren
	pop	r16			;EEPROM-Adresse restaurieren
xtre20:	st	x+, r18			;Zeichen in Puffer legen
xtre30:	dec	r19			;alle Zeichen ausgegeben?
	brne	xtrest			;nein -> Schleife
	ret
;
; Unterprogramm zum Konvertieren der Umlaute in 2 Zeichen fr die Datenausgabe zum PC
; Register: r4
;	    r16 <- Zeichencode
;	    x   <- Zeiger auf ASCII-Puffer
;
xtruml:	push	zl
	push	zh
	in	zl, rampz
	push	zl			;Register sichern
	ldi	zl, byte3(umltab * 2)
	mov	r4, zl			;B3 Wert zwischenspeichern
	ldi	zl, low(umltab * 2)
	ldi	zh, high(umltab * 2)	;Zeiger auf Umlaut-Tabelle
	subi	r16, 0x80		;Zeichencode als Offset
	add	r16, r16		;Offset verdoppeln (2 Zeichen)
	add	zl, r16			;zur Tabellenadresse addieren
	clr	r16
	adc	zh, r16			;bertrag addieren (H)
	adc	r4, r16			;bertrag addieren (B3)
	out	rampz, r4
	elpm	r16, z+			;erstes Zeichen holen
	st	x+, r16			;Zeichen in Puffer legen
	elpm	r16, z+			;zweites Zeichen holen
	pop	zl
	out	rampz, zl
	pop	zh
	pop	zl			;Register wieder herstellen
	ret
;
; Unterprogramm zum Extrahieren der Zeitdaten aus einem EEPROM-Datensatz und Ausgabe auf dem LCD
; Register: r0, r1, r13, r14, r15, r16, r17, r18, r19, r23, r24, zl, zh, t
;	    r20 <- X-Koordinate (0-239)
;	    r21 <- Y-Koordinate (0-127)
;
xtdtim:	lds	r18, xedbuf + 2		;komprimierte Zeit Byte 2 holen
	andi	r18, 0x1f		;nicht relevante Bits filtern
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Tag Einer sichern
	mov	r22, r17		;Tag Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Tag Einer holen
	rcall	asclcd			;ausgeben
;
	ldi	r22, '.'		;Trennzeichen
	rcall	asclcd			;ausgeben
	lds	r18, xedbuf + 3		;komprimierte Zeit Byte 3 holen
	bst	r18, 0			;LSB sichern, gehrt zum Monat
	lds	r18, xedbuf + 2		;komprimierte Zeit Byte 2 holen
	ror	r18			;Monat nach rechts schieben
	bld	r18, 7			;LSB von Byte 3 holen, Monat an
	swap	r18			;richtige Position schieben
	andi	r18, 0x0f		;Nicht relevante Bits filtern
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Monat Einer sichern
	mov	r22, r17		;Monat Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Monat Einer holen
	rcall	asclcd			;ausgeben
;
	ldi	r22, '.'		;Trennzeichen
	rcall	asclcd			;ausgeben
	lds	r18, xedbuf + 3		;komprimierte Zeit Byte 3 holen
	lsr	r18			;Jahr an Position schieben
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Jahr Einer sichern
	mov	r22, r17		;Jahr Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Jahr Einer holen
	rcall	asclcd			;ausgeben
;
	ldi	r22, 0xa0		;schmales Leerzeichen
	rcall	asclcd			;ausgeben
	lds	r18, xedbuf + 1		;komprimierte Zeit Byte 1 holen
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Stunde Einer sichern
	mov	r22, r17		;Stunde Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Stunde Einer holen
	rcall	asclcd			;ausgeben
	ldi	r22, ':'		;Trennzeichen holen
	rcall	asclcd			;ausgeben
	lds	r18, xedbuf		;komprimierte Zeit Byte 0 holen
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Minute Einer sichern
	mov	r22, r17		;Minute Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Minute Einer holen
	rcall	asclcd			;ausgeben
	ldi	r22, ':'		;Trennzeichen holen
	rcall	asclcd			;ausgeben
;
	lds	r18, xedbuf + 4		;komprimierte Sekunden holen
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Sekunde Einer sichern
	mov	r22, r17		;Sekunde Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Sekunde Einer holen
	rcall	asclcd			;ausgeben
	ret
;
; Unterprogramm zur Ausgabe einer Zeichenkette aus dem EEPROM zum LCD
; Register: r0, r1, r14, r15, r18, r23, r24, z, t
;	    r16, r17 <- EEPROM-Adresse der Zeichenkette
;	    r19      <- Lnge der Zeichenkette
;	    r20      <- X-Koordinate (0-239)
;	    r21      <- Y-Koordinate (0-127)
;
eeplcd:	call	eeread			;Zeichen aus EEPROM holen
	cpi	r18, 0x21		;Zeichen <= Leerzeichen?
	brcs	eepl10			;ja -> Ende
	mov	r22, r18		;sonst Zeichen kopieren
	push	r16
	push	r17			;EEPROM-Adresse sichern
	push	r19			;Zhler sichern
	call	asclcd			;Zeichen ausgeben
	pop	r19			;Zhler wieder herstellen
	pop	r17
	pop	r16			;EERPOM-Adresse restaurieren
	dec	r19			;alle Zeichen ausgegeben?
	brne	eeplcd			;nein -> Schleife
eepl10:	ret
;
; Unterprogramm zur Ermittlung der Pixel-Lnge einer EEPROM-Zeichenkette
; Register: r0, r1, r18, z
;	    r16, r17 <- EEPROM-Adresse der Zeichenkette
;	    r19      <- Lnge der Zeichenkette
;	    r20      -> Stringlnge in Pixel
;
eeplen:	clr	r20			;Lnge Anfangswert = 0 setzen
eepl20:	call	eeread			;Zeichen aus EEPROM holen
	cpi	r18, 0x20		;Leerzeichen gefunden?
	brne	eepl30			;nein -> weiter
	ret				;sonst Ende
;
eepl30:	subi	r18, 0x20		;Korrektur fr Tabelle
	cpi	r18, 0xe0		;Bereich berschritten?
	brcs	eepl40			;nein -> weiter
	ret				;sonst Ende (Fehler)
;
eepl40:	push	r16
	push	r17			;EEPROM-Adresse sichern
	ldi	r16, 12			;Tabellen-Offset
	mul	r18, r16		;Tabellenplatz berechnen
	ldi	zl, low(chrset*2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrset*2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrset*2)	;Zeiger auf Zeichentabelle B3
	clr	r17
	add	zl, r0			;Tabellenadresse L berechnen
	adc	zh, r1			;Tabellenadresse H berechnen
	adc	r16, r17		;Tabellenadresse B3 berechnen
	out	rampz, r16
	elpm	r16, z			;Zeichenbreite holen
	inc	r16			;Zeichenbreite korrigieren
	add	r20, r16		;zur Gesamtlnge addieren
	pop	r17
	pop	r16			;EEPROM-Adresse holen
	dec	r19			;alle Zeichen bearbeitet?
	brne	eepl20			;nein -> Schleife
	ret				;sonst Ende
;
; Unterprogramm zum Kopieren eines EEPROM-Textes in den Texteingabepuffer
; Register: r0, r1, r20, x
;	    r16 <- EEPROM-Adresse L
;	    r17 <- EEPROM-Adresse H
;	    r18 <- Textnummer in Tabelle (Sensorname/Alarmname/Preset-Name)
;	    r19 <- Maximale Textlnge (8, 11 oder 16 Zeichen)
;
cpeeab:	sts	ascibm, r19		;maximale Zeichenanzahl setzen
	mul	r18, r19		;Tabellenoffset ermitteln
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	ldi	xl, low(ascibf)
	ldi	xh, high(ascibf)	;Zeiger auf Texteingabepuffer
	clr	r20			;aktuelle Position lschen
cpee10:	call	eeread			;Zeichen aus EEPROM holen
	cpi	r18, 0x20		;String-Ende?
	breq	cpee20			;ja -> Ende, sonst Zeichen im
	st	x+, r18			;Texteingabepuffer speichern
	inc	r20			;Position erhhen
	cp	r20, r19		;letzte Position erreicht?
	brne	cpee10			;nein -> Schleife
cpee20:	sts	ascibp, r20		;aktuelle Position speichern
	ret
;
; Unterprogramm zum Kopieren des Texteingabepuffers in das EEPROM
; Register: r0, r1, r20, r21, x
;	    r16 <- EEPROM-Adresse L
;	    r17 <- EEPROM-Adresse H
;	    r18 <- Textnummer in Tabelle (Sensorname/Alarmname/Preset-Name)
;	    r19 <- Maximale Textlnge (8, 11 oder 16 Zeichen)
;
cpabee:	mul	r18, r19		;Tabellenoffset ermitteln
	add	r16, r0			;Offset zur Adresse L addieren
	adc	r17, r1			;Offset zur Adresse H addieren
	ldi	xl, low(ascibf)
	ldi	xh, high(ascibf)	;Zeiger auf Texteingabepuffer
	lds	r21, ascibp		;aktuelle Pufferposition holen
	clr	r20			;Positionszhler lschen
cpab10:	cp	r20, r21		;aktuelle Position erreicht?
	breq	cpab20			;ja -> weiter
	ld	r18, x+			;Zeichen aus Eingabepuffer holen
	call	eewrit			;im EEPROM sichern
	inc	r20			;Positionszhler erhhen
	rjmp	cpab10			;Schleife
cpab20:	cp	r20, r19		;letzte Position erreicht?
	brcc	cpab30			;ja -> Ende
	ldi	r18, ' '		;sonst Leerzeichen
	call	eewrit			;im EEPROM sichern
	inc	r20			;Positionszhler erhhen
	rjmp	cpab20			;Schleife
cpab30:	ret
;
; Unterprogramm zur Ausgabe des Texteingabepuffers auf dem LCD
; Register: r0, r1, r14, r15, r16, r17, r18, r19, r23, r24, zl, zh, t
;	    r20 <- X-Koordinate (0-239)
;	    r21 <- Y-Koordinate (0-127)
;
txblcd:	ldi	zl, low(ascibf)
	ldi	zh, high(ascibf)	;Zeiger auf Texteingabepuffer
	lds	r17, ascibp		;aktuelle Eingabeposition holen
	tst	r17			;Puffer leer?
	breq	txbl20			;ja -> berspringen
	clr	r16			;Positionszhler lschen
txbl10:	ld	r22, z+			;Zeichen holen
	push	r16			;Positionszhler sichern
	push	zl
	push	zh			;Zeiger sichern
	rcall	asclcd			;Zeichen ausgeben
	pop	zh
	pop	zl			;Zeiger restaurieren
	pop	r16			;Positionszhler restaurieren
	inc	r16			;Position erhhen
	lds	r17, ascibp		;aktuelle Eingabeposition holen
	cp	r16, r17		;Eingabeposition erreicht?
	brne	txbl10			;nein -> Schleife
	lds	r17, ascibm		;maximale Zeichenanzahl holen
	cp	r16, r17		;letzte Position erreicht?
	brcc	txbl30			;ja -> senkrechter Cursor
txbl20:	ldi	r22, 0xac		;sonst Cursorsymbol
	rcall	asclcd			;ausgeben
	ret
txbl30:	ldi	r22, 0x7f		;Endezeichen laden
	ldi	r16, 0x01
	mov	txflag, r16		;Textflag invers setzen
	rcall	asclcd			;ausgeben
	clr	txflag			;Textflag lschen
	ret
;
; Unterprogramm zur Ermittlung der Lnge des Texteingabepuffers
; Register: r0, r1, r16, r17, r19, zl, zh
;	    r18 -> Pufferlnge (Pixel)
;
txblen:	ldi	zl, low(ascibf)
	ldi	zh, high(ascibf)	;Zeiger auf Texteingabepuffer
	clr	r19			;Positionszhler lschen
	clr	r18			;Lngenzhler lschen
	lds	r16, ascibp		;aktuelle Eingabeposition holen
	tst	r16			;Puffer leer?
	brne	txbl50			;nein -> weiter
	ret				;sonst Ende
;
txbl50:	ld	r16, z+			;Zeichen holen
	subi	r16, 0x20		;Korrektur fr Tabelle
	cpi	r16, 0xe0		;Bereich berschritten?
	brcs	txbl60			;nein -> weiter
	ret				;sonst Ende (Fehler)
;
txbl60:	ldi	r17, 12			;Tabellen-Offset
	mul	r16, r17		;Tabellenplatz berechnen
	push	zl
	push	zh			;Puffer-Zeiger sichern
	clr	r17
	ldi	zl, low(chrset*2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrset*2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrset*2)	;Zeiger auf Zeichenzabelle B3
	add	zl, r0			;Tabellenadresse L berechnen
	adc	zh, r1			;Tabellenadresse H berechnen
	adc	r16, r17
	out	rampz, r16
	elpm	r16, z			;Zeichenbreite holen
	pop	zh
	pop	zl			;Puffer-Zeiger restaurieren
	inc	r16			;Zeichenbreite korrigieren
	add	r18, r16		;zur Gesamtlnge addieren
;
	inc	r19			;Position erhhen
	lds	r16, ascibp		;aktuelle Eingabeposition holen
	cp	r16, r19		;Eingabeposition erreicht?
	brne	txbl50			;nein -> Schleife
	ret
;
; Unterprogramm zum Senden des ASCII-Pufferinhaltes + CR/LF zum PC
; Register: r16, z
;	    r19 <- Anzahl der zu sendenden Bytes
;
senasc:	tst	r19			;sind Zeichen zu senden?
	breq	sencrl			;nein -> nur CR/LF senden
	ldi	zl, low(ascbuf)
	ldi	zh, high(ascbuf)	;Zeiger auf ASCII-Puffer
sena10:	ld	r16, z+			;Zeichen aus Puffer holen
	call	put0ch			;Zeichen senden
	dec	r19			;alle Zeichen gesendet?
	brne	sena10			;nein -> Schleife
;
; Unterprogramm zum Senden von CR/LF zum PC
; Register: r16
;
sencrl:	ldi	r16, cr			;CR laden
	call	put0ch			;CR ausgeben
	ldi	r16, lf			;LF laden
	call	put0ch			;LF ausgeben
	ret
;
; Unterprogramm zum Umwandeln eines Bytes in 2 Dezimalstellen
; Register: r16 -> Ergebnis L
;	    r17 -> Ergebnis H
;	    r18 <- Byte
;
bytasc:	cpi	r18, 100		;Wert < 100?
	brcs	byta10			;ja -> weiter
	subi	r18, 100		;sonst 100 subtrahieren
	rjmp	bytasc
byta10:	clr	r17			;H-Byte Anfangswert
byta20:	cpi	r18, 10			;Wert < 10?
	brcs	byta30			;ja -> weiter
	subi	r18, 10			;sonst 10 subtrahieren
	inc	r17			;H-Wert erhhen
	rjmp	byta20
byta30:	mov	r16, r18		;L-Wert kopieren
	ori	r17, 0x30		;H-Wert in ASCII wandeln
	ori	r16, 0x30		;L-Wert in ASCII wandeln
	ret
;
; Unterprogramm zum Umwandeln von 2 Bytes in 4 Dezimalstellen (0-9999)
; Register: r4  -> Ergebnis Stelle 1
;	    r5  -> Ergebnis Stelle 2
;	    r6  -> Ergebnis Stelle 3
;	    r7  -> Ergebnis Stelle 4
;	    r16 <- Byte L
;	    r17 <- Byte H
;
bytas4:	clr	r4			;Stelle 1 Anfangswert
	movw	r6, r16			;Zahl kopieren
	ldi	r16, low(9999)
	ldi	r17, high(9999)		;Vergleichswert laden
	sub	r16, r6
	sbc	r17, r7			;Zahl < 10000?
	brcc	byta40			;ja -> weiter
	clr	r4
	clr	r5
	clr	r6
	clr	r7			;sonst alle Stellen lschen
	rjmp	bytab0			;in ASCII wandeln
;
byta40:	clr	r4			;Stelle 1 lschen
byta50:	ldi	r16, low(999)
	ldi	r17, high(999)		;Vergleichswert laden
	sub	r16, r6
	sbc	r17, r7			;Zahl < 1000?
	brcc	byta60			;ja -> weiter
	ldi	r16, low(1000)
	ldi	r17, high(1000)		;sonst 1000
	sub	r6, r16
	sbc	r7, r17			;subtrahieren
	inc	r4			;Stelle 1 erhhen
	rjmp	byta50			;Schleife
;
byta60:	clr	r5			;Stelle 2 lschen
byta70:	ldi	r16, 99
	clr	r17			;Vergleichswert laden
	sub	r16, r6
	sbc	r17, r7			;Zahl < 100?
	brcc	byta80			;ja -> weiter
	ldi	r16, 100
	clr	r17			;sonst 100
	sub	r6, r16
	sbc	r7, r17			;subtrahieren
	inc	r5			;Stelle 2 erhhen
	rjmp	byta70			;Schleife
;
byta80:	mov	r16, r6			;Restwert holen
	clr	r6			;Stelle 3 lschen
byta90:	cpi	r16, 10			;Zahl < 10?
	brcs	bytaa0			;ja -> weiter
	subi	r16, 10			;sonst 10 subtrahieren
	inc	r6			;Stelle 3 erhhen
	rjmp	byta90			;Schleife
;
bytaa0:	mov	r7, r16			;Stelle 4 kopieren
bytab0:	ldi	r16, 0x30
	or	r4, r16			;Stelle 1 in ASCII wandeln
	or	r5, r16			;Stelle 2 in ASCII wandeln
	or	r6, r16			;Stelle 3 in ASCII wandeln
	or	r7, r16			;Stelle 4 in ASCII wandeln
	ret
;
; Unterprogramm zum Umwandeln eines Bytes in 2 Hexadezimalstellen
; Register: r16 -> Ergebnis L
;	    r17 -> Ergebnis H
;	    r18 <- Byte
;
hexasc:	mov	r16, r18		;Byte in L-Wert kopieren
	mov	r17, r18		;Byte in H-Wert kopieren
	ldi	r18, 7			;fr Umrechnung in Hex-Wert
	andi	r17, 0xf0		;oberes Nibble filtern
	swap	r17			;Nibbles tauschen
	ori	r17, 0x30		;in ASCII wandeln
	cpi	r17, 0x3a		;Wert>"9"?
	brcs	hexa10			;nein -> weiter
	add	r17, r18		;sonst in Buchstaben wandeln
hexa10:	andi	r16, 0x0f		;unteres Nibble filtern
	ori	r16, 0x30		;in ASCII wandeln
	cpi	r16, 0x3a		;Wert>"9"?
	brcs	hexa20			;nein -> weiter
	add	r16, r18		;sonst in Buchstaben wandeln
hexa20:	ret
;
; Unterprogramm zur Wandlung eines Temperaturwertes ASCII -> Hex
; Register: r14, r15, r18
;	    rxdat1 <- ASCII-Wert
;	    r16    -> Ergebnis Hexwert L \ Ergebnis in
;	    r17    -> Ergebnis Hexwert H / Zehntel Grad
;
temhex:	clr	r14
	clr	r15			;Anfangswert lschen
	lds	r18, rxdat1 + 2		;Hunderterstelle holen
	cpi	r18, '1'		;Hunderterstelle belegt?
	brne	temh10			;nein -> weiter
	ldi	r16, low(1000)		;sonst 1000 addieren
	ldi	r17, high(1000)		;(wegen Zehntel)
	add	r14, r16		;addieren
	adc	r15, r17		;bertrag addieren
temh10:	lds	r18, rxdat1 + 3		;Zehnerstelle holen
	cpi	r18, '-'		;Minuszeichen?
	breq	temh30			;ja -> weiter
	cpi	r18, ' '		;Leerzeichen?
	breq	temh30			;ja -> weiter
	andi	r18, 0x0f		;sonst Wandlung in Ziffer
	ldi	r16, 100		;100 addieren
	clr	r17			;(wegen Zehntel)
temh20:	tst	r18			;Zhler = 0?
	breq	temh30			;ja -> weiter
	add	r14, r16		;sonst addieren
	adc	r15, r17		;bertrag addieren
	dec	r18			;Zhler vermindern
	rjmp	temh20			;Schleife
temh30:	lds	r18, rxdat1 + 4		;Einerstelle holen
	andi	r18, 0x0f		;Wandlung in Ziffer
	ldi	r16, 10			;10 addieren
	clr	r17			;(wegen Zehntel)
temh40:	tst	r18			;Zhler = 0?
	breq	temh50			;ja -> weiter
	add	r14, r16		;sonst addieren
	adc	r15, r17		;bertrag addieren
	dec	r18			;Zhler vermindern
	rjmp	temh40			;Schleife
temh50:	lds	r18, rxdat1 + 6		;Zehntelstelle holen
	andi	r18, 0x0f		;Wandlung in Ziffer
	ldi	r16, 1			;1 addieren
	clr	r17			;(wegen Zehntel)
temh60:	tst	r18			;Zhler = 0?
	breq	temh70			;ja -> weiter
	add	r14, r16		;sonst addieren
	adc	r15, r17		;bertrag addieren
	dec	r18			;Zhler vermindern
	rjmp	temh60			;Schleife
temh70:	lds	r18, rxdat1 + 2		;Hunderterstelle holen
	cpi	r18, '-'		;Minuszeichen?
	breq	temh80			;ja -> Zweierkomplement bilden
	lds	r18, rxdat1 + 3		;Zehnerstelle holen
	cpi	r18, '-'		;Minuszeichen?
	breq	temh80			;ja -> Zweierkomplement bilden
	movw	r16, r14		;sonst Ergebnis kopieren
	ret
temh80:	clr	r16
	clr	r17			;Umwandlung ins 16-Bit-Zweierkomplement
	sub	r16, r14		;0x10000 - Ergebnis
	sbc	r17, r15		;bertrag
	ret
;
; Unterprogramm zur Wandlung eines Sensorwertes (Luftdruck oder Luftfeuchtigkeit) ASCII -> Hex
; Register: r14, r15, r18
;	    rxdat1 <- ASCII-Wert
;	    r16    -> Ergebnis Hexwert L \ Ergebnis in
;	    r17    -> Ergebnis Hexwert H / Hektopascal bzw. %
;
senhex:	clr	r14
	clr	r15			;Anfangswert lschen
	lds	r18, rxdat1 + 3		;Tausenderstelle holen
	cpi	r18, '1'		;Tausenderstelle belegt?
	brne	senh10			;nein -> weiter
	ldi	r16, low(1000)		;sonst
	ldi	r17, high(1000)		;1000 addieren
	add	r14, r16		;addieren
	adc	r15, r17		;bertrag
senh10:	lds	r18, rxdat1 + 4		;Hunderterstelle holen
	cpi	r18, ' '		;Leerzeichen?
	breq	senh30			;ja -> weiter
	andi	r18, 0x0f		;sonst Wandlung in Ziffer
	ldi	r16, 100		;100 addieren
	clr	r17
senh20:	tst	r18			;Zhler = 0?
	breq	senh30			;ja -> weiter
	add	r14, r16		;sonst addieren
	adc	r15, r17		;bertrag
	dec	r18			;Zhler vermindern
	rjmp	senh20			;Schleife
senh30:	lds	r18, rxdat1 + 5		;Zehnerstelle holen
	cpi	r18, ' '		;Leerzeichen?
	breq	senh50			;ja -> weiter
	andi	r18, 0x0f		;Wandlung in Ziffer
	ldi	r16, 10			;10 addieren
	clr	r17
senh40:	tst	r18			;Zhler = 0?
	breq	senh50			;ja -> weiter
	add	r14, r16		;sonst addieren
	adc	r15, r17		;bertrag
	dec	r18			;Zhler vermindern
	rjmp	senh40			;Schleife
senh50:	lds	r18, rxdat1 + 6		;Einerstelle holen
	andi	r18, 0x0f		;Wandlung in Ziffer
	ldi	r16, 1			;1 addieren
	clr	r17
senh60:	tst	r18			;Zhler = 0?
	breq	senh70			;ja -> weiter
	add	r14, r16		;sonst addieren
	adc	r15, r17		;bertrag
	dec	r18			;Zhler vermindern
	rjmp	senh60			;Schleife
senh70:	movw	r16, r14		;Ergebnis kopieren
	ret
;
; Unterprogramm zur Wandlung eines Temperaturwertes Hex -> ASCII
; Register: r13, r14, r15, r18
;	    r16   <- Hexwert L \ Temperaturwert
;	    r17   <- Hexwert H / in Zehntel Grad
;	    r4-r7 -> Ergebnis in ASCII (r4 = MSB, r7 = LSB)
;
temasc:	ldi	r18, '0'		;Hunderter initialisieren
	mov	r13, r17		;Vorzeichen sichern
	bst	r17, 7			;Wert negativ?
	brtc	tema10			;nein -> bearbeiten
	movw	r14, r16		;Eingabewert kopieren
	clr	r16
	clr	r17			;Umwandlung ins 16 Bit-Zweierkomplement
	sub	r16, r14		;0x10000 - Ergebnis
	sbc	r17, r15		;bertrag
;
tema10:	movw	r14, r16		;Eingabewert kopieren
tema20:	ldi	r16, low(1000)		;Hunderterstelle ermitteln
	ldi	r17, high(1000)		;(Wert 1000 wegen Zehntel benutzen)
	cp	r14, r16
	cpc	r15, r17		;Hunderterstelle erreicht?
	brcs	tema30			;ja -> weiter
	sub	r14, r16		;sonst
	sbc	r15, r17		;1000 subtrahieren
	inc	r18			;Hunderter erhhen
	rjmp	tema20			;Schleife
tema30:	cpi	r18, '0'		;Hunderter = 0?
	brne	tema40			;nein -> weiter
	ldi	r18, ' '		;sonst durch Leerzeichen ersetzen
tema40:	mov	r4, r18			;Hunderterstelle speichern
;
	ldi	r18, '0'		;Zehner initialisieren
	ldi	r16, 100		;Zehnerstelle ermitteln
	clr	r17			;(Wert 100 wegen Zehntel benutzen)
tema50:	cp	r14, r16
	cpc	r15, r17		;Zehnerstelle erreicht?
	brcs	tema60			;ja -> weiter
	sub	r14, r16		;sonst
	sbc	r15, r17		;100 subtrahieren
	inc	r18			;Zehner erhhen
	rjmp	tema50			;Schleife

tema60:	mov	r5, r18			;Zehnerstelle speichern
	cpi	r18, '0'		;war Zehner = 0?
	brne	tema70			;nein -> weiter zum Einer
	mov	r18, r4			;sonst ASCII Hunderter nochmals holen
	cpi	r18, ' '		;war Hunderter ebenfalls = 0?
	brne	tema70			;nein -> weiter zum Einer
	mov	r5, r18			;sonst Leerzeichen an ASCII Zehner setzen
;
tema70:	ldi	r18, '0'		;Einer initialisieren
	ldi	r16, 10			;Einerstelle ermitteln
tema80:	cp	r14, r16		;Einerstelle erreicht?
	brcs	tema90			;ja -> weiter
	sub	r14, r16		;sonst 10 subtrahieren
	inc	r18			;Einer erhhen
	rjmp	tema80			;Schleife
tema90:	mov	r6, r18			;Einerstelle speichern
;
	mov	r18, r14		;Zehntel (Rest) kopieren
	ori	r18, 0x30		;in ASCII wandeln
	mov	r7, r18			;und speichern
	bst	r13, 7			;war Hexwert negativ?
	brtc	tem120			;nein -> Ende
	ldi	r16, '-'		;sonst Minuszeichen laden
	mov	r17, r5			;Zehner holen
	cpi	r17, ' '		;Platz fr Minuszeichen?
	brne	tem130			;nein -> weiter
	mov	r5, r16			;sonst Minuszeichen anstatt Zehner
tem120:	ret				;einsetzen
tem130:	mov	r4, r16			;Minuszeichen anstatt Hunderter
	ret				;einsetzen
;
; Unterprogramm zur Wandlung eines Luftfeuchtigkeitswertes Hex -> ASCII
; Register: r13, r14, r15, r18
;	    r16   <- Hexwert L \ Luftfeuchtigkeitswert
;	    r17   <- Hexwert H / in Prozent
;	    r4-r7 -> Ergebnis in ASCII (r4 = MSB, r7 = LSB)
;
humasc:	call	temasc			;zunchst wie Temperaturwert in ASCII wandeln
	mov	r16, r6			;Zehnerstelle holen
	cpi     r16, '0'                ;Zehner = '0'?
        brne    huma10                  ;nein -> Ende
        mov     r16, r5                 ;sonst Hunderterstelle holen
        cpi     r16, '1'                ;Hunderter = '1' (Wert = 100)?
        breq    huma10			;ja -> Ende
        ldi     r16, ' '                ;sonst
	mov	r6, r16			;Zehner durch Leerzeichen ersetzen
huma10:	ret
;
; Unterprogramm zur Wandlung eines Batteriespannungswertes Hex -> ASCII
; Register: r13, r14, r15, r18
;	    r16   <- Hexwert L \ Batteriespannunngswert
;	    r17   <- Hexwert H / in Hundertstel Volt
;	    r4-r7 -> Ergebnis in ASCII (r4 = MSB, r7 = LSB)
;
batasc:	call	temasc			;zunchst wie Temperaturwert in ASCII wandeln
	mov	r16, r5			;Einerstelle holen
        cpi     r16, ' '                ;Leerzeichen an Einerstelle?
        brne    bata10                  ;nein -> Ende
        ldi     r16, '0'                ;sonst
	mov	r5, r16			;Einer durch '0' ersetzen
bata10:	ret
;
; Unterprogramm zur Wandlung eines Zeitwertes Hex->ASCII
; Register: r14, r15, r18
;	    r16   <- Hexwert L \ Zeit als
;	    r17   <- Hexwert H / Minutenwert
;	    r4-r7 -> Ergebnis in ASCII (r4 = MSB, r7 = LSB)
;
timasc:	movw	r14, r16		;Eingabewert kopieren
	ldi	r16, low(60)
	ldi	r17, high(60)		;Stunden und Minuten trennen
	clr	r18			;Zhler lschen
tima10:	sub	r14, r16		;60 subtrahieren
	sbc	r15, r17		;bertrag subtrahieren
	brcs	tima20			;zuviel subtrahiert? ja -> weiter
	inc	r18			;sonst Zhler erhhen und
	rjmp	tima10			;nochmals 60 subtrahieren
tima20:	add	r14, r16		;Originalwert
	adc	r15, r17		;wieder herstellen
	mov	r6, r14			;Minuten zwischenspeichern
	rcall	bytasc			;Byte in ASCII Zeichen wandeln
	cpi	r17, '0'		;Stunden Zehner = 0?
	brne	tima30			;nein -> weiter
	ldi	r17, ' '		;sonst Leerzeichen benutzen
tima30:	mov	r4, r17			;Stunden Zehner speichern
	mov	r5, r16			;Stunden Einer speichern
	mov	r18, r6			;Minutenwert holen
	rcall	bytasc			;Byte in ASCII Zeichen wandeln
	mov	r6, r17			;Minuten Zehner speichern
	mov	r7, r16			;Minuten Einer speichern
	ret
;
; Unterprogramm zur Ausgabe der Zeichenkette "Bitte warten" in der Menzeile (zentriert)
; Register: r0, r1, r14, r15, r16, r17, r18, r19, r20, r21, r22, r23, r24, z, t
;
strwai:	ldi	r22, 0x43		;String "Bitte warten"
	rcall	strlng			;Lnge ermitteln
	asr	r18			;Lnge halbieren
	ldi	r20, 119		;X-Mittenposition laden
	sub	r20, r18		;Text-Anfangsposition ermitteln
	ldi	r18, 2			;X-Anfangsposition Menzeile
	ldi	r19, 115		;Y-Anfangsposition Menzeile
	ldi	r21, 126		;Y-Endposition zum Lschen
	rcall	clrwnd			;Bereich vor dem Text lschen
	ldi	r21, 115		;Y-Textposition
	ldi	r22, 0x43		;String "Bitte warten"
	rcall	strlcd			;ausgeben
	mov	r18, r20		;X-Position Textende kopieren
	ldi	r19, 115		;Y-Position Textende
	ldi	r20, 238		;X-Endposition Menzeile
	ldi	r21, 126		;Y-Endposition Menzeile
	rcall	clrwnd			;Bereich hinter dem Text lsch.
	ret
;
; Unterprogramm zur Ausgabe einer Zeichenkette aus dem Flash-ROM
; Register: r0, r1, r14, r15, r16, r17, r18, r19, r23, r24, z, t
;	    r20 <- X-Koordinate (0-239)
;	    r21 <- Y-Koordinate (0-127)
;	    r22 <- Stringnummer (0x00-0xfd)
;
strlcd:	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 0			;Sprache englisch?
	brts	str010			;nein -> weiter
	ldi	zl, byte3(stengl * 2)	;sonst
	out	rampz, zl
	ldi	zl, low(stengl * 2)
	ldi	zh, high(stengl * 2)	;Zeiger auf englische Texte
	rjmp	str020
;
str010:	ldi	zl, byte3(stgerm * 2)
	out	rampz,zl
	ldi	zl, low(stgerm * 2)
	ldi	zh, high(stgerm * 2)	;Zeiger auf deutsche Texte
;
str020:	elpm	r16, z+			;Zeichen holen
	cpi	r16, 0xfe		;String-Kennung gefunden?
	brne	str020			;nein -> weiter suchen
	elpm	r16, z+			;Zeichen holen
	cp	r16, r22		;Zeichenkette gefunden?
	breq	str030			;ja -> ausgeben
	cpi	r16, 0xfe		;Endezeichen gefunden?
	brne	str020			;nein -> weiter suchen
	ret				;sonst Ende
;
str030:	elpm	r22, z+			;Zeichen holen
	cpi	r22, 0xff		;Endezeichen?
	brne	str040			;nein -> weiter
	ldi	r22, 0x7f		;sonst Endezeichen
	rcall	asclcd			;Zeichen ausgeben
	ret
str040:	cpi	r22, ' '		;Leerzeichen?
	brne	str050			;nein -> ausgeben
	ldi	r22, 0xa0		;sonst schmales Leerzeichen
str050:	push	zl
	push	zh
	in	zl, rampz
	push	zl			;Flash-Zeiger sichern
	rcall	asclcd			;Zeichen ausgeben
	pop	zl
	out	rampz, zl
	pop	zh
	pop	zl			;Flashzeiger wieder herstellen
	rjmp	str030			;nchstes Zeichen bearbeiten
;
; Unterprogramm zur Ermittlung der Pixel-Lnge einer Zeichenkette
; Register: r0, r1, r16, r17, z
;	    r22 <- Stringnummer  (0x00-0xfc)
;	    r18 -> Stringlnge in Pixel
;
strlng:	clr	r18			;Lnge Anfangswert = 0 setzen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 0			;Sprache englisch?
	brts	str110			;nein -> weiter
	ldi	zl, byte3(stengl * 2)	;sonst
	out	rampz, zl
	ldi	zl, low(stengl * 2)
	ldi	zh, high(stengl * 2)	;Zeiger auf englische Texte
	rjmp	str120
;
str110:	ldi	zl, byte3(stgerm * 2)
	out	rampz, zl
	ldi	zl, low(stgerm * 2)
	ldi	zh, high(stgerm * 2)	;Zeiger auf deutsche Texte
;
str120:	elpm	r16, z+			;Zeichen holen
	cpi	r16, 0xfe		;String-Kennung gefunden?
	brne	str120			;nein -> weiter suchen
	elpm	r16, z+			;Zeichen holen
	cp	r16, r22		;Zeichenkette gefunden?
	breq	str130			;ja -> Lnge ermitteln
	cpi	r16, 0xfe		;Endezeichen gefunden?
	brne	str120			;nein -> weitersuchen
	ret				;sonst Ende
;
str130:	elpm	r16, z+			;Zeichen holen
	cpi	r16, 0xff		;Endezeichen?
	brne	str140			;nein -> weiter
	ret				;sonst Ende
str140:	cpi	r16, ' '		;Leerzeichen?
	brne	str150			;nein -> weiter
	ldi	r16, 0xa0		;sonst schmales Leerzeichen
str150:	subi	r16, 0x20		;Korrektur fr Tabelle
	cpi	r16, 0xe0		;Bereich berschritten?
	brcs	str160			;nein -> weiter
	ret				;sonst Ende (Fehler)
str160:	ldi	r17, 12			;Tabellen-Offset
	mul	r16, r17		;Tabellenplatz berechnen
	push	zl
	push	zh
	in	zl, rampz
	push	zl			;String-Zeiger sichern
;
	ldi	zl, low(chrset * 2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrset * 2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrset * 2)	;Zeiger auf Zeichentabelle B3
	clr	r17
	add	zl, r0			;Tabellenadresse L berechnen
	adc	zh, r1			;Tabellenadresse H berechnen
	adc	r16, r17		;Tabellenadresse B3 berechnen
	out	rampz, r16
	elpm	r16, z			;Zeichenbreite holen
;
	pop	zl
	out	rampz, zl
	pop	zh
	pop	zl			;String-Zeiger restaurieren
	inc	r16			;Zeichenbreite korrigieren
	add	r18, r16		;zur Gesamtlnge addieren
	rjmp	str130			;Schleife
;
; Unterprogramm zur Ausgabe eines ASCII-Zeichens auf dem LCD, die Darstellung ist abhngig vom Textflag (invers, unterstrichen, Mini)
; Register: r0, r1, r14, r15, r16, r17, r18, r19, r23, r24, zl, zh, t
;	    r20 <- X-Koordinate (0-239)
;	    r21 <- Y-Koordinate (0-127)
;	    r22 <- ASCII-Zeichencode
;
asclcd:	mov	r16, r22		;ASCII-Zeichen kopieren
	subi	r16, 0x20		;Korrektur fr Tabelle
	cpi	r16, 0xe0		;Bereich berschritten?
	brcs	asc010			;nein -> weiter
	ret				;sonst Ende (Fehler)
;
asc010:	bst	txflag, 2		;Mini-Zeichensatz?
	brts	asc020			;ja -> weiter
	ldi	r17, 12			;Tabellen-Offset Normal-Font
	mul	r16, r17		;Tabellenplatz berechnen
	clr	r17
	ldi	zl, low(chrset * 2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrset * 2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrset * 2)	;Zeiger auf Zeichentabelle B3
	rjmp	asc030
;
asc020:	ldi	r17, 6			;Tabellen-Offset Mini-Font
	mul	r16, r17		;Tabellenplatz berechnen
	clr	r17
	ldi	zl, low(chrsmi * 2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrsmi * 2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrsmi * 2)	;Zeiger auf Zeichentabelle B3
;
asc030:	add	zl, r0			;Tabellenadresse L
	adc	zh, r1			;Tabellenadresse H
	adc	r16, r17		;Tabellenadresse B3
	out	rampz, r16
	elpm	r14, z+			;Zeichenbreite holen
	ldi	r23, 11			;11 Zeilen ausgeben
	brtc	asc040			;Mini-Font? nein -> weiter
	ldi	r23, 5			;sonst nur 5 Zeilen ausgeben
asc040:	mov	r19, r21		;Y-Koordinate kopieren
asc050:	mov	r18, r20		;X-Koordinate kopieren
	bst	txflag, 0		;Inversflag holen
	bld	r19, 7			;und kopieren
	cpi	r23, 2			;vorletzte Zeile?
	brne	asc070			;nein -> normale Ausgabe
;
	bst	txflag, 1		;Unterstreichung?
	brtc	asc070			;nein -> normale Ausgabe
	mov	r16, txflag		;Textflags kopieren und
	com	r16			;invertieren (fr Unterstrich)
	bst	r16, 0			;Inversflag holen
	bld	r19, 7			;und in Y-Koordinate einsetzen
	mov	r24, r14		;Zeichenbreite kopieren
	inc	r24			;und korrigieren
asc060:	rcall	pixlcd			;Pixel ausgeben
	inc	r18			;nchste X-Koordinate setzen
	dec	r24			;alle Pixel ausgegeben?
	brne	asc060			;nein -> Schleife
	elpm	r0, z+			;nchste Zeichenzeile
	rjmp	asc110			;weiter mit normaler Ausgabe
;
asc070:	rcall	pixlcd			;1 Pixel lschen (vor Zeichen)
	inc	r18			;nchste X-Koordinate setzen
	elpm	r15, z+			;eine Zeichenzeile holen
	bst	txflag, 0		;Invers-Darstellung?
	brtc	asc080			;nein -> weiter
	com	r15			;sonst alle Pixel invertieren
asc080:	ldi	r24, 8			;Zhler fr Bitverschiebung
asc090:	cp	r24, r14		;Zhler = Zeichenbreite?
	breq	asc100			;ja -> Ausgabe beginnen
	rol	r15			;sonst Zeichenzeile verschieben
	dec	r24			;komplettes Byte verschoben?
	brne	asc090			;nein -> Schleife
	rjmp	asc110			;sonst keine weiteren Pixel
asc100:	bst	r15, 7			;Bit aus Zeichenzeile holen
	bld	r19, 7			;und in Y-Koordinate einsetzen
	rcall	pixlcd			;Zeichenpixel ausgeben
	rol	r15			;nchstes Pixel bearbeiten
	inc	r18			;nchste X-Koordinate setzen
	dec	r24			;alle Pixel ausgegeben?
	brne	asc100			;nein -> nchstes Pixel
asc110:	andi	r19, 0x7f		;Y-Koordinate filtern
	inc	r19			;nchste Y-Koordinate setzen
	dec	r23			;alle Zeilen ausgegeben?
	brne	asc050			;nein -> nchste Zeile
	add	r20, r14		;X-Koordinate fr nchstes
	inc	r20			;Zeichen ermitteln
	ret
;
; Unterprogramm zur Ausgabe eines Bytes als 2 ASCII-Zeichen auf dem LCD
; Register: r0, r1, r14, r15, r16, r17, r18, r19, r23, r24, zl, zh, t
;	    r18 <- Byte (0-99)
;	    r20 <- X-Koordinate (0-239)
;	    r21 <- Y-Koordinate (0-127)
;
bytlcd:	call	bytasc			;Byte in ASCII wandeln
	push	r16			;Einer sichern
	mov	r22, r17		;Zehner holen und
	call	asclcd			;ausgeben
	pop	r22			;gesicherte Einer holen und
	call	asclcd			;ausgeben
	ret
;
; Unterprogramm zur Ermittlung der Pixel-Lnge eines Zeichens
; Register: r0, r1, r17, z
;	    r16 <- ASCII-Zeichencode (0x20-0x9f)
;	    r16 -> Zeichenlnge in Pixel
;
asclen:	subi	r16, 0x20		;Korrektur fr Tabelle
	cpi	r16, 0xe0		;Bereich berschritten?
	brcs	asc120			;nein -> weiter
	ret				;sonst Ende (Fehler)
;
asc120:	bst	txflag, 2		;Mini-Zeichensatz?
	brts	asc130			;ja -> weiter
	ldi	r17, 12			;Tabellen-Offset Normal-Font
	mul	r16, r17		;Tabellenplatz berechnen
	clr	r17
	ldi	zl, low(chrset * 2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrset * 2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrset * 2)	;Zeiger auf Zeichentabelle B3
	rjmp	asc140
;
asc130:	ldi	r17, 6			;Tabellen-Offset Mini-Font
	mul	r16, r17		;Tabellenplatz berechnen
	clr	r17
	ldi	zl, low(chrsmi * 2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrsmi * 2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrsmi * 2)	;Zeiger auf Zeichentabelle B3
;
asc140:	add	zl, r0			;Tabellenadresse L berechnen
	adc	zh, r1			;Tabellenadresse H berechnen
	adc	r16, r17		;Tabellenadresse B3 berechnen
	out	rampz, r16
	elpm	r16, z			;Zeichenbreite holen
	inc	r16			;Zeichenbreite korrigieren
	ret
;
; Unterprogramm zur Ausgabe eines ASCII-Zeichens auf dem LCD im Gro-Format (2x6 normale Zeichen = 16 x 33 Pixel), mglich sind die Zeichen 0-9, Minus und
; Leerzeichen; das Textflag wird nicht ausgewertet
; Register: r0, r1, r14, r15, r16, r17, r18, r19, r23, r24, zl, zh, t
;	    r20 <- X-Koordinate (0-239)
;	    r21 <- Y-Koordinate (0-127)
;	    r22 <- ASCII-Zeichencode
;
biglcd:	cpi	r22, '-'		;Minus-Zeichen?
	brne	bigl10			;nein -> weiter testen
	ldi	r22, 10			;sonst Zeichencode fr Minus
	rjmp	bigl30			;setzen und ausgeben
bigl10:	cpi	r22, ' '		;Leerzeichen?
	brne	bigl20			;nein -> ausgeben
	ldi	r22, 11			;sonst Zeichencode fr Leer-
	rjmp	bigl30			;zeichen setzen und ausgeben
bigl20:	andi	r22, 0x0f		;Ziffernwert ermitteln
bigl30:	ldi	r16, 6			;Offset fr Zeichentabelle
	mul	r22, r16		;Tabellenposition ermitteln
	ldi	r22, 0x98		;ASCII-Wert fr Grozeichen
	add	r22, r0			;ermitteln
	push	r20			;X Koordinate sichern
	rcall	ascfrg			;Fragment oben links ausgeben
	inc	r22			;ASCII-Wert nchstes Fragment
	rcall	ascfrg			;Fragment oben rechts ausgeben
	pop	r20			;X Koordinate wieder herstellen
	ldi	r16, 11			;Y Koordinate fr mittlere
	add	r21, r16		;Fragmente ermitteln
	inc	r22			;ASCII-Wert nchstes Fragment
	push	r20			;X Koordinate sichern
	rcall	ascfrg			;Fragment Mitte links ausgeben
	inc	r22			;ASCII-Wert nchstes Fragment
	rcall	ascfrg			;Fragment Mitte rechts ausgeben
	pop	r20			;X Koordinate wieder herstellen
	ldi	r16, 11			;Y Koordinate fr untere
	add	r21, r16		;Fragmente ermitteln
	inc	r22			;ASCII-Wert nchstes Fragment
	rcall	ascfrg			;Fragment unten links ausgeben
	inc	r22			;ASCII-Wert nchstes Fragment
	rcall	ascfrg			;Fragment unten rechts ausgeben
	ret
;
; Unterprogramm zur Ausgabe eines Grozeichen-Fragments
;
ascfrg:	mov	r16, r22		;Zeichencode kopieren
	ldi	r17, 12			;Tabellen-Offset
	mul	r16, r17		;Tabellenplatz berechnen
	clr	r17
	ldi	zl, low(chrset * 2)	;Zeiger auf Zeichentabelle L
	ldi	zh, high(chrset * 2)	;Zeiger auf Zeichentabelle H
	ldi	r16, byte3(chrset * 2)	;Zeiger auf Zeichentabelle B3
	add	zl, r0			;Tabellenadresse L
	adc	zh, r1			;Tabellenadresse H
	adc	r16, r17		;Tabellenadresse B3
	out	rampz, r16
	elpm	r14, z+			;Zeichenbreite holen
	ldi	r23, 11			;11 Zeilen ausgeben
	mov	r19, r21		;Y-Koordinate kopieren
ascf10:	mov	r18, r20		;X-Koordinate kopieren
	elpm	r15, z+			;eine Zeichenzeile holen
	ldi	r24, 8			;Zhler fr Bitverschiebung
ascf20:	cp	r24, r14		;Zhler = Zeichenbreite?
	breq	ascf30			;ja -> Ausgabe beginnen
	rol	r15			;sonst Zeichenzeile verschieben
	dec	r24			;komplettes Byte verschoben?
	brne	ascf20			;nein -> Schleife
	rjmp	ascf40			;sonst keine weiteren Pixel
ascf30:	bst	r15, 7			;Bit aus Zeichenzeile holen
	bld	r19, 7			;und in Y-Koordinate einsetzen
	rcall	pixlcd			;Zeichenpixel ausgeben
	rol	r15			;nchstes Pixel bearbeiten
	inc	r18			;nchste X-Koordinate setzen
	dec	r24			;alle Pixel ausgegeben?
	brne	ascf30			;nein -> nchstes Pixel
ascf40:	andi	r19, 0x7f		;Y-Koordinate filtern
	inc	r19			;nchste Y-Koordinate setzen
	dec	r23			;alle Zeilen ausgegeben?
	brne	ascf10			;nein -> nchste Zeile
	add	r20, r14		;X-Koordinate fr nchstes
	ret				;Zeichen ermitteln, Ende
;
; Unterprogramm zum Lschen eines Pixelfensters auf dem LCD
; Register: r0, r1, r15, r16, r17
;	    r18 <- X-Koordinate Startpunkt (0-239)
;           r19 <- Y-Koordinate Startpunkt (0-127, Bit7 = 1 zum Setzen der Pixel)
;	    r20 <- X-Koordinate Endpunkt (0-239)
;	    r21 <- Y-Koordinate Endpunkt (0-127, Bit7 = 1 zum Setzen der Pixel)
;
clrwnd:	mov	r15, r18		;X-Startkoordinate sichern
clrw10:	mov	r18, r15		;X-Startkoordinate holen
clrw20:	rcall	pixlcd			;Pixel lschen
	inc	r18			;nchstes Pixel, gleiche Zeile
	cp	r18, r20		;X-Endwert erreicht?
	brcs	clrw20			;nein -> Schleife
	inc	r19			;sonst nchste Zeile
	cp	r19, r21		;Y-Endwert erreicht?
	brcs	clrw10			;nein -> Schleife
	ret
;
; Unterprogramm zum Zeichnen oder Lschen eines Pixels auf dem LCD
; Register: r0, r1, r16, r17, t
;	    r18 <- X-Koordinate (0-239)
;	    r19 <- Y-Koordinate (0-127, Bit7 = 1: setzen, Bit7 = 0: lschen)
;
pixlcd:	mov	r16, r19		;Y-Koordinate kopieren
	andi	r16, 0x7f		;Koordinatenbits filtern
	ldi	r17, 30			;mit Anzahl der Byte pro Zeile
	mul	r16, r17		;multiplizieren
	mov	r16, r18		;X-Koordinate kopieren
	ror	r16
	ror	r16
	ror	r16			;Wert durch 8 dividieren
	andi	r16, 0x1f		;relevante Bits filtern
	clr	r17			;H-Byte fr 16-Bit-Addition
	add	r0, r16			;Spaltenoffset addieren
	adc	r1, r17			;bertrag addieren
;
	mov	r17, r0			;berechnete Adresse Low
	clt				;LCD in Data Mode
	rcall	outlcd			;Byte ausgeben
	mov	r17, r1			;berechnete Adresse High
	rcall	outlcd			;Byte ausgeben
	ldi	r17, 0x24		;Address Pointer Set
	set				;LCD in Command Mode
	rcall	outlcd			;Byte ausgeben
	mov	r17, r18		;X-Koordinate kopieren
	com	r17			;Bitposition fr LCD ermitteln
	andi	r17, 0x07		;Pixel-Bitnummer filtern
	ori	r17, 0xf0		;Command-Bits ergnzen
	bst	r19, 7			;Setzen/Lschen-Bit holen
	bld	r17, 3			;und in Command einsetzen
	set				;LCD in Command Mode
	rcall	outlcd			;Byte ausgeben
	ret
;
; Unterprogramm zur Ausgabe eines Bytes auf das LCD, vorher Flag prfen, ob LCD eingeschaltet ist
; Register: r16
;	    r17    <- Datenbyte
;	    t-flag <- 0 = Data, 1 = Command
;
outlcd:	lds	r16, dflags		;Anzeige-Flags holen
	andi	r16, 0x80		;Anzeige abgeschaltet?
	breq	outl10			;nein -> Byte ausgeben
	ret				;sonst Ende
;
outl10:	cbi	portd, 7		;Testausgabe
;
	cbi	portc, 2		;/CE auf Low
	cbi	portc, 1		;/RD auf Low
	nop
	nop				;warten (mindestens 150ns)
outl20:	in	r16, pina		;LCD-Datenbyte lesen
	andi	r16, 0x03		;STA0 und STA1 filtern
	cpi	r16, 0x03		;STA0 und STA1 gesetzt?
	brne	outl20			;nein -> Abfrage wiederholen
	sbi	portc, 1		;/RD wieder auf High
	brts	outl30			;Command? ja -> weiter
	cbi	portc, 3		;sonst C/D auf Low (= Data)
outl30:	out	porta, r17		;Byte auf LCD-Port ausgeben
	ser	r16
	out	ddra, r16		;LCD-Datenport auf Ausgang
	cbi	portc, 0		;/WR auf Low
	sbi	portc, 0		;/WR wieder auf High
	out	porta, r16		;Pull-up einschalten
	clr	r16
	out	ddra, r16		;LCD-Datenport auf Eingang
	sbi	portc, 2		;/CE wieder auf High
	sbi	portc, 3		;C/D wieder auf High
;
	sbi	portd, 7		;Testausgabe
;
	ret
;
; Unterprogramm zum Einlesen eines Datenbytes aus dem LCD, vorher Flag prfen, ob LCD eingeschaltet ist
; Register: r16
;	    r17 -> Datenbyte
;
in_lcd:	lds	r16, dflags		;Anzeige-Flags holen
	andi	r16, 0x80		;Anzeige abgeschaltet?
	breq	in_l10			;nein -> Byte einlesen
	ret				;sonst Ende
;
in_l10:	cbi	portd, 7		;Testausgabe
;
	cbi	portc, 2		;/CE auf Low
	cbi	portc, 1		;/RD auf Low
	nop
	nop				;warten (mindestens 150ns)
in_l20:	in	r16, pina		;LCD-Datenbyte lesen
	andi	r16, 0x03		;STA0 und STA1 filtern
	cpi	r16, 0x03		;STA0 und STA1 gesetzt?
	brne	in_l20			;nein -> Abfrage wiederholen
	sbi	portc, 1		;sonst /RD wieder auf High
	cbi	portc, 3		;C/D auf Low (= Data)
	nop				;90ns warten
	cbi	portc, 1		;/RD auf Low
	nop
	nop				;warten (mindestens 150ns)
	in	r17, pina		;LCD-Datenbyte lesen
	sbi	portc, 1		;/RD wieder auf High
	sbi	portc, 2		;/CE wieder auf High
	sbi	portc, 3		;C/D wieder auf High
;
	sbi	portd, 7		;Testausgabe
;
	ret
;
; Unterprogramm zum Lschen des LCD Grafikspeichers
; Register: r16, r17, r18, r19, t
;
clrlcd:	ldi	r17, 0			;Pointer auf Anfangsposition
	clt				;LCD in Data Mode
	rcall	outlcd			;Byte ausgeben
	rcall	outlcd			;Byte ausgeben
	ldi	r17, 0x24		;Address Pointer Set
	set				;LCD in Command Mode
	rcall	outlcd			;Byte ausgeben
	ldi	r19, 128		;Zeilenzhler setzen
clrl10:	ldi	r18, 30			;Spaltenzhler setzen
clrl20:	clr	r17			;0-Werte schreiben
	clt				;LCD in Data Mode
	rcall	outlcd			;Byte ausgeben
	ldi	r17, 0xc0		;Datawrite Auto Increment
	set				;LCD in Command Mode
	rcall	outlcd			;Byte ausgeben
	dec	r18			;alle Spalten ausgegeben?
	brne	clrl20			;nein -> Schleife
	dec	r19			;alle Zeilen ausgegeben?
	brne	clrl10			;nein -> Schleife
	ret
;
; Unterprogramm zum Holen eines Zeichens aus der Zeichenliste entsprechend der Position
; Register: r17, z
;           r16 <- Position
;           r16 -> ASCII-Zeichen
;
getchr: clr     r17                     ;H-Byte = 0
        ldi     zl, byte3(chrlst * 2)
        out     rampz, zl
        ldi     zl, low(chrlst * 2)
        ldi     zh, high(chrlst * 2)    ;Zeiger auf Zeichenliste
        add     zl, r16                 ;Position addieren
        adc     zh, r17                 ;bertrag
        elpm    r16, z                  ;Zeichen holen
        ret
;
; Unterprogramm zum Erzeugen der Daten fr die grafische Luftdruckanzeige (Nutzung von xerbuf)
; Register: r14, r15, r16, r17, r18, r19, r20, r21, r22, r23, r24, r25, x, y, z
;
; zunchst 34 Datenstze (15min-Werte) vom aktuellen Zeitpunkt an im Abstand von 1h einlesen
;
genprs: lds	r20, xes1po
	lds	r21, xes1po + 1		;aktuellen Zeiger der 15min-Werte holen,
	lds	r22, xes1po + 2		;zeigt auf ltesten Datensatz
        subi    r20, xes1da
        sbci    r21, 0
        sbci    r22, 0                  ;Zeiger auf letzten Datensatz setzen
        brcc    gen010                  ;bertrag? nein -> weiter
        subi    r20, low(-xs1len)
        sbci    r21, high(-xs1len)      ;sonst
        sbci    r22, byte3(-xs1len)     ;Zeiger korrigieren
;
gen010: ldi     r24, 34                 ;34 Datenstze im Stundenabstand lesen
        sts     xerbuf + 51, r20
        sts     xerbuf + 52, r21
        sts     xerbuf + 53, r22        ;Zeiger auf letzten Datensatz speichern
        ldi     zl, low(xerbuf + 54)
        ldi     zh, high(xerbuf + 55)   ;Zeiger auf Druckwerte setzen
;
gen020: lds     r20, xerbuf + 51
        lds     r21, xerbuf + 52
        lds     r22, xerbuf + 53        ;Datensatzzeiger holen
        ldi     r23, xes1da             ;51 Bytes lesen (1 Datensatz)
	call	xread			;Datensatz aus Ext-EEPROM lesen
	brcc	gen030			;Fehler? nein -> weiter
	call	twierr			;sonst Speicherfehler setzen
	ret			        ;Ende
;
gen030:	lds	r16, xerbuf		;Zeitbyte 0 holen
	lds	r17, xerbuf + 1		;Zeitbyte 1 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 2		;Zeitbyte 2 holen
	and	r16, r17		;verknpfen
	lds	r17, xerbuf + 3		;Zeitbyte 3 holen
	and	r16, r17		;verknpfen
	cpi	r16, 0xff		;gltiger Datenssatz?
        breq    gen040                  ;nein -> weiter
;
        lds     r16, xerbuf + 44        ;Luftdruckwert Low holen
        lds     r17, xerbuf + 45        ;Luftdruckwert High holen
        swap    r17                     ;High-Wert an richtige Position bringen
        andi    r17, 0x0f               ;und filtern
        rjmp    gen050
;
gen040: clr     r16                     ;bei ungltigem Datensatz ->
        ldi     r17, 0x08               ;Wert lschen
gen050: st      z+, r16                 ;Druckwert Low speichern
        st      z+, r17                 ;Druckwert High speichern
;
        lds     r20, xerbuf + 51
        lds     r21, xerbuf + 52
        lds     r22, xerbuf + 53        ;Datensatzzeiger holen
        subi    r20, low(xes1da * 4)
        sbci    r21, high(xes1da * 4)
        sbci    r22, byte3(xes1da * 4)  ;Zeiger auf Datensatz eine Stunde zuvor setzen
        brcc    gen060                  ;bertrag? nein -> weiter
        subi    r20, low(-xs1len)
        sbci    r21, high(-xs1len)      ;sonst
        sbci    r22, byte3(-xs1len)     ;Zeiger korrigieren
gen060: sts     xerbuf + 51, r20
        sts     xerbuf + 52, r21
        sts     xerbuf + 53, r22        ;neuen Zeiger speichern
        dec     r24                     ;alle Datenstze gelesen?
        brne    gen020                  ;nein -> Schleife
;
; ersten gltigen Wert finden, anschlieend Min/Max-Werte bis zu einer Differenz von maximal 10 ermitteln (entspricht der Pixelhhe der Minigrafik)
;
        ldi     r24, 34                 ;34 Werte fr grafischen Luftdruck
        ldi     zl, low(xerbuf + 54)
        ldi     zh, high(xerbuf + 55)   ;Zeiger auf Druckwerte setzen
;
gen070: ld      r16, z+                 ;Druckwert Low lesen
        ld      r17, z+                 ;Druckwert High lesen
        cpi     r17, 0x08               ;Wert gltig?
        brne    gen090                  ;ja -> Startwert gefunden
        dec     r24                     ;sonst alle Werte bearbeitet?
        brne    gen070                  ;nein -> Schleife
;
        ldi     r24, 17                 ;sonst alle 17 Grafikwertepaare
        ldi     r16, 0x55               ;auf Mittellinie setzen (2 mal Wert = 5)
        ldi     yl, low(prstab)
        ldi     yh, high(prstab)        ;Zeiger auf Luftdrucktabelle
gen080: st      y+, r16                 ;vorbereitetes Wertepaar speichern
        dec     r24                     ;alle Wertepaare bearbeitet?
        brne    gen080                  ;nein -> Schleife
        ret
;
gen090: movw    r14, r16                ;Startwert speichern
        movw    r18, r16                ;Minimalwert voreinstellen
        movw    r20, r16                ;Maximalwert voreinstellen
        clr     r25                     ;Differenz = 0 setzen
;
gen100: cp      r16, r18
        cpc     r17, r19                ;wurde der Minimalwert unterschritten?
        brcc    gen120                  ;nein -> weiter
        movw    r22, r18                ;sonst Minimalwert kopieren
        sub     r22, r16
        sbc     r23, r17                ;aktuellen Wert subtrahieren
        add     r22, r25                ;bisherige Differenz addieren
        cpi     r22, 11                 ;wrde die maximale Differenz berschritten?
        brcs    gen110                  ;nein -> weiter
;
        movw    r18, r20                ;sonst Maximalwert kopieren
        subi    r18, low(10)            ;und den Minimalwert auf
        sbci    r19, high(10)           ;10 Einheiten unter den Maximalwert setzen
        rjmp    gen150                  ;Daten fr Diagramm erzeugen
;
gen110: mov     r25, r22                ;neue Differenz speichern
        movw    r18, r16                ;neuen Minimalwert speichern
;
gen120: cp      r20, r16
        cpc     r21, r17                ;wurde der Maximalwert berschritten
        brcc    gen140                  ;nein -> weiter
        movw    r22, r16                ;sonst aktuellen Wert kopieren
        sub     r22, r20
        sbc     r23, r21                ;Maximalwert subtrahieren
        add     r22, r25                ;bisherige Differenz addieren
        cpi     r22, 11                 ;wrde die maximale Differenz berschritten?
        brcs    gen130                  ;nein -> weiter
;
        movw    r20, r18                ;sonst Minimalwert kopieren
        subi    r20, low(-10)           ;und den Maximalwert auf
        sbci    r21, high(-10)          ;10 Einheiten ber den Minimalwert setzen
        rjmp    gen150                  ;Daten fr Diagramm erzeugen
;
gen130: mov     r25, r22                ;neue Differenz speichern
        movw    r20, r16                ;neuen Maximalwert speichern
;
gen140: dec     r24                     ;alle Werte bearbeitet?
        breq    gen150                  ;ja -> Daten fr Diagramm erzeugen
;
        ld      r16, z+                 ;nchsten Druckwert Low lesen
        ld      r17, z+                 ;nchsten Druckwert High lesen
        cpi     r17, 0x08               ;Wert gltig?
        brne    gen100                  ;ja -> bearbeiten
        rjmp    gen140                  ;sonst berspringen
;
gen150: movw    r22, r18                ;jetzt Mittelwert berechnen
        add     r22, r20
        adc     r23, r21                ;Minimal- und Maximalwert addieren
        lsr     r23
        ror     r22                     ;und halbieren -> Mittelwert
        subi    r22, low(5)
        sbci    r23, high(5)            ;Differenz zur Mittellinie ermitteln
        movw    r18, r22                ;und als Bezugswert verwenden
;
; Daten fr Diagramm erzeugen - Luftdruckwerte werden in Pixelwerte (0-10) umgewandelt und jeweils 2 Werte in einem Tabellen-Byte gespeichert
;
        sub     r14, r18                ;Tabellenwert fr fehlende Daten ermitteln
        sbc     r15, r19                ;(entspricht Pixelwert des Startwertes)
        mov     r22, r14                ;und kopieren
;
        ldi     r24, 34                 ;34 Werte fr grafischen Luftdruck
        ldi     zl, low(xerbuf + 54)
        ldi     zh, high(xerbuf + 55)   ;Zeiger auf Druckwerte setzen
        ldi     yl, low(prstab + 17)
        ldi     yh, high(prstab + 17)   ;Zeiger auf Ende der Luftdruck-Pixelwerttabelle
;
gen200: ld      r16, z+
        ld      r17, z+                 ;Luftdruckwert holen
        cpi     r17, 0x08               ;Wert ungltig?
        brne    gen210                  ;nein -> berspringen
        mov     r16, r22                ;sonst durch vorherigen Pixelwert ersetzen
        rjmp    gen230
;
gen210: sub     r16, r18                ;Pixelwert ermitteln
        sbc     r17, r19                ;Pixelwert unterhalb des Grafikbereichs?
        brcs    gen220                  ;ja -> Pixelwert auf 0 setzen
        cpi     r16, 11                 ;Pixelwert oberhalb des Grafikbereichs?
        brcs    gen230                  ;nein -> weiter
        ldi     r16, 10                 ;sonst Pixelwert auf 10 setzen
        rjmp    gen230
gen220: clr     r16                     ;Pixelwert auf 0 setzen
;
gen230: mov     r22, r16                ;neuen Pixelwert fr fehlende Luftdruckwerte speichern
        bst     r24, 0                  ;erster Wert des Paares?
        brts    gen240                  ;nein -> weiter
        mov     r20, r16                ;sonst Wert zwischenspeichern
        rjmp    gen250
;
gen240: swap    r20                     ;vorherigen Wert in oberes Nibble legen
        or      r16, r20                ;mit aktuellem Wert verknpfen
        st      -y, r16                 ;und speichern
gen250: dec     r24                     ;alle Werte bearbeitet?
        brne    gen200                  ;nein -> Schleife
        ret
;
; Unterprogramm (Grafikanzeige) zum Lschen des Grafik-Datenpuffers, Initialisieren des Pointers, des Zeit-Zwischenspeichers und einiger Variablen
; Register: r16, r17, r18, r19, x
;
grclrb:	ldi	xl, low(grbuff)
	ldi	xh, high(grbuff)	;Zeiger auf Grafik-Datenpuffer
	ldi	r16, 0
	ldi	r17, 0x80		;ungltiger Sensorwert
	ldi	r18, 0xff		;Wert fr ungltige Zeitdaten
	ldi	r19, 222		;222 Datenstze initialisieren
grcl10:	st	x+, r18
	st	x+, r18
	st	x+, r18			;Datum/Zeit ungltig machen
	st	x+, r18			;(alle Bytes 0xff)
	st	x+, r16
	st	x+, r17			;Tempwert 1 lschen (0x8000)
	st	x+, r16
	st	x+, r17			;Tempwert 2 lschen (0x8000)
	dec	r19			;alle Datenstze bearbeitet?
	brne	grcl10			;nein -> Schleife
;
	sts	grcdat, r18
	sts	grcdat + 1, r18
	sts	grcdat + 2, r18		;Zeit-Zwischenspeicher
	sts	grcdat + 3, r18		;initialisieren
	sts	grsca1, r16		;Y-Skalierung 1 initialisieren
	sts	grsca2, r16		;Y-Skalierung 2 initialisieren
	sts	grcen1, r16		;Y-Mittelpunktwert
	sts	grcen1 + 1, r16		;fr Diagramm 1 initialisieren
	sts	grcen2, r16		;Y-Mittelpunktwert
	sts	grcen2 + 1, r16		;fr Diagramm 2 initialisieren
;
	lds	r17, xflag3		;verschiedene Flags 3 holen
	bst	r17, 3			;nchste Seite anzeigen?
	brtc	grcl20			;ja -> Zeiger ok
	ldi	r16, 221		;sonst Zeiger auf Puffer-Ende
grcl20:	sts	grpoin, r16		;Grafikpuffer-Zeiger speichern
	ret
;
; Unterprogramm (Grafikanzeige) zum Ermitteln der Adresse des vorherigen EEPROM-Datensatzes, ohne Test auf Erreichen des Log-Anfangs
; Register: r16, r17, r18, r19
;	    r20, r21, r22 <- EEPROM-Zeiger fr Grafikpuffer
;
grdecx:	lds	r19, grmod1		;Grafikanzeige-Modus holen
	tst	r19			;15-Minuten-Werte anzeigen?
	breq	grde20			;ja -> bearbeiten
	rjmp	grde60			;sonst 24h-Werte bearbeiten

; Unterprogramm (Grafikanzeige) zum Ermitteln der Adresse des vorherigen EEPROM-Datensatzes, vorher Test auf Erreichen des Log-Anfangs
; Register: r16, r17, r18, r19, r20, r21, r22
;
grdeca:	lds	r20, grptem
	lds	r21, grptem + 1		;EEPROM-Zeiger fr Grafikpuffer
	lds	r22, grptem + 2		;holen
	lds	r19, grmod1		;Grafikanzeige-Modus holen
	tst	r19			;15-Minuten-Werte anzeigen?
	brne	grde50			;nein -> weiter
;
	lds	r16, xes1po		;aktuellen EEPROM-Zeiger auf
	lds	r17, xes1po + 1		;Sensordaten 1 holen, zeigt
	lds	r18, xes1po + 2		;auf ltesten Eintrag
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;erster Datensatz erreicht?
	brne	grde20			;nein -> weiter
grde10:	lds	r19, xflag3		;sonst verschiedene Flags holen
	sbr	r19, 1 << 5		;erster Datensatz erreicht
	sts	xflag3, r19		;Flags wieder speichern
	ret
;
grde20:	ldi	r16, low(xs1sta)
	ldi	r17, high(xs1sta)	;Anfangsadresse Sensordaten
	ldi	r18, byte3(xs1sta)	;15min laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Anfangsadresse erreicht?
	brne	grde30			        ;nein -> weiter
	ldi	r20, low(xs1sta + xs1len)	;sonst
	ldi	r21, high(xs1sta + xs1len)	;End-Adresse Sensordaten
	ldi	r22, byte3(xs1sta + xs1len)     ;15min laden
grde30:	subi	r20, xes1da
	sbci	r21, 0			;Zeiger auf vorherige Position
	sbci	r22, 0			;setzen (zeitlich davor)
	sts	grptem, r20
	sts	grptem + 1, r21		;neuen EEPROM-Zeiger fr den
	sts	grptem + 2, r22		;Grafikpuffer speichern
	ret
;
grde50:	lds	r16, xes2po		;aktuellen EEPROM-Zeiger auf
	lds	r17, xes2po + 1		;Sensordaten 1 holen,
	lds	r18, xes2po + 2		;zeigt auf ltesten Eintrag
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;erster Datensatz erreicht?
	breq	grde10			;ja -> Ende
;
grde60:	ldi	r16, low(xs2sta)
	ldi	r17, high(xs2sta)	;Anfangsadresse Sensordaten
	ldi	r18, byte3(xs2sta)	;24h laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Anfangsadresse erreicht?
	brne	grde70			        ;nein -> weiter
	ldi	r20, low(xs2sta + xs2len)	;sonst
	ldi	r21, high(xs2sta + xs2len)	;End-Adresse Sensordaten
	ldi	r22, byte3(xs2sta + xs2len)     ;24h laden
grde70:	subi	r20, xes2da
	sbci	r21, 0			;Zeiger auf vorherige Position
	sbci	r22, 0			;setzen (zeitlich davor)
	sts	grptem, r20
	sts	grptem + 1, r21		;neuen EEPROM-Zeiger fr den
	sts	grptem + 2, r22		;Grafikpuffer speichern
	ret
;
; Unterprogramm (Grafikanzeige) zum Ermitteln der Adresse des nchsten EEPROM-Datensatzes
; Register: r16, r17, r18, r19, r20, r21, r22
;
grinca:	lds	r20, grptem
	lds	r21, grptem + 1		;EEPROM-Zeiger fr Grafikpuffer
	lds	r22, grptem + 2		;holen
	clr	r17			;Offset zum nchsten Datensatz H
	lds	r19, grmod1		;Grafikanzeige-Modus holen
	tst	r19			;15-Minuten-Werte anzeigen?
	brne	grin50			;nein -> weiter
;
	ldi	r16, xes1da		;Offset zum nchsten Datensatz L
	add	r20, r16
	adc	r21, r17		;Zeiger auf nchsten Eintrag
	adc	r22, r17		;setzen (zeitlich danach)
	sts	grptem, r20
	sts	grptem + 1, r21		;neuen EEPROM-Zeiger fr
	sts	grptem + 2, r22		;Grafikpuffer speichern
;
	ldi	r16, low(xs1sta + xs1len)
	ldi	r17, high(xs1sta + xs1len)	;Endadresse Sensordaten
	ldi	r18, byte3(xs1sta + xs1len)     ;15min laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	grin10			;nein -> weiter
	ldi	r20, low(xs1sta)	;sonst
	ldi	r21, high(xs1sta)	;Anfangsadresse Sensordaten
	ldi	r22, byte3(xs1sta)	;15min laden
	sts	grptem, r20
	sts	grptem + 1, r21		;neuen EEPROM-Zeiger fr
	sts	grptem + 2, r22		;Grafikpuffer speichern
;
grin10:	lds	r16, xes1po		;aktuellen EEPROM-Zeiger auf
	lds	r17, xes1po + 1		;Sensordaten 1 holen, zeigt
	lds	r18, xes1po + 2		;auf letzten Listeneintrag
grin20:	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;letzter Datensatz erreicht?
	brne	grin30			;nein -> weiter
	lds	r19, xflag3		;sonst verschiedene Flags holen
	sbr	r19, 1 << 4		;letzter Datensatz erreicht
	sts	xflag3, r19		;Flags wieder speichern
grin30:	ret
;
grin50:	ldi	r16, xes2da		;Offset zum nchsten Datensatz L
	add	r20, r16
	adc	r21, r17		;Zeiger auf nchsten Eintrag
	adc	r22, r17		;setzen (zeitlich danach)
	sts	grptem, r20
	sts	grptem + 1, r21		;neuen EEPROM-Zeiger fr
	sts	grptem + 2, r22		;Grafikpuffer speichern
;
	ldi	r16, low(xs2sta + xs2len)
	ldi	r17, high(xs2sta + xs2len)	;Endadresse Sensordaten
	ldi	r18, byte3(xs2sta + xs2len)     ;24h laden
	sub	r16, r20
	sbc	r17, r21
	sbc	r18, r22		;Endadresse erreicht?
	brne	grin60			;nein -> weiter
	ldi	r20, low(xs2sta)	;sonst
	ldi	r21, high(xs2sta)	;Anfangsadresse Sensordaten
	ldi	r22, byte3(xs2sta)	;24h laden
	sts	grptem, r20
	sts	grptem + 1, r21		;neuen EEPROM-Zeiger fr
	sts	grptem + 2, r22		;Grafikpuffer speichern
;
grin60:	lds	r16, xes2po		;aktuellen EEPROM-Zeiger auf
	lds	r17, xes2po + 1		;Sensordaten 2 holen, zeigt
	lds	r18, xes2po + 2		;auf letzten Listeneintrag
	rjmp	grin20
;
; Unterprogramm (Grafikanzeige) zum Ermitteln des vorherigen Log-Zeitpunktes
; Register: r16, r17, r18, r19, r20, r21
;
grprev:	rcall	grexti			;Zeitdaten extrahieren
	lds	r21, grmod1		;Grafikanzeige-Modus holen
	tst	r21			;15-Minuten-Werte anzeigen?
	brne	grpr10			;nein -> weiter
;
	subi	r16, 15			;Minute -15
	brcc	grpr30			;bertrag? nein -> Ende
	ldi	r16, 45			;sonst Minute = 45 setzen
	subi	r17, 1			;Stunde -1
	brcc	grpr30			;bertrag? nein -> Ende
	ldi	r17, 23			;sonst Stunde = 23 setzen
;
grpr10:	dec	r18			;Tag -1
	brne	grpr30			;bertrag? nein -> Ende
	dec	r19			;sonst Monat -1
;
	ldi	r18, 31			;Tag = 31 setzen
	cpi	r19, 1			;Monat Januar?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 3			;Monat Mrz?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 5			;Monat Mai?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 7			;Monat Juli?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 8			;Monat August?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 10			;Monat Oktober?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 0			;Monat Dezember?
	breq	grpr20			;ja -> Tag ok
;
	ldi	r18, 30			;Tag = 30 setzen
	cpi	r19, 4			;Monat April?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 6			;Monat Juni?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 9			;Monat September?
	breq	grpr20			;ja -> Tag ok
	cpi	r19, 11			;Monat November?
	breq	grpr20			;ja -> Tag ok
;
	ldi	r18, 29			;Tag = 29 setzen
	mov	r21, r20		;sonst Jahr kopieren
	andi	r21, 0x03		;Jahr durch 4 teilbar?
	breq	grpr20			;ja -> Tag ok
	ldi	r18, 28			;sonst Tag= 28 setzen
;
grpr20:	cpi	r19, 0			;Monat =0 (bertrag)?
	brne	grpr30			;nein -> Ende
	ldi	r19, 12			;sonst Monat = 12 setzen
	dec	r20			;Jahr -1
;
grpr30:	rcall	grcpti			;Zeitdaten wieder komprimieren
	ret
;
; Unterprogramm (Grafikanzeige) zum Ermitteln des nchsten Log-Zeitpunktes
; Register: r16, r17, r18, r19, r20, r21
;
grnext:	rcall	grexti			;Zeitdaten extrahieren
	lds	r21, grmod1		;Grafikanzeige-Modus holen
	tst	r21			;15-Minuten-Werte anzeigen?
	brne	grne10			;nein -> weiter
;
	ldi	r21, 15			;Minutenabstand = 15
	add	r16, r21		;Minute + 15
	cpi	r16, 60			;bertrag?
	brcs	grne70			;nein -> Ende
	ldi	r16, 0			;sonst Minute = 0 setzen
	inc	r17			;Stunde +1
	cpi	r17, 24			;bertrag?
	brcs	grne70			;bertrag? nein -> Ende
	ldi	r17, 0			;sonst Stunde = 0 setzen
;
grne10:	inc	r18			;Tag + 1
	cpi	r19, 1			;Monat Januar?
	breq	grne20			;ja -> weiter (31 Tage)
	cpi	r19, 2			;Monat Februar?
	breq	grne40			;ja -> weiter (28/29 Tage)
	cpi	r19, 3			;Monat Mrz?
	breq	grne20			;ja -> weiter (31 Tage)
	cpi	r19, 4			;Monat April?
	breq	grne30			;ja -> weiter (30 Tage)
	cpi	r19, 5			;Monat Mai?
	breq	grne20			;ja -> weiter (31 Tage)
	cpi	r19, 6			;Monat Juni?
	breq	grne30			;ja -> weiter (30 Tage)
	cpi	r19, 7			;Monat Juli?
	breq	grne20			;ja -> weiter (31 Tage)
	cpi	r19, 8			;Monat August?
	breq	grne20			;ja -> weiter (31 Tage)
	cpi	r19, 9			;Monat September?
	breq	grne30			;ja -> weiter (30 Tage)
	cpi	r19, 10			;Monat Oktober?
	breq	grne20			;ja -> weiter (31 Tage)
	cpi	r19, 11			;Monat November?
	breq	grne30			;ja -> weiter (30 Tage)
;
grne20:	cpi	r18, 32			;bertrag (Tag = 32)?
	rjmp	grne60			;Test auf Monatswechsel
;
grne30:	cpi	r18, 31			;bertrag (Tag = 31)?
	rjmp	grne60			;Test auf Monatswechsel
;
grne40:	mov	r21, r20		;Jahr kopieren
	andi	r21, 0x03		;Jahr durch 4 teilbar?
	breq	grne50			;ja -> weiter
	cpi	r18, 29			;berlauf (Tag = 29)?
	rjmp	grne60			;Test auf Monatswechsel
;
grne50:	cpi	r18, 30			;berlauf (Tag = 30)?
grne60:	brcs	grne70			;berlauf? nein -> Ende
	ldi	r18, 1			;sonst Tag = 1 setzen
	inc	r19			;Monat + 1
	cpi	r19, 13			;berlauf?
	brcs	grne70			;nein -> Ende	
	ldi	r19, 1			;sonst Monat = 1 setzen
	inc	r20			;Jahr + 1
grne70:	rcall	grcpti			;Zeitdaten wieder komprimieren
	ret
;
; Unterprogramm (Grafikanzeige) zum Extrahieren der Zeitdaten
; Register: r16, r17, r18, r19, r20 -> extrahierte Zeit (min, h, d, mon, y)
;
grexti:	lds	r20, grcdat + 3		;komprimierte Zeit Byte 3 holen
	bst	r20, 0			;LSB sichern, gehrt zum Monat
	lsr	r20			;Jahr an Position schieben
	lds	r19, grcdat + 2		;komprimierte Zeit Byte 2 holen
	ror	r19			;Monat nach rechts schieben
	bld	r19, 7			;LSB von Byte 3 holen, Monat an
	swap	r19			;richtige Position schieben
	andi	r19, 0x0f		;nicht relevante Bits filtern
	lds	r18, grcdat + 2		;komprimierte Zeit Byte 2 holen
	andi	r18, 0x1f		;nicht relevante Bits filtern
	lds	r17, grcdat + 1		;Stunde speichern
	lds	r16, grcdat		;Minute speichern
	ret
;
; Unterprogramm (Grafikanzeige) zum Komprimieren der Zeitdaten
; Register: r16, r17, r18, r19, r20 <- komprimierte Zeit (min, h, d, mon, y)
;
grcpti:	sts	grcdat, r16		;Minuten speichern
	sts	grcdat + 1, r17		;Stunde speichern
	swap	r19			;Monat, Nibbles tauschen
	bst	r19, 7			;MSB sichern, kommt in Byte 3
	lsl	r19			;Wert verschieben und
	or	r19, r18		;mit Tag verknpfen
	sts	grcdat + 2, r19		;Byte 2 speichern
	lsl	r20			;Jahr, Wert verschieben
	bld	r20, 0			;MSB vom Monat ergnzen
	sts	grcdat + 3, r20		;Byte 3 speichern
	ret
;
; Unterprogramm (Grafikanzeige) zum Suchen der Log-Position eines Sensors in der Log-Tabelle 1
; Register: r17, x
;	    r16 <- Sensornummer
;	    r16 -> gesuchte Log-Position
;
grslg1:	ldi	xl, low(logtb1)
	ldi	xh, high(logtb1)	;Zeiger auf Log-Tabelle 1
        rjmp    grsl10
;
; Unterprogramm (Grafikanzeige) zum Suchen der Log-Position eines Sensors in der Log-Tabelle 2
; Register: r17, x
;	    r16 <- Sensornummer
;	    r16 -> gesuchte Log-Position
;
grslg2:	ldi	xl, low(logtb2)
	ldi	xh, high(logtb2)	;Zeiger auf Log-Tabelle 2
grsl10:	clr	r17			;H-Byte fr Berechnung
	add	xl, r16
	adc	xh, r17			;Tabellenplatz berechnen
	ld	r16, x			;Log-Position holen
        andi    r16, 0x1f               ;Log-Position filtern
	ret
;
; Unterprogramm (Grafikanzeige) zum Extrahieren eines Sensorwertes aus einem EEPROM-Datensatz
; Register: r0, r1, r18, x, y, t
;	    r16      <- Sensornummer (0-30)
;           r17      <- Grafikmodus (0 = 15min, 1 = 24h-Min, 2 = 24h-Max, 3 = 24h-Avg)
;	    r16, r17 -> Sensorwert im Zweierkomplement
;
grgsen:	ldi	yl, low(xedbuf + 4)
	ldi	yh, high(xedbuf + 4)	;Adresse der Sensordaten
	tst	r17			;15-Minuten-Werte anzeigen?
	breq	grg010			;ja -> weiter
	dec	r17			;sonst Anzeigemodus korrigieren
	ldi	r18, 47			;Offset fr Wertebereich
	mul	r17, r18		;Adressoffset berechnen
	add	yl, r0
	adc	yh, r1			;Zeiger auf Min/Max/Mittelwerte
;
grg010: cpi     r16, 24                 ;Sensornummer < 24 (Temperatur, Log-Gruppe 1)?
        brcc    grg100                  ;nein -> weiter testen
        rcall   grslg1                  ;Log-Position Gruppe 1 ermitteln
;
grg020:	bst	r16, 0			;LSB der Nummer sichern
	lsr	r16			;Nummer halbieren
	ldi	r17, 3			;Wertepaare jeweils 3 Byte
	mul	r16, r17		;Offset vom Wertepaar berechnen
	add	yl, r0
	adc	yh, r1			;Adresse vom Wertepaar ermitteln
;
	ld	r16, y			;L-Byte von Wert 1 holen
	ldd	r17, y + 2		;H-Byte von beiden Werten holen
	brtc	grg030			;erster Wert? ja -> weiter
	ldd	r16, y + 1		;sonst L-Byte von Wert 2 holen
	swap	r17			;H-Byte an richtige Stelle legen
grg030:	andi	r17, 0x0f		;relevante Bits filtern
	cpi	r17, 0x08		;Wert ungltig?
	brne	grg040			;nein -> weiter
	swap	r17			;sonst H-Byte auf 0x80 setzen
	ret
grg040:	brcs	grg050			;Wert positiv? ja -> weiter
	ori	r17, 0xf0		;sonst H-Byte korrigieren
grg050: ret
;
grg100: cpi     r16, 27                 ;Sensornummer < 27 (Temperatur, Log-Gruppe 2)?
        brcc    grg200                  ;nein -> weiter testen
        subi    r16, 24                 ;Sensornummer korrigieren
        adiw    yl, 36                  ;Adresse auf Sensordaten 24-27
        rcall   grslg2                  ;Log-Position Gruppe 2 ermitteln
        rjmp    grg020
;
grg200: cpi     r16, 28                 ;Sensornummer < 28 (Luftdruck)?
        brcc    grg300                  ;nein -> weiter testen
        adiw    yl, 36                  ;Adresse auf Sensordaten 24-27
        ldi     r16, 3                  ;feste Log-Position
        rjmp    grg020
;
grg300: cpi     r16, 31                 ;Sensornummer < 31 (Luftfeuchtigkeit, Log-Gruppe 2)
        brcc    grg400                  ;nein -> weiter
        adiw    yl, 42                  ;Adresse auf Sensordaten 28-30
        subi    r16, 28                 ;Sensornummer korrigieren
        rcall   grslg2                  ;Log-Position Gruppe 2 ermitteln
        clr     r17                     ;H-Byte = 0
        add     yl, r16
        adc     yh, r17                 ;Adresse des Feuchtigkeitssensors ermitteln
        ld      r16, y                  ;Sensorwert holen
        cpi     r16, 0x80               ;Wert ungltig?
        brne    grg310                  ;nein -> weiter
        mov     r17, r16                ;Kennung an richtige Stelle kopieren
        clr     r16                     ;L-Byte = 0 setzen
        ret
grg310: ldi     r17, 10                 ;Multiplikator 10
        mul     r16, r17                ;Wert verzehnfachen
        movw    r16, r0                 ;und kopieren
        ret
;
grg400: adiw    yl, 45                  ;Adresse auf Batteriewert
        ld      r16, y+                 ;L-Byte holen
        ld      r17, y                  ;H-Byte holen
        andi    r17, 0x01               ;relevantes Bits filtern
        ret
;
; Unterprogramm (Grafikanzeige) zum Ablegen eines gelesenen Sensorwertes in den Grafik-Datenpuffer als Sensorwert 1
; Register: r0, r1, r18, r19, z
;	    r16, r17 <- Sensorwert im Zweierkomplement
;
grbts1:	lds	r18, grpoin		;Zeiger holen
	ldi	r19, 8			;Pufferplatz 8 Bytes gro
	mul	r18, r19		;Puffer-Offset berechnen
	ldi	zl, low(grbuff)
	ldi	zh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	add	zl, r0
	adc	zh, r1			;Datenpuffer-Adresse berechnen
	std	z+4, r16
	std	z+5, r17		;Sensorwert 1 speichern
	lds	r16, grcdat
	lds	r17, grcdat + 1
	lds	r18, grcdat + 2
	lds	r19, grcdat + 3		;Zeit aus Zwischenspeicher holen
	st	z, r16
	std	z + 1, r17
	std	z + 2, r18
	std	z + 3, r19		;Zeit in Datenpuffer speichern
	ret
;
; Unterprogramm (Grafikanzeige) zum Ablegen eines gelesenen Sensorwertes in den Grafik-Datenpuffer als Sensorwert 2
; Register: r0, r1, r18, r19, z
;	    r16, r17 <- Sensorwert im Zweierkomplement
;
grbts2:	lds	r18, grpoin		;Zeiger holen
	ldi	r19, 8			;Pufferplatz 8 Bytes gro
	mul	r18, r19		;Puffer-Offset berechnen
	ldi	zl, low(grbuff)
	ldi	zh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	add	zl, r0
	adc	zh, r1			;Datenpuffer-Adresse berechnen
	std	z + 6, r16
	std	z + 7, r17		;Sensorwert 2 speichern
	ret				;Zeitdaten bereits im Puffer
;
; Unterprogramm (Grafikanzeige) zum Ermitteln eines passenden Sensorwertes entsprechend des Skalierungsfaktors
; Register: r18, r19, r20
;	    r16, r17 <- Sensorwert im Zweierkomplement
;	    r21      <- Skalierungsfaktor
;	    Z-Flag   -> gesetzt, wenn Sensorwert ganzzahlig
;
grchek:	movw	r18, r16		;Sensorwert kopieren
	bst	r19, 7			;Wert negativ?
	brtc	grch10			;nein -> weiter
	clr	r18
	clr	r19
	sub	r18, r16
	sbc	r19, r17		;in positiven Wert wandeln
grch10:	mov	r20, r18
	or	r20, r19		;Ergebnis = 0?
	breq	grch20			;ja -> Ende
	cpi	r21, 1			;Skalierungsfaktor 1?
	brne	grch30			;nein -> weiter testen
	subi	r18, 10			;sonst 10 subtrahieren
	sbci	r19, 0			;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
grch20:	ret				;sonst Ende
;
grch30:	cpi	r21, 2			;Skalierungsfaktor 2?
	brne	grch40			;nein -> weiter testen
	subi	r18, 20			;sonst 20 subtrahieren
	sbci	r19, 0			;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
	ret				;sonst Ende
;
grch40:	cpi	r21, 4			;Skalierungsfaktor 4?
	brne	grch50			;nein -> weiter testen
	subi	r18, 30			;sonst 30 subtrahieren
	sbci	r19, 0			;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
	ret				;sonst Ende
;
grch50:	cpi	r21, 8			;Skalierungsfaktor 8?
	brne	grch60			;nein -> weiter testen
	subi	r18, 50			;sonst 50 subtrahieren
	sbci	r19, 0			;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
	ret				;sonst Ende
;
grch60:	cpi	r21, 16			;Skalierungsfaktor 16?
	brne	grch70			;nein -> weiter testen
	subi	r18, 100		;sonst 100 subtrahieren
	sbci	r19, 0			;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
	ret				;sonst Ende
;
grch70:	cpi	r21, 32			;Skalierungsfaktor 32?
	brne	grch80			;nein -> weiter testen
	subi	r18, 200		;sonst 200 subtrahieren
	sbci	r19, 0			;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
	ret				;sonst Ende
;
grch80:	cpi	r21, 64			;Skalierungsfaktor 64?
	brne	grch90			;nein -> weiter
	subi	r18, low(500)		;sonst 500 subtrahieren
	sbci	r19, high(500)		;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
	ret				;sonst Ende
;
grch90:	subi	r18, low(1000)		;sonst 1000 subtrahieren
	sbci	r19, high(1000)		;bertrag, Ergebnis positiv?
	brcc	grch10			;ja -> Schleife
	ret				;sonst Ende
;
; Unterprogramm (Grafikanzeige) zum Ermitteln des nchsten Skalenwertes
; Register: r18, r19
;	    r16, r17 <- Sensorwert im Zweierkomplement
;	    r21      <- Skalierungsfaktor
;
grysca:	clr	r19			;H-Byte lschen
	cpi	r21, 1			;Skalierungsfaktor 1?
	brne	grys10			;nein -> weiter testen
	ldi	r18, 10			;sonst
	ldi	r19, 0			;Wert 10 laden
	rjmp	grys80
;
grys10:	cpi	r21, 2			;Skalierungsfaktor 2?
	brne	grys20			;nein -> weiter testen
	ldi	r18, 20			;sonst Wert 20 laden
	rjmp	grys80
;
grys20:	cpi	r21, 4			;Skalierungsfaktor 4?
	brne	grys30			;nein -> weiter testen
	ldi	r18, 30			;sonst Wert 30 laden
	rjmp	grys80
;
grys30:	cpi	r21, 8			;Skalierungsfaktor 8?
	brne	grys40			;nein -> weiter testen
	ldi	r18, 50			;sonst Wert 50 laden
	rjmp	grys80
;
grys40:	cpi	r21, 16			;Skalierungsfaktor 16?
	brne	grys50			;nein -> weiter testen
	ldi	r18, 100		;sonst Wert 100 laden
	rjmp	grys80
;
grys50:	cpi	r21, 32			;Skalierungsfaktor 32?
	brne	grys60			;nein -> weiter testen
	ldi	r18, 200		;sonst Wert 200 laden
	rjmp	grys80
;
grys60:	cpi	r21, 64			;Skalierungsfaktor 64?
	brne	grys70			;nein -> weiter
	ldi	r18, low(500)		;sonst
	ldi	r19, high(500)		;Wert 500 laden
	rjmp	grys80
;
grys70:	ldi	r18, low(1000)
	ldi	r19, high(1000)		;Wert 1000 laden
grys80:	add	r16, r18
	adc	r17, r19		;Y-Skalenwert + 10
	ret
;
; Unterprogramm (Grafikanzeige) zum Extrahieren des Datums aus einem EEPROM-Datensatz und Ausgabe auf dem LCD
; Register: r0, r1, r13, r14, r15, r16, r17, r18, r19, r23, r24, zl, zh, t
;	    r8-r11 <- Zeitdaten im Format eines EEPROM-Datensatzes
;	    r20	   <- X-Koordinate (0-239)
;	    r21    <- Y-Koordinate (0-127)
;
grelcd:	mov	r18, r10		;komprimierte Zeit Byte 2 holen
	andi	r18, 0x1f		;nicht relevante Bits filtern
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Tag Einer sichern
	mov	r22, r17		;Tag Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Tag Einer holen
	rcall	asclcd			;ausgeben
	ldi	r22, '.'		;Trennzeichen
	rcall	asclcd			;ausgeben
	mov	r18, r11		;komprimierte Zeit Byte 3 holen
	bst	r18, 0			;LSB sichern, gehrt zum Monat
	mov	r18, r10		;komprimierte Zeit Byte 2 holen
	ror	r18			;Monat nach rechts schieben
	bld	r18, 7			;LSB von Byte 3 holen, Monat an
	swap	r18			;richtige Position schieben
	andi	r18, 0x0f		;Nicht relevante Bits filtern
	rcall	bytasc			;in ASCII-Wert wandeln
	mov	r13, r16		;Monat Einer sichern
	mov	r22, r17		;Monat Zehner kopieren
	rcall	asclcd			;ausgeben
	mov	r22, r13		;gesicherten Monat Einer holen
	rcall	asclcd			;ausgeben
	ret
;
; Unterprogramm (Grafikanzeige) zum Extrahieren der Zeit aus einem EEPROM-Datensatz und Ausgabe auf dem LCD
; Register: r0, r1, r13, r14, r15, r16, r17, r18, r19, r23, r24, zl, zh, t
;	    r8-r11 <- Zeitdaten im Format eines EEPROM-Datensatzes
;	    r20	   <- X-Koordinate (0-239)
;	    r21    <- Y-Koordinate (0-127)
;
grelc2:	mov	r18, r9			;Stundenwert holen
	call	bytasc			;in ASCII wandeln
	cpi	r17, '0'		;Zehner = 0?
	brne	gre010			;nein -> weiter
	ldi	r17, ' '		;sonst durch Leerzeichen ersetzen
gre010:	mov	r13, r16		;Einer sichern
	mov	r22, r17		;Zehner holen
	call	asclcd			;ausgeben
	mov	r22, r13		;gesicherte Einer holen
	call	asclcd			;ausgeben
	ldi	r22, ':'		;Doppelpunkt
	call	asclcd			;ausgeben
	mov	r18, r8			;Minutenwert holen
	call	bytasc			;in ASCII wandeln
	mov	r13, r16		;Einer sichern
	mov	r22, r17		;Zehner holen
	call	asclcd			;ausgeben
	mov	r22, r13		;gesicherte Einer holen
	call	asclcd			;ausgeben
	ret
;
; Unterprogramm (Grafikanzeige) zum Berechnen der X-Koordinaten fr das Fadenkreuz
; Register: r16, r17, r19, x
;	    r18	<- Fadenkreuz-Position (0-221)
;
grscxp:	clr	r16
	sts	grry1p, r16		;Y-Position 1 lschen
	sts	grry2p, r16		;Y-Position 2 lschen
	ldi	xl, low(grbuff)
	ldi	xh, high(grbuff)	;Basisadresse Grafikdatenpuffer
	ldi	r19, 8			;Offset 8 Bytes
	mul	r18, r19		;Puffer-Offset berechnen
	add	xl, r0			;Pufferadresse ermitteln
	adc	xh, r1			;bertrag
	adiw	xl, 4			;Adresse auf Sensorwert 1
;
	ld	r16, x+
	ld	r17, x+			;Sensorwert 1 holen
	cpi	r17, 0x80		;Sensorwert 1 ungltig?
	breq	grsc40			;ja -> berspringen
	lds	r18, grcen1
	lds	r19, grcen1 + 1		;Mittelpunktwert 1 holen
	sub	r16, r18
	sbc	r17, r19		;Sensorwert - Mittelpunktwert
	lds	r18, grsca1		;Skalierungsfaktor 1 holen
grsc10:	lsr	r18			;Halbierung notwendig?
	breq	grsc20			;nein -> weiter
	asr	r17
	ror	r16			;sonst Anzeigewert halbieren
	brcc	grsc10			;Runden? nein -> Schleife
	inc	r16			;sonst aufrunden
	brne	grsc10			;bertrag? nein -> Schleife
	inc	r17			;sonst bertrag addieren
	rjmp	grsc10			;Schleife
;
grsc20:	ldi	r19, 59			;Y-Position 1 Mittelwert = 59
	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 4			;Doppelgrafik?
	brtc	grsc30			;nein -> Y-Position ok
	ldi	r19, 37			;sonst Y-Position Mittelwert = 37
grsc30:	sub	r19, r16		;Y-Pixel-Position ermitteln
	sts	grry1p, r19		;Y-Position 1 speichern
;
grsc40:	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 4			;Doppelgrafik?
	brtc	grsc70			;nein -> nur ein Diagramm
	ld	r16, x+			;sonst
	ld	r17, x+			;Sensorwert 2 holen
	cpi	r17, 0x80		;Sensorwert 2 ungltig?
	breq	grsc70			;ja -> berspringen
	lds	r18, grcen2
	lds	r19, grcen2 + 1		;Mittelpunktwert 2 holen
	sub	r16, r18
	sbc	r17, r19		;Sensorwert - Mittelpunktwert
	lds	r18, grsca2		;Skalierungsfaktor 2 holen
grsc50:	lsr	r18			;Halbierung notwendig?
	breq	grsc60			;nein -> weiter
	asr	r17
	ror	r16			;sonst Anzeigewert halbieren
	brcc	grsc50			;Runden? nein -> Schleife
	inc	r16			;sonst aufrunden
	brne	grsc50			;bertrag? nein -> Schleife
	inc	r17			;sonst bertrag addieren
	rjmp	grsc50			;Schleife
;
grsc60:	ldi	r19, 81			;Y-Position 2 Mittelwert = 81
	sub	r19, r16		;Y-Pixel-Position ermitteln
	sts	grry2p, r19		;Y-Position 1 speichern
grsc70:	ret

; Unterprogramm (Grafikanzeige) zum Sichern der LCD-Daten im Fadenkreuzbereich
; Register: r16, r17, r19, r20, x
;	    r18	<- Fadenkreuz-Position (0-221)
;
grscrb:	ldi	r16, 16			;Anfangsposition der Diagramme
	add	r18, r16		;Fadenkreuz X-Position ermitteln
	lsr	r18
	lsr	r18
	lsr	r18			;Byte-Adresse ermitteln
	clr	r19			;H-Byte = 0
	ldi	r16, low(16 * 30)	;Adresse von erster Diagrammzeile
	ldi	r17, high(16 * 30)	;laden
	add	r18, r16		;Adresse von oberster Fadenkreuzzeile
	adc	r19, r17		;ermitteln
;
	ldi	xl, low(grrbu0)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu0)	;Fadenkreuz
	ldi	r20, 87			;87 Pixel in Y-Richtung
;
grsb10:	bst	r20, 0			;Zhler geradzahlig?
	brtc	grsb20			;ja -> Ausgabe berspringen
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
	ldi	r17, 0xc5		;Kommando "Data Read"
	rcall	outlcd			;Byte an LCD ausgeben
	rcall	in_lcd			;Datenbyte aus LCD einlesen
	st	x+, r17			;Datenbyte in Fadenkreuzpuffer
;
grsb20:	ldi	r16, 30			;Adress-Offset zur nchsten
	clr	r17			;LCD-Zeile
	add	r18, r16		;Adresse von nchster Fadenkreuzzeile
	adc	r19, r17		;ermitteln
	dec	r20			;alle Datenbytes gelesen?
	brne	grsb10			;nein -> Schleife
;
; Fadenkreuz-Zeile Diagramm 1 sichern
;
	lds	r16, grry1p		;Y-Position von Wert 1 holen
	tst	r16			;Y-Position gltig?
	breq	grsb40			;nein -> keine Sicherung
	ldi	r18, 2
	ldi	r19, 0			;Adress-Offset Diagrammdaten
	ldi	r17, 30			;Adress-Offset nchste Zeile
	mul	r16, r17		;Zeilenadresse berechnen
	add	r18, r0
	adc	r19, r1			;Adresse Messwertzeile ermitteln
	ldi	xl, low(grrbu1)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu1)	;Fadenkreuzlinie 1
	ldi	r20, 28			;28 Bytes aus LCD lesen
;
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
grsb30:	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc1		;Kommando "Data Read Increment"
	rcall	outlcd			;Byte an LCD ausgeben
	rcall	in_lcd			;Datenbyte aus LCD einlesen
	st	x+, r17			;Datenbyte in Fadenkreuzpuffer
	dec	r20			;alle Datenbytes gelesen?
	brne	grsb30			;nein -> Schleife
;
; Fadenkreuz-Zeile Diagramm 2 sichern
;
grsb40:	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 4			;Doppelgrafik?
	brtc	grsb60			;nein -> nur ein Diagramm
	lds	r16, grry2p		;Y-Position von Wert 2 holen
	tst	r16			;Y-Position gltig?
	breq	grsb60			;nein -> keine Sicherung
	ldi	r18, 2
	ldi	r19, 0			;Adress-Offset Diagrammdaten
	ldi	r17, 30			;Adress-Offset nchste Zeile
	mul	r16, r17		;Zeilenadresse berechnen
	add	r18, r0
	adc	r19, r1			;Adresse Messwertzeile ermitteln
	ldi	xl, low(grrbu2)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu2)	;Fadenkreuzlinie 2
	ldi	r20, 28			;28 Bytes aus LCD lesen
;
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
grsb50:	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc1		;Kommando "Data Read Increment"
	rcall	outlcd			;Byte an LCD ausgeben
	rcall	in_lcd			;Datenbyte aus LCD einlesen
	st	x+, r17			;Datenbyte in Fadenkreuzpuffer
	dec	r20			;alle Datenbytes gelesen?
	brne	grsb50			;nein -> Schleife
grsb60:	ret
;
; Unterprogramm (Grafikanzeige) zum Wiederherstellen der LCD-Daten im Fadenkreuzbereich
; Register: r16, r17, r19, r20, x
;	    r18	<- Fadenkreuz-Position (0-221)
;
grscrr:	ldi	r16, 16			;Anfangsposition der Diagramme
	add	r18, r16		;Fadenkreuz X-Position ermitteln
	lsr	r18
	lsr	r18
	lsr	r18			;Byte-Adresse ermitteln
	clr	r19			;H-Byte=0
	ldi	r16, low(16 * 30)	;Adresse von erster Diagrammzeile
	ldi	r17, high(16 * 30)	;laden
	add	r18, r16		;Adresse von oberster Fadenkreuzzeile
	adc	r19, r17		;ermitteln
;
	ldi	xl, low(grrbu0)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu0)	;Fadenkreuz
	ldi	r20, 87			;87 Pixel in Y-Richtung
;
grsr10:	bst	r20, 0			;Zhler geradzahlig?
	brtc	grsr20			;ja -> Ausgabe berspringen
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
	clt				;Datenbyte an LCD ausgeben
	ld	r17, x+			;Datenbyte aus Fadenkreuzpuffer
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc4		;Kommando "Data Write"
	rcall	outlcd			;Byte an LCD ausgeben
;
grsr20:	ldi	r16, 30			;Adress-Offset zur nchsten
	clr	r17			;LCD-Zeile
	add	r18, r16		;Adresse von nchster Fadenkreuzzeile
	adc	r19, r17		;ermitteln
	dec	r20			;alle Datenbytes gelesen?
	brne	grsr10			;nein -> Schleife
;
; Fadenkreuz-Zeile Diagramm 1 wiederherstellen
;
	lds	r16, grry1p		;Y-Position von Wert 1 holen
	tst	r16			;Y-Position gltig?
	breq	grsr40			;nein -> nicht wiederherstellen
	ldi	r18, 2
	ldi	r19, 0			;Adress-Offset Diagrammdaten
	ldi	r17, 30			;Adress-Offset nchste Zeile
	mul	r16, r17		;Zeilenadresse berechnen
	add	r18, r0
	adc	r19, r1			;Adresse Wertzeile ermitteln
	ldi	xl, low(grrbu1)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu1)	;Fadenkreuzlinie 1
	ldi	r20, 28			;28 Bytes an LCD ausgeben
;
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
grsr30:	clt				;Datenbyte an LCD ausgeben
	ld	r17, x+			;Datenbyte aus Fadenkreuzpuffer
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc0		;Kommando "Data Write Increment"
	rcall	outlcd			;Byte an LCD ausgeben
	dec	r20			;alle Datenbytes gelesen?
	brne	grsr30			;nein -> Schleife
;
; Fadenkreuz-Zeile Diagramm 2 wiederherstellen
;
grsr40:	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 4			;Doppelgrafik?
	brtc	grsr60			;nein -> nur ein Diagramm
	lds	r16, grry2p		;Y-Position von Wert 2 holen
	tst	r16			;Y-Position gltig?
	breq	grsr60			;nein -> nicht wiederherstellen
	ldi	r18, 2
	ldi	r19, 0			;Adress-Offset Diagrammdaten
	ldi	r17, 30			;Adress-Offset nchste Zeile
	mul	r16, r17		;Zeilenadresse berechnen
	add	r18, r0
	adc	r19, r1			;Adresse Wertzeile ermitteln
	ldi	xl, low(grrbu2)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu2)	;Fadenkreuzlinie 2
	ldi	r20, 28			;28 Bytes an LCD ausgeben
;
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
grsr50:	clt				;Datenbyte an LCD ausgeben
	ld	r17, x+			;Datenbyte aus Fadenkreuzpuffer
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc0		;Kommando "Data Write Increment"
	rcall	outlcd			;Byte an LCD ausgeben
	dec	r20			;alle Datenbytes gelesen?
	brne	grsr50			;nein -> Schleife
grsr60:	ret
;
; Unterprogramm (Grafikanzeige) zum Zeichnen des Fadenkreuzes, es werden die gesicherten LCD-Daten ausgegeben, Pixel an der Fadenkreuz-Position werden gesetzt,
; an den gespeicherten Y-Positionen wird eine Rasterlinie gezeichnet
; Register: r16, r17, r19, r20, r21, r22, x
;	    r18	<- Fadenkreuz-Position (0-221)
;
grcros:	mov	r22, r18		;Fadenkreuz-Position sichern
	mov	r20, r18		;Fadenkreuz-Position kopieren
	andi	r20, 7			;Pixelposition filtern
	ldi	r21, 0x80		;Maske Anfangswert
grcr10:	tst	r20			;Maske auf richtiger Position?
	breq	grcr20			;ja -> weiter
	lsr	r21			;Maske verschieben
	dec	r20			;Positionszhler vermindern
	rjmp	grcr10			;Schleife
;
grcr20:	ldi	r16, 16			;Anfangsposition der Diagramme
	add	r18, r16		;Fadenkreuz X-Position ermitteln
	lsr	r18
	lsr	r18
	lsr	r18			;Byte-Adresse ermitteln
	clr	r19			;H-Byte = 0
	ldi	r16, low(16 * 30)	;Adresse erste Diagrammzeile
	ldi	r17, high(16 * 30)	;laden
	add	r18, r16		;Adresse von oberster Fadenkreuzzeile
	adc	r19, r17		;ermitteln
;
	ldi	xl, low(grrbu0)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu0)	;Fadenkreuz
	ldi	r20, 87			;87 Pixel in Y-Richtung
;
grcr30:	bst	r20, 0			;Zhler geradzahlig?
	brtc	grcr40			;ja -> Ausgabe berspringen
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
	ld	r17, x+			;Datenbyte aus Fadenkreuzpuffer
	or	r17, r21		;Fadenkreuz-Pixel setzen
	clt				;Datenbyte an LCD ausgeben
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc4		;Kommando "Data Write"
	rcall	outlcd			;Byte an LCD ausgeben
;
grcr40:	ldi	r16, 30			;Adress-Offset zur nchsten
	clr	r17			;LCD-Zeile
	add	r18, r16		;Adresse von nchster Fadenkreuzzeile
	adc	r19, r17		;ermitteln
	dec	r20			;alle Datenbytes gelesen?
	brne	grcr30			;nein -> Schleife
;
; Fadenkreuz-Zeile Diagramm 1 zeichnen
;
	lds	r16, grry1p		;Y-Position von Wert 1 holen
	tst	r16			;Y-Position gltig?
	breq	grcr60			;nein -> keine Linie zeichnen
	ldi	r18, 2
	ldi	r19, 0			;Adress-Offset Diagrammdaten
	ldi	r17, 30			;Adress-Offset nchste Zeile
	mul	r16, r17		;Zeilenadresse berechnen
	add	r18, r0
	adc	r19, r1			;Adresse Wertzeile ermitteln
	ldi	xl, low(grrbu1)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu1)	;Fadenkreuzlinie 1
	ldi	r20, 28			;28 Bytes an LCD ausgeben
;
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
grcr50:	clt				;Datenbyte an LCD ausgeben
	ld	r17, x+			;Datenbyte aus Fadenkreuzpuffer
	ori	r17, 0x55		;Rasterlinie berlagern
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc0		;Kommando "Data Write Increment"
	rcall	outlcd			;Byte an LCD ausgeben
	dec	r20			;alle Datenbytes gelesen?
	brne	grcr50			;nein -> Schleife
;
; Fadenkreuz-Zeile Diagramm 2 zeichnen
;
grcr60:	lds	r18, grflag		;Grafik-Flags holen
	bst	r18, 4			;Doppelgrafik?
	brtc	grcr80			;nein -> nur ein Diagramm
	lds	r16, grry2p		;Y-Position von Wert 2 holen
	tst	r16			;Y-Position gltig?
	breq	grcr80			;nein -> keine Linie zeichnen
	ldi	r18, 2
	ldi	r19, 0			;Adress-Offset Diagrammdaten
	ldi	r17, 30			;Adress-Offset nchste Zeile
	mul	r16, r17		;Zeilenadresse berechnen
	add	r18, r0
	adc	r19, r1			;Adresse Wertzeile ermitteln
	ldi	xl, low(grrbu2)		;Zeiger auf LCD-Datenpuffer fr
	ldi	xh, high(grrbu2)	;Fadenkreuzlinie 2
	ldi	r20, 28			;28 Bytes an LCD ausgeben
;
	clt				;Datenbyte an LCD ausgeben
	mov	r17, r18		;Adresse L-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	mov	r17, r19		;Adresse H-Byte kopieren
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0x24		;Kommando "Adress Pointer Set"
	rcall	outlcd			;Byte an LCD ausgeben
grcr70:	clt				;Datenbyte an LCD ausgeben
	ld	r17, x+			;Datenbyte aus Fadenkreuzpuffer
	ori	r17, 0x55		;Rasterlinie berlagern
	rcall	outlcd			;Byte an LCD ausgeben
	set				;Kommando an LCD ausgeben
	ldi	r17, 0xc0		;Kommando "Data Write Increment"
	rcall	outlcd			;Byte an LCD ausgeben
	dec	r20			;alle Datenbytes gelesen?
	brne	grcr70			;nein -> Schleife
grcr80:	ret
;
; Unterprogramm zum Schreiben von Default-Werten und einer CRC8-Checksumme ins EEPROM
; Register: r14, r15, r16, r17, r18, r19, r20, z
;
; Anzeige-Positions-Tabelle fr beide Bildschirm-Seiten im EEPROM speichern
;
eedcfg:	ldi	r20, 0			;Anzeige-Position, Anfangswert = 0
	ldi	r16, low(epotab)
	ldi	r17, high(epotab)	;EEPROM-Adresse der Tabelle
eedpot:	mov	r18, r20		;Anzeige-Position holen
	rcall	eewrit			;Daten im EEPROM speichern
	inc	r20			;nchste Anzeige-Position
        cpi     r20, 32                 ;alle 32 Werte gespeichert?
	brcs	eedpot			;nein -> Schleife
;
; Defaultwerte fr Sensor-Flags im EERPOM speichern, alle Flags werden auf 0 gesetzt (alle Sensoren sind aktiv)
;
	ldi	r20, 32 		;32 Werte
	ldi	r16, low(esnflg)
	ldi	r17, high(esnflg)	;Zeiger auf EERPOM Daten
eedtfl:	ldi	r18, 0x80		;Inaktivitts-Flag setzen
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eedtfl			;nein -> Schleife
;
; Defaultwerte fr Groanzeige-Sensornummern auf Seite 1 und Seite 2 im EEPROM speichern
;
	ldi	r16, low(ebiflg)
	ldi	r17, high(ebiflg)	;EEPROM-Adresse Sensornummern Groanzeige
	clr	r18			;Groanzeige Seite 1, Sensornummer 0
	rcall	eewrit			;im EERPOM speichern
	clr	r18			;Groanzeige Seite 2, Sensornummer 0
	rcall	eewrit			;Byte im EERPOM speichern
;
; Defaultwerte fr Sensortexte im EERPOM speichern
;
	ldi	r20, 31			;31 Texte
	ldi	r16, low(esentx)
	ldi	r17, high(esentx)	;EEPROM-Adresse der Texte
	ldi	zl, byte3(dsentx * 2)
	out	rampz,zl
	ldi	zl, low(dsentx * 2)
	ldi	zh, high(dsentx * 2)	;Zeiger auf Default-Daten
eedtt1:	ldi	r19, 9			;jeweils 9 Zeichen Lnge
eedtt2:	elpm	r18, z+			;Textzeichen holen
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r19			;Text komplett?
	brne	eedtt2			;nein -> Schleife
	dec	r20			;alle Texte gespeichert?
	brne	eedtt1			;nein -> Schleife
;
; Defaultwerte fr obere Temperaturlimits im EERPOM speichern, alle Werte werden auf 0x8000 gesetzt (Limit inaktiv)
;
	ldi	r19, 27 		;27 Werte
        ldi     r20, low(0x8000)
        ldi     r21, high(0x8000)       ;Wert laden
	ldi	r16, low(eselih)
	ldi	r17, high(eselih)	;Zeiger auf EERPOM Daten
eedlih:	mov	r18, r20		;Byte holen (Low)
	rcall	eewrit			;Daten im EEPROM speichern
        mov     r18, r21                ;Byte holen (High)
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r19			;alle Werte gespeichert?
	brne	eedlih			;nein -> Schleife
;
; Defaultwerte fr untere Temperaturlimits im EERPOM speichern, alle Werte werden auf 0x8000 gesetzt (Limit inaktiv)
;
	ldi	r19, 27 		;27 Werte
        ldi     r20, low(0x8000)
        ldi     r21, high(0x8000)       ;Wert laden
	ldi	r16, low(eselil)
	ldi	r17, high(eselil)	;Zeiger auf EERPOM Daten
eedlil:	mov	r18, r20		;Byte holen (Low)
	rcall	eewrit			;Daten im EEPROM speichern
	mov	r18, r21		;Byte holen (High)
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r19			;alle Werte gespeichert?
	brne	eedlil			;nein -> Schleife
;
; Defaultwerte fr Sensor-Log-Tabelle 1 im EEPROM speichern
;
	ldi	r20, 0			;Sensor-Log-Position, Anfangswert = 0
	ldi	r16, low(elotb1)
	ldi	r17, high(elotb1)	;EEPROM-Adresse der Log-Tabelle 1
eedlt1:	mov	r18, r20		;Log-Position holen
	rcall	eewrit			;Daten im EEPROM speichern
	inc	r20			;nchste Anzeige-Position
        cpi     r20, 24                 ;alle 24 Werte gespeichert?
	brcs	eedlt1			;nein -> Schleife
;
; Defaultwerte fr Sensor-Log-Tabelle 2 im EEPROM speichern
;
	ldi	r20, 0			;Sensor-Log-Position, Anfangswert = 0
	ldi	r16, low(elotb2)
	ldi	r17, high(elotb2)	;EEPROM-Adresse der Tabelle
eedlt2:	mov	r18, r20		;Log-Position holen
	rcall	eewrit			;Daten im EEPROM speichern
	inc	r20			;nchste Log-Position
        cpi     r20, 3                  ;alle 3 Werte gespeichert?
	brcs	eedlt2			;nein -> Schleife
;
; Defaultwerte fr Alarm-Anzeige-Positions-Tabelle im EEPROM speichern
;
	ldi	r20, 0			;Anzeige-Positions-Tabelle, Anfangswert = 0
	ldi	r16, low(ealtab)
	ldi	r17, high(ealtab)	;EEPROM-Adresse der Tabelle
eedatb:	mov	r18, r20		;Anzeige-Position holen
        sbr     r18, 0x80               ;Inaktivitts-Flag setzen
	rcall	eewrit			;Daten im EERPOM speichern
	inc	r20			;nchste Anzeige-Position
        cpi     r20, 4                  ;alle 4 Werte gespeichert?
	brcs	eedatb			;nein -> Schleife
;
; Defaultwerte fr Alarm-Invertierungs-Tabelle im EEPROM speichern
;
	ldi	r20, 4			;Invertierungs-Tabelle, 4 Werte
	ldi	r16, low(ealinv)
	ldi	r17, high(ealinv)	;EEPROM-Adresse der Tabelle
eedinv:	clr	r18			;Invertierung = 0 setzen (aus)
	rcall	eewrit			;Daten im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eedinv			;nein -> Schleife
;
; Defaultwerte fr Alarmtexte im EERPOM speichern
;
	ldi	r20, 4			;4 Texte
	ldi	r16, low(ealmtx)
	ldi	r17, high(ealmtx)	;EEPROM-Adresse der Texte
	ldi	zl, byte3(dalmtx * 2)
	out	rampz, zl
	ldi	zl, low(dalmtx * 2)
	ldi	zh, high(dalmtx * 2)	;Zeiger auf Default-Daten
eedat1:	ldi	r19, 11			;jeweils 11 Zeichen Lnge
eedat2:	elpm	r18, z+			;Textzeichen holen
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r19			;Text komplett?
	brne	eedat2			;nein -> Schleife
	dec	r20			;alle Texte gespeichert?
	brne	eedat1			;nein -> Schleife
;
; Defaultwerte fr Signalnummern-Tabelle im EEPROM speichern
;
	ldi	r20, 8			;Signalnummern-Tabelle, 8 Werte
	ldi	r16, low(esignr)
	ldi	r17, high(esignr)	;EEPROM-Adresse der Tabelle
	ldi	zl, byte3(dsignr * 2)
	out	rampz, zl
	ldi	zl, low(dsignr * 2)
	ldi	zh, high(dsignr * 2)	;Zeiger auf Default-Daten
eedsnu:	elpm	r18, z+			;Signalnummer holen
	rcall	eewrit			;Daten im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eedsnu			;nein -> Schleife
;
; Defaultwerte fr Signaldauer-Tabelle im EEPROM speichern
;
	ldi	r20, 8			;Signaldauer-Tabelle, 8 Werte
	ldi	r16, low(esigdu)
	ldi	r17, high(esigdu)	;EEPROM-Adresse der Tabelle
	ldi	zl, byte3(dsigdu * 2)
	out	rampz, zl
	ldi	zl, low(dsigdu * 2)
	ldi	zh, high(dsigdu * 2)	;Zeiger auf Default-Daten
eedsdu:	elpm	r18, z+			;Signaldauer holen
	rcall	eewrit			;Daten im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eedsdu			;nein -> Schleife
;
; Default-Timer-Wert zur Erzeugung der Signalfrequenz im EEPROM speichern
;
	ldi	r16, low(esigtm)
	ldi	r17, high(esigtm)	;EEPROM-Adresse Signal-Frequenz
	ldi	zl, byte3(dsigtm * 2)
	out	rampz, zl
	ldi	zl, low(dsigtm * 2)
	ldi	zh, high(dsigtm * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Default-Lautstrke-Wert 1 (Tag) im EEPROM speichern
;
	ldi	r16, low(esigv1)
	ldi	r17, high(esigv1)	;EEPROM-Adresse Lautstrke 1
	ldi	zl, byte3(dsigv1 * 2)
	out	rampz, zl
	ldi	zl, low(dsigv1 * 2)
	ldi	zh, high(dsigv1 * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Default-Lautstrke-Wert 2 (Nacht) im EEPROM speichern
;
	ldi	r16, low(esigv2)
	ldi	r17, high(esigv2)	;EEPROM-Adresse Lautstrke 2
	ldi	zl, byte3(dsigv2 * 2)
	out	rampz, zl
	ldi	zl, low(dsigv2 * 2)
	ldi	zh, high(dsigv2 * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Default-Nachtzeiten fr akustische Signalisierung im EEPROM speichern
;
	ldi	r20, 4			;Nachtschaltung, 2x2 Werte
	ldi	r16, low(esigni)
	ldi	r17, high(esigni)	;EERPOM-Adresse Startzeit
	ldi	zl, byte3(dsigni * 2)
	out	rampz, zl
	ldi	zl, low(dsigni * 2)
	ldi	zh, high(dsigni * 2)	;Zeiger auf Default-Daten
eedsig:	elpm	r18, z+			;Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eedsig			;nein -> Schleife
;
; Default-Nachtzeiten fr die LCD-Hintergrundbeleuchtung im EEPROM speichern
;
	ldi	r20, 4			;Nachtschaltung, 2x2 Werte
	ldi	r16, low(enigbl)
	ldi	r17, high(enigbl)	;EERPOM-Adresse Startzeit
	ldi	zl, byte3(dnigbl * 2)
	out	rampz, zl
	ldi	zl, low(dnigbl * 2)
	ldi	zh, high(dnigbl * 2)	;Zeiger auf Default-Daten
eednib:	elpm	r18, z+			;Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eednib			;nein -> Schleife
;
; Default-Auto-Reset-Zeiten fr Min- und Max-Werte im EEPROM speichern
;
	ldi	r20, 4			;Reset-Zeiten, 2x2 Werte
	ldi	r16, low(earesm)
	ldi	r17, high(earesm)	;EERPOM-Adresse Startzeit
	ldi	zl, byte3(daresm * 2)
	out	rampz, zl
	ldi	zl, low(daresm * 2)
	ldi	zh, high(daresm * 2)	;Zeiger auf Default-Daten
eedare:	elpm	r18, z+			;Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eedare			;nein -> Schleife
;
; Default-Nachtzeiten fr das Bluetooth-Modul im EEPROM speichern
;
	ldi	r20, 4			;Nachtschaltung, 2x2 Werte
	ldi	r16, low(enigbt)
	ldi	r17, high(enigbt)	;EERPOM-Adresse Startzeit
	ldi	zl, byte3(dnigbt * 2)
	out	rampz, zl
	ldi	zl, low(dnigbt * 2)
	ldi	zh, high(dnigbt * 2)	;Zeiger auf Default-Daten
eednbt:	elpm	r18, z+			;Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eednbt			;nein -> Schleife
;
; Defaultwert fr Konfigurationsbyte 1 im EEPROM speichern
;
	ldi	r16, low(econf1)
	ldi	r17, high(econf1)	;EEPROM-Adresse Konfigurationsbyte 1
	ldi	zl, byte3(dconf1 * 2)
	out	rampz, zl
	ldi	zl, low(dconf1 * 2)
	ldi	zh, high(dconf1 * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Defaultwert fr Konfigurationsbyte 2 im EEPROM speichern
;
	ldi	r16, low(econf2)
	ldi	r17, high(econf2)	;EEPROM-Adresse Konfigurationsbyte 2
	ldi	zl, byte3(dconf2 * 2)
	out	rampz, zl
	ldi	zl, low(dconf2 * 2)
	ldi	zh, high(dconf2 * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Default-Nacht-Helligkeit der Hintergrundbeleuchtung speichern
;
	ldi	r16, low(enligh)
	ldi	r17, high(enligh)	;EEPROM-Adresse Nacht-Helligkeit
	ldi	zl, byte3(dnligh * 2)
	out	rampz, zl
	ldi	zl, low(dnligh * 2)
	ldi	zh, high(dnligh * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Default-Baudrate fr die PC-Verbindung (RS232-0) im EEPROM speichern
;
	ldi	r16, low(ebaud0)
	ldi	r17, high(ebaud0)	;EEPROM-Adresse Baudrate RS232-0
	ldi	zl, byte3(dbaud0 * 2)
	out	rampz, zl
	ldi	zl, low(dbaud0 * 2)
	ldi	zh, high(dbaud0 * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Defaultwert fr Meldungsfilter im EEPROM speichern
;
	ldi	r16, low(emsgfi)
	ldi	r17, high(emsgfi)	;EEPROM-Adresse Meldungsfilter
	ldi	zl, byte3(dmsgfi * 2)
	out	rampz, zl
	ldi	zl, low(dmsgfi * 2)
	ldi	zh, high(dmsgfi * 2)	;Zeiger auf Default-Daten
	elpm	r18, z			;Byte holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Default-Grafik-Presets 1-24 im EEPROM speichern
;
eedgrp:	ldi	r20, 24 * 7		;Grafik-Presets, 24 x 7 Werte
	ldi	r16, low(egrpre)
	ldi	r17, high(egrpre)	;EERPOM-Adresse Presets
	ldi	zl, byte3(dgrpre * 2)
	out	rampz, zl
	ldi	zl, low(dgrpre * 2)
	ldi	zh, high(dgrpre * 2)	;Zeiger auf Default-Daten
eedg10:	elpm	r18, z+			;Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eedg10			;nein -> Schleife
;
; Default-Grafik-Preset-Texte 1-24 im EERPOM speichern
;
	ldi	r20, 24			;24 Texte
	ldi	r16, low(egrprt)
	ldi	r17, high(egrprt)	;EEPROM-Adresse der Texte
	ldi	zl, byte3(dgrprt * 2)
	out	rampz, zl
	ldi	zl, low(dgrprt * 2)
	ldi	zh, high(dgrprt * 2)	;Zeiger auf Default-Daten
eedg20:	ldi	r19, 16			;jeweils 16 Zeichen Lnge
eedg30:	elpm	r18, z+			;Textzeichen holen
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r19			;Text komplett?
	brne	eedg30			;nein -> Schleife
	dec	r20			;alle Texte gespeichert?
	brne	eedg20			;nein -> Schleife
;
; ermittelten CRC-Wert im EEPROM speichern
;
	call	calcrc			;EEPROM CRC-Berechnung
	mov	r18, r20		;ermittelten CRC-Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
	ret
;
; Unterprogramm zum Aktivieren der Konfigurationsspeicherung im EEPROM
; Register: r16
;
eewcon:	lds	r16, xflag3		;verschiedene Flags 3 holen
	sbr	r16, 0x01		;Flag fr Konfiguratiosspeicherung setzen
	sts	xflag3, r16		;Flags wieder speichern
	ret
;
; Unterprogramm zum Schreiben aller Konfigurationssdaten und einer CRC8 Checksumme ins EEPROM
; Register: r14, r15, r16, r17, r18, r19, r20, x, y
;
; Anzeige-Positions-Tabelle fr beide Bildschirm-Seiten im EEPROM speichern
;
eewcfg:	ldi	r20, 32			;Anzeige-Positions-Tabelle, 32 Werte
	ldi	r16, low(epotab)
	ldi	r17, high(epotab)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(postab)
	ldi	xh, high(postab)	;Zeiger auf RAM-Adresse
eewp1t:	ld	r18, x+			;Anzeige-Position holen
        andi    r18, 0x3f               ;Flag-Bits lschen
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewp1t			;nein -> Schleife
;
; Sensor-Flags im EEPROM speichern (nur Inaktivitts-Flag)
;
	ldi	r20, 32			;Sensor-Inaktivitts-Flags, 32 Werte
	ldi	r16, low(esnflg)
	ldi	r17, high(esnflg)	;EEPROM-Adresse der Flags
	ldi	xl, low(senflg)
	ldi	xh, high(senflg)	;Zeiger auf RAM-Adresse
eewflg:	ld	r18, x+			;Sensor-Flags holen
        andi    r18, 0x80               ;Inaktivitts-Bit filtern
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewflg			;nein -> Schleife
;
; Groanzeige-Flags im EEPROM speichern
;
	ldi	r16, low(ebiflg)
	ldi	r17, high(ebiflg)	;EEPROM-Adresse der Flags fr die Groanzeigen
	lds	r18, bigflg		;Flag fr Groanzeige Seite 1 holen
        andi    r18, 0x3f               ;Sensornummer filtern
	rcall	eewrit			;Daten im EEPROM speichern
	lds	r18, bigflg + 1		;Flag fr Groanzeige Seite 2 holen
        andi    r18, 0x3f               ;Sensornummer filtern
	rcall	eewrit			;Daten im EEPROM speichern
;
; obere Temperaturlimits im EERPOM speichern
;
	ldi	r20, 27 * 2		;27 Werte (27 * 2 Bytes)
	ldi	r16, low(eselih)
	ldi	r17, high(eselih)	;Zeiger auf EERPOM Daten
	ldi	xl, low(senlih)
	ldi	xh, high(senlih)	;Zeiger auf obere Limits
eewlih:	ld	r18, x+			;Byte aus RAM holen
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewlih			;nein -> Schleife
;
; untere Temperaturlimits im EERPOM speichern
;
	ldi	r20, 27 * 2		;27 Werte (27 * 2 Bytes)
	ldi	r16, low(eselil)
	ldi	r17, high(eselil)	;Zeiger auf EERPOM Daten
	ldi	xl, low(senlil)
	ldi	xh, high(senlil)	;Zeiger auf untere Limits
eewlil:	ld	r18, x+			;Byte aus RAM holen
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewlil			;nein -> Schleife
;
; Sensor-Log-Tabelle 1 im EEPROM speichern
;
	ldi	r20, 24			;24 Werte
	ldi	r16, low(elotb1)
	ldi	r17, high(elotb1)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(logtb1)
	ldi	xh, high(logtb1)	;Zeiger auf RAM-Adresse
eewlt1:	ld	r18, x+			;Log-Position holen
        andi    r18, 0x1f               ;Log-Position filtern
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewlt1			;nein -> Schleife
;
; Sensor-Log-Tabelle 2 im EEPROM speichern
;
	ldi	r20, 3			;3 Werte
	ldi	r16, low(elotb2)
	ldi	r17, high(elotb2)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(logtb2)
	ldi	xh, high(logtb2)	;Zeiger auf RAM-Adresse
eewlt2:	ld	r18, x+			;Log-Position holen
        andi    r18, 0x1f               ;Log-Position filtern
	rcall	eewrit			;Daten im EEPROM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewlt2			;nein -> Schleife
;
; Alarm-Anzeige-Positions-Tabelle im EEPROM speichern
;
	ldi	r20, 4			;Anzeige-Positions-Tabelle, 4 Werte
	ldi	r16, low(ealtab)
	ldi	r17, high(ealtab)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(almtab)
	ldi	xh, high(almtab)	;Zeiger auf RAM-Adresse
	ldi	yl, low(almflg)
	ldi	yh, high(almflg)	;Zeiger auf Alarmflags
eewama:	ld	r18, x+			;Anzeigeposition holen
	ld	r19, y+			;Alarmflags holen
	bst	r19, 7			;Inaktivitts-Flag holen
	bld	r18, 7			;in Anzeigeposition einsetzen
	rcall	eewrit			;Daten im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewama			;nein -> Schleife
;
; Alarm-Invertierungs-Tabelle im EEPROM speichern
;
	ldi	r20, 4			;Invertierungs-Tabelle, 4 Werte
	ldi	r16, low(ealinv)
	ldi	r17, high(ealinv)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(alminv)
	ldi	xh, high(alminv)	;Zeiger auf RAM-Adresse
eewinv:	ld	r18, x+			;Invertierung holen
	rcall	eewrit			;Daten im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewinv			;nein -> Schleife
;
; Signalnummern-Tabelle im EEPROM speichern
;
	ldi	r20, 8			;Signalnummern-Tabelle, 8 Werte
	ldi	r16, low(esignr)
	ldi	r17, high(esignr)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(signum)
	ldi	xh, high(signum)	;Zeiger auf RAM-Adresse
eewsnu:	ld	r18, x+			;Signalnummer holen
	rcall	eewrit			;Daten im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewsnu			;nein -> Schleife
;
; Signaldauer-Tabelle im EEPROM speichern
;
	ldi	r20, 8			;Signaldauer-Tabelle, 8 Werte
	ldi	r16, low(esigdu)
	ldi	r17, high(esigdu)	;EEPROM-Adresse der Tabelle
	ldi	xl, low(sigdur)
	ldi	xh, high(sigdur)	;Zeiger auf RAM-Adresse
eewsdu:	ld	r18, x+			;Signaldauer holen
	rcall	eewrit			;Daten im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewsdu			;nein -> Schleife
;
; Timer-Wert zur Erzeugung der Signalfrequenz im EEPROM speichern
;
	ldi	r16, low(esigtm)
	ldi	r17, high(esigtm)	;EEPROM-Adresse Signal-Frequenz
	in	r18, ocr0		;Timer0-Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Lautstrke-Wert 1 (Tag) im EEPROM speichern
;
	ldi	r16, low(esigv1)
	ldi	r17, high(esigv1)	;EEPROM-Adresse Lautstrke 1
	lds	r18, sigvo1		;Lautstrke-Wert 1 holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Lautstrke-Wert 2 (Nacht) im EEPROM speichern
;
	ldi	r16, low(esigv2)
	ldi	r17, high(esigv2)	;EEPROM-Adresse Lautstrke 2
	lds	r18, sigvo2		;Lautstrke-Wert 2 holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Nachtzeiten fr akustische Signalisierung im EEPROM speichern
;
	ldi	r20, 4			;Nachtschaltung, 2x2 Werte
	ldi	r16, low(esigni)
	ldi	r17, high(esigni)	;EERPOM-Adresse Startzeit
	ldi	xl, low(signig)
	ldi	xh, high(signig)	;Zeiger auf RAM-Adresse
eewsig:	ld	r18, x+			;Wert aus RAM holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewsig			;nein -> Schleife
;
; Nachtzeiten fr die LCD-Hintergrundbeleuchtung im EEPROM speichern
;
	ldi	r20, 4			;Nachtschaltung, 2x2 Werte
	ldi	r16, low(enigbl)
	ldi	r17, high(enigbl)	;EERPOM-Adresse Startzeit
	ldi	xl, low(nighbl)
	ldi	xh, high(nighbl)	;Zeiger auf RAM-Adresse
eewnib:	ld	r18, x+			;Wert aus RAM holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewnib			;nein -> Schleife
;
; Nachtzeiten fr das Bluetooth-Modul im EEPROM speichern
;
	ldi	r20, 4			;Nachtschaltung, 2x2 Werte
	ldi	r16, low(enigbt)
	ldi	r17, high(enigbt)	;EERPOM-Adresse Startzeit
	ldi	xl, low(nighbt)
	ldi	xh, high(nighbt)	;Zeiger auf RAM-Adresse
eewnbt:	ld	r18, x+			;Wert aus RAM holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eewnbt			;nein -> Schleife
;
; Auto-Reset-Zeiten fr Minimal- und Maximalwerte im EEPROM speichern
;
	ldi	r20, 4			;Reset-Zeiten, 2x2 Werte
	ldi	r16, low(earesm)
	ldi	r17, high(earesm)	;EERPOM-Adresse Startzeit
	ldi	xl, low(aresmm)
	ldi	xh, high(aresmm)	;Zeiger auf RAM-Adresse
eeware:	ld	r18, x+			;Wert aus RAM holen
	rcall	eewrit			;Byte im EERPOM speichern
	dec	r20			;alle Werte gespeichert?
	brne	eeware			;nein -> Schleife
;
; Konfigurationsbyte 1 im EEPROM speichern
;
	ldi	r16, low(econf1)
	ldi	r17, high(econf1)	;EEPROM-Adresse Konfig-Byte 1
	lds	r18, confg1		;Konfig-Byte 1 holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Konfigurationsbyte 2 im EEPROM speichern
;
	ldi	r16, low(econf2)
	ldi	r17, high(econf2)	;EEPROM-Adresse Konfig-Byte 2
	lds	r18, confg2		;Konfig-Byte 2 holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Nacht-Helligkeit der Hintergrundbeleuchtung speichern
;
	ldi	r16, low(enligh)
	ldi	r17, high(enligh)	;EEPROM-Adresse Nacht-Helligkeit
	lds	r18, nlight		;Nacht-Helligkeit holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Baudrate fr die PC-Verbindung (RS232-0) im EEPROM speichern
;
	ldi	r16, low(ebaud0)
	ldi	r17, high(ebaud0)	;EEPROM-Adr. Baudrate RS232-0
	lds	r18, baud0		;Baudrate RS232-0 holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; Meldungsfilter im EEPROM speichern
;
	ldi	r16, low(emsgfi)
	ldi	r17, high(emsgfi)	;EEPROM-Adresse Meldungsfilter
	lds	r18, msgfil		;Meldungsfilter holen
	rcall	eewrit			;Byte im EERPOM speichern
;
; ermittelten CRC-Wert im EEPROM speichern, Konfig-Fehler lschen
;
eewcrc:	call	calcrc			;EEPROM CRC-Berechnung
	mov	r18, r20		;ermittelten CRC-Wert holen
	rcall	eewrit			;Byte im EERPOM speichern
	lds	r16, errflg		;Fehler-Flags holen
	cbr	r16, 1 << 2		;Konfigurationsfehler lschen
	sts	errflg, r16		;Fehler-Flags wieder speichern
	ret
;
; Unterprogramm zum Lesen eines Byte aus dem EEPROM, nach dem Lesen wird die EEPROM-Adresse um 1 erhht
; Register: r16 <- EEPROM-Adresse L
;	    r17 <- EEPROM-Adresse H
;	    r18 -> EEPROM-Daten
;
eeread:	sbic	eecr, eewe		;luft ein Schreibzyklus?
	rjmp	eeread			;ja -> warten
	out	eearl, r16		;EEPROM-Adresse L setzen
	out	eearh, r17		;EEPROM-Adresse H setzen
	sbi	eecr, eere		;EEPROM lesen aktivieren
	in	r18, eedr		;Daten lesen
	inc	r16			;nchste EEPROM-Adresse
	brne	eere10			;bertrag? nein -> weiter
	inc	r17			;sonst H-Byte erhhen
eere10:	ret
;
; Unterprogramm zum Schreiben eines Byte in das EEPROM, nach dem Schreiben wird die EEPROM-Adresse um 1 erhht
; Register: r16 <- EEPROM-Adresse L
;	    r17 <- EEPROM-Adresse H
;	    r18 <- EEPROM-Daten
;
eewrit:	sbic	eecr, eewe		;luft ein Schreibzyklus?
	rjmp	eewrit			;ja -> warten
	cli				;Interrupts sperren
	out	eearl, r16		;EEPROM-Adresse L setzen
	out	eearh, r17		;EEPROM-Adresse H setzen
	out	eedr, r18		;EEPROM-Daten setzen
	sbi	eecr, eemwe		;EEPROM schreiben freigeben
	sbi	eecr, eewe		;EEPROM schreiben aktivieren
	sei				;Interrupts wieder freigeben
	inc	r16			;nchste EEPROM-Adresse
	brne	eewr10			;bertrag? nein -> weiter
	inc	r17			;sonst H-Byte erhhen
eewr10:	ret
;
; Unterprogramm Warteschleife ca. 50ms
; Register: r14, r15, r16
;
wait50:	ldi	r16, 206
	mov	r15, r16		;Zhler setzen (M)
	ldi	r16, 3			;Zhler setzen (H)
	clr	r14			;Zhler setzen (L)
wait51:	dec	r14			;L-Zhler abgelaufen?
	brne	wait51			;nein -> Schleife
	dec	r15			;M-Zhler abgelaufen?
	brne	wait51			;nein -> Schleife
	dec	r16			;H-Zhler abgelaufen?
	brne	wait51			;nein -> Schleife
	ret
;
; Unterprogramm bei Fehler in den TWI-Routinen
; Register: r16
;
twierr:	lds	r16, errflg		;Fehler-Flags holen
	sbr	r16, 1 << 3		;Speicherfehler-Flag setzen
	sts	errflg, r16		;Flags wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 1 << 2		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
	ret
;
; Unterprogramm zum Lschen eines Speicherbereiches im Externen EEPROM
; Register: r15, r16, r17, r18, r19
;	    r20-r22 <- Anfangsadresse
;	    r23-r25 <- Endadresse
;	    Carry   -> 0 = ok, 1 = Fehler
;
eraepr:	ldi	r16, xeeadr		;Slave-Adresse + Write-Bit (0)
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
;
	mov	r15, r16		;Slave-Adresse kopieren
	call	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	era040			;ACK empfangen? nein -> Fehler
	mov	r16, r21		;Adresse H-Byte holen
	call	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	era040			;nein -> Fehler
	mov	r16, r20		;Adresse L-Byte holen
	call	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	era040			;nein -> Fehler
;
era010:	ldi	r16, 0xff		;Leer-Byte holen
	call	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	era040			;nein -> Fehler
	inc	r20			;Adresse L erhhen
	brne	era020			;bertrag? nein -> weiter
	inc	r21			;sonst Adresse H erhhen
	brne	era020			;bertrag? nein -> weiter
	inc	r22			;sonst Adresse B3 erhhen
era020:	cp	r20, r23		;Endadresse L erreicht?
	brne	era030			;nein -> weiter
	cp	r21, r24		;Endadresse H erreicht?
	brne	era030			;nein -> weiter
	cp	r22, r25		;Endadresse B3 erreicht?
	brne	era030			;nein -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	clc				;Carry = 0, Schreiben ok
	ret
;
era030:	mov	r16, r20		;L-Byte holen
	andi	r16, 0x7f		;Bits 6-0 filtern, EEPROM-Page-Wechsel?
	brne	era010			;nein -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	rjmp	eraepr			;sonst neuer Schreibzyklus
;
era040:	sec				;Carry = 1, Fehler
	ret
;
; Unterprogramm zum Sichern der Konfigurationsdaten im Externen EEPROM
; Register: r0, r1, r12, r13, r15, r16, r17, r18, r19, r20, r21, r22, r24, r25, z
;	    Carry -> 0 = ok, 1 = Fehler
;
xwconf:	call	cpstim			;aktuelle Zeitdaten komprimieren und im Ext-EEPROM-Schreibpuffer speichern
        ldi     zl, low(xewbuf)
        ldi     zh, high(xewbuf)        ;Zeiger auf Ext-EEPROM-Schreibpuffer
        ldi     r16, 0x5a               ;Prfzeichen laden
        sts     xewbuf + 4, r16         ;und in den Ext-EEPROM-Schreibpuffer legen
;
	ldi	r20, low(cfbsta)
	ldi	r21, high(cfbsta)
	ldi	r22, byte3(cfbsta)	;Anfangsadresse Backup-Bereich
	ldi	r16, low(eeprom)
	ldi	r17, high(eeprom)	        ;Beginn Konfigurations-Daten im EEPROM
	ldi	r24, low(eprcrc - eeprom + 6)   ;Anzahl Konfigurations-Daten im EEPROM
	ldi	r25, high(eprcrc - eeprom + 6)  ;zuzglich CRC-Byte und 5 Zeit-Bytes
	movw	r12, r16			;EEPROM-Adresse sichern
;
xwc010:	ldi	r16, xeeadr		;Slave-Adresse + Write-Bit
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
;
	mov	r15, r16		;kopieren
	rcall	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xwc080			;ACK empfangen? nein -> Fehler
	mov	r16, r21		;Adresse H-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xwc080			;nein -> Fehler
	mov	r16, r20		;Adresse L-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xwc080			;nein -> Fehler
;
xwc020: ldi     r18, low(6)
        ldi     r19, high(6)            ;Vergleichs-Wert 6 laden
        cp      r24, r18
        cpc     r25, r19                ;Vergleichs-Wert unterschritten?
        brcc    xwc030                  ;nein -> weiter Konfigurations-Daten bearbeiten
        ld      r16, z+                 ;sonst Zeitdaten holen
        rjmp    xwc040                  ;und ausgeben

xwc030:	movw	r16, r12		;EEPROM-Adresse restaurieren
	rcall	eeread			;Byte aus EEPROM lesen
	movw	r12, r16		;EEPROM-Adresse wieder sichern
	mov	r16, r18		;gelesenes Byte kopieren
;
xwc040:	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xwc080			;nein -> Fehler
	inc	r20			;Adresse L erhhen
	brne	xwc050			;bertrag? nein -> weiter
	inc	r21			;sonst Adresse H erhhen
	brne	xwc050			;bertrag? nein -> weiter
	inc	r22			;sonst Adresse B3 erhhen
xwc050:	mov	r16, r20		;L-Byte holen
	andi	r16, 0x7f		;Bits 6-0 filtern, EEPROM-Page-Wechsel?
	brne	xwc060			;nein -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	sbiw	r24, 1			;alle Bytes kopiert?
	breq	xwc070			;ja -> Ende
	rjmp	xwc010			;sonst neuer Schreibzyklus
;
xwc060:	sbiw	r24, 1			;alle Bytes kopiert?
	brne	xwc020			;nein -> Schleife
	rcall	twisto			;sonst TWI Stop Condition
xwc070:	clc				;Carry = 0, Schreiben ok
	ret
xwc080:	sec				;Carry = 1, Fehler
	ret
;
; Unterprogramm zum Vergleichen der Konfigurationsdaten aus dem Externen EEPROM und dem Einlesen des Backup-Datums
; Register: r12, r13, r15, r16, r17, r18, r20, r21, r22, r24, r25, z
;           r14   -> 0 = Vergleich ok, 1 = Vergleichsfehler
;	    Carry -> 0 = ok, 1 = Fehler
;
xvconf: ldi     zl, low(xerbuf)
        ldi     zh, high(xerbuf)        ;Zeiger auf Ext-EEPROM-Lesepuffer
        clr     r14                     ;Flag fr Vergleichsfehler lschen
	ldi	r20, low(cfbsta)
	ldi	r21, high(cfbsta)
	ldi	r22, byte3(cfbsta)	;Anfangsadresse Backup-Bereich
	ldi	r16, low(eeprom)
	ldi	r17, high(eeprom)	        ;Beginn Konfigurations-Daten im EEPROM
	ldi	r24, low(eprcrc - eeprom + 6)   ;Anzahl Konfigurations-Daten im EEPROM
	ldi	r25, high(eprcrc - eeprom + 6)  ;zuzglich CRC-Byte und 5 Zeit-Bytes
	movw	r12, r16			;EEPROM-Adresse sichern
;
xvc010:	ldi	r16, xeeadr		;Slave-Adresse + Write-Bit
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
;
	mov	r15, r16		;kopieren
	rcall	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xvc090			;ACK empfangen? nein -> Fehler
	mov	r16, r21		;Adresse H-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xvc090			;nein -> Fehler
	mov	r16, r20		;Adresse L-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xvc090			;nein -> Fehler
;
	rcall	twista			;TWI Start Condition
	cpi	r17, 0x10		;Repeat Start ok?
	brne	xvc090			;nein -> Fehler
	ldi	r16, xeeadr + 1		;Slave-Adresse + Read-Bit		
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xvc090			;nein -> Fehler
;
xvc020:	set				;ACK zum Slave senden
	cpi	r20, 0xff		;wird bertrag L erwartet?
	brne	xvc030			;nein -> weiter
	cpi	r21, 0xff		;wird bertrag H erwartet?
	brne	xvc030			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xvc030:	cpi	r25, 0			;Zhler H-Byte = 0?
	brne	xvc040			;nein -> weiter
	cpi	r24, 1			;letztes Byte?
	brne	xvc040			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xvc040:	rcall	twircv			;Byte vom Ext-EEPROM lesen
	andi	r17, 0xf0		;ACK-Status ausblenden
	cpi	r17, 0x50		;Empfang ok?
	brne	xvc090			;nein -> Fehler
;
        ldi     r18, low(6)
        ldi     r19, high(6)            ;Vergleichs-Wert 6 laden
        cp      r24, r18
        cpc     r25, r19                ;Vergleichs-Wert unterschritten?
        brcc    xvc050                  ;nein -> weiter Konfigurations-Daten vergleichen
        st      z+, r16                 ;sonst Zeitdaten speichern
        rjmp    xvc060
;
xvc050: mov     r19, r16                ;empfangenes Byte kopieren
	movw	r16, r12		;EEPROM-Adresse restaurieren
	rcall	eeread			;Byte aus EEPROM lesen
	movw	r12, r16		;EEPROM-Adresse wieder sichern
        cp      r18, r19                ;Byte mit Backup-Wert vergleichen
        breq    xvc060                  ;Byte ok? ja -> weiter
        ldi     r16, 1                  ;Flag fr Vergleichsfehler setzen
        mov     r14, r16                ;und speichern
;
xvc060:	inc	r20			;Adresse L erhhen
	brne	xvc070			;bertrag? nein -> weiter
	inc	r21			;Adresse H erhhen
	brne	xvc070			;bertrag? nein -> weiter
	inc	r22			;Adresse B3 erhhen
	sbiw	r24, 1			;alle Bytes gelesen?
	breq	xvc080			;ja -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	rjmp	xvc010			;Schleife
;
xvc070:	sbiw	r24, 1			;alle Bytes gelesen?
	brne	xvc020			;nein -> Schleife
xvc080:	rcall	twisto			;TWI Stop Condition
	clc				;Carry = 0, Lesen ok
	ret
xvc090:	sec				;Carry = 1, Fehler
	ret
;
; Unterprogramm zum Laden der Konfigurationsdaten aus dem Externen EEPROM
; Register: r12, r13, r15, r16, r17, r18, r20, r21, r22, r24, r25
;	    Carry -> 0 = ok, 1 = Fehler
;
xrconf:	ldi	r20, low(cfbsta)
	ldi	r21, high(cfbsta)
	ldi	r22, byte3(cfbsta)	;Anfangsadresse Backup-Bereich
	ldi	r16, low(eeprom)
	ldi	r17, high(eeprom)	        ;Beginn Konfigurations-Daten im EEPROM
	ldi	r24, low(eprcrc - eeprom + 1)   ;Anzahl Konfigurations-Daten im EEPROM
	ldi	r25, high(eprcrc - eeprom + 1)  ;zuzglich CRC-Byte
	movw	r12, r16			;EEPROM-Adresse sichern
;
xrc010:	ldi	r16, xeeadr		;Slave-Adresse + Write-Bit
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
;
	mov	r15, r16		;kopieren
	rcall	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xrc070			;ACK empfangen? nein -> Fehler
	mov	r16, r21		;Adresse H-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xrc070			;nein -> Fehler
	mov	r16, r20		;Adresse L-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xrc070			;nein -> Fehler
;
	rcall	twista			;TWI Start Condition
	cpi	r17, 0x10		;Repeat Start ok?
	brne	xrc070			;nein -> Fehler
	ldi	r16, xeeadr + 1		;Slave-Adresse + Read-Bit		
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xrc070			;nein -> Fehler
;
xrc020:	set				;ACK zum Slave senden
	cpi	r20, 0xff		;wird bertrag L erwartet?
	brne	xrc030			;nein -> weiter
	cpi	r21, 0xff		;wird bertrag H erwartet?
	brne	xrc030			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xrc030:	cpi	r25, 0			;Zhler H-Byte = 0?
	brne	xrc040			;nein -> weiter
	cpi	r24, 1			;letztes Byte?
	brne	xrc040			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xrc040:	rcall	twircv			;Byte vom Ext-EEPROM lesen
	andi	r17, 0xf0		;ACK-Status ausblenden
	cpi	r17, 0x50		;Empfang ok?
	brne	xrc070			;nein -> Fehler
	mov	r18, r16		;empfangenes Byte kopieren
	movw	r16, r12		;EEPROM-Adresse restaurieren
	rcall	eewrit			;Byte in EEPROM schreiben
	movw	r12, r16		;EEPROM-Adresse wieder sichern
;
	inc	r20			;Adresse L erhhen
	brne	xrc050			;bertrag? nein -> weiter
	inc	r21			;Adresse H erhhen
	brne	xrc050			;bertrag? nein -> weiter
	inc	r22			;Adresse B3 erhhen
	sbiw	r24, 1			;alle Bytes gelesen?
	breq	xrc060			;ja -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	rjmp	xrc010			;Schleife
;
xrc050:	sbiw	r24, 1			;alle Bytes gelesen?
	brne	xrc020			;nein -> Schleife
xrc060:	rcall	twisto			;TWI Stop Condition
	clc				;Carry = 0, Lesen ok
	ret
xrc070:	sec				;Carry = 1, Fehler
	ret
;
; Unterprogramm zum Schreiben des Pufferinhaltes in das Externe EEPROM
; Register: r15, r16, r17, r18, r19, y
;	    r20-r22 <- aktuelle Adresse im externen EEPROM
;	    r23	    <- Anzahl der zu sendenden Bytes (maximal 145)
;	    Carry   -> 0 = ok, 1 = Fehler aufgetreten
;
xwrite:	ldi	yl, low(xewbuf)
	ldi	yh, high(xewbuf)	;Zeiger auf Ext-EEPROM-Schreib-Puffer setzen
xwr010:	ldi	r16, xeeadr		;Slave-Adresse + Write-Bit (0)
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
;
	mov	r15, r16		;Adresse kopieren
	rcall	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xwr060			;ACK empfangen? nein -> Fehler
	mov	r16, r21		;Adresse H-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xwr060			;nein -> Fehler
	mov	r16, r20		;Adresse L-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xwr060			;nein -> Fehler
;
xwr020:	ld	r16, y+			;Byte aus EEPROM-Puffer holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xwr060			;nein -> Fehler
	inc	r20			;Adresse L erhhen
	brne	xwr030			;bertrag? nein -> weiter
	inc	r21			;sonst Adresse H erhhen
	brne	xwr030			;bertrag? nein -> weiter
	inc	r22			;sonst Adresse B3 erhhen
xwr030:	mov	r16, r20		;L-Byte holen
	andi	r16, 0x7f		;Bits 6-0 filtern, EEPROM-Page-Wechsel?
	brne	xwr040			;nein -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	dec	r23			;alle Bytes ausgegeben?
	breq	xwr050			;ja -> Ende
	rjmp	xwr010			;sonst neuer Schreibzyklus
;
xwr040:	dec	r23			;alle Bytes ausgegeben?
	brne	xwr020			;nein -> Schleife
	rcall	twisto			;sonst TWI Stop Condition
xwr050:	clc				;Carry = 0, Schreiben ok
	ret
xwr060:	sec				;Carry = 1, Fehler
	ret
;
; Unterprogramm zum Lesen des Pufferinhaltes aus dem Externen EEPROM in den Lesepuffer fr die PC-Datenausgabe
; Register: r15, r16, r17, r18, r19, y
;	    r20-r22 <- aktuelle Adresse im externen EEPROM
;	    r23	    <- Anzahl der zu lesenden Bytes (maximal 145)
;	    Carry   -> 0 = ok, 1 = Fehler
;
xread:	ldi	yl, low(xerbuf)
	ldi	yh, high(xerbuf)	;Zeiger auf Ext-EEPROM Lese-Puffer setzen
xre010:	ldi	r16, xeeadr		;Slave-Adresse + Write-Bit (0)
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
;
	mov	r15, r16		;kopieren
	rcall	twitio			;zum Ext-EEPROM senden (15ms)
	brcs	xre070			;ACK empfangen? nein -> Fehler
	mov	r16, r21		;Adresse H-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xre070			;nein -> Fehler
	mov	r16, r20		;Adresse L-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xre070			;nein -> Fehler
;
	rcall	twista			;TWI Start Condition
	cpi	r17, 0x10		;Repeat Start ok?
	brne	xre070			;nein -> Fehler
	ldi	r16, xeeadr + 1		;Slave-Adresse + Read-Bit (1)
        mov     r17, r22                ;Adress-Bits 17+16 holen
        rol     r17                     ;auf EEPROM-Adress-Position schieben
        andi    r17, 0x06               ;filtern und
        or      r16, r17                ;mit Slave-Adresse verknpfen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xre070			;nein -> Fehler
;
xre020:	set				;ACK zum Slave senden
	cpi	r20, 0xff		;wird bertrag L erwartet?
	brne	xre030			;nein -> weiter
	cpi	r21, 0xff		;wird bertrag H erwartet?
	brne	xre030			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xre030:	cpi	r23, 1			;letztes Byte?
	brne	xre040			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xre040:	rcall	twircv			;Byte vom Ext-EEPROM lesen
	andi	r17, 0xf0		;ACK-Status ausblenden
	cpi	r17, 0x50		;Empfang ok?
	brne	xre070			;nein -> Fehler
	st	y+, r16			;sonst Byte im Puffer speichern
	inc	r20			;Adresse L erhhen
	brne	xre050			;bertrag? nein -> weiter
	inc	r21			;Adresse H erhhen
	brne	xre050			;bertrag? nein -> weiter
	inc	r22			;Adresse B3 erhhen
	dec	r23			;alle Bytes gelesen?
	breq	xre060			;ja -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	rjmp	xre010			;Schleife
;
xre050:	dec	r23			;alle Bytes gelesen?
	brne	xre020			;nein -> Schleife
xre060:	rcall	twisto			;TWI Stop Condition
	clc				;Carry = 0, Lesen ok
	ret
xre070:	sec				;Carry = 1, Fehler
	ret
;
; Unterprogramm zum Lesen des Pufferinhaltes aus dem Externen EEPROM in den Lesepuffer fr die Anzeige auf dem LCD
; Register: r15, r16, r17, r18, r19, y
;	    r20-r22 <- aktuelle Adresse im externen EEPROM
;	    r23	    <- Anzahl der zu lesenden Bytes (maximal 145)
;	    Carry   -> 0 = ok, 1 = Fehler
;
xdread:	ldi	yl,low (xedbuf)
	ldi	yh,high (xedbuf)	;Zeiger auf Ext-EEPROM Lese-Puffer fr LCD setzen
	rjmp	xre010
;
; Unterprogramm zum Durchsuchen des externen EEPROMs nach der aktuellen Schreibposition fr den nchsten Eintrag
; Register: r15, r16, r17, r18, r23, r24, r25, y
;	    r4-r7      Zeit-Zwischenspeicher des ltesten Datensatzes
;	    r8-r11     Rechenregister fr Zeiten-Vergleich
;	    r12-r14 -> ermittelte Position des ltesten Eintrages
;	    r19     <- Datensatz-Offset
;	    r20-r22 <- aktuelle Adresse im EEPROM-Bereich
;	    Carry   -> 0 = ok, 1 = Fehler
;
xswpos:	clr	r4			;Zeit-Zwischenspeicher fr
	clr	r5			;niedrigsten gelesenen Wert
	movw	r6, r4			;initialisieren
	movw	r12, r20
	mov	r14, r22		;Adress-Zeiger initialisieren
xsw010:	ldi	yl, low(xerbuf)
	ldi	yh, high(xerbuf)	;Zeiger auf Ext-EEPROM Lese-Puffer setzen
	ldi	r18, 4			;4 Bytes lesen
	mov	r23, r20
	mov	r24, r21
	mov	r25, r22		;aktuelle Adresse zwischenspeichern
xsw020:	rcall	xesadr			;Adresse an Ext-EEPROM senden
	brcc	xsw040			;ok? ja -> weiter
xsw030:	rcall	twisto			;TWI bertragung beenden
	sec				;Carry = 1 Fehler
	ret
;
xsw040:	set				;ACK zum Slave senden
	cpi	r20, 0xff		;wird bertrag L erwartet?
	brne	xsw050			;nein -> weiter
	cpi	r21, 0xff		;wird bertrag H erwartet?
	brne	xsw050			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xsw050:	cpi	r18, 1			;letztes Byte?
	brne	xsw060			;nein -> weiter
	clt				;sonst kein ACK zum Slave senden
xsw060:	rcall	twircv			;Byte vom Ext-EEPROM lesen
	andi	r17, 0xf0		;ACK-Status ausblenden
	cpi	r17, 0x50		;Empfang ok?
	brne	xsw030			;nein -> Ende
	st	y+, r16			;Byte im Puffer speichern
	inc	r20			;Adresse L erhhen
	brne	xsw070			;bertrag? nein -> weiter
	inc	r21			;Adresse H erhhen
	brne	xsw070			;bertrag? nein -> weiter
	inc	r22			;Adresse B3 erhhen
	dec	r18			;alle Bytes gelesen?
	breq	xsw080			;ja -> weiter
	rcall	twisto			;sonst TWI Stop Condition
	rjmp	xsw020			;Schleife
;
xsw070:	dec	r18			;alle Bytes gelesen?
	brne	xsw040			;nein -> Schleife
xsw080:	rcall	twisto			;TWI Stop Condition
	mov	r20, r23
	mov	r21, r24
	mov	r22, r25		;zwischengespeicherte Adresse holen
	ld	r11, -y
	ld	r10, -y
	ld	r9, -y			;gelesene Zeitdaten holen
	ld	r8, -y			;(insgesamt 4 Bytes)
	mov	r16, r8
	and	r16, r9
	and	r16, r10
	and	r16, r11		;alles Bytes = 0xff?
	cpi	r16, 0xff		;(noch nicht genutzter Speicher)
	brne	xsw110			;nein -> Zeitwert vergleichen
xsw090:	movw	r12, r20		;sonst Speicheradresse verwenden
	mov	r14, r22		;(insgesamt 3 Bytes)
xsw100:	clc				;Carry = 0, normales Ende
	ret
;
xsw110:	sub	r8, r4
	sbc	r9, r5			;ist der gelesene Zeitwert
	sbc	r10, r6			;niedriger (lter) als der
	sbc	r11, r7			;gespeicherte Zeitwert?
	brcs	xsw090			;ja -> Position gefunden
	ld	r4, y+
	ld	r5, y+
	ld	r6, y+			;sonst gelesene Zeitdaten als
	ld	r7, y+			;neuen Wert speichern
;
	clr	r16			;H-Byte = 0 fr Berechnung
	add	r20, r19		;Offset zur Adresse addieren
	adc	r21, r16		;bertrag addieren
	adc	r22, r16		;bertrag addieren
	lds	r16, xetemp
	lds	r17, xetemp + 1		;Endadresse holen
	lds	r18, xetemp + 2		;(insgesamt 3 Bytes)
	sub	r16, r20
	sbc	r17, r21		;berechnete Adresse noch im
	sbc	r18, r22		;gltigen Speicherbereich?
	brcs	xsw100			;nein -> Ende
	rjmp	xsw010			;sonst nchsten Datensatz lesen
;
; Unterprogramm zur Ausgabe der Adresse an das Externe EEPROM zum Lesen
; Register: r16, r17
;	    r20-r22 <- aktuelle Adresse im EEPROM-Bereich
;	    Carry   -> 0 = ok, 1 = Fehler
;
xesadr:	rcall	twista			;TWI Start Condition
	cpi	r17, 0x08		;Start ok?
	brne	xesa10			;nein -> Fehler
	ldi	r16, xeeadr		;Slave-Adresse + Write-Bit
	bst	r22, 0			;Adress-Bit 16 holen und als
	bld	r16, 1			;EEPROM-Select-Bit einsetzen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x18		;ACK empfangen?
	brne	xesa10			;nein -> Fehler
	mov	r16, r21		;Adresse H-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xesa10			;nein -> Fehler
	mov	r16, r20		;Adresse L-Byte holen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x28		;ACK empfangen?
	brne	xesa10			;nein -> Fehler
;
	rcall	twista			;TWI Start Condition
	cpi	r17, 0x10		;Repeat Start ok?
	brne	xesa10			;nein -> Fehler
	ldi	r16, xeeadr + 1		;Slave-Adresse + Read-Bit		
	bst	r22, 0			;Adress-Bit 16 holen und als
	bld	r16, 1			;EEPROM-Select-Bit einsetzen
	rcall	twisnd			;zum Ext-EEPROM senden
	cpi	r17, 0x40		;ACK empfangen?
	brne	xesa10			;nein -> Fehler
	clc				;Carry = 0 alles ok
	ret
;
xesa10:	sec				;Carry = 1 Fehler
	ret
;
; Unterprogramm TWI (I2C) Start, Slave-Adresse senden, Timeout 15ms
; Register: r16, r17, r18, r19
;	    r15 <- Slave-Adresse
;	    Carry -> 0 = ok, 1 = Fehler
;
twitio:	ldi	r19, high(600)
	ldi	r18, low(600)		;Zeitkonstante ca. 15ms
twit10:	rcall	twista			;TWI Start Condition
	mov	r16, r15		;Slave-Adresse kopieren
	rcall	twisnd			;senden
	cpi	r17, 0x18		;ACK empfangen?
	brne	twit20			;nein -> weiter
	clc				;sonst Carry lschen (ok)
	ret
twit20:	rcall	twisto			;TWI Stop Condition
	dec	r18			;Schleifenzhler L vermindern
	brne	twit10			;Ende? nein -> wiederholen
	dec	r19			;Schleifenzhler H vermindern
	brne	twit10			;Ende? nein -> wiederholen
	sec				;sonst Carry setzen (Fehler)
	ret
;
; Unterprogramm TWI (I2C) Start Condition
; Register: r16
;
twista:	ldi	r16, (1 << twint) | (1 << twsta) | (1 << twen)
	sts	twcr, r16		;Start Condition senden
	rjmp	twis10			;warten bis ausgefhrt
;
; Unterprogramm TWI (I2C) Senden
; Register: r16 <- zu sendendes Byte
;	    r17 -> Status
;
twisnd:	sts	twdr, r16		;Byte in Ausgaberegister
	ldi	r16, (1 << twint) | (1 << twen)
	sts	twcr, r16		;Byte senden
twis10:	lds	r16, twcr		;Status holen
	sbrs	r16, twint		;Kommando ausgefhrt?
	rjmp	twis10			;nein -> warten
	lds	r17, twsr		;sonst Statusregister lesen
	ret
;
; Unterprogramm TWI (I2C) Empfangen
; Register: T   <- Acknowledge (1), Not Acknowledge (0)
;	    r16 -> empfangenes Byte
;	    r17 -> Status
;
twircv:	ldi	r16, (1 << twint) | (1 << twen)
	bld	r16, twea
	sts	twcr, r16		;Byte senden
twir10:	lds	r16, twcr		;Status holen
	sbrs	r16, twint		;Kommando ausgefhrt?
	rjmp	twir10			;nein -> warten
	lds	r16, twdr		;sonst empfangene Daten holen
	lds	r17, twsr		;Statusregister lesen
	ret
;
; Unterprogramm TWI (I2C) Stop Condition
; Register: r16
;
twisto:	ldi	r16, (1 << twint) | (1 << twsto) | (1 << twen)
	sts	twcr, r16		;Stop Condition senden
	ret
;
; Unterprogramm Zeitdaten vom SRAM in die RTC bertragen
; Register: r0, r1, r16, r17, r18
;	    Carry -> 0 = ok, 1 = Fehler
;
tiwrtc:	cli				;Interrupts sperren
	lds	r16, t1cnt2
	sts	ct1cnt, r16		;Timer1 Int-Zhler 2 kopieren
	lds	r16, secndl
	sts	csecdl, r16		;Sekunde Einer kopieren
	lds	r16, secndh
	sts	csecdh, r16		;Sekunde Zehner kopieren
	lds	r16, minutl
	sts	cminul, r16		;Minute Einer kopieren
	lds	r16, minuth
	sts	cminuh, r16		;Minute Zehner kopieren
	lds	r16, hourl
	sts	chourl, r16		;Stunde Einer kopieren
	lds	r16, hourh
	sts	chourh, r16		;Stunde Zehner kopieren
	lds	r16, dayw
	sts	cdayw, r16		;Wochentag kopieren
	lds	r16, dayl
	sts	cdayl, r16		;Kalendertag Einer kopieren
	lds	r16, dayh
	sts	cdayh, r16		;Kalendertag Zehner kopieren
	lds	r16, monthl
	sts	cmontl, r16		;Monat Einer kopieren
	lds	r16, monthh
	sts	cmonth, r16		;Monat Zehner kopieren
	lds	r16, yearl
	sts	cyearl, r16		;Jahr Einer kopieren
	lds	r16, yearh
	sts	cyearh, r16		;Jahr Zehner kopieren
	sei				;Interrupts wieder freigeben
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2csta			;I2C Start Condition + Adresse
	brcs	tiw100			;ACK empfangen? nein -> Fehler
	ldi	r16, rtflag		;RTC-RAM, Flag-Adresse
	rcall	i2csen			;senden
	brcs	tiw100			;ACK empfangen? nein -> Fehler
	clr	r16			;Flag = 0
	rcall	i2csen			;senden
	brcs	tiw100			;ACK empfangen? nein -> Fehler
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tiw100			;ACK empfangen? nein -> Fehler
	clr	r16			;RTC-RAM, Adresse = 0
	rcall	i2csen			;senden
	brcs	tiw100			;ACK empfangen? nein -> Fehler
	ldi	r16, 0x80		;RTC-Adresse = 0, Zhler stoppen
	rcall	i2csen			;senden
	brcs	tiw100			;ACK empfangen? nein -> Fehler
;
	lds	r18, ct1cnt		;Vorteiler 2 holen (1/50s)
	clr	r16			;Anfangswert = 0, Umrechnen in
	ldi	r17, 16			;Zehntel-Sekunden Increment
tiw010:	subi	r18, 5			;5 Zehntel subtrahieren
	brcs	tiw020			;berlauf? ja -> weiter
	add	r16, r17		;sonst Zehntel-Ergebnis erhhen
	rjmp	tiw010			;Schleife
tiw020:	rcall	i2csen			;Zehntel im oberen Nibble senden
	brcs	tiw100			;ACK empfangen? nein -> Fehler
;
	lds	r17, csecdh		;Sekunden Zehner holen
	swap	r17			;in oberes Nibble legen
	lds	r16, csecdl		;Sekunden Einer holen
	or	r16, r17		;Zehner und Einer verknpfen
	rcall	i2csen			;senden
	brcs	tiw100			;ACK empfangen? nein -> Fehler
;
	lds	r17, cminuh		;Minuten Zehner holen
	swap	r17			;in oberes Nibble legen
	lds	r16, cminul		;Minuten Einer holen
	or	r16, r17		;Zehner und Einer verknpfen
	rcall	i2csen			;senden
	brcs	tiw100			;ACK empfangen? nein -> Fehler
;
	lds	r17, chourh		;Stunden Zehner holen
	swap	r17			;in oberes Nibble legen
	lds	r16, chourl		;Stunden Einer holen
	or	r16, r17		;Zehner und Einer verknpfen
	rcall	i2csen			;senden
	brcc	tiw110			;ACK empfangen? ja -> weiter
tiw100:	rjmp	tiwerr			;sonst Fehler
;
tiw110:	lds	r16, cyearh		;Jahr Zehner holen
	ldi	r17, 10			;mit 10 multiplizieren
	mul	r16, r17		;Umrechnung in Binrwert
	lds	r16, cyearl		;Jahr Einer holen
	add	r16, r0			;Zehner und Einer addieren
	ror	r16
	ror	r16			;Bits 1,0 an Position 7,6
	ror	r16			;schieben
	andi	r16, 0xc0		;und filtern
	lds	r17, cdayh		;Tag Zehner holen
	swap	r17			;in oberes Nibble legen
	or	r17, r16		;Jahr und Tag Zehner verknpfen
	lds	r16, cdayl		;Tag Einer holen
	or	r16, r17		;Zehner und Einer verknpfen
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
;
	lds	r16, cdayw		;Wochentag holen
	lsl	r16			;in Bits 3-1 schieben
	lds	r17, cmonth		;Monat Zehner holen
	or	r17, r16		;Monat und Wochentag verknpfen
	swap	r17			;in oberes Nibble legen
	lds	r16,cmontl		;Monat Einer holen
	or	r16, r17		;Zehner und Einer verknpfen
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
	clr	r16			;RTC-RAM, Adresse = 0
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
	clr	r16			;RTC-Adresse = 0, Zhler starten
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtyear		;RTC-RAM, Jahr-Adresse
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
	lds	r17, cyearh		;Jahr Zehner holen
	swap	r17			;in oberes Nibble legen
	lds	r16, cyearl		;Jahr Einer holen
	or	r16, r17		;Zehner und Einer verknpfen
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtflag		;RTC-RAM, Flag-Adresse
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
	ldi	r16, 0x5a		;Flag = Uhrzeit gltig
	rcall	i2csen			;senden
	brcs	tiwerr			;ACK empfangen? nein -> Fehler
;
	rcall	i2csto			;I2C Stop Condition
	clc				;Carry lschen (ok)
	ret
;
tiwerr:	rcall	i2csto			;I2C Stop Condition
	sec				;Carry setzen (Fehler)
	ret
;
; Unterprogramm Zeitdaten von der RTC in das SRAM bertragen
; Register: r0, r1, r15, r17, r18, r19
;	    r16 -> Inhalt das Flagbytes (wenn Carry = 0)
;	    Carry -> 0 = ok, 1 = Fehler
;
tirrtc:	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2csta			;I2C Start Condition + Adresse
	brcs	tir100			;ACK empfangen? nein -> Fehler
	ldi	r16, 1			;RTC-RAM, Adresse 1
	rcall	i2csen			;senden
	brcs	tir100			;ACK empfangen? nein -> Fehler
	ldi	r16, rtcadr + 1		;I2C Basisadresse RTC + Read
	rcall	i2crst			;I2C Repeat Start Condition
	brcc	tir110			;ACK empfangen? ja -> weiter
tir100:	rjmp	tirerr			;sonst Fehler
;
tir110:	clc				;ACK zum Slave senden
	rcall	i2crcv			;Zehntel/Hundertstel empfangen
	swap	r16			;Zehntel in unteres Nibble
	andi	r16, 0x0f		;legen und filtern
	ldi	r17, 5			;mit 5
	mul	r16, r17		;multiplizieren
	sts	ct1cnt, r0		;und als 1/50s speichern
;
	clc				;ACK zum Slave senden
	rcall	i2crcv			;Sekunden empfangen
	mov	r17, r16		;Wert kopieren
	swap	r17			;Zehner in unteres Nibble
	andi	r17, 0x0f		;legen und filtern
	sts	csecdh, r17		;Sekunden Zehner speichern
	andi	r16, 0x0f		;Einer filtern
	sts	csecdl, r16		;Sekunden Einer speichern
;
	clc				;ACK zum Slave senden
	rcall	i2crcv			;Minuten empfangen
	mov	r17, r16		;Wert kopieren
	swap	r17			;Zehner in unteres Nibble
	andi	r17, 0x0f		;legen und filtern
	sts	cminuh, r17		;Minuten Zehner speichern
	andi	r16, 0x0f		;Einer filtern
	sts	cminul, r16		;Minuten Einer speichern
;
	clc				;ACK zum Slave senden
	rcall	i2crcv			;Stunden empfangen
	mov	r17, r16		;Wert kopieren
	swap	r17			;Zehner in unteres Nibble
	andi	r17, 0x03		;legen und filtern
	sts	chourh, r17		;Stunden Zehner speichern
	andi	r16, 0x0f		;Einer filtern
	sts	chourl, r16		;Stunden Einer speichern
;
	clc				;ACK zum Slave senden
	rcall	i2crcv			;Tage empfangen
	mov	r17, r16		;Wert kopieren
	swap	r17			;Zehner in unteres Nibble
	mov	r19, r17		;Wert kopieren
	lsr	r19
	lsr	r19			;Jahr an Position 1,0 schieben
	andi	r19, 0x01		;LSB filtern
	andi	r17, 0x03		;legen und filtern
	sts	cdayh, r17		;Tag Zehner speichern
	andi	r16, 0x0f		;Einer filtern
	sts	cdayl, r16		;Tag Einer speichern
;
	sec				;kein ACK zum Slave senden
	rcall	i2crcv			;Monate/Wochentag empfangen
	mov	r17, r16		;Wert kopieren
	swap	r17			;Zehner/Tag in unteres Nibble
	mov	r18, r17		;Wert kopieren
	lsr	r18			;Tag an Position 2-0 schieben
	andi	r18, 0x07		;Wochentag filtern
	sts	cdayw, r18		;Wochentag speichern
	andi	r17, 0x01		;Monat Zehner filtern
	sts	cmonth, r17		;Monat Zehner speichern
	andi	r16, 0x0f		;Einer filtern
	sts	cmontl, r16		;Monat Einer speichern
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tirerr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtyear		;RTC-RAM, Jahr-Adresse
	rcall	i2csen			;senden
	brcs	tirerr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtcadr + 1		;I2C Basisadresse RTC + Read
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tirerr			;ACK empfangen? nein -> Fehler
	sec				;kein ACK zum Slave senden
	rcall	i2crcv			;Jahr empfangen
	mov	r17, r16		;Wert kopieren
	swap	r17			;Zehner in unteres Nibble
	andi	r17, 0x0f		;legen und filtern
	sts	cyearh, r17		;Jahr Zehner speichern
	andi	r16, 0x0f		;Einer filtern
	sts	cyearl, r16		;Jahr Einer speichern
	mov	r18, r16		;Einer kopieren
	andi	r18, 0x01		;Einer LSB filtern
	eor	r18, r19		;LSB identisch?
	breq	tir300			;ja -> Jahr ok, weiter
;
	inc	r16			;sonst Jahr Einer erhhen
	sts	cyearl, r16		;Jahr Einer speichern
	cpi	r16, 10			;bertrag?
	brcs	tir200			;nein -> weiter
	clr	r16			;sonst Jahr Einer zurcksetzen
	sts	cyearl, r16		;Jahr Einer speichern
	inc	r17			;Jahr Zehner erhhen
	sts	cyearh, r17		;Jahr Zehner speichern
	cpi	r17, 10			;bertrag?
	brcs	tir200			;nein -> weiter
	clr	r17			;sonst Jahr Zehner zurcksetzen
	sts	cyearh, r17		;Jahr Zehner speichern
tir200:	swap	r17			;in oberes Nibble legen
	or	r16, r17		;Zehner und Einer verknpfen
	mov	r18, r16		;Jahr kopieren
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tirerr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtyear		;RTC-RAM, Jahr-Adresse
	rcall	i2csen			;senden
	brcs	tirerr			;ACK empfangen? nein -> Fehler
	mov	r16, r18		;Jahr holen
	rcall	i2csen			;senden
	brcs	tirerr			;ACK empfangen? nein -> Fehler
;
tir300:	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	tirerr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtflag		;RTC-RAM, Flag-Adresse
	rcall	i2csen			;senden
	brcs	tirerr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtcadr + 1		;I2C Basisadresse RTC + Read
	rcall	i2crst			;I2C Repeat Start Condition
	brcc	tirend			;ACK empfangen? ja -> weiter
;
tirerr:	rcall	i2csto			;sonst I2C Stop Condition
	sec				;Carry setzen (Fehler)
	ret
;
tirend:	sec				;kein ACK zum Slave senden
	rcall	i2crcv			;Flag-Byte empfangen
	rcall	i2csto			;I2C Stop Condition
	cli				;Interrupts sperren
	lds	r17, ct1cnt
	sts	t1cnt2, r17		;Timer1 Int-Zhler 2 kopieren
	lds	r17, csecdl
	sts	secndl, r17		;Sekunde Einer kopieren
	lds	r17, csecdh
	sts	secndh, r17		;Sekunde Zehner kopieren
	lds	r17, cminul
	sts	minutl, r17		;Minute Einer kopieren
	lds	r17, cminuh
	sts	minuth, r17		;Minute Zehner kopieren
	lds	r17, chourl
	sts	hourl, r17		;Stunde Einer kopieren
	lds	r17, chourh
	sts	hourh, r17		;Stunde Zehner kopieren
	lds	r17, cdayw
	sts	dayw, r17		;Wochentag kopieren
	lds	r17, cdayl
	sts	dayl, r17		;Kalendertag Einer kopieren
	lds	r17, cdayh
	sts	dayh, r17		;Kalendertag Zehner kopieren
	lds	r17, cmontl
	sts	monthl, r17		;Monat Einer kopieren
	lds	r17, cmonth
	sts	monthh, r17		;Monat Zehner kopieren
	lds	r17, cyearl
	sts	yearl, r17		;Jahr Einer kopieren
	lds	r17, cyearh
	sts	yearh, r17		;Jahr Zehner kopieren
	sei				;Interrupts wieder freigeben
	clc				;Carry lschen (ok)
	ret
;
; Unterprogramm zum Schreiben eines Backups (Minimal-, Maximal- und Mittelwerte) in das RTC-RAM
; Register: r16, r17, r18, r19, r20, r21, r22, x
;	    Carry -> 0 = ok, 1 = Fehler
;
i2back:	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2csta			;I2C Start Condition + Adresse
	brcc	i2ba10			;ACK empfangen? ja -> weiter
	rjmp	i2berr			;sonst Fehler
i2ba10:	ldi	r16, rtbflg		;RTC-RAM, Adresse Backup-Flag
	rcall	i2csen			;senden
	brcc	i2ba20			;ACK empfangen? ja -> weiter
	rjmp	i2berr			;sonst Fehler
i2ba20:	clr	r16			;Flag = 0 setzen, Backup ungltig
	rcall	i2csen			;senden
	brcc	i2ba30			;ACK empfangen? ja -> weiter
	rjmp	i2berr			;sonst Fehler
;
i2ba30:	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Repeat Start Condition + Adresse
	brcc	i2ba40			;ACK empfangen? ja -> weiter
	rjmp	i2berr			;sonst Fehler
i2ba40:	ldi	r16, rtbval		;Adresse der Minimal-, Maximal-, und Mittelwerte
	rcall	i2csen			;senden
	brcc	i2ba60			;ACK empfangen? ja -> weiter
i2ba50:	rjmp	i2berr			;sonst Fehler
;
; Minimalwerte komprimieren und als 12-Bit-Werte paarweise in jeweils 3 Bytes speichern (Low1, Low2, High2|High1)
;
i2ba60:	ldi	r19, 31 		;31 Werte bearbeiten
	ldi	xl, low(semin2)
	ldi	xh, high(semin2)	;Zeiger auf Minimalwerte fr Log
;
i2ba70:	ld	r16, x+			;Minimalwert Low-Byte holen
	rcall	i2csen			;und senden
	brcs	i2ba50			;ACK empfangen? nein -> Fehler
        bst     r19, 0                  ;Wertepaar komplett?
        brtc    i2ba80                  ;ja -> weiter
	ld	r18, x+			;sonst Minimalwert High-Byte zwischenspeichern
        cpi     r18, 0x80               ;Wert ungltig?
        brne    i2baa0                  ;nein -> weiter
        ldi     r18, 0x08               ;sonst Kennung auf untere 4 Bit legen
        rjmp    i2baa0
;
i2ba80: ld      r16, x+                 ;Minimalwert High-Byte holen
        cpi     r16, 0x80               ;Wert ungltig?
        breq    i2ba90                  ;ja -> Nibble-Tausch berspringen
        swap    r16                     ;4 Bits verwenden und auf oberes Nibble legen
i2ba90: andi    r16, 0xf0               ;oberes Bibble filtern
        andi    r18, 0x0f               ;obere 4 Bits vom vorherigen High-Wert filtern
        or      r16, r18                ;beide High-Bytes verknpfen
	rcall	i2csen			;und senden
	brcs	i2ba50			;ACK empfangen? nein -> Fehler
;
i2baa0:	dec	r19			;alle Werte bearbeitet?
	brne	i2ba70			;nein -> Schleife
        mov     r16, r18                ;High-Byte vom letzten Wert holen
	rcall	i2csen			;und direkt senden
	brcs	i2ba50			;ACK empfangen? nein -> Fehler
;
; Maximalwerte komprimieren und als 12-Bit-Werte paarweise in jeweils 3 Bytes speichern (Low1, Low2, High2|High1)
;
	ldi	r19, 31 		;31 Werte bearbeiten
	ldi	xl, low(semax2)
	ldi	xh, high(semax2)	;Zeiger auf Maximalwerte fr Log
;
i2bab0:	ld	r16, x+			;Maximalwert Low-Byte holen
	rcall	i2csen			;und senden
	brcs	i2ba50			;ACK empfangen? nein -> Fehler
        bst     r19, 0                  ;Wertepaar komplett?
        brtc    i2bac0                  ;ja -> weiter
	ld	r18, x+			;sonst Maximalwert High-Byte zwischenspeichern
        cpi     r18, 0x80               ;Wert ungltig?
        brne    i2bae0                  ;nein -> weiter
        ldi     r18, 0x08               ;sonst Kennung auf untere 4 Bit legen
        rjmp    i2bae0
;
i2bac0: ld      r16, x+                 ;Maximalwert High-Byte holen
        cpi     r16, 0x80               ;Wert ungltig?
        breq    i2bad0                  ;ja -> Nibble-Tausch berspringen
        swap    r16                     ;4 Bits verwenden und auf oberes Nibble legen
i2bad0: andi    r16, 0xf0               ;oberes Bibble filtern
        andi    r18, 0x0f               ;obere 4 Bits vom vorherigen High-Wert filtern
        or      r16, r18                ;beide High-Bytes verknpfen
	rcall	i2csen			;und senden
	brcs	i2ba50			;ACK empfangen? nein -> Fehler
;
i2bae0:	dec	r19			;alle Werte bearbeitet?
	brne	i2bab0			;nein -> Schleife
        mov     r16, r18                ;High-Byte vom letzten Wert holen
	rcall	i2csen			;und direkt senden
	brcs	i2ba50			;ACK empfangen? nein -> Fehler
;
; Mittelwerte ausrechnen und gemeinsam mit dem Mittelwertzhler in jeweils 4 Bytes speichern (Zhler-Low, Zhler-High, Mittelwert-Low, Mittelwert-High)
;
        ldi     r22, 31                 ;31 Werte bearbeiten
        ldi     xl, low(senavs)
        ldi     xh, high(senavs)        ;Zeiger auf Summen fr Mittelwertberechnung
        ldi     yl, low(senavc)
        ldi     yh, high(senavc)        ;Zeiger auf Zhler fr Mittelwertberechnung
;
i2bag0: ld      r20, y+                 ;Zhlerwert Low holen
        ld      r21, y+                 ;Zhlerwert High holen
        mov     r16, r20                ;Low-Wert kopieren
	rcall	i2csen			;und senden
	brcs	i2bal0			;ACK empfangen? nein -> Fehler
        mov     r16, r21                ;High-Wert kopieren
	rcall	i2csen			;und senden
	brcs	i2bal0			;ACK empfangen? nein -> Fehler
;
        ld      r16, x+                 ;Summenwert Low holen
        ld      r17, x+                 ;Summenwert High holen
        ld      r18, x+                 ;Summenwert Byte3 holen
        mov     r19, r20
        or      r19, r21                ;Zhlerwert = 0?
        breq    i2bak0                  ;ja -> Berechnung berspringen
;
        bst     r18, 7                  ;Summenwert negativ?
        brtc    i2bah0                  ;nein -> weiter
	com	r16                     ;Zweierkomplement bilden, Low invertieren
	com	r17                     ;High invertieren
        com     r18                     ;Byte3 invertieren
	subi	r16, low(-1)            ;1 addieren
	sbci	r17, high(-1)           ;1 addieren (bertrag)
	sbci	r18, byte3(-1)          ;1 addieren (bertrag)
;
i2bah0:	lsl	r16
	rol	r17			;Wert verdoppeln zur Erhhung
	rol	r18			;der Genauigkeit
        call    divide                  ;Division r18:R16 / r21:r20
        movw    r18, r16                ;Ergebnis kopieren
	lsr	r19
	ror	r18			;Ergebnis halbieren
	brcc	i2bai0			;Nachkommastelle = 0? ja -> weiter
	inc	r18			;sonst Ergebnis aufrunden
	brne	i2bai0			;bertrag? nein -> weiter
	inc	r19			;sonst H-Byte erhhen
;
i2bai0:	brtc	i2baj0			;war Mittelwertsumme negativ? nein -> weiter
	com	r18			;Zweierkomplement bilden, Low invertieren
	com	r19			;High invertieren
	subi	r18, low(-1)            ;1 addieren
	sbci	r19, high(-1)		;1 addieren (bertrag)
i2baj0:	mov	r16, r18		;Low-Wert kopieren
	rcall	i2csen			;und senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
        mov     r16, r19                ;High-Wert kopieren
	rcall	i2csen			;und senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
        rjmp    i2bam0
;
i2bak0: clr     r16                     ;Nullwert laden
	rcall	i2csen			;und senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
        clr     r16                     ;Nullwert laden
	rcall	i2csen			;und senden
i2bal0:	brcs	i2berr			;ACK empfangen? nein -> Fehler
;
i2bam0: dec     r22                     ;alle Werte bearbeitet?
        brne    i2bag0                  ;nein -> Schleife
;
; Zeit und Flag schreiben
;
	lds	r16, dayl		;Kalendertag Einer holen
	rcall	i2csen			;senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
	lds	r16, dayh		;Kalendertag Zehner holen
	rcall	i2csen			;senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
	lds	r16, monthl		;Monat Einer holen
	rcall	i2csen			;senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
	lds	r16, monthh		;Monat Zehner holen
	rcall	i2csen			;senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
	ldi	r16, 0x5a		;Flag fr Backup ok
	rcall	i2csen			;senden
;
i2berr:	rcall	i2csto			;I2C Stop Condition
	ret
;
; Unterprogramm zum Zurcklesen eines Backups (Minimal-, Maximal- und Mittelwerte) aus dem RTC-RAM
; Register: r15, r16, r17, r18, r19, r20, r21, r22, r23, x
;	    Carry -> 0 = ok, 1 = Fehler
;
i2rest:	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2csta			;I2C Start Condition + Adresse
	brcs	i2berr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtbval		;Adresse der Temperaturwerte
	rcall	i2csen			;senden
	brcs	i2berr			;ACK empfangen? nein -> Fehler
	ldi	r16, rtcadr + 1		;I2C Basisadresse RTC + Read
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	i2berr			;ACK empfangen? nein -> Fehler
;
; Minimalwerte dekomprimieren: jeweils 3 Bytes lesen (Low1, Low2, High2|High1) und in 2 12-Bit-Werte zurckwandeln
;
	ldi	r19, 15 		;15 Wertepaare bearbeiten
	ldi	xl, low(semin2)
	ldi	xh, high(semin2)	;Zeiger auf Minimalwerte fr Log
;
i2re10: clc                             ;ACK zum Slave senden
	rcall	i2crcv			;Wert 1 Low-Byte empfangen
        st      x+, r16                 ;und speichern
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;Wert 2 Low-Byte empfangen
	mov	r18, r16		;Wert zwischenspeichern
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;Wert 1+2 High-Byte empfangen
        mov     r17, r16                ;Wert zwischenspeichern
        andi    r16, 0x0f               ;Wert 1 High-Byte filtern
        cpi     r16, 0x08               ;Wert ungltig?
        brne    i2re20                  ;nein -> weiter
        ldi     r16, 0x80               ;sonst Originalwert wieder herstellen
        rjmp    i2re30
;
i2re20:	brcs	i2re30			;Wert positiv? ja -> weiter
	ori	r16, 0xf0		;sonst High-Byte korrigieren
i2re30: st      x+, r16                 ;Wert 1 High-Byte speichern
        st      x+, r18                 ;Wert 2 Low-Byte speichern
        swap    r17                     ;High-Byte an richtige Position
        andi    r17, 0x0f               ;Wert 2 High-Byte filtern
        cpi     r17, 0x08               ;Wert ungltig?
        brne    i2re40                  ;ja -> weiter
        ldi     r17, 0x80               ;sonst Originalwert wieder herstellen
        rjmp    i2re50
;
i2re40: brcs    i2re50			;Wert positiv? ja -> weiter
	ori	r17, 0xf0		;sonst High-Byte korrigieren
i2re50: st      x+, r17                 ;und speichern
	dec	r19			;alle Werte bearbeitet?
	brne	i2re10			;nein -> Schleife
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;letzten Wert Low-Byte empfangen
        st      x+, r16                 ;und speichern
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;letzten Wert High-Byte empfangen
        cpi     r16, 0x08               ;Wert ungltig?
        brne    i2re60                  ;nein -> weiter
        ldi     r16, 0x80               ;sonst Originalwert wieder herstellen
i2re60: st      x+, r16                 ;und speichern
;
; Maximalwerte dekomprimieren: jeweils 3 Bytes lesen (Low1, Low2, High2|High1) und in 2 12-Bit-Werte zurckwandeln
;
	ldi	r19, 15 		;15 Wertepaare bearbeiten
	ldi	xl, low(semax2)
	ldi	xh, high(semax2)	;Zeiger auf Maximalwerte fr Log
;
i2re70: clc                             ;ACK zum Slave senden
	rcall	i2crcv			;Wert 1 Low-Byte empfangen
        st      x+, r16                 ;und speichern
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;Wert 2 Low-Byte empfangen
	mov	r18, r16		;Wert zwischenspeichern
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;Wert 1+2 High-Byte empfangen
        mov     r17, r16                ;Wert zwischenspeichern
        andi    r16, 0x0f               ;Wert 1 High-Byte filtern
        cpi     r16, 0x08               ;Wert ungltig?
        brne    i2re80                  ;nein -> weiter
        ldi     r16, 0x80               ;sonst Originalwert wieder herstellen
        rjmp    i2re90
;
i2re80:	brcs	i2re90			;Wert positiv? ja -> weiter
	ori	r16, 0xf0		;sonst High-Byte korrigieren
i2re90: st      x+, r16                 ;Wert 1 High-Byte speichern
        st      x+, r18                 ;Wert 2 Low-Byte speichern
        swap    r17                     ;High-Byte an richtige Position
        andi    r17, 0x0f               ;Wert 2 High-Byte filtern
        cpi     r17, 0x08               ;Wert ungltig?
        brne    i2rea0                  ;ja -> weiter
        ldi     r17, 0x80               ;sonst Originalwert wieder herstellen
        rjmp    i2reb0
;
i2rea0: brcs    i2reb0			;Wert positiv? ja -> weiter
	ori	r17, 0xf0		;sonst High-Byte korrigieren
i2reb0: st      x+, r17                 ;und speichern
	dec	r19			;alle Werte bearbeitet?
	brne	i2re70			;nein -> Schleife
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;letzten Wert Low-Byte empfangen
        st      x+, r16                 ;und speichern
        clc                             ;ACK zum Slave senden
	rcall	i2crcv			;letzten Wert High-Byte empfangen
        cpi     r16, 0x08               ;Wert ungltig?
        brne    i2rec0                  ;nein -> weiter
        ldi     r16, 0x80               ;sonst Originalwert wieder herstellen
i2rec0: st      x+, r16                 ;und speichern
;
; Mittelwerte restaurieren: jeweils 4 Bytes lesen (Zhler-Low, Zhler-High, Mittelwert-Low, Mittelwert-High) und Mittelwertsumme wiederherstellen
;
        ldi     r23, 31                 ;31 Werte bearbeiten
        ldi     xl, low(senavs)
        ldi     xh, high(senavs)        ;Zeiger auf Summen fr Mittelwertberechnung
        ldi     yl, low(senavc)
        ldi     yh, high(senavc)        ;Zeiger auf Zhler fr Mittelwertberechnung
;
i2red0: clc                             ;ACK zum Slave senden
        rcall   i2crcv                  ;Mittelwertzhler Low-Byte empfangen
        mov     r18, r16                ;und kopieren
        clc                             ;ACK zum Slave senden
        rcall   i2crcv                  ;Mittelwertzhler High-Byte empfangen
        mov     r19, r16                ;und kopieren
        clc                             ;ACK zum Slave senden
        rcall   i2crcv                  ;Mittelwert Low-Byte empfangen
        mov     r20, r16                ;und kopieren
        cpi     r23, 1                  ;letzter Wert?
        brne    i2ree0                  ;nein -> weiter
        sec                             ;sonst kein ACK zum Slave senden
        rjmp    i2ref0
;
i2ree0: clc                             ;ACK zum Slave senden
i2ref0: rcall   i2crcv                  ;Mittelwert High-Byte empfangen
        mov     r21, r16                ;und kopieren
        st      y+, r18                 ;Zhlerwert Low speichern
        st      y+, r19                 ;Zhlerwert High speichern
        movw    r16, r20                ;Mittelwert kopieren
        bst     r17, 7                  ;Mittelwert negativ?
        brtc    i2reg0                  ;nein -> weiter
	com	r16			;Zweierkomplement bilden, Low invertieren
	com	r17			;High invertieren
	subi	r16, low(-1)            ;1 addieren
	sbci	r17, high(-1)		;1 addieren (bertrag)
;
i2reg0: call    multip                  ;Mittelwert und Mittelwertzhler multiplizieren
        brtc    i2reh0                  ;war Mittelwertsumme negativ? nein -> weiter
	com	r18                     ;Zweierkomplement bilden, Low invertieren
	com	r19                     ;High invertieren
        com     r20                     ;Byte3 invertieren
	subi	r18, low(-1)            ;1 addieren
	sbci	r19, high(-1)           ;1 addieren (bertrag)
	sbci	r20, byte3(-1)          ;1 addieren (bertrag)
;
i2reh0:	st      x+, r18                 ;Mittelwertsumme Low speichern
	st	x+, r19			;Mittelwertsumme High speichern
	st	x+, r20			;Mittelwertsumme Byte3 speichern
        dec     r23                     ;alle Werte bearbeitet?
        brne    i2red0                  ;nein -> Schleife
;
	clc				;Carry lschen (alles ok)
	rcall	i2csto			;I2C Stop Condition
	ret
;
; Unterprogramm zum Senden eines Ext-EEPROM-Pointers zum RTC-RAM
; Register: r16, r17, r19
;	    x     <- Zeiger auf Pointer im SRAM
;	    r18   <- RTC Pointer-Adresse
;	    Carry -> 0 = ok, 1 = Fehler
;
i2psen:	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2csta			;I2C Start Condition + Adresse
	brcs	i2pser			;ACK empfangen? nein -> Fehler
	ldi	r16, 3			;Offset fr Flagbyte laden
	add	r16, r18		;RTC-RAM, Pointer-Adresse addieren
	rcall	i2csen			;senden
	brcs	i2pser			;ACK empfangen? nein -> Fehler
	clr	r16			;Flag = 0 setzen
	rcall	i2csen			;senden
	brcs	i2pser			;ACK empfangen? nein -> Fehler
;
	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2crst			;I2C Start Condition + Adresse
	brcs	i2pser			;ACK empfangen? nein -> Fehler
	mov	r16, r18		;RTC-RAM, Pointer-Adresse
	rcall	i2csen			;senden
	brcs	i2pser			;ACK empfangen? nein -> Fehler
	ldi	r19, 3			;3 Pointer-Bytes senden
i2ps10:	ld	r16, x+			;Byte aus Puffer holen
	rcall	i2csen			;senden
	brcs	i2pser			;ACK empfangen? nein -> Fehler
	dec	r19			;alle Bytes gesendet?
	brne	i2ps10			;nein -> Schleife
	ldi	r16, 0x5a		;Flag-Byte laden
	rcall	i2csen			;senden
i2pser:	rcall	i2csto			;I2C Stop Condition
	ret
;
; Unterprogramm zum Empfangen eines Ext-EEPROM-Pointers vom RTC-RAM
; Register: r17
;	    x     <- Zeiger auf Datenblock
;	    r18   <- Startadresse
;	    r16   -> Flag-Byte
;	    Carry -> 0 = ok, 1 = Fehler
;
i2prcv:	ldi	r16, rtcadr		;I2C Basisadresse RTC + Write
	rcall	i2csta			;I2C Start Condition + Adresse
	brcs	i2prer			;ACK empfangen? nein -> Fehler
	mov	r16, r18		;RTC-RAM, Block Start-Adresse
	rcall	i2csen			;senden
	brcs	i2prer			;ACK empfangen? nein -> Fehler
	ldi	r16, rtcadr + 1		;I2C Basisadresse RTC + Read
	rcall	i2crst			;I2C Repeat Start Condition
	brcs	i2prer			;ACK empfangen? nein -> Fehler
	ldi	r19, 3			;3 Pointer-Bytes empfangen
i2pr10:	clc				;ACK zum Slave senden
	rcall	i2crcv			;Byte empfangen
	st	x+, r16			;Byte in Puffer speichern
	dec	r19			;alle Bytes empfangen?
	brne	i2pr10			;nein -> Schleife
	sec				;kein ACK zum Slave senden
	rcall	i2crcv			;Flag-Byte empfangen
	clc				;Carry lschen (alles ok)
i2prer:	rcall	i2csto			;I2C Stop Condition
	ret
;
; I2C Implementierung nach Application Note AVR300, modifiziert und eingestellt auf PortB2 (SCL) + PortB3 (SDA), Frequenz ca. 100 KHz
;
; Unterprogramm I2C Start Condition, Slave-Adresse senden, Timeout 15ms
; Register: r16, r17
;	    r15   <- I2C Slave-Adresse
;	    Carry -> 0 = ok, 1 = Fehler
;
i2ctio:	ldi	r18, 120		;Zeitkonstante ca. 15ms
i2ct10:	mov	r16, r15		;Slave-Adresse kopieren
	rcall	i2csta			;I2C Start Condition senden
	brcs	i2ct20			;ACK empfangen? nein -> weiter
	ret				;sonst Ende (ok)
i2ct20:	rcall	i2csto			;I2C Stop Condition
	dec	r18			;Schleifenzhler vermindern
	brne	i2ct10			;Ende? nein -> wiederholen
	sec				;sonst Carry setzen (Fehler)
	ret
;
; Unterprogramm I2C Repeat Start Condition und Slave-Adresse senden
; Register: r17
;	    r16   <- I2C Slave-Adresse (Bit0 = 0: Write, Bit0 = 1: Read)
;	    Carry -> ACK vom Slave
;
i2crst:	sbi	ddrb, 2			;SCL low
	cbi	ddrb, 3			;SDA freigeben
	rcall	i2hdel			;halbe Periode warten
	cbi	ddrb, 2			;SCL freigeben
	rcall	i2hdel			;halbe Periode warten
;
; Unterprogramm I2C Start Condition und Slave-Adresse senden
; Register: r17
;	    r16   <- I2C Slave-Adresse (Bit0 = 0: Write, Bit0 = 1: Read)
;	    Carry -> ACK vom Slave
;
i2csta:	sbi	ddrb, 3			;SDA low
	rcall	i2hdel			;halbe Periode warten
;
; Unterprogramm I2C ein Byte senden
; Register: r17
;	    r16   <- zu sendendes Byte
;	    Carry -> ACK vom Slave
;
i2csen:	sec				;Carry setzen
	rol	r16			;hchstes Bit in Carry schieben
	rjmp	i2sbfi			;erstes Bit ausgeben
i2sbit:	lsl	r16			;nchstes Bit, alles ausgegeben?
i2sbfi:	breq	i2sack			;ja -> ACK holen
	sbi	ddrb, 2			;SCL low
	brcc	i2slow			;0-Bit? ja -> ausgeben
	nop				;Zyklen ausgleichen
	cbi	ddrb, 3			;SDA freigeben
	rjmp	i2shig			;1-Bit ausgeben
i2slow:	sbi	ddrb, 3			;SDA low
	rjmp	i2shig			;Zyklen ausgleichen
i2shig:	rcall	i2hdel			;halbe Periode warten
	cbi	ddrb, 2			;SCL freigeben
	rcall	i2hdel			;halbe Periode warten
	rjmp	i2sbit			;Schleife
;
i2sack:	sbi	ddrb, 2			;ACK lesen, SCL low
	cbi	ddrb, 3			;SDA freigeben
	rcall	i2hdel			;halbe Periode warten
	cbi	ddrb, 2			;SCL freigeben
i2sawa:	sbis	pinb, 2			;auf SCL high warten, falls
	rjmp	i2sawa			;Wait-States eingefgt wurden
	clc				;Carry lschen
	sbic	pinb, 3			;SDA high? nein -> weiter
	sec				;sonst Carry setzen
	rcall	i2hdel			;halbe Periode warten
	ret
;
; Unterprogramm I2C ein Byte empfangen
; Register: r15, r17
;	    r16   -> gelesenes Byte
;	    Carry <- ACK zum Slave
;
i2crcv:	rol	r15			;ACK speichern
	ldi	r16, 0x01		;Daten = 0x01 setzen
i2rbit:	sbi	ddrb, 2			;SCL low
	cbi	ddrb, 3			;SDA freigeben (vorheriges ACK)
	rcall	i2hdel			;halbe Periode warten
	cbi	ddrb, 2			;SCL freigeben
	rcall	i2hdel			;halbe Periode warten
	clc				;Carry lschen
	sbic	pinb, 3			;SDA high? nein -> weiter
	sec				;sonst Carry setzen
	rol	r16			;Datenbit speichern
	brcc	i2rbit			;alle Bits empfangen? -> Schleife
;
	sbi	ddrb, 2			;SCL low
	ror	r15			;Status Bit holen
	brcc	i2ralo			;Bit = 0? ja -> ausgeben
	cbi	ddrb, 3			;sonst SDA freigeben
	rjmp	i2rahi			;und High ausgeben
i2ralo:	sbi	ddrb, 3			;SDA low
i2rahi:	rcall	i2hdel			;halbe Periode warten
	cbi	ddrb, 2			;SCL freigeben
i2rawa:	sbis	pinb, 2			;auf SCL high warten
	rjmp	i2rawa			;Schleife
	rcall	i2hdel			;halbe Periode warten
	ret
;
; Unterprogramm I2C Stop Condition
; Register: r17
;
i2csto:	sbi	ddrb, 2			;SCL low
	sbi	ddrb, 3			;SDA low
	rcall	i2hdel			;halbe Periode warten
	cbi	ddrb, 2			;SCL freigeben
	rcall	i2hdel			;halbe Periode warten
	cbi	ddrb, 3			;SDA freigeben
	rcall	i2hdel			;halbe Periode warten
	ret
;
; Unterprogramm I2C Warteschleife fr halbe Periode
; Register: r17
;
i2hdel:	ldi	r17, 16			;Wartezeit ca. 5,0s
i2hd10:	dec	r17
	brne	i2hd10
	ret
;
; Unterprogramm bei Fehler in den I2C-Routinen, Fehlerstatus setzen und Infozeile neu ausgeben
; Register: r16
;
i2cerr:	lds	r16, errflg		;Fehler-Flags holen
	sbr	r16, 0x10		;RTC-Fehler-Flag setzen
	sts	errflg, r16		;Flags wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
	ret
;
; Unterprogramm bei Pointer-Fehler, Fehlerstatus setzen und Infozeile neu ausgeben
;
poierr:	lds	r16, errflg		;Fehler-Flags holen
	sbr	r16, 0x20		;Pointer-Fehler-Flag setzen
	sts	errflg, r16		;Flags wieder speichern
	lds	r16, dflags		;Anzeige-Flags holen
	sbr	r16, 0x04		;Infozeile neu ausgeben
	sts	dflags, r16		;Anzeige-Flags wieder speichern
	ret
;
; Unterprogramm zum Setzen der Baudrate der PC-Schnittstelle (RS232-0)
; Register: r16, r17, r18
;
setbd0:	lds	r18, baud0		                ;Baudratenwert RS232-0 holen
	cpi	r18, 0			                ;9600 Baud?
	brne	setb10			                ;nein -> weiter testen
	ldi	r16, low((clock / (16 * 9600)) - 1)     ;RS232-0 auf 9600 Baud
	ldi	r17, high((clock / (16 * 9600)) - 1)    ;einstellen
	rjmp	setb70
setb10:	cpi	r18, 1			                ;19200 Baud?
	brne	setb20			                ;nein -> weiter testen
	ldi	r16, low((clock / (16 * 19200)) - 1)    ;RS232-0 auf 19200 Baud
	ldi	r17, high((clock / (16 * 19200)) - 1)   ;einstellen
	rjmp	setb70
setb20:	cpi	r18, 2			                ;38400 Baud?
	brne	setb30			                ;nein -> weiter testen
	ldi	r16, low((clock / (16 * 38400)) - 1)    ;RS232-0 auf 38400 Baud
	ldi	r17, high((clock / (16 * 38400)) - 1)   ;einstellen
	rjmp	setb70
setb30:	cpi	r18, 3			                ;57600 Baud?
	brne	setb40			                ;nein -> weiter testen
	ldi	r16, low((clock / (16 * 57600)) - 1)    ;RS232-0 auf 57600 Baud
	ldi	r17, high((clock / (16 * 57600)) - 1)   ;einstellen
	rjmp	setb70
setb40:	cpi	r18, 4			                ;115200 Baud?
	brne	setb50			                ;nein -> weiter testen
	ldi	r16, low((clock / (16 * 115200)) - 1)   ;sonst RS232-0 auf
	ldi	r17, high((clock / (16 * 115200)) - 1)  ;115200 Baud einstellen
	rjmp	setb70
setb50:	cpi	r18, 5			                ;230400 Baud?
	brne	setb60			                ;nein -> weiter testen
	ldi	r16, low((clock / (16 * 230400)) - 1)   ;sonst RS232-0 auf
	ldi	r17, high((clock / (16 * 230400)) - 1)  ;230400 Baud einstellen
	rjmp	setb70
setb60:	cpi	r18, 6			                ;460800 Baud?
	brne	setb90			                ;nein -> keine nderung
	ldi	r16, low((clock / (8 * 460800)) - 1)    ;sonst RS232-0 auf
	ldi	r17, high((clock / (8 * 460800)) - 1)   ;460800 Baud einstellen
	ldi	r18, 1 << u2x0		                ;USART Taktverdopplung setzen
	rjmp	setb80
setb70:	ldi	r18, 0 << u2x0		                ;USART Taktverdopplung lschen
setb80:	out	ucsr0a, r18		                ;Taktverdopplung einstellen
	out	ubrr0l, r16		                ;Baudratenwert L setzen
	sts	ubrr0h, r17		                ;Baudratenwert H setzen
setb90:	ret
;
; Unterprogramm zum Schreiben eines Zeichens in den RS232-0 Sendepuffer
; Register: x, y
;	    r16 <- zu sendendes Zeichen
;
put0ch:	cli				;Interrupts vorbergehend sperren
	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Schreibzeiger holen
	st	x+, r16			        ;Byte speichern, Zeiger + 1
	ldi	yl, low(txbuf0 + txbu0l)
	ldi	yh, high(txbuf0 + txbu0l)	;Adresse Pufferende laden
	sub	yl, xl
	sbc	yh, xh			;Differenz berechnen
	or	yh, yl			;Endadresse erreicht?
	brne	put010			;nein -> weiter
	ldi	xl, low(txbuf0)
	ldi	xh, high(txbuf0)	;sonst Adresse auf Anfangswert
put010:	sts	txp0wr, xl
	sts	txp0wr + 1, xh		;Schreibzeiger speichern
	sbi	ucsr0b, udrie0		;Sendeinterrupt aktivieren
	sei				;Interrupts wieder freigeben
        clr     r16
        sts     comflg, r16             ;Kommunikations-Flag setzen
	ret
;
; Unterprogramm zum Auslesen eines Zeichens aus dem RS232-0 Empfangspuffer
; Register: x, y
;	    r16 -> gelesenes Zeichen
;	    r17 -> 0 = Zeichen gltig, 0xff = kein Zeichen im Puffer
;
get0ch:	cli				;Interrupts vorbergehend sperren
	lds	xl, rxp0wr
	lds	xh, rxp0wr + 1		;Schreibzeiger holen
	sei				;Interrupts wieder freigeben
	lds	yl, rxp0rd
	lds	yh, rxp0rd + 1		;Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;sind Zeichen im Puffer?
	brne	get010			;ja -> weiter
	ser	r17			;kein Zeichen im Puffer
	ret
get010:	ld	r16, y+			        ;Zeichen holen, Pointer + 1
	ldi	xl, low(rxbuf0 + rxbu0l)
	ldi	xh, high(rxbuf0 + rxbu0l)	;Adresse Pufferende laden
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;Endadresse erreicht?
	brne	get020			;nein -> weiter
	ldi	yl, low(rxbuf0)
	ldi	yh, high(rxbuf0)		;sonst Adresse auf Pufferanfang
get020:	sts	rxp0rd, yl
	sts	rxp0rd + 1, yh		;Schreibzeiger speichern
	clr	r17			;Zeichen ist gltig
	ret
;
; Unterprogramm zum Auslesen eines Zeichens aus dem RS232-1 Empfangspuffer
; Register: x, y
;	    r16 -> gelesenes Zeichen
;	    r17 -> 0 = Zeichen gltig, 0xff = kein Zeichen im Puffer
;
get1ch:	cli				;Interrupts vorbergehend sperren
	lds	xl, rxp1wr
	lds	xh, rxp1wr + 1		;Schreibzeiger holen
	sei				;Interrupts wieder freigeben
	lds	yl, rxp1rd
	lds	yh, rxp1rd + 1		;Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;sind Zeichen im Puffer?
	brne	get110			;ja -> weiter
	ser	r17			;kein Zeichen im Puffer
	ret
get110:	ld	r16, y+			        ;Zeichen holen, Pointer + 1
	ldi	xl, low(rxbuf1 + rxbu1l)
	ldi	xh, high(rxbuf1 + rxbu1l)	;Adresse Pufferende laden
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;Endadresse erreicht?
	brne	get120			;nein -> weiter
	ldi	yl, low(rxbuf1)
	ldi	yh, high(rxbuf1)	;sonst Adresse auf Pufferanfang
get120:	sts	rxp1rd, yl
	sts	rxp1rd + 1, yh		;Schreibzeiger speichern
	clr	r17			;Zeichen ist gltig
	ret
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Interrupt-Routinen
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Interrupt-Routine fr Timer1, wird alle 625s (= 1600Hz) aufgerufen
;
t1comp:	in	insreg, sreg		;SREG sichern
	push	r16			;diverse Register sichern
	push	r17
	push	r18
	push	yl
	push	yh
	push	r0
	push	r1
;
; Teil 1 luft alle 625s und bedient den Drehimpulsgeber
;
	lds	yl, encstp		;aktuellen Schrittwert des Impulsgebers holen Low
	lds	yh, encstp + 1		;aktuellen Schrittwert des Impulsgebers holen High
	in	r18, pine		;Impulsgeber Jumper 1 (Geber-Typ) und Jumper 2 (Richtung) holen
	in	r16, pinc		;Impulsgeber lesen
	andi	r16, 0x30		;Impulsgeber-Bits filtern
	lds	r17, ensta1		;letzten Impulsgeber-Status holen
	sts	ensta1, r16		;neuen Impulsgeber-Status speichern
	cp	r16, r17		;Statusnderung?
	brne	t1c080			;ja -> Wert unstabil, Ende
	lds	r17, ensta2		;letzten stabilen Status holen
	sts	ensta2, r16		;neuen stabilen Status speichern
	cp	r16, r17		;Statusnderung?
	breq	t1c080			;nein -> Ende
;
	cpi	r17, 0x00		;war alter Status = 0x00?
	brne	t1c020			;nein -> weiter testen
	cpi	r16, 0x10		;neuer Status = 0x10 (rechts)?
	brne	t1c010			;nein -> weiter testen
	adiw	yl, 1			;sonst Impulsgeberschritt + 1
	rjmp	t1c080
t1c010:	cpi	r16, 0x20		;neuer Status = 0x20 (links)?
	brne	t1c080			;nein -> Ende
	sbiw	yl, 1			;sonst Impulsgeberschritt - 1
	rjmp	t1c080
;
t1c020:	cpi	r17, 0x10		;war alter Status = 0x10?
	brne	t1c040			;nein -> weiter testen
	cpi	r16, 0x30		;neuer Status = 0x30 (rechts)?
	brne	t1c030			;nein -> weiter testen
	adiw	yl, 1			;sonst Impulsgeberschritt + 1
	rjmp	t1c080
t1c030:	cpi	r16, 0x00		;neuer Status = 0x00 (links)?
	brne	t1c080			;nein -> Ende
	sbiw	yl, 1			;sonst Impulsgeberschritt - 1
	sbrs	r18, 4			;Jumper 1 geschlossen (Impulsgeber mit 4 Schritten)?
	andi	yl, 0xfc		;ja -> Schrittwert korrigieren
	rjmp	t1c080
;
t1c040:	cpi	r17, 0x20		;war alter Status = 0x20?
	brne	t1c060			;nein -> weiter testen
	cpi	r16, 0x00		;neuer Status = 0x00 (rechts)?
	brne	t1c050			;nein -> weiter testen
	adiw	yl, 1			;sonst Impulsgeberschritt + 1
	sbrs	r18, 4			;Jumper 1 geschlossen (Impulsgeber mit 4 Schritten)?
	andi	yl, 0xfc		;ja -> Schrittwert korrigieren
	rjmp	t1c080
t1c050:	cpi	r16, 0x30		;neuer Status = 0x30 (links)?
	brne	t1c080			;nein -> Ende
	sbiw	yl, 1			;sonst Impulsgeberschritt - 1
	rjmp	t1c080
;
t1c060:	cpi	r17, 0x30		;war alter Status = 0x30?
	brne	t1c080			;nein -> Ende
	cpi	r16, 0x20		;neuer Status = 0x20 (rechts)?
	brne	t1c070			;nein -> weiter testen
	adiw	yl, 1			;sonst Impulsgeberschritt + 1
	rjmp	t1c080
t1c070:	cpi	r16, 0x10		;neuer Status = 0x10 (links)?
	brne	t1c080			;nein -> Ende
	sbiw	yl, 1			;sonst Impulsgeberschritt - 1
;
t1c080:	sts	encstp, yl		;neuen Schrittwert speichern Low
	sts	encstp + 1, yh		;neuen Schrittwert speichern High
	asr	yh			;Schrittwert halbieren High
	ror	yl			;Schrittwert halbieren Low, fertiger Wert fr Geber mit 2 Schritten
	andi	r18, 0x10		;Jumper 1 geschlossen (Impulsgeber mit 4 Schritten)?
	brne	t1c090			;nein -> Ende
	asr	yh			;sonst Schrittwert nochmals halbieren High
	ror	yl			;Schrittwert nochmals halbieren Low
t1c090:	in	r18, pine		;Impulsgeber Jumper 1 (Geber-Typ) und Jumper 2 (Richtung) holen
	sbrs	r18, 5			;Jumper 2 geschlossen (Impulsgeber Richtungsumkehr)?
	neg	yl			;ja -> Geberposition negieren
	sts	encpos, yl		;neue Impulsgeberposition speichern
;
; Teil 2 luft alle 20ms (Software-Teilung durch 32) und bedient den DCF77-Empfang
;
t1c100:	lds	r16, t1cnt1		;Timer1 Int-Zhler1 holen
	inc	r16			;erhhen
	sts	t1cnt1, r16		;und wieder speichern
	cpi	r16, 32			;Endwert erreicht?
	brcc	t1c110			;ja -> weiter
	rjmp	t1cend			;sonst Ende
;
t1c110:	clr	r16			;Timer1 Int-Zhler1 lschen
	sts	t1cnt1, r16		;und wieder speichern
;
	lds	r16, dcfcnt		;DCF77-Zhler holen
	inc	r16			;Zhler erhhen
	sts	dcfcnt, r16		;und wieder speichern
	in	r16, pine		;aktuellen DCF77-Status lesen
        lds     r17, confg2             ;Konfigurationsbyte 2 holen
        sbrc    r17, 4                  ;DCF77-Signal invertiert? nein -> berspringen
        com     r16                     ;sonst DCF77-Signal invertieren
	bst	r16, 2			;DCF77-Status prfen
;
	in	r16, portd
	bld	r16, 6			;Testausgabe
	out	portd, r16
;
	lds	r16, dcflag		;DCF77-Flags holen
	bld	r16, 0			;und aktuellen Status speichern
	brts	t1c130			;ist DCF77 High? ja -> weiter
	bst	r16, 1			;vorherigen DCF77-Status prfen
	brtc	t1c150			;DCF77 vorher High? nein -> weiter
	lds	r17, dcfcnt		;DCF77-Zhler holen
	cpi	r17, 75			;Zhler < 75 (1,5s)?
	brcs	t1c120			;nein -> weiter
	sbr	r16, 0x08		;neuen Minutenbeginn setzen
t1c120:	cpi	r17, 45			;Zhler < 45 (900ms) ?
	brcs	t1c150			;ja -> ignorieren
	clr	r17			;sonst
	sts	dcfcnt, r17		;DCF77-Zhler lschen
	rjmp	t1c150			;Ende
t1c130:	bst	r16, 1			;sonst vorherigen DCF-Status prfen
	brts	t1c150			;war DCF77 vorher High? ja -> Ende
	lds	r17,dcfcnt		;sonst DCF77-Zhler holen
	cpi	r17, 13			;DCF-Zhler < 13 (260ms)?
	brcc	t1c150			;nein -> ignorieren
	cpi	r17, 3			;DCF-Zhler < 3 (60ms)?
	brcs	t1c150			;ja -> ignorieren
	lds	r18, dcfimp		;sonst DCF-Impulsanzahl holen
	inc	r18			;Zhler erhhen
	sts	dcfimp, r18		;und wieder speichern
	cpi	r17, 7			;Zhler < 7 (140ms)? -> Carry
	ldi	r17, 6			;6 Pufferbytes bearbeiten
	ldi	yl, low(dcftab + 6)
	ldi	yh, high(dcftab + 6)	;Zeiger auf Pufferende setzen
	rol	r18			;Carry in Register legen
	com	r18			;alle Bits invertieren
	ror	r18			;wieder in Carry schieben
t1c140:	ld	r18, -y			;Pufferbyte holen
	ror	r18			;neues Bit einschieben als MSB
	st	y, r18			;LSB in Carry und speichern
	dec	r17			;alle Pufferbytes bearbeitet?
	brne	t1c140			;nein -> Schleife
t1c150:	bst	r16, 0			;aktuellen DCF77-Status holen
	bld	r16, 1			;und als neuen Status speichern
	sts	dcflag, r16		;DCF77-Flags speichern
;
; Teil 3 luft einmal pro Sekunde (Software-Teilung durch 50), steuert die Uhr und andere Zhler
;
t1c400:	lds	r16, t1cnt2		;Timer1 Int-Zhler2 holen
	inc	r16			;erhhen
	sts	t1cnt2, r16		;und wieder speichern
	cpi	r16, 50			;Endwert erreicht?
	brcc	t1c410			;ja -> weiter
	rjmp	t1cend			;sonst Ende
;
t1c410:	clr	r16			;Timer1 Int-Zhler2 lschen
	sts	t1cnt2, r16		;und wieder speichern
;
; Sekunden-Timer
;
	lds	r16, secndl		;Sekunden Einer holen
	inc	r16			;erhhen
	sts	secndl, r16		;und wieder speichern
	cpi	r16,10			;Endwert erreicht?
	brcc	t1c420			;ja -> weiter
	rjmp	t1cend			;sonst Ende
;
t1c420:	clr	r16			;Sekunden Einer lschen
	sts	secndl, r16		;und wieder speichern
	lds	r16, secndh		;Sekunden Zehner holen
	inc	r16			;erhhen
	sts	secndh, r16		;und wieder speichern
	cpi	r16, 6			;Endwert erreicht?
	brcc	t1c430			;ja -> weiter
	rjmp	t1cend			;sonst Ende
t1c430:	clr	r16			;Sekunden Zehner lschen
	sts	secndh, r16		;und wieder speichern
;
; Minuten-Timer
;
	lds	r16, minutl		;Minuten Einer holen
	inc	r16			;erhhen
	sts	minutl, r16		;und wieder speichern
	cpi	r16, 10			;Endwert erreicht?
	brcc	t1c440			;ja -> weiter
	rjmp	t1cend			;sonst Ende
t1c440:	clr	r16			;Minuten Einer lschen
	sts	minutl, r16		;und wieder speichern
	lds	r16, minuth		;Minuten Zehner holen
	inc	r16			;erhhen
	sts	minuth, r16		;und wieder speichern
	cpi	r16, 6			;Endwert erreicht?
	brcc	t1c450			;ja -> weiter
	rjmp	t1cend			;sonst Ende
t1c450:	clr	r16			;Minuten Zehner lschen
	sts	minuth, r16		;und wieder speichern
;
; Stunden-Timer
;
	lds	r16, hourl		;Stunden Einer holen
	inc	r16			;erhhen
	sts	hourl, r16		;und wieder speichern
	cpi	r16, 10			;Endwert erreicht?
	brcs	t1c460			;nein -> Test auf '24'
	clr	r16			;sonst zurcksetzen
	sts	hourl, r16		;und wieder speichern
	lds	r16, hourh		;Stunden Zehner holen
	inc	r16			;erhhen
	sts	hourh, r16		;und wieder speichern
	rjmp	t1cend			;Ende
;
t1c460:	lds	r16, hourh		;Stunden Zehner holen
	cpi	r16, 2			;Stunden Zehner = 2?
	breq	t1c470			;ja -> weiter
	rjmp	t1cend			;sonst Ende
t1c470:	lds	r16, hourl		;nochmals Stunden Einer holen
	cpi	r16, 4			;Stunden Einer = 4 (24)?
	breq	t1c480			;ja -> weiter
	rjmp	t1cend			;sonst Ende
t1c480:	clr	r16			;sonst Stunden zurcksetzen
	sts	hourl, r16		;Stunden Einer speichern
	sts	hourh, r16		;Stunden Zehner speichern
;
; Tages-Timer
;
	lds	r16, dayw		;Wochentag holen
	inc	r16			;erhhen
	sts	dayw, r16		;und wieder speichern
	cpi	r16, 7			;Endwert erreicht?
	brcs	t1c490			;nein -> weiter
	clr	r16			;sonst Wochentag lschen
	sts	dayw, r16		;und wieder speichern
;
t1c490:	lds	r16, dayl		;Tag Einer holen
	inc	r16			;erhhen
	sts	dayl, r16		;und wieder speichern
	cpi	r16, 10			;Endwert erreicht?
	brcs	t1c500			;nein -> Monatsende testen
	clr	r16			;sonst Tag Einer lschen
	sts	dayl, r16		;und wieder speichern
	lds	r16, dayh		;Tag Zehner holen
	inc	r16			;erhhen
	sts	dayh, r16		;und wieder speichern
;
t1c500:	lds	r16, monthh		;Monat Zehner holen
	ldi	r17, 10			;mit 10
	mul	r16, r17		;multiplizieren
	lds	r18, monthl		;Monat Einer holen
	add	r18, r0			;mit Zehner verknpfen
	lds	r16, dayh		;Tag Zehner holen
	mul	r16, r17		;mit 10 multiplizieren
	lds	r16, dayl		;Tag Einer holen
	add	r16, r0			;mit Zehner verknpfen
;
	cpi	r18, 1			;Monat Januar?
	brne	t1c510			;nein -> weiter testen
	cpi	r16, 32			;Tag > 31?
	brcc	t1c515			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c510:	cpi	r18, 2			;Monat Februar?
	brne	t1c530			;nein -> weiter testen
	lds	r18, yearh		;Jahr Zehner holen
	mul	r18, r17		;mit 10 multiplizieren
	lds	r18, yearl		;Jahr Einer holen
	add	r18, r0			;mit Zehner verknpfen
	andi	r18, 0x03		;Schaltjahr?
	breq	t1c520			;ja -> 29 Tage
	cpi	r16, 29			;sonst Tag > 28?
t1c515:	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
t1c520:	cpi	r16, 30			;Tag > 29?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c530:	cpi	r18, 3			;Monat Mrz?
	brne	t1c540			;nein -> weiter testen
	cpi	r16, 32			;Tag > 31?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c540:	cpi	r18, 4			;Monat April?
	brne	t1c550			;nein -> weiter testen
	cpi	r16, 31			;Tag > 30?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c550:	cpi	r18, 5			;Monat Mai?
	brne	t1c560			;nein -> weiter testen
	cpi	r16, 32			;Tag > 31?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c560:	cpi	r18, 6			;Monat Juni?
	brne	t1c570			;nein -> weiter testen
	cpi	r16, 31			;Tag > 30?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c570:	cpi	r18, 7			;Monat Juli?
	brne	t1c580			;nein -> weiter testen
	cpi	r16, 32			;Tag > 31?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c580:	cpi	r18, 8			;Monat August?
	brne	t1c590			;nein -> weiter testen
	cpi	r16, 32			;Tag > 31?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c590:	cpi	r18, 9			;Monat September?
	brne	t1c600			;nein -> weiter testen
	cpi	r16, 31			;Tag > 30?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c600:	cpi	r18, 10			;Monat Oktober?
	brne	t1c610			;nein -> weiter testen
	cpi	r16, 32			;Tag > 31?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c610:	cpi	r18, 11			;Monat November?
	brne	t1c620			;nein -> Monat Dezember
	cpi	r16, 31			;Tag > 30?
	brcc	t1c700			;ja -> Monatswechsel
	rjmp	t1cend			;sonst Ende
;
t1c620:	cpi	r16, 32			;Tag > 31?
	brcs	t1cend			;nein -> Ende
;
t1c700:	ldi	r16, 1			;Tag Einer
	sts	dayl, r16		;lschen
	clr	r16			;Tag Zehner
	sts	dayh, r16		;lschen
;
; Monats-Timer
;
	lds	r17, monthh		;Monat Zehner holen
	lds	r16, monthl		;Monat Einer holen
	inc	r16			;erhhen
	sts	monthl, r16		;und wieder speichern
	cpi	r16, 10			;Endwert erreicht?
	brcs	t1c800			;nein -> Jahrende testen
	clr	r16			;sonst Monat Einer lschen
	sts	monthl, r16		;und wieder speichern
	inc	r17			;erhhen
	sts	monthh, r17		;und wieder speichern
;
t1c800:	cpi	r17, 1			;Monat Zehner = 1?
	brne	t1cend			;nein -> Ende
	cpi	r16, 3			;Monat Einer = 3?
	brne	t1cend			;nein -> Ende
	sts	monthl, r17		;sonst Monat Einer = 1 setzen
	clr	r17
	sts	monthh, r17		;Monat Zehner lschen
;
	lds	r16, yearl		;Jahr Einer holen
	inc	r16			;erhhen
	sts	yearl, r16		;und wieder speichern
	cpi	r16, 10			;Endwert erreicht?
	brcs	t1cend			;nein -> Ende
	clr	r16			;sonst Jahr Einer lschen
	sts	yearl, r16		;und wieder speichern
	lds	r16, yearh		;Jahr Zehner holen
	inc	r16			;erhhen
	sts	yearh, r16		;und wieder speichern
;
t1cend:	pop	r1
	pop	r0
	pop	yh
	pop	yl
	pop	r18
	pop	r17
	pop	r16			;diverse Register restaurieren
	out	sreg, insreg		;SREG wiederherstellen
	reti
;
; Interrupt-Routine fr Timer3, wird alle 20ms (=50Hz) aufgerufen
;
t3comp:	in	insreg, sreg		;SREG sichern
	push	r16			;diverse Register sichern
	push	r17
	push	r18
	push	zl
	push	zh
	in	zl, rampz
	push	zl
;
; Taster lesen und entprellen
;
	in	r16, pinc		;Taster lesen
	com	r16			;invertieren
	andi	r16, 0xc0		;Taster-Bits filtern
	mov	r17, r16		;und kopieren
	lds	r18, keyst1		;letzten Tastenstatus holen
	sts	keyst1, r16		;neuen Tastenstatus speichern
	eor	r17, r18		;Statusnderungs-Maske erzeugen
	lds	r18, keyst2		;gltige Tasten holen
	and	r18, r17		;unvernderte Tasten filtern
	com	r17			;Maske invertieren
	and	r16, r17		;vernderte Tasten filtern
	or	r18, r16		;gltige Tasten kombinieren
	sts	keyst2, r18		;und speichern
	lds	r17, keyst3		;Bearbeitungs-Flags holen
	and	r17, r18		;losgelassene Tasten lschen
	sts	keyst3, r17		;Bearbeitungs-Flags speichern
;
; Weiches Ein- und /Ausblenden der LCD-Hintergrundbeleuchtung steuern
;
	in	r16, ocr2		;aktuellen PWM-Wert holen
        lds     r17, bright             ;berechneten PWM-Wert (Zielwert) holen
	mov	r18, r16		;aktuellen PWM-kopieren
        cp      r16, r17                ;gewnschte Helligkeit erreicht?
        breq    t3c070                  ;ja -> Ende
	brcs	t3c030			;Helligkeit erhhen? -> weiter
;
	swap	r18			;obere 3 Bits als Decrement
	lsr	r18			;verwenden
	andi	r18, 0x07		;relevante Bits filtern
	brne	t3c010			;Decrement > 0? ja -> weiter
	inc	r18			;sonst Decrement auf 1 setzen
t3c010:	sub	r16, r18		;PWM-Wert vermindern 
	brcc	t3c020			;Wert >= 0? ja -> weiter
	clr	r16			;sonst durch unteren Endwert ersetzen
t3c020: cp      r16, r17                ;PWM-Zielwert unterschritten?
        brcc    t3c070                  ;nein -> speichern
	rjmp	t3c060			;sonst durch Zielwert ersetzen
;
t3c030:	swap	r18			;obere 3 Bits als Increment
	lsr	r18			;verwenden
	andi	r18, 0x07		;relevante Bits filtern
	brne	t3c040			;Increment > 0? ja -> weiter
	inc	r18			;sonst Increment auf 1 setzen
t3c040:	add	r16, r18		;PWM-Wert erhhen
	brcc	t3c050			;Wert <= 255? ja -> weiter
	ldi	r16, 0xff		;sonst oberen Endwert setzen
t3c050: cp      r17, r16                ;PWM-Zielwert berschritten?
        brcc    t3c070                  ;nein -> speichern
t3c060: mov     r16, r17                ;sonst durch Zielwert ersetzen
t3c070:	out	ocr2, r16		;neuen PWM-Wert speichern
;
; LED-Debug-Modus: LED1 als DCF77 und LED2 als RS-232 Anzeige
;
	in	r18, portd		;Status PortD holen (LEDs)
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 7			;Abwesenheit aktiviert?
	brtc	t3c080			;nein -> weiter testen
	sbr	r18, 0x30		;sonst LEDs auschalten
	rjmp	t3c210			;neuen LED1/2-Status setzen
;
t3c080:	lds	r16, confg2		;Konfigurationsbyte 2 holen
	bst	r16, 0			;LEDs im Normal-Modus?
	brtc	t3c090			;ja -> normale LED-Ausgabe
;
        lds     r17, dcflag             ;sonst DCF77-Flags holen
        bst     r17, 0                  ;aktuellen DCF77-Status holen
	bld	r18, 4			;als neuen LED1-Status setzen
	lds	r17, rcvflg		;RS-232 Empfangsflag holen
	bst	r17, 0			;LSB holen und
	bld	r18, 5			;als neuen LED2-Status setzen
	sec				;Carry setzen
	ror	r17			;in MSB einschieben
	sts	rcvflg, r17		;Empfangsflag wieder speichern
	rjmp	t3c210			;neuen LED1/2-Status setzen
;
; Blinken und Dauerlicht von LED1 (rot) und LED2 (gelb) steuern
;
t3c090:	lds	r16, ledflg		;LED-Flags holen
	lds	r17, t3cnt2		;Timer3 Sekundenzhler holen
	andi	r16, 0x05		;LED1 (rot) ansteuern?
	brne	t3c120			;ja -> bearbeiten
	lds	r16, ledflg		;sonst LED-Flags nochmals holen
	bst	r16, 4			;Erinnerungs-Flag gesetzt?
	brts	t3c100			;ja -> bearbeiten
	sbr	r18, 0x10		;sonst LED1 auschalten
	rjmp	t3c150			;weiter mit LED2
;
t3c100:	bst	r17, 0			;Sekunde geradzahlig?
	brtc	t3c110			;ja -> LED1 ausschalten
	lds	r16, t3cnt1		;sonst Timer3 Int-Zhler holen
	tst	r16			;Zhlerstand = 0?
	brne	t3c110			;nein -> LED1 ausschalten
	cbr	r18, 0x10		;sonst LED1 einschalten
	rjmp	t3c150			;weiter mit LED2
t3c110:	sbr	r18, 0x10		;sonst LED1 ausschalten
	rjmp	t3c150			;weiter mit LED2
;
t3c120:	bst	r16, 0			;LED1 Dauerlicht?
	brtc	t3c130			;nein -> berspringen
	cbr	r18, 0x10		;sonst LED1 einschalten
t3c130:	bst	r16, 2			;LED1 Blinken?
	brtc	t3c150			;nein -> berspringen
	bst	r16, 0			;zustzlich LED1 Dauerlicht?
	brtc	t3c140			;nein -> direkt zum Blinken
	bst	r17, 0			;Sekunde geradzahlig?
	brtc	t3c150			;ja -> berspringen
t3c140:	lds	r16, t3cnt1		;Timer3 Interrupt-Zhler holen
	sbr	r18, 0x10		;LED1 ausschalten
	cpi	r16, 9			;Zhlerstand < 9?
	brcs	t3c150			;ja -> LED1-Zustand ok
	cbr	r18, 0x10		;sonst LED1 einschalten
	cpi	r16, 17			;Zhlerstand < 17?
	brcs	t3c150			;ja -> LED1-Zustand ok
	sbr	r18, 0x10		;sonst LED1 ausschalten
	cpi	r16, 25			;Zhlerstand < 25?
	brcs	t3c150			;ja -> LED1-Zustand ok
	cbr	r18, 0x10		;sonst LED1 einschalten
	cpi	r16, 33			;Zhlerstand < 33?
	brcs	t3c150			;ja -> LED1-Zustand ok
	sbr	r18, 0x10		;sonst LED1 ausschalten
	cpi	r16, 42			;Zhlerstand < 42?
	brcs	t3c150			;ja -> LED1-Zustand ok
	cbr	r18, 0x10		;sonst LED1 einschalten
;
t3c150:	lds	r16, ledflg		;LED-Flags holen
	andi	r16, 0x0a		;LED2 (gelb) ansteuern?
	brne	t3c180			;ja -> bearbeiten
	lds	r16, ledflg		;sonst LED-Flags nochmals holen
	bst	r16, 5			;Erinnerungs-Flag gesetzt?
	brts	t3c160			;ja -> bearbeiten
	sbr	r18, 0x20		;sonst LED2 auschalten
	rjmp	t3c210			;Ende
;
t3c160:	bst	r17, 0			;Sekunde geradzahlig?
	brtc	t3c170			;ja -> LED2 ausschalten
	lds	r16, t3cnt1		;sonst Timer3 Int-Zhler holen
	tst	r16			;Zhlerstand = 0?
	brne	t3c170			;nein -> LED2 ausschalten
	cbr	r18, 0x20		;sonst LED2 einschalten
	rjmp	t3c210			;Ende
t3c170:	sbr	r18, 0x20		;sonst LED2 ausschalten
	rjmp	t3c210			;Ende
;
t3c180:	bst	r16, 1			;LED2 Dauerlicht?
	brtc	t3c190			;nein -> berspringen
	cbr	r18, 0x20		;sonst LED2 einschalten
t3c190:	bst	r16, 3			;LED2 Blinken?
	brtc	t3c210			;nein -> berspringen
	bst	r16, 1			;zustzlich LED2 Dauerlicht?
	brtc	t3c200			;nein -> direkt zum Blinken
	bst	r17, 0			;Sekunde geradzahlig?
	brtc	t3c210			;ja -> berspringen
t3c200:	lds	r16, t3cnt1		;Timer3 Interrupt-Zhler holen
	sbr	r18, 0x20		;LED2 ausschalten
	cpi	r16, 9			;Zhlerstand < 9?
	brcs	t3c210			;ja -> LED2-Zustand ok
	cbr	r18, 0x20		;sonst LED2 einschalten
	cpi	r16, 17			;Zhlerstand < 17?
	brcs	t3c210			;ja -> LED2-Zustand ok
	sbr	r18, 0x20		;sonst LED2 ausschalten
	cpi	r16, 25			;Zhlerstand < 25?
	brcs	t3c210			;ja -> LED2-Zustand ok
	cbr	r18, 0x20		;sonst LED2 einschalten
	cpi	r16, 33			;Zhlerstand < 33?
	brcs	t3c210			;ja -> LED2-Zustand ok
	sbr	r18, 0x20		;sonst LED2 ausschalten
	cpi	r16, 42			;Zhlerstand < 42?
	brcs	t3c210			;ja -> LED2-Zustand ok
	cbr	r18, 0x20		;sonst LED2 einschalten
t3c210:	out	portd, r18		;neuen LED1/2-Status setzen
;
; Ausgabe von LED3 (blau) steuern
;
        in      r18, porte              ;Status von PortE holen
	lds	r16, confg1		;Konfigurationsbyte 1 holen
	bst	r16, 7			;Abwesenheit aktiviert?
	brtc	t3c220			;nein -> bearbeiten
	sbr	r18, 1 << 3		;sonst LED ausschalten
	rjmp	t3c240			;neuen LED-Status setzen
;
t3c220: lds     r17, pccmd              ;laufendes PC-Kommando holen
        tst     r17                     ;PC-Kommunikation aktiv?
        breq    t3c230                  ;nein -> Kommunikations-Flag prfen
        cbr     r18, 1 << 3             ;sonst LED3 einschalten
        rjmp    t3c240
t3c230:	lds	r17, comflg		;Kommunikations-Flag holen
	bst	r17, 0			;LSB holen und
	bld	r18, 3			;als neuen LED3-Status setzen
	sec				;Carry setzen
	ror	r17			;in MSB einschieben
	sts	comflg, r17		;Kommunikations-Flag wieder speichern
t3c240: out     porte, r18              ;neuen Status von LED3 setzen
;
; Tonausgabe steuern
;
	lds	r16, sigtim		;Signaldauer holen
	tst	r16			;Signal ausgeben?
	brne	t3c250			;ja -> bearbeiten
	rjmp	t3c280			;sonst berspringen

t3c250:	lds	r16, dflags		;Anzeige-Flags holen
	bst	r16, 5			;Lautsprecher eingeschaltet?
	brts	t3c260			;ja -> normale Signalausgabe
	ldi	r16, 49			;sonst Signalzhler auf Endwert
	sts	sigcnt, r16		;Signalzhler speichern
	ldi	r16, 1			;Signaldauer auf 1s setzen
	sts	sigtim, r16		;Signaldauer speichern

t3c260:	ldi	zl, low(sndtab * 2)
	ldi	zh, high(sndtab * 2)
	ldi	r18, byte3(sndtab * 2)	;Zeiger auf Sound-Tabelle
	clr	r17			;H-Byte fr Adressberechnung
	lds	r16, sigsnd		;Signalsound-Nummer holen
	add	r16, r16
	add	r16, r16
	add	r16, r16		;Offset * 8 fr Tabellenzugriff
	add	zl, r16
	adc	zh, r17
	adc	r18, r17		;Tabellenplatz berechnen
	out	rampz, r18
;
	lds	r16, sigcnt		;Signalzhler holen
	in	r18, tccr0		;Timer0 Steuerregister holen
	sbr	r18, 1 << com00		;Ton einschalten
	elpm	r17, z+			;1. Ton-Einschaltzeit ignorieren
	elpm	r17, z+			;1. Ton-Ausschaltzeit holen
	cp	r16, r17		;Zhlerstand < Ausschaltzeit?
	brcs	t3c270			;ja -> Tonzustand ok
	cbr	r18, 1 << com00		;Ton ausschalten
	elpm	r17, z+			;2. Ton-Einschaltzeit holen
	cp	r16, r17		;Zhlerstand < Ausschaltzeit?
	brcs	t3c270			;ja -> Tonzustand ok
	sbr	r18, 1 << com00		;Ton einschalten
	elpm	r17, z+			;2. Ton-Ausschaltzeit holen
	cp	r16, r17		;Zhlerstand < Ausschaltzeit?
	brcs	t3c270			;ja -> Tonzustand ok
	cbr	r18, 1 << com00		;Ton ausschalten
	elpm	r17, z+			;3. Ton-Einschaltzeit holen
	cp	r16, r17		;Zhlerstand < Ausschaltzeit?
	brcs	t3c270			;ja -> Tonzustand ok
	sbr	r18, 1 << com00		;Ton einschalten
	elpm	r17, z+			;3. Ton-Ausschaltzeit holen
	cp	r16, r17		;Zhlerstand < Ausschaltzeit?
	brcs	t3c270			;ja -> Tonzustand ok
	cbr	r18, 1 << com00		;Ton ausschalten
	elpm	r17, z+			;4. Ton-Einschaltzeit holen
	cp	r16, r17		;Zhlerstand < Ausschaltzeit?
	brcs	t3c270			;ja -> Tonzustand ok
	sbr	r18, 1 << com00		;Ton einschalten
	elpm	r17, z+			;4. Ton-Ausschaltzeit holen
	cp	r16, r17		;Zhlerstand < Ausschaltzeit?
	brcs	t3c270			;ja -> Tonzustand ok
	cbr	r18, 1 << com00		;sonst Ton immer ausschalten
t3c270:	out	tccr0, r18		;Timer0 Steueregister speichern
;
	inc	r16			;Signalzhler erhhen
	sts	sigcnt, r16		;Signalzhler wieder speichern
	cpi	r16, 50			;Endwert erreicht?
	brcs	t3c280			;nein -> weiter
	clr	r16			;sonst Signalzhler lschen
	sts	sigcnt, r16		;Signalzhler wieder speichern
	lds	r16, sigtim		;Signaldauer holen
	dec	r16			;Wert um 1s vermindern
	sts	sigtim, r16		;Signaldauer wieder speichern
;
; ADC nach folgendem Muster steuern:
;   0ms: ADC-Ref-Wert lesen (fr Spannungsberechnung), anschlieend ADC0 auswhlen
; 100ms: Wandlung starten
; 200ms: ADC0-Wert lesen (Lichtsensor), anschlieend ADC1 auswhlen
; 300ms: Wandlung starten
; 400ms: ADC1-Wert lesen (Batterie-Spannung), anschlieend ADC-Ref auswhlen
; 500ms: Wandlung starten
;
t3c280:	lds	r18, t3cnt1		;Timer1 Int-Zhler2 holen
	cpi	r18, 0			;Zeitpunkt 0ms?
        breq    t3c290                  ;ja -> bearbeiten
	rjmp	t3c310			;sonst weiter testen
;
t3c290:	lds	r16, adcref + 4
	lds	r17, adcref + 5		;Wert 2 holen
	sts	adcref + 6, r16
	sts	adcref + 7, r17		;und in Wert 3 kopieren
	lds	r16, adcref + 2
	lds	r17, adcref + 3		;Wert 1 holen
	sts	adcref + 4, r16
	sts	adcref + 5, r17		;und in Wert 2 kopieren
	lds	r16, adcref
	lds	r17, adcref + 1		;Wert 0 holen
	sts	adcref + 2, r16
	sts	adcref + 3, r17		;und in Wert 1 kopieren
	in	r16, adcl
	in	r17, adch		;ADC-Wert holen
	sts	adcref, r16
	sts	adcref + 1, r17		;und als Wert 0 speichern
;
        lds     r16, adcref + 7         ;Wert 3 High holen
        lds     r17, adcref + 6         ;Wert 3 Low holen
        or      r16, r17                ;verknpfen
        lds     r17, adcref + 5         ;Wert 2 High holen
        or      r16, r17                ;mit Summenwert verknpfen
        lds     r17, adcref + 4         ;Wert 2 Low holen
        or      r16, r17                ;mit Summenwert verknpfen
        lds     r17, adcref + 3         ;Wert 1 High holen
        or      r16, r17                ;mit Summenwert verknpfen
        lds     r17, adcref + 2         ;Wert 1 Low holen
        or      r16, r17                ;mit Summenwert verknpfen
        brne    t3c300                  ;Summerwert = 0? nein -> berspringen
;
        lds     r16, adcref             ;sonst
        lds     r17, adcref + 1         ;Wert 0 holen
        sts     adcref + 2, r16
        sts     adcref + 3, r17         ;in Wert 1 kopieren
        sts     adcref + 4, r16
        sts     adcref + 5, r17         ;in Wert 2 kopieren
        sts     adcref + 6, r16
        sts     adcref + 7, r17         ;in Wert 3 kopieren
;
t3c300:	ldi	r17, 0
	out	admux, r17		;Multiplexer auf ADC0 setzen
	rjmp	t3c400
;
t3c310:	cpi	r18, 5			                        ;Zeitpunkt 100ms?
	brne	t3c320			                        ;nein -> weiter testen
	ldi	r17, (1 << aden) | (1 << adsc) | (7 << adps0)   ;ADC ein, Vorteiler = 128
	out	adcsr, r17		                        ;ADC Start Conversion
	rjmp	t3c400
;
t3c320:	cpi	r18, 10			;Zeitpunkt 200ms?
	brne	t3c330			;nein -> weiter testen
	in	r16, adcl
	in	r17, adch		;sonst ADC-Wert holen
	lsr	r17
	ror	r16
	lsr	r17			;10-Bit-Wert in 8-Bit-Wert
	ror	r16			;umwandeln
	sts	adclig, r16		;und als ADC0-Wert speichern
	ldi	r17, 1 << mux0
	out	admux, r17		;Multiplexer auf ADC1 setzen
	rjmp	t3c400
;
t3c330:	cpi	r18, 15			                        ;Zeitpunkt 300ms?
	brne	t3c340			                        ;nein -> weiter testen
	ldi	r17, (1 << aden) | (1 << adsc) | (7 << adps0)   ;ADC ein, Vorteiler = 128
	out	adcsr, r17		                        ;ADC Start Conversion
	rjmp	t3c400
;
t3c340:	cpi	r18, 20			;Zeitpunkt 400ms?
        breq    t3c350                  ;ja -> bearbeiten
	rjmp	t3c370			;sonst weiter testen
;
t3c350:	lds	r16, adcbat + 4
	lds	r17, adcbat + 5		;Wert 2 holen
	sts	adcbat + 6, r16
	sts	adcbat + 7, r17		;und in Wert 3 kopieren
	lds	r16, adcbat + 2
	lds	r17, adcbat + 3		;Wert 1 holen
	sts	adcbat + 4, r16
	sts	adcbat + 5, r17		;und in Wert 2 kopieren
	lds	r16, adcbat
	lds	r17, adcbat + 1		;Wert 0 holen
	sts	adcbat + 2, r16
	sts	adcbat + 3, r17		;und in Wert 1 kopieren
	in	r16, adcl
	in	r17, adch		;ADC-Wert holen
	sts	adcbat, r16
	sts	adcbat + 1, r17		;und als Wert 0 speichern
;
        lds     r16, adcbat + 7         ;Wert 3 High holen
        lds     r17, adcbat + 6         ;Wert 3 Low holen
        or      r16, r17                ;verknpfen
        lds     r17, adcbat + 5         ;Wert 2 High holen
        or      r16, r17                ;mit Summenwert verknpfen
        lds     r17, adcbat + 4         ;Wert 2 Low holen
        or      r16, r17                ;mit Summenwert verknpfen
        lds     r17, adcbat + 3         ;Wert 1 High holen
        or      r16, r17                ;mit Summenwert verknpfen
        lds     r17, adcbat + 2         ;Wert 1 Low holen
        or      r16, r17                ;mit Summenwert verknpfen
        brne    t3c360                  ;Summerwert = 0? nein -> berspringen
;
        lds     r16, adcbat             ;sonst
        lds     r17, adcbat + 1         ;Wert 0 holen
        sts     adcbat + 2, r16
        sts     adcbat + 3, r17         ;in Wert 1 kopieren
        sts     adcbat + 4, r16
        sts     adcbat + 5, r17         ;in Wert 2 kopieren
        sts     adcbat + 6, r16
        sts     adcbat + 7, r17         ;in Wert 3 kopieren
;
t3c360:	ldi	r17, 0x1e << mux0
	out	admux, r17		;Multiplexer auf ADC-Ref setzen
	rjmp	t3c400
;
t3c370:	cpi	r18, 25			                        ;Zeitpunkt 500ms?
	brne	t3c400			                        ;nein -> weiter testen
	ldi	r17, (1 << aden) | (1 << adsc) | (7<<adps0)     ;ADC ein, Vorteiler = 128
	out	adcsr, r17		                        ;ADC Start Conversion
;
; zum Zeitpunkt 500ms Flag setzen, um Datenfelder der Status-Anzeige neu auszugeben
;
	lds	r16, xflag2		;sonst verschiedene Flags 2 holen
	sbr	r16, 1 << 4		;Datenfelder neu ausgeben
	sts	xflag2, r16		;Flags wieder speichern
;
; Rckwrtszhlung des Timeout-Zhlers fr die zyklische Fehleranzeige
;
t3c400:	lds	r16, errtio		;Fehler-Timeout holen
	cpi	r16, 0			;Timeout-Zhler = 0 (Ende)?
	breq	t3c410			;ja -> nicht bearbeiten
	cpi	r16, 0xff		;Timeout-Zhler = 0xff (Stopp)?
	breq	t3c410			;ja -> nicht bearbeiten
	dec	r16			;sonst Zhler vermindern
	sts	errtio, r16		;Zhler wieder speichern
;
; Rckwrtszhlung des Timeout-Zhlers fr die kurzzeitige Umschaltung auf Nacht-Helligkeit
;
t3c410: lds     r16, nlitio             ;Timeout-Zhler fr Nacht-Helligkeit holen
        cpi     r16, 0                  ;Timeout-Zhler = 0 (Ende)?
        breq    t3c500                  ;ja -> nicht bearbeiten
        cpi     r16, 0xff               ;Timeout-Zhler = 0xff (Stopp)?
        breq    t3c500                  ;ja -> nicht bearbeiten
        dec     r16                     ;sonst Zhler vermindern
        sts     nlitio, r16		;Zhler wieder speichern
;
; Sekunden-Timer (Software-Teilung durch 50)
;
t3c500:	lds	r16, t3cnt1		;Timer1 Int-Zhler2 holen
	inc	r16			;erhhen
	sts	t3cnt1, r16		;und wieder speichern
	cpi	r16, 50			;Endwert erreicht?
	brcc	t3c510			;ja -> weiter
	rjmp	t3cend			;sonst Ende
;
t3c510:	clr	r16			;Timer1 Int-Zhler2 lschen
	sts	t3cnt1, r16		;und wieder speichern
;
; Rckwrtszhlung der Timeout-Zhler fr die 31 Sensoren, den Meldungs-Timeout, den Men-Timeout und den Lautsprecher-Timeout
;
	ldi	zl, low(sentio)
	ldi	zh, high(sentio)	;Zeiger auf Sensor-Timeouts
	ldi	r17, 31			;31 Werte bearbeiten
t3c520:	ld	r16, z			;Timeout-Zhler holen
	cpi	r16, 0			;Timeout-Zhler = 0 (Ende)?
	breq	t3c530			;ja -> nicht bearbeiten
	cpi	r16, 0xff		;Timeout-Zhler = 0xff (Stopp)?
	breq	t3c530			;ja -> nicht bearbeiten
	dec	r16			;sonst Zhler vermindern
t3c530:	st	z+, r16			;Zhler wieder speichern
	dec	r17			;alle Zhler bearbeitet?
	brne	t3c520			;nein -> Schleife
;
	lds	r16, msgtio		;Meldungs-Timeout holen
	cpi	r16, 0			;Timeout-Zhler = 0 (Ende)?
	breq	t3c540			;ja -> nicht bearbeiten
	cpi	r16, 0xff		;Timeout-Zhler = 0xff (Stopp)?
	breq	t3c540			;ja -> nicht bearbeiten
	dec	r16			;sonst Zhler vermindern
	sts	msgtio, r16		;Zhler wieder speichern
;
t3c540:	lds	r16, mentio		;Men-Timeout holen
	cpi	r16, 0			;Timeout-Zhler = 0 (Ende)?
	breq	t3c550			;ja -> nicht bearbeiten
	cpi	r16, 0xff		;Timeout-Zhler = 0xff (Stopp)?
	breq	t3c550			;ja -> nicht bearbeiten
	dec	r16			;sonst Zhler vermindern
	sts	mentio, r16		;Zhler wieder speichern
;
t3c550:	lds	r16, lsptio		;Lautsprecher-Timeout holen
	cpi	r16, 0			;Timeout-Zhler = 0 (Ende)?
	breq	t3c600			;ja -> nicht bearbeiten
	cpi	r16, 0xff		;Timeout-Zhler = 0xff (Stopp)?
	breq	t3c600			;ja -> nicht bearbeiten
	dec	r16			;sonst Zhler vermindern
	sts	lsptio, r16		;Zhler wieder speichern
;
; Minuten-Timer (Softwareteilung durch 60)
;
t3c600:	lds	r16, t3cnt2		;Timer3 Sekunden-Zhler holen
	inc	r16			;erhhen
	sts	t3cnt2, r16		;und wieder speichern
	cpi	r16, 60			;Endwert erreicht?
	brcc	t3c610			;ja -> weiter
	rjmp	t3cend			;sonst Ende
;
t3c610:	clr	r16			;Timer3 Sekunden-Zhler lschen
	sts	t3cnt2, r16		;und wieder speichern
;
; Rckwrtszhlung des Timeout-Zhler fr die LCD-Abschaltung
;
	lds	r16, lcdtio		;LCD-Timeout holen
	cpi	r16, 0			;Timeout-Zhler = 0 (Ende)?
	breq	t3c800			;ja -> nicht bearbeiten
	cpi	r16, 0xff		;Timeout-Zhler = 0xff (Stopp)?
	breq	t3c800			;ja -> nicht bearbeiten
	dec	r16			;sonst Zhler vermindern
	sts	lcdtio, r16		;Zhler wieder speichern
;
; Stunden-Timer (Softwarteilung durch 60)
;
t3c800:	lds	r16, t3cnt3		;Timer3 Minuten-Zhler holen
	inc	r16			;erhhen
	sts	t3cnt3, r16		;und wieder speichern
	cpi	r16, 60			;Endwert erreicht?
	brcc	t3c810			;ja -> weiter
	rjmp	t3cend			;sonst Ende
;
t3c810:	clr	r16			;Timer3 Minuten-Zhler lschen
	sts	t3cnt3, r16		;und wieder speichern
;
; Rckwrtszhlung des DCF77-Timeout-Zhlers
;
	lds	r16, dcftio		;DCF77-Timeout holen
	cpi	r16, 0			;Timeout-Zhler = 0 (Ende)?
	breq	t3cend			;ja -> nicht bearbeiten
	cpi	r16, 0xff		;Timeout-Zhler = 0xff (Stopp)?
	breq	t3cend			;ja -> nicht bearbeiten
	dec	r16			;sonst Zhler vermindern
	sts	dcftio, r16		;Zhler wieder speichern
;
t3cend:	pop	zl
	out	rampz, zl
	pop	zh
	pop	zl
	pop	r18
	pop	r17
	pop	r16			;diverse Register restaurieren
	out	sreg, insreg		;SREG wiederherstellen
	reti
;
; Interrupt-Routine "USART0 Daten-Register leer", nchstes Byte aus dem Sende-Ringpuffer0 holen und senden
;
txdre0:	in	insreg, sreg		;SREG sichern
	push	r16
	push	xl
	push	xh
	push	yl
	push	yh			;diverse Register sichern
	lds	xl, txp0wr
	lds	xh, txp0wr + 1		;Schreibzeiger holen
	lds	yl, txp0rd
	lds	yh, txp0rd + 1		;Lesezeiger holen
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;sind Zeichen im Puffer?
	breq	txd030			;nein -> Sendeint abschalten
	ld	r16, y+			;sonst nchstes Byte holen
	out	udr0, r16		        ;und senden
	ldi	xl, low(txbuf0 + txbu0l)
	ldi	xh, high(txbuf0 + txbu0l)	;Adresse Pufferende laden
	sub	xl, yl
	sbc	xh, yh			;Differenz berechnen
	or	xh, xl			;Endadresse erreicht?
	brne	txd010			;nein -> weiter
	ldi	yl, low(txbuf0)
	ldi	yh, high(txbuf0)	;sonst Adresse auf Pufferanfang
txd010:	sts	txp0rd, yl
	sts	txp0rd + 1, yh		;Lesezeiger wieder speichern
txd020:	pop	yh
	pop	yl
	pop	xh
	pop	xl
	pop	r16			;diverse Register restaurieren
	out	sreg, insreg		;SREG wiederherstellen
	reti
;
txd030:	cbi	ucsr0b, udrie0		;Sendeinterrupt abschalten
	rjmp	txd020			;Ende
;
; Interrupt-Routine "USART0 Empfang komplett", empfangenes Byte im Empfangs-Ringpuffer0 speichern
;
rxcpl0:	in	insreg, sreg		;SREG sichern
	push	r16
	push	xl
	push	xh
	push	yl
	push	yh			;diverse Register sichern
	lds	xl, rxp0wr
	lds	xh, rxp0wr + 1		;Schreibzeiger holen
	in	r16, udr0		;empfangenes Byte lesen
	st	x+, r16			        ;Byte speichern, Zeiger + 1
	ldi	yl, low(rxbuf0 + rxbu0l)
	ldi	yh, high(rxbuf0 + rxbu0l)	;Adresse Pufferende laden
	sub	yl, xl
	sbc	yh, xh			;Differenz berechnen
	or	yh, yl			;Endadresse erreicht?
	brne	rxc010			;nein -> weiter
	ldi	xl, low(rxbuf0)
	ldi	xh, high(rxbuf0)	;sonst Adresse auf Anfangswert
rxc010:	sts	rxp0wr, xl
	sts	rxp0wr + 1, xh		;Schreibzeiger speichern
	pop	yh
	pop	yl
	pop	xh
	pop	xl
	pop	r16			;diverse Register restaurieren
	out	sreg, insreg		;SREG wiederherstellen
	reti
;
; Interrupt-Routine "USART1 Empfang komplett", empfangenes Byte im Empfangs-Ringpuffer1 speichern
;
rxcpl1:	in	insreg, sreg		;SREG sichern
	push	r16
	push	xl
	push	xh
	push	yl
	push	yh			;diverse Register sichern
	lds	xl, rxp1wr
	lds	xh, rxp1wr + 1		;Schreibzeiger holen
	lds	r16, udr1		;empfangenes Byte lesen
	st	x+, r16			        ;Byte speichern, Zeiger + 1
	ldi	yl, low(rxbuf1 + rxbu1l)
	ldi	yh, high(rxbuf1 + rxbu1l)	;Adresse Pufferende laden
	sub	yl, xl
	sbc	yh, xh			;Differenz berechnen
	or	yh, yl			;Endadresse erreicht?
	brne	rxc110			;nein -> weiter
	ldi	xl, low(rxbuf1)
	ldi	xh, high(rxbuf1)	;sonst Adresse auf Anfangswert
rxc110:	sts	rxp1wr, xl
	sts	rxp1wr + 1, xh		;Schreibzeiger speichern
	pop	yh
	pop	yl
	pop	xh
	pop	xl
	pop	r16			;diverse Register restaurieren
	out	sreg, insreg		;SREG wiederherstellen
	reti
;
;
.cseg
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Listen, Tabellen, Datenfelder, Zeichensatz-Definitionen
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; ASCII Zeichentabelle, jedes Zeichen wird mit 12 Byte definiert, Byte 0 enthlt die Zeichenbreite (0-8), die restlichen Bytes enthalten die Bitmap des
; Zeichens, Beispiel Zeichen 'A':
;
; Byte  0 = 5 (Zeichen ist 5 Pixel breit)
; Byte  1 = 0		. . . . . . . .         (fr Umlaute)
; Byte  2 = 4	        . . . . . o . .
; Byte  3 = 0x0a	. . . . o . o .
; Byte  4 = 0x11	. . . o . . . o
; Byte  5 = 0x11	. . . o . . . o
; Byte  6 = 0x1f	. . . o o o o o
; Byte  7 = 0x11	. . . o . . . o
; Byte  8 = 0x11	. . . o . . . o
; Byte  9 = 0x11	. . . o . . . o
; Byte 10 = 0		. . . . . . . .		(fr Unterlngen)
; Byte 11 = 0		. . . . . . . .		(fr Unterlngen)
;
; ASCII 0x20 - 0x2f
;
chrset:	.db     5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x20 ' '
	.db     1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0			                ;0x21 '!'
	.db     5, 0, 0x1b, 0x1b, 9, 0x12, 0, 0, 0, 0, 0, 0		                ;0x22 '"'
	.db     5, 0, 0, 0x0a, 0x0a, 0x1f, 0x0a, 0x1f, 0x0a, 0x0a, 0, 0                 ;0x23 '#'
	.db     5, 0, 4, 0x0e, 0x14, 0x0e, 5, 0x15, 0x0e, 4, 0, 0	                ;0x24 '$'
	.db     5, 0, 0, 0x18, 0x19, 2, 4, 8, 0x13, 3, 0, 0	                        ;0x25 '%'
	.db     5, 0, 0x0c, 0x12, 0x12, 0x14, 8, 0x15, 0x12, 0x0d, 0, 0                 ;0x26 '&'
	.db     2, 0, 3, 3, 1, 2, 0, 0, 0, 0, 0, 0			                ;0x27 '''
	.db     3, 0, 1, 2, 4, 4, 4, 4, 2, 1, 0, 0			                ;0x28 '('
	.db     3, 0, 4, 2, 1, 1, 1, 1, 2, 4, 0, 0			                ;0x29 ')'
	.db     5, 0, 0, 0, 0x0a, 4, 0x1f, 4, 0x0a, 0, 0, 0		                ;0x2a '*'
	.db     5, 0, 0, 0, 4, 4, 0x1f, 4, 4, 0, 0, 0			                ;0x2b '+'
	.db     2, 0, 0, 0, 0, 0, 0, 0, 3, 3, 1, 2			                ;0x2c ','
	.db     5, 0, 0, 0, 0, 0, 0x1f, 0, 0, 0, 0, 0			                ;0x2d '-'
	.db     2, 0, 0, 0, 0, 0, 0, 0, 3, 3, 0, 0			                ;0x2e '.'
	.db     4, 0, 1, 1, 2, 2, 4, 4, 8, 8, 0, 0			                ;0x2f '/'
;
; ASCII 0x30 - 0x3f
;
	.db     5, 0, 0x0e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0, 0              ;0x30 '0'
	.db     5, 0, 2, 6, 0x0a, 2, 2, 2, 2, 2, 0, 0			                ;0x31 '1'
	.db     5, 0, 0x0e, 0x11, 1, 2, 4, 8, 0x10, 0x1f, 0, 0		                ;0x32 '2'
	.db     5, 0, 0x0e, 0x11, 1, 6, 1, 1, 0x11, 0x0e, 0, 0		                ;0x33 '3'
	.db     5, 0, 2, 6, 0x0a, 0x12, 0x1f, 2, 2, 2, 0, 0		                ;0x34 '4'
	.db     5, 0, 0x1f, 0x10, 0x10, 0x1e, 1, 1, 0x11, 0x0e, 0, 0	                ;0x35 '5'
	.db     5, 0, 0x0e, 0x11, 0x10, 0x1e, 0x11, 0x11, 0x11, 0x0e, 0, 0              ;0x36 '6'
	.db     5, 0, 0x1f, 1, 1, 2, 2, 4, 4, 4, 0, 0   			        ;0x37 '7'
	.db     5, 0, 0x0e, 0x11, 0x11, 0x0e, 0x11, 0x11, 0x11, 0x0e, 0, 0              ;0x38 '8'
	.db     5, 0, 0x0e, 0x11, 0x11, 0x11, 0x0f, 1, 0x11, 0x0e, 0, 0                 ;0x39 '9'
	.db     2, 0, 0, 0, 0, 3, 3, 0, 3, 3, 0, 0			                ;0x3a ':'
	.db     2, 0, 0, 0, 0, 3, 3, 0, 3, 3, 1, 2			                ;0x3b ';'
	.db     4, 0, 0, 1, 2, 4, 8, 4, 2, 1, 0, 0			                ;0x3c '<'
	.db     5, 0, 0, 0, 0, 0x1f, 0, 0x1f, 0, 0, 0, 0		                ;0x3d '='
	.db     4, 0, 0, 8, 4, 2, 1, 2, 4, 8, 0, 0			                ;0x3e '>'
	.db     5, 0, 0x0e, 0x11, 1, 2, 4, 4, 0, 4, 0, 0		                ;0x3f '?'
;
; ASCII 0x40 - 0x4f
;
	.db     7, 0, 0x1e, 0x21, 0x4d, 0x55, 0x55, 0x4e, 0x20, 0x1e, 0, 0              ;0x40 '@'
	.db     5, 0, 4, 0x0a, 0x11, 0x11, 0x1f, 0x11, 0x11, 0x11, 0, 0                 ;0x41 'A'
	.db     5, 0, 0x1e, 0x11, 0x11, 0x1e, 0x11, 0x11, 0x11, 0x1e, 0, 0              ;0x42 'B'
	.db     5, 0, 0x0e, 0x11, 0x10, 0x10, 0x10, 0x10, 0x11, 0x0e, 0, 0              ;0x43 'C'
	.db     5, 0, 0x1c, 0x12, 0x11, 0x11, 0x11, 0x11, 0x12, 0x1c, 0, 0              ;0x44 'D'
	.db     5, 0, 0x1f, 0x10, 0x10, 0x1e, 0x10, 0x10, 0x10, 0x1f, 0, 0              ;0x45 'E'
	.db     5, 0, 0x1f, 0x10, 0x10, 0x1e, 0x10, 0x10, 0x10, 0x10, 0, 0              ;0x46 'F'
	.db     5, 0, 0x0e, 0x11, 0x10, 0x10, 0x13, 0x11, 0x11, 0x0e, 0, 0              ;0x47 'G'
	.db     5, 0, 0x11, 0x11, 0x11, 0x1f, 0x11, 0x11, 0x11, 0x11, 0, 0              ;0x48 'H'
	.db     1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0			                ;0x49 'I'
	.db     4, 0, 1, 1, 1, 1, 1, 1, 9, 6, 0, 0			                ;0x4a 'J'
	.db     5, 0, 0x11, 0x12, 0x14, 0x18, 0x18, 0x14, 0x12, 0x11, 0, 0              ;0x4b 'K'
	.db     5, 0, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x1f, 0, 0              ;0x4c 'L'
	.db     5, 0, 0x11, 0x1b, 0x15, 0x15, 0x11, 0x11, 0x11, 0x11, 0, 0              ;0x4d 'M'
	.db     5, 0, 0x11, 0x19, 0x19, 0x15, 0x15, 0x13, 0x13, 0x11, 0, 0              ;0x4e 'N'
	.db     5, 0, 0x0e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0, 0              ;0x4f '0'
;
; ASCII 0x50 - 0x5f
;
	.db     5, 0, 0x1e, 0x11, 0x11, 0x11, 0x1e, 0x10, 0x10, 0x10, 0, 0              ;0x50 'P'
	.db     5, 0, 0x0e, 0x11, 0x11, 0x11, 0x11, 0x15, 0x12, 0x0d, 0, 0              ;0x51 'Q'
	.db     5, 0, 0x1e, 0x11, 0x11, 0x11, 0x1e, 0x14, 0x12, 0x11, 0, 0              ;0x52 'R'
	.db     5, 0, 0x0e, 0x11, 0x10, 0x0e, 1, 1, 0x11, 0x0e, 0, 0	                ;0x53 'S'
	.db     5, 0, 0x1f, 4, 4, 4, 4, 4, 4, 4, 0, 0			                ;0x54 'T'
	.db     5, 0, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0, 0              ;0x55 'U'
	.db     5, 0, 0x11, 0x11, 0x11, 0x11, 0x0a, 0x0a, 4, 4, 0, 0	                ;0x56 'V'
	.db     5, 0, 0x11, 0x11, 0x11, 0x11, 0x15, 0x15, 0x15, 0x0a, 0, 0              ;0x57 'W'
	.db     5, 0, 0x11, 0x11, 0x0a, 4, 0x0a, 0x11, 0x11, 0x11, 0, 0                 ;0x58 'X'
	.db     5, 0, 0x11, 0x11, 0x11, 0x0a, 4, 4, 4, 4, 0, 0		                ;0x59 'Y'
	.db     6, 0, 0x3f, 1, 2, 4, 8, 0x10, 0x20, 0x3f, 0, 0		                ;0x5a 'Z'
	.db     3, 0, 7, 4, 4, 4, 4, 4, 4, 7, 0, 0      			        ;0x5b '['
	.db     4, 0, 8, 8, 4, 4, 2, 2, 1, 1, 0, 0			                ;0x5c '\'
	.db     3, 0, 7, 1, 1, 1, 1, 1, 1, 7, 0, 0			                ;0x5d ']'
	.db     5, 0, 4, 0x0a, 0x11, 0, 0, 0, 0, 0, 0, 0		                ;0x5e '^'
	.db     5, 0, 0, 0, 0, 0, 0, 0, 0, 0x1f, 0, 0			                ;0x5f '_'
;
; ASCII 0x60 - 0x6f
;
	.db     3, 0, 4, 2, 1, 0, 0, 0, 0, 0, 0, 0			                ;0x60 '`'
	.db     5, 0, 0, 0, 0, 0x0e, 1, 0x0f, 0x11, 0x0f, 0, 0		                ;0x61 'a'
	.db     5, 0, 0x10, 0x10, 0x10, 0x1e, 0x11, 0x11, 0x11, 0x1e, 0, 0              ;0x62 'b'
	.db     4, 0, 0, 0, 0, 7, 8, 8, 8, 7, 0, 0			                ;0x63 'c'
	.db     5, 0, 1, 1, 1, 0x0f, 0x11, 0x11, 0x11, 0x0f, 0, 0	                ;0x64 'd'
	.db     5, 0, 0, 0, 0, 0x0e, 0x11, 0x1f, 0x10, 0x0e, 0, 0	                ;0x65 'e'
	.db     3, 0, 1, 2, 2, 7, 2, 2, 2, 2, 0, 0			                ;0x66 'f'
	.db     5, 0, 0, 0, 0, 0x0f, 0x11, 0x11, 0x11, 0x0f, 1, 0x0e	                ;0x67 'g'
	.db     5, 0, 0x10, 0x10, 0x10, 0x1e, 0x11, 0x11, 0x11, 0x11, 0, 0              ;0x68 'h'
	.db     1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0			                ;0x69 'i'
	.db     2, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 2			                ;0x6a 'j'
	.db     4, 0, 8, 8, 8, 9, 0x0a, 0x0c, 0x0a, 9, 0, 0		                ;0x6b 'k'
	.db     2, 0, 2, 2, 2, 2, 2, 2, 2, 1, 0, 0			                ;0x6c 'l'
	.db     5, 0, 0, 0, 0, 0x1a, 0x15, 0x15, 0x15, 0x15, 0, 0	                ;0x6d 'm'
	.db     5, 0, 0, 0, 0, 0x16, 0x19, 0x11, 0x11, 0x11, 0, 0	                ;0x6e 'n'
	.db     5, 0, 0, 0, 0, 0x0e, 0x11, 0x11, 0x11, 0x0e, 0, 0	                ;0x6f 'o'
;
; ASCII 0x70 - 0x7f
;
	.db     5, 0, 0, 0, 0, 0x1e, 0x11, 0x11, 0x11, 0x1e, 0x10, 0x10                 ;0x70 'p'
	.db     5, 0, 0, 0, 0, 0x0f, 0x11, 0x11, 0x11, 0x0f, 1, 1	                ;0x71 'q'
	.db     4, 0, 0, 0, 0, 0x0a, 0x0d, 8, 8, 8, 0, 0		                ;0x72 'r'
	.db     5, 0, 0, 0, 0, 0x0e, 0x10, 0x0e, 1, 0x1e, 0, 0		                ;0x73 's'
	.db     3, 0, 2, 2, 2, 7, 2, 2, 2, 1, 0, 0			                ;0x74 't'
	.db     5, 0, 0, 0, 0, 0x11, 0x11, 0x11, 0x11, 0x0e, 0, 0	                ;0x75 'u'
	.db     5, 0, 0, 0, 0, 0x11, 0x11, 0x11, 0x0a, 4, 0, 0		                ;0x76 'v'
	.db     5, 0, 0, 0, 0, 0x11, 0x11, 0x15, 0x15, 0x0a, 0, 0	                ;0x77 'w'
	.db     5, 0, 0, 0, 0, 0x11, 0x0a, 4, 0x0a, 0x11, 0, 0		                ;0x78 'x'
	.db     5, 0, 0, 0, 0, 0x11, 0x11, 0x11, 0x11, 0x0f, 1, 0x0e	                ;0x79 'y'
	.db     5, 0, 0, 0, 0, 0x1f, 2, 4, 8, 0x1f, 0, 0		                ;0x7a 'z'
	.db     3, 0, 1, 2, 2, 4, 2, 2, 2, 1, 0, 0			                ;0x7b '{'
	.db     1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0			                ;0x7c '|'
	.db     3, 0, 4, 2, 2, 1, 2, 2, 2, 4, 0, 0			                ;0x7d '}'
	.db     5, 0, 8, 0x15, 2, 0, 0, 0, 0, 0, 0, 0			                ;0x7e '~'
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0		                        ;0x7f Endezeichen
;
; ASCII 0x80 - 0x8f (fr Umlaute und Sonderzeichen)
;
	.db     5, 0x11, 4, 0x0a, 0x11, 0x11, 0x1f, 0x11, 0x11, 0x11, 0, 0              ;0x80 ''
	.db     5, 0x0a, 0, 0x0e, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0, 0              ;0x81 ''
	.db     5, 0x0a, 0, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0e, 0, 0              ;0x82 ''
	.db     5, 0, 0, 0x0a, 0, 0x0e, 1, 0x0f, 0x11, 0x0f, 0, 0	                ;0x83 ''
	.db     5, 0, 0, 0x0a, 0, 0x0e, 0x11, 0x11, 0x11, 0x0e, 0, 0	                ;0x84 ''
	.db     5, 0, 0, 0x0a, 0, 0x11, 0x11, 0x11, 0x11, 0x0e, 0, 0	                ;0x85 ''
	.db     5, 0, 0x0e, 0x11, 0x11, 0x16, 0x11, 0x11, 0x11, 0x16, 0, 0              ;0x86 ''
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x87 frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x88 frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x89 frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x8a frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x8b frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x8c frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x8d frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x8e frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0			                ;0x8f frei
;
; ^  bis an diese Stelle knnen eigene Zeichen definiert werden
; v  ab hier stehen Sonderzeichen
;
; ASCII 0x90 - 0x9f (nur fr Sonderzeichen)
;
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0	                                ;0x90 frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0	                                ;0x91 frei
	.db     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0	                                ;0x92 frei
	.db     5, 0, 4, 0x0e, 0x1f, 4, 4, 0x1f, 0x0e, 4, 0, 0                          ;0x93 Symbol fr Skala-Angleichung
	.db     4, 0, 0, 0, 0, 6, 0x0f, 0x0f, 6, 0, 0, 0			        ;0x94 Punkt fr Groanzeige
	.db     4, 0, 0, 0, 0, 6, 0x0f, 0x0f, 7, 3, 2, 4			        ;0x95 Komma fr Groanzeige
	.db     7, 0x60, 0x51, 0x62, 0x56, 0x64, 0x0e, 0x14, 0x16, 0x12, 0x11, 0x38	;0x96 Antennen-Symbol + Bluetooth
	.db     7, 0, 8, 8, 0x14, 0x14, 0x22, 0x22, 0x41, 0x7f, 0, 0                    ;0x97 Delta-Symbol
	.db     6, 0, 0, 0, 0x20, 0x11, 9, 5, 3, 0x1f, 0, 0	                        ;0x98 Minimum-Symbol
	.db     6, 0, 0, 0, 0x1f, 3, 5, 9, 0x11, 0x20, 0, 0                 	        ;0x99 Maximum-Symbol
	.db     7, 0, 0, 0x1d, 0x22, 0x45, 0x49, 0x51, 0x22, 0x5c, 0, 0                 ;0x9a Mittelwert
	.db     5, 0, 1, 2, 4, 8, 0x0f, 1, 0x0a, 0x0c, 0x0e, 0              	        ;0x9b Blitz-Symbol
	.db     7, 0, 1, 2, 6, 4, 0x0e, 0x14, 0x16, 0x12, 0x11, 0x38                    ;0x9c Antennen-Symbol
	.db     6, 1, 3, 7, 0x3d, 0x25, 0x25, 0x25, 0x3d, 7, 3, 1                       ;0x9d Lautsprecher-Symbol
	.db     5, 0, 0, 0, 0x11, 0x0a, 4, 0x0a, 0x11, 0, 0, 0	                        ;0x9e Antenne/Lautsprecher 'aus'
	.db     5, 0, 4, 2, 9, 5, 0x15, 5, 9, 2, 4, 0		                        ;0x9f Antenne/Lautsprecher 'ein'
;
; ASCII 0xa0 - 0xaf (nur fr Sonderzeichen)
;
	.db     3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0		                        ;0xa0 schmales Leerzeichen
	.db     7, 8, 0x1c, 0x1c, 0x3e, 0x3e, 0x3e, 0x7f, 0x10, 0x30, 0x30, 0	        ;0xa1 Alarmsymbol
	.db     7, 8, 0x14, 0x14, 0x14, 0x14, 0x1c, 0x1c, 0x3e, 0x3e, 0x1c, 0		;0xa2 Temperatursymbol
	.db     7, 0x0e, 0x0e, 0x0e, 0x0c, 0x1c, 0x18, 0x18, 0, 0x30, 0x30, 0		;0xa3 Fehlersymbol
	.db     7, 0x41, 0x63, 0x22, 0x36, 0x1c, 0x1c, 0x36, 0x22, 0x63, 0x41, 0	;0xa4 Nicht-Ok-Symbol
	.db     2, 0, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0			                ;0xa5 ''
	.db     8, 0, 0xce, 0xd1, 0x10, 0x10, 0x10, 0x10, 0x11, 0x0e, 0, 0              ;0xa6 'C'
	.db     5, 0, 0x18, 0x18, 0, 0, 0, 0, 0, 0, 0, 0		                ;0xa7 ' ' breites Gradzeichen (lscht anschlieendes Ausfallzeichen)
	.db     5, 0, 0x18, 0x18, 0, 0x0f, 3, 5, 9, 0x10, 0, 0		                ;0xa8 '^' Temperaturberschreitung
	.db     5, 0, 0x18, 0x18, 0, 0x10, 9, 5, 3, 0x0f, 0, 0		                ;0xa9 'v' Temperaturunterschreitung
	.db     5, 0, 0x19, 0x19, 1, 1, 1, 1, 0, 1, 0, 0		                ;0xaa '!' Temperatursensorausfall
	.db     2, 0, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0		                        ;0xab schmales ''
	.db     3, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7	                                ;0xac Cursorsymbol
	.db     5, 0, 4, 0x0a, 0x0a, 0x0a, 0x1f, 0x1b, 0x1b, 0x1f, 0, 0			;0xad Schlosssymbol
	.db     7, 1, 1, 3, 3, 6, 4, 0x4c, 0x68, 0x38, 0x10, 0	                        ;0xae Ok-Symbol
	.db     8, 0, 0, 0x13, 0x23, 0x43, 0xff, 0x43, 0x23, 0x13, 0, 0			;0xaf Lschsymbol
;
; ASCII 0xb0 - 0xb7 (nur fr Sonderzeichen)
;
	.db     5, 0, 1, 3, 5, 9, 0x11, 9, 5, 3, 1, 0		                        ;0xb0 Pfeil links hohl
	.db     5, 0, 1, 3, 7, 0x0f, 0x1f, 0x0f, 7, 3, 1, 0	                        ;0xb1 Pfeil links voll
	.db     5, 0, 0x10, 0x18, 0x14, 0x12, 0x11, 0x12, 0x14, 0x18, 0x10, 0		;0xb2 Pfeil rechts hohl
	.db     5, 0, 0x10, 0x18, 0x1c, 0x1e, 0x1f, 0x1e, 0x1c, 0x18, 0x10, 0		;0xb3 Pfeil rechts voll
	.db     6, 0, 0, 0, 0x3f, 0x21, 0x21, 0x21, 0x21, 0x3f, 0, 0                    ;0xb4 Checkbox1 aus (leeres Quadrat)
	.db     6, 0, 0, 0, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0, 0                    ;0xb5 Checkbox1 ein (ausgeflltes Quadrat)
	.db     6, 0, 0, 0, 0x0c, 0x12, 0x21, 0x21, 0x12, 0x0c, 0, 0                    ;0xb6 Checkbox2 aus (nicht mehr genutzt)
	.db     6, 0, 0, 0, 0x0c, 0x1e, 0x3f, 0x3f, 0x1e, 0x0c, 0, 0                    ;0xb7 Checkbox2 ein (nicht mehr genutzt)
;
;
; Ziffern 0-9, sowie '-' und Leerzeichen im Groformat; besteht aus jeweils 2 Blcken im weiter oben beschriebenen 12-Byte-Format fr den oberen Teil, 2 fr
; den mittleren Teil und 2 fr den unteren Teil
;
; 0xb8 - 0xbd '0'
;
	.db     8, 0, 0, 7, 0x1f, 0x3f, 0x3f, 0x7c, 0x78, 0x78, 0xf8, 0xf0
	.db     8, 0, 0, 0xe0, 0xf8, 0xfc, 0xfc, 0x3e, 0x1e, 0x1e, 0x1f, 0x0f
	.db     8, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0
	.db     8, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f
	.db     8, 0xf0, 0xf8, 0x78, 0x78, 0x7c, 0x3f, 0x3f, 0x1f, 7, 0, 0
	.db     8, 0x0f, 0x1f, 0x1e, 0x1e, 0x3e, 0xfc, 0xfc, 0xf8, 0xe0, 0, 0
;
; 0xbe - 0xc3 '1'
;
	.db     8, 0, 0, 0, 0, 0, 0, 1, 3, 7, 0x0f, 0x1f
	.db     8, 0, 0, 0x78, 0x78, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0x78
	.db     8, 0x1e, 0x1c, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0, 0
;
; 0xc4 - 0xc9 '2'
;
	.db     8, 0, 0, 7, 0x1f, 0x3f, 0x7f, 0x7c, 0xf8, 0xf0, 0, 0
	.db     8, 0, 0, 0xe0, 0xf8, 0xfc, 0xfe, 0x3e, 0x1f, 0x0f, 0x0f, 0x0f
	.db     8, 0, 0, 0, 0, 0, 0, 0, 1, 3, 7, 0x0f
	.db     8, 0x0f, 0x1f, 0x1e, 0x3e, 0x3c, 0x7c, 0xf8, 0xf0, 0xe0, 0xc0, 0x80
	.db     8, 0x1f, 0x3e, 0x7c, 0x78, 0xf8, 0xff, 0xff, 0xff, 0xff, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0xff, 0xff, 0xff, 0xff, 0, 0
;
; 0xca - 0xcf '3'
;
	.db     8, 0, 0, 3, 0x0f, 0x1f, 0x3f, 0x3e, 0x7c, 0x78, 0, 0
	.db     8, 0, 0, 0xc0, 0xf0, 0xf8, 0xfc, 0x7c, 0x3e, 0x1e, 0x1e, 0x1e
	.db     8, 0, 0, 0, 7, 7, 7, 7, 0, 0, 0, 0
	.db     8, 0x1e, 0x3e, 0x7c, 0xfc, 0xf8, 0xf8, 0xfc, 0x7e, 0x3e, 0x1f, 0x0f
	.db     8, 0, 0, 0xf0, 0xf8, 0x7c, 0x7f, 0x3f, 0x1f, 7, 0, 0
	.db     8, 0x0f, 0x0f, 0x0f, 0x1f, 0x3e, 0xfe, 0xfc, 0xf8, 0xe0, 0, 0
;
; 0xd0 - 0xd5 '4'
;
	.db     8, 0, 0, 0, 0, 0, 0, 1, 1, 3, 3, 7
	.db     8, 0, 0, 0x78, 0x78, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8
	.db     8, 7, 0x0f, 0x0f, 0x1e, 0x1e, 0x3c, 0x3c, 0x78, 0x78, 0xff, 0xff
	.db     8, 0xf8, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0xff, 0xff
	.db     8, 0xff, 0xff, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0xff, 0xff, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0x78, 0, 0
;
; 0xd6 - 0xdb '5'
;
	.db     8, 0, 0, 0x1f, 0x1f, 0x1f, 0x1f, 0x1e, 0x3e, 0x3e, 0x3c, 0x3c
	.db     8, 0, 0, 0xfe, 0xfe, 0xfe, 0xfe, 0, 0, 0, 0, 0
	.db     8, 0x3c, 0x7c, 0x7f, 0x7f, 0x7f, 0x7f, 0x78, 0, 0, 0, 0
	.db     8, 0, 0, 0xe0, 0xf8, 0xfc, 0xfe, 0x3e, 0x1f, 0x0f, 0x0f, 0x0f
	.db     8, 0, 0, 0xf0, 0xf8, 0x7c, 0x7f, 0x3f, 0x1f, 7, 0, 0
	.db     8, 0x0f, 0x0f, 0x0f, 0x1f, 0x3e, 0xfe, 0xfc, 0xf8, 0xe0, 0, 0
;
; 0xdc - 0xe1 '6'
;
	.db     8, 0, 0, 7, 0x1f, 0x3f, 0x3f, 0x7c, 0x78, 0x78, 0xf8, 0xf0
	.db     8, 0, 0, 0xe0, 0xf8, 0xfc, 0xfc, 0x3e, 0x1e, 0x1e, 0, 0
	.db     8, 0xf0, 0xf0, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xf8, 0xf0, 0xf0, 0xf0
	.db     8, 0, 0, 0xe0, 0xf8, 0xfc, 0xfe, 0x3e, 0x1f, 0x0f, 0x0f, 0x0f
	.db     8, 0xf0, 0xf0, 0xf0, 0xf8, 0x7c, 0x7f, 0x3f, 0x1f, 7, 0, 0
	.db     8, 0x0f, 0x0f, 0x0f, 0x1f, 0x3e, 0xfe, 0xfc, 0xf8, 0xe0, 0, 0
;
; 0xe2 - 0xe7 '7'
;
	.db     8, 0, 0, 0xff, 0xff, 0xff, 0xff, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x1f, 0x1e, 0x3e, 0x3c
	.db     8, 0, 0, 0, 0, 1, 1, 1, 3, 3, 3, 3
	.db     8, 0x7c, 0x78, 0xf8, 0xf0, 0xf0, 0xe0, 0xe0, 0xe0, 0xc0, 0xc0, 0xc0
	.db     8, 7, 7, 7, 7, 7, 7, 7, 7, 7, 0, 0
	.db     8, 0xc0, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0, 0
;
; 0xe8 - 0xed '8'
;
	.db     8, 0, 0, 3, 0x0f, 0x1f, 0x3f, 0x3e, 0x7c, 0x78, 0x78, 0x78
	.db     8, 0, 0, 0xc0, 0xf0, 0xf8, 0xfc, 0x7c, 0x3e, 0x1e, 0x1e, 0x1e
	.db     8, 0x78, 0x7c, 0x3e, 0x3f, 0x1f, 0x3f, 0x7f, 0x7c, 0xf8, 0xf0, 0xf0
	.db     8, 0x1e, 0x3e, 0x7c, 0xfc, 0xf8, 0xfc, 0xfe, 0x3e, 0x1f, 0x0f, 0x0f
	.db     8, 0xf0, 0xf0, 0xf0, 0xf8, 0x7c, 0x7f, 0x3f, 0x1f, 7, 0, 0
	.db     8, 0x0f, 0x0f, 0x0f, 0x1f, 0x3e, 0xfe, 0xfc, 0xf8, 0xe0, 0, 0
;
; 0xee - 0xf3 '9'
;
	.db     8, 0, 0, 7, 0x1f, 0x3f, 0x7f, 0x7c, 0xf8, 0xf0, 0xf0, 0xf0
	.db     8, 0, 0, 0xe0, 0xf8, 0xfc, 0xfe, 0x3e, 0x1f, 0x0f, 0x0f, 0x0f
	.db     8, 0xf0, 0xf0 ,0xf0, 0xf8, 0x7c, 0x7f, 0x3f, 0x1f, 7, 0, 0
	.db     8, 0x0f, 0x0f, 0x0f, 0x1f, 0x3f, 0xff, 0xff, 0xff, 0xef, 0x0f, 0x0f
	.db     8, 0, 0, 0x78, 0x78, 0x7c, 0x3f, 0x3f, 0x1f, 7, 0, 0
	.db     8, 0x0f, 0x1f, 0x1e, 0x1e, 0x3e, 0xfc, 0xfc, 0xf8, 0xe0, 0, 0
;
; 0xf4 - 0xf9 '-' (10)
;
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0x3f, 0x7f, 0x7f, 0x3f, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0xfc, 0xfe, 0xfe, 0xfc, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
;
; 0xfa - 0xff ' ' (11)
;
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
	.db     8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
;
; ASCII Mini-Zeichentabelle fr Grafik-Beschriftung (5x5 Pixel), jedes Zeichen wird mit 6 Byte definiert: Byte 0 enthlt die Zeichenbreite (0-8), die
; restlichen Bytes enthalten die Bitmap des Zeichens, Beispiel Zeichen 'A':
;
; Byte  0 = 3 (Zeichen ist 3 Pixel breit)
; Byte  1 = 2		. . . . . . o .
; Byte  2 = 5		. . . . . o . o
; Byte  3 = 5		. . . . . o . o
; Byte  4 = 7		. . . . . o o o
; Byte  5 = 5		. . . . . o . o
;
; ASCII 0x20 - 0x2f
;
chrsmi:	.db     3, 0, 0, 0, 0, 0			;0x20 ' '
	.db     1, 1, 1, 1, 0, 1			;0x21 '!'
	.db     3, 5, 0, 0, 0, 0			;0x22 '"'
	.db     5, 0x0a, 0x1f, 0x0a, 0x1f, 0x0a		;0x23 '#'
	.db     5, 0x1f, 0x14, 0x1f, 5, 0x1f		;0x24 '$'
	.db     3, 5, 1, 2, 4, 5			;0x25 '%'
	.db     4, 4, 0x0a, 4, 0x0a, 0x0d		;0x26 '&'
	.db     1, 1, 0, 0, 0, 0			;0x27 '''
	.db     2, 1, 2, 2, 2, 1			;0x28 '('
	.db     2, 2, 1, 1, 1, 2			;0x29 ')'
	.db     5, 0x0a, 4, 0x1f, 4, 0x0a		;0x2a '*'
	.db     3, 0, 2, 7, 2, 0			;0x2b '+'
	.db     2, 0, 0, 0, 1, 2			;0x2c ','
	.db     3, 0, 0, 7, 0, 0			;0x2d '-'
	.db     1, 0, 0, 0, 0, 1			;0x2e '.'
	.db     3, 1, 1, 2, 4, 4			;0x2f '/'
;
; ASCII 0x30 - 0x3f
;
	.db     3, 7, 5, 5, 5, 7			;0x30 '0'
	.db     3, 1, 3, 1, 1, 1			;0x31 '1'
	.db     3, 7, 1, 7, 4, 7			;0x32 '2'
	.db     3, 7, 1, 3, 1, 7			;0x33 '3'
	.db     3, 5, 5, 7, 1, 1			;0x34 '4'
	.db     3, 7, 4, 7, 1, 7			;0x35 '5'
	.db     3, 2, 4, 7, 5, 7			;0x36 '6'
	.db     3, 7, 1, 2, 2, 2			;0x37 '7'
	.db     3, 7, 5, 7, 5, 7			;0x38 '8'
	.db     3, 7, 5, 7, 1, 2			;0x39 '9'
	.db     1, 0, 1, 0, 1, 0			;0x3a ':'
	.db     2, 0, 1, 0, 1, 2			;0x3b ';'
	.db     3, 1, 2, 4, 2, 1			;0x3c '<'
	.db     3, 0, 7, 0, 7, 0			;0x3d '='
	.db     3, 4, 2, 1, 2, 4			;0x3e '>'
	.db     3, 7, 1, 2, 0, 2			;0x3f '?'
;
; ASCII 0x40 - 0x4f
;
	.db     5, 0x0e, 0x11, 0x17, 0x15, 0x17		;0x40 '@'
	.db     3, 2, 5, 5, 7, 5			;0x41 'A'
	.db     3, 6, 5, 6, 5, 6			;0x42 'B'
	.db     3, 3, 4, 4, 4, 3			;0x43 'C'
	.db     3, 6, 5, 5, 5, 6			;0x44 'D'
	.db     3, 7, 4, 6, 4, 7			;0x45 'E'
	.db     3, 7, 4, 6, 4, 4			;0x46 'F'
	.db     4, 6, 8, 0x0b, 9, 6			;0x47 'G'
	.db     3, 5, 5, 7, 5, 5			;0x48 'H'
	.db     1, 1, 1, 1, 1, 1			;0x49 'I'
	.db     3, 3, 1, 1, 5, 2			;0x4a 'J'
	.db     4, 9, 0x0a, 0x0c, 0x0a, 9		;0x4b 'K'
	.db     3, 4, 4, 4, 4, 7			;0x4c 'L'
	.db     5, 0x0a, 0x015, 0x15, 0x15, 0x15	;0x4d 'M'
	.db     4, 9, 0x0d, 0x0b, 9, 9			;0x4e 'N'
	.db     3, 7, 5, 5, 5, 7			;0x4f '0'
;
; ASCII 0x50 - 0x5f
;
	.db     3, 7, 5, 7, 4, 4			;0x50 'P'
	.db     4, 6, 9, 9, 0x0b, 7			;0x51 'Q'
	.db     3, 7, 5, 6, 5, 5			;0x52 'R'
	.db     3, 7, 4, 7, 1, 7			;0x53 'S'
	.db     3, 7, 2, 2, 2, 2			;0x54 'T'
	.db     3, 5, 5, 5, 5, 7			;0x55 'U'
	.db     3, 5, 5, 5, 5, 2			;0x56 'V'
	.db     5, 0x11, 0x11, 0x15, 0x15, 0x0a		;0x57 'W'
	.db     3, 5, 5, 2, 5, 5			;0x58 'X'
	.db     3, 5, 5, 7, 2, 2			;0x59 'Y'
	.db     3, 7, 1, 2, 4, 7			;0x5a 'Z'
	.db     2, 3, 2, 2, 2, 3			;0x5b '['
	.db     3, 4, 4, 2, 1, 1			;0x5c '\'
	.db     2, 3, 1, 1, 1, 3			;0x5d ']'
	.db     3, 2, 5, 0, 0, 0			;0x5e '^'
	.db     3, 0, 0, 0, 0, 7			;0x5f '_'
;
; ASCII 0x60 - 0x6f
;
	.db     1, 1, 0, 0, 0, 0			;0x60 '`'
	.db     3, 2, 5, 5, 7, 5			;0x61 'a'
	.db     3, 6, 5, 6, 5, 6			;0x62 'b'
	.db     3, 3, 4, 4, 4, 3			;0x63 'c'
	.db     3, 6, 5, 5, 5, 6			;0x64 'd'
	.db     3, 7, 4, 6, 4, 7			;0x65 'e'
	.db     3, 7, 4, 6, 4, 4			;0x66 'f'
	.db     4, 6, 8, 0x0b, 9, 6			;0x67 'g'
	.db     3, 5, 5, 7, 5, 5			;0x68 'h'
	.db     1, 1, 1, 1, 1, 1			;0x69 'i'
	.db     3, 3, 1, 1, 5, 2			;0x6a 'j'
	.db     4, 9, 0x0a, 0x0c, 0x0a, 9		;0x6b 'k'
	.db     3, 4, 4, 4, 4, 7			;0x6c 'l'
	.db     5, 0x0a, 0x015, 0x15, 0x15, 0x15	;0x6d 'm'
	.db     4, 9, 0x0d, 0x0b, 9, 9			;0x6e 'n'
	.db     3, 7, 5, 5, 5, 7			;0x6f 'o'
;
; ASCII 0x70 - 0x7f
;
	.db     3, 7, 5, 7, 4, 4			;0x70 'p'
	.db     4, 6, 9, 9, 0x0b, 7			;0x71 'q'
	.db     3, 7, 5, 6, 5, 5			;0x72 'r'
	.db     3, 7, 4, 7, 1, 7			;0x73 's'
	.db     3, 7, 2, 2, 2, 2			;0x74 't'
	.db     3, 5, 5, 5, 5, 7			;0x75 'u'
	.db     3, 5, 5, 5, 5, 2			;0x76 'v'
	.db     5, 0x11, 0x11, 0x15, 0x15, 0x0a		;0x77 'w'
	.db     3, 5, 5, 2, 5, 5			;0x78 'x'
	.db     3, 5, 5, 7, 2, 2			;0x79 'y'
	.db     3, 7, 1, 2, 4, 7			;0x7a 'z'
	.db     3, 3, 2, 4, 2, 3			;0x7b '{'
	.db     1, 1, 1, 1, 1, 1			;0x7c '|'
	.db     3, 6, 2, 1, 2, 6			;0x7d '}'
	.db     4, 5, 0x0a, 0, 0, 0			;0x7e '~'
	.db     0, 0, 0, 0, 0, 0			;0x7f Endezeichen
;
; ASCII 0x80 - 0x8f (reserviert fr Umlaute und Sonderzeichen)
;
	.db     3, 5, 2, 5, 7, 5			;0x80 ''
	.db     3, 5, 0, 7, 5, 7			;0x81 ''
	.db     3, 5, 0, 5, 5, 7			;0x82 ''
	.db     3, 5, 2, 5, 7, 5			;0x83 ''
	.db     3, 5, 0, 7, 5, 7			;0x84 ''
	.db     3, 5, 0, 5, 5, 7			;0x85 ''
	.db     4, 0x0e, 9, 0x0a, 9, 0x0b		;0x86 ''
	.db     0, 0, 0, 0, 0, 0			;0x87 frei
	.db     0, 0, 0, 0, 0, 0			;0x88 frei
	.db     0, 0, 0, 0, 0, 0			;0x89 frei
	.db     0, 0, 0, 0, 0, 0			;0x8a frei
	.db     0, 0, 0, 0, 0, 0			;0x8b frei
	.db     0, 0, 0, 0, 0, 0			;0x8c frei
	.db     0, 0, 0, 0, 0, 0			;0x8d frei
	.db     0, 0, 0, 0, 0, 0			;0x8e frei
	.db     0, 0, 0, 0, 0, 0			;0x8f frei
;
; ^  bis an diese Stelle knnen eigene Zeichen definiert werden
; v  ab hier stehen Sonderzeichen
;
; ASCII 0x90 - 0x9f (nur fr Sonderzeichen)
;
	.db     0, 0, 0, 0, 0, 0	                ;0x90 frei
	.db     0, 0, 0, 0, 0, 0	                ;0x91 frei
	.db     0, 0, 0, 0, 0, 0	                ;0x92 frei
	.db     3, 2, 7, 2, 7, 2                        ;0x93 Symbol fr 'Skala angleichen'
	.db     0, 0, 0, 0, 0, 0			;0x94 Punkt fr Groanzeige (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0			;0x95 Komma fr Groanzeige (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0	                ;0x96 Antennen-Symbol + Bluetooth  (hier nicht genutzt)
	.db     5, 4, 0x0a, 0x0a, 0x11, 0x1f            ;0x97 Delta-Symbol
	.db     5, 0x10, 9, 5, 3, 0x0f	                ;0x98 Minimum-Symbol
	.db     5, 0x0f, 3, 5, 9, 0x10                 	;0x99 Maximum-Symbol
	.db     6, 0x0d, 0x12, 0x15, 9, 0x16            ;0x9a Mittelwert
	.db     0, 0, 0, 0, 0, 0              	        ;0x9b Blitz-Symbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0                        ;0x9c Antennen-Symbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0                        ;0x9d Lautsprecher-Symbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0	                ;0x9e Antenne/Lautsprecher 'aus' (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0		        ;0x9f Antenne/Lautsprecher 'ein' (hier nicht genutzt)
;
; ASCII 0xa0 - 0xaf (nur fr Sonderzeichen)
;
	.db     2, 0, 0, 0, 0, 0		        ;0xa0 schmales Leerzeichen
	.db     0, 0, 0, 0, 0, 0	                ;0xa1 Alarmsymbol
	.db     0, 0, 0, 0, 0, 0		        ;0xa2 Temperatursymbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0		        ;0xa3 Fehlersymbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0	                ;0xa4 Nicht-Ok-Symbol (hier nicht genutzt)
	.db     1, 1, 0, 0, 0, 0			;0xa5 ''
	.db     5, 0x13, 4, 4, 4, 3                     ;0xa6 'C'
	.db     0, 0, 0, 0, 0, 0		        ;0xa7 ' ' breites Gradzeichen (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0		        ;0xa8 '^' Temperaturberschreitung (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0		        ;0xa9 'v' Temperaturunterschreitung (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0		        ;0xaa '!' Temperatursensorausfall (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0		        ;0xab schmales '' (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0	                ;0xac Cursorsymbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0			;0xad Schlosssymbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0	                ;0xae Ok-Symbol (hier nicht genutzt)
	.db     0, 0, 0, 0, 0, 0			;0xaf Lschsymbol (hier nicht genutzt)
;
;
; Umlaut-Wandlungs-Tabelle, wird zur Datenbertragung zum PC bentigt, alle Umlaute im Code-Bereich 0x80-0x8f werden bei der Datenausgabe zum PC durch zwei
; Zeichen ersetzt:
;
umltab:	.db	"Ae"	;Zeichencode 0x80,  wird zu Ae
	.db	"Oe"	;Zeichencode 0x81,  wird zu Oe
	.db	"Ue"	;Zeichencode 0x82,  wird zu Ue
	.db	"ae"	;Zeichencode 0x83,  wird zu ae
	.db	"oe"	;Zeichencode 0x84,  wird zu oe
	.db	"ue"	;Zeichencode 0x85,  wird zu ue
	.db	"ss"	;Zeichencode 0x86,  wird zu ss
	.db	"  "	;Zeichencode 0x87, frei
	.db	"  "	;Zeichencode 0x88, frei
	.db	"  "	;Zeichencode 0x89, frei
	.db	"  "	;Zeichencode 0x8a, frei
	.db	"  "	;Zeichencode 0x8b, frei
	.db	"  "	;Zeichencode 0x8c, frei
	.db	"  "	;Zeichencode 0x8d, frei
	.db	"  "	;Zeichencode 0x8e, frei
	.db	"  "	;Zeichencode 0x8f, frei
;
;
; Zeichenliste fr die Texteingabe (Sensor-, Alarm- und Grafik-Preset-Namen), die Anzahl und die Reihenfolge kann gendert werden, letztes Zeichen: 0xff
;
; "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+-/."
;
chrlst: .db     "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 128, 129, 130, "abcdefghijklmnopqrstuvwxyz", 131, 132, 133, 134, "+-/.", 0xff
;
;
; Sound-Tabelle, enthlt 16 vordefinierte Tonfolgen (Tonfolge 16 ist als Tasten-Quittungston reserviert); jede Folge besteht aus 8 Bytes, welche den zeitlichen
; Verlauf im 20ms-Raster von maximal 4 Tnen festlegen; die gesamte Dauer darf 1s (Maximalwert 49) nicht berschreiten; Byte 0 ist immer 0, Beispiel Tonfolge
; 13:
;
; Byte 0 =  0	Zeitpunkt  0x 20ms =   0ms, Beginn von Ton 1 (immer 0)
; Byte 1 =  4	Zeitpunkt  4x 20ms =  80ms, Ende von Ton 1
; Byte 2 =  8	Zeitpunkt  8x 20ms = 160ms, Beginn von Ton 2
; Byte 3 = 20	Zeitpunkt 20x 20ms = 400ms, Ende von Ton 2
; Byte 4 =  0	Beginn von Ton 3 \ hier nicht genutzt, in diesem
; Byte 5 =  0	Ende von Ton 3   / Fall beide Werte = 0 setzen
; Byte 6 =  0	Beginn von Ton 4 \ hier nicht genutzt, in diesem
; Byte 7 =  0	Ende von Ton 4   / Fall beide Werte = 0 setzen
;
sndtab:	.db	0, 4, 8, 12, 16, 20, 24, 28	; 1. 4x 80ms/80ms
	.db	0, 4, 8, 12, 16, 20, 0, 0	; 2. 3x 80ms/80ms
	.db	0, 4, 8, 12, 0, 0, 0, 0	        ; 3. 2x 80ms/80ms
	.db	0, 4, 0, 0, 0, 0, 0, 0		; 4. 1x 80ms
	.db	0, 2, 4, 6, 8, 10, 12, 14	; 5. 4x 40ms/40ms
	.db	0, 2, 4, 6, 8, 10, 0, 0	        ; 6. 3x 40ms/40ms
	.db	0, 2, 4, 6, 0, 0, 0, 0		; 7. 2x 40ms/40ms
	.db	0, 2, 0, 0, 0, 0, 0, 0		; 8. 1x 40ms
	.db	0, 2, 25, 27, 0, 0, 0, 0	; 9. 2x 40ms/460ms
	.db	0, 25, 0, 0, 0, 0, 0, 0	        ;10. 1x 500ms/500ms
	.db	0, 12, 25, 37, 0, 0, 0, 0	;11. 2x 240ms/260ms
	.db	0, 6, 12, 18, 25, 31, 37, 43	;12. 4x	120ms/120ms
	.db	0, 4, 8, 20, 0, 0, 0, 0	        ;13. 1x 80ms/80ms + 240ms
	.db	0, 12, 16, 20, 0, 0, 0, 0	;14. 1x 240ms/80ms + 80ms
	.db	0, 8, 17, 25, 34, 42, 0, 0	;15. 3x 160ms/180ms
	.db	0, 3, 0, 0, 0, 0, 0, 0		;16. 1x 60ms (reserviert)
;
;
; Kommando-Liste fr die PC-Datenkommunikation, jedes Kommando darf maximal 8 Zeichen lang sein (der Rest wird mit Leerzeichen aufgefllt), das Listenende wird
; mit 0xff gekennzeichnet
;
pccmdl:	.db	"end     "	;Kommando 1 (Ende/Abbruch)
	.db	"info    "	;Kommando 2 (Infozeile senden)
	.db	"gett15  "	;Kommando 3 (15min-Werte senden)
	.db	"getmin  "	;Kommando 4 (Minimal-Werte senden)
	.db	"getmax  "	;Kommando 5 (Maximal-Werte senden)
	.db	"getavg  "	;Kommando 6 (Mittel-Werte senden)
	.db	"getmsg  "	;Kommando 7 (Meldungsliste senden)
	.db	"getcfg  "	;Kommando 8 (Konfiguration senden)
        .db     "getscr  "      ;Kommando 9 (Bildschirminhalt senden)
	.db	0xff,0xff	;Listenende
;
;
; Various strings - English
;
.include "Anzeigemodul2a-eng.asm"
;
;
; Diverse Texte - Deutsch
;
.include "Anzeigemodul2a-deu.asm"
;
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Mentabellen, enthalten die komplette Menstruktur in einer Index-Tabelle und weiteren Tabellen fr jedes Untermen
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Die Index-Tabelle enthlt die Speicheradressen der einzelnen Mentabellen
;
menidx:	.dw	menu00		;Men 00, Hauptmen
	.dw	menu01		;Men 01, Minimal-Werte
	.dw	menu02		;Men 02, Maximal-Werte
	.dw	menu03		;Men 03, Meldungen
	.dw	menu04		;Men 04, Status
	.dw	menu05		;Men 05, Konfiguration
	.dw	menu06		;Men 06, Konfiguration - Sensoren
	.dw	menu07		;Men 07, Konfiguration - Sensoren - Einstellungen 1
	.dw	menu08		;Men 08, Konfiguration - Sensoren - Grenzwerte 1
	.dw	menu09		;Men 09, Konfiguration - Sensoren - Untere Grenzwerte 1
	.dw	menu10		;Men 10, Konfiguration - Sensoren - Obere Grenzwerte 1
	.dw	menu11		;Men 11, Konfiguration - Alarme
	.dw	menu12		;Men 12, Konfiguration - Signale
	.dw	menu13		;Men 13, Konfiguration - Signale - Zuordnung
	.dw	menu14		;Men 14, Konfiguration - Signale - Audio
	.dw	menu15		;Men 15, Konfiguration - Zeiten
	.dw	menu16		;Men 16, Konfiguration - Zeiten - Schaltzeiten
	.dw	menu17		;Men 17, Konfiguration - Zeiten - Uhr stellen
	.dw	menu18		;Men 18, Konfiguration - Sonstiges 1
	.dw	menu19		;Men 19, Konfiguration - Backup
	.dw	menu20		;Men 20, Konfiguration - Grundeinstellung
	.dw	menu21		;Men 21, Konfiguration - Grundeinstellung - Basiskonfiguration laden
	.dw	menu22		;Men 22, Konfiguration - Grundeinstellung - 15min Werte lschen
	.dw	menu23		;Men 23, Konfiguration - Grundeinstellung - 24h Werte lschen
	.dw	menu24		;Men 24, Konfiguration - Grundeinstellung - Meldungen lschen
	.dw	menu25		;Men 25, Konfiguration - Sensoren - Anzeige
	.dw	menu26		;Men 26, Grafik
	.dw	menu27		;Men 27, Grafik - Preset-01
	.dw	menu28		;Men 28, Grafik - Preset-02
	.dw	menu29		;Men 29, Grafik - Preset-03
	.dw	menu30		;Men 30, Grafik - Preset-04
	.dw	menu31		;Men 31, Grafik - Preset-05
	.dw	menu32		;Men 32, Grafik - Preset-06
	.dw	menu33		;Men 33, Grafik - Preset-07
	.dw	menu34		;Men 34, Grafik - Preset-08
	.dw	menu35		;Men 35, Grafik - Preset-09
	.dw	menu36		;Men 36, Grafik - Preset-10
	.dw	menu37		;Men 37, Grafik - Preset-11
	.dw	menu38		;Men 38, Grafik - Preset-12
	.dw	menu39		;Men 39, Grafik - Preset-13
	.dw	menu40		;Men 40, Grafik - Preset-14
	.dw	menu41		;Men 41, Grafik - Preset-15
	.dw	menu42		;Men 42, Grafik - Preset-16
	.dw	menu43		;Men 43, Grafik - Preset-17
	.dw	menu44		;Men 44, Grafik - Preset-18
	.dw	menu45		;Men 45, Grafik - Preset-19
	.dw	menu46		;Men 46, Grafik - Preset-20
	.dw	menu47		;Men 47, Grafik - Preset-21
	.dw	menu48		;Men 48, Grafik - Preset-22
	.dw	menu49		;Men 49, Grafik - Preset-23
	.dw	menu50		;Men 50, Grafik - Preset-24
	.dw	menu51		;Men 51, Grafik - Preset-01 - Fadenkreuz
	.dw	menu52		;Men 52, Grafik - Preset-02 - Fadenkreuz
	.dw	menu53		;Men 53, Grafik - Preset-03 - Fadenkreuz
	.dw	menu54		;Men 54, Grafik - Preset-04 - Fadenkreuz
	.dw	menu55		;Men 55, Grafik - Preset-05 - Fadenkreuz
	.dw	menu56		;Men 56, Grafik - Preset-06 - Fadenkreuz
	.dw	menu57		;Men 57, Grafik - Preset-07 - Fadenkreuz
	.dw	menu58		;Men 58, Grafik - Preset-08 - Fadenkreuz
	.dw	menu59		;Men 59, Grafik - Preset-09 - Fadenkreuz
	.dw	menu60		;Men 60, Grafik - Preset-10 - Fadenkreuz
	.dw	menu61		;Men 61, Grafik - Preset-11 - Fadenkreuz
	.dw	menu62		;Men 62, Grafik - Preset-12 - Fadenkreuz
	.dw	menu63		;Men 63, Grafik - Preset-13 - Fadenkreuz
	.dw	menu64		;Men 64, Grafik - Preset-14 - Fadenkreuz
	.dw	menu65		;Men 65, Grafik - Preset-15 - Fadenkreuz
	.dw	menu66		;Men 66, Grafik - Preset-16 - Fadenkreuz
	.dw	menu67		;Men 67, Grafik - Preset-17 - Fadenkreuz
	.dw	menu68		;Men 68, Grafik - Preset-18 - Fadenkreuz
	.dw	menu69		;Men 69, Grafik - Preset-19 - Fadenkreuz
	.dw	menu70		;Men 70, Grafik - Preset-20 - Fadenkreuz
	.dw	menu71		;Men 71, Grafik - Preset-21 - Fadenkreuz
	.dw	menu72		;Men 72, Grafik - Preset-22 - Fadenkreuz
	.dw	menu73		;Men 73, Grafik - Preset-23 - Fadenkreuz
	.dw	menu74		;Men 74, Grafik - Preset-24 - Fadenkreuz
	.dw	menu75		;Men 75, Konfiguration - Grafik-Presets
	.dw	menu76		;Men 76, Konfiguration - Grundeinstellung - Grafik-Presets zurcksetzen
        .dw     menu77		;Men 77, Konfiguration - Sensoren - Einstellungen 2
	.dw	menu78		;Men 78, Konfiguration - Sensoren - Grenzwerte 2
	.dw	menu79		;Men 79, Konfiguration - Sensoren - Untere Grenzwerte 2
	.dw	menu80		;Men 80, Konfiguration - Sensoren - Obere Grenzwerte 2
        .dw     menu81          ;Men 81, Konfiguration - Sensoren - Groanzeige
        .dw     menu82          ;Men 82, Konfiguration - Sensoren - Log-Zordnung 1
        .dw     menu83          ;Men 83, Konfiguration - Sensoren - Log-Zordnung 2
	.dw	menu84		;Men 84, Konfiguration - Backup - Konfiguration Sichern
	.dw	menu85		;Men 85, Konfiguration - Backup - Konfiguration laden/Neustart
	.dw	menu86		;Men 86, Konfiguration - Sonstiges 2
;
; Eine Mentabelle besteht aus 4 Startbytes und jeweils 4 Bytes fr ein Untermen:
;
; Byte  0 = Anzahl der Untermens
; Byte  1 = Nummer der Zeichenkette fr die Infoanzeige
; Byte  2 = Nummer des bergeordneten Mens
; Byte  3 = Nummer der bergeordneten Menoption, Bit7: 1 = Flag fr Neuausgabe der Temperaturanzeige
;
; Byte  4 = Nummer der Zeichenkette fr die Menanzeige \
; Byte  5 = Nummer des Untermens oder 0xff              \ Daten fr
; Byte  6 = \ Startadresse fr das Menprogramm	         / Menoption 1
; Byte  7 = / oder 0 (wenn keine Aktion)                /
;
; Byte  8 = Nummer der Zeichenkette fr die Menanzeige \
; Byte  9 = Nummer des Untermens oder 0xff              \ Daten fr
; Byte 10 = \ Startadresse fr das Menprogramm	         / Menoption 2
; Byte 11 = / oder 0 (wenn keine Aktion)                /
;
; Byte 12 = gegebenenfalls weitere Optionen...
;
; Beim ersten Byte einer Menoption (Byte 4, 8, 12, ...) gibt es 2 Sonderflle:
; Sonderfall 1: Byte 4 = 10 - Ausgabe des Grafik-Presets in der Men-Zeile
; Sonderfall 2: Byte 4 = 11 - Ausgabe der Fadenkreuzwerte anstatt der Men-Zeile
;
;
; Men 00 - Hauptmen
;
menu00:	.db	7, 0		;7 Untermens, keine Infoanzeige
	.db	0, 0		;kein bergeordnetes Men
        .db     0x1c, 0xff      ;Option 1: "Seite wechseln", kein Untermen
        .dw     m8d000          ;          Programmadresse
	.db	0x20, 1		;Option 2: "Min-Werte", Men 1
	.dw	m80000		;	   Programmadresse
	.db	0x21, 2		;Option 3: "Max-Werte", Men 2
	.dw	m80200		;	   Programmadresse
	.db	0x23, 26	;Option 4: "Grafik", Men 26
	.dw	0		;	   keine Aktion durchfhren
	.db	0x22, 3		;Option 5: "Meldungen", Men 3
	.dw	m81000		;	   Programmadresse
	.db	0x24, 4		;Option 6: "Status", Men 4
	.dw	m82000		;	   Programmadresse
	.db	0x25, 5		;Option 7: "Konfiguration", Men 5
	.dw	0		;	   keine Aktion durchfhren
;
; Men 01 - Minimal-Werte
;
menu01:	.db	1, 0x20		;1 Untermen, Infotext "Minimal-Werte"
	.db	0, 2 | 0x80	;bergeordnetes Men 0, Option 2
	.db	0x26, 0xff	;Option 1: "Werte lschen", kein Untermen
	.dw	m80100		;	   Programmadresse
;
; Men 02 - Maximal-Werte
;
menu02:	.db	1, 0x21		;1 Untermen, Infotext "Maximal-Werte"
	.db	0, 3 | 0x80	;bergeordnetes Men 0, Option 3
	.db	0x26, 0xff	;Option 1: "Werte lschen", kein Untermen
	.dw	m80300		;	   Programmadresse
;
; Men 03 - Meldungen
;
menu03:	.db	3, 0x22		;3 Untermens, Infotext "Meldungen"
	.db	0, 5 | 0x80	;bergeordnetes Men 0, Option 5
	.db	0x28, 0xff	;Option 1: "vorherige Seite"
	.dw	m81100		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: "nchste Seite"
	.dw	m81200		;	   Programmadresse
	.db	0x29, 0xff	;Option 3: "Filter"
	.dw	m81300		;	   Programmadresse
;
; Men 04 - Status
;
menu04:	.db	5, 0x24		;5 Untermens, Infotext "Status"
	.db	0, 6 | 0x80	;bergeordnetes Men 0, Option 6
	.db	0x3b, 0xff	;Option 1: "Normal/Abwesend"
	.dw	m82400		;	   Programmadresse
	.db	0x3c, 0xff	;Option 2: "LED umschalten"
	.dw	m82500		;	   Programmadresse
	.db	0x3d, 0xff	;Option 3: "Speicherfehler lschen"
	.dw	m82600		;	   Programmadresse
	.db	0x3e, 0xff	;Option 4: "RTC-Fehler lschen"
	.dw	m82700		;	   Programmadresse
	.db	0x3f, 0xff	;Option 5: "Pointer-Fehler lschen"
	.dw	m82800		;	   Programmadresse
;
; Men 05 - Konfiguration
;
menu05:	.db	9, 0x25		;9 Untermens, Infotext "Konfiguration"
	.db	0, 7		;bergeordnetes Men 0, Option 7
	.db	0x50, 6		;Option 1: "Sensoren", Men 6
	.dw	0		;	   keine Aktion durchfhren
	.db	0x51, 11	;Option 2: "Alarme", Men 11
	.dw	m85000		;	   Programmadresse
	.db	0x52, 12	;Option 3: "Signale", Men 12
	.dw	0		;	   keine Aktion durchfhren
	.db	0x53, 15	;Option 4: "Zeiten", Men 15
	.dw	0		;	   Programmadresse
	.db	0x54, 18	;Option 5: "Sonstiges 1", Men 18
	.dw	m88500		;	   keine Aktion durchfhren
	.db	0x4f, 86	;Option 6: "Sonstiges 2", Men 86
	.dw	m8e000		;	   keine Aktion durchfhren
	.db	0xaf, 75	;Option 7: "Grafik-Presets", Men 75
	.dw	m8b000		;	   Programmadresse
	.db	0x55, 19	;Option 8: "Backup", Men 19
	.dw	0		;	   keine Aktion durchfhren
	.db	0x56, 20	;Option 9: "Grundeinstellung", Men 20
	.dw	0		;	   keine Aktion durchfhren
;
; Men 06 - Konfiguration - Sensoren
;
menu06:	.db	8, 0x50		;8 Untermens, Infotext "Sensoren"
	.db	5, 1		;bergeordnetes Men 5, Option 1
	.db	0x44, 7		;Option 1: "Einstellungen 1", Men 7
	.dw	m83000		;	   Programmadresse
	.db	0x45, 77	;Option 2: "Einstellungen 2", Men 77
	.dw	m83050		;	   Programmadresse
	.db	0x5e, 8		;Option 3: "Grenzwerte 1"
	.dw	m84000		;	   Programmadresse
	.db	0x5f, 78	;Option 4: "Grenzwerte 2"
	.dw	m84050		;	   Programmadresse
        .db     0x57, 25        ;Option 5: "Anzeige"
	.dw	m84600		;	   Programmadresse
        .db     0x47, 81        ;Option 6: "Groanzeige"
        .dw     m84900          ;	   Programmadresse
        .db     0x97, 82        ;Option 7: "Log-Zuordnung 1"
        .dw     m84c00          ;	   Programmadresse
        .db     0x98, 83        ;Option 8: "Log-Zuordnung 2"
        .dw     m84f00          ;	   Programmadresse
;
; Men 07 - Konfiguration - Sensoren - Einstellungen 1
;
menu07:	.db	3, 0x44		;3 Untermens, Infotext "Einstellungen 1"
	.db	6, 1 | 0x80	;bergeordnetes Men 6, Option 1
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m83100		;	   Programmadresse
	.db	0x5b, 0xff	;Option 2: "Ein/Ausschalten"
	.dw	m83200		;	   Programmadresse
	.db	0x5c, 0xff	;Option 3: "Name"
	.dw	m83300		;	   Programmadresse
;
; Men 77 - Konfiguration - Sensoren - Einstellungen 2
;
menu77:	.db	3, 0x45		;3 Untermens, Infotext "Einstellungen 2"
	.db	6, 2 | 0x80	;bergeordnetes Men 6, Option 2
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m83100		;	   Programmadresse
	.db	0x5b, 0xff	;Option 2: "Ein/Ausschalten"
	.dw	m83200		;	   Programmadresse
	.db	0x5c, 0xff	;Option 3: "Name"
	.dw	m83300		;	   Programmadresse
;
; Men 08 - Konfiguration - Sensoren - Grenzwerte 1
;
menu08:	.db	3, 0x5e		;3 Untermens, Infotext "Grenzwerte 1"
	.db	6, 3 | 0x80	;bergeordnetes Men 6, Option 3
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m84100		;	   Programmadresse
	.db	0x58, 9		;Option 2: "Unterer Grenzwert", Men 9
	.dw	0		;	   keine Aktion durchfhren
	.db	0x59, 10	;Option 3: "Oberer Grenzwert", Men 10
	.dw	0		;	   keine Aktion durchfhren
;
; Men 09 - Konfiguration - Sensoren - Grenzwerte - Unterer Grenzwert 1
;
menu09:	.db	2, 0x58		;2 Untermens, Infotext "Unterer Grenzwert"
	.db	8, 2		;bergeordnetes Men 8, Option 2
	.db	0x60, 0xff	;Option 1: "ndern"
	.dw	m84200		;	   Programmadresse
	.db	0x61, 0xff	;Option 2: "Lschen"
	.dw	m84300		;	   Programmadresse
;
; Men 10 - Konfiguration - Sensoren - Grenzwerte - Oberer Grenzwert 1
;
menu10:	.db	2, 0x59		;2 Untermens, Infotext "Oberer Grenzwert"
	.db	8, 3		;bergeordnetes Men 8, Option 3
	.db	0x60, 0xff	;Option 1: "ndern"
	.dw	m84400		;	   Programmadresse
	.db	0x61, 0xff	;Option 2: "Lschen"
	.dw	m84500		;	   Programmadresse
;
; Men 78 - Konfiguration - Sensoren - Grenzwerte 2
;
menu78:	.db	3, 0x5f		;3 Untermens, Infotext "Grenzwerte 2"
	.db	6, 4 | 0x80	;bergeordnetes Men 6, Option 4
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m84100		;	   Programmadresse
	.db	0x58, 79	;Option 2: "Unterer Grenzwert", Men 79
	.dw	0		;	   keine Aktion durchfhren
	.db	0x59, 80	;Option 3: "Oberer Grenzwert", Men 80
	.dw	0		;	   keine Aktion durchfhren
;
; Men 79 - Konfiguration - Sensoren - Grenzwerte - Unterer Grenzwert 2
;
menu79:	.db	2, 0x58		;2 Untermens, Infotext "Unterer Grenzwert"
	.db	78, 2		;bergeordnetes Men 78, Option 2
	.db	0x60, 0xff	;Option 1: "ndern"
	.dw	m84200		;	   Programmadresse
	.db	0x61, 0xff	;Option 2: "Lschen"
	.dw	m84300		;	   Programmadresse
;
; Men 80 - Konfiguration - Sensoren - Grenzwerte - Oberer Grenzwert 2
;
menu80:	.db	2, 0x59		;2 Untermens, Infotext "Oberer Grenzwert"
	.db	78, 3		;bergeordnetes Men 78, Option 3
	.db	0x60, 0xff	;Option 1: "ndern"
	.dw	m84400		;	   Programmadresse
	.db	0x61, 0xff	;Option 2: "Lschen"
	.dw	m84500		;	   Programmadresse
;
; Men 25 - Konfiguration - Sensoren - Anzeige
;
menu25:	.db	2, 0x57		;2 Untermens, Infotext "Anzeige"
	.db	6, 5 | 0x80	;bergeordnetes Men 6, Option 5
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m84700		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m84800		;	   Programmadresse
;
; Men 81 - Konfiguration - Sensoren - Groanzeige
;
menu81:	.db	2, 0x47		;2 Untermens, Infotext "Groanzeige"
	.db	6, 6 | 0x80	;bergeordnetes Men 6, Option 6
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m84a00		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m84b00		;	   Programmadresse
;
; Men 82 - Konfiguration - Sensoren - Log-Zuordnung 1
;
menu82:	.db	2, 0x97		;2 Untermens, Infotext "Log-Zuordnung 1"
	.db	6, 7 | 0x80	;bergeordnetes Men 6, Option 7
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m84d00		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m84e00		;	   Programmadresse
;
; Men 83 - Konfiguration - Sensoren - Log-Zuordnung 2
;
menu83:	.db	2, 0x98		;2 Untermens, Infotext "Log-Zuordnung 2"
	.db	6, 8 | 0x80	;bergeordnetes Men 6, Option 8
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m84g00		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m84h00		;	   Programmadresse
;
; Men 11 - Konfiguration - Alarme
;
menu11:	.db	5, 0x51		;5 Untermens, Infotext "Alarme"
	.db	5, 2 | 0x80	;bergeordnetes Men 5, Option 2
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m85100		;	   Programmadresse
	.db	0x5b, 0xff	;Option 2: "Ein/Ausschalten"
	.dw	m85200		;	   Programmadresse
	.db	0x5c, 0xff	;Option 3: "Namen"
	.dw	m85300		;	   Programmadresse
	.db	0x64, 0xff	;Option 4: "Invertieren"
	.dw	m85400		;	   Programmadresse
	.db	0x5d, 0xff	;Option 5: "Verschieben"
	.dw	m85500		;	   Programmadresse
;
; Men 12 - Konfiguration - Signale
;
menu12:	.db	2, 0x52		;2 Untermens, Infotext "Signale"
	.db	5, 3		;bergeordnetes Men 5, Option 3
	.db	0x6a, 13	;Option 1: "Zuordnung", Men 13
	.dw	m86000		;	   Programmadresse
	.db	0x6b, 14	;Option 2: "Audio", Men 14
	.dw	m87000		;	   Programmadresse
;
; Men 13 - Konfiguration - Signale - Zuordnung
;
menu13:	.db	4, 0x6a		;4 Untermens, Infotext "Zuordnung"
	.db	12, 1 | 0x80	;bergeordnetes Men 12, Option 1
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m86100		;	   Programmadresse
	.db	0x6c, 0xff	;Option 2: "Nummer"
	.dw	m86200		;	   Programmadresse
	.db	0x6d, 0xff	;Option 3: "Dauer"
	.dw	m86300		;	   Programmadresse
	.db	0x6e, 0xff	;Option 3: "Abspielen"
	.dw	m86400		;	   Programmadresse
;
; Men 14 - Konfiguration - Signale - Audio
;
menu14:	.db	3, 0x6b		;3 Untermens, Infotext "Audio"
	.db	12, 2 | 0x80	;bergeordnetes Men 12, Option 2
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m87100		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m87200		;	   Programmadresse
	.db	0x6e, 0xff	;Option 3: "Abspielen"
	.dw	m87300		;	   Programmadresse
;
; Men 15 - Konfiguration - Zeiten
;
menu15:	.db	2, 0x53		;2 Untermens, Infotext "Zeiten"
	.db	5, 4		;bergeordnetes Men 5, Option 4
	.db	0x72, 16	;Option 1: "Schaltzeiten", Men 16
	.dw	m87500		;	   Programmadresse
	.db	0x73, 17	;Option 1: "Uhr stellen", Men 17
	.dw	m88000		;	   Programmadresse
;
; Men 16 - Konfiguration - Zeiten - Schaltzeiten
;
menu16:	.db	3, 0x72		;3 Untermens, Infotext "Schaltzeiten"
	.db	15, 1 | 0x80	;bergeordnetes Men 15, Option 1
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m87600		;	   Programmadresse
	.db	0x5b, 0xff	;Option 2: "Ein/Ausschalten"
	.dw	m87700		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m87800		;	   Programmadresse
;
; Men 17 - Konfiguration - Zeiten - Uhr stellen
;
menu17:	.db	2,0x73		;2 Untermens, Infotext "Uhr stellen"
	.db	15,2 | 0x80	;bergeordnetes Men 15, Option 2
	.db	0x5a,0xff	;Option 1: "Auswhlen"
	.dw	m88100		;	   Programmadresse
	.db	0x60,0xff	;Option 2: "ndern"
	.dw	m88200		;	   Programmadresse
;
; Men 18 - Konfiguration - Sonstiges 1
;
menu18:	.db	2, 0x54		;2 Untermens, Infotext "Sonstiges 1"
	.db	5, 5 | 0x80	;bergeordnetes Men 5, Option 5
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m88600		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m88700		;	   Programmadresse
;
; Men 86 - Konfiguration - Sonstiges 2
;
menu86:	.db	2, 0x4f		;2 Untermens, Infotext "Sonstiges 2"
	.db	5, 6 | 0x80	;bergeordnetes Men 5, Option 6
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m8e100		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m8e200		;	   Programmadresse
;
; Men 75 - Konfiguration - Grafik-Presets
;
menu75:	.db	2, 0xaf		;2 Untermens,Infotext "Grafik-Presets"
	.db	5, 7 | 0x80	;bergeordnetes Men 5, Option 7
	.db	0x5a, 0xff	;Option 1: "Auswhlen"
	.dw	m8b100		;	   Programmadresse
	.db	0x60, 0xff	;Option 2: "ndern"
	.dw	m8b200		;	   Programmadresse
;
; Men 19 - Konfiguration - Backup
;
menu19:	.db	2, 0x55		;2 Untermens, Infotext "Backup"
	.db	5, 8		;bergeordnetes Men 5, Option 8
	.db	0x90, 84	;Option 1: "Konfiguration sichern", Men 84
	.dw	0		;	   keine Aktion durchfhren
	.db	0x91, 85	;Option 2: "Konfiguration laden/Neustart", Men 85
	.dw	0		;	   keine Aktion durchfhren
;
; Men 84 - Konfiguration - Backup - Sichern
;
menu84: .db     1, 0x90         ;1 Untermen, Infotext "Konfiguration sichern"
        .db     19, 1           ;bergeordnetes Men 19, Option 1
        .db     0x96, 0xff      ;Option 1: "Sicher?"
	.dw	m89000		;	   Programmadresse
;
; Men 85 - Konfiguration - Backup - Laden/Neustart
;
menu85: .db     1, 0x91         ;1 Untermen, Infotext "Konfiguration laden/Neustart"
        .db     19, 2           ;bergeordnetes Men 19, Option 2
        .db     0x96, 0xff      ;Option 1: "Sicher?"
	.dw	m89100		;	   Programmadresse
;
; Men 20 - Konfiguration - Grundeinstellung
;
menu20:	.db	5, 0x56		;4 Untermens, Infotext "Grundeinstellung"
	.db	5, 9		;bergeordnetes Men 5, Option 9
	.db	0x92, 21	;Option 1: "Basiskonfiguration laden", Men 21
	.dw	0		;	   keine Aktion durchfhren
	.db	0x9a, 76	;Option 2: "Grafik-Presets rcksetzen", Men 76
	.dw	0		;	   keine Aktion durchfhren
	.db	0x93, 22	;Option 3: "15min Werte lschen", Men 22
	.dw	0		;	   keine Aktion durchfhren
	.db	0x94, 23	;Option 4: "24h Werte lschen", Men 23
	.dw	0		;	   keine Aktion durchfhren
	.db	0x95, 24	;Option 5: "Meldungen lschen", Men 24
	.dw	0		;	   keine Aktion durchfhren
;
; Men 21 - Konfiguration - Grundeinstellung - Basiskonfiguration laden
;
menu21:	.db	1, 0x92		;1 Untermen, Infotext "Basiskonfiguration"
	.db	20, 1		;bergeordnetes Men 20, Option 1
	.db	0x96, 0xff	;Option 1: "Sicher?"
	.dw	m89500		;	   Programmadresse
;
; Men 76 - Konfiguration - Grundeinstellung - Grafik-Presets zurcksetzen
;
menu76:	.db	1, 0x9a		;1 Untermen, Infotext "Grafik-Presets zurcksetzen"
	.db	20, 2		;bergeordnetes Men 20, Option 2
	.db	0x96, 0xff	;Option 1: "Sicher?"
	.dw	m8c000		;	   Programmadresse
;
; Men 22 - Konfiguration - Grundeinstellung - 15min Werte lschen
;
menu22:	.db	1, 0x93		;1 Untermen, Infotext "15min Werte lschen"
	.db	20, 3		;bergeordnetes Men 20, Option 3
	.db	0x96, 0xff	;Option 1: "Sicher?"
	.dw	m89600		;	   Programmadresse
;
; Men 23 - Konfiguration - Grundeinstellung - 24h Werte lschen
;
menu23:	.db	1, 0x94		;1 Untermen, Infotext "24h Werte lschen"
	.db	20, 4		;bergeordnetes Men 20, Option 4
	.db	0x96, 0xff	;Option 1: "Sicher?"
	.dw	m89700		;	   Programmadresse
;
; Men 24 - Konfiguration - Grundeinstellung - Meldungen lschen
;
menu24:	.db	1, 0x95		;1 Untermen, Infotext "Meldungen lschen"
	.db	20, 5		;bergeordnetes Men 20, Option 5
	.db	0x96, 0xff	;Option 1: "Sicher?"
	.dw	m89800		;	   Programmadresse
;
; Men 26 - Grafik
;
menu26:	.db	24, 0x23	;24 Untermens, Infotext "Grafik"
	.db	0, 4		;bergeordnetes Men 0, Option 4
	.db	10, 27		;Option 01: "Preset 01", Men 27
	.dw	m8a000		;	    Programmadresse
	.db	10, 28		;Option 02: "Preset 02", Men 28
	.dw	m8a010		;	    Programmadresse
	.db	10, 29		;Option 03: "Preset 03", Men 29
	.dw	m8a020		;	    Programmadresse
	.db	10, 30		;Option 04: "Preset 04", Men 30
	.dw	m8a030		;	    Programmadresse
	.db	10, 31		;Option 05: "Preset 05", Men 31
	.dw	m8a040		;	    Programmadresse
	.db	10, 32		;Option 06: "Preset 06", Men 32
	.dw	m8a050		;	    Programmadresse
	.db	10, 33		;Option 07: "Preset 07", Men 33
	.dw	m8a060		;	    Programmadresse
	.db	10, 34		;Option 08: "Preset 08", Men 34
	.dw	m8a070		;	    Programmadresse
	.db	10, 35		;Option 09: "Preset 09", Men 35
	.dw	m8a080		;	    Programmadresse
	.db	10, 36		;Option 10: "Preset 10", Men 36
	.dw	m8a090		;	    Programmadresse
	.db	10, 37		;Option 11: "Preset 11", Men 37
	.dw	m8a100		;	    Programmadresse
	.db	10, 38		;Option 12: "Preset 12", Men 38
	.dw	m8a110		;	    Programmadresse
	.db	10, 39		;Option 13: "Preset 13", Men 39
	.dw	m8a120		;	    Programmadresse
	.db	10, 40		;Option 14: "Preset 14", Men 40
	.dw	m8a130		;	    Programmadresse
	.db	10, 41		;Option 15: "Preset 15", Men 41
	.dw	m8a140		;	    Programmadresse
	.db	10, 42		;Option 16: "Preset 16", Men 42
	.dw	m8a150		;	    Programmadresse
	.db	10, 43		;Option 17: "Preset 17", Men 43
	.dw	m8a160		;	    Programmadresse
	.db	10, 44		;Option 18: "Preset 18", Men 44
	.dw	m8a170		;	    Programmadresse
	.db	10, 45		;Option 19: "Preset 19", Men 45
	.dw	m8a180		;	    Programmadresse
	.db	10, 46		;Option 20: "Preset 20", Men 46
	.dw	m8a190		;	    Programmadresse
	.db	10, 47		;Option 21: "Preset 21", Men 47
	.dw	m8a200		;	    Programmadresse
	.db	10, 48		;Option 22: "Preset 22", Men 48
	.dw	m8a210		;	    Programmadresse
	.db	10, 49		;Option 23: "Preset 23", Men 49
	.dw	m8a220		;	    Programmadresse
	.db	10, 50		;Option 24: "Preset 24", Men 50
	.dw	m8a230		;	    Programmadresse
;
; Men 27 - Grafik - Preset-01
;
menu27:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 1 | 0x80	;bergeordnetes Men 26, Option 1
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 51	;Option 3: "Fadenkreuz", Men 51
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 28 - Grafik - Preset-02
;
menu28:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 2 | 0x80	;bergeordnetes Men 26, Option 2
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 52	;Option 3: "Fadenkreuz", Men 52
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 29 - Grafik - Preset-03
;
menu29:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 3 | 0x80	;bergeordnetes Men 26, Option 3
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 53	;Option 3: "Fadenkreuz", Men 53
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 30 - Grafik - Preset-04
;
menu30:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 4 | 0x80	;bergeordnetes Men 26, Option 4
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 54	;Option 3: "Fadenkreuz", Men 54
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 31 - Grafik - Preset-05
;
menu31:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 5 | 0x80	;bergeordnetes Men 26, Option 5
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 55	;Option 3: "Fadenkreuz", Men 55
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 32 - Grafik - Preset-06
;
menu32:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 6 | 0x80	;bergeordnetes Men 26, Option 6
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 56	;Option 3: "Fadenkreuz", Men 56
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 33 - Grafik - Preset-07
;
menu33:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 7 | 0x80	;bergeordnetes Men 26, Option 7
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 57	;Option 3: "Fadenkreuz", Men 57
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 34 - Grafik - Preset-08
;
menu34:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 8 | 0x80	;bergeordnetes Men 26, Option 8
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 58	;Option 3: "Fadenkreuz", Men 58
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 35 - Grafik - Preset-09
;
menu35:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 9 | 0x80	;bergeordnetes Men 26, Option 9
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 59	;Option 3: "Fadenkreuz", Men 59
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 36 - Grafik - Preset-10
;
menu36:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 10 | 0x80	;bergeordnetes Men 26, Option 10
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 60	;Option 3: "Fadenkreuz", Men 60
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 37 - Grafik - Preset-11
;
menu37:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 11 | 0x80	;bergeordnetes Men 26, Option 11
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 61	;Option 3: "Fadenkreuz", Men 61
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 38 - Grafik - Preset-12
;
menu38:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 12 | 0x80	;bergeordnetes Men 26, Option 12
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 62	;Option 3: "Fadenkreuz", Men 62
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 39 - Grafik - Preset-13
;
menu39:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 13 | 0x80	;bergeordnetes Men 26, Option 13
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 63	;Option 3: "Fadenkreuz", Men 63
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 40 - Grafik - Preset-14
;
menu40:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 14 | 0x80	;bergeordnetes Men 26, Option 14
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 64	;Option 3: "Fadenkreuz", Men 64
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 41 - Grafik - Preset-15
;
menu41:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 15 | 0x80	;bergeordnetes Men 26, Option 15
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 65	;Option 3: "Fadenkreuz", Men 65
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 42 - Grafik - Preset-16
;
menu42:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 16 | 0x80	;bergeordnetes Men 26, Option 16
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 66	;Option 3: "Fadenkreuz", Men 66
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 43 - Grafik - Preset-17
;
menu43:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 17 | 0x80	;bergeordnetes Men 26, Option 17
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 67	;Option 3: "Fadenkreuz", Men 67
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 44 - Grafik - Preset-18
;
menu44:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 18 | 0x80	;bergeordnetes Men 26, Option 18
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 68	;Option 3: "Fadenkreuz", Men 68
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 45 - Grafik - Preset-19
;
menu45:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 19 | 0x80	;bergeordnetes Men 26, Option 19
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 69	;Option 3: "Fadenkreuz", Men 69
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 46 - Grafik - Preset-20
;
menu46:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 20 | 0x80	;bergeordnetes Men 26, Option 20
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 70	;Option 3: "Fadenkreuz", Men 70
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 47 - Grafik - Preset-21
;
menu47:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 21 | 0x80	;bergeordnetes Men 26, Option 21
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 71	;Option 3: "Fadenkreuz", Men 71
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 48 - Grafik - Preset-22
;
menu48:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 22 | 0x80	;bergeordnetes Men 26, Option 22
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 72	;Option 3: "Fadenkreuz", Men 72
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 49 - Grafik - Preset-23
;
menu49:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 23 | 0x80	;bergeordnetes Men 26, Option 23
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 73	;Option 3: "Fadenkreuz", Men 73
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 50 - Grafik - Preset-24
;
menu50:	.db	6, 10		;6 Untermens, Infotext "Preset"
	.db	26, 24 | 0x80	;bergeordnetes Men 26, Option 24
	.db	0x28, 0xff	;Option 1: "Vorherige Seite"
	.dw	m8a300		;	   Programmadresse
	.db	0x27, 0xff	;Option 2: Nchste Seite"
	.dw	m8a400		;	   Programmadresse
	.db	0xa1, 74	;Option 3: "Fadenkreuz", Men 74
	.dw	m8a500		;	   Programmadresse
	.db	0xa2, 0xff	;Option 4: "Sensornamen anzeigen"
	.dw	m8a600		;	   Programmadresse
	.db	0xa3, 0xff	;Option 5: "Skala angleichen"
	.dw	m8a700		;	   Programmadresse
	.db	0x4e, 0xff	;Option 6: "Schriftgre"
	.dw	m8a800		;	   Programmadresse
;
; Men 51 - Grafik - Preset-01 - Fadenkreuz
;
menu51:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	27, 3		;bergeordnetes Men 27, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 52 - Grafik - Preset-02 - Fadenkreuz
;
menu52:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	28, 3		;bergeordnetes Men 28, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 53 - Grafik - Preset-03 - Fadenkreuz
;
menu53:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	29, 3		;bergeordnetes Men 29, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 44 - Grafik - Preset-04 - Fadenkreuz
;
menu54:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	30, 3		;bergeordnetes Men 30, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 55 - Grafik - Preset-05 - Fadenkreuz
;
menu55:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	31, 3		;bergeordnetes Men 31, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 56 - Grafik - Preset-06 - Fadenkreuz
;
menu56:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	32, 3		;bergeordnetes Men 32, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 57 - Grafik - Preset-07 - Fadenkreuz
;
menu57:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	33, 3		;bergeordnetes Men 33, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 58 - Grafik - Preset-08 - Fadenkreuz
;
menu58:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	34, 3		;bergeordnetes Men 34, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 59 - Grafik - Preset-09 - Fadenkreuz
;
menu59:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	35, 3		;bergeordnetes Men 35, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 60 - Grafik - Preset-10 - Fadenkreuz
;
menu60:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	36, 3		;bergeordnetes Men 36, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 61 - Grafik - Preset-11 - Fadenkreuz
;
menu61:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	37, 3		;bergeordnetes Men 37, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 62 - Grafik - Preset-12 - Fadenkreuz
;
menu62:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	38, 3		;bergeordnetes Men 38, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 63 - Grafik - Preset-13 - Fadenkreuz
;
menu63:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	39, 3		;bergeordnetes Men 39, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 64 - Grafik - Preset-14 - Fadenkreuz
;
menu64:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	40, 3		;bergeordnetes Men 40, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 65 - Grafik - Preset-15 - Fadenkreuz
;
menu65:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	41, 3		;bergeordnetes Men 41, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 66 - Grafik - Preset-16 - Fadenkreuz
;
menu66:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	42, 3		;bergeordnetes Men 42, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 67 - Grafik - Preset-17 - Fadenkreuz
;
menu67:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	43, 3		;bergeordnetes Men 43, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 68 - Grafik - Preset-18 - Fadenkreuz
;
menu68:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	44, 3		;bergeordnetes Men 44, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 69 - Grafik - Preset-19 - Fadenkreuz
;
menu69:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	45, 3		;bergeordnetes Men 45, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 70 - Grafik - Preset-20 - Fadenkreuz
;
menu70:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	46, 3		;bergeordnetes Men 46, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 71 - Grafik - Preset-21 - Fadenkreuz
;
menu71:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	47, 3		;bergeordnetes Men 47, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 72 - Grafik - Preset-22 - Fadenkreuz
;
menu72:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	48, 3		;bergeordnetes Men 48, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 73 - Grafik - Preset-23 - Fadenkreuz
;
menu73:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	49, 3		;bergeordnetes Men 49, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
; Men 74 - Grafik - Preset-24 - Fadenkreuz
;
menu74:	.db	1, 10		;1 Untermen, Infotext "Preset"
	.db	50, 3		;bergeordnetes Men 50, Option 3
	.db	11, 0xff	;Option 1: Datenanzeige
	.dw	0		;	   keine Aktion durchfhren
;
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Default-Konfiguration, wird bei Men-Funktion "Konfiguration - Grundeinstellung - Basiskonfiguration laden" Bedarf ins EEPROM geschrieben
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Sensortexte, werden an der in den Anzeige-Positions-Tabellen festgelegten Position angezeigt, Lnge = 9 Zeichen
;
dsentx:	.db	"T01      ", "T02      "        ;Text fr Temperatursensor 01/02 (1/2)
	.db	"T03      ", "T04      "	;Text fr Temperatursensor 03/04 (3/4)
	.db	"T05      ", "T06      "	;Text fr Temperatursensor 05/06 (5/6)
	.db	"T07      ", "T08      "	;Text fr Temperatursensor 07/08 (7/8)
	.db	"T09      ", "T10      "	;Text fr Temperatursensor 09/10 (a/b)
	.db	"T11      ", "T12      "	;Text fr Temperatursensor 11/12 (c/d)
	.db	"T13      ", "T14      "	;Text fr Temperatursensor 13/14 (e/f)
	.db	"T15      ", "T16      "	;Text fr Temperatursensor 15/16 (g/h)
	.db	"T17      ", "T18      "	;Text fr Temperatursensor 17/18 (i/j)
	.db	"T19      ", "T20      "	;Text fr Temperatursensor 19/20 (k/l)
	.db	"T21      ", "T22      "	;Text fr Temperatursensor 21/22 (m/n)
	.db	"T23      ", "T24      "	;Text fr Temperatursensor 23/24 (o/p)
	.db	"T25      ", "T26      "	;Text fr Temperatursensor 25/26 (q/r)
	.db	"T27      ", "P01      "        ;Text fr Temperatursensor 27 (s) und Luftdrucksensor 28 (w)
        .db     "H01      ", "H02      "        ;Text fr Luftfeuchtigkeitssensor 01/02 (x/y)
        .db     "H03      ", "B01      "        ;Text fr Luftfeuchtigkeitssensor 03 (z) und Batterie
;
; Alarmtexte, werden an der von der Anzeige-Positions-Tabelle festgelegten Position angezeigt, Lnge = 11 Zeichen
;
dalmtx:	.db	"Al1        ", "Al2        "	;Text fr Alarmeingang 1 / Alarmeingang 2
	.db	"Al3        ", "Al4        "    ;Text fr Alarmeingang 3 / Alarmeingang 4
;
; Signaltabelle, enthlt die Nummer des akustischen Signals (0-15) fr 8 verschiedene Ereignisse
;
dsignr:	.db	0, 1	                        ;Signal fr Alarmeingang 1 / Alarmeingang 2
	.db	2, 3	                        ;Signal fr Alarmeingang 3 / Alarmeingang 4
	.db	12, 8	                        ;Signal fr Temperatur-berschreitung / Temperatur-Unterschreitung
	.db	9, 10	                        ;Signal fr Fehlermeldungen / Sensorausflle
;
; Signaltabelle, enthlt die Dauer des akustischen Signals in Sekunden fr 8 verschiedene Ereignisse (1-30s), 0 = akustisches Signal aus
;
dsigdu:	.db	4, 4	                        ;Dauer fr Alarmeingang 1 / Alarmeingang 2
	.db	4, 4	                        ;Dauer fr Alarmeingang 3 / Alarmeingang 4
	.db	3, 3	                        ;Dauer fr Temperatur-berschreitung / Temperatur-Unterschreitung
	.db	10, 3	                        ;Dauer fr Fehlermeldungen / Sensorausflle
;
; Timerwert fr die Signalfrequenz
;
dsigtm:	.db	122, 0	                        ;Timerwert (28-255), Beispiel-Wert fr 1405Hz, zweites Byte wird nicht genutzt
;
; Standard-Signallautstrke
;
dsigv1:	.db	3, 0	                        ;Lautstrkewert (0-15), zweites Byte wird nicht genutzt
;
; Signallautstrke whrend der Nachtzeit
;
dsigv2:	.db	1, 0	                        ;Lautstrkewert (0-15), zweites Byte wird nicht genutzt
;
; Start- und Endzeit fr die Nachtschaltung der Signalisierung, zur Startzeit wird auf Nacht-Lautstrke umgeschaltet und zur Endzeit zurck auf Standard-
; Lautstrke; Wert in Minuten (16 Bit, 0-1439)
;
dsigni:	.dw	22 * 60	                        ;Startzeit (22:00 Uhr)
	.dw	8 * 60  	                ;Endzeit (8:00 Uhr)
;
; Start- und Endzeit fr die Nachtschaltung der LCD-Hintergrundbeleuchtung, zur Startzeit wird die Beleuchtung abgeschaltet und zur Endzeit wieder
; eingeschaltet; Wert in Minuten (16 Bit, 0-1439)
;
dnigbl:	.dw	22 * 60	                        ;Startzeit (22:00 Uhr)
	.dw	8 * 60	                        ;Endzeit (8:00 Uhr)
;
; Zeiten fr automatisches Rcksetzen der Minimal- und Maximalwerte Wert in Minuten (16 Bit, 0-1439)
;
daresm:	.dw	20 * 60	                        ;Auto-Reset-Zeit fr Minimalwerte (20:00)
	.dw	8 * 60	                        ;Auto-Reset-Zeit fr Maximalwerte (8:00)
;
; Start- und Endzeit fr die Nachtschaltung des Bluetooth-Moduls, zur Startzeit wird das Modul abgeschaltet und zur Endzeit wieder eingeschaltet; Wert in
; Minuten (16 Bit, 0-1439)
;
dnigbt:	.dw	22 * 60	                        ;Startzeit (22:00 Uhr)
	.dw	8 * 60	                        ;Endzeit (8:00 Uhr)
;
; Konfigurationsbyte 1, zweites Byte wird nicht genutzt                                                         [Voreinstellung]
;
dconf1:	.db     0b01111011, 0                   ;B0: Sprache (0 = englisch, 1 = deutsch)                        [1 = deutsch]
			                        ;B1: Trennzeichen (0 = Punkt, 1 = Komma)                        [1 = Komma]
			                        ;B2: Temperaturalarme immer signalisieren                       [0 = nur einmal signalisieren]
			                        ;B3: Nachtschaltung fr LCD-Beleuchtung aktiv                   [1 = aktiv]
			                        ;B4: Auto-Reset fr Min/Max-Werte aktiv                         [1 = aktiv]
			                        ;B5: Nachtschaltung fr Signal-Lautstrke aktiv                 [1 = aktiv]
			                        ;B6: LCD abschalten fr Uhr-Synchronisierung                    [1 = aktiv]
			                        ;B7: Modus, 0 = Normal, 1 = Abwesend (Licht und Ton aus)        [0 = normal]
;
; Konfigurationsbyte 2, zweites Byte wird nicht genutzt                                                         [Voreinstellung]
;
dconf2:	.db     0b00101100, 0                   ;B0: LED-Modus (0 = normal, 1 = debug [DCF, RS-232])            [0 = normal]
			                        ;B1: LCD-Beleuchtungs-Automatik                                 [0 = inaktiv]
			                        ;B2: Nachtschaltung fr Bluetooth-Modul aktiv                   [1 = aktiv]
                                                ;B3: Automatisch auf Seite 1 zurckschalten                     [1 = aktiv]
                                                ;B4: DCF77-Signal invertiert                                    [0 = normal]
                                                ;B5: DCF77-Eingang mit Pull-up Widerstand                       [1 = aktiv]
;
; Nacht-Helligkeit der Hintergrundbeleuchtung (0-255)
;
dnligh:	.db	10, 0	                        ;Nacht-Helligkeit der Hintergrundbeleuchtung (10), zweites Byte wird nicht genutzt
;
; Baudrate fr die PC-Verbindung (RS232-0): 0 = 9600, 1 = 19200, 2 = 38400, 3 = 57600, 4 = 115200, 5 = 230400, 6 = 460800
;
dbaud0:	.db	4, 0	                        ;Baudrate USART0 (115200 Baud), zweites Byte wird nicht genutzt
;
; Meldungsfilter, steuert die Menanzeige und die Datenausgabe, folgende Werte sind mglich: 0 = alles anzeigen, 1 = nur Alarme anzeigen, 2 = nur Temperaturen
; anzeigen, 3 = nur Fehler anzeigen
;
dmsgfi:	.db	0, 0	                        ;Meldungsfilter (alles anzeigen), zweites Byte wird nicht genutzt
;
; Grafik-Presets, 24 Voreinstellungen fr die Grafikanzeige mit jeweils 7 Werten:
; Byte 0: Grafik 1, Sensornummer 1
; Byte 1: Grafik 1, Sensornummer 2
; Byte 2: Grafik 2, Sensornummer 1
; Byte 3: Grafik 2, Sensornummer 2
; Byte 4: Grafik 1, Anzeigemodus
; Byte 5: Grafik 2, Anzeigemodus
; Byte 6: Optionen fr die Grafikanzeige
;
; Alles Presets zeigen Einzeldiagramme mit 15min-Werten
;
dgrpre:	.db	0, 0, 0, 0, 0, 0, 1,    1, 0, 0, 0, 0, 0, 1		;Preset 01, Preset 02
	.db	2, 0, 0, 0, 0, 0, 1,    3, 0, 0, 0, 0, 0, 1		;Preset 03, Preset 04
	.db	4, 0, 0, 0, 0, 0, 1,    5, 0, 0, 0, 0, 0, 1		;Preset 05, Preset 06
	.db	6, 0, 0, 0, 0, 0, 1,    7, 0, 0, 0, 0, 0, 1		;Preset 07, Preset 08
	.db	8, 0, 0, 0, 0, 0, 1,    9, 0, 0, 0, 0, 0, 1		;Preset 09, Preset 10
	.db	10, 0, 0, 0, 0, 0, 1,   11, 0, 0, 0, 0, 0, 1		;Preset 11, Preset 12
	.db	12, 0, 0, 0, 0, 0, 1,   13, 0, 0, 0, 0, 0, 1		;Preset 13, Preset 14
	.db	14, 0, 0, 0, 0, 0, 1,   15, 0, 0, 0, 0, 0, 1		;Preset 15, Preset 16
	.db	16, 0, 0, 0, 0, 0, 1,   17, 0, 0, 0, 0, 0, 1	        ;Preset 17, Preset 18
	.db	18, 0, 0, 0, 0, 0, 1,   19, 0, 0, 0, 0, 0, 1	        ;Preset 19, Preset 20
	.db	20, 0, 0, 0, 0, 0, 1,   21, 0, 0, 0, 0, 0, 1	        ;Preset 21, Preset 22
	.db	22, 0, 0, 0, 0, 0, 1,   23, 0, 0, 0, 0, 0, 1	        ;Preset 23, Preset 24
;
; Grafik-Preset-Texte, Lnge = 16 Zeichen
;
dgrprt:	.db	"T01-15min       "	        ;Text fr Preset 01
	.db	"T02-15min       "	        ;Text fr Preset 02
	.db	"T03-15min       "	        ;Text fr Preset 03
	.db	"T04-15min       "	        ;Text fr Preset 04
	.db	"T05-15min       "	        ;Text fr Preset 05
	.db	"T06-15min       "	        ;Text fr Preset 06
	.db	"T07-15min       "	        ;Text fr Preset 07
	.db	"T08-15min       "	        ;Text fr Preset 08
	.db	"T09-15min       "	        ;Text fr Preset 09
	.db	"T10-15min       "	        ;Text fr Preset 10
	.db	"T11-15min       "	        ;Text fr Preset 11
	.db	"T12-15min       "	        ;Text fr Preset 12
	.db	"T13-15min       "	        ;Text fr Preset 13
	.db	"T14-15min       "	        ;Text fr Preset 14
	.db	"T15-15min       "	        ;Text fr Preset 15
	.db	"T16-15min       "	        ;Text fr Preset 16
	.db	"T17-15min       "	        ;Text fr Preset 17
	.db	"T18-15min       "	        ;Text fr Preset 18
	.db	"T19-15min       "	        ;Text fr Preset 19
	.db	"T20-15min       "	        ;Text fr Preset 20
	.db	"T21-15min       "	        ;Text fr Preset 21
	.db	"T22-15min       "	        ;Text fr Preset 22
	.db	"T23-15min       "	        ;Text fr Preset 23
	.db	"T24-15min       "	        ;Text fr Preset 24
;
;
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
; Belegung des internen EEPROMs
; -------------------------------------------------------------------------------------------------------------------------------------------------------------
;
.eseg
;
epotab: .byte   32              ;Anzeige-Positions-Tabelle fr beide Bildschirm-Seiten, enthlt fr jede Bild-Position den anzuzeigenden Sensor:
                                ;00: Sensornummer an Anzeigeposition 00 von Seite 1 (Sensornummer 0-31 oder >31 = kein Sensor anzeigen)
                                ;Byte  0: Sensornummer an Anzeigeposition  0 von Seite 1 (0-31)
                                ;Byte 15: Sensornummer an Anzeigeposition 15 von Seite 1 (0-31)
                                ;Byte 16: Sensornummer an Anzeigeposition  0 von Seite 2 (0-31)
                                ;Byte 31: Sensornummer an Anzeigeposition 15 von Seite 2 (0-31)
esnflg: .byte   32              ;Sensor-Flag, enthlt nur jeweils Bit7 zur Kennzeichnung eines inaktiven Sensors
ebiflg: .byte   2               ;Groanzeige-Flags fr beide Bildschirmseiten, enhalten die Sensornummer fr jede Seite (jeweils 0-26)
esentx:	.byte	31 * 9	        ;31 Sensortexte: werden entsprechend der Anzeige-Positions-Tabellen angezeigt, Lnge = 9 Zeichen
eselih:	.byte	27 * 2 	        ;Obere Temperaturlimits in Zehntel Grad (16 Bit Hex), 0x8000 = Limitberwachung ausgeschaltet
eselil:	.byte	27 * 2	        ;Untere Temperaturlimits in Zehntel Grad (16 Bit Hex), 0x8000 = Limitberwachung ausgeschaltet
elotb1: .byte   24              ;Log-Tabelle 1, enthlt die Positionen der Sensoren 0-23 im EEPROM-Log
elotb2: .byte   3               ;Log-Tabelle 2, enthlt die Positionen der Sensoren 24-26 (28-30) im EEPROM-Log
ealtab:	.byte	4	        ;Alarm-Anzeige-Positions-Tabelle: hier werden die Anzeigepositionen der Alarmdaten festgelegt (0-3)
                                ;Bit7 = 1: Alarm nicht genutzt
ealinv:	.byte	4	        ;Alarm Invertierungs-Tabelle: hier wird festgelegt, ob der entsprechende Alarm invertiert ausgewertet werden soll
                                ;0 = normal, 1 = invertiert
ealmtx:	.byte	4 * 11	        ;Alarmtexte: werden an der von der Anzeige-Tabelle festgelegten Position auf dem LCD angezeigt, Lnge = 11 Zeichen
esignr:	.byte	8	        ;Signaltabelle, enthlt die Nummer des akustischen Signals (0-15) fr 8 verschiedene Ereignisse
esigdu:	.byte	8	        ;Signaltabelle, enthlt die Dauer des akustischen Signals in Sekunden fr 8 verschiedene Ereignisse (1-30s),
                                ;0 = akustisches Signal aus
esigtm:	.byte	1	        ;Timerwert fr die Signalfrequenz
esigv1:	.byte	1	        ;Standard-Signallautstrke
esigv2:	.byte	1	        ;Signallautstrke whrend der Nachtzeit
esigni:	.byte	2 * 2	        ;Start- und Endzeit fr die Nachtschaltung der Signalisierung, zur Startzeit wird auf Nacht-Lautstrke umgeschaltet und zur
                                ;Endzeit auf Standard-Lautstrke; Wert in Minuten (16 Bit, 0-1439)
enigbl:	.byte	2 * 2	        ;Start- und Endzeit fr die Nachtschaltung der LCD-Hintergrundbeleuchtung, zur Startzeit wird die Beleuchtung abgeschaltet und
                                ;zur Endzeit wieder eingeschaltet; Wert in Minuten (16 Bit, 0-1439)
earesm:	.byte	2 * 2	        ;Zeiten fr automatisches Rcksetzen der Minimal- und Maximalwerte; Wert in Minuten (16 Bit, 0-1439)
enigbt:	.byte	2 * 2	        ;Start- und Endzeit fr die Nachtschaltung des Bluetooth-Moduls, zur Startzeit wird das Modul abgeschaltet und zur Endzeit
                                ;wieder eingeschaltet; Wert in Minuten (16 Bit, 0-1439)
econf1:	.byte	1	        ;Konfigurationsbyte 1
			        ;B0: Sprache (0 = englisch, 1 = deutsch)
			        ;B1: Trennzeichen (0 = Punkt, 1 = Komma)
			        ;B2: Temperaturalarme immer signalisieren
			        ;B3: Nachtschaltung fr LCD-Beleuchtung aktiv
			        ;B4: Auto-Reset fr Min/Max-Werte aktiv
			        ;B5: Nachtschaltung fr Signal-Lautstrke aktiv
			        ;B6: LCD abschalten fr Uhr-Synchronisierung
			        ;B7: Modus 0 = Normal, 1 = Abwesend (Licht und Ton aus)
econf2:	.byte	1	        ;Konfigurationsbyte 2
			        ;B0: LED-Modus (0 = normal, 1 = debug [DCF, RS-232])
				;B1: LCD-Beleuchtung-Automatik
				;B2: Nachtschaltung fr Bluetooth-Modul aktiv
                                ;B3: Automatisch auf Seite 1 zurckschalten
                                ;B4: DCF77-Signal invertiert
                                ;B5: DCF77-Eingang mit Pull-up Widerstand (fr DCF-Empfnger mit offenem Kollektor-Ausgang)
enligh:	.byte	1	        ;Nacht-Helligkeit der Hintergrundbeleuchtung (0-255)
ebaud0:	.byte	1	        ;Baudrate fr die PC-Verbindung (RS232-0) 0 = 9600, 1 = 19200, 2 = 38400, 3 = 57600, 4 = 115200, 5 = 230400, 6 = 460800
emsgfi:	.byte	1	        ;Meldungsfilter, steuert die Menanzeige und die Datenausgabe, folgende Werte sind mglich:
			        ;0 = alles anzeigen
			        ;1 = nur Alarme anzeigen
			        ;2 = nur Temperaturen anzeigen
			        ;3 = nur Fehler anzeigen
egrpre:	.byte	24 * 7	        ;24 Grafik-Presets mit jeweils 5 Bytes:
			        ;Byte 0: Grafik 1, Sensornummer 1
			        ;Byte 1: Grafik 1, Sensornummer 2
			        ;Byte 2: Grafik 2, Sensornummer 1
			        ;Byte 3: Grafik 2, Sensornummer 2
			        ;Byte 4: Mode 1 (0-3): 0 = 15min-Werte, 1 = 24h-Minimalwerte, 2 = 24h-Maximalwerte, 3 = 24h-Mittelwerte
			        ;Byte 5: Mode 2 (0-3): 0 = 15min-Werte, 1 = 24h-Minimalwerte, 2 = 24h-Maximalwerte, 3 = 24h-Mittelwerte
			        ;Byte 6: Verschiedene Grafik-Optionen:
			        ;	 B0 = Sensornamen in Grafik einblenden
			        ;	 B1 = Skala angleichen
			        ;	 B2 = Differenzmodus fr Diagramm 1
			        ;	 B3 = Differenzmodus fr Diagramm 2
			        ;	 B4 = Doppeldiagramm
                                ;        B5 = Schriftgre (0 = normal, 1 = Mini-Schrift)
egrprt:	.byte	24 * 16	        ;24 Grafik-Preset-Texte mit jeweils 16 Zeichen
eprcrc:	.byte	1	        ;8-Bit-CRC, enthlt die Prfsumme ber den genutzten EEPROM-Bereich vom Byte 0 beginnend, CRC wird bei jedem Schreibvorgang neu
                                ;gesetzt und beim Lesen geprft
;
